#include "Basic3DMath.h"
#include "Matrix.h"
#include "Point4D.h"

#include <math.h>

Matrix* Basic3DMath::identity( Matrix* pOut )
{
    for ( int i = 0; i < 4; ++i )
    {
        for ( int j = 0; j < 4; ++j )
        {
            pOut->m[i][j] = ( i == j );
        }
    }
    return pOut;
}


Matrix* Basic3DMath::translate( Matrix* pOut, const Point3D& t )
{
    return translate( pOut, t.x, t.y, t.z );
}


Matrix* Basic3DMath::translate( Matrix* pOut, double x, double y, double z )
{
    pOut->_41 = x;
    pOut->_42 = y;
    pOut->_43 = z;
    return pOut;
}

Matrix* Basic3DMath::affine( Matrix* pOut,
                        const Vector3D* u,
                        const Vector3D* v,
                        const Vector3D* w,
                        const Vector3D* vrp )
{
    pOut->_11 = u->x; pOut->_12 = u->y; pOut->_13 = u->z; pOut->_14 = 0;
    pOut->_21 = v->x; pOut->_22 = v->y; pOut->_23 = v->z; pOut->_24 = 0;
    pOut->_31 = w->x; pOut->_32 = w->y; pOut->_33 = w->z; pOut->_34 = 0;
    pOut->_41 = vrp->x; pOut->_42 = vrp->y; pOut->_43 = vrp->z; pOut->_44 = 1;
    return pOut;
}

Matrix* Basic3DMath::perspective( Matrix* pOut,
                             double fp,
                             double bp,
                             const Basic3D::Rectangle* vp )
{
    pOut->_11 = vp->width / 2.0;
    pOut->_22 = vp->height / 2.0;
    pOut->_31 = ( vp->x + vp->width ) / 2.0;
    pOut->_32 = ( vp->y + vp->height ) / 2.0;
    pOut->_33 = bp / ( bp - fp ); pOut->_34 = 1.0;
    pOut->_43 = -fp / ( bp - fp ); pOut->_44 = 0.0;

    return pOut;
}


Matrix* Basic3DMath::scale( Matrix* pOut, const Point3D& s )
{
    return scale( pOut, s.x, s.y, s.z );
}


Matrix* Basic3DMath::scale( Matrix* pOut, double sx, double sy, double sz )
{
    pOut->_11 = sx;
    pOut->_22 = sy;
    pOut->_33 = sz;
    return pOut;
}

Point4D* Basic3DMath::transform( Point4D* pOut, const Matrix* pIn )
{
    Point4D p;

    p.x = pOut->x * pIn->_11 + pOut->y * pIn->_21 +
              pOut->z * pIn->_31 + pOut->w * pIn->_41;

    p.y = pOut->x * pIn->_12 + pOut->y * pIn->_22 +
              pOut->z * pIn->_32 + pOut->w * pIn->_42;

    p.z = pOut->x * pIn->_13 + pOut->y * pIn->_23 +
              pOut->z * pIn->_33 + pOut->w * pIn->_43;

    p.w = pOut->x * pIn->_14 + pOut->y * pIn->_24 +
              pOut->z * pIn->_34 + pOut->w * pIn->_44;

    *pOut = p;

    return pOut;
}


Point4D Basic3DMath::transform( const Point4D& p, const Matrix& m )
{
    double x = p.x * m._11 + p.y * m._21 +
              p.z * m._31 + p.w * m._41;

    double y = p.x * m._12 + p.y * m._22 +
              p.z * m._32 + p.w * m._42;

    double z = p.x * m._13 + p.y * m._23 +
              p.z * m._33 + p.w * m._43;

    double w = p.x * m._14 + p.y * m._24 +
              p.z * m._34 + p.w * m._44;

    return Point4D( x ,y, z, w );
}


Matrix* Basic3DMath::invert( Matrix* pOut, const Matrix* pIn )
{
    /*
        double a11 = m[0][0], a12 = m[0][1], a13 = m[0][2];
        double a21 = m[1][0], a22 = m[1][1], a23 = m[1][2];
        double a31 = m[2][0], a32 = m[2][1], a33 = m[2][2];

        double adj11 = a22 * a33 - a23 * a32;
        double adj12 = -a21 * a33 + a23 * a31;
        double adj13 = a21 * a32 - a22 * a31;

        double adj21 = -a12 * a33 + a13 * a32;
        double adj22 = a11 * a33 - a13 * a31;
        double adj23 = -a11 * a32 + a12 * a31;

        double adj31 = a12 * a23 - a13 * a22;
        double adj32 = -a11 * a23 + a13 * a21;
        double adj33 = a11 * a22 - a12 * a21;


        double det = a11 * adj11 + a12 * adj12 + a13 * adj13;
        if (det == 0) return;

        m[0][0] =  adj11/det;  m[0][1] = adj21/det; m[0][2] = adj31/det;
        m[1][0] =  adj12/det;  m[1][1] = adj22/det; m[1][2] = adj32/det;
        m[2][0] =  adj13/det;  m[2][1] = adj23/det; m[2][2] = adj33/det;

        double p1  = m[3][0],  p2 = m[3][1],  p3 = m[3][2];

        m[3][0] = -p1 * m[0][0] - p2 * m[1][0] - p3 * m[2][0];
        m[3][1] = -p1 * m[0][1] - p2 * m[1][1] - p3 * m[2][1];
        m[3][2] = -p1 * m[0][2] - p2 * m[1][2] - p3 * m[2][2];

    */
    Matrix m;
    identity( &m );

    m._11 = pIn->_22 * pIn->_33 - pIn->_23 * pIn->_32;
    m._12 = -pIn->_21 * pIn->_33 + pIn->_23 * pIn->_31;
    m._13 = pIn->_21 * pIn->_32 - pIn->_22 * pIn->_31;

    m._21 = -pIn->_12 * pIn->_33 + pIn->_13 * pIn->_32;
    m._22 = pIn->_11 * pIn->_33 - pIn->_13 * pIn->_31;
    m._23 = -pIn->_11 * pIn->_32 + pIn->_12 * pIn->_31;

    m._31 = pIn->_12 * pIn->_23 - pIn->_13 * pIn->_22;
    m._32 = -pIn->_11 * pIn->_23 + pIn->_13 * pIn->_21;
    m._33 = pIn->_11 * pIn->_22 - pIn->_12 * pIn->_21;

    //Matrix::print( &m );

    double det = pIn->_11 * m._11 + pIn->_12 * m._12 + pIn->_13 * m._13;

    if ( det == 0 ) { return NULL; }

    identity( pOut );

    pOut->_11 = m._11 / det; pOut->_12 = m._21 / det; pOut->_13 = m._31 / det;
    pOut->_21 = m._12 / det; pOut->_22 = m._22 / det; pOut->_23 = m._32 / det;
    pOut->_31 = m._13 / det; pOut->_32 = m._23 / det; pOut->_33 = m._33 / det;

    double p1  = pIn->_41,  p2 = pIn->_42,  p3 = pIn->_43;

    pOut->_41 = -p1 * pOut->_11 - p2 * pOut->_21 - p3 * pOut->_31;
    pOut->_42 = -p1 * pOut->_12 - p2 * pOut->_22 - p3 * pOut->_32;
    pOut->_43 = -p1 * pOut->_13 - p2 * pOut->_23 - p3 * pOut->_33;

    return pOut;
}

void Basic3DMath::print( const Matrix* pIn )
{
    for ( int i = 0; i < 4; ++i )
    {
        for ( int j = 0; j < 4; ++j )
        {
            printf( "%4.2f ", pIn->m[i][j] );
        }
        printf( "\n" );
    }
}

Matrix* Basic3DMath::rotateX( Matrix* pOut, double angle )
{
    pOut->_22 = cos( angle );   pOut->_23 = sin( angle );
    pOut->_32 = -sin( angle );  pOut->_33 = cos( angle );

    return pOut;
}

Matrix* Basic3DMath::rotateY( Matrix* pOut, double angle )
{
    pOut->_11 = cos( angle ); pOut->_13 = -sin( angle );
    pOut->_31 = sin( angle ); pOut->_33 = cos( angle );

    return pOut;
}

Matrix* Basic3DMath::rotateZ( Matrix* pOut, double angle )
{
    pOut->_11 = cos( angle ); pOut->_12 = sin( angle );
    pOut->_21 = -sin( angle );  pOut->_22 = cos( angle );

    return pOut;
}

