#ifndef BASIC3D_MATH_H
#define BASIC3D_MATH_H

#include "Rectangle.h"
#include "Point3D.h"

class Matrix;
class Point4D;

class Basic3DMath
{
    public:
        static Matrix* identity( Matrix* pOut );
        static Matrix* invert( Matrix* pOut, const Matrix* pIn );
        static Matrix* translate( Matrix* pOut, double x, double y, double z );
        static Matrix* translate( Matrix* pOut, const Point3D& t );
        static Matrix* affine( Matrix* pOut,
                                       const Vector3D* u,
                                       const Vector3D* v,
                                       const Vector3D* w,
                                       const Vector3D* vrp );
        static Matrix* perspective( Matrix* pOut,
                                            double fp,
                                            double bp,
                                            const Basic3D::Rectangle* vp );
        static Matrix* scale( Matrix* pOut, double sx, double sy, double sz );
        static Matrix* scale( Matrix* pOut, const Point3D& s );
        static Matrix* rotateX( Matrix* pOut, double angle );
        static Matrix* rotateY( Matrix* pOut, double angle );
        static Matrix* rotateZ( Matrix* pOut, double angle );
        static Point4D* transform( Point4D* pOut, const Matrix* pIn );
        static Point4D transform( const Point4D& p, const Matrix& m );

        void print( const Matrix* pIn );

    private:
        ~Basic3DMath();
};


#endif // BASIC3D_MATH_H
