#ifndef SURFACE3D_H
#define SURFACE3D_H

#include "Primitive3D.h"
#include "../common/types/IntArray.h"


class Surface3D : public Primitive3D
{
    public:
        Surface3D();
        Surface3D( size_t rows, size_t cols );
        Surface3D( const Point4DArray& points, int cols, int rows );
        virtual ~Surface3D();


        const Point4D&      getPoint( size_t row, size_t col );
        const Point4D&      getPoint( size_t index );
        const Point4D&      getTransformedPoint( size_t row, size_t col );
        void                setPoint( size_t row, size_t col, const Point4D& p );
        void                setMinY( double min );

        ///Primitive3D virtual function
        void                setSize( size_t rows, size_t cols );
        void                draw( wxDC& dc, int w, int h );
        void                transform( const Matrix* m );
        void                homDiv();

    private:
        Point4DArray        m_transformedPoints;
        IntArray            m_indicies;
        IntArray            m_invisibleEdges;
        size_t              m_rows;
        size_t              m_cols;
        double              m_minY;

        void                createIndicies();
        void                setCornerPoint( size_t row, size_t col, const Point4D& p );
        void                findInvisibleEdges();
};


enum MarkerType
{
    MARKER_CIRCLE   = 0,
    MARKER_CROSS    = 1
};

class Marker3D : public Primitive3D
{
    public:
        Marker3D(   const Point3D& p,
                    const Point3D& p0 = Point3D(),
                    int size = 5,
                    MarkerType m = MARKER_CIRCLE,
                    wxColor c = *wxWHITE,
                    bool drawLines = true  );


        //Primitive3D virtual function
        void draw( wxDC& dc, int w, int h );
        const Point4D& getPoint( size_t index ) { return *m_points[index]; }

    private:
        MarkerType  m_markerType;
        bool        m_isDrawLines;
        int         m_size;

        void init( const Point3D& p, const Point3D& p0, int size, MarkerType m, wxColor c, bool drawLines );
};


WX_DEFINE_ARRAY( wxColour*, ColourArray );

class Axis3D : public Primitive3D
{
    public:
        enum Axis { X_AXIS = 0, Y_AXIS = 1, Z_AXIS = 2 };

    public:
        Axis3D();
        ~Axis3D();

        void            draw( wxDC& dc, int w, int h );

        void            setVisible( bool visible ) { m_isVisible = visible; }
        void            setAxisColour( Axis axis, wxColour color );
        void            setAxisLength( Axis axis, double length );
        void            setCenter( const Point4D& center );

        bool            getVisible() const { return m_isVisible; }
        wxColour        getAxisColour( Axis axis ) const;

    private:
        IntArray        m_indicies;
        ColourArray     m_colors;
        bool            m_isVisible;

        void            createPoints( const Point4D& center );
        void            createIndicies();
        void            createColors();
};


#endif // SURFACE3D_H
