#include "TableEditor3DConfigHelper.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"
#include "../common/types/String2IntHash.h"

#include "../logger.h"


void TableEditor3DConfigHelper::loadParams( const wxString& name, TableEditor3DParams& params )
{
    ConfigBase* mainCfg = ConfigHelper2::getConfig();
    ConfigBase& config = *mainCfg;

    config.setPath( _T("/table_editor_3d/") + name );

    if( !config.getNumberOfEntries() ) return;

    wxString result;

    if ( config.read( _T("RotationX"), &result ) )
    {
        result.ToDouble( &params.rotationAngleX );
    }

    if ( config.read( _T("RotationY"), &result ) )
    {
        result.ToDouble( &params.rotationAngleY );
    }

    if ( config.read( _T("CameraEyeX"), &result ) )
    {
        result.ToDouble( &params.cameraEyeX );
    }

    if ( config.read( _T("CameraEyeY"), &result ) )
    {
        result.ToDouble( &params.cameraEyeY );
    }

    if ( config.read( _T("Zoom"), &result ))
    {
        result.ToDouble( &params.zoom );
    }

    if ( config.read( _T("ShowAxes"), &result ) )
    {
        params.showAxes = result.IsSameAs( _T("true"), false );
    }
}

void TableEditor3DConfigHelper::saveParams( const wxString& name, const TableEditor3DParams& params )
{
    ConfigBase* mainCfg = ConfigHelper2::getConfig();
    ConfigBase& config = *mainCfg;

    config.setPath( _T("/table_editor_3d/") + name );

    config.write( _T("RotationX"), wxString::Format( _T("%f"), params.rotationAngleX ) );
    config.write( _T("RotationY"), wxString::Format( _T("%f"), params.rotationAngleY ) );
    config.write( _T("CameraEyeX"), wxString::Format( _T("%f"), params.cameraEyeX ) );
    config.write( _T("CameraEyeY"), wxString::Format( _T("%f"), params.cameraEyeY ) );
    config.write( _T("Zoom"), wxString::Format( _T("%f"), params.zoom ) );

    wxString showAxes = params.showAxes ? _T("true") : _T("false");
    config.write( _T("ShowAxes"), showAxes );

    config.save();
}

void TableEditor3DConfigHelper::loadGlobalParams( TableEditor3DParams& params )
{
    ConfigBase* mainCfg = ConfigHelper2::getConfig();
    ConfigBase& config = *mainCfg;

    config.setPath( _T("/table_editor_3d") );

    wxString result;

    /// cursor moving mode
    if ( !config.read( _T("cursorMovingMode"), &result ) )
    {
        Logger::logf( LOG_MSG, _T("TableEditor3D::loadParams() - ")
            _T("Cannot load cursorMovingMode from config! Use default: RELATIVE") );
        params.cursorMovingMode = TE3D_CursorMovingMode::CMM_RELATIVE;
    } else {
        params.cursorMovingMode = TE3D_CursorMovingMode::stringToId( result );
    }

    /// selected label
    if ( !config.read( _T("selectedLabelAlign"), &result ) )
    {
        Logger::logf( LOG_MSG, _T("TableEditor3D::loadParams() - ")
            _T("Cannot load selectedLabelAlign from config! Use default: TOP_LEFT") );
        params.selectedLabelAlign = TE3D_Align::TOP_LEFT;
    } else {
        params.selectedLabelAlign = TE3D_Align::stringToId( result );
    }

    /// marked label
    if ( !config.read( _T("markedLabelAlign"), &result ) )
    {
        Logger::logf( LOG_MSG, _T("TableEditor3D::loadParams() - ")
            _T("Cannot load markedLabelAlign from config! Use default: TOP_LEFT") );
        params.markedLabelAlign = TE3D_Align::TOP_LEFT;
    } else {
        params.markedLabelAlign = TE3D_Align::stringToId( result );
    }

    /// label font size
    if ( !config.read( _T("labelFontSize"), &result ) )
    {
        Logger::logf( LOG_MSG, _T("TableEditor3D::loadParams() - ")
            _T("Cannot load markedLabelAlign from config! Use default: TOP_LEFT") );
        params.labelFontSize = 14;
    } else {
        long labelFontSize;

        if ( !result.ToLong( &labelFontSize ) ) {
            params.labelFontSize = 14;
        } else {
            params.labelFontSize = (int)labelFontSize;
        }
    }
}

void TableEditor3DConfigHelper::saveGlobalParams( TableEditor3DParams& params )
{
    ConfigBase* mainCfg = ConfigHelper2::getConfig();
    ConfigBase& config = *mainCfg;

    config.setPath( _T("/table_editor_3d") );

    /// cursor moving mode
    wxString cursorMovingMode = TE3D_CursorMovingMode::idToConfigString( params.cursorMovingMode );
    config.write( _T("cursorMovingMode"), cursorMovingMode );

    /// selected label
    wxString selectedLabelAlign = TE3D_Align::idToConfigString( params.selectedLabelAlign );
    config.write( _T("selectedLabelAlign"), selectedLabelAlign );

    /// marked label
    wxString markedLabelAlign = TE3D_Align::idToConfigString( params.markedLabelAlign );
    config.write( _T("markedLabelAlign"), markedLabelAlign );

    /// label font size
    config.write( _T("labelFontSize"), wxString::Format( _T("%d"), params.labelFontSize ) );

    config.save();
}
