#include "AIMParser.h"
#include "common/typedef.h"
#include "common/TxData.h"
#include "vemsprotocol/SIPRPacket.h"
#include "vemsprotocol/SIPRTypes.h"
#include "vemsprotocol/VemsCommand.h"
#include "events/InputEvent.h"
#include "logger.h"


const uint8_t AIMData::AIM_CONST = 0xA3;
const int MAX_EMPTY_COUNT = 10;

void AIMData::reset()
{
    channel = 0;
    high    = 0;
    low     = 0;
    sum     = 0;
}

AIMParser::AIMParser( wxEvtHandler* eventHandler )
{
    m_pEventHandler = eventHandler;

    m_prevByte  = 0;
    m_sum       = 0;
    m_state     = 0;

    m_pCurrentCommand = NULL;

    reset();

    m_iEmptyCount = 0;
}

AIMParser::~AIMParser()
{
    //dtor
}

int AIMParser::syncRecord( uint8_t* c )
{
    int i;
    for(i=0; i<5; i++){
        int state = feed(c[i]);
        if( state == END ) return AIMParser::SYNC_OK;
    }
    if(c[0] == AIMData::AIM_CONST) return AIMParser::SYNC_0;

//    if(c[1] == AIMData::AIM_CONST) return AIMParser::SYNC_OK;

    if(c[2] == AIMData::AIM_CONST) return AIMParser::SYNC_2;
    if(c[3] == AIMData::AIM_CONST) return AIMParser::SYNC_3;
    if(c[4] == AIMData::AIM_CONST) return AIMParser::SYNC_4;
    return -1;
}

int AIMParser::feed( uint8_t c )
{
    //Logger::logf( LOG_MSG, _T("AIMParser::feed() m_state = %d, c = %02X"),
//        m_state, c );

    if( (c == AIMData::AIM_CONST) && (m_state == 0) )
    {
        data.reset();
        data.channel = m_prevByte;
        //m_sum = 0;
        m_sum = data.channel + c;
        m_state = START;
        return m_state;
    }

    switch( m_state )
    {
        case START:
        {
            data.high = c;
            m_sum += c;
            return ++m_state;
        }
        case 3 :
        {
            data.low = c;
            m_sum += c;
            return ++m_state;
        }
        case 4 :
        {
            m_state = 0;
            if( m_sum == c )
            {
                data.sum = c;
                return END;
            }else{
                return CHECKSUM_ERROR;
            }
        }
    }

    m_prevByte = c;

    return m_state;
}

int AIMParser::rx( uint8_t* c, int size )
{
    //Logger::logf( LOG_MSG, _T("AIMParser::rx() - size: %d"), size );

    if( !size )
    {
        //Logger::logf( LOG_MSG, _T("AIMParser::rx() - m_iEmptyCount: %d"), m_iEmptyCount );

        if ( m_iEmptyCount++ < MAX_EMPTY_COUNT )
        {
            return 0;
        }

        m_iEmptyCount = 0;

        if ( m_bNeedDisabled && !m_pCurrentCommand )
        {
//        Logger::logf( LOG_MSG, _T("AIMParser::rx() - Need disable") );

            m_bIsDisabled   = true;
            m_bNeedDisabled = false;
            return 0;
        }
    }

    for( int i = 0; i < size; ++i )
    {
        int state = feed( c[i] );

        if (  state == END )
        {
            //Logger::logf( LOG_MSG, _T(" AIMParser::rx() - END") );

            if ( m_pEventHandler )
            {
                SIPRPacket p( 0, SIPR_AIM );
                InputEvent e;
                e.setPacket( p );

                m_pEventHandler->AddPendingEvent( e );
            }
        }
    }

    return 0; //TODO - enum NO_ERROR,
}

int AIMParser::tx(uint8_t* buff, int& size)
{
    if ( m_pCurrentCommand )
    {
        Logger::logf( LOG_MSG, _T("-----AIMParser::tx() - send set state MAN") );
        size = m_pCurrentCommand->size;
        memcpy( buff, m_pCurrentCommand->data, m_pCurrentCommand->size );
        delete m_pCurrentCommand;
        m_pCurrentCommand = NULL;
        return 0;
    }

    if ( m_txQueue.empty() )
    {
        return 0;
    }

    TxData* txData = m_txQueue.front();
    m_txQueue.pop_front();

    size = txData->size;

    memcpy(buff, txData->data, txData->size);

    delete txData;
    return size;
}

void AIMParser::sendChannel(uint8_t channel)
{
    if(channelDataHash.find(channel) == channelDataHash.end() ) return;

    int value = channelDataHash[channel];
    uint8_t high = (65535 & value) >> 8;
    uint8_t low = (255 & value);

    TxData* tx = new TxData(5);
    uint8_t* txBuff = (uint8_t*)tx->data;

    txBuff[0] = channel;
    txBuff[1] = AIMData::AIM_CONST;
    txBuff[2] = high;
    txBuff[3] = low;

    setSum(tx);

    m_txQueue.push_back(tx); //FIFO
}

void AIMParser::setSum(TxData* tx)
{
    uint8_t size = tx->size;
    uint8_t* txBuff = (uint8_t*)tx->data;

    uint8_t sum = 0;
    for(uint8_t i = 0; i < size-1; i++){
            sum += txBuff[i];
    }
    txBuff[size-1] = sum;
}

void AIMParser::request( VemsCommand* command )
{
//
//    Logger::logf( LOG_MSG, _T("-----AIMParser::request() ") );
//    if ( command->getType() == VemsCommand::SET_STATE )
//    {
//         Logger::logf( LOG_MSG, _T("-----AIMParser::request() SET STATE") );
//        //if ( command->getState() == ECUInfo::MAN )
//        //{
//            //const char* data = "ManbyebyeMan";
//            const char* data = "ManbyebyeMan";
//            m_pCurrentCommand = new TxData( 12 );
//            memcpy( m_pCurrentCommand->data, data, 12 );
//            //const char* data = "mx5";
//            //m_pCurrentCommand = new TxData( 3 );
//            //memcpy( m_pCurrentCommand->data, data, 3 );
//        //}
//    }else{
//         if ( m_pEventHandler )
//            {
//                Logger::logf( LOG_MSG, _T("-----AIMParser::request() - SEND TIME_OUT") );
//                SIPRPacket p( 0, SIPR_TIMEOUT );
//                InputEvent e;
//                e.setPacket( p );
//                m_pEventHandler->AddPendingEvent( e );
//            }
//    }
}
