#include "CommModelGUIMain.h"

#include "UserLog.h"
#include "UserLogWindow.h"
#include "logger.h"
#include "Math.h"
#include "DirHelper.h"
#include "RawLogHelper.h"
#include "MTConfigHelper.h"
#include "SimpleFwDlg.h"
#include "FirmwareTypes.h"
#include "firmwareconfigitem.h"
#include "ProgressBarObserver.h"
#include "SDImageHandler.h"
#include "SerialPortHelper.h"

#include "fileparser/IHexParser.h"

#include "stdatamanager/STDataModelSerial.h"

#include "gui/ECUConfigPanel2.h"
// #include "gui/SDCardDialog.h"
#include "gui/FirmwareVerifyDialog.h"
#include "gui/FirmwareUploadDialog.h"
#include "gui/BootModeDialog.h"
#include "gui/DeviceControllerProgressBarManager.h"

#include "events/InputEvent.h"
#include "events/SDCardEvent.h"
#include "events/VemsDeviceControllerEvent.h"
#include "events/CommunicationEvent.h"
#include "events/OffsetReaderEvent.h"

#include "vemsprotocol/SIPRTypes.h"
#include "vemsprotocol/STDataModelWriter.h"
#include "vemsprotocol/ECUInfo.h"
#include "vemsprotocol/SIPRFileLogger.h"
//#include "vemsprotocol/VemsCommand2.h"
#include "vemsprotocol/VemsDeviceController.h"
#include "vemsprotocol/ConfigDownloadController.h"

#include "communication/clients/ConfigSaver2.h"

#include "configmanager/ConfigHelper2.h"
#include "configmanager/VemsDeviceControllerConfig.h"
#include "configmanager/ConfigMgrVemsDeviceController.h"

#include "triggerlog/TriggerLogEventManager.h"

#include <wx/tokenzr.h>
#include <wx/textdlg.h>
#include <wx/filedlg.h>
#include <wx/textdlg.h>

#include "fileparser/MapFileParser.h"
#include "fileparser/parseheader.h"

//#include "firmwareconfig/FirmwareConfigManager.h"

#include "vemsdialog/DummyUserDialogBase.h"

#include "commmodel/GaugeFrame.h"

/// menuitems
int idMenuECUConfig         = wxNewId();

// int idMenuTestTest          = wxNewId();

int idSlider                = wxNewId();
// int idRestartRT             = wxNewId();
int idStartV3               = wxNewId();


BEGIN_EVENT_TABLE(MyFrame, wxFrame)

    EVT_MENU ( idMenuECUConfig, MyFrame::onECUConfig )

    // EVT_MENU( idMenuTestTest, MyFrame::onTestMenu )

    EVT_MENU( wxID_OPEN, MyFrame::onFileOpen )
    // EVT_MENU( idRestartRT, MyFrame::onRestart )

    EVT_BUTTON( idStartV3, MyFrame::onV3Button )

    EVT_SCROLL( MyFrame::onSliderScroll )

END_EVENT_TABLE()

MyFrame::MyFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title)
{
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();

    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append( wxID_OPEN, _T("Open VEMSLog"), _T("Open VEMSLog"));
    // fileMenu->Append( idRestartRT, _T("Restart real-time"), _T("Restart real-time"));
    mbar->Append( fileMenu, _T("File") );

    ///-----Preferences menu
    wxMenu* settingsMenu = new wxMenu(_(""));
	settingsMenu->Append( idMenuECUConfig, _("E&CU Communication\t"), _("ECU Communication"));
    mbar->Append(settingsMenu, _("&Preferences"));

    //wxMenu* testMenu = new wxMenu(_T(""));
    // testMenu->Append( idMenuReadPage, _T("Read Page 1\tF2"), _T("Read Page 1"));
    //testMenu->Append( idMenuTestTest, _T("Test (kkrisz)"), _T("Test (kkrisz)") );
    //mbar->Append(testMenu, _("&Test"));

    SetMenuBar(mbar);

    m_v3Online = false;

    wxPanel* mainPanel = new wxPanel( this );
    mainPanel->SetSizer( new wxBoxSizer( wxVERTICAL ) );

    ///toolbar

    wxBoxSizer* buttons = new wxBoxSizer( wxHORIZONTAL );

    m_filePosSlider = new wxSlider( mainPanel, idSlider, 0, 0, 100, wxDefaultPosition, wxSize( 280, 30 ) );
    buttons->Add( m_filePosSlider, 0, wxALL, 5 );
    m_filePosSlider->Disable();
    m_startStopButton = new wxButton( mainPanel, idStartV3, _T("Start/Stop V3") );
    buttons->Add( m_startStopButton, 0, wxALL, 5 );

    mainPanel->GetSizer()->Add( buttons );

    ///log window
    UserLogWindow* logWindow = new UserLogWindow( mainPanel, wxID_ANY );
    mainPanel->GetSizer()->Add( logWindow, 1, wxEXPAND );
    UserLog::Init( logWindow );

    startDevice();
}

MyFrame::~MyFrame()
{
	m_controller->stop();

	if ( m_progressBarManager )
        delete m_progressBarManager;

	if ( m_controller )
        delete m_controller;

	delete m_model;

    if ( m_configSaver )
        delete m_configSaver;

	if ( m_configDownloadController )
        delete m_configDownloadController;
}

void MyFrame::startDevice()
{
        ///ConfigDownloadController
    m_configDownloadController = NULL;

    ///ConfigSaver2
    m_configSaver = new ConfigSaver2();
    m_configDownloadController = new ConfigDownloadController();

    ///device controller
    VemsDeviceControllerConfig deviceControllerConfig;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrVemsDeviceController cfgMgr(cfg);
    cfgMgr.load(&deviceControllerConfig);

	MTConfigData* mtConfig = MTConfigHelper::getMTConfigData();
    SymbolTable& symbolTbl = mtConfig->getSymbolTable();
    SIPRDescriptor* siprDesc = mtConfig->getSIPRDescriptor();

	m_model = new STDataModelSerial( symbolTbl, siprDesc );

	// m_siprFileLogger = new SIPRFileLogger( 512 );

	m_controller = new VemsDeviceController( deviceControllerConfig );

	m_controller->Connect( VDC_EVENT_DEVICE_FOUND, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceFound ), NULL, this );
	m_controller->Connect( VDC_EVENT_DEVICE_NOT_FOUND, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceNotFound ), NULL, this );
    m_controller->Connect( VDC_EVENT_DEVICE_CONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceConnected ), NULL, this );
    m_controller->Connect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceDisconnected ), NULL, this );
    m_controller->Connect( VDC_EVENT_STARTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceControllerStarted ), NULL, this );
    m_controller->Connect( VDC_EVENT_STOPPED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceControllerStopped ), NULL, this );

    // set baudrate to 115200 -> then start reading
    m_controller->Connect( VDC_EVENT_DEVICE_BAUDRATE_CHANGED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceBaudRateChanged ), NULL, this );

    //m_controller->Connect( VDC_EVENT_RESUMED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceResumed ), NULL, this );

    m_controller->Connect( CommunicationEvent::CANNOT_OPEN_PORT, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCannotOpenPort ), NULL, this );
    m_controller->Connect( CommunicationEvent::COMMUNICATION_ERROR, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCommunicationError ), NULL, this );
    m_controller->Connect( CommunicationEvent::TIMEOUT, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCommunicationError ), NULL, this );

	m_controller->Connect( wxEVT_INPUT, InputEventHandler( MyFrame::onInputEvent ), NULL, this );
	Connect( wxEVT_INPUT, InputEventHandler( MyFrame::onInputEvent ), NULL, this );

    m_progressBarManager = new DeviceControllerProgressBarManager( this, m_controller );

    m_dummyUserDialogBase = new DummyUserDialogBase( this );

    //m_pFirmwareConfigManager = new FirmwareConfigManager( this, UserLog::GetObserver(), m_dummyUserDialogBase );

    m_gaugeFrame = NULL;

    m_controller->start();
}

void MyFrame::notify( int i, void* ptr )
{
    if ( ptr == m_gaugeFrame )
    {
        m_filePosSlider->SetRange( 0, i );
        m_filePosSlider->SetValue( 0 );
    }
}

void MyFrame::onSliderScroll( wxScrollEvent& event )
{
    if ( m_gaugeFrame )
        m_gaugeFrame->refreshIdx( m_filePosSlider->GetValue() );
}

void MyFrame::onFileOpen( wxCommandEvent& event )
{
    wxFileDialog dlg( this, _T("Choose VEMS Log file"), DirHelper::GetPath( dLOG ), _T(""), _T("Vemslog files (*.vemslog)|*.vemslog|All files (*.*)|*.*") );
    if ( dlg.ShowModal() == wxID_OK )
    {
        reloadGauges( NULL, NULL );
        m_gaugeFrame->loadLogFile( dlg.GetPath() );
        m_filePosSlider->Enable();
    }
}

void MyFrame::onECUConfig( wxCommandEvent & event )
{
    ECUConfigPanel::findAndShow( this, m_controller );
}

void MyFrame::onCannotOpenPort( CommunicationEvent& event )
{
    wxString message = _T("Cannot Open Port!"); //wxString::Format( _T("Cannot Open Port! ( )") + event.get );
    UserLog::uLog( ULOG_ERROR, message );
}

void MyFrame::onCommunicationError( CommunicationEvent& event )
{
    UserLog::uLog( ULOG_ERROR, _T("Communication Error!") );
}

void MyFrame::onDeviceFound( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("Device Found!") );
}

void MyFrame::onDeviceNotFound( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("Device Not Found!") );
}

void MyFrame::onDeviceConnected( VemsDeviceControllerEvent& event )
{
    if ( event.isBootMode() ) {
        BootModeDialog dlg(this);

        switch ( dlg.ShowModal() )
        {
            case BootModeDialog::BMD_STAY:
                UserLog::uLog( ULOG_OK, _T("Device is in Boot mode!") );
                break;

            case BootModeDialog::BMD_LEAVE:
                m_controller->leaveBootMode();
                break;
        }
    } else {
        UserLog::uLog( ULOG_OK, wxString::Format( _T("Device Connected!, Baudrate = %d"), event.getBaudrate() ));
        m_v3Online = true;
        reloadGauges( MTConfigHelper::getMTConfigData(), m_controller );
        m_startStopButton->SetLabel( _T("Stop") );
    }
}

void MyFrame::reloadGauges(  MTConfigData * mtconfigdata, VemsDeviceController* controller )
{
    /// show Gauges & start Logging
    if ( m_gaugeFrame )
    {
        m_gaugeFrame->unRegisterObserver( this );
        m_gaugeFrame->Destroy();
    }

    if ( mtconfigdata != NULL && controller != NULL )
    {
        //m_controller->changeBaudRate( 115200 );
        m_gaugeFrame = new GaugeFrame(this, MTConfigHelper::getMTConfigData(), m_controller, MTConfigHelper::m_commodelAddresses );
        m_gaugeFrame->registerObserver( this );

    }
    else
    {
        m_gaugeFrame = new GaugeFrame( this );
        m_gaugeFrame->registerObserver( this );
    }
}

void MyFrame::onDeviceBaudRateChanged( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_OK, wxString::Format( _T("Baudrate changed to %d"), event.getBaudrate() ) );
}

void MyFrame::onDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("Device Disconnected!") );
    event.Skip();
}

void MyFrame::onDeviceControllerStarted( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("DeviceController Started!") );

}

void MyFrame::onDeviceControllerStopped( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("DeviceController Stopped!") );
}

void MyFrame::onInputEvent( InputEvent& event )
{
    const SIPRPacket& packet = event.getPacket();

    switch ( packet.type() )
    {
        case SIPR_READ_SENSORS:
        case SIPR_UPDATE_FREQUENCY:
            break;

        case SIPR_GET_TABLEDATA:
        case SIPR_GET_CONFIG_1:
        case SIPR_GET_CONFIG_2:
            m_model->process( packet );
            break;

        default:
            break;
    }
}

void MyFrame::onConfigDownloadEvent( InputEvent& event )
{
    const SIPRPacket& packet = event.getPacket();

    switch ( packet.type() )
    {
        case SIPR_TABLES_END:
            m_configDownloadController->Disconnect( wxEVT_INPUT, InputEventHandler( MyFrame::onConfigDownloadEvent ), NULL, this );
            m_configDownloadController->Disconnect( wxEVT_INPUT, InputEventHandler( ConfigSaver2::onInputEvent ), NULL, m_configSaver );
//            m_controller->start();
            m_controller->resume();
            break;
    }
}

void MyFrame::onAIMButton( wxCommandEvent& event )
{
    startDeviceAIM();

    m_gaugeFrame = new GaugeFrame(this, MTConfigHelper::getMTConfigData() );

    m_gaugeFrame->registerObserver( this );

}

void MyFrame::onV3Button( wxCommandEvent& event )
{
    if ( m_v3Online )
    {
        m_controller->stop();
        m_startStopButton->SetLabel( _T("Start") );
        m_v3Online = false;
    }
    else
    {
        m_gaugeFrame->saveGauges(_T("gauges_temp.xml"));
        m_gaugeFrame->Hide();
        startDevice();
    }

}
void MyFrame::onTestMenu( wxCommandEvent& event )
{
    AIMSendPanel* f = new AIMSendPanel(this);
    f->Show();

    f->SetPacketSinker(this);


    VemsDeviceControllerConfig deviceControllerConfig;

    m_aimController = new AIMDeviceController( deviceControllerConfig, this );

    f->SetAIMDeviceController(m_aimController);

//
//
//    MTConfigData* mtConfig = MTConfigHelper::getMTConfigData();
//
//    MTConfigHelper::parseAIMConfig(mtConfig);
//
//    ///device controller
//
////    VemsDeviceControllerConfig deviceControllerConfig;
////    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
////    ConfigMgrVemsDeviceController cfgMgr(cfg);
////    cfgMgr.load(&deviceControllerConfig);
////    m_aimController = new AIMDeviceController( deviceControllerConfig, this );
////    m_aimController->start();
////
//
//    Connect( wxEVT_INPUT, InputEventHandler( MyFrame::onInputEvent ), NULL, this );
//
//
//    m_gaugeFrame = new GaugeFrame(this, MTConfigHelper::getMTConfigData() );
//

}

/*
void MyFrame::onTestMenu( wxCommandEvent& event )
{
#ifndef COMM_MODEL_WITH_GUI
    // parsing map file
    MapFileParser parser;
    LOGP( _T("Parsing MAP file..."))
    parser.parse( DirHelper::GetPath(dLOG) + _T("test\\vems.map") );
    LOGP( _T("Parsing MAP file finished"))

    long engine_addr;

    {
        MapSymbolHash mapHash = parser.getMapSymbols();
        MapSymbolHash::iterator it = mapHash.find( _T("engine") );

        if ( it == mapHash.end() )
        {
            LOGP( _T("Engine struct not found in map file!") )
            return;
        }
        else
        {
            engine_addr = it->second.address;
        }
    }

    // parsing global.h
    LOGP( _T("Parsing global.h..."))
    SymbolTable* symTable = new SymbolTable;

    ParseHeader globalHParser( *symTable );
    globalHParser.parse( DirHelper::GetPath(dLOG) + _T("test\\global.h") );
    LOGP( _T("Parsing global.h finished"))

    long engine_t_tps_offset;
    {
        SymbolTable::iterator it = symTable->find( _T("engine_t_tps") );

        if ( it == symTable->end() )
        {
            LOGP( _T("engine_t_tps not found in global.h") )
            return;
        }
        else
        {
            Symbol* sym = it->second;
            engine_t_tps_offset = sym->getOffset();
        }
    }

    int tps_addr = engine_addr + engine_t_tps_offset;

    wxString tpsHex = wxString::Format( _T("%x"), tps_addr );
    LOGP( _T("TPS addr:")+ tpsHex )

    StringHash h;

    h[ _T("engine_t_tps")] = tpsHex;

    m_controller->readOffset( true, h, this );
#else

    // GaugeFrame* frame = new GaugeFrame(this,MTConfigHelper::getMTConfigData(), m_controller );
#endif
}
*/
