#include "CommunicationModelMainFrame.h"
#include "UserLog.h"
#include "UserLogWindow.h"
#include "logger.h"
#include "Math.h"
#include "DirHelper.h"
#include "RawLogHelper.h"
#include "MTConfigHelper.h"
#include "SimpleFwDlg.h"
#include "FirmwareTypes.h"
#include "firmwareconfigitem.h"
#include "ProgressBarObserver.h"
#include "SDImageHandler.h"
#include "SerialPortHelper.h"

#include "stdatamanager/STDataModelSerial.h"

#include "gui/ECUConfigPanel2.h"
#include "gui/SDCardDialog.h"
#include "gui/BootModeDialog.h"
#include "gui/DeviceControllerProgressBarManager.h"
#include "gui/ConfigDownloadDirector.h"
#include "gui/FirmwareDirector.h"

#include "events/InputEvent.h"
#include "events/SDCardEvent.h"
#include "events/VemsDeviceControllerEvent.h"
#include "events/CommunicationEvent.h"
#include "events/OffsetReaderEvent.h"

#include "vemsprotocol/SIPRTypes.h"
#include "vemsprotocol/STDataModelWriter.h"
#include "vemsprotocol/ECUInfo.h"
#include "vemsprotocol/SIPRFileLogger.h"
#include "vemsprotocol/VemsDeviceController.h"
//#include "vemsprotocol/ConfigDownloadController.h"
//
//#include "communication/clients/ConfigSaver2.h"

#include "configmanager/ConfigHelper2.h"
#include "configmanager/VemsDeviceControllerConfig.h"
#include "configmanager/ConfigMgrVemsDeviceController.h"

#include "triggerlog/TriggerLogEventManager.h"

#include <wx/tokenzr.h>
#include <wx/textdlg.h>
#include <wx/filedlg.h>
#include <wx/textdlg.h>

#include "fileparser/MapFileParser.h"
#include "fileparser/parseheader.h"

#include "firmwareconfig/FirmwareConfigManager.h"

#include "vemsdialog/DummyUserDialogBase.h"

#ifdef COMM_MODEL_WITH_GUI
#include "commmodel/GaugeFrame.h"
#endif


//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

/// menuitems
int idMenuECUConfig         = wxNewId();
int idMenuFirmwareVerify    = wxNewId();
int idMenuFirmwareUpload    = wxNewId();

int idMenuConfigDownload    = wxNewId();
int idMenuConfigUpload      = wxNewId();
int idMenuFilteredConfigUpload = wxNewId();
int idMenuConfigVerify      = wxNewId();
int idMenuSDCardV3          = wxNewId();
int idMenuSDCardOpenImage   = wxNewId();
int idMenuReadPage          = wxNewId();
int idMenuSDCardStart       = wxNewId();
int idMenuSDCardStop        = wxNewId();
int idMenuSDCardErase       = wxNewId();
int idMenuTriggerLog        = wxNewId();

int idMenuTestTest          = wxNewId();

/// toolbar items
int idButtonDevice              = wxNewId();
int idButtonDeviceMode          = wxNewId();
int idComboBoxBaudrates         = wxNewId();
int idReadOffsetButton          = wxNewId();

BEGIN_EVENT_TABLE(MyFrame, wxFrame)

    /// MenuItemEvents
    EVT_MENU( idMenuFirmwareVerify, MyFrame::onFirmwareVerify )
    EVT_MENU( idMenuFirmwareUpload, MyFrame::onFirmwareUpload )
    EVT_MENU( idMenuConfigDownload, MyFrame::onConfigDownload )
    EVT_MENU( idMenuConfigUpload, MyFrame::onConfigUpload )
    EVT_MENU( idMenuConfigVerify, MyFrame::onConfigVerify )

    EVT_MENU( idMenuSDCardV3, MyFrame::onSDCardSave )
    EVT_MENU( idMenuSDCardOpenImage, MyFrame::onSDCardSave )
    EVT_MENU ( idMenuECUConfig, MyFrame::onECUConfig )
    EVT_MENU ( idMenuReadPage, MyFrame::onReadPage )
    EVT_MENU( idMenuSDCardStart, MyFrame::onSDAction )
    EVT_MENU( idMenuSDCardStop, MyFrame::onSDAction )
    EVT_MENU( idMenuSDCardErase, MyFrame::onSDAction )
    EVT_MENU( idMenuTriggerLog, MyFrame::onTriggerLog )

    EVT_MENU( idMenuTestTest, MyFrame::onTestMenu )


    /// Toolbar item events
    EVT_BUTTON( idButtonDevice, MyFrame::onDeviceButton )
    EVT_COMBOBOX( idComboBoxBaudrates, MyFrame::onBaudrateComboBox )
    EVT_TOGGLEBUTTON( idReadOffsetButton, MyFrame::onReadOffset )

    EVT_SDCARD( MyFrame::onSDCardEvent )
    EVT_OFFSETREADER( MyFrame::onOffsetREvent )
END_EVENT_TABLE()

MyFrame::MyFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title)
{
    m_sdcardDlg = new wxProgressDialog( _T(""), _T(""), 100, this );
    m_sdcardDlg->Hide();

#if wxUSE_MENUS
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();

    ///-----Preferences menu
    wxMenu* settingsMenu = new wxMenu(_(""));
	settingsMenu->Append( idMenuECUConfig, _("E&CU Communication\t"), _("ECU Communication"));
    mbar->Append(settingsMenu, _("&Preferences"));

    ///-----Firmware menu
    wxMenu* firmwareMenu = new wxMenu ( _T("") );
	firmwareMenu->Append( idMenuFirmwareVerify, _T("Firmware Verify"), _T("Firmware Verify") );
	firmwareMenu->Append( idMenuFirmwareUpload, _T("Firmware Upload"), _T("Firmware Upload") );
	firmwareMenu->AppendSeparator();
	firmwareMenu->Append( idMenuConfigDownload, _T("Config Download"), _T("Config Download") );
	firmwareMenu->AppendSeparator();
    firmwareMenu->Append( idMenuConfigUpload, _("Config upload"), _("Config upload") );
//    firmwareMenu->Enable( idMenuConfigUpload, false );
    firmwareMenu->Append( idMenuFilteredConfigUpload, _("Filtered config upload"), _("Filtered config upload") );
    firmwareMenu->Enable( idMenuFilteredConfigUpload, false );
    firmwareMenu->Append( idMenuConfigVerify, _("Config verify"), _("Config verify") );
    mbar->Append( firmwareMenu, _T("Firmware") );

    ///-----SDCard menu
    wxMenu* sdcardmenu = new wxMenu( _T("") );
    sdcardmenu->Append( idMenuSDCardV3, _T("Search SD files"), _T("Search SD files") );
    sdcardmenu->Append( idMenuSDCardOpenImage, _T("Open image file"), _T("Open image file") );
    sdcardmenu->Append( idMenuSDCardStart, _T("Start log"), _T("Start log") );
    sdcardmenu->Append( idMenuSDCardStop, _T("Stop log"), _T("Stop log") );
    sdcardmenu->Append( idMenuSDCardErase, _T("Erase SD Card"), _T("Erase SD Card") );
    mbar->Append( sdcardmenu, _T("SD Card") );

    ///-----Trigger Log menu
    wxMenu* triggerLogMenu = new wxMenu( _T("") );
    triggerLogMenu->Append( idMenuTriggerLog, _T("TriggerLog"), _T("TriggerLog") );
    mbar->Append( triggerLogMenu, _T("TriggerLog") );

    wxMenu* testMenu = new wxMenu(_T(""));
    testMenu->Append( idMenuReadPage, _T("Read Page 1\tF2"), _T("Read Page 1"));
    testMenu->Append( idMenuTestTest, _T("Test (kkrisz)"), _T("Test (kkrisz)") );

    mbar->Append(testMenu, _("&Test"));

    SetMenuBar(mbar);

    enableFirmwareMenu( false );

    int firmwareMenuIdx = mbar->FindMenu( _T("Firmware") );
    mbar->EnableTop( firmwareMenuIdx, false );
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
    CreateStatusBar(4);
    resetStatusBar();
#endif // wxUSE_STATUSBAR

    wxPanel* mainPanel = new wxPanel( this );
    mainPanel->SetSizer( new wxBoxSizer( wxVERTICAL ) );

    ///toolbar
    wxButton* deviceButton = new wxButton( mainPanel, idButtonDevice, _T("N/A") );
    deviceButton->Enable( false );

    wxButton* deviceModeButton = new wxButton( mainPanel, idButtonDeviceMode, _T("N/A") );
    deviceModeButton->Enable( false );

    m_readOffsetButton = new wxToggleButton( mainPanel, idReadOffsetButton, _T("Read offset") );

    wxArrayString baudrates = SerialPortHelper::getSerialBaudrates();

    wxComboBox* baudrateComboBox = new wxComboBox(  mainPanel,
                                                    idComboBoxBaudrates,
                                                    _T("19200"),
                                                    wxDefaultPosition,
                                                    wxDefaultSize,
                                                    baudrates,
                                                    wxCB_READONLY );
    baudrateComboBox->Enable( false );

    wxBoxSizer* toolbar = new wxBoxSizer( wxHORIZONTAL );
    toolbar->Add( deviceButton, 0, wxRIGHT, 5 );
    toolbar->Add( deviceModeButton, 0, 0, 0 );
    toolbar->Add( baudrateComboBox, 0, wxLEFT | wxALIGN_CENTRE_VERTICAL, 5 );
    toolbar->Add( m_readOffsetButton, 0, wxLEFT, 5 );
    mainPanel->GetSizer()->Add( toolbar, 0, wxEXPAND | wxALL, 5 );

    ///log window
    UserLogWindow* logWindow = new UserLogWindow( mainPanel, wxID_ANY );
    mainPanel->GetSizer()->Add( logWindow, 1, wxEXPAND );
    UserLog::Init( logWindow );

    ///device controller
    VemsDeviceControllerConfig deviceControllerConfig;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrVemsDeviceController cfgMgr(cfg);
    cfgMgr.load(&deviceControllerConfig);

	MTConfigData* mtConfig = MTConfigHelper::getMTConfigData();
    SymbolTable& symbolTbl = mtConfig->getSymbolTable();
    SIPRDescriptor* siprDesc = mtConfig->getSIPRDescriptor();

	m_model = new STDataModelSerial( symbolTbl, siprDesc );

	m_siprFileLogger = new SIPRFileLogger( 512 );

	m_controller = new VemsDeviceController( deviceControllerConfig );

	m_controller->Connect( VDC_EVENT_DEVICE_FOUND, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceFound ), NULL, this );
	m_controller->Connect( VDC_EVENT_DEVICE_NOT_FOUND, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceNotFound ), NULL, this );
    m_controller->Connect( VDC_EVENT_DEVICE_CONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceConnected ), NULL, this );
    m_controller->Connect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceDisconnected ), NULL, this );
    m_controller->Connect( VDC_EVENT_STARTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceControllerStarted ), NULL, this );
    m_controller->Connect( VDC_EVENT_STOPPED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceControllerStopped ), NULL, this );
    m_controller->Connect( VDC_EVENT_DEVICE_BAUDRATE_CHANGED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceBaudRateChanged ), NULL, this );
    m_controller->Connect( VDC_EVENT_RESUMED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( MyFrame::onDeviceResumed ), NULL, this );

    m_controller->Connect( CommunicationEvent::CANNOT_OPEN_PORT, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCannotOpenPort ), NULL, this );
    m_controller->Connect( CommunicationEvent::COMMUNICATION_ERROR, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCommunicationError ), NULL, this );
    m_controller->Connect( CommunicationEvent::TIMEOUT, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCommunicationError ), NULL, this );

	m_controller->Connect( wxEVT_INPUT, InputEventHandler( MyFrame::onInputEvent ), NULL, this );
	Connect( wxEVT_INPUT, InputEventHandler( MyFrame::onInputEvent ), NULL, this );

    m_progressBarManager = new DeviceControllerProgressBarManager( this, m_controller );

    m_dummyUserDialogBase = new DummyUserDialogBase( this );

    m_pFirmwareConfigManager = new FirmwareConfigManager( this, m_controller, UserLog::GetObserver(), m_dummyUserDialogBase );

    m_configDownloadDirector = new ConfigDownloadDirector( this, m_controller );
    m_firmwareDirector = new FirmwareDirector( this, m_controller );

    m_controller->start();
}

MyFrame::~MyFrame()
{
	m_controller->stop();

	delete m_progressBarManager;
	delete m_controller;
	delete m_model;
	delete m_siprFileLogger;

    delete m_configDownloadDirector;
	delete m_pFirmwareConfigManager;
}

void MyFrame::startLogging()
{
    m_siprFileLogger->enableLog(true);

    const FirmwareInfo& fwInfo = m_controller->getFirmwareInfo();

    wxString baseFileName = fwInfo.getSerialNumber() + _T(".vemslog");
    wxString logdir = DirHelper::GetPath(dLOG);
    wxString fileName = DirHelper::CreateNewFileName(baseFileName,logdir,true,true);

    wxString header = RawLogHelper::getVemslogHeader(fwInfo, 2);

    m_siprFileLogger->setHeader(header);
    m_siprFileLogger->setFile(fileName);

    MTConfigData* mtConfig = MTConfigHelper::getMTConfigData();
    SIPRDescriptor* siprDesc = mtConfig->getSIPRDescriptor();

    STDataModelWriter::logPages( m_model, m_siprFileLogger, siprDesc );

    LOGP( _T("header: ") + header );
    LOGP( _T("fileName: ") + fileName );

    UserLog::uLog( ULOG_OK, fileName );

    wxArrayString lines = wxStringTokenize( header, _T("\n") );
    for ( size_t i = 0; i < lines.GetCount(); ++i ){
        UserLog::uLog( ULOG_OK, lines[i] );
    }
}

void MyFrame::stopLogging()
{
    m_siprFileLogger->enableLog(false);
}

void MyFrame::refreshToolbar( const VemsDeviceControllerEvent& event )
{
    LOG

    wxString deviceString = _T("N/A");
    wxString deviceModeString = _T("N/A");

    int eventId = event.GetId();

    wxColor connectedColor( 0, 128, 0 );

    switch ( eventId )
    {
        case VDC_EVENT_DEVICE_NOT_FOUND:
        case VDC_EVENT_DEVICE_DISCONNECTED:
        {
            refreshToolbarButton( idButtonDevice, deviceString, *wxRED, true );
            refreshToolbarButton( idButtonDeviceMode, deviceModeString, *wxRED, true );
            refreshBaudrateComboBox( false, 0 );
        } break;

        case VDC_EVENT_DEVICE_CONNECTED:
        case VDC_EVENT_RESUMED:
        {
            deviceString = ECUInfo::getDeviceAsString( event.getDevice() );
            deviceModeString = ECUInfo::getStateAsString( event.getDeviceMode() );

            refreshToolbarButton( idButtonDevice, deviceString, connectedColor, true );
            refreshToolbarButton( idButtonDeviceMode, deviceModeString, connectedColor, true );
            refreshBaudrateComboBox( true, event.getBaudrate() );
        } break;

        case VDC_EVENT_STOPPED:
        {
            refreshToolbarButton( idButtonDevice, deviceString, *wxLIGHT_GREY, true );
            refreshToolbarButton( idButtonDeviceMode, deviceModeString, *wxLIGHT_GREY, true );
            refreshBaudrateComboBox( false, 0 );
        } break;

        default:
            LOGP( wxString::Format( _T("Unhandled EventId - %d!"), eventId ) );
            break;
    }
}

void MyFrame::refreshToolbarButton( int buttonId, const wxString& label, const wxColour& color, bool isEnabled )
{
    wxButton* button = (wxButton*)wxWindow::FindWindowById( buttonId );

    if ( !button ) {
        LOGP( wxString::Format( _T("Button not found! - buttonId: %d"), buttonId ) );
    }

    button->SetForegroundColour( color );
    button->Enable( true );
    button->SetLabel( label );
}

void MyFrame::refreshBaudrateComboBox( bool isEnabled, size_t baudrate )
{
    wxComboBox* baudrateComboBox = (wxComboBox*)wxWindow::FindWindowById( idComboBoxBaudrates );
    baudrateComboBox->Enable( isEnabled );
    int itemIdx = baudrateComboBox->FindString( wxString::Format( _T("%d"), baudrate ) );
    if ( itemIdx != wxNOT_FOUND ) {
        baudrateComboBox->Select( itemIdx );
    }
}

void MyFrame::resetStatusBar()
{
    wxString secl( _T("secl: N/A") );
    wxString tps( _T("tps: N/A") );
    wxString tps_engine( _T("engine_t_tps: N/A") );
    wxString ups( _T("Update frequency: N/A") );

    SetStatusText( secl, 0 );
    SetStatusText( tps, 1 );
    SetStatusText( tps_engine, 2 );
    SetStatusText( ups, 3 );
}

void MyFrame::refreshStatusBar( const SIPRPacket& packet )
{
#if wxUSE_STATUSBAR
    uint8_t* data = packet.data();

    if ( !data ) {
        LOGP( _T("data is NULL!") );
        return;
    }

    switch ( packet.type() )
    {
        case SIPR_READ_SENSORS:
        {
            wxString secl( wxString::Format( _T("secl: %3d"), data[3] ) );
            wxString tps( wxString::Format( _T("tps: %3d"), data[11] ) );

            SetStatusText( secl, 0 );
            SetStatusText( tps, 1 );
        } break;

        case SIPR_UPDATE_FREQUENCY:
        {
            int updateFrequency = Math::getNumber( data );
            wxString ups( wxString::Format( _T("Update frequency: %3d"), updateFrequency ) );

            SetStatusText( ups, 3 );
        } break;

        default:
            break;
    }
#endif
}

void MyFrame::refreshMenuState()
{
//    wxMenuBar* mbar = GetMenuBar();

    if ( m_controller->isConnected() ) {
        enableFirmwareMenu( true );
        //mbar->Enable( idMenuFirmwareVerify )


    } else {
        enableFirmwareMenu( false );
    }
}

void MyFrame::enableFirmwareMenu( bool isEnabled  )
{
    wxMenuBar* mbar = GetMenuBar();
    int firmwareMenuIdx = mbar->FindMenu( _T("Firmware") );
    mbar->EnableTop( firmwareMenuIdx, isEnabled );

//    // disable SD Card file search on v3 too :)
//    int item = mbar->FindMenuItem(_T("SD Card"), _T("Search SD files"));
//    mbar->FindItem(item)->Enable( isEnabled );
}

void MyFrame::onECUConfig( wxCommandEvent & event )
{
    ECUConfigPanel::findAndShow( this, m_controller );
}

void MyFrame::onFirmwareVerify( wxCommandEvent& event )
{
    m_firmwareDirector->verifyFirmware();
}

void MyFrame::onFirmwareUpload( wxCommandEvent& event )
{
    m_firmwareDirector->uploadFirmware();
}

void MyFrame::onConfigDownload( wxCommandEvent& event )
{
    m_configDownloadDirector->saveDeviceConfig(true);
}

void MyFrame::onSDAction( wxCommandEvent& event )
{
    int id = event.GetId();

    if ( id == idMenuSDCardStart )
    {
        m_controller->SDCardLogging( 0, this );
    }
    if ( id == idMenuSDCardStop )
    {
        m_controller->SDCardLogging( 1, this );
    }
    if ( id == idMenuSDCardErase )
    {
        m_controller->SDCardLogging( 2, this );
    }
}

void MyFrame::onTriggerLog( wxCommandEvent& event )
{
    new TriggerLogEventManager( this, m_controller );
}

void MyFrame::onSDCardEvent( SDCardEvent& event )
{
    switch ( event.GetType() )
    {
        case SDCardEventTypes::COMM_FAILED:
            wxMessageBox( _T("SDCard - Communication failed"), _T("ERROR!"), wxOK | wxICON_ERROR );
            m_controller->start();
            break;

        case SDCardEventTypes::TIMEOUT:
            wxMessageBox( _T("SDCard - Timeout"), _T("Timeout!"), wxOK | wxICON_HAND );
            m_controller->start();
            break;

        case SDCardEventTypes::LOGGING_STOPPED:
            wxMessageBox( _T("SDCard - Logging stopped"), _T("Info"), wxOK | wxICON_INFORMATION );
            m_controller->start();
            break;

        case SDCardEventTypes::LOGGING_STARTED:
        {
            m_sdcardDlg->Update( 0 ); m_sdcardDlg->Hide();
            wxMessageBox( _T("SDCard - Logging started"), _T("Info"), wxOK | wxICON_INFORMATION );
            m_controller->start();
            break;
        }

        case SDCardEventTypes::STARTLOG_WAITING:
        {
            if ( m_sdcardDlg->IsShown() == false )
                m_sdcardDlg->Show( true );

            m_sdcardDlg->Update( event.GetInt(), _T("SDCard startlog. Waiting...") );
            break;
        }

        case SDCardEventTypes::STARTLOG_FAILED:
        {
            m_sdcardDlg->Update( 0 ); m_sdcardDlg->Hide();
            wxMessageBox( _T("SDCard - Startlog failed"), _T("Error"), wxOK | wxICON_HAND );
            m_controller->start();
            break;
        }

        case SDCardEventTypes::STOPLOG_FAILED:
        {
            m_sdcardDlg->Update( 0 ); m_sdcardDlg->Hide();
            wxMessageBox( _T("SDCard - Stoplog failed"), _T("Error"), wxOK | wxICON_HAND );
            m_controller->start();
            break;
        }

        case SDCardEventTypes::ERASE_SUCCESS:
        {
            m_sdcardDlg->Update( 0 ); m_sdcardDlg->Hide();
            wxMessageBox( _T("SDCard - SDCard empty"), _T("Info"), wxOK | wxICON_INFORMATION );
            m_controller->start();
            break;
        }

        case SDCardEventTypes::ERASE_FAILED:
        {
            m_sdcardDlg->Update( 0 ); m_sdcardDlg->Hide();
            wxMessageBox( _T("SDCard - Erase failed"), _T("Error"), wxOK | wxICON_ERROR );
            m_controller->start();
            break;
        }

        case SDCardEventTypes::ERASE_WAITING:
        {
            if ( m_sdcardDlg->IsShown() == false )
                m_sdcardDlg->Show( true );

            m_sdcardDlg->Update( event.GetInt(), _T("Checking SD card") );
            break;
        }

        default: break;
    }
}

void MyFrame::onSDCardSave( wxCommandEvent& event )
{
    int id = event.GetId();

    MTConfigData* mtConfig = MTConfigHelper::getMTConfigData();

    FirmwareInfo fwinfo = m_controller->getFirmwareInfo();

    SDCardDialog* dialog = NULL;

    if ( id == idMenuSDCardV3 )
    {
        dialog = new SDCardDialog( this, &fwinfo, m_model, mtConfig, m_controller );
    }
    else if ( id == idMenuSDCardOpenImage )
    {
        wxFileDialog dlg( this, _T("Choose SD Card image file"), DirHelper::GetPath( dLOG ) );
        if ( dlg.ShowModal() == wxID_OK )
        {
            wxString fname = dlg.GetPath();

            SDImageHandler* handler = new SDImageHandler( fname );

            dialog = new SDCardDialog( this, &fwinfo, m_model, mtConfig, handler );
        }
        else return;
    }
    else {
        return;
    }

    dialog->ShowModal();
}

void MyFrame::onReadOffset( wxCommandEvent& event )
{
    wxButton* button = dynamic_cast<wxButton*>( wxWindow::FindWindowById( idButtonDevice ) );

    if ( !button )
        return;

    if ( button->GetLabel() == _T("N/A") )
        return;

    if ( m_readOffsetButton->GetValue() == true )
    {
        /*wxTextEntryDialog dlg( this, _T("Entry offset:"), _T("Entry offset"), _T("") );
        int ret = dlg.ShowModal();

        if ( ret == wxOK && dlg.GetValue() != wxEmptyString )
        {
            int offset = Math::hex2int( dlg.GetValue() );
            m_controller->readOffset( true, offset );
        }*/

        wxArrayString offsets; offsets.Add(_T("14")); offsets.Add(_T("16"));

        m_controller->readOffset( true, offsets );

    }
    else
    {
        m_controller->readOffset( false );
    }
}

void MyFrame::onReadPage( wxCommandEvent& event )
{
    LOG
//    m_controller->request( new ReadPageCmd(5) );
}

void MyFrame::onDeviceButton( wxCommandEvent& event )
{
    LOG
    wxButton* deviceButton = (wxButton*)wxWindow::FindWindowById( idButtonDevice );
    deviceButton->Enable( false );

    if ( m_controller->isWorking() ) {
        m_controller->stop();
    } else {
//        m_controller->start();
        m_controller->resume();
    }

}

void MyFrame::onBaudrateComboBox( wxCommandEvent& event )
{
    LOG
    refreshBaudrateComboBox( false, 0 );
    unsigned long baudrate;
    event.GetString().ToULong( &baudrate);
    m_controller->changeBaudRate( baudrate );
}

void MyFrame::onCannotOpenPort( CommunicationEvent& event )
{
    wxString message = _T("Cannot Open Port!"); //wxString::Format( _T("Cannot Open Port! ( )") + event.get );
    UserLog::uLog( ULOG_ERROR, message );
}

/**
    m_controler
*/

void MyFrame::onCommunicationError( CommunicationEvent& event )
{
    UserLog::uLog( ULOG_ERROR, _T("Communication Error!") );
    //wxMessageBox( _T("Please, plug off and plug in  USB-TO-SERIAL converter!") );

//    switch ( event.GetId() )
//    {
//        case CommunicationEvent::TIMEOUT:
//        {
//            wxObject* eventObject = event.GetEventObject();
//
//            if ( eventObject ) {
//                if ( eventObject->IsKindOf(CLASSINFO(ConfigDownloadController)) ) {
//                    LOG
//                        m_configDownloadController->Disconnect( CommunicationEvent::TIMEOUT, wxEVT_COMMUNICATION, CommunicationEventHandler( MyFrame::onCommunicationError ), NULL, this );
//
//                        m_configDownloadController->Disconnect( wxEVT_INPUT, InputEventHandler( ConfigSaver2::onInputEvent ), NULL, m_configSaver );
//
//                        m_configDownloadController->Disconnect(  wxID_ANY, wxEVT_COMMUNICATION,
//                                                                CommunicationEventHandler( ConfigSaver2::onCommunicationEvent ),
//                                                                NULL, m_configSaver );
//
//                        delete m_configDownloadController;
//                }
//            }
//        } break;
//
//        default:
//            wxMessageBox( _T("Please, plug off and plug in  USB-TO-SERIAL converter!") );
//            m_controller->start();
//            break;
//    }
}

void MyFrame::onDeviceFound( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("Device Found!") );
}

void MyFrame::onDeviceNotFound( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("Device Not Found!") );

    refreshToolbar( event );
}

void MyFrame::onDeviceConnected( VemsDeviceControllerEvent& event )
{
    if ( event.isBootMode() ) {
        BootModeDialog dlg(this);

        switch ( dlg.ShowModal() )
        {
            case BootModeDialog::BMD_STAY:
                UserLog::uLog( ULOG_OK, _T("Device is in Boot mode!") );
                enableFirmwareMenu( true );
                refreshToolbar( event );
                break;

            case BootModeDialog::BMD_LEAVE:
                m_controller->leaveBootMode();
                break;
        }
    } else {
        UserLog::uLog( ULOG_OK, _T("Device Connected!") );
        enableFirmwareMenu( true );
        refreshToolbar( event );
        startLogging();
    }
}

void MyFrame::onDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("Device Disconnected!") );

    enableFirmwareMenu( false );

    resetStatusBar();

    refreshToolbar( event );

    stopLogging();

    event.Skip();
}

void MyFrame::onDeviceControllerStarted( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("DeviceController Started!") );

}

void MyFrame::onDeviceControllerStopped( VemsDeviceControllerEvent& event )
{
    UserLog::uLog( ULOG_WARNING, _T("DeviceController Stopped!") );

    enableFirmwareMenu( false );

    resetStatusBar();

    refreshToolbar( event );

    stopLogging();
}

void MyFrame::onDeviceBaudRateChanged( VemsDeviceControllerEvent& event )
{
    wxString message = wxString::Format( _T("Baudrate changed to %d!"), event.getBaudrate() );
    UserLog::uLog( ULOG_INFO, message );
    refreshBaudrateComboBox( true, event.getBaudrate() );
}

void MyFrame::onDeviceResumed( VemsDeviceControllerEvent& event )
{
    refreshMenuState();
    resetStatusBar();
    refreshToolbar( event );
}

void MyFrame::onInputEvent( InputEvent& event )
{
    const SIPRPacket& packet = event.getPacket();

    if ( m_controller->isConnected() ) {
        m_siprFileLogger->process( packet );
    }

    switch ( packet.type() )
    {
        case SIPR_READ_SENSORS:
        case SIPR_UPDATE_FREQUENCY:
            refreshStatusBar( packet );
            break;

        case SIPR_GET_TABLEDATA:
        case SIPR_GET_CONFIG_1:
        case SIPR_GET_CONFIG_2:
            m_model->process( packet );
            break;
    }
}

//void MyFrame::onConfigDownloadEvent( InputEvent& event )
//{
//    const SIPRPacket& packet = event.getPacket();
//
//    switch ( packet.type() )
//    {
//        case SIPR_TABLES_END:
//            m_configDownloadController->Disconnect( wxEVT_INPUT, InputEventHandler( MyFrame::onConfigDownloadEvent ), NULL, this );
//            m_configDownloadController->Disconnect( wxEVT_INPUT, InputEventHandler( ConfigSaver2::onInputEvent ), NULL, m_configSaver );
////            m_controller->start();
//            m_controller->resume();
//            break;
//    }
//}


void MyFrame::onTestMenu( wxCommandEvent& event )
{
#ifndef COMM_MODEL_WITH_GUI
    // parsing map file
    /*MapFileParser parser;
    LOGP( _T("Parsing MAP file..."))
    parser.parse( DirHelper::GetPath(dLOG) + _T("test\\vems.map") );
    LOGP( _T("Parsing MAP file finished"))

    long engine_addr;

    {
        MapSymbolHash mapHash = parser.getMapSymbols();
        MapSymbolHash::iterator it = mapHash.find( _T("engine") );

        if ( it == mapHash.end() )
        {
            LOGP( _T("Engine struct not found in map file!") )
            return;
        }
        else
        {
            engine_addr = it->second.address;
        }
    }

    // parsing global.h
    LOGP( _T("Parsing global.h..."))
    SymbolTable* symTable = new SymbolTable;

    ParseHeader globalHParser( *symTable );
    globalHParser.parse( DirHelper::GetPath(dLOG) + _T("test\\global.h") );
    LOGP( _T("Parsing global.h finished"))

    long engine_t_tps_offset;
    {
        SymbolTable::iterator it = symTable->find( _T("engine_t_tps") );

        if ( it == symTable->end() )
        {
            LOGP( _T("engine_t_tps not found in global.h") )
            return;
        }
        else
        {
            Symbol* sym = it->second;
            engine_t_tps_offset = sym->getOffset();
        }
    }

    int tps_addr = engine_addr + engine_t_tps_offset;

    wxString tpsHex = wxString::Format( _T("%x"), tps_addr );
    LOGP( _T("TPS addr:")+ tpsHex )

    StringHash h;

    h[ _T("engine_t_tps")] = tpsHex;

    m_controller->readOffset( true, h, this );*/
#else

    GaugeFrame* frame = new GaugeFrame(this,MTConfigHelper::getMTConfigData(), m_controller );
#endif
}

void MyFrame::onConfigVerify( wxCommandEvent& event )
{
    m_pFirmwareConfigManager->verifyConfigFromFile();
}

void MyFrame::onConfigUpload( wxCommandEvent& event )
{
    m_pFirmwareConfigManager->easyUploadConfig();
}

void MyFrame::onOffsetREvent( OffsetReaderEvent& evt )
{
    /// todo
    double v = (evt.values)[_T("engine_t_tps")];
    wxString tpsVal = wxString::Format(_T("engine_t_tps: %.f"), v );
    SetStatusText( tpsVal, 2 );
}

/**
    V3GUI - CommunicationModel common/conflicted files

        ECUConfigPanel - ECUConfigPanel2
        VemsCommand - VemsCommand2
*/
/** TODO
    - !!!TESTING 1.1.27!!!

    - !!!TESTING!!!
        - handle if detect bad rt length // because short timeout or interwrite delay
             - set timeout and interwritedelay



        - 1.1.27 - fetching wrong firmware info because pMan give wrong answer

    - !!!VS compile!!!

    - !!!set MegatuneSyncer to sync pages!!!
    - !!!FwDesc::cleanup() memory leak!!!
    - !!!MegatuneSyncer error when detect realtimelength( need sync if too many crcerror)!!!
    - !!!VemsDeviceController->isRunning() doesn't work in DetectingState because thread isn't running!!!

    - ECU ConfigPanel

    - show crc errors / rt errors

    - Detecting
        - try to detect ECU with the config params first

    - make ECUConfig panel applies changes without restart app
    - create, test SerialPortScannerClient

    - FirmwareVerifier
        - handle patch files
        - handle comm error
        - handle patches
        - handle round

    - FirmwareUploader
        - handle patch files

    - ModeSelector
    - BaudRateSelector

    - DETECTING
        - handle previous port, baud, device

    - handle preferred baudrate
    - Scanning / ECUConfigPanel
    - handle MAN/MEGTUNE/BOOT
    - Config Panel
    - timeout, interWriteDelay
    - main gui
    - ConfigDownloader send events
*/

/**
    VemsDeviceController->start();
        cannot open port

        started
            show dialog
                Detecting device...
                    device not found / close

                    device found /
                        boot / Boot mode found / close

                        other / Checking password protection / close (testing)


                    communication error
                        / close



    VemsDeviceController::configPanel()
        saveCurrentState()
            m_firmwareInfo
            m_device
            m_deviceMode
            m_port
            m_baudrate

            m_fsm->isConnected()

        stop()

        resume()



    ECUConfigPanel( VemsDeviceController* )

        CANNOT_OPEN_PORT

            change params
                port
                    searches device on the selected port ???
                        found
                        not found

            detect

            save/close
                resume()




        DETECTING


        CONNECTED


        STOPPED


*/

/**
    stat
        update per sec
        received packets
        crc errors
        rt errors


*/


/**
    Layers

        Basic
            class CommunicationController


            class DeviceDetector


            class DeviceController( )
                selectManMode()
                selectBootMode()
                selectMegatuneMode()

                sendPageAddress( ... )
                getPageChecksum( ... )


            DeviceControllerEvent
                Type
                    MAN_MODE_SELECTED
                    BOOT_MODE_SELECTED
                    MEGATUNE_MODE_SELECTED


        First
            VemsDeviceController

                FirmwareUpdater( DeviceController )
                {
                    start()
                       DeviceController
                           selectBootMode()
                           sendPageAddres( ... )
                           getPageCheckSum( ... )

                    OnDeviceControllerEvent
                        BOOT_MODE_SELECTED
                        PAGE_ADDRESS_OK
                        PAGE_ADDRESS_BAD
                        PAGE_CHECKSUM

                        TIMEDOUT

                    FirmwareUpdaterEvent
                }

        Second //GUI
            DeviceController
            FirmwareUpdater( DeviceController )

            OnFirmwareUpdaterEvent


*/

/**
    //CommunicationClient???( id, )
    CommunicationClient->register( CommunicationClient )

    write()
        m_client->write()

    read()
        m_client->read()

    handleEvent( event )

        meth = m_hash[id][event];

        if ( meth ) {
            meth();
        }

        switch ( event )
        {
            EVENT_1:
                switch ( client->id )
                {
                    ID_1:
                        //removeClient();
                        registerClient( );
                }

            default:
                break;
        }

    DeviceSelector( parent )
        // addEventHandler( event, obj, meth )
        parent->handleEvent( )


    SelectManMode( parent )
        write()
            send




*/
