#ifndef MAIN_H
#define MAIN_H

#include <wx/wx.h>
#include <wx/progdlg.h>
#include <wx/tglbtn.h>

#include "Observable.h"

class VemsDeviceControllerEvent;
class CommunicationEvent;
class InputEvent;
class OffsetReaderEvent;

class VemsDeviceController;
class SIPRFileLogger;
class STDataModelBase;
class FirmwareConfigItemArray;
class SIPRPacket;
class DeviceControllerProgressBarManager;
class SDCardEvent;
//class ConfigDownloadController;
//class ConfigSaver2;
class ConfigDownloadDirector;
class FirmwareConfigManager;
class DummyUserDialogBase;
class FirmwareDirector;

class MyFrame: public wxFrame, public Observable
{
	public:
		MyFrame(wxFrame *frame, const wxString& title);
		~MyFrame();

	private:
        VemsDeviceController*   m_controller;
        STDataModelBase*        m_model;
        SIPRFileLogger*         m_siprFileLogger;

        DeviceControllerProgressBarManager* m_progressBarManager;
        DummyUserDialogBase*                m_dummyUserDialogBase;

        ConfigDownloadDirector*     m_configDownloadDirector;
        FirmwareConfigManager*      m_pFirmwareConfigManager;
        FirmwareDirector*           m_firmwareDirector;

    private:

        wxProgressDialog*       m_sdcardDlg;
        wxToggleButton*         m_readOffsetButton;

        void startLogging();
        void stopLogging();

        void refreshToolbar( const VemsDeviceControllerEvent& event );
        void refreshToolbarButton( int buttonId, const wxString& label, const wxColour& color, bool isEnabled );
        void refreshBaudrateComboBox( bool isEnabled, size_t baudrate );

        void resetStatusBar();
        void refreshStatusBar( const SIPRPacket& packet );

        void enableFirmwareMenu( bool isEnabled );
        void refreshMenuState();

        /// MenuItemEvent handlers
		void onECUConfig( wxCommandEvent & event );
		void onFirmwareVerify( wxCommandEvent& event );
		void onFirmwareUpload( wxCommandEvent& event );
		void onConfigDownload( wxCommandEvent& event );
		void onSDCardSave( wxCommandEvent& event );
        void onReadPage( wxCommandEvent & event );
        void onSDAction( wxCommandEvent& event );
        void onTriggerLog( wxCommandEvent& event );
        void onTestMenu( wxCommandEvent& event );
        void onConfigVerify( wxCommandEvent& event );
        void onConfigUpload( wxCommandEvent& event );

		/// ToolBarButton handlers
        void onDeviceButton( wxCommandEvent& event );
        void onBaudrateComboBox( wxCommandEvent& event );
        void onReadOffset( wxCommandEvent& event );

        /// CommunicationEvent handlers
        void onCannotOpenPort( CommunicationEvent& event );
        void onCommunicationError( CommunicationEvent& event );

        /// VemsDeviceControllerEvent handlers
        void onDeviceFound( VemsDeviceControllerEvent& event );
        void onDeviceNotFound( VemsDeviceControllerEvent& event );
        void onDeviceConnected( VemsDeviceControllerEvent& event );
        void onDeviceDisconnected( VemsDeviceControllerEvent& event );
        void onDeviceControllerStarted( VemsDeviceControllerEvent& event );
        void onDeviceControllerStopped( VemsDeviceControllerEvent& event );
        void onDeviceBaudRateChanged( VemsDeviceControllerEvent& event );
        void onDeviceResumed( VemsDeviceControllerEvent& event );


		/// InputEvent handler
		void onInputEvent( InputEvent& event );
//		void onConfigDownloadEvent( InputEvent& event );

		///SDCardEvent
		void onSDCardEvent( SDCardEvent& event );

		void onOffsetREvent( OffsetReaderEvent& evt );

		DECLARE_EVENT_TABLE();
};

#endif // MAIN_H
