#include "ConfigEditPanel.h"

#include <wx/sizer.h>
#include <wx/intl.h>
#include <wx/filename.h>
#include <wx/string.h>

#include "ConfigBase.h"
#include "logger.h"

const long ConfigEditPanel::ID_saveButton = wxNewId();

BEGIN_EVENT_TABLE(ConfigEditPanel,wxPanel)
	EVT_BUTTON( ID_saveButton, ConfigEditPanel::saveValueInFile )
END_EVENT_TABLE()

ConfigEditPanel::ConfigEditPanel( wxWindow* parent, wxWindowID id, const wxString& boxTitle, ConfigBase* configBase
                                  , const wxString& path, const wxString& defPath, const wxString& subPath )
{
    m_configBase    = configBase;
    m_path          = path;
    m_defaultPath   = defPath;
    m_subPath       = subPath;

	Create( parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("Panel") );

	buildGUI( boxTitle, path );
}

ConfigEditPanel::~ConfigEditPanel()
{
    //dtor
}

void ConfigEditPanel::buildGUI( const wxString& boxTitle, const wxString& path )
{

    wxStaticBox* box = new wxStaticBox( this, wxID_ANY, boxTitle );
    wxFont font = box->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    box->SetFont( font );
    box->SetForegroundColour( *wxBLACK );

    wxStaticBoxSizer* BoxSizer = new wxStaticBoxSizer( box , wxVERTICAL );
	//wxBoxSizer* BoxSizer            = new wxStaticBoxSizer( wxVERTICAL, this, boxTitle );
	wxFlexGridSizer* FlexGridSizer1 = new wxFlexGridSizer( 0, 2, 0, 0 );
    //wxBoxSizer* BoxSizer = new wxBoxSizer(wxVERTICAL );

    FlexGridSizer1->AddGrowableCol(1,1);

    wxString oldPath = m_configBase->getPath();
    m_configBase->setPath( path );
    int size = m_configBase->getNumberOfEntries();

    //if no entry use default params
    if( size < 1 )
    {
         m_configBase->setPath( m_defaultPath );
    }

    long dummy      = 0;
    int idx         = 0;
    wxString group  = wxEmptyString;
    wxString value  = wxEmptyString;
    wxTextCtrl* tmpTextCtrl     = NULL;
    wxStaticText* tmpStaticText = NULL;

    bool bCont = m_configBase->getFirstEntry( group, dummy );

    while( bCont )
    {
        m_configBase->read( group, &value );

        tmpTextCtrl = new wxTextCtrl( this, wxID_ANY, value , wxDefaultPosition, wxSize(100,10), 0, wxDefaultValidator, _T("wxTextCtrlNameStr") );
        m_textCtrlArray.Add( tmpTextCtrl );
        //tmpTextCtrl->SetInsertionPoint(0);

        tmpStaticText = new wxStaticText( this, wxID_ANY, group,wxDefaultPosition,wxSize(125,25),0,_T("staticText"));
        m_staticTextArray.Add( tmpStaticText );

        idx++;

        bCont = m_configBase->getNextEntry( group, dummy );
    }

    m_configBase->setPath( oldPath );

	FlexGridSizer1->SetRows( idx );

	for( int cnt = 0 ; cnt < idx; cnt++ )
	{
        FlexGridSizer1->Add( m_staticTextArray[cnt],0, wxALIGN_LEFT );
        FlexGridSizer1->Add( m_textCtrlArray[cnt], 0, wxEXPAND, 0 );
	}

	BoxSizer->Add( FlexGridSizer1, 0, wxEXPAND );

	wxButton* saveButton = new wxButton(this,ID_saveButton,_T("Save"),wxDefaultPosition,wxDefaultSize,0,wxDefaultValidator,_T(""));
	BoxSizer->Add( saveButton ,0 ,0|wxALL,5);

    wxString warningMessage = _T("Not applied until vemsTune restart!");
    wxStaticText* warningStaticText = new wxStaticText( this, wxID_ANY, warningMessage );
    font = warningStaticText->GetFont();
    font.SetWeight( wxFONTWEIGHT_BOLD );
    warningStaticText->SetFont( font );
    warningStaticText->SetForegroundColour( *wxRED );

    BoxSizer->Add( warningStaticText, 0, wxEXPAND | wxALL, 5 );

	SetSizer( BoxSizer );
	BoxSizer->Fit( this );
	BoxSizer->SetSizeHints( this );

	//SetInitialSize( GetBestSize() );

    //m_textCtrlArray[0]->SetValue(_T("HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH"));

}


void ConfigEditPanel::loadValues( const wxString& session )
{
    m_path = session + m_subPath;
    wxString oldPath    = m_configBase->getPath();

    m_configBase->setPath( m_path );

    int size = m_configBase->getNumberOfEntries();

    //if no entry use default params
    if( size < 1 )
    {
         m_configBase->setPath( m_defaultPath );
    }

    wxString value = wxEmptyString;
    wxString group = wxEmptyString;

    size_t paramCount = m_textCtrlArray.GetCount();

    for( size_t i = 0; i < paramCount; ++i )
    {
        group = m_staticTextArray[ i ]->GetLabel();
        m_configBase->read( group, &value );
        m_textCtrlArray[ i ]->SetValue( value );
    }

    m_configBase->setPath( oldPath );
}


void ConfigEditPanel::saveValueInFile( wxCommandEvent& event )
{
    wxString oldPath = m_configBase->getPath();
    m_configBase->setPath( m_path );
    wxString value;
    wxString group;
    size_t num = m_textCtrlArray.GetCount();

    for(size_t cnt = 0; cnt < num; cnt++){
        group = m_staticTextArray[cnt]->GetLabel();
        value = m_textCtrlArray[cnt]->GetValue();
        // bool is =m_configBase->write(group,value);
    }
    m_configBase->save();
    m_configBase->setPath( oldPath );
}

void ConfigEditPanel::notify( const wxString& str )
{
    loadValues( str );
}
