#ifndef MAIN_H
#define MAIN_H

#include <wx/wx.h>
#include <wx/progdlg.h>
#include <wx/tglbtn.h>

#include "Observable.h"
#include "vemsprotocol/DeviceInfo.h"
#include "common/types/IntArray.h"

class SerialPortScanner;
class SerialPortScannerEvent;
class CommunicationEvent;
class VemsDeviceControllerEvent;
class DeviceModeController;

class DetectionTestMainFrame: public wxFrame, public Observable
{
	public:
		DetectionTestMainFrame(wxFrame *frame, const wxString& title);
		~DetectionTestMainFrame();

    private:
        SerialPortScanner*  m_serialPortScanner;
        wxTimer             m_detectionTimer;
        size_t              m_detectionSleep;

        DeviceInfo          m_deviceInfo;

        DeviceModeController* m_deviceModeController;

        wxTimer             m_changeModeTimer;

        IntArray m_modeSequence;
        size_t  m_modeSequenceIdx;
        size_t  m_changeModeTestCount;
        void startDetecting();
        void startDetectionTimer();
        void refreshToolbarButton( int buttonId, const wxString& label, const wxColour& color, bool isEnabled );
        void refreshBaudrateComboBox( bool isEnabled, size_t baudrate );

        void OnDeviceFound( SerialPortScannerEvent& event );
        void OnDeviceNotFound( SerialPortScannerEvent& event );
        void OnCannotOpenPort( SerialPortScannerEvent& event );

        void OnStartDetection( wxTimerEvent& event );
        void OnChangeMode( wxTimerEvent& event );

        void OnCommunicationEvent( CommunicationEvent& event );

        void OnDeviceMode( wxMouseEvent& event );

        void OnDeviceModeChanged( VemsDeviceControllerEvent& event );
        void OnDeviceDisconnected( VemsDeviceControllerEvent& event );

        void OnDetectButton( wxCommandEvent& event );

		DECLARE_EVENT_TABLE();
};

#endif // MAIN_H
