#include "DrawHelper.h"

#include <wx/regex.h>
#include <wx/filename.h>
#include <wx/artprov.h>
#include <wx/tokenzr.h>

#include "common/typedef.h"
#include "logger.h"
#include "TinyChars.h"
#include "StringHelper.h"

#include "common/typedef.h"
#include "wxHSLColor.h"

#ifndef M_PI
	#define M_PI		3.14159265358979323846
#endif
#define M_PI_2		1.57079632679489661923
#define M_PI_4		0.78539816339744830962


#define chkminmax(val,min,max) \
    if( (val) > (max) ) (val) = (max); \
    if( (val) < (min) ) (val) = (min);


bool DrawHelper::m_tinyInit = false;
TinyCharHash DrawHelper::m_tinyChars;

DrawHelper::DrawHelper()
{
    //ctor
}

DrawHelper::~DrawHelper()
{
    //dtor
}


wxColor DrawHelper::ModLuminance(const wxColor& color, double mod)
{
    wxHSLColor hsl( color );

    hsl.m_lum += mod;

    chkminmax(hsl.m_lum,0,1);

    return hsl.ToRGB();
}


void DrawHelper::SetColorA(wxColor& c, unsigned char a)
{
    c.Set(c.Red(), c.Green(), c.Blue(), a);
}

wxColor DrawHelper::ParseExtColorStr(const wxString &str)
{

    wxRegEx exp1; /// format: rgba(255,0,12,128)
//    exp1.Compile(_T("rgba\\(([0-9]{1,3})\\,([0-9]{1,3})\\,([0-9]{1,3})\\,([0-9]{1,3})\\)"));
    exp1.Compile(_T("rgba\\([ ]?([0-9]{1,3})[ ]?\\,[ ]?([0-9]{1,3})[ ]?\\,[ ]?([0-9]{1,3})[ ]?\\,[ ]?([0-9]{1,3})[ ]?\\)")); // with spaces

    bool res = exp1.Matches(str);

    if(!res) return wxColor(str);

    long r = 255;
    long g = 255;
    long b = 255;
    long a = 255;

    exp1.GetMatch(str,1).ToLong(&r);
    exp1.GetMatch(str,2).ToLong(&g);
    exp1.GetMatch(str,3).ToLong(&b);
    exp1.GetMatch(str,4).ToLong(&a);

    return wxColor(r,g,b,a);
}

wxString DrawHelper::GetColorAsString(const wxColor& c, long flags)
{
    if( c.Alpha() == wxALPHA_OPAQUE ) return c.GetAsString(flags);

    return wxString::Format(_T("rgba(%i,%i,%i,%i)"),c.Red(), c.Green(), c.Blue(), c.Alpha());
}

wxColor DrawHelper::InvertColor(const wxColour &colour, int type, int flags)
{
    unsigned char red,green,blue,alpha=0;

    red   = 255 - colour.Red();
    green = 255 - colour.Green();
    blue  = 255 - colour.Blue();
    alpha = colour.Alpha();

    if( !(flags & CIF_NO_ALPHA) )
        alpha = 255 - colour.Alpha();


    return wxColor(red,green,blue,alpha);
}

void DrawHelper::DrawArcText(wxDC *dc, const wxString &str, const wxPoint &center, double radius, double angle, double textExpandAngle, short maxTextHeight)
{

    wxFont font = dc->GetFont();

    if( font == wxNullFont ) return;  /// actually....

    Logger::logf(LOG_MSG,_T("DrawHelper::DrawArcText str=%s, center.x=%i, center.y=%i, radius=%.1f, angle=%.1f, textExpandAngle=%.1f, maxTextHeight=%i"),str.c_str(),center.x,center.y,radius,angle,textExpandAngle,maxTextHeight);

    angle -= 270.0;

    int tmp = (int)angle / 360;

    angle -= tmp*360;

    if(angle <= 180){

    }
    else{

    }

    double circlePer = 2 * radius * M_PI;

    wxSize linearTextSize = dc->GetTextExtent(str);

    double angleLenght = (linearTextSize.x / circlePer) * 360.0;

    int ps = font.GetPointSize();

    //Logger::logf(LOG_MSG,_T("DrawArcText linearTextSize.x=%i, angleLenght=%.1f, ps=%i"),linearTextSize.x,angleLenght,ps);

    if(textExpandAngle != -1) {

        ps *= (int)ceil( ( textExpandAngle / angleLenght ) );

        if(maxTextHeight != -1 && ps > maxTextHeight)
            ps = maxTextHeight;

        font.SetPointSize(ps);

        dc->SetFont(font);

        linearTextSize = dc->GetTextExtent(str);
        angleLenght = (linearTextSize.x / circlePer) * 360.0;

//        Logger::logf(LOG_MSG,_T("DrawArcText linearTextSize.x=%i, angleLenght=%.1f, ps=%i"),linearTextSize.x,angleLenght,ps);
    }


    wxArrayInt strCharsWidth;
    strCharsWidth.Clear();
    //dc->GetPartialTextExtents(str,strCharsWidth);

    for(uint16_t b = 0; b < str.Len(); b++)
        strCharsWidth.Add( (dc->GetTextExtent(str[b])).x);


    double charAngle = 0;
    double charDiffAngle = angle - angleLenght/2.0;
    wxCoord charX, charY;


    for(uint16_t a=0; a < strCharsWidth.GetCount(); a++){


        //Logger::logf(LOG_MSG,_T("DrawArcText charDiffAngle=%0.2f,strCharsWidth[%i]=%i "),charDiffAngle,a,strCharsWidth[a]);

        charAngle = (charDiffAngle) * (M_PI/180.0);

        charX = (wxCoord)(center.x - (cos(charAngle+M_PI_2) * radius*0.96));
        charY = (wxCoord)(center.y + (sin(charAngle+M_PI_2) * radius*0.96));

        wxString charStr = str[a];

        //Logger::logf(LOG_MSG,_T("DrawArcText charAngle=%0.4f, charX=%i, charY=%i, char=")+charStr,charAngle,charX,charY);

        dc->DrawRotatedText( charStr, charX, charY, charAngle*(180.0/M_PI) );

        charDiffAngle += (((double)strCharsWidth[a] / (double)linearTextSize.x) * angleLenght);

    }

}

void DrawHelper::DrawMultiLineText(wxDC *dc, const wxString &str, const wxPoint &leftTopPos, int align)
{
/**
    wxALIGN_CENTER
    wxALIGN_LEFT
    wxALIGN_RIGHT

    wxALIGN_TOP
    wxALIGN_BOTTOM
    wxALIGN_CENTER_VERTICAL
*/



    wxArrayString lines;
    lines.Clear();

    StringHelper::Split(str,'\n',lines,true);

    wxSize boxSize = dc->GetMultiLineTextExtent(str);

    double d = 0.0;

    for( uint8_t a = 0; a < lines.GetCount(); a++){

        wxSize lineExtent = dc->GetTextExtent(lines[a]);

        wxPoint linePos;

        //if(align & wxALIGN_LEFT)
        linePos.x = leftTopPos.x;

        if(align & wxALIGN_CENTER)
            linePos.x = leftTopPos.x + (boxSize.x - lineExtent.x)/2;

        if(align & wxALIGN_RIGHT)
            linePos.x = leftTopPos.x + boxSize.x - lineExtent.x;



        if(lines.GetCount() == 1) d=0.5;

        linePos.y = (int)(( a + d ) * lineExtent.y + leftTopPos.y);

        dc->DrawText(lines[a],linePos);

    }

}

void TinyChar::SetCharMatrix(bool cmp[5][5])
{
    for(short a=0;a<5;a++)
        for(short b=0;b<5;b++)
            charmatrix[a][b] = cmp[a][b];

}

void DrawHelper::InitTinyCharContainer()
{

    ///normal chars
    for(unsigned char a = 'A' ; a <= 'Z' ; a++){
        TinyChar tc;
        tc.width = TinyCharsWidth[a-'A'];
        tc.SetCharMatrix(TinyCharMatrix[a-'A']);
        m_tinyChars[a] = tc;

    }

    ///special chars

    ///    '/'
    TinyChar tcp;
    tcp.width = TinyCharsWidth[28];
    tcp.SetCharMatrix(TinyCharMatrix[28]);
    m_tinyChars['/'] = tcp;

    ///    ' '
    TinyChar tcs;
    tcs.width = TinyCharsWidth[27];
    tcs.SetCharMatrix(TinyCharMatrix[27]);
    m_tinyChars[' '] = tcs;

    ///    full
    TinyChar tcf;
    tcf.width = TinyCharsWidth[26];
    tcf.SetCharMatrix(TinyCharMatrix[26]);
    m_tinyChars[0] = tcf;

    m_tinyInit = true;
}

void DrawHelper::DrawTinyText(wxBitmap *bmp, const wxString &str, const wxPoint &pos, const wxColor &color, bool multiLine)
{
    if(!bmp) return;

    if(!bmp->IsOk()) return;

    wxMemoryDC dc;

    dc.SelectObjectAsSource( *bmp );

    DrawTinyText( &dc, str, pos, color, multiLine);

}

void DrawHelper::DrawTinyText(wxDC *dc, const wxString &str, const wxPoint &pos, const wxColor &color, bool multiLine)
{

    if(!m_tinyInit)
        InitTinyCharContainer();

    uint32_t cx = pos.x; /// current position (for the char)
    uint32_t cy = 0;

    wxString ustr = str.Upper();

    dc->SetPen( wxPen( color ) );

    wxFont font = dc->GetFont();
    bool visitor_exists = font.SetFaceName(_T("Visitor TT2 (BRK)"));

    if(visitor_exists) {

        font.SetPointSize(9);
        dc->SetFont(font);
        dc->SetTextForeground( color );
        if(multiLine){
            wxArrayString lines = wxStringTokenize(str,_T("\n"));
            int y = pos.y;
            for(size_t a=0;a<lines.GetCount();a++){
                dc->DrawText(lines[a],pos.x,y);
                y += 6;
            }
        } else {
            dc->DrawText(str,pos);
        }

    } else {

        for(uint16_t a = 0; a < ustr.Length(); a++){

            int tmp = *(ustr(a,1).mb_str());

            //Logger::logf(LOG_MSG,_T("DrawTinyText codes=%i"),tmp);

        if(m_tinyChars.find(tmp) == m_tinyChars.end())
        tmp = 0;

        for(uint8_t y = 0; y < 5; y++) {
            cy = pos.y + y;
            for(uint8_t x = 0; x < m_tinyChars[tmp].width; x++) {
                if(m_tinyChars[tmp].charmatrix[y][x])
                    dc->DrawPoint(cx+x,cy);
            }
        }

        cx += 1 + m_tinyChars[tmp].width;

    }
    }

}

wxSize DrawHelper::GetTinyTextExtent(const wxString &str)
{

    if(!m_tinyInit)
        InitTinyCharContainer();

    wxSize ret;

    wxString ustr = str.Upper();

    wxMemoryDC dc;

    wxFont font = dc.GetFont();
    bool visitor_exists = font.SetFaceName(_T("Visitor TT2 (BRK)"));

    if(visitor_exists) {

        font.SetPointSize(9);
        dc.SetFont(font);

        ret = dc.GetMultiLineTextExtent(str);

        wxArrayString lines = wxStringTokenize(str,_T("\n"));

        ret.DecBy(0,lines.GetCount()*2);

    } else {

    ret.y = 5;
    ret.x = 0;

    for(uint16_t a = 0; a < ustr.Length(); a++){
        int tmp = *(ustr(a,1).mb_str());
        ret.x += m_tinyChars[tmp].width;
    }

    ret.x += str.Length()-1;

    }

    return ret;

}

wxBitmap DrawHelper::GetBitmap(const wxString& filename)
{
    wxString fp = DirHelper::GetPath(dICON) + filename;

    if( wxFileName::FileExists(fp) ) {
        wxBitmap bmp;
        bmp.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return bmp;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(16,16));
}

wxImage DrawHelper::GetImage(const wxString& filename, DirHelperTypes dir)
{
    wxString fp = DirHelper::GetPath(dir) + filename;

    if( wxFileName::FileExists(fp) ) {
        wxImage img;
        img.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return img;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(16,16)).ConvertToImage();
}

wxImage DrawHelper::GetIconImg(const wxString& filename)
{
    wxString fp = DirHelper::GetPath(dICON) + filename;

    if( wxFileName::FileExists(fp) ) {
        wxImage img;
        img.LoadFile( fp , wxBITMAP_TYPE_PNG);
        return img;
    }

    return wxArtProvider::GetBitmap(wxART_MISSING_IMAGE,wxART_OTHER, wxSize(16,16)).ConvertToImage();
}

wxSize DrawHelper::GetTextExtent(const wxString& text, int fontSize, bool isMultiLine)
{
    wxMemoryDC memdc;

    wxFont font = memdc.GetFont();
    font.SetPointSize(fontSize);
    memdc.SetFont(font);

    if(isMultiLine)
        return memdc.GetMultiLineTextExtent(text);
    else
        return memdc.GetTextExtent(text);
}

wxSize DrawHelper::GetTextExtent(const wxString& text, const wxFont& font, bool isMultiLine)
{
    wxMemoryDC memdc;

    memdc.SetFont(font);

    if(isMultiLine)
        return memdc.GetMultiLineTextExtent(text);
    else
        return memdc.GetTextExtent(text);
}


void DrawHelper::OffsetPath(RealPointArray& path, const wxRealPoint& offset)
{
    for(size_t a=0; a < path.GetCount(); a++) {
        wxRealPoint& p = path[a];
        p.x += offset.x;
        p.y += offset.y;
    }
}

