#ifndef DRAWHELPER_H
#define DRAWHELPER_H

#include <wx/wx.h>

/// why?
//void DrawTinyText();

#include "common/types/RealPointArray.h"
#include "DirHelper.h"

enum ColorInvertTypes {
    CIT_DEFAULT = 0
};

enum ColorInvertFlags {
    CIF_DEFAULT = 1,
    CIF_NO_ALPHA = 2
};


class TinyChar
{
    public:
        TinyChar() {};
        ~TinyChar() {};

        unsigned char width;
        bool charmatrix[5][5];

        void SetCharMatrix(bool cmp[5][5]);

};

WX_DECLARE_HASH_MAP( int, TinyChar, wxIntegerHash, wxIntegerEqual, TinyCharHash );

class DrawHelper
{
    public:

        DrawHelper();
        virtual ~DrawHelper();

        static wxColor InvertColor(const wxColour &colour, int type = 0, int flags = CIF_NO_ALPHA);

        static void DrawArcText(wxDC *dc, const wxString &str, const wxPoint &center, double radius, double angle, double textExpandAngle = -1, short maxTextHeight = -1);
        static void DrawTinyText(wxBitmap *bmp, const wxString &str, const wxPoint &pos, const wxColor &color = wxColor(0,0,0), bool multiLine = false);
        static void DrawTinyText(wxDC *dc, const wxString &str, const wxPoint &pos, const wxColor &color = wxColor(0,0,0), bool multiLine = false);
        static void DrawMultiLineText(wxDC *dc, const wxString &str, const wxPoint &leftTopPos, int align);



        static wxSize GetTinyTextExtent(const wxString &str);

        static void InitTinyCharContainer();

        static wxColor ParseExtColorStr(const wxString &str);
        static wxString GetColorAsString(const wxColor& c, long flags = wxC2S_HTML_SYNTAX);

        static wxBitmap GetBitmap(const wxString& filename);
        static wxImage  GetIconImg(const wxString& filename);
        static wxImage  GetImage(const wxString& filename, DirHelperTypes dir = dGRAPHICS);

        static void SetColorA(wxColor& c, unsigned char a);

        static wxSize GetTextExtent(const wxString& text, const wxFont& font, bool isMultiLine = false);
        static wxSize GetTextExtent(const wxString& text, int fontSize, bool isMultiLine = false);

        static void OffsetPath(RealPointArray& path, const wxRealPoint& offset);

        static wxColor ModLuminance(const wxColor& color, double mod);

    private:

        static bool m_tinyInit;
        static TinyCharHash m_tinyChars;


};




#endif // DRAWHELPER_H
