#include "ECUStateChangePanel.h"

#include "logger.h"
#include "serialhelper.h"
#include "vemsprotocol/ECUInfo.h"
#include "vemsprotocol/ECUStateMgrBase.h"
#include "vemsprotocol/ECUStateMgr.h"
#include "VemsTuneController.h"

int idStateButton = wxNewId();
int idDeviceCombo = wxNewId();
int idStateCombo  = wxNewId();

BEGIN_EVENT_TABLE( ECUStateChangePanel, wxPanel )
    EVT_COMBOBOX( wxID_ANY, ECUStateChangePanel::OnTextUpdated )
    EVT_BUTTON( wxID_ANY, ECUStateChangePanel::OnButton )
END_EVENT_TABLE()

ECUStateChangePanel::ECUStateChangePanel( wxWindow* parent, VemsTuneController* vemsTuneController)
    :wxPanel( parent )
{
    m_vemsTuneController = vemsTuneController;
    m_states = m_vemsTuneController->getEcuStateMgr();

    buildGUI();
}

ECUStateChangePanel::~ECUStateChangePanel()
{
    //dtor
}

void ECUStateChangePanel::buildGUI()
{
    //int state   = m_states->getState();
    //int device  = m_states->getDevice();
    int device  = m_states->getDevice();  //::DEVICE_V3;
    //int state   = 0;

    wxString label = ECUInfo::getDeviceAsString( device );

    wxStaticBoxSizer* sizer = new wxStaticBoxSizer( wxHORIZONTAL, this, label );
    wxButton* stateButton   = new wxButton( this, idStateButton, _T("Go to state!") );

/*
    m_deviceCombo = new wxComboBox( this, idDeviceCombo, _T("")
                                    , wxDefaultPosition
                                    , wxDefaultSize
                                    , 0
                                    , NULL
                                    , wxCB_READONLY
                                    , wxDefaultValidator
                                    , _T("comboBox") );
*/
    m_stateCombo = new wxComboBox(  this, idStateCombo, _T("")
                                    , wxDefaultPosition
                                    , wxDefaultSize
                                    , 0
                                    , NULL
                                    , wxCB_READONLY
                                    , wxDefaultValidator
                                    , _T("comboBox") );
/*
     m_baudRateCombo = new wxComboBox( this, idDeviceCombo, _T("")
                                    , wxDefaultPosition
                                    , wxDefaultSize
                                    , 0
                                    , NULL
                                    , wxCB_READONLY
                                    , wxDefaultValidator
                                    , _T("comboBox") );
*/

    //sizer->Add( m_deviceCombo, 0, wxALL, 5 );
    sizer->Add( m_stateCombo, 0, wxALL, 5 );
//    sizer->Add( m_baudRateCombo, 0, wxALL, 5 );
    sizer->Add( stateButton, 0, wxALL, 5 );

    stateButton->SetMinSize( stateButton->GetBestSize() );

    SetSizer( sizer );

    /*TEST*/
/*
    m_deviceCombo->Append( _T("V3"), new int ( ECUInfo::DEVICE_V3 ) );
    m_deviceCombo->Append( _T("Round"), new int ( ECUInfo::DEVICE_ROUND ) );
    m_deviceCombo->Append( _T("Arm"), new int ( ECUInfo::DEVICE_ARM ) );

    m_deviceCombo->SetSelection( 0 );
    //m_deviceCombo->SetWindowStyle( m_deviceCombo->GetWindowStyle() | wxCB_DROPDOWN );
    m_deviceCombo->SetMinSize( m_deviceCombo->GetBestSize() );
*/
    if( device == ECUInfo::DEVICE_UNKNOWN )
    {
        return;
    }

    fillStateCombo( device );
    m_stateCombo->SetMinSize( m_stateCombo->GetBestSize() );

    SetInitialSize( GetBestSize() );
}


void ECUStateChangePanel::fillStateCombo( int device )
{
    const ECUStateBaseArray* states = m_states->getStates( device );

    if( !states )
    {
        m_stateCombo->Clear();
        return;
    }

    for( size_t i = 0; i < states->GetCount(); ++i )
    {
        ECUStateBase* item = states->Item( i );
        m_stateCombo->Append( item->getName(), new int( item->getID() ) );

        int stateID = item->getID();
        if(stateID == m_states->getStoredState() )
        {
            m_stateCombo->SetSelection( i );
        }
    }


}


void ECUStateChangePanel::OnTextUpdated( wxCommandEvent& event )
{

    int id = event.GetId();

    if( id == idDeviceCombo )
    {
        m_stateCombo->Clear();

        int* device = (int*)m_deviceCombo->GetClientData( m_deviceCombo->GetSelection() );

        if( device )
        {
            fillStateCombo( *device );
        }
    }
}


void ECUStateChangePanel::OnButton( wxCommandEvent& event )
{
    int id = event.GetId();

    if( id == idStateButton )
    {
        int* state = (int*)m_stateCombo->GetClientData( m_stateCombo->GetSelection() );

        if( state )
        {
            Logger::logf(LOG_MSG, _T("ECUStateChangePanel::OnButton(): setState: ")+
            ECUInfo::getStateAsString( *state) );

            if(m_vemsTuneController->isECUOnline() ){
                m_vemsTuneController->setECUOnline(false, true);
                m_states->setState( *state );
                m_vemsTuneController->setECUOnline(true, true);
            }else{
                m_states->setState( *state );
            }
        }
    }
}


ECUStateChangeFrame::ECUStateChangeFrame( wxWindow* parent, VemsTuneController* vemsTuneController )
    :wxFrame( parent, wxID_ANY, _T("Go to state frame") )
{
    SetWindowStyle( wxCAPTION | wxSYSTEM_MENU | wxCLOSE_BOX | wxCLIP_CHILDREN );
    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    ECUStateChangePanel* panel = new ECUStateChangePanel( this, vemsTuneController );
    SetClientSize( panel->GetSize() );
    SetMinSize( GetSize() );
}

ECUStateChangeFrame::~ECUStateChangeFrame()
{
    //dtor
}

