#include "EcuTimer.h"

#include "logger.h"
#include "RTDataMgrFacade.h"
#include "RTDataInputBase.h"
#include "DirHelper.h"
#include "Observable.h"
#include "stdatamanager/STDataMgrBase.h"

#include <wx/msgdlg.h>

EcuTimer::EcuTimer(RTDataMgrBase *rtdatamgrbase, STDataMgrBase* stDataMgr, MTConfigData* mtConfig,
VemsProtocol* vp)
    :m_state        (NOTCONNECTED_RB)
    ,m_prevstate    (NOTCONNECTED_RB)
    ,m_interval     (INTERVAL_NOTCONNECTED)
    ,m_observer     (NULL)
    ,m_locked       (false)
    ,m_filePlay     (false)
{

    m_rtDataMgr = rtdatamgrbase;


    //m_ringBufferMgr.init(m_rtDataMgr);
    m_STDataMgrBase = stDataMgr;

    //m_ringBufferMgr.init(m_rtDataMgr, stDataMgr, mtConfig, vp);
    //ctor
    m_first = false;
}

EcuTimer::~EcuTimer()
{
    //dtor
    m_observerRegistry.clear();
}

void EcuTimer::StartTimer()
{
    Start(m_interval);
}

/*
void EcuTimer::SetState(EcuStates state)
{
    //if(state != )
    m_prevstate = m_state;
    m_state = state;
}
*/


void EcuTimer::setCurrState()
{
    int inputType = ((RTDataMgrFacade*)m_rtDataMgr)->getInputType();
    switch(inputType){
        case INPUT_TYPE_NONE:
            m_state = DISABLED;
            break;
        case INPUT_TYPE_RAWFILE:
        case INPUT_TYPE_FILE:
            m_state = FILELOGPLAY;
            break;
        case INPUT_TYPE_RINGBUFFER_ECU:
            if(m_state != CONNECTED_RB){
                m_state = NOTCONNECTED_RB;
            }
            break;
    }
}
// select inputsource:
// None->  set the state to DISABLED
// File -> set the state to FILELOGPLAY
// ECU ->  set the state to NOTCONNECTED
// if ECU is present then the state goes to CONNECTED
void EcuTimer::Notify(void)
{
//     Logger::log(LOG_MSG,_T("EcuTimer::Notify START"));

    //Logger::log(LOG_MSG,_T("EcuTimer::Notify 1"));
    setCurrState();

/*
    for(ObserverHashMap::iterator it = m_observerRegistry.begin(); it != m_observerRegistry.end(); it++)
        it->second->notify(m_prevstate, m_state);
  */

    observerNotify(m_prevstate, m_state);
     //Logger::logf(LOG_MSG, _T("EcuTimer::Notify() %d,%d"), m_prevstate, m_state);
    /*
    if( (m_state != m_prevstate) && (m_state == CONNECTED_RB || m_state ==FILELOGPLAY ) ){
        Logger::logf(LOG_MSG, _T("EcuTimer::Notify() send readRequestAll"));
        m_STDataMgrBase->readRequestAll();
    }
*/
    if(m_ringBufferMgr.isBootMode() && m_first == false){
         m_first = true;
        wxString msg =_T("ECU Boot Loader Detected.");
        wxMessageBox(msg,msg, wxOK);

    }
     // notify and set the appropriate interval
     if(m_prevstate != m_state) {
        //refresh observer change the display value
        //Logger::logf(LOG_MSG, _T("EcuTimer::Notify()2- %d,%d"), m_prevstate, m_state);
         if( m_state == CONNECTED_RB ){
            Logger::logf(LOG_MSG, _T("EcuTimer::Notify() send readRequestAll ECU"));
            //m_STDataMgrBase->readRequestAll();
            m_STDataMgrBase->selectInput( STDataMgrBase::ECU );
            m_ringBufferMgr.setActiveInput(INPUT_TYPE_RINGBUFFER_ECU);
        }

         if( m_state == FILELOGPLAY  ){
            Logger::logf(LOG_MSG, _T("EcuTimer::Notify() send readRequestAll FILE"));
            //m_STDataMgrBase->readRequestAll();
            m_STDataMgrBase->selectInput( STDataMgrBase::FILE );
            m_ringBufferMgr.setActiveInput(INPUT_TYPE_RINGBUFFER_RAWFILE);
        }

        m_prevstate = m_state;
        switch(m_state){
            case FILELOGPLAY: // // cannot change the state the user can select this state or go to NOTCONNECTED or DISABLED
                //Logger::log(LOG_MSG,_T("EcuTimer changed FILELOGPLAY"));
            /*    Stop();
                Start(INTERVAL_FILELOGPLAY);
            */
                break;
            case DISABLED: // cannot change the state
            case NOTCONNECTED_RB:
            case CONNECTED_RB:
                Stop();
                Start(INTERVAL_NOTCONNECTED);
                break;
            default:
                break;
        }
        return;
    }

    if(!m_rtDataMgr) return;

    switch(m_state){
        case FILELOGPLAY: // // cannot change the state the user can select this state or go to NOTCONNECTED or DISABLED
       //     Logger::log(LOG_MSG,_T("EcuTimer FILELOGPLAY"));
            ///pause???
           /* if(m_filePlay){
                m_rtDataMgr->updateInputModel();
                m_rtDataMgr->updateViewers();
            }
            */
            break;
        case DISABLED: // do nothing cannot change the state
            break;
        case NOTCONNECTED_RB:
        case CONNECTED_RB:
             if(m_locked) return;

             m_prevstate = m_state;
            if( m_ringBufferMgr.isConnected() ){
                m_state = CONNECTED_RB;
            }else{
                m_state = NOTCONNECTED_RB;
            }
            break;

    }
}

wxString EcuTimer::GetStateAsString()
{

    switch(m_state){
      case NOTCONNECTED_RB:
        return _T("ECU not connected");

      case CONNECTED_RB:
        return _T("ECU connected");

      case DISABLED:
        return _T("No input source");

      case FILELOGPLAY:
        return _T("File input source");

    }

    return wxEmptyString;
}

void EcuTimer::SetFilelogPlayInterval( long interval, bool resetTimer)
{


}

void EcuTimer::setLocked(bool lock)
{
    m_locked = lock;
    m_ringBufferMgr.setLocked(lock);
}

void EcuTimer::setRawDataLog(bool b)
{
    m_ringBufferMgr.setRawDataLog(b);
}

/** @brief GetStateID
  *
  * @todo: document this function
  */
EcuStates EcuTimer::GetStateID()
{
    return m_state;
}

bool EcuTimer::IsEcuConnected()
{
    return m_ringBufferMgr.isConnected();
}

RingBuffer * EcuTimer::getRingBuffer()
{
    return m_ringBufferMgr.getRingBuffer();
}
