#include "FileDialog.h"

#include <wx/artprov.h>
#include <wx/filename.h>
#include <wx/dir.h>
#include <wx/imaglist.h>

#include "DirHelper.h"
#include "logger.h"

int idListCtrl = wxNewId();
int idTimer = wxNewId();

BEGIN_EVENT_TABLE(FileDialog, wxDialog)

    EVT_LIST_ITEM_ACTIVATED(idListCtrl, FileDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(idListCtrl, FileDialog::OnItemSelected)
    EVT_TIMER(idTimer, FileDialog::OnTimer)

END_EVENT_TABLE()

FileDialog::FileDialog(wxWindow *parent, long cols, wxWindowID id, const wxString &title, wxPoint pos, wxSize size, long style)
        :m_timer(this, idTimer)
{

    Create(parent, id, title, pos, size, style);
    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    m_cols = cols;
    m_isClick = false;

    m_listCtrl = new wxListCtrl(this, idListCtrl, wxDefaultPosition, wxSize(240,300), wxLC_REPORT);
    m_currPath = new wxTextCtrl(this, wxID_ANY,_T(""),wxDefaultPosition, wxDefaultSize, wxTE_READONLY);

    m_mainsizer = new wxBoxSizer(wxVERTICAL);

    m_currentDir = wxEmptyString;

    InitList();

    m_mainsizer->Add(m_currPath, 0, wxEXPAND);

    m_mainsizer->Add(m_listCtrl, 1, wxEXPAND| wxTOP | wxBOTTOM, 3);

    wxSizer *buttonsSizer = CreateButtonSizer( wxCANCEL | wxOK );
    m_mainsizer->Add(buttonsSizer ,0, wxLEFT | wxRIGHT | wxALIGN_CENTER_HORIZONTAL,5 );

	SetSizer(m_mainsizer);
	m_mainsizer->Fit(this);
	m_mainsizer->SetSizeHints(this);


	m_listCtrl->Connect(wxEVT_LEFT_DOWN, wxMouseEventHandler(FileDialog::OnListCtrlLeftDown),NULL, this);


    ((wxButton*)buttonsSizer->GetItem((size_t)1)->GetWindow())->SetLabel(_T("&Open"));


    m_responseTime = 200;
}

FileDialog::~FileDialog()
{
    //dtor
}

int FileDialog::ShowModal()
{
 //   Logger::logf(LOG_MSG,_T("FileDialog::ShowModal()"));

    if(m_currentDir == wxEmptyString)
        m_currentDir = DirHelper::GetPath();

    FillList();

    return wxDialog::ShowModal();
}

void FileDialog::OnItemActivated(wxListEvent &evt)
{
    wxString str = evt.GetItem().GetText();

    //Logger::logf(LOG_MSG,_T("FileDialog::OnItemActivated = ")+str);

    if(!wxFileName::DirExists(m_currentDir + str)) return;

    OpenFolder(str);

}

void FileDialog::OnListCtrlLeftDown(wxMouseEvent &evt)
{


//    Logger::logf(LOG_MSG,_T("FileDialog::OnListCtrlLeftDown"));

    m_isClick = true;

    evt.Skip();

}

void FileDialog::GetAllPath(wxArrayString *names, bool isFullPath)
{
    if(names == NULL) return;

    wxString opath = _T("");

    if(isFullPath)
        opath += m_currentDir;

    long item = -1;

    while(1) {
        item =  m_listCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        if ( item == -1 )
            break;

        names->Add(opath + m_listCtrl->GetItemText(item));

    }

}

bool FileDialog::OpenFolder(const wxString &name)
{

    m_currentDir += name + DirHelper::Separ();

    wxFileName fname(m_currentDir);

    fname.Normalize();

    m_currentDir = fname.GetPath();

    if(m_currentDir.Last() != DirHelper::Separ())
        m_currentDir += DirHelper::Separ();

    FillList();

    return 1;
}

void FileDialog::SetActualPath(const wxString &path)
{
    m_currPath->SetValue(path);
    m_currPath->SetInsertionPointEnd();
}

void FileDialog::OnItemSelected(wxListEvent &evt)
{
    m_currItem = evt.GetItem().GetText();

//   Logger::logf(LOG_MSG,_T("FileDialog::OnItemSelected = ")+m_currItem);

    m_timer.Start( m_responseTime, true );
}

void FileDialog::FillList()
{
    SetActualPath(m_currentDir);

    m_listCtrl->DeleteAllItems();

    wxDir dir(m_currentDir);
    wxString tmp;
    bool res;

    /// get dirnames
    res = dir.GetFirst( &tmp ,wxEmptyString, wxDIR_DIRS | wxDIR_HIDDEN | wxDIR_DOTDOT );

#if defined (__WINCE_STANDARDSDK__)

    if(m_currentDir != DirHelper::Separ())
        AddLine(_T(".."),DIR);

    if(res)
        AddLine(tmp,DIR);

#else
    if(tmp != _T(".") && res)
        AddLine(tmp,DIR);
#endif

    while(res){
        if(!dir.GetNext(&tmp)) break;
        AddLine(tmp,DIR);
    }

    /// get filenames
    res = dir.GetFirst( &tmp ,wxEmptyString, wxDIR_FILES | wxDIR_HIDDEN  );

    if(res)
        AddLine(tmp,FILE);

    while(res){
        if(!dir.GetNext(&tmp)) break;
        AddLine(tmp,FILE);

    }
}

void FileDialog::InitList()
{

    wxImageList *imglist = new wxImageList(16,15,true);
    imglist->Add( wxArtProvider::GetBitmap(wxART_NORMAL_FILE));
    imglist->Add( wxArtProvider::GetBitmap(wxART_FOLDER ));

    m_listCtrl->SetImageList(imglist, wxIMAGE_LIST_SMALL);

    if(m_cols & FD_NAME)
        m_listCtrl->InsertColumn(0,_T("Name"),wxLIST_FORMAT_LEFT,GetSize().x/2);

    if(m_cols & FD_DATE)
        m_listCtrl->InsertColumn(1,_T("Date"),wxLIST_FORMAT_LEFT,GetSize().x/4);

    if(m_cols & FD_SIZE)
        m_listCtrl->InsertColumn(2,_T("Size"),wxLIST_FORMAT_LEFT,GetSize().x/4);



}

void FileDialog::AddLine(const wxString &name, NameType type)
{

    wxListItem item0, item1, item2;


    item0.SetId( m_listCtrl->GetItemCount() );
    item1.SetId( m_listCtrl->GetItemCount() );
    item2.SetId( m_listCtrl->GetItemCount() );

    item0.SetColumn(0);
    item1.SetColumn(1);
    item2.SetColumn(2);

    wxFont font = item0.GetFont();

    switch(type){
        case DIR:
            item0.SetImage(1);
            font.SetWeight(wxFONTWEIGHT_BOLD);

          break;

        case FILE:
            item0.SetImage(0);

            wxFileName fname(m_currentDir + name);

            wxDateTime acc,mod, created;
            fname.GetTimes(&acc, &mod, &created);
            wxString date = created.FormatISODate() + _T(" ") + created.FormatISOTime();

            item1.SetText(date);

            item2.SetText(fname.GetHumanReadableSize());

          break;

    }


    item0.SetFont(font);
    item0.SetText(name);



    m_listCtrl->InsertItem(item0);
    m_listCtrl->SetItem(item1);
    m_listCtrl->SetItem(item2);

}

void FileDialog::OnTimer( wxTimerEvent& event )
{


    //Logger::logf(LOG_MSG,_T("FileDialog::OnTimer"));



    if(wxFileName::FileExists(m_currentDir + m_currItem)){
        m_selectedFile = m_currentDir + m_currItem;
        SetActualPath(m_selectedFile);
    }else{
        SetActualPath(m_currentDir);
        if(m_isClick)
            OpenFolder(m_currItem);
    }


    m_isClick = false;

}
