#include "FilePosSlider.h"

#include <wx/dcbuffer.h>

#include "logger.h"
#include "appHelper.h"
#include "common/VemsContext.h"
#include "loghistory/FilePlayCommander.h"
#include "events/SliderPosEvent.h"

BEGIN_EVENT_TABLE(FilePosSlider,wxSlider)

//    EVT_SCROLL_THUMBRELEASE(FilePosSlider::OnScrollThumbRelease)

    EVT_SCROLL_THUMBRELEASE(FilePosSlider::OnScroll)
    EVT_SCROLL_TOP(FilePosSlider::OnScroll)
    EVT_SCROLL_BOTTOM(FilePosSlider::OnScroll)
    EVT_SCROLL_LINEUP(FilePosSlider::OnScroll)
    EVT_SCROLL_LINEDOWN(FilePosSlider::OnScroll)
    EVT_SCROLL_PAGEUP(FilePosSlider::OnScroll)
    EVT_SCROLL_PAGEDOWN(FilePosSlider::OnScroll)

    EVT_SCROLL_CHANGED(FilePosSlider::OnScroll)

    EVT_LEFT_DOWN(FilePosSlider::OnMouseDown)

    EVT_ENTER_WINDOW(FilePosSlider::OnMouse)
    EVT_LEAVE_WINDOW(FilePosSlider::OnMouse)
    EVT_MOTION(FilePosSlider::OnMouse)

    //EVT_LEFT_UP(

    //EVT_SCROLL(FilePosSlider::OnScroll)

END_EVENT_TABLE()

FilePosSlider::FilePosSlider(wxWindow* parent, wxWindowID id, int value , int minValue, int maxValue, const wxPoint& point, const wxSize& size, long style, const wxValidator& validator, const wxString& name)
{
    //ctor
    m_currentTime = 0;
    m_sensibility = 1000;
    m_isMouseLeftDown = false;

    m_timeStr = _T("");
    m_ttip = NULL;

    value = 0;
    minValue = 0;
    maxValue = m_sensibility;
    Create(parent, id, value , minValue, maxValue, point, size, style, validator, name);
}

FilePosSlider::~FilePosSlider()
{
    //dtor
}

void FilePosSlider::notify( int i, void* classType )
{
    m_currentTime = i;
}

void FilePosSlider::notify(int i, int idx)
{
//    Logger::logf(LOG_MSG, _T("FPS::notify() i=%d idx=%i"),i,idx);

    // also used for setting slider range
    if ( i == -1 )
    {
        Enable(true);
        SetSensibility(idx);
    }
    else if ( i == -2 )
    {
        SetValue( 0 );
        Enable(false);
    }// and set slider position
    else
    {
        Enable(true);
        if(m_isMouseLeftDown) return;
        int val = (int)(((double)i / (double)idx) * m_sensibility);
        SetValue( val );

        wxFrame* main_frame = dynamic_cast<wxFrame*>(appHelper::getMainFrameWindow());
        wxStatusBar* statusBar = NULL;
        if ( main_frame != NULL )
            statusBar = main_frame->GetStatusBar();

        if ( statusBar != NULL )
        {
            wxString txt;

            int t = m_currentTime;
            wxString tmpS;

            wxString hS = _T("0"), mS; if ( t > 60 ) { int temp = t / 60; t = t % 60;
                hS = wxString::Format( _T("%d"), temp ); }
            if ( t < 10 ) mS = wxString::Format(_T("0%d"), t ); else mS << t;

            txt = hS + _T(":") + mS;

            txt += _T(" / ") + m_timeStr + _T(" (");

            txt << val << _T(" / ") << m_sensibility << _T(")");
            statusBar->SetStatusText( txt, 5 );
        }
    }

  //  Logger::logf(LOG_MSG,_T("FilePosSlider::notify %i, %i, %0.1f%%"),i,idx,(val/((double)m_sensibility))*100.0);
}

void FilePosSlider::Init(const wxString& name, const wxString& msg, const int max)
{


}

void FilePosSlider::notify( int i )
{

}

void FilePosSlider::notify(const wxString& str)
{
    str.ToDouble( &m_time );
    int t = (int)m_time;

    wxString hS = _T("0"), mS; if ( m_time > 60 ) { int temp = t / 60; t = t % 60;
        hS = wxString::Format( _T("%d"), temp );
    }
    if ( t < 10 ) mS = wxString::Format(_T("0%d"), t ); else mS << t;
    m_timeStr = hS + _T(":") + mS;

    wxFrame* main_frame = dynamic_cast<wxFrame*>(appHelper::getMainFrameWindow());
    wxStatusBar* statusBar = NULL;
    if ( main_frame != NULL )
        statusBar = main_frame->GetStatusBar();
    if ( statusBar != NULL )
    {
        wxString txt;
        txt = _T("00:00 / ") + m_timeStr + _T(" (0 /"); txt << m_sensibility;
        txt += _T(")");
        statusBar->SetStatusText( txt, 5 );
    }
    //m_positionStr = str;
    //SetToolTip(str);
}

void FilePosSlider::OnScrollThumbRelease( wxScrollEvent &evt )
{

    Logger::logf(LOG_MSG,_T("FilePosSlider::OnScrollThumbRelease pos=%i"),evt.GetPosition());

}

void FilePosSlider::OnScroll( wxScrollEvent &evt )
{

   // Logger::logf(LOG_MSG,_T("FilePosSlider::OnScroll pos=%i"),evt.GetPosition());

/*    if(m_vemsLogReader == NULL) return;

    m_vemsLogReader->SetFilePos( (double)evt.GetPosition() / (double)m_sensibility );*/

    // int position = (int)( ((double)evt.GetPosition() / (double)m_sensibility ) * 100.0 );
    int position = evt.GetPosition();

    if ( evt.GetEventType() == wxEVT_SCROLL_CHANGED ) {
    observerNotify( position );
    }

    wxFrame* main_frame = dynamic_cast<wxFrame*>(appHelper::getMainFrameWindow());
    wxStatusBar* statusBar = NULL;
    if ( main_frame != NULL )
        statusBar = main_frame->GetStatusBar();

    if ( statusBar != NULL )
    {
        wxString txt;

        double percent = (double)position / (double)m_sensibility;
        int t = (int)((double) m_time * percent);
        wxString tmpS;

        wxString hS = _T("0"), mS; if ( t > 60 ) { int temp = t / 60; t = t % 60;
            hS = wxString::Format( _T("%d"), temp ); }
        if ( t < 10 ) mS = wxString::Format(_T("0%d"), t ); else mS << t;

        txt = hS + _T(":") + mS;

        txt += _T(" / ") + m_timeStr + _T(" (");

        txt << position << _T(" / ") << m_sensibility << _T(")");
        statusBar->SetStatusText( txt, 5 );
    }


    m_isMouseLeftDown = false;

    FilePlayCommander* fpc = dynamic_cast<FilePlayCommander*>( VemsContext::getObject(_T("FilePlayCommander")) );
    if ( fpc )
    {
        SliderPosEvent e;
        e.setCurrentPos( position );
        e.setTotal( m_sensibility );
        fpc->AddPendingEvent( e );
}
}

//void FilePosSlider::SetVemsLogReader(VemsLogReader* vemsLogReader)
//{
//    m_vemsLogReader = vemsLogReader;
//    m_vemsLogReader->registerObserver(this);
//}


void FilePosSlider::OnMouseDown( wxMouseEvent &evt )
{
    //Logger::logf(LOG_MSG,_T("FilePosSlider::OnMouseDown"));

    m_isMouseLeftDown = true;

    evt.Skip();

}

void FilePosSlider::SetSensibility(size_t sensibility)
{
    m_sensibility = sensibility;
    SetRange( 0, m_sensibility );
}

void FilePosSlider::OnMouse( wxMouseEvent& event )
{
    wxPoint sPos = GetScreenPosition();
    wxPoint ePos = event.GetPosition();
    wxSize sSize = GetSize();

    wxString tStr;

    double ratio =
    #if defined ( __WXMSW__) || defined ( WIN32 )
        (double)( ePos.x - 13 ) / (double) ( sSize.x - 28 );
    #else
        (double) GetValue() / (double) m_sensibility;
    #endif

    if ( ratio < 0 ) ratio = 0.0;
    if ( ratio > 1.0 ) ratio = 1.0;
    // calc pos
    int idxPos = (int)((double)m_sensibility * ratio);
    int timePos = (int)((double)m_time * ratio);

    wxString tsM = _T("0"), tsS; if ( timePos > 60 ) { int temp = timePos / 60;
        tsM = wxString::Format( _T("%d"), temp ); timePos = timePos % 60;
    }
    if ( timePos < 10 ) tsS = wxString::Format( _T("0%d"), timePos );
    else tsS << timePos;

    #if defined ( __WXMSW__) || defined ( WIN32 )
    tStr = wxString::Format( _T("appr. %d / %d\n"), idxPos, m_sensibility );
    tStr += ( _T("appr. ") + tsM + _T(":") + tsS );
    tStr += _T(" / ") + m_timeStr;
    #else
    tStr = wxString::Format( _T("%d / %d\n"), GetValue(), m_sensibility );
    tStr += tsM + _T(":") + tsS ;
    tStr += _T(" / ") + m_timeStr;
    #endif

    if ( m_ttip == NULL )
    {
        wxPoint pos = sPos;
        wxSize size = sSize;
        pos += wxSize( 0, size.y );

        m_ttip = new AggToolTipFrame( this, pos );
        m_ttip->SetToolTipText( tStr );
    }

    if ( event.Entering() )
    {
        m_mouseIn = true; m_ttip->Show();
        m_ttip->SetToolTipText( tStr );
        event.Skip(); return;
    }
    if ( event.Leaving() )
    {
        m_mouseIn = false; m_ttip->Hide();
        m_ttip->SetToolTipText( tStr );
        event.Skip(); return;
    }

    if ( event.Dragging() || event.Moving() )
    {
        wxPoint mpos = ePos;
        wxPoint pos = sPos;

        mpos += pos;
        mpos += wxSize( 0, sSize.y );
        m_ttip->Move( mpos );
        m_ttip->SetToolTipText( tStr );
    }

    event.Skip();
}

