#ifndef FIRMWAREINFO_H
#define FIRMWAREINFO_H

#include <wx/string.h>
#include "common/types/BoolHash.h"
#include "common/types/Int2StringHash.h"
#include "FirmwareVersion.h"

/*
FirmwareCommandDesc:
baudrate
name 	mcV
state	=Man
exp_Length;
exp_End_string;
*/

/*
Firmware
compilation time: mdv: Sat Apr 14 14:23:51 CEST 2007<0>
version: mdV: 1.0.73<0>
serial number: mci: 12 byte: decimal
*/
enum firmwareOptions{
    PS2KEYBOARD = 1,
    N2O_RETARD = 2,
    WARNING_BEEP= 4,
    EXPENSIVE_MISC1OUT = 8,
    EXPENSIVE_MISC2OUT = 16,
    INJECTOR_DEADTIME_SIMPLE = 32,
    INJECTOR_TRIM = 64,
    KNOCK_ALTERNATIVE = 128,

    TOYOTA_1JZ_VVTI = 256,
    A340E = 512,
    ALTERNATIVE_AE_DE_FILM = 1024
};


class FirmwareInfo: public wxObject
{
    DECLARE_DYNAMIC_CLASS(FirmwareInfo)
    public:
        FirmwareInfo();
        virtual ~FirmwareInfo();

        void setCompilationTime(const wxString& str);
        void getProhibit(int& prohibit);
        void setFirmwareVersion(const wxString& str);
        void setSerialNumber(const wxString& str);
        void setSignature(const wxString& str);
        void setFirmwareOptions(const wxString& str);
        //void setFirmwareOptions( const wxString& str, int op );
		void setFirmwareCompileOptions(const wxString& str);

        wxString getCompilationTime() const { return m_compilationTime; };
        wxString getFirmwareVersionAsString() const { return m_firmwareVersion; };
        wxString getSerialNumber() const { return m_serialNumber; };
        wxString getSignature() const { return m_signature; };

		wxString getOptions() const { return m_fwOptionsStr; };
		wxString getCompileOptions() const;
        FirmwareVersion getFirmwareVersion() const;

        bool isEmpty() const;
        bool is12x12() const;
        bool is16x14() const;
        bool isHighBoost() const;
        bool isNormalBoost()  const;

        const BoolHash& getDetectedParams()const { return m_detectedParams; }
        static void extractFirmwareVersion( const wxString& s, FirmwareVersion& v );
        void forceParams(const BoolHash& detectedParams);

        bool isValid() const;

        wxString getHeaderFile() const;
		wxString geDefaultConfigFile() const;

		void setDevice(int device);
		int getDevice() const;

		long getSerialNumberAsLong() const;
        wxString getIniFile() const;
        static wxArrayString getAvailableIniFiles( int device );

        bool isVersionAndCompileOptionsIdentical( const FirmwareInfo& firmware ) const;

    protected:
    private:
        int m_device;
        wxString m_compilationTime;
        wxString m_firmwareVersion;
        wxString m_serialNumber;
        wxString m_signature;
        int m_fwOptions;
		wxString m_fwOptionsStr;
        Int2StringHash m_fwOptionsH;
        BoolHash m_detectedParams;


        void fillDetectedParams();
        void setHighAndLowBoost();

        static wxString getIniFileDevicePrefix( int device );
        static wxString getIniFilePlatformPrefix();


//        FirmwareCommandDescA firmvereInfoA;
//        firmvereInfoA[COMP_TIME]
//        firmvereInfoA[SERIAL_NUMBER]
//        firmvereInfoA[VERSION]


};



#endif // FIRMWAREINFO_H
