#include "FirmwareMgrDialog.h"

#include <wx/msgdlg.h>
#include <wx/thread.h>
#include <wx/filename.h>
#include <wx/dir.h>

#include "common/VemsMessageHashHelper.h"

#include "CommunicationBase.h"
#include "VemsTuneController.h"
#include "ProgressBarObserver.h"

#include "vemsprotocol.h"

#include "vemsprotocol/FirmwareMgr.h"
#include "vemsprotocol/BootModeBase.h"
#include "vemsprotocol/BootModeFactory.h"
#include "vemsprotocol/BootModeARM.h"
#include "vemsprotocol/SIPRTypes.h"

#include "project/ProjectHelper.h"
#include "project/ProjectManager.h"

#include "ringbuffer/SFloodThread.h"
#include "logger.h"
#include "UserLog.h"
#include "SimpleFwDlg.h"

#include "vemsprotocol/VemsCommand.h"

#include "DirHelper.h"
#include "events/FirmwareUploadedEvent.h"

LockerHelper::LockerHelper(VemsTuneController* vemsController){
    m_vemsController = vemsController;
    m_vemsController->setECUOnline(false, true);
}


LockerHelper::~LockerHelper(){
    m_vemsController->setECUOnline(true, true);
}


FirmwareMgrDialog::FirmwareMgrDialog(wxWindow* parent, VemsTuneController* vemsController,
    ObserverBase* observer)
{
    m_parent = parent;
    m_vemsController = vemsController;
    m_observer = observer;
    m_projectCaption = _T("Project selection");
    BootModeFactory bootModeFactory;

    //VemsProtocol* vp = m_vemsController->getVemsProtocol();
    m_pVemsProtocol = new VemsProtocol( m_vemsController->getEcuStateMgr() );

    //int device = vp->getDevice();

    int device = m_pVemsProtocol->getDevice();

    m_vemsController->setECUOnline(false, true);


    m_bootModeBase = bootModeFactory.create( m_pVemsProtocol->getCommunicationBase(), device );
    //m_bootModeBase = new BootModeV3(m_vemsController->getVemsProtocol() );
    //m_bootModeBase = new BootModeARM(m_vemsController->getVemsProtocol() );
    if(m_bootModeBase != NULL) m_bootModeBase->registerObserver(m_observer);

}

FirmwareMgrDialog::~FirmwareMgrDialog()
{
    //dtor
    m_vemsController->setECUOnline(true, true);
}

void FirmwareMgrDialog::quickFirmwareMgr(bool enable, int mode)
{
    //LockerHelper locker (m_vemsController);
    //setLocked(true);

    SimpleFwDlg dlg(m_parent, mode);
    //dlg.Init( ProjectHelper::GetPrjMgr()->GetCurrPrj()  );
    dlg.Init(m_vemsController->getFirmwareInfo());

    FirmwareConfigItemArray configItemA;

    if (dlg.ShowModal() == wxID_OK){
        dlg.getFirmwareConfigItemArray( configItemA );
        verifyUpdate(mode, &configItemA);
    }

    //setLocked(false);


    Logger::logf(LOG_MSG,_T("FirmwareMgrDialog::quickFirmwareMgr END"));

}

void FirmwareMgrDialog::verify()
{
    verifyUpdate(VERIFY);
}


void FirmwareMgrDialog::upload()
{
    verifyUpdate(UPLOAD);
}


void FirmwareMgrDialog::verifyUpdate(int type)
{

    SimpleFwDlg dlg(m_parent, type);
    dlg.Init( ProjectHelper::GetPrjMgr()->GetCurrPrj()  );


    FirmwareConfigItemArray configItemA;

    if (dlg.ShowModal() == wxID_OK){
        dlg.getFirmwareConfigItemArray( configItemA );
        verifyUpdate(type, &configItemA);
    }

    /*
    Unidlg2Test dlg(m_parent);
    setupVerifyUploadDialog(type, dlg);
 //   dlg.addBox(m_projectCaption);
   // dlg.sessionSelect();

//    wxString currSession = Helper::getCurSession();
  //  dlg.openSession(currSession);
    Project *prj = ProjectHelper::GetPrjMgr()->GetCurrPrj();

    dlg.setProjectConfigSaver(prj);
    dlg.loadProject( prj->CreateFirmwareConfigItems(dlg.getTypes()) );

    LockerHelper locker (m_vemsController);
    //setLocked(true);
    if (dlg.ShowModal() == wxID_OK){
        //if(m_bootModeBase == NULL) return; ///TODO: MESSAGE DEVICE NOT FOUND

        //setLocked(true);
        FirmwareConfigItemArray* configItemA = dlg.getConfigItemArray();
        //Firmware fw; // = new Firmware();

        verifyUpdate(type, configItemA);

        //TODO: verify = firmware is identical, is different

    }
    //setLocked(false);
    */
}

void FirmwareMgrDialog::showFirmwareResult(int result,  int type)
{

        wxString shortMsg;
        wxString longMsg;
        switch (type)
        {
            case VERIFY:
                shortMsg = _T("Firmware verify");
                longMsg = _T("Firmware verify was successful!");
            break;
            case UPLOAD:
                shortMsg = _T("Firmware upload");
                longMsg = _T("Firmware upload was successful!");
            break;
        }

        Logger::logf(LOG_MSG, _T("FirmwareMgrDialog::showFirmwareResult() result=%i"),result);
        wxMessageDialog * msgDlg;
        //wxMessageDialog(this, _T("Firmvare config is identical!"),
        //_T("Result"),wxOK |wxICON_ERROR, wxDefaultPosition);
        if(result == 0){
            msgDlg = new wxMessageDialog( m_parent, longMsg,
            shortMsg, wxOK, wxDefaultPosition);
            msgDlg->ShowModal();

            // send event
            if ( type == UPLOAD )
            {
                FirmwareUploadedEvent evt;
                m_parent->AddPendingEvent( evt );
            }

        }else if(result == -1){
            serialErrorMsgDialog();
        } else if ( result == -2) {
            msgDlg = new wxMessageDialog( m_parent, _T("HexFile doesn't exists!"),
                shortMsg, wxOK |wxICON_ERROR, wxDefaultPosition);
            msgDlg->ShowModal();
        }else if ( result > 0 ){
            msgDlg = new wxMessageDialog( m_parent, _T("Firmware is different!\nMore details can be found on the Debug tab."),
            shortMsg, wxOK |wxICON_ERROR, wxDefaultPosition);
            msgDlg->ShowModal();
        } else {
            LOGP( _T("INVALID RESULT!") );
        }

        //delete msgDlg;
}


void FirmwareMgrDialog::serialErrorMsgDialog()
{
     wxMessageDialog *dialog = new wxMessageDialog( m_parent,
        _T("It seems that there is some problems with Vems communication.\nCheck power, cables and serial port settings."),
        _T("Communication Error"), wxOK);

    dialog->ShowModal();
    //delete dialog;
}

void FirmwareMgrDialog::serialErrorMsgDialog(SerialException& se)
{
     wxMessageDialog *dialog = new wxMessageDialog( m_parent,
        se.toString(),
        _T("Communication Error"), wxOK);

    dialog->ShowModal();
    //delete dialog;
}


/*
void FirmwareMgrDialog::showFirmwareConfigResult(int result)
{
        wxMessageDialog * msgDlg;
        //wxMessageDialog(this, _T("Firmvare config is identical!"),
        //_T("Result"),wxOK |wxICON_ERROR, wxDefaultPosition);
        if(result == 0){
            msgDlg = new wxMessageDialog( m_parent, _T("Firmware config is identical!"),
            _T("Firmware config verify"), wxOK, wxDefaultPosition);
              msgDlg->ShowModal();
        }else if(result < 0){
            serialErrorMsgDialog();
        }else{
             msgDlg = new wxMessageDialog( m_parent, _T("Firmware config is different!\nMore details can be found on the Debug tab."),
            _T("Firmware config verify"), wxOK |wxICON_ERROR, wxDefaultPosition);
              msgDlg->ShowModal();
        }

        //delete msgDlg;
}
*/


void FirmwareMgrDialog::roundSFloodHack()
{
    int tryCount = 0;
    int ecuState =  m_pVemsProtocol->setState(ECUInfo::BOOT);

    while( (ecuState != ECUInfo::BOOT) && tryCount < 3){

        wxThread* thread = startSflood();
        VemsMessage vemsMsg = VemsMessageHashHelper::GetVemsMsg(_T("boot_with_sflood"));

        wxString longMsg = vemsMsg.getHtmlMsg();
        wxString shortMsg= vemsMsg.getMsg();

        wxMessageBox(longMsg, shortMsg, wxOK | wxICON_ERROR |wxSTAY_ON_TOP);

        stopSflood(thread);
        m_pVemsProtocol->clear();
        ecuState =  m_pVemsProtocol->setState(ECUInfo::BOOT);
        tryCount++;
    }
}

void FirmwareMgrDialog::verifyUpdate(int type, FirmwareConfigItemArray* configItemA)
{
        if(m_bootModeBase == NULL){
            Logger::logf(LOG_MSG, _T("FirmwareMgrDialog::verifyUpdate() m_bootModeBase == NULL") );
            serialErrorMsgDialog();
            return;
        }

        // FirmwareMgr fw(m_vemsController->getVemsProtocol(), m_bootModeBase);
        FirmwareMgr fw( m_pVemsProtocol, m_bootModeBase );

        ProgressBarObserver * observer = new ProgressBarObserver();
        observer->setWindow(m_parent);
        m_bootModeBase->registerObserver(observer);

        fw.registerObserver((ObserverBase *)m_observer);
        int result; //= fw->firmwareMgr(configItemA, type);
        LOGP( wxString::Format( _T("observer: %p, m_observer: %p"),observer, m_observer ) );
        fw.observerNotify(_T("Error: hex File does not exist!"));

        int device = m_pVemsProtocol->getDevice();
        int ecuState =  m_pVemsProtocol->setState(ECUInfo::BOOT);

        switch(device){
            case ECUInfo::DEVICE_ROUND:
                roundSFloodHack();
                break;
            default:
                if( ecuState != ECUInfo::BOOT ){
                    Logger::logf(LOG_MSG, _T("FirmwareMgrDialog::verifyUpdate() ECU NOT IN BOOT MODE state:%d"),ecuState );
                    VemsMessage vemsMsg = VemsMessageHashHelper::GetVemsMsg(_T("boot_failed"));
                    wxString longMsg = vemsMsg.getHtmlMsg();
                    wxString shortMsg= vemsMsg.getMsg();
                    wxMessageBox(longMsg, shortMsg, wxOK | wxICON_ERROR |wxSTAY_ON_TOP  );
                    return;
                }
                break;
        }

        /// successfully entered in boot mode!
        try{
            //result = fw.firmwareMgr(configItemA, (process_type) type);

            if(type == UPLOAD){
                result = fw.upload(configItemA);
            }else{// if(type == VERIFY){
                result = fw.verify(configItemA);
            }

        }catch(SerialException& se){
            Logger::logf(LOG_MSG, _T("FirmwareMgrDialog::verifyUpdate() SerialException") );
            serialErrorMsgDialog(se);
            return;
        }
        showFirmwareResult(result, type);
}

void FirmwareMgrDialog::setupVerifyUploadDialog(int type, Unidlg2Test& dlg)
{
    wxString deviceStr;
    if(m_bootModeBase != NULL){
        ECUInfo::Device device = m_bootModeBase->getDevice();
        deviceStr = ECUInfo::getDeviceAsString(device);
    }else{
        deviceStr  = _T("UNKNOWN");
    }

    wxString name, tmp;
    if(type == VERIFY){
        name = _T("Verify Firmware " + deviceStr);
        tmp = _T("Verify");
    }else if(type == UPLOAD){
        name = _T("Upload Firmware " + deviceStr);
        tmp = _T("Upload");
    }else{
        return;
    }
    dlg.setButtonOK(tmp);
    dlg.SetLabel(name);

    //dlg-setTitle(name);
    dlg.setDialogType(_T("firmware"));
}

wxThread* FirmwareMgrDialog::startSflood()
{
    // CommunicationBase* commBase = m_vemsController->getVemsProtocol()->getCommunicationBase();
    CommunicationBase* commBase = m_pVemsProtocol->getCommunicationBase();
    commBase->setBaudRate( 19200 );

    SFloodThread* floodThread = new SFloodThread( m_vemsController, commBase );

    if( floodThread->Create() != wxTHREAD_NO_ERROR )
    {
        Logger::logf(LOG_MSG, _T("MainFrame::OnSFlood() - Can't create \"S\" flood thread!"));
    }

    floodThread->Run();

    return floodThread;

}

void FirmwareMgrDialog::stopSflood(wxThread* thread)
{
    thread->Delete();
}

CfgUploadDlg::CfgUploadDlg( wxWindow * parent, wxString& filename ) : wxDialog( parent, -1, _T("Select config file to upload"), wxDefaultPosition )
{
    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    wxStaticText* txt = new wxStaticText( this, -1, _T("Select a config file to upload. Press now OK to upload the previous config file\nto the ECU or you can browse for another file and then press OK\nYou can skip this step by pressing Cancel") );

    sizer->Add(txt, 0, wxEXPAND | wxALL, 5 );

    m_filepicker = new wxFilePickerCtrl( this, -1, filename, _T("Browse"), _T("*.vemscfg"), wxDefaultPosition, wxSize( 360, -1 ) );

    sizer->Add( m_filepicker, 1, wxEXPAND | wxALL, 5 );

    sizer->Add( CreateButtonSizer( wxCANCEL | wxOK ), 1, wxEXPAND | wxALL, 5 );

    SetSizer( sizer );
    sizer->SetSizeHints( this );
}

