#ifndef FRAMELAYOUTHANDLER_H
#define FRAMELAYOUTHANDLER_H

#include <wx/aui/aui.h>
#include <wx/artprov.h>
#include <wx/laywin.h>
#include <wx/xml/xml.h>
#include <wx/menu.h>
#include <wx/arrimpl.cpp>
#include <wx/config.h>
#include <wx/hash.h>
#include <wx/hashset.h>

#define TB_VERTICAL     _T("vertical")
#define TB_HORIZONTAL   _T("horizontal")

#include "AuiPaneToolBar.h"
#include "toolbartools/VemsToolBarToolBase.h"
#include "SymbolNameSaverBase.h"
#include "LayoutUpdaterBase.h"
#include "SendObjectBase.h"

#include "panels/DebugPanel.h"
#include "common/MenuExtendsHandlerBase.h"

///forwards
class VemsAuiManager;
class RTDataMgrBase;
class DebugTabObserver;
class VemsFlagWidget;
class FilePosSlider;
class VemsDeviceControllerEvent;
class CommunicationEvent;


WX_DECLARE_HASH_MAP( int, VemsToolBarToolBase*, wxIntegerHash, wxIntegerEqual, VemsToolBarToolBaseHash);
WX_DECLARE_HASH_MAP( int, VemsFlagWidget*, wxIntegerHash, wxIntegerEqual, VemsFlagWidgetHash);
WX_DECLARE_HASH_SET( VemsFlagWidget*, wxPointerHash, wxPointerEqual, VemsFlagWidgetSet );
WX_DECLARE_HASH_SET( FilePosSlider*, wxPointerHash, wxPointerEqual, FilePosSliderSet );

class VtbtNeedObj
{

    public:
        VtbtNeedObj(ToolBarToolsNeedId id, VemsToolBarToolBase * targettool) {
            m_id   = id;
            m_tool = targettool; };

        virtual ~VtbtNeedObj() {};

        VemsToolBarToolBase *m_tool;
        ToolBarToolsNeedId  m_id;


};

WX_DEFINE_ARRAY(VtbtNeedObj*, VtbtNeedArray);


class FrameLayoutHandler: public wxEvtHandler, public SymbolNameSaverBase, public LayoutUpdaterBase, public SendObjectBase, public MenuExtendsHandlerBase
{
    public:
        FrameLayoutHandler(wxFrame *frame, RTDataMgrBase * rtDataMgr);
        void SetAuiMDIParentFrame( wxAuiMDIParentFrame * frame ) { m_auiMDIFrame = frame; }

        virtual ~FrameLayoutHandler();

        void ResetToolBarPositions();

        void SetMainWindow(wxWindow *win);

        //void SetPerspective(const wxString &perspective) { m_perspective = perspective; };
        void SetMenu(wxMenu *menu) {
            m_menu = new wxMenu();
            menu->AppendSubMenu(m_menu,_T("ToolBars"));
            m_isMenu = true;  };

        wxString GetPerspective();
        VemsAuiManager* GetAuiMgr() { return m_auiMgr; };

//        void SavePerspective( const wxRect& rect, bool isFullScreen, bool isMaximized, int mdiClientHeight = -1 );
        //bool SendObject(ToolBarToolsNeedId id, wxObject *obj, const wxString &message = wxEmptyString);
        bool SendObject(int id, wxObject *obj, const wxString &message = wxEmptyString);

        //void SetVemsTuneCtrl(VemsTuneController *ctrl) { m_vemsTuneController = ctrl; };
        void SendEvent( ToolBarToolsNeedId event, int value );

        void LoadMainPanel();
        void LoadLayout();


        bool Init( const wxString& perspective, const wxString& layoutFile );

        //bool LoadFromXML();
        bool LoadFromSQL();

//        DebugTabObserver * GetDebugTabObserver() { return m_debugPanel->GetObserver(); };

        bool SetToolBarIconParam(const wxString &tbname, const wxString &itemname, const wxString &oldparam, const wxString &newparam);
        bool SetToolBarIconParam(const wxString &param, int itemid);

        void AddToolBarIconWithParam(const wxString &tbname, const wxString &itemname, const wxString &param);
        int  AddToolBarIconWithParam(int toolbar_id, const wxString &itemname, const wxString &param, const wxString& type = _T("Icon"));

        VemsToolBarToolBase* AddNewToolBarTool(wxToolBar *toolbar, const wxString &tbname, const wxString &name, short pos, const wxString &param1);

        void RemoveToolBarTool(wxToolBar *toolbar, const wxString &tbname, int toolid, int itemid);

        void RefreshToolBar(wxToolBar *toolbar, const wxString &tbname);

        short isUseSQL();

        DebugPanel * GetDebugPanel() { return m_debugPanel; };

        void ShowToolBars(bool isShow);

        void Clear();

        virtual bool SaveSymbol(int itemid, const wxString& name);

        void EnableAllTool( bool enable = true );

        void UpdateAUI();

        wxString GetToolBarPositions();

        bool IsCorruptedToolbarLayout();

        void UpdateLayout(const wxVariantHash& params);

        void LoadWelcomeScreen();
        void LoadLogTab();
        void CloseWelcomeScreen();

        void OnTabClose(wxCloseEvent &evt);

        void SetTabsMenu(wxMenu* menu) { m_tabsMenu = menu; };

        void ExtendMenu(wxMenu* menu, wxObject* sender);

    protected:

        VemsToolBarToolBase* AddToolBarTool(wxToolBar *toolbar, const wxString &name, short pos = -1, const wxString &param1 = wxEmptyString, int itemid = -1);
        int AddControl(wxToolBar *toolbar, const wxString &name, const wxSize& size, int itemid = -1);
        VemsFlagWidget* AddFlag(wxToolBar * toolbar, int FlagSize, const wxString& iName, int itemid);
        void AddPaneSize(AuiPaneToolBar *paneTB, wxSize size, const wxString &ToolBarOri);

        void OnFlagMenu(wxCommandEvent& event);

    private:
        /// evt handlers
        void OnMove(wxMoveEvent& event);
        void OnMenu(wxCommandEvent& event);
        void OnVemsDeviceControllerEvent( VemsDeviceControllerEvent& event );
        void OnCommunicationEvent( CommunicationEvent& event );
        void OnTabMenu(wxCommandEvent& event);

        void RemoveToolBarTool(wxToolBar *toolbar, const wxString &tbname, int toolid );
        void RemoveToolBarToolFromDB( int itemid );

        void ClearFlags();
        void ClearToolBarTools();
        void ClearControls();
        void ClearToolBars();

        /// not erasable pointers
        wxWindow        *m_mainWindow;
        wxFrame         *m_parentFrame;
        RTDataMgrBase   *m_rtDataMgr;
        wxMenu          *m_menu;
        wxMenu*         m_tabsMenu;

        wxAuiMDIParentFrame * m_auiMDIFrame;

        /// Erasable pointers
        //wxAuiManager    *m_auiMgr;
        VemsAuiManager      *m_auiMgr;

        /// vars
        wxString            m_perspective;
        AuiPaneToolBarHash  m_auiToolHash;
        bool                m_isMenu;
        VtbtNeedArray       m_vtbtNeedArray;


        DebugPanel          *m_debugPanel;

        wxString            m_guiLayoutFile;
        wxXmlDocument       m_guiLayoutXmlDoc;


        VemsToolBarToolBaseHash m_toolBarTools;

        VemsFlagWidgetHash      m_flags;
        FilePosSliderSet        m_controls;

        enum {
            idTabStartHere = 29000,
            idTabLog,
            idMenuTabStartHere,
            idMenuTabLog
        };

};


#endif // FRAMELAYOUTHANDLER_H
