#include "InterpolateViewerData.h"
#include "logger.h"
#include "mtconfigdata.h"

InterpolateViewerData::InterpolateViewerData(const DataLogEntryArray* entries)
{
    m_entries = entries;
    //ctor
}

InterpolateViewerData::~InterpolateViewerData()
{
    //dtor
}


void InterpolateViewerData::SetRefKey(wxString key)
{
    m_refKey = key;
}

void InterpolateViewerData::SetValues(wxArrayString* valueList)
{
    m_valueList = valueList;
}

bool InterpolateViewerData::AddRecord(DoubleHash* table)
{
    //Logger::logf(LOG_MSG, _T("InterpolateViewerData::AddRecord() table size:%i"), table->size() );

    bool isBlockEnd = false;
    DoubleHash* localtable = new DoubleHash();
    *localtable = *table;       /// copy

    //Logger::logf(LOG_MSG, _T("InterpolateViewerData::AddRecord() 111") );
    if( (*localtable)[m_refKey] == 0.0 ){ //if 0 received new GPS data
        makeInterpolate(localtable);
        interlaceStringLines();
        //Logger::logf(LOG_MSG, _T("InterpolateViewerData::AddRecord:  m_DoubleHashArraySize: %d\n"), m_DoubleHashArray.GetCount());

        for ( size_t i = 0; i < m_DoubleHashArray.GetCount(); ++i )
        {
            m_DoubleHashArray[i]->clear();
            delete m_DoubleHashArray[i];
        }
        m_DoubleHashArray.Clear();

        isBlockEnd = true;
    }
    //Logger::logf(LOG_MSG, _T("InterpolateViewerData::AddRecord() 333") );
    m_DoubleHashArray.Add(localtable);

    return isBlockEnd;
}
int InterpolateViewerData::makeInterpolate(DoubleHash *doubleHashLastLine)
{

    //doubleHashFirstLine = new DoubleHash();
    size_t arraySize = m_DoubleHashArray.GetCount();
    //Logger::logf(LOG_MSG, _T("InterpolateViewerData::makeInterpolate() m_DoubleHashArray size:%i"),  arraySize);
    if(arraySize == 0){
        return 0;
    }
    DoubleHash* doubleHashFirstLine = m_DoubleHashArray[0];

    //Logger::logf(LOG_MSG, _T("InterpolateViewerData::makeInterpolate() 222") );
    for(unsigned int cnt1 = 0; cnt1 < m_valueList->GetCount(); cnt1++){
        //Logger::logf(LOG_MSG, _T("InterpolateViewerData::makeInterpolate() 333") );
        wxString key = (*m_valueList)[cnt1];
        double firstValue = (*doubleHashFirstLine)[key];
        double lastValue  = (*doubleHashLastLine)[key];
        double delta = (lastValue - firstValue)/(double)arraySize;
        //Logger::logf(LOG_MSG, _T("InterpolateViewerData::makeInterpolate:  firstvalue %f\n"), firstValue);
        //Logger::logf(LOG_MSG, _T("InterpolateViewerData::makeInterpolate:  lastvalue %f\n"), Lastvalue);
        for(unsigned int cnt2 = 0;cnt2 < m_DoubleHashArray.GetCount();cnt2++ ){
           // Logger::logf(LOG_MSG, _T("InterpolateViewerData::makeInterpolate:  value %d\n"), firstValue+delta*cnt2);
           (*m_DoubleHashArray[cnt2])[key] = firstValue+delta*cnt2;
        }
    }

    return 1;
}
wxString InterpolateViewerData::makeStringLine(DoubleHash *doubleHashLine)
{

    //static wxString CSVConverter::getLine(const DataLogEntryArray& dataDesc, DoubleHash* data);

    DoubleHash::const_iterator it;
    wxString line;
    for(size_t i=0; i<m_entries->GetCount(); i++){

        wxString name = m_entries->Item(i)->m_name;

    //    Logger::log(LOG_MSG, _T("RTDataLogWriter::refresh() name: ") +name);
        it = doubleHashLine->find(name);
        if(it == doubleHashLine->end()){
            line += _T("---");
            line += _T("\t");
    //         Logger::log(LOG_MSG, _T("RTDataLogWriter::refresh() name Not Found!!!!"));
        }else{
            if(m_entries->Item(i)->m_type == _T("int")){
                line += wxString::Format(m_entries->Item(i)->m_format, (int)it->second) + _T("\t");
            }else{
                line += wxString::Format(m_entries->Item(i)->m_format, it->second) + _T("\t");
            }
        }
    }
    return line;
}
void InterpolateViewerData::interlaceStringLines()
{   DoubleHash *doubleHashLine;
    //doubleHashLine = new DoubleHash();
    m_lines.Clear();
    for(unsigned int cnt = 0; cnt < m_DoubleHashArray.GetCount();cnt++){
        doubleHashLine = m_DoubleHashArray[cnt];
        m_lines += makeStringLine(doubleHashLine) +_T("\n");
    }

    //doubleHashLine->clear();
    //delete doubleHashLine;
}
wxString  InterpolateViewerData::getLines()
{
    return m_lines;
}
void InterpolateViewerData::clearDoubleHashArray()
{
    m_DoubleHashArray.Clear();
}
