#include "LoggerWidget.h"

#include "DirHelper.h"

#include "rtdatamanager/RTDataMgrFacade.h"
//#include "rtdatamanager/RTDataMgrSerial.h"

#include "MTConfigHelper.h"
#include "rtdatamanager/RTDataInputRingBuffer.h"
//#include "vemsgauge/GaugeFrameBase.h"

#include "logger.h"
#include <wx/filename.h>

//#define PATHSEP wxFileName::GetPathSeparator()

#if defined (__WINCE_STANDARDSDK__)
    #include <aygshell.h>
#endif

int idHighEdit = wxNewId();
int idHighCheckBox = wxNewId();
int idStartButton = wxNewId();
int idStopButton = wxNewId();
int idFileMethodsComboBox = wxNewId();


BEGIN_EVENT_TABLE(LoggerWidget, wxControl)

    EVT_BUTTON(idStartButton, LoggerWidget::OnStartClick)
    EVT_BUTTON(idStopButton , LoggerWidget::OnStopClick)
    EVT_CHECKBOX(idHighCheckBox, LoggerWidget::OnHighCheckClick)

END_EVENT_TABLE()

LoggerWidget::LoggerWidget(wxWindow* parent, wxWindowID id, wxPoint gPos, int gSize, bool isShow)
            :m_DataLogWriter (NULL)
            ,m_rtDataMgr (NULL)
{
    Logger::logf(LOG_MSG,_T("LoggerWidget CTOR START"));
    //ctor
    Create(parent,id,gPos,wxSize(gSize,(int)(gSize*0.7)), 0 ,  wxDefaultValidator,_T("LoggerWidget"));
    Show(isShow);


    //
    HighText = new wxStaticTextEx(this,wxID_ANY, _T("XLS log"),wxPoint(5,12),wxSize(50,15));
    HighEdit = new wxTextCtrl(this,idHighEdit,_T(""),wxPoint(60,10), wxSize(140,20));
    HighCheckBox = new  wxCheckBox(this, idHighCheckBox, _T(""), wxPoint(205,13));
    HighText->SetForegroundColour(wxColour(255,255,255));
    //HighEdit->Enable(false);
    HighEdit->SetValue(_T("datalog.log"));

    HighEdit->Connect(wxEVT_SET_FOCUS, wxFocusEventHandler(LoggerWidget::OnTextEditFocus));
    HighEdit->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(LoggerWidget::OutTextEditFocus));


    //
    choices.Add(_T("Overwrite"));
    choices.Add(_T("Append"));
    choices.Add(_T("New with timestamp"));
    choices.Sort();
    FileMethodsComboBox = new wxComboBox(this,idFileMethodsComboBox,_T("New with timestamp"), wxPoint(95,80), wxSize(125,20), choices, wxCB_READONLY);

    FileMethodsText = new wxStaticTextEx(this,wxID_ANY, _T("When file is exist:"),wxPoint(5,83),wxSize(90,15));
    FileMethodsText->SetForegroundColour(wxColour(255,255,255));


    //
    startButton = new wxButton(this, idStartButton, _T("Start"), wxPoint(gSize/4 - 30, 110 ) , wxSize(60,20) );
    stopButton  = new wxButton(this, idStopButton , _T("Stop") , wxPoint(gSize/4*3 - 30, 110 ) , wxSize(60,20) );
    startButton->Enable(false);

    //m_hWnd = (HWND)(GetHandle());

    m_window = parent;

    Logger::logf(LOG_MSG,_T("LoggerWidget CTOR END"));

}

LoggerWidget::~LoggerWidget()
{
    //dtor

    delete HighText;
    delete HighEdit;
    delete HighCheckBox;

    delete startButton;
    delete stopButton;
    delete FileMethodsText;
    delete FileMethodsComboBox;

}

void LoggerWidget::OnTextEditFocus(wxFocusEvent& event)
{
    Logger::logf(LOG_MSG,_T("OnTextEditFocus"));

#if defined (__WINCE_STANDARDSDK__)
    SHSipPreference((HWND)(GetHandle()), SIP_UP);
#endif

}

void LoggerWidget::OutTextEditFocus(wxFocusEvent& event)
{
    Logger::logf(LOG_MSG,_T("OutTextEditFocus"));

#if defined (__WINCE_STANDARDSDK__)
    SHSipPreference((HWND)(GetHandle()), SIP_DOWN);
#endif

}


void LoggerWidget::setHighEdit(wxString name)
{

    HighEdit->SetValue(name);

}

void LoggerWidget::SetDataLogWriter( RTDataLogWriter * DataLogWriter)
{

    m_DataLogWriter = DataLogWriter;

    if(m_DataLogWriter->IsRegistered()) {
        //setHighEdit(m_DataLogWriter->getFileName());
        HighCheckBox->SetValue(true);

        HighCheckBox->Enable(false);
        FileMethodsComboBox->Enable(false);
        HighEdit->Enable(false);
    }

}

void LoggerWidget::SetDataMgr( RTDataMgrBase *rtDataMgr)
{

    m_rtDataMgr = rtDataMgr;


}


void LoggerWidget::SetAndOpenHighLogFile()
{
    if(m_DataLogWriter == NULL) return ;

    wxString comboValue = FileMethodsComboBox->GetValue();

    if(comboValue == _T("Overwrite") ) {
        m_HighLogName = DirHelper::GetPath(dLOG) + HighEdit->GetValue();
        m_DataLogWriter->set(m_HighLogName,wxFile::write);

    }else if(comboValue == _T("Append") ){
        m_HighLogName = DirHelper::GetPath(dLOG) + HighEdit->GetValue();
        m_DataLogWriter->set(m_HighLogName,wxFile::write_append);

    }else if(comboValue == _T("New with timestamp") ){
        m_HighLogName = DirHelper::GetPath(dLOG) + HighEdit->GetValue();

        if(wxFileExists(m_HighLogName)){

            m_HighLogName = DirHelper::CreateNewFileName(HighEdit->GetValue(),DirHelper::GetPath(dLOG),true,0);

            m_DataLogWriter->set(m_HighLogName,wxFile::write);

            //HighEdit->SetValue();

        }
        else {
           m_DataLogWriter->set(m_HighLogName,wxFile::write);
        }


    }

}

wxString LoggerWidget::StartLogging(bool overcheck)
{

    if(m_rtDataMgr == NULL) return _T("LoggerWidget::StartLogging() m_rtDataMgr is NULL!");

    if(overcheck) HighCheckBox->SetValue(true);

    if(HighCheckBox->GetValue() ) {
        if(m_DataLogWriter == NULL) return _T("LoggerWidget::StartLogging() m_DataLogWriter is NULL!");

        SetAndOpenHighLogFile();
/*
        if( ((RTDataMgrFacade*)m_rtDataMgr)->getInputType() != INPUT_TYPE_ECU){
            ((RTDataMgrFacade*)m_rtDataMgr)->SetInput(
                    new RTDataMgrSerial(VemsProtocolHelper::getVemsProtocol(),MTConfigHelper::getMTConfigData()));
        }
*/
/*
        if( ((RTDataMgrFacade*)m_rtDataMgr)->getInputType() != INPUT_TYPE_RINGBUFFER){
            ((RTDataMgrFacade*)m_rtDataMgr)->SetInput(
                    new RTDataInputRingBuffer(m_timer->getRingBuffer(),MTConfigHelper::getMTConfigData()));
        }
*/
        m_rtDataMgr->RegisterViewer(m_DataLogWriter);
        m_DataLogWriter->SetDataMgr(m_rtDataMgr);

        HighCheckBox->Enable(false);
        HighEdit->Enable(false);
    }

    startButton->Enable(false);
    FileMethodsComboBox->Enable(false);


    return _T("LoggerWidget::StartLogging() END");

}

void LoggerWidget::OnStartClick(wxCommandEvent& event)
{

    StartLogging();

}

void LoggerWidget::OnStopClick(wxCommandEvent& event)
{

    if(m_DataLogWriter->IsRegistered()) {
        HighEdit->Enable(true);
        startButton->Enable(true);
        HighCheckBox->Enable(true);
        FileMethodsComboBox->Enable(true);
        m_DataLogWriter->UnRegisterViewer();
    }

}

void LoggerWidget::OnHighCheckClick(wxCommandEvent& event)
{

    startButton->Enable(HighCheckBox->GetValue());

}

