/*
  -----------------------------------------------------------------------
    MGraphics library header
  -----------------------------------------------------------------------
    Majer Jnos
    GD
    2007.07.28
  -----------------------------------------------------------------------
*/



//-----------------------------------------------------------------------
#ifndef MGRAPHICS_H
  #define MGRAPHICS_H


//-----------------------------------------------------------------------
#include <wx/dcbuffer.h>
#include <wx/wx.h>
#include "wx/rawbmp.h"
#include "wx/image.h"

//#include "../vemsgaugedesc.h"
#include "../common/wxhashdef.h"

//-----------------------------------------------------------------------
#define SIZE_ROW 300 // 3 * 100 BYTE

//-----------------------------------------------------------------------
// RAW (native) pixel acces iterator
typedef wxPixelData<wxBitmap, wxNativePixelFormat> PixelData;
// typedef wxPixelData<wxBitmap, wxAlphaPixelFormat> PixelData; // slow very slow


//-----------------------------------------------------------------------
// Super-sampling hitlist structure
struct TSSHit
  {
    BYTE  Hit;   // How many hits captured, equal 0 (zero) sign the pixel isn't hitted yet, and XPos is not used, so can modified it
    int   XPos;  // X coordinate position in given row

    TSSHit()
      {
        Hit   = 0;
        XPos  = 0;
      }
  };//struct TSSHit


//-----------------------------------------------------------------------
// I will port my efficient geometric point classes, until I use this simple
// structure
//
struct TDPoint
  {
    double x;
    double y;

    TDPoint()
      {
        x = 0;
        y = 0;
      }

    TDPoint(double sx, double sy)
      {
        x = sx;
        y = sy;
      }
  };//struct TDPoint

void ChangePoint(TDPoint *P1, TDPoint *P2);


//-----------------------------------------------------------------------
// utility structure for TChainBuffer
struct TSegChain
  {
    TSegChain *Prev;
    TSegChain *Next;      // pointer to next element in the chain

    int Pos;              // stored value of bound position

    TSegChain()
      {
        Next = Prev = NULL;
        Pos = -1;
      }

    TSegChain(int p)
      {
        Next = Prev = NULL;
        Pos = p;
      }
  };//struct TSegChain

//-----------------------------------------------------------------------
void InsertItem(TSegChain **Root, TSegChain *Item);

//-----------------------------------------------------------------------
void Clear(TSegChain *Root);

//-----------------------------------------------------------------------
bool SearchBound(TSegChain *Root, int *From, int *To);

//-----------------------------------------------------------------------
// Simple RGB structure for acces a pixel color elements
// Not too effecient, late I put a more efficient solution
struct TRGB
  {
    BYTE  R;
    BYTE  G;
    BYTE  B;
    TRGB()
      {
        R = 0;
        G = 0;
        B = 0;
      }
    TRGB(BYTE r, BYTE g, BYTE b)
      {
        R = r;
        G = g;
        B = b;
      }
    void GetRGB(BYTE *r, BYTE *g, BYTE *b)
      {
        *r = R;
        *g = G;
        *b = B;
      }
  };


//-----------------------------------------------------------------------
// Pixel set macros:
// -----------------
//  To use this macros you need define R, G, an B variables on
//  caller level in the source code


//-----------------------------------------------------------------------
// Ezt mr elfogadta, de ez nem embereknek val!
//#define SET_PIXEL pB = SL[Y] + 3 * X; *pB++ = B; *pB++ = G; *pB = R;

//-----------------------------------------------------------------------
// Fontos hogy a az inkrementl s indirekcis opertorok csak "egy sorban"
// fordulhatnak el, az informatika istenei ltal sszetkolt fordt (MVS)
// csak ezt tudja rtelmezni, n meg ezt nem, mrmint hogy ez mirt van gy....
//
#define SET_PIXEL             \
        pB = SL[Y] + 3 * X;   \
        *pB++ = B; *pB++ = G; *pB = R;

//-----------------------------------------------------------------------
#define BEGIN_ALIAS           \
        pB = SL[Y] + 3 * X; *pB++ = (BYTE)(e * *pB + f * B); *pB++ = (BYTE)(e * *pB + f * G); *pB = (BYTE)(e * *pB + f * R);

//-----------------------------------------------------------------------
#define END_ALIAS             \
        pB = SL[Y] + 3 * X; *pB++ = (BYTE)(f * *pB + e * B); *pB++ = (BYTE)(f * *pB + e * G); *pB = (BYTE)(f * *pB + e * R);


/**
// Megdbbent hogy a MVS az albbi makrkat nem tudja megemszteni
// (C++Builder-nek meg se kottyan)
// C/C++ szempontjbl az albbi makrk megegyeznek a fentiekkel, MVS szerint nem!
// Elvileg egy "normlis" fordt (annak preprocesszora) az albbi tbb soros makrt
// sz nlkl feldolgozza, nem gy az MVS...

//-----------------------------------------------------------------------
#define SET_PIXEL \
        pB = SL[Y] + 3 * X; \
        *pB++ = B;          \
        *pB++ = G;          \
        *pB   = R;

//-----------------------------------------------------------------------
#define BEGIN_ALIAS \
        pB = SL[Y] + 3 * X;       \
        *pB++ = e * *pB + f * B;  \
        *pB++ = e * *pB + f * G;  \
        *pB = e * *pB + f * R;

//-----------------------------------------------------------------------
#define END_ALIAS \
        pB = SL[Y] + 3 * X;       \
        *pB++ = f * *pB + e * B;  \
        *pB++ = f * *pB + e * G;  \
        *pB = f * *pB + e * R;

**/



//-----------------------------------------------------------------------
#define STORE_CBUFFER \
        Item = new TSegChain(); \
        Item->Pos = X;          \
        InsertItem(&(CBuffer[Y]),Item);

//-----------------------------------------------------------------------
#define STORE_SB            \
        if(SB[Y][0] == -1)  \
          SB[Y][0] = X;     \
        else                \
          SB[Y][1] = X;

//-----------------------------------------------------------------------
// Colour setter macros

#define RGB_RED       R = 255;  G = 0;    B = 0;
#define RGB_GREEN     R = 0;    G = 255;  B = 0;
#define RGB_BLUE      R = 0;    G = 0;    B = 255;
#define RGB_MAGENTA   R = 255;  G = 0;    B = 255;
#define RGB_YELLOW    R = 255;  G = 255;  B = 0;
#define RGB_CYAN      R = 0;    G = 255;  B = 255;


//-----------------------------------------------------------------------
int Round(double x);

//-----------------------------------------------------------------------
// int Sign(double x);

//-----------------------------------------------------------------------
// Majer graphics class with antialiased drawing, and "fast" (enough fast)
// bitmap manipulating with wxWidget graphichal classes.
//
class TMGraphics
  {
    public:
      TMGraphics();
      ~TMGraphics();
      bool CopyToBitmap(wxBitmap **bmp_T);
      bool CopyFromBitmap(wxBitmap *bmp_S);
      bool CopyFrom(TMGraphics *From);

      void GetMemoryDC(wxMemoryDC *DC);
      bool CheckRawAccess();

      bool IsValid();
      bool Reset(int w, int h);
      bool BuildSL();

      //-----------------------------------------------------------------------
      // Drawing functions

      //-----------------------------------------------------------------------
      // Last version of smooth line drawer function
      void DrawLine(TRGB RGB, int w, double x1, double y1, double x2, double y2);

      //-----------------------------------------------------------------------
      // Antialiased line drawer function for 1 pixel width lines
      void  DrawLine_1(TRGB RGB, double x1, double y1, double x2, double y2);

      //-----------------------------------------------------------------------
      // Draw line with super-sampling smoothing
      void SSDrawLine(TRGB RGB, TDPoint P1, TDPoint P2, int S);

      //-----------------------------------------------------------------------
      // Draw orthogonal lines
      void DrawOrthoLine(TRGB RGB, int w, int x1, int y1, int x2, int y2);

      //-----------------------------------------------------------------------
      // Smoothed segment drawer funtion
      void DrawSmootLine(TRGB RGB, TDPoint P1, TDPoint P2, TSegChain **SBuffer);

      //-----------------------------------------------------------------------
      // Smoothed polydon drawer function
      // Outline and fill color are same!
      // Important:
      //    First and end point will joint
      //    Points in P array, have to in counter clockwise order (ccw)!
      void DrawPolygon_1(TRGB RGB, TDPoint *P, int Ct);

      //-----------------------------------------------------------------------
      // Antialiased cirle drawer function
      void DrawCircle(TRGB RGB, int w, double cx, double cy, double r);

      //-----------------------------------------------------------------------
      void DrawFillCircle(TRGB RGB, double cx, double cy, double r);

      //-----------------------------------------------------------------------
      void DrawDonut(TRGB RGB, double cx, double cy, double R1, double R2);

      //-----------------------------------------------------------------------
      void DrawGradientCircle(TRGB rgb_A, TRGB rgb_B, double cx, double cy, double r, double Alpha);

      void FillRow(TRGB RGB);

      void FillSegment(int Y, int X1, int X2);
      void FillGradient(int Y, int X0, int X1, int X2);


      //---------------------------------------------------------------------
      // class level (static) functions
      //

      //-----------------------------------------------------------------------
      static bool CheckRawAccess(wxBitmap Bitmap);

      //-----------------------------------------------------------------------
      // Copy BMP_B to BMP_A
      static bool CopyBitmap(wxBitmap **BMP_A, wxBitmap *BMP_B);

      //-----------------------------------------------------------------------
      // Create a colour gradient in the GT array
      void CreateGradient(TRGB rgb_A, TRGB rgb_B, int n);

    private:
      wxBitmap  *BMP;     // Own Bitmap
      int       W, H;     // Horizontal and vertical dimensions
      BYTE      **SL;     // ScanLines for quick access. Remember the BYTE order: 0)Blue 1)Green 2)Red (Not RGB!!!)
      BYTE      *ROW;     // Filler row fill with one colour and copy with memcpy, size is always (3 * 100 BYTE)
      TRGB      *GT;      // Gradient Row
      int       CtG;      // Number of colors of gradient row

      bool RAW_ACCESS;    // Possibble access to raw data?
  };//class TMGraphics

//-----------------------------------------------------------------------
#endif

