#include "MTConfigChecker.h"
#include "mtconfigdata.h"
#include "DirHelper.h"
#include "vemsdialog/UserMenu.h"
#include "vemsdialog/UserDialog.h"
#include "logger.h"


//SuperCurveDescValidator::SuperCurveDescValidator(   const SuperCurveDescriptorH* superCurveDescs,
//                                                    const VemsCurveDialogDescH* curveDescs,
//                                                    UserMenuList* userMenu )
//{
//    m_superCurveDescs   = superCurveDescs;
//    m_curveDescs        = curveDescs;
//    m_userMenu          = userMenu;
//}
//
//bool SuperCurveDescValidator::validate()
//{
//    SuperCurveDescriptorH invalidSuperDescs;
//
//    SuperCurveDescriptorH::const_iterator it = m_superCurveDescs->begin();
//
//    for ( ; it != m_superCurveDescs->end(); ++it ) {
//        SuperCurveDescriptor* superDesc = it->second;
//
//        if ( superDesc->records.IsEmpty() ) {
//            invalidSuperDescs[it->first] = superDesc;
//        }
//    }
//
//    return false;
//}



MTConfigChecker::MTConfigChecker( MTConfigData* config )
{
    m_config            = config;
    wxString version    = m_config->getFileVersion();
    m_logFileName       = DirHelper::GetPath( dDEBUGLOG ) + _T("configVerify-") + version + _T(".log");
    Logger::logf( LOG_MSG, _T("MTConfigChecker::MTConfigChecker() - m_logFileName: ") +  m_logFileName );
}


void MTConfigChecker::check()
{
    openLogFile();

//    checkUserDialogs();
    checkTableEditors();
    checkUserMenuList();
//    checkSymbols();

    closeLogFile();
}


void MTConfigChecker::openLogFile()
{
    if ( !wxFile::Exists( m_logFileName ) )
    {
        m_logFile.Create( m_logFileName );
    }

    m_logFile.Open( m_logFileName );
    m_logFile.Clear();
}


void MTConfigChecker::closeLogFile()
{
    if ( !m_logFile.IsOpened() ) return;
    m_logFile.Write();
    m_logFile.Close();
}


void MTConfigChecker::doLog( const wxString& msg )
{
    if ( !m_logFile.IsOpened() ) return;
    m_logFile.AddLine( msg );
}


void MTConfigChecker::checkUserDialogs()
{
    UserDialogList* udl = m_config->getUserDialogList();
    StringToUserDialogHash& dialogs = udl->getUserDialogs();
    StringToUserDialogHash::const_iterator it = dialogs.begin();
    StringToUserDialogHash wrongDialogs;

    for ( ; it != dialogs.end(); ++it )
    {
        if ( checkUserDialog( it->second ) ) continue;
        wrongDialogs[ it->first ] = it->second;
    }

    for ( it = wrongDialogs.begin(); it != wrongDialogs.end(); ++it )
    {
        dialogs.erase( it->first );
    }

    WX_CLEAR_HASH_MAP( StringToUserDialogHash, wrongDialogs );
}


bool MTConfigChecker::checkUserDialog( UserDialog* desc )
{
    UserEditFieldArray& fields = desc->getFields();
    UserEditFieldArray goodFields;
    bool isFirstMissing = true;

    for ( size_t i = 0; i < fields.GetCount(); ++i )
    {
        if ( !checkUserEditField( fields[i] ) )
        {
            if ( isFirstMissing )
            {
                doLog( _T("Check UserDialog - ") + desc->getName() );
                isFirstMissing = false;
            }

            doLog( _T("\tUserEditField - ") + fields[i].getLabel() + _T(" - symbol not found! - ") + fields[i].getSymbol() );

            continue;
        }

        goodFields.Add( fields[i] );
    }

    fields.Clear();
    WX_APPEND_ARRAY( fields, goodFields );

    return true;
}


bool MTConfigChecker::checkUserEditField( UserEditField& desc )
{
    SymbolMgr& symbolManager = m_config->getSymbolMgr();
    SymbolTable& symbols = symbolManager.getSymbolTable();

    if ( desc.isLabel() ) return true;

    return symbols.find( desc.getSymbol() ) != symbols.end();
}


void MTConfigChecker::checkTableEditors()
{
    VemsTableDescH* tables = m_config->getTables();

    VemsTableDescH::const_iterator it = tables->begin();

    VemsTableDescH wrongTables;

    //Logger::logf( LOG_MSG, _T("MTConfigChecker::checkTableEditors() - tables.size() - %d"), tables->size() );

    for ( ;it != tables->end(); ++it )
    {
        if ( checkTableEditor( it->second ) ) continue;
        LOGP( _T("Wrong table: ") + it->first );
        wrongTables[ it->first ] = it->second;
    }

    for ( it = wrongTables.begin(); it != wrongTables.end(); ++it )
    {
        Logger::logf( LOG_MSG, _T("MTConfigChecker::checkTableEditors() - earse table ") );
        tables->erase( it->first );
    }

    WX_CLEAR_HASH_MAP( VemsTableDescH, wrongTables );

    //Logger::logf( LOG_MSG, _T("MTConfigChecker::checkTableEditors() - tables.size() - %d"), tables->size() );
}


bool MTConfigChecker::checkTableEditor( const VemsTableDesc* desc )
{
    SymbolMgr& symbolManager = m_config->getSymbolMgr();
    SymbolTable& symbols = symbolManager.getSymbolTable();

    bool error      = false;
    bool warning    = false;
    wxString msg    = _T("TableEditor - ") + desc->name + _T("\n");

    if ( symbols.find( desc->xSTSymbol ) == symbols.end() )
    {
        error = true;
        msg += _T("\terror - xSTSymbol: ") + desc->xSTSymbol + _T(" not found!\n");
    }

    if ( symbols.find( desc->ySTSymbol ) == symbols.end() )
    {
        error = true;
        msg += _T("\terror - ySTSymbol: ") + desc->ySTSymbol + _T(" not found!\n");
    }

    if ( symbols.find( desc->zSTSymbol ) == symbols.end() )
    {
        error = true;
        msg += _T("\terror - zSTSymbol: ") + desc->zSTSymbol + _T(" not found!\n");
    }

    if ( symbols.find( desc->xRTSymbol ) == symbols.end() )
    {
        warning = true;
        msg += _T("\twarning - xRTSymbol: ") + desc->xRTSymbol + _T(" not found!\n");
    }

    if ( symbols.find( desc->yRTSymbol ) == symbols.end() )
    {
        warning = true;
        msg += _T("\twarning - yRTSymbol: ") + desc->yRTSymbol + _T(" not found!\n");
    }

    if ( !desc->zRTSymbol.IsEmpty() && symbols.find( desc->zRTSymbol ) == symbols.end() )
    {
        warning = true;
        msg += _T("\twarning - zRTSymbol: ") + desc->zRTSymbol + _T(" not found!\n");
    }

    if ( error || warning )
    {
        //Logger::logf( LOG_MSG, msg );
        doLog( msg );
    }

    return !error;
}


void MTConfigChecker::checkUserMenuList()
{
    UserMenuList* uml = m_config->getUserMenuList();

    for( size_t i = 0; i < uml->getCount(); ++i )
    {
        checkUserMenu( (*uml)[i] );
    }
}


void MTConfigChecker::checkUserMenu( UserMenu* um )
{
    UserMenuItemArray& items = um->getItems();
    UserMenuItemArray goodItems;
    bool isFirstMissing = true;

    for( size_t i = 0; i < items.GetCount(); ++i )
    {
        if ( !checkUserMenuItem( &items[i] ) )
        {
            if ( isFirstMissing )
            {
                doLog( _T("Check UserMenu - ") + um->getLabel() );
                isFirstMissing = false;
            }

            doLog( _T("\t") + items[i].getLabel() + _T("\tDialog not found! - ") + items[i].getDialog() );
            continue;
        }
        goodItems.Add( items[i] );
    }

    items.Clear();
    WX_APPEND_ARRAY( items, goodItems );
}


bool MTConfigChecker::checkUserMenuItem( UserMenuItem* umi )
{
    if( umi->isSeparator() ) return true;

    if ( findTableEditor( umi->getDialog() ) ) return true;

    if ( findUserDialog( umi->getDialog() ) ) return true;

    if ( findCurveEditor( umi->getDialog() ) ) return true;

    if ( findSuperCurveEditor( umi->getDialog() ) ) return true;

    if ( findTableMapper( umi->getDialog() ) ) return true;

    return false;
}


bool MTConfigChecker::findUserDialog( const wxString& name )
{
    UserDialogList* udl = m_config->getUserDialogList();
    return udl->find( name );
}


bool MTConfigChecker::findTableEditor( const wxString& name )
{
    const VemsTableDescH* tables = m_config->getTables();
    return tables->find( name ) != tables->end();
}


bool MTConfigChecker::findCurveEditor( const wxString& name )
{
    const VemsCurveDialogDescH& curves = m_config->getCurveDialogs();
    return curves.find( name ) != curves.end();
}

///TODO need more serious test
bool MTConfigChecker::findSuperCurveEditor( const wxString& name )
{
    const SuperCurveDescriptorH&  superCurves = m_config->getSuperCurveDialogs();
    return superCurves.find( name ) != superCurves.end();
}

bool MTConfigChecker::findTableMapper( const wxString& name )
{
    const TblMapDescHash& mappers = m_config->m_tableMappers;
    return mappers.find( name) != mappers.end();
}


void MTConfigChecker::checkSymbols()
{
    SymbolMgr& symbolManager = m_config->getSymbolMgr();
    SymbolTable& symbols = symbolManager.getSymbolTable();
    SymbolTable::const_iterator it = symbols.begin();

    for ( ; it != symbols.end(); ++it )
    {
        Symbol* currentSymbol = it->second;

        if ( currentSymbol )
        {
            if ( !currentSymbol->isConst() ) continue;
        }

        if ( !findSymbolInDialogs( it->first ) ) { doLog( _T("Symbol not found in dialogs! ") + it->first ); }
    }
}


bool MTConfigChecker::findSymbolInDialogs( const wxString& name )
{
    UserDialogList* udl = m_config->getUserDialogList();
    StringToUserDialogHash& dialogs = udl->getUserDialogs();
    StringToUserDialogHash::const_iterator it = dialogs.begin();

    for ( ; it != dialogs.end(); ++it )
    {
        if ( findSymbolInDialog( name, it->second ) ) return true;
    }

    return false;
}


bool MTConfigChecker::findSymbolInDialog( const wxString& name, UserDialog* desc )
{
    UserEditFieldArray& fields = desc->getFields();

    for ( size_t i = 0; i < fields.GetCount(); ++i )
    {
        if ( fields[i].getSymbol().IsSameAs( name ) ) return true;
    }

    return false;
}
