#ifndef WX_PRECOMP
    #include "wx/app.h"
    #include "wx/frame.h"
    #include "wx/menu.h"
    #include "wx/msgdlg.h"
    #include "wx/icon.h"
    #include "wx/textctrl.h"
    #include "wx/filedlg.h"
#endif

#include "wx/numdlg.h"
#include "wx/textdlg.h"
#include <wx/xrc/xmlres.h>
#include <stdlib.h>
#include "wx/tooltip.h"

#include "MainPanel.h"
#include "mainframe.h"

#include "logger.h"
#include "DirHelper.h"
// #include "firmware.h" //Do not use it!
// (in case of firmware management we need to stop the thread (which we use in Megatune mode)
// (see the FirmwareMgrDialog) via vemsController, and dont forget start it again!!)
#include "UnicodeHelper.h"

#include "FirmwareMgrDialog.h"
#include "DebugTabObserver.h"
#include "confighelper.h"

//----------------------------------------------------------------------
// MainPanel
//----------------------------------------------------------------------

#ifdef WXPLOT
#include "wx/plot/plot.h"

#include "wx/image.h"
#include "wx/listctrl.h"
#include "wx/sizer.h"
#include "wx/log.h"
#include "wx/intl.h"
#include "wx/math.h"

uint16_t *datax;
long lSize;

class MyPlotCurve;

// MyPlotCurve

class MyPlotCurve: public wxPlotCurve
{
public:
    MyPlotCurve( int offsetY, double startY, double endY ) :
        wxPlotCurve( offsetY, startY, endY ) {}

    virtual wxInt32 GetStartX()
        { return 0; }
    virtual wxInt32 GetEndX()
        { return lSize; }

    virtual double GetY( wxInt32 x )
        {
            double dx = (double)datax[x];
            return dx;
        }
};

#endif

const int  ID_BOOK      = wxNewId();
/*
int ID_CONFIG_BUTTON = wxNewId();
int ID_TESTANDREAD_BUTTON = wxNewId();
int ID_VERIFY_BUTTON = wxNewId();
int ID_UPLOAD_BUTTON = wxNewId();
*/
int ID_READ_CONFIG_BUTTON = wxNewId();


int ID_DOWNLOAD_CONFIG = wxNewId();
int ID_GENERATE_CONFIG = wxNewId();
int ID_UPLOAD_FIRMWARE = wxNewId();
int ID_UPLOAD_CONFIG = wxNewId();

int ID_OPEN = wxNewId();
int ID_INVERT = wxNewId();
int ID_START = wxNewId();
int ID_STOP = wxNewId();

BEGIN_EVENT_TABLE(MainPanel, wxPanel)
   // EVT_BUTTON    (ID_TESTANDREAD_BUTTON,      MainFrame::OnDownloadConfig) //config read
    //EVT_BUTTON    (ID_CONFIG_BUTTON,      MainFrame::On)    //config test
     EVT_BUTTON    (ID_DOWNLOAD_CONFIG,    MainPanel::OnDownloadConfig)
     EVT_BUTTON    (ID_UPLOAD_FIRMWARE,    MainPanel::OnUploadFirmware) //firmware upload
     EVT_BUTTON    (ID_UPLOAD_CONFIG,      MainPanel::OnUploadConfig) //config upload
#ifdef WXPLOT
    EVT_SIZE    (MainPanel::OnResize)
    EVT_BUTTON    (ID_OPEN, MainPanel::OnOpen)
    EVT_BUTTON    (ID_START, MainPanel::OnStart)
    EVT_BUTTON    (ID_STOP, MainPanel::OnStop)
    EVT_TOGGLEBUTTON    (ID_INVERT, MainPanel::OnInvert)
#endif
END_EVENT_TABLE()

// ============================================================================
// implementation
// ============================================================================

MainPanel::MainPanel( wxFrame *frame, int x, int y, int w, int h,
    VemsTuneController* vemsController)
       //: wxPanel( frame, wxID_ANY, wxPoint(x, y), wxSize(w, h) )
       : wxPanel( frame, wxID_ANY, wxPoint(x, y), wxDefaultSize )
{

#define    Image_Combo -1
#define    Image_Gauge -1
#define    Image_Max -1

    m_vemsController = vemsController;
    bmp_button = NULL;
    bmp_button2 = NULL;

    m_book = new wxBookCtrl(this, ID_BOOK);
/// 1. TAB
    wxPanel *panel = new wxPanel(m_book);
    wxBoxSizer *combo_sizer = new wxBoxSizer( wxHORIZONTAL );
    panel->SetSizer( combo_sizer );
    wxStaticBoxSizer *param_sizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Firmware") );
    combo_sizer->Add(param_sizer);
/*    wxStaticText *m_combo_text1 = new wxStaticText( panel, wxID_ANY,
                            _T("Trigger type"),
                            wxDefaultPosition,
                            wxSize(150, wxDefaultCoord)
                          );*/
    //param_sizer->Add(m_combo_text1, 0, wxLEFT, 10);

///Button tests
/*    wxBitmap bitmap( 100, 100 );
    wxMemoryDC dc;
    dc.SelectObject( bitmap );
    dc.SetBackground(*wxGREEN);
    dc.SetPen(*wxRED_PEN);
    dc.Clear();
    dc.DrawEllipse(5, 5, 90, 90);
    dc.DrawText(_T("Upload"), 30, 35);
    dc.DrawText(_T("Firmware"), 30, 47);
    dc.SelectObject( wxNullBitmap );
*/
    wxString downloadConfig = ConfigHelper::getDeviceFile(_T("ICON_downloadConfig"), _T("downloadConfig.png1"));
    wxString generateConfig = ConfigHelper::getDeviceFile(_T("ICON_generateConfig"),  _T("generateConfig.png1"));
    wxString uploadFirmware = ConfigHelper::getDeviceFile(_T("ICON_uploadFirmware"), _T("uploadFirmware.png1"));
    wxString uploadConfig = ConfigHelper::getDeviceFile(_T("ICON_uploadConfig"), _T("uploadConfig.png1"));

    wxBitmap bitmap = wxBitmap(DirHelper::GetPath(dICON) + downloadConfig ,wxBITMAP_TYPE_PNG);
    if (bitmap.Ok())
    {
        bmp_button = new wxBitmapButton(panel, ID_DOWNLOAD_CONFIG, bitmap);
        param_sizer->Add(bmp_button, 0, wxLEFT, 0);
    }
    bitmap = wxBitmap(DirHelper::GetPath(dICON) + generateConfig ,wxBITMAP_TYPE_PNG);
    if (bitmap.Ok())
    {
        bmp_load = new wxBitmapButton(panel, ID_GENERATE_CONFIG, bitmap);
        bmp_load->Disable();
        param_sizer->Add(bmp_load, 0, wxLEFT, 0);
    }
    bitmap = wxBitmap(DirHelper::GetPath(dICON) + uploadFirmware, wxBITMAP_TYPE_PNG);
    if (bitmap.Ok())
    {
        bmp_read = new wxBitmapButton(panel, ID_UPLOAD_FIRMWARE, bitmap);
        bmp_read->Disable();
         param_sizer->Add(bmp_read, 0, wxLEFT, 0);
    }
    bitmap = wxBitmap(DirHelper::GetPath(dICON) + uploadConfig, wxBITMAP_TYPE_PNG);
    if (bitmap.Ok())
    {
        bmp_verify = new wxBitmapButton(panel, ID_UPLOAD_CONFIG, bitmap);
        bmp_verify->Disable();
        param_sizer->Add(bmp_verify, 0, wxLEFT, 0);
    }

///Button tests
    m_book->AddPage(panel, _T("Firmware"), false, Image_Combo);
/// 1. TAB
/// 2. TAB
    panel = new wxPanel(m_book);
    wxBoxSizer *main_sizer = new wxBoxSizer( wxHORIZONTAL );
    panel->SetSizer( main_sizer );
    wxStaticBoxSizer *gauge_sizer = new wxStaticBoxSizer( wxHORIZONTAL, panel, _T("&Config") );
    main_sizer->Add( gauge_sizer, 0, wxALL, 5 );
///Button tests
    wxBitmap bmp1(wxArtProvider::GetBitmap(wxART_INFORMATION)),
             bmp2(wxArtProvider::GetBitmap(wxART_TICK_MARK)),
             bmp3(wxArtProvider::GetBitmap(wxART_ERROR));

    bmp_button2 = new wxBitmapButton(panel, ID_READ_CONFIG_BUTTON, bmp1);
    bmp_button2->SetBitmapSelected(bmp2);
    //bmp_button2->SetBitmapFocus(bmp2);
///Button tests
    gauge_sizer->Add(bmp_button2, 0, wxLEFT, 10);
    m_config = new wxTextCtrl(panel, wxID_ANY, _T(""),
                            wxPoint(0, 0), wxSize(200, 200), wxTE_MULTILINE);
    gauge_sizer->Add(m_config, 0, wxLEFT, 10);

    m_book->AddPage(panel, _T("Config"), false, Image_Gauge);
/// 2. TAB

/// 3. TAB
    panel = new wxPanel(m_book);
    wxBoxSizer *debug_sizer = new wxBoxSizer( wxVERTICAL );
    panel->SetSizer( debug_sizer );
    m_debug_text = new wxTextCtrl(panel, wxID_ANY, _T(""),
                            wxPoint(0, 0), wxSize(100, 150), wxTE_MULTILINE);
    debug_sizer->Add(m_debug_text,1, wxEXPAND);
    m_book->AddPage(panel, _T("Log"), false, Image_Gauge);
/// 3. TAB

#ifdef WXPLOT
    lSize = 7000;
    datax = (uint16_t*)malloc((unsigned)(lSize*sizeof(uint16_t)));
    if (datax != NULL)// break;
    {
        for(long b=0;b<lSize;b++)
        {
            double dx = b;
            dx /= 100;
            datax[b] = (uint16_t) (1000+1000*sin( dx ));
        }
    }

/// 4. TAB
    panel = new wxPanel(m_book);
    wxBoxSizer *all_sizer = new wxBoxSizer( wxVERTICAL );
    wxBoxSizer *but_sizer = new wxBoxSizer( wxHORIZONTAL );
    wxBoxSizer *plot_sizer = new wxBoxSizer( wxVERTICAL );
    all_sizer->Add(but_sizer);
    all_sizer->Add(plot_sizer,1, wxEXPAND);

    //wxGridSizer *plot_sizer = new wxGridSizer( 1 );
    panel->SetSizer( all_sizer );
    wxButton *load = new wxButton( panel, ID_OPEN, _T("&Load File"), wxPoint(0,0), wxSize(140,30) );
    wxButton *start = new wxButton( panel, ID_START, _T("&Start"), wxPoint(0,0), wxSize(140,30) );
    wxButton *stop = new wxButton( panel, ID_STOP, _T("&Stop"), wxPoint(0,0), wxSize(140,30) );
    invert = new wxToggleButton(panel,ID_INVERT,_T("&Invert"),wxPoint(0,0), wxSize(140,30) );
    but_sizer->Add(load);
    but_sizer->Add(start);
    but_sizer->Add(stop);
    but_sizer->Add(invert);

    //plot_sizer->Add(plot_sizery);
    //wxPlotWindow* m_plot = new wxPlotWindow( panel, wxID_ANY, wxPoint(0,0), wxSize(640, 480) , wxSUNKEN_BORDER | wxPLOT_DEFAULT );
    m_plot = new wxPlotWindow( panel, wxID_ANY, wxPoint(0,0), wxSize(640, 480) , wxSUNKEN_BORDER | wxPLOT_BUTTON_ENLARGE|wxPLOT_BUTTON_ZOOM|wxPLOT_X_AXIS|wxPLOT_Y_AXIS );


    m_plot->SetUnitsPerValue( 0.5 );
//    m_plot->SetScrollOnThumbRelease( true );

    //Add a blue, 16pt chart title
    wxString titleText( _T("Trigger Log") );
    wxFont titleFont( *wxNORMAL_FONT );
    titleFont.SetPointSize( 16 );
    wxColour titleColour( *wxBLUE );
    m_plot->AddChartTitle( titleText, titleFont, titleColour );
    m_curve = new MyPlotCurve( 0,  0, 2000 );
    m_plot->Add( m_curve );
/*    wxPlotOnOffCurve *oo = new wxPlotOnOffCurve( 10 );
    oo->Add( 10, 20 );
    oo->Add( 25, 30 );
    oo->Add( 100, 400 );
    oo->Add( 1005, 1030 );
    oo->Add( 1055, 1080 );
    oo->Add( 1200, 2000 );
    m_plot->Add( oo );*/

    plot_sizer->Add(m_plot,1, wxEXPAND);
    m_book->AddPage(panel, _T("Trigger Log"), false, Image_Gauge);
/// 4. TAB
#endif
    wxGridSizer *sizer = new wxGridSizer(1);
    //wxBoxSizer *sizer = new wxBoxSizer( wxVERTICAL );

    // set the sizer for the panel itself
    //sizer = new wxBoxSizer(wxVERTICAL);
    sizer->Add(m_book, wxSizerFlags().Border().Expand());
    SetSizer(sizer);

    //EVENT SETUP
    /*
    Connect(ID_TESTANDREAD_BUTTON, wxEVT_COMMAND_BUTTON_CLICKED,
        wxCommandEventHandler(MainFrame::OnDownloadConfig), NULL, (MainFrame*)frame);
    */
}

MainPanel::~MainPanel()
{
#ifdef WXPLOT
    if (datax != NULL) free (datax);
#endif
}

void MainPanel::OnDownloadConfig(wxCommandEvent& event)
{
/*
    DebugTabObserver debugTabObserver(m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsController, &debugTabObserver);
    switch(firmwareMgrDlg.downloadConfig())
    {
            case -1: break; //cancel & exception
            case 1: bmp_read->Enable(); //ok
                    break;
            default: break;
    }
*/
}

void MainPanel::OnUploadFirmware(wxCommandEvent& event)
{
    DebugTabObserver debugTabObserver(m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsController, &debugTabObserver);
    firmwareMgrDlg.upload();
    bmp_verify->Enable();
}


void MainPanel::OnUploadConfig(wxCommandEvent& event)
{
    /*
    DebugTabObserver debugTabObserver(m_debug_text);
    FirmwareMgrDialog firmwareMgrDlg(this, m_vemsController, &debugTabObserver);
    firmwareMgrDlg.uploadConfig();
    // disable all ?
*/
}

/*
void MainPanel::OnTestAndRead(wxCommandEvent& event)
{
        wxMessageDialog *dialog=NULL;
        Firmware *fw = new Firmware();
        m_config->Clear();
        bmp_load->Disable();
        bmp_read->Disable();
        bmp_verify->Disable();


        Logger::log(LOG_MSG,_T("FETCH CONFIG!!"));
        FirmwareConfig* ecuConfig = NULL;
        ecuConfig = fw->getConfig(TYPE_MCD | TYPE_MCT, NULL);
        if(  ecuConfig == NULL ){
             //Helper::Message(vp->getErrorMessage(SP_CANNOT_OPEN) );
            m_config->AppendText(_T("Communication error"));
            //wxMessageBox( _T("Config Read successful (NOT REALLY)!!"), _T("Developer warning"), wxOK);
            while(ecuConfig==NULL)
            {
                dialog = new wxMessageDialog( NULL, _T("It seems that there is some problems\nwith Vems communication. Check power and cables\nand try again."),
                _T("Retry?"), wxNO_DEFAULT|wxYES_NO);

                switch ( dialog->ShowModal() )
                {
                    case wxID_YES:
                        ecuConfig = fw->getConfig(TYPE_MCD | TYPE_MCT, NULL);
                        break;
                    case wxID_NO:
                        return;
                    default:
                        wxLogError(wxT("Unexpected wxMessageDialog return code!"));
                }
            }
        }
        wxMessageBox( _T("Config Read successful"), _T("Developer warning"), wxOK);
        // print config to m_config textcontrol
        bmp_load->Enable();
}
*/
/*
void MainPanel::OnRead(wxCommandEvent& event)
{
    wxMessageBox( _T("Config prepare successful (NOT REALLY)!!"), _T("Developer warning"), wxOK);
    bmp_read->Enable();
}

void MainPanel::OnVerify(wxCommandEvent& event)
{
    wxMessageBox( _T("Upload firmware successful (NOT REALLY)!!"), _T("Developer warning"), wxOK);
    bmp_verify->Enable();
}
*/

/*
void MainPanel::OnReadConfig(wxCommandEvent& event)
{
        wxBitmap bmp1(wxArtProvider::GetBitmap(wxART_INFORMATION)),
                bmp2(wxArtProvider::GetBitmap(wxART_TICK_MARK)),
                bmp3(wxArtProvider::GetBitmap(wxART_ERROR));
        Firmware *fw = new Firmware();
        m_config->Clear();
        Logger::log(LOG_MSG,_T("FETCH CONFIG!!"));
        FirmwareConfig* ecuConfig = fw->getConfig(TYPE_MCD | TYPE_MCT, NULL);
        if(  ecuConfig == NULL ){
             //Helper::Message(vp->getErrorMessage(SP_CANNOT_OPEN) );
            m_config->AppendText(_T("Communication error"));
            bmp_button2->SetBitmapLabel(bmp3); //error
            return;
        }
        // print config to m_config textcontrol
        bmp_button2->SetBitmapLabel(bmp2);

}
*/
#ifdef WXPLOT
void MainPanel::OnOpen(wxCommandEvent& WXUNUSED(event))
{
    wxFileDialog dialog(this, _T("Please choose the triggerlog file"),
                        wxEmptyString, wxEmptyString, wxT("*.log;*.txt"), wxFD_OPEN);
    if (dialog.ShowModal() == wxID_OK)
    {
        wxString filename(dialog.GetPath());
        file = filename;
        wxString msg;
        uint16_t byte=0;
        uint16_t prev=0;
        FILE *test;

        test = fopen(UnicodeHelper::U2C(filename), "rb");
        if(test == NULL) {
            msg.Printf(_T("cannot open file ") + filename + _T(" for reading, reverting to stdout"));
            Logger::log(LOG_WARNING, msg);
            //wavout = stdout;
        } else {
            msg.Printf(_T("reading from file: ") + filename);
            Logger::log(LOG_MSG, msg);

            fseek (test , 0 , SEEK_END);
            lSize = ftell (test);
            rewind (test);

            // allocate memory to contain the whole file.
            unsigned char *data = (unsigned char*)malloc((unsigned)(lSize*sizeof(unsigned char)));
            if (data == NULL);// break;

            // copy the file into the buffer.
            fread (data,1,lSize,test);
            // Close file
            fclose(test);
            if (datax != NULL) free (datax);
            datax = (uint16_t*)malloc((unsigned)(lSize*sizeof(uint16_t)));
            if (datax != NULL)// break;
            {
                for(long b=0;b<lSize;b++)
                {
                    byte=0;
                    if(b<(lSize-2))
                    {
                        if(!invert->GetValue())
                            byte=((data[b]<<8)+data[b+1]);
                        else
                            byte=((data[b+1]<<8)+data[b+2]);
                    }
                    int16_t diff = byte-prev;
                    //msg.Printf(_T("Byte: %d, prev: %d, diff: %d\n"),byte,prev,diff);
                    //m_config->AppendText(msg);
                    if(diff<0)
                    {
                        diff=diff-65535;
                    }
                    prev=byte;
                    datax[b]=(uint16_t)diff;
                    datax[b+1]=0;
                    b++;
                }
            }
            m_plot->Delete( m_curve );
            m_plot->RedrawEverything();
            m_curve = new MyPlotCurve( 0,  0, 5000 );
            m_plot->Add(m_curve);
            m_plot->RedrawEverything();
            if (data != NULL) free (data);
        }
    }
}

void MainPanel::OnStart(wxCommandEvent& WXUNUSED(event))
{
    wxMessageBox( _T("This function is not implemented yet"), _T("Developer warning"), wxOK);
}
void MainPanel::OnStop(wxCommandEvent& WXUNUSED(event))
{
    wxMessageBox( _T("This function is not implemented yet"), _T("Developer warning"), wxOK);
}

void MainPanel::OnInvert(wxCommandEvent& WXUNUSED(event))
{
    if (file)
    {
        wxString msg;
        wxString filename;
        filename = file;
        uint16_t byte=0;
        uint16_t prev=0;
        FILE *test;

        test = fopen(UnicodeHelper::U2C(filename), "rb");
        if(test == NULL) {
            msg.Printf(_T("cannot open file ") + filename + _T(" for reading, reverting to stdout"));
            Logger::log(LOG_WARNING, msg);
            //wavout = stdout;
        } else {
            msg.Printf(_T("reading from file: ") + filename);
            Logger::log(LOG_MSG, msg);

            fseek (test , 0 , SEEK_END);
            lSize = ftell (test);
            rewind (test);

            // allocate memory to contain the whole file.
            unsigned char *data = (unsigned char*)malloc((unsigned)(lSize*sizeof(unsigned char)));
            if (data == NULL);// break;

            // copy the file into the buffer.
            fread (data,1,lSize,test);
            // Close file
            fclose(test);
            if (datax != NULL) free (datax);
            datax = (uint16_t*)malloc((unsigned)(lSize*sizeof(uint16_t)));
            if (datax != NULL)// break;
            {
                for(long b=0;b<lSize;b++)
                {
                    byte=0;
                    if(b<(lSize-2))
                    {
                        if(!invert->GetValue())
                            byte=((data[b]<<8)+data[b+1]);
                        else
                            byte=((data[b+1]<<8)+data[b+2]);
                    }
                    int16_t diff = byte-prev;
                    //msg.Printf(_T("Byte: %d, prev: %d, diff: %d\n"),byte,prev,diff);
                    //m_config->AppendText(msg);
                    if(diff<0)
                    {
                        diff=diff-65535;
                    }
                    prev=byte;
                    datax[b]=(uint16_t)diff;
                    datax[b+1]=0;
                    b++;
                }
            }
            m_plot->Delete( m_curve );
            m_plot->RedrawEverything();
            m_curve = new MyPlotCurve( 0,  0, 5000 );
            m_plot->Add(m_curve);
            m_plot->RedrawEverything();
            if (data != NULL) free (data);
        }
    }
}

void MainPanel::OnResize(wxSizeEvent& event)
{
    MainPanel::OnSize(event);
    if(m_plot)
        m_plot->RedrawEverything();
}
#endif
