#include "ProgressBar.h"

#include <wx/dcbuffer.h>
#include "logger.h"

BEGIN_EVENT_TABLE(ProgressBar,wxControl)

    EVT_SIZE(ProgressBar::OnSize)
    EVT_PAINT(ProgressBar::OnPaint)

    //EVT_MOUSE_EVENTS(ProgressBar::OnMouse)
    EVT_UPDATE_VIEWER_EVENT( wxID_ANY, ProgressBar::OnUpdateValue )

END_EVENT_TABLE()


ProgressBar::ProgressBar(wxWindow *parent, wxWindowID id, wxPoint pos, wxSize size, long style,const wxValidator &validator, const wxString& name)
            :m_value (0.0)
            ,m_minValue (0.0)
            ,m_maxValue (100.0)
            ,m_deltaValue (0.1)
            ,m_title (_T("Title"))
            ,m_valueFormat (_T("%.1f %%"))
            ,m_progressBarStyle (PB_DEFAULT)
{

    //ctor
    Create(parent,id,pos,size,style,validator, name);

    SetInitialSize(size);
    SetBarForegroundColour(wxColor(0,102,151));
    SetBarBackgroundColour(wxColor(255,255,255));

    m_size = size;
    m_parentBGColor = parent->GetBackgroundColour();

    m_percentFont = wxFont( (int)((m_size.y)*0.4) ,wxSWISS,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_BOLD,0,_T("Arial"),wxFONTENCODING_DEFAULT);

    //SetBackgroundColour(wxColor(128,128,128));

}

ProgressBar::~ProgressBar()
{
    //dtor
}

///********************************************************
///         EVENT HANDLERS


void ProgressBar::OnPaint(wxPaintEvent& event)
{

    //Logger::logf(LOG_MSG,_T("ProgressBar::OnPaint() width=%i,height=%i"),GetSize().x,GetSize().y);
    wxPoint bar[4];

    int barWidth  = (int)(m_size.x * m_value/(m_maxValue-m_minValue));
    int barHeight = m_size.y;

    bar[0] = wxPoint(0,0);
    bar[1] = wxPoint(m_size.x,0);
    bar[2] = wxPoint(m_size.x,m_size.y);
    bar[3] = wxPoint(0,m_size.y);

    wxString text = wxString::Format(m_valueFormat,m_value);

    wxBufferedPaintDC dc(this);
    dc.Clear();

    wxBitmap bmp1 = wxBitmap(m_size.x,m_size.y);
    wxBitmap bmp2 = wxBitmap(m_size.x,m_size.y);


    wxMemoryDC dc1;
    dc1.SelectObject(bmp1);
    dc1.SetBrush(m_barFGColor);
    dc1.SetPen(*wxTRANSPARENT_PEN);
    dc1.SetBackground(m_barBGColor);
    dc1.SetBackgroundMode(wxTRANSPARENT);
    dc1.SetFont(m_percentFont);
    dc1.SetTextForeground(m_barBGColor);
    dc1.Clear();


    wxMemoryDC dc2;
    dc2.SelectObject(bmp2);
    dc2.SetBrush(m_barBGColor);
    dc2.SetPen(*wxTRANSPARENT_PEN);
    dc2.SetBackground(m_barFGColor);
    dc2.SetBackgroundMode(wxTRANSPARENT);
    dc2.SetFont(m_percentFont);
    dc2.SetTextForeground(m_barFGColor);
    dc2.Clear();


    dc1.DrawPolygon(4,bar);
    dc2.DrawPolygon(4,bar);

    if(m_progressBarStyle & PB_PERCENTTEXT_INNER)     {

        wxSize textsize = dc1.GetTextExtent(text);
        dc1.DrawText(text,(m_size.x-textsize.x)/2,(m_size.y-textsize.y)/2);
        dc2.DrawText(text,(m_size.x-textsize.x)/2,(m_size.y-textsize.y)/2);

    }

    dc.Blit(0,0,barWidth,barHeight,&dc1,0,0);
    dc.Blit(barWidth,0,m_size.x-barWidth,barHeight,&dc2,barWidth,0);

}

void ProgressBar::OnSize(wxSizeEvent& event)
{
    wxSize newSize = GetClientSize();//event.GetSize();

    if(newSize.y != m_size.y)
        m_percentFont.SetPointSize( (int)((newSize.y)*0.6) );

    m_size = newSize;
    Refresh(false);
}

void ProgressBar::OnMouse(wxMouseEvent& event)
{


}

///********************************************************
///         SETTERS Functions


void ProgressBar::SetValueFormat(const wxString &format)
{
    m_valueFormat = format;
}

void ProgressBar::SetValueLimits(double minValue, double maxValue, double deltaValue)
{
    m_minValue = minValue;
    m_maxValue = maxValue;
    m_deltaValue = deltaValue;
}

void ProgressBar::SetProgressStyle(ProgressBarStyle style)
{
    m_progressBarStyle = style;
}

void ProgressBar::SetProgressStyleFlag(ProgressBarStyle style)
{

}

void ProgressBar::SetBarBackgroundColour(const wxColour &colour)
{
    m_barBGColor = colour;
}

void ProgressBar::SetBarForegroundColour(const wxColour &colour)
{
    m_barFGColor = colour;
}

///********************************************************
///         WIDGETS Updaters


void ProgressBar::Update(double value, bool isFilterSmallChanges)
{
    if(value < m_minValue || value > m_maxValue) return;
    if( fabs(value-m_value) < m_deltaValue && isFilterSmallChanges && value != m_minValue && value != m_maxValue ) return;

    m_value = value;

    Update();

}

void ProgressBar::Update()
{


    UpdateViewerEvent event;
   // event.m_variant = it->second;
    this->AddPendingEvent( event );
}


///********************************************************
///         OTHERS Functions


void ProgressBar::OnUpdateValue( UpdateViewerEvent& event )
{
    Refresh(false);
}

