#include "Raw2HighConverterDialog.h"

#include "DirHelper.h"
#include <wx/filename.h>

#include "MTConfigHelper.h"
#include "mtconfigdata.h"
#include "logger.h"
#include "RawLogHelper.h"

#include "configmanager/ConfigHelper2.h"
#include "configmanager/ConfigMgrLogConverter.h"

//#include "rtdatamanager/RTDataLogWriter.h"
#include "rtdatamanager/RTDataCSVWriter.h"
#include "rtdatamanager/RTDataMgrFacade.h"
#include "rtdatamanager/RTDataInputSIPR.h"

#include "stdatamanager/STDataMgrFacade.h"
#include "stdatamanager/STDataModelSerial.h"
#include "stdatamanager/STDataViewerHash.h"

#include "vemsprotocol/VemsLogReader.h"

#include "expression/RPNHelper.h"
#include "UserLog.h"

#include "loghistory/LogHistoryMgr.h"
#include "loghistory/LogFilter.h"
#include "loghistory/LogHistoryProcessFilterThread.h"

int idListInput = wxNewId();
int idTextInput = wxNewId();
int idBtnBrwInput = wxNewId();
int idBtnBrwInputDel = wxNewId();

int idEditIni = wxNewId();
int idTextIni = wxNewId();
int idBtnBrwIni = wxNewId();

int idConcatCheck = wxNewId();
int idAutoCloseCB = wxNewId();

int idTextOutput = wxNewId();
int idEditOutputExt = wxNewId();
int idEditOutputPath = wxNewId();
int idBtnBrwOutput = wxNewId();

int idBtnConvert= wxNewId();
int idBtnStop   = wxNewId();
int idBtnPause  = wxNewId();

int idComboOutput = wxNewId();

BEGIN_EVENT_TABLE(Raw2HighConverterDialog, wxFrame)

    EVT_BUTTON(idBtnBrwInput, Raw2HighConverterDialog::OnBrwInput)
    EVT_BUTTON(idBtnBrwInputDel, Raw2HighConverterDialog::OnBrwInputDel)
    EVT_BUTTON(idBtnBrwOutput, Raw2HighConverterDialog::OnBrwOutput)
    EVT_BUTTON(idBtnBrwIni, Raw2HighConverterDialog::OnBrwIni)
    EVT_BUTTON(idBtnConvert, Raw2HighConverterDialog::OnConvert)
    EVT_BUTTON(idBtnPause, Raw2HighConverterDialog::OnPause)
    EVT_BUTTON(idBtnStop, Raw2HighConverterDialog::OnStop)

    EVT_CLOSE( Raw2HighConverterDialog::OnClose)

    EVT_CHECKBOX(idAutoCloseCB, Raw2HighConverterDialog::OnAutoCloseCB)

END_EVENT_TABLE()


Raw2HighConverterDialog::Raw2HighConverterDialog(wxWindow *parent, DoubleHash* viewerModel )
        :m_progBarObWidg (NULL)
        ,m_iniFileName (wxEmptyString)
        ,m_cond1 (0)
        //,m_converter (NULL)
        ,m_mtConfigData (NULL)
{
    //ctor

    int controlWidth = 300;

    Create(parent,wxNewId(),_T("Convert vemslog to csv!"),wxPoint(50,50),wxSize(500,350),wxDEFAULT_FRAME_STYLE |wxSTAY_ON_TOP);

    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    wxSizer* sizerMain = new wxBoxSizer(wxVERTICAL);

    ///-------------- INI sizer level
    m_tEditIni  = new wxTextCtrl(this,idEditIni,_T(""),wxDefaultPosition, wxSize(100,20),0);
    wxStaticText* textIni   = new wxStaticText(this,idTextIni,_T("Ini File"),wxDefaultPosition, wxSize(60,15),wxST_NO_AUTORESIZE);
    m_tEditIni->Enable(false);
    //m_btnBrwIni = new wxButton(this, idBtnBrwIni, _T("..."),wxDefaultPosition, wxSize(20,15), 0  );
    wxSizer* sizerIni  = new wxBoxSizer(wxHORIZONTAL);

    sizerIni->Add(textIni,0,wxALL|wxALIGN_CENTER_HORIZONTAL,3);
    sizerIni->Add(m_tEditIni,1,wxALIGN_CENTER_HORIZONTAL,5);
    //m_sizerIni->Add(m_btnBrwIni,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_RIGHT,3);

    sizerMain->Add(sizerIni,0,wxALL|wxEXPAND,5);

    ///-------------- INPUT sizer level
    m_tListInput  = new wxListBox(this, idListInput,wxDefaultPosition, wxSize(100,50));
    wxStaticText* textInput   = new wxStaticText(this, idTextInput,_T("Input File(s)"),wxDefaultPosition, wxSize(60,15),wxST_NO_AUTORESIZE);
    wxButton* btnBrwInput = new wxButton(this, idBtnBrwInput, _T("..."),wxDefaultPosition, wxSize(20,15), 0  );
    wxButton* btnBrwInputDel = new wxButton(this, idBtnBrwInputDel, _T("clr"),wxDefaultPosition, wxSize(20,15), 0  );
    wxSizer* sizerInput  = new wxBoxSizer(wxHORIZONTAL);

    m_tListInput->Enable(0);

    wxBoxSizer *sizerButtons  = new wxBoxSizer(wxVERTICAL);
    sizerButtons->Add(btnBrwInput,0,wxBOTTOM|wxALIGN_CENTER_HORIZONTAL|wxALIGN_RIGHT,3);
    sizerButtons->Add(btnBrwInputDel,0,wxALIGN_CENTER_HORIZONTAL|wxALIGN_RIGHT,3);

    sizerInput->Add(textInput,0,wxALL|wxALIGN_CENTER_HORIZONTAL,3);
    sizerInput->Add(m_tListInput,1,wxEXPAND);
    sizerInput->Add(sizerButtons,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_RIGHT,3);

    sizerMain->Add(sizerInput,1,wxALL|wxEXPAND,5);

    ///-------------- Checkbox level


    ConfigMgrLogConverter configLoader( ConfigHelper2::getConfig() );
    configLoader.load(&m_csvConfig);

    m_autoCloseCB = new wxCheckBox(this,idAutoCloseCB,_T("Auto close"),wxDefaultPosition, wxDefaultSize, 0);
    m_autoCloseCB->Enable(true);
    m_autoCloseCB->SetValue(m_csvConfig.m_autoClose);

    wxCheckBox* concatCheck = new wxCheckBox(this,idConcatCheck,_T("Concatenate input files"),wxDefaultPosition, wxDefaultSize, 0);
    concatCheck->Enable(false);

    wxBoxSizer *cbSizer = new wxBoxSizer(wxHORIZONTAL);
    cbSizer->Add(m_autoCloseCB,0,wxALL|wxALIGN_LEFT,5);
    cbSizer->Add(concatCheck,0,wxALL|wxALIGN_RIGHT,5);

    sizerMain->Add(cbSizer,0,wxALIGN_RIGHT);

    ///-------------- OUTPUT sizer level
    wxStaticText* textOutput = new wxStaticText(this, idTextOutput,_T("Output extension and path"),wxDefaultPosition, wxSize(130,15)/*,wxST_NO_AUTORESIZE*/);
    m_tEditOutputExt    = new wxTextCtrl(this,idEditOutputExt,_T(""),wxDefaultPosition, wxSize(40,20),wxTE_RIGHT);
    m_tEditOutputPath   = new wxTextCtrl(this,idEditOutputPath,_T(""),wxDefaultPosition, wxSize(100,20),0);
    wxButton* btnBrwOutput      = new wxButton(this, idBtnBrwOutput, _T("..."),wxDefaultPosition, wxSize(20,15), 0  );
    wxSizer* sizerOutput       = new wxBoxSizer(wxHORIZONTAL);

    m_tEditOutputExt->SetValue(m_csvConfig.m_extension);

    sizerOutput->Add(textOutput,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_LEFT,3);
    sizerOutput->Add(m_tEditOutputExt,0,wxLEFT|wxRIGHT|wxALIGN_CENTER_HORIZONTAL,3);
    sizerOutput->Add(m_tEditOutputPath,1,wxLEFT,3);
    sizerOutput->Add(btnBrwOutput,0,wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_RIGHT,3);

    sizerMain->Add(sizerOutput,0,wxALL|wxEXPAND,5);

    ///-------------- Add line

    wxStaticLine* line1 = new wxStaticLine(this,wxID_ANY,wxDefaultPosition, wxSize(controlWidth,2), wxLI_HORIZONTAL );
    sizerMain->Add(line1,0,wxALL|wxEXPAND,5);

    ///-------------- PROGRESS sizer level

    wxSizer* sizerProgress = new wxBoxSizer(wxHORIZONTAL);

    m_progBarObWidg = new ProgressBarObserverWidget(this,wxID_ANY,wxDefaultPosition,wxSize(200,20));
    //m_progBarObWidg->SetValueLimits(0.0, 100.0, 1.0);

    m_progBarObWidg->SetWindowStyle( wxBORDER_SUNKEN );
    m_progBarObWidg->SetProgressStyle(PB_PERCENTTEXT_INNER);
    m_progBarObWidg->Update(0);

    sizerProgress->Add(m_progBarObWidg,1,wxLEFT|wxRIGHT|wxALIGN_CENTER_HORIZONTAL,5);

    sizerMain->Add(sizerProgress,0,wxALL|wxEXPAND,2);

    ///-------------- Add line

    wxStaticLine* line2 = new wxStaticLine(this,wxID_ANY,wxDefaultPosition, wxSize(controlWidth,2), wxLI_HORIZONTAL );
    sizerMain->Add(line2,0,wxALL|wxEXPAND,5);

    ///-------------- CONTROL sizer level


    m_btnConvert    = new wxButton(this,idBtnConvert,_T("&Convert"),wxDefaultPosition, wxDefaultSize, 0 );
    m_btnStop       = new wxButton(this,idBtnStop,_T("&Stop"),wxDefaultPosition, wxDefaultSize, 0 );
    m_btnPause      = new wxButton(this,idBtnPause,_T("&Pause"),wxDefaultPosition, wxDefaultSize, 0 );

    wxSizer* sizerControl  = new wxBoxSizer(wxHORIZONTAL);

    sizerControl->Add(m_btnConvert,0,wxALL,5);
    sizerControl->Add(m_btnStop,0,wxALL,5);
    sizerControl->Add(m_btnPause,0,wxALL,5);

    sizerMain->Add(sizerControl,0,wxALL|wxALIGN_CENTER_HORIZONTAL,5);

    //m_btnConvert->Enable(false);
    m_btnStop->Enable(false);
    m_btnPause->Enable(false);

	SetSizer(sizerMain);
	sizerMain->Fit(this);
	sizerMain->SetSizeHints(this);


	m_fileDlg = new wxFileDialog(this);
	m_dirDialog = new wxDirDialog(this);

    m_iniDefStr = _T("Use default ini file");

    SetIniCtrlStyle(false);

	btnBrwInput->SetFocus();

    m_tEditIni->Connect(wxEVT_SET_FOCUS, wxFocusEventHandler(Raw2HighConverterDialog::OnIniCtrlFocus), NULL, this);
    m_tEditIni->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(Raw2HighConverterDialog::OnIniCtrlKillFocus), NULL, this);

    init();
}

void Raw2HighConverterDialog::init()
{
    m_logInitDir = DirHelper::GetPath(dLOG, true);
    m_mtConfigData = new MTConfigData();

    initSTModule();
    // initRTModule();
    m_vemsLogReader = new VemsLogReader();
    SymbolMgr* symbolMgr = &(m_mtConfigData->getSymbolMgr());
    m_rtDataMgr = new RTDataMgrFacade();
    m_rtInput = new RTDataInputSIPR(symbolMgr, &m_viewerModel);
    m_rtDataMgr->registerInput(m_rtInput );
    m_rtDataMgr->selectInput( INPUT_TYPE_SIPR );

    //m_dataLogWriter = new RTDataLogWriter(m_mtConfigData->getDataLogEntries());
    m_dataLogWriter = new RTDataCSVWriter(m_mtConfigData->getDataLogEntries() );
	m_dataLogWriter->SetDataMgr(m_rtDataMgr);
    //m_rtDataMgr->RegisterViewer(m_dataLogWriter);
    //m_dataLogWriter->SetDataMgr(m_rtDataMgr);
    //m_dataLogWriter->setGPSCapturedRef( m_vemsLogReader->getGPSCapturedRef() );


    m_logHisoryMgr = new  LogHistoryMgr(m_rtDataMgr, m_stController, symbolMgr);

    /// register progressbar
    m_logHisoryMgr->registerObserver(m_progBarObWidg);

    m_logHisoryMgr->registerLogFilter(m_dataLogWriter);

    m_historyConverter = new LogHistoryProcessFilterThread(m_logHisoryMgr);

    /// notify this object if convert finished to start next file
    m_historyConverter->registerObserver(this);
    //ProcessBase m_histConverter = historyConverter;
    //Observable  m_histObs = historyConverter;
}

void Raw2HighConverterDialog::initSTModule()
{
    Logger::log(LOG_MSG,_T("Raw2HighConverterDialog::initSTModule()"));

    m_stController = new STDataMgrFacade();
    SymbolTable& symbolTbl = m_mtConfigData->getSymbolTable();
//    SymbolMgr&  symbolMgr = m_mtConfigData->getSymbolMgr();

    m_staticModel = new STDataModelSerial( symbolTbl, m_mtConfigData->getSIPRDescriptor() );
    m_stController->setModel( m_staticModel );
    m_stController->setDeviceModel( m_staticModel );
    //STDataIOSIPR* siprInput  = new STDataIOSIPR( NULL, &symbolMgr );

    //m_stController->setInput( STDataMgrFacade::SIPR, siprInput );
    //m_stController->selectInput( STDataMgrFacade::SIPR );
}

void Raw2HighConverterDialog::registerStaticViewers()
{
    SymbolMgr symbolMgr = m_mtConfigData->getSymbolMgr();
    RPNHelper::setUsedStatic(symbolMgr);
    SymbolTable& symbolTable = symbolMgr.getUsedStaticAndBit();

    //register scalar viewers to connect RT and ST model

    for( SymbolTable::const_iterator it = symbolTable.begin(); it != symbolTable.end(); ++it )
    {
        Symbol* symbol= it->second;
        STDataViewerHash* viewer = new STDataViewerHash( &m_viewerModel );
        viewer->setSymbol( *symbol );
        //m_stScalarViewerA.Add( viewer );
        m_stController->RegisterViewer( viewer );
    }
}

void Raw2HighConverterDialog::clearStaticViewers()
{
    m_stController->UnregisterAll();
    /*
    for( size_t i = 0; i < m_stScalarViewerA.Count(); ++i){
        STDataViewerHash* viewer = m_stScalarViewerA.Item(i);
        m_stController->UnRegisterViewer( viewer );
        delete viewer;
    }
    */
    //m_stScalarViewerA.clear();
}

Raw2HighConverterDialog::~Raw2HighConverterDialog()
{
    m_historyConverter->stop();
	delete m_historyConverter;

    m_logHisoryMgr->unregisterLogFilter(m_dataLogWriter);
    delete m_logHisoryMgr;

	delete m_dataLogWriter;

	m_rtDataMgr->UnregisterAll();
	delete m_rtDataMgr;

	delete m_rtInput;

	clearStaticViewers();

	delete m_stController;
	delete m_staticModel;

	delete m_mtConfigData;
	delete m_vemsLogReader;

	m_fileDlg = NULL;
	m_dirDialog = NULL;
}

void Raw2HighConverterDialog::OnAutoCloseCB(wxCommandEvent &evt)
{

    bool isChecked = m_autoCloseCB->IsChecked();
    m_csvConfig.m_autoClose = isChecked;

    ConfigMgrLogConverter configMgr( ConfigHelper2::getConfig() );
    configMgr.save(&m_csvConfig);

}

void Raw2HighConverterDialog::OnClose(wxCloseEvent& event)
{

    m_csvConfig.m_autoClose = m_autoCloseCB->IsChecked();
    m_csvConfig.m_extension = m_tEditOutputExt->GetValue();

    ConfigMgrLogConverter configMgr( ConfigHelper2::getConfig() );
    configMgr.save(&m_csvConfig);

    Destroy();

}


void Raw2HighConverterDialog::setInputFile(const wxString& inputFile)
{
    m_tListInput->Append(inputFile);
    m_inputFiles.Add(inputFile);

    wxFileName fileName(inputFile);
    m_tEditOutputPath->SetValue(fileName.GetPath());
}

void Raw2HighConverterDialog::OnIniCtrlFocus(wxFocusEvent& evt)
{
    if( m_tEditIni->GetValue() != m_iniDefStr) return;

    SetIniCtrlStyle(true);

}

void Raw2HighConverterDialog::SetIniCtrlStyle(bool isactive)
{
    if(isactive){

        m_tEditIni->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
        m_tEditIni->SetValue(wxEmptyString);

        wxFont font = m_tEditIni->GetFont();
        font.SetStyle(wxFONTSTYLE_NORMAL);

        m_tEditIni->SetFont(font);

    }
    else{

        m_tEditIni->SetBackgroundColour(wxColor(245,250,169));
        m_tEditIni->SetValue(m_iniDefStr);

        wxFont font = m_tEditIni->GetFont();
        font.SetStyle(wxFONTSTYLE_ITALIC);

        m_tEditIni->SetFont(font);

    }
}

void Raw2HighConverterDialog::OnIniCtrlKillFocus(wxFocusEvent& evt)
{
    if( m_tEditIni->GetValue() == m_iniDefStr || m_tEditIni->GetValue() == wxEmptyString )

    SetIniCtrlStyle(false);

}

void Raw2HighConverterDialog::OnStop(wxCommandEvent& event)
{

}

void Raw2HighConverterDialog::OnPause(wxCommandEvent& event)
{
    /*
    if(m_converter == NULL) return;

    if(m_converter->IsPaused()) {
        m_btnPause->SetLabel(_T("Pause"));
        m_converter->Resume();
    }
    else {
        m_btnPause->SetLabel(_T("Resume"));
        m_converter->Pause();
    }
    */
}

void Raw2HighConverterDialog::OnConvert(wxCommandEvent& event)
{
    m_btnConvert->Enable(false);

    //iniParser();

    bool res = ConvertFirstItem();
    m_btnConvert->Enable(!res);
}


bool Raw2HighConverterDialog::ConvertFirstItem()
{
    if(m_inputFiles.GetCount() < 1) return false;



    //m_btnStop->Enable(true);
    m_btnPause->Enable(true);


    wxFileName tmp = wxFileName(m_inputFiles.Item(0));

    wxString oname = tmp.GetName();

    wxString fnameend = _T("");

    if(m_tEditOutputExt->GetValue() != wxEmptyString)
        fnameend = _T(".") + m_tEditOutputExt->GetValue();


    return DoConvert(m_inputFiles.Item(0),m_tEditOutputPath->GetValue() + DirHelper::Separ() + oname + fnameend);


}


void Raw2HighConverterDialog::OnBrwIni(wxCommandEvent& event)
{
    m_fileDlg->SetWildcard(_T("Ini files (*.ini)|*.ini|Any files (*.*)|*.*"));
    m_fileDlg->SetMessage(_T("Choose an Ini file"));
    m_fileDlg->SetWindowStyle(wxFD_OPEN|wxFD_FILE_MUST_EXIST);

    if(m_fileDlg->ShowModal() != wxID_OK) return;

   // m_iniFileName = m_fileDlg->GetDirectory() + DirHelper::Separ() + m_fileDlg->GetFilename();

    wxString iniFile =  m_fileDlg->GetDirectory() + DirHelper::Separ() + m_fileDlg->GetFilename();

    SetIniCtrlStyle(true);

    m_tEditIni->SetValue(iniFile);
}


void Raw2HighConverterDialog::OnBrwOutput(wxCommandEvent& event)
{

    m_dirDialog->SetPath(m_tEditOutputPath->GetValue());

    if(m_dirDialog->ShowModal() != wxID_OK) return;

    m_tEditOutputPath->SetValue(m_dirDialog->GetPath());


}

void Raw2HighConverterDialog::OnBrwInput(wxCommandEvent& event)
{
    m_fileDlg->SetPath(m_logInitDir + DirHelper::Separ());

    m_fileDlg->SetWildcard(_T("Log files (*.vemslog)|*.vemslog;|Any files (*.*)|*.*"));
    m_fileDlg->SetMessage(_T("Choose a Vems RawLog input file(s)"));
    m_fileDlg->SetWindowStyle(wxFD_OPEN|wxFD_FILE_MUST_EXIST
    |wxFD_MULTIPLE
    );

    if(m_fileDlg->ShowModal() != wxID_OK) return;

    m_logInitDir = m_fileDlg->GetDirectory();

    wxArrayString files;

    m_fileDlg->GetFilenames(files);

    for(uint32_t a=0;a<files.GetCount();a++)
        m_inputFiles.Add(m_logInitDir + DirHelper::Separ() + files.Item(a));

    m_tListInput->Append(m_inputFiles);

    m_tEditOutputPath->SetValue(m_logInitDir);

}

void Raw2HighConverterDialog::OnBrwInputDel(wxCommandEvent& event)
{
    m_tListInput->Clear();
    m_inputFiles.Clear();
}

bool Raw2HighConverterDialog::DoConvert(const wxString &input, const wxString &output)
{
    m_progBarObWidg->Update(0);
    FirmwareInfo fwInfo;
    if(! ParseHeader(input,fwInfo) ){
        UserLog::uLog(ULOG_ERROR, _T("Convert failed: corrupted header found in: ")+ input);
        clearStaticViewers();
        startNext();
        return true;
    }
    wxString inifile = fwInfo.getIniFile();
    if( inifile == wxEmptyString ) {
        UserLog::uLog(ULOG_ERROR, _T("Vemslog 2 csv converter: ini file is not found  for: ")+ input);
        //wxMessageBox(_T("Ini file error"),_T("Cannot find the current ini file"));
        clearStaticViewers();
        startNext();
        return true;
    }
    UserLog::uLog(ULOG_INFO, _T("Convert: <b>")+ input+_T("</b> to <b>")+ output+ _T("</b>"));

    /// this may take (1-3 sec)
    m_logHisoryMgr->emptyHistory();
    m_logHisoryMgr->loadHistoryFromLog(input);

    m_mtConfigData->clear();
    MTConfigHelper::parseMTConfigData( fwInfo, m_mtConfigData);

	m_rtDataMgr->RegisterViewer(m_dataLogWriter);

    m_logHisoryMgr->initST(*(m_mtConfigData->getSIPRDescriptor()) );
    size_t count = m_mtConfigData->getDataLogEntries()->GetCount();
    Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() dataLogEntries size: %i"), count );

    SetIniCtrlStyle(true);
    m_tEditIni->SetValue( inifile );

    m_dataLogWriter->setFileName(output);
    if( ! m_dataLogWriter->writeHeaders() ){
        return false;
    };

    Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() run") );
    registerStaticViewers();
    //printDoubleHash();
    //m_rtInput->printViewerModel();
    Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() before start") );
    m_historyConverter ->start();

    Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() finish") );
    //printDoubleHash();
    //m_rtInput->printViewerModel();
    //clearStaticViewers();
    return true;

}

/*
bool Raw2HighConverterDialog::DoConvert(const wxString &input, const wxString &output)
{
    if(m_converter != NULL) return false;
    m_progBarObWidg->Update(0);
    FirmwareInfo fwInfo;
    if(! ParseHeader(input,fwInfo) ){
        UserLog::uLog(ULOG_ERROR, _T("Convert failed: corrupted header found in: ")+ input);
        TSNotify(TS_EXIT);
        return true;
    }
    //Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() input:")+input );
    //Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() output:")+output );
    //if(!m_converter->SetOutputFile(output)) return false;
    //RTDataLogWriter* dataLogWriter = new  RTDataLogWriter(m_mtConfigData->getDataLogEntries());

    //m_rtDataMgr->RegisterViewer(dataLogWriter);
    //dataLogWriter->SetDataMgr(m_rtDataMgr);
    //dataLogWriter->setGPSCapturedRef( m_vemsLogReader->getGPSCapturedRef() );

    wxString inifile = fwInfo.getIniFile();
    if( inifile == wxEmptyString ) {
        UserLog::uLog(ULOG_ERROR, _T("Vemslog 2 csv converter: ini file is not found  for: ")+ input);
        //wxMessageBox(_T("Ini file error"),_T("Cannot find the current ini file"));
        TSNotify(TS_EXIT);
        return true;
    }
    UserLog::uLog(ULOG_INFO, _T("Convert: <b>")+ input+_T("</b> to <b>")+ output+ _T("</b>"));

    MTConfigHelper::parseMTConfigData( fwInfo, m_mtConfigData);
    size_t count = m_mtConfigData->getDataLogEntries()->GetCount();
    Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() dataLogEntries size: %i"), count );

    ///m_converter->SetIniFile( fwInfo );

    SetIniCtrlStyle(true);
    m_tEditIni->SetValue( inifile );

    m_dataLogWriter->setFileName(output);
    if( ! m_dataLogWriter->writeHeaders() ){
        return false;
    };

    Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() run") );
    registerStaticViewers();
    //printDoubleHash();
    //m_rtInput->printViewerModel();
    m_converter = new RawToHighLogConverter(m_rtDataMgr, m_stController, m_vemsLogReader);

    if ( m_converter->Create() != wxTHREAD_NO_ERROR ){
        Logger::logf(LOG_MSG, _T("Can't create converter thread!"));
        return false;
    }
    m_converter->SetThreadStatus(this);
    m_converter->Run();
     Logger::logf(LOG_MSG,_T("Raw2HighConverterDialog::DoConvert() finish") );
    //printDoubleHash();
    //m_rtInput->printViewerModel();
    //clearStaticViewers();
    return true;

}
*/

void Raw2HighConverterDialog::startNext()
{
    Logger::logf(LOG_MSG, _T("Raw2HighConverterDialog::startNext()"));
    m_historyConverter->stop();

    m_dataLogWriter->closeFile();

    m_rtDataMgr->UnregisterAll();

	clearStaticViewers();

//    m_dataLogWriter->flushBuffers();
    Logger::logf(LOG_MSG, _T("m_converter = NULL;\n"));
    //m_converter = NULL;
    //Logger::logf(LOG_MSG, _T("m_inputFiles.RemoveAt(0);\n"));
    m_inputFiles.RemoveAt(0);
    //Logger::logf(LOG_MSG, _T("m_tListInput->Delete(0);\n"));
    m_tListInput->Delete(0);
    if(m_inputFiles.GetCount()){
        ConvertFirstItem();
        Logger::logf(LOG_MSG, _T("TSNotify ConvertFirstItem();\n"));
    }
    else {
        Logger::logf(LOG_MSG, _T("m_btnStop->Enable(false);\n"));
        m_btnStop->Enable(false);
        //Logger::logf(LOG_MSG, _T("m_btnStop->Enable(false);\n"));
        m_btnPause->Enable(false);
        //Logger::logf(LOG_MSG, _T(" m_btnPause->Enable(false);\n"));
        m_btnConvert->Enable(true);
        //Logger::logf(LOG_MSG, _T("  if(m_autoCloseCB->GetValue()) Destroy();\n"));
        if(m_autoCloseCB->GetValue()) Destroy();
    }

}

/// LogHistoryProcessFilterThread will invoke this when finised: observerNotify(_T("LogHistoryProcessFilterThreadFinished"));
void Raw2HighConverterDialog::notify(const wxString& str)
{
    Logger::logf(LOG_MSG, _T("Raw2HighConverterDialog::notify"));
    startNext();
}

/*
void Raw2HighConverterDialog::TSNotify(ThreadState state)
{
    //printDoubleHash();
    //m_rtInput->printViewerModel();
    clearStaticViewers();

    if(state == TS_EXIT) {
        startNext();
    }
}
*/

void Raw2HighConverterDialog::printDoubleHash()
{
    size_t s = m_viewerModel.size();
    Logger::logf(LOG_MSG, _T("Raw2HighConverterDialog::printDoubleHash() size: %d"), s);
    DoubleHash::const_iterator IT =  m_viewerModel.begin();
    for(; IT != m_viewerModel.end();++IT ){
        wxString key = IT->first;
        double value = IT->second;
        Logger::logf(LOG_MSG, _T("Raw2HighConverterDialog::printDoubleHash(): ")+key+_T("=%f"), value);
    }
}

bool Raw2HighConverterDialog::ParseHeader(const wxString& filename, FirmwareInfo &fwInfo )
{
    //m_vemsLogReader = new RTDataInputRawLog( &(m_mtConfigData->getSymbolMgr()), m_viewerModel );

    if( m_vemsLogReader->SetInputFile( filename ) < 0 ) {
        //delete m_vemsLogReader;
        return false;
    }

    if( m_vemsLogReader->ParseHeader() < 0 ) {
        //delete m_vemsLogReader;
        return false;
    }

    //((RTDataMgrFacade*)m_rtDataMgr)->SetInput(m_vemsLogReader);

    const StringHash& headerHash = m_vemsLogReader->getHeaderHash();
    RawLogHelper::fillFirmwareInfo(fwInfo, headerHash);

    return true;
}
