#include "RawLogHelper.h"
#include "common/types/StringHash.h"
#include "FirmwareInfo.h"
#include "version.h"
#include "logger.h"


wxString RawLogHelper::creator = _T("Creator");
wxString RawLogHelper::blockSize = _T("BlockSize");
wxString RawLogHelper::dataLen = _T("DataLength");
wxString RawLogHelper::fwcompTime = _T("CompilationTime");
wxString RawLogHelper::fwVersion = _T("Version");
wxString RawLogHelper::logVersion = _T("LogVersion");
wxString RawLogHelper::fwSN = _T("SerialNumber");
wxString RawLogHelper::gpsDataLen = _T("GPSDataLength");
wxString RawLogHelper::gpsDataFormat = _T("GPSDataFormat");
wxString RawLogHelper::signature = _T("Signature");
wxString RawLogHelper::options = _T("Options");
wxString RawLogHelper::compileOptions = _T("CompileOptions");

RawLogHelper::RawLogHelper()
{
    //ctor
}

RawLogHelper::~RawLogHelper()
{
    //dtor
}

void RawLogHelper::fillFirmwareInfo(FirmwareInfo& fwInfo, const StringHash& headerH)
{
    fwInfo.setFirmwareVersion( getValue(headerH, fwVersion) );
    fwInfo.setSignature( getValue(headerH, signature) );
    fwInfo.setSerialNumber( getValue(headerH, fwSN) );
    fwInfo.setCompilationTime( getValue(headerH, fwcompTime) );
    fwInfo.setFirmwareOptions( getValue(headerH, options) );
    fwInfo.setFirmwareCompileOptions(getValue(headerH, compileOptions) );
}

/** @brief getAsFileString
  *
  * @todo: document this function
  */
wxString RawLogHelper::getVemsCfgHeader(const FirmwareInfo& fwInfo)
{
    wxString line;
    wxDateTime now = wxDateTime::Now();
    wxString currDateTime = now.FormatISODate() +_T(" ")+ now.FormatISOTime();
    line   = _T("# CurrentTime: ")+ currDateTime + _T("\n");

    line += _T("# ") + RawLogHelper::creator + _T(": ")+  AutoVersion::getFullVersion() + _T("\n");
    line += _T("# ") + RawLogHelper::fwVersion + _T(": ")+ fwInfo.getFirmwareVersionAsString() + _T("\n");
    line += _T("# ") + RawLogHelper::fwSN + _T(": ") + fwInfo.getSerialNumber() + _T("\n");
    line += _T("# ") + RawLogHelper::fwcompTime + _T(": ")+ fwInfo.getCompilationTime() + _T("\n");
    line += _T("# ") + RawLogHelper::signature + _T(": ")+ fwInfo.getSignature() + _T("\n");

    wxString compileOptions = fwInfo.getCompileOptions();
    if( compileOptions != wxEmptyString){
        //line += _T("# ") + RawLogHelper::options + _T(": ")+ fwInfo.getOptions() + _T("\n");
        line += _T("# ") + RawLogHelper::compileOptions + _T(": ")+ compileOptions + _T("\n");
    }
    Logger::logf( LOG_MSG, _T("RawLogHelper::getAsFileString(): header size: %i"), line.size());
    return line;
}

/*

wxString fileName = DirHelper::CreateNewFileName(baseFileName,logdir,true,true);
    Logger::logf(LOG_MSG,_T("VemsTuneController::fillLogHeader() fileName: ")+ fileName);

    m_siprFileLogger->setFile( fileName);
    m_siprFileLogger->setHeaderItem(RawLogHelper::creator,AutoVersion::getFullVersion() );
    m_siprFileLogger->setHeaderItem(RawLogHelper::logVersion,_T("2"));
    m_siprFileLogger->setHeaderItem(RawLogHelper::blockSize,_T("512"));
    m_siprFileLogger->setHeaderItem(RawLogHelper::fwVersion,m_fwInfo.getFirmwareVersionAsString());
    m_siprFileLogger->setHeaderItem(RawLogHelper::fwcompTime,m_fwInfo.getCompilationTime());
    m_siprFileLogger->setHeaderItem(RawLogHelper::fwSN,m_fwInfo.getSerialNumber());
    m_siprFileLogger->setHeaderItem(RawLogHelper::signature,m_fwInfo.getSignature());
    m_siprFileLogger->setHeaderItem(RawLogHelper::options,m_fwInfo.getOptions());
    wxString compiledOptions = m_fwInfo.getCompileOptions();
    if(compiledOptions != wxEmptyString){
        m_siprFileLogger->setHeaderItem(RawLogHelper::compileOptions,compiledOptions);
    }
*/
wxString RawLogHelper::getVemslogHeader(const FirmwareInfo& fwInfo, int logVersion,bool isTriggerLog)
{
    wxString line;
    if ( isTriggerLog )
        line = _T("# Vems Trigger Log Header, 512 byte\n");
    else
    {
        line = _T("# Vems Log Header, 512 byte\n");
    }

    line += RawLogHelper::blockSize +_T("=512\n");
    line += RawLogHelper::logVersion + wxString::Format( _T("=%i\n"), logVersion);
    wxDateTime now = wxDateTime::Now();
    wxString currDateTime = now.FormatISODate() +_T(" ")+ now.FormatISOTime();
    line += _T("CurrentTime: ")+ currDateTime + _T("\n");
    line += RawLogHelper::creator + _T("=")+  AutoVersion::getFullVersion() + _T("\n");
    line += RawLogHelper::fwVersion + _T("=")+ fwInfo.getFirmwareVersionAsString() + _T("\n");
    line += RawLogHelper::fwSN + _T("=") + fwInfo.getSerialNumber() + _T("\n");
    line += RawLogHelper::fwcompTime + _T("=")+ fwInfo.getCompilationTime() + _T("\n");
    line += RawLogHelper::signature + _T("=")+ fwInfo.getSignature() + _T("\n");

    wxString compileOptions = fwInfo.getCompileOptions();
    if( compileOptions != wxEmptyString){
        //line += _T("# ") + RawLogHelper::options + _T(": ")+ fwInfo.getOptions() + _T("\n");
        line += RawLogHelper::compileOptions + _T("=")+ compileOptions + _T("\n");
    }
    Logger::logf( LOG_MSG, _T("RawLogHelper::getAsFileString(): header size: %i"), line.size());
    return line;
}

wxString RawLogHelper::getValue(const StringHash& headerH, const wxString& key)
{
    StringHash::const_iterator IT = headerH.find(key);
    if(IT == headerH.end() ) return wxEmptyString;
    return IT->second;
}

