#include "SerialPortHelper.h"


#if defined(__WXMSW__)
#include <wx/msw/registry.h>
#include <wx/regex.h>
#endif


#include "logger.h"

wxArrayString SerialPortHelper::GetCOMPortList()
{
    wxArrayString ports;

#if defined(__WXMSW__)

    /// USE WinAPI, all other methods seems to FAIL

    BOOL bSuccess = FALSE;

    HKEY hSERIALCOMM;
    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("HARDWARE\\DEVICEMAP\\SERIALCOMM"), 0, KEY_QUERY_VALUE, &hSERIALCOMM) == ERROR_SUCCESS)
    {
		//Get the max value name and max value lengths
		DWORD dwMaxValueNameLen;
		DWORD dwMaxValueLen;
		DWORD dwQueryInfo = RegQueryInfoKey(hSERIALCOMM, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &dwMaxValueNameLen, &dwMaxValueLen, NULL, NULL);
		if (dwQueryInfo == ERROR_SUCCESS)
		{
			DWORD dwMaxValueNameSizeInChars = dwMaxValueNameLen + 1;
			DWORD dwMaxValueNameSizeInBytes = dwMaxValueNameSizeInChars * sizeof(TCHAR);
			DWORD dwMaxValueDataSizeInChars = dwMaxValueLen/sizeof(TCHAR) + 1;
			DWORD dwMaxValueDataSizeInBytes = dwMaxValueDataSizeInChars * sizeof(TCHAR);

            TCHAR* szValueName = new TCHAR[dwMaxValueNameSizeInChars];
            BYTE* byValue = new BYTE[dwMaxValueDataSizeInBytes];

            bSuccess = TRUE;

            //Enumerate all the values underneath HKEY_LOCAL_MACHINE\HARDWARE\DEVICEMAP\SERIALCOMM
            DWORD dwIndex = 0;
            DWORD dwType;
            DWORD dwValueNameSize = dwMaxValueNameSizeInChars;
            DWORD dwDataSize = dwMaxValueDataSizeInBytes;
            memset(szValueName, 0, dwMaxValueNameSizeInBytes);
            memset(byValue, 0, dwMaxValueDataSizeInBytes);
            LONG nEnum = RegEnumValue(hSERIALCOMM, dwIndex, szValueName, &dwValueNameSize, NULL, &dwType, byValue, &dwDataSize);
            while (nEnum == ERROR_SUCCESS)
            {
                //If the value is of the correct type, then add it to the array
                if (dwType == REG_SZ)
                {
                    TCHAR* szPort = reinterpret_cast<TCHAR*>(byValue);
                    ports.Add(szPort);
				}

                //Prepare for the next time around
                dwValueNameSize = dwMaxValueNameSizeInChars;
                dwDataSize = dwMaxValueDataSizeInBytes;
                memset(szValueName, 0, dwMaxValueNameSizeInBytes);
                memset(byValue, 0, dwMaxValueDataSizeInBytes);
                ++dwIndex;
                nEnum = RegEnumValue(hSERIALCOMM, dwIndex, szValueName, &dwValueNameSize, NULL, &dwType, byValue, &dwDataSize);
            }

            delete[] szValueName;
            delete[] byValue;
        }

		//Close the registry key now that we are finished with it
        RegCloseKey(hSERIALCOMM);
    }

/*
    for ( int i = 1; i < 257; ++i )
    {
        wxString cport = wxString::Format(_T("\\\\.\\COM%d"),i);

        //Try to open the port
        bool success = false;

        HANDLE hPort = ::CreateFile( cport.c_str(), GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0 );
        if ( hPort == INVALID_HANDLE_VALUE )
        {
            if ( GetLastError() == ERROR_ACCESS_DENIED )    // port is valid, but used by another program
                success = true;
        }
        else
        {
            success = true;
            CloseHandle( hPort ); // close port
        }

        if ( success )
            ports.Add( wxString::Format(_T("COM%d"), i ) );
    }

    DWORD length = 32768*8; // should be enough :)
    LPTSTR buffer = (LPTSTR)malloc(length * sizeof(TCHAR));

    // query all devices (included video, disks etc)
    int retCode = QueryDosDeviceW( NULL, buffer, length );

    // a serial port device looks like COMxy
    wxRegEx r( _T("COM[1-9]+[0-9]*") );

    if ( retCode != 0 )
    {
        int& len = retCode;

        wxChar* data = new wxChar[len];

        // data contains all devices separated by '\0'
        memcpy( data, buffer, len );

        wxString str = _T("");

        for ( int j = 0; j < len; ++j )
        {
            wxChar ch = data[j];

            // tokenize data
            if ( ch != '\0' )
                str += ch;
            else
            {
                // two 0x00-s means end of data
                LOGP(str)
                //if ( str == _T("")) break;

                str = str.Upper();

                if ( str.StartsWith( _T("COM") ) )
                {
                    if ( r.Matches( str ) )
                        ports.Add( str );
                    else
                    {
                        //Logger::logf(LOG_MSG, _T("no match /") +  str +_T("/"));
                    }
                }
                str = _T("");
            }
        }
        delete data;
    }
    free( buffer );*/
/*
    /// --> THIS CODE FAILED ON WINDOWS 7 DUE TO MEMORY PROBLEMS (PROBABLY IT IS A WX BUG)
    wxRegKey *pRegKey = new wxRegKey(_T("HKEY_LOCAL_MACHINE\\HARDWARE\\DEVICEMAP\\SERIALCOMM"));


    wxString name;
    long idx = 0;

    pRegKey->GetFirstValue(name,idx);

    wxString val;
    //if( pRegKey.QueryValue(name.GetData(),val) )
    if( pRegKey->QueryValue(name,val) )
        ports.Add( val );

    while(pRegKey->GetNextValue(name,idx)){
        wxString val = _T("");
        if( pRegKey->QueryValue(name,val) )
        {
            Logger::logf(LOG_MSG, name + _T("") + val.GetData());
            ports.Add( val );
        }
    }
    /// <---
*/
#else

    // something...

#endif

    return ports;
}

IntArray SerialPortHelper::getSerialBaudratesInt()
{
    IntArray baudrates;

	baudrates.Add( 9600 );
	baudrates.Add( 19200 );
	baudrates.Add( 38400 );
	baudrates.Add( 57600 );
	baudrates.Add( 115200 );

    return baudrates;
}

IntArray SerialPortHelper::getGPSSerialBaudRates()
{
    IntArray baudrates;

    baudrates.Add( 1200 );
    baudrates.Add( 2400 );
    baudrates.Add( 4800 );
    baudrates.Add( 9600 );
    baudrates.Add( 19200 );
    baudrates.Add( 38400 );
    baudrates.Add( 57600 );
    baudrates.Add( 115200 );

    return baudrates;
}

wxArrayString SerialPortHelper::getSerialBaudrates()
{
    wxArrayString baudrates;
    IntArray baudratesInt = getSerialBaudratesInt();

    for ( size_t i = 0; i < baudratesInt.GetCount(); ++i ) {
        baudrates.Add( wxString::Format( _T("%d"), baudratesInt[i] ) );
    }

    return baudrates;
}
