#include "SimpleFwDlg.h"

#include <wx/dir.h>
#include <wx/tokenzr.h>

#include "FirmwareTypes.h"

#include "DirHelper.h"
#include "logger.h"
#include "project/ProjectManager.h"

#include "ConfigBase.h"
#include "configmanager/ConfigHelper2.h"
#include "FwWebToolHelper.h"
#include "vemsprotocol/ECUInfo.h"

#define NTCSTR _T("NTC")
#define PTCSTR _T("PTC")

#define DEVSTR_V3 _T("v3")
#define DEVSTR_ROUND _T("round")

wxString2wxVariantHash FwDesc::FwDescriptorHash;


BEGIN_EVENT_TABLE(SimpleFwDlg, wxDialog)

    EVT_COMBOBOX( idDevList, SimpleFwDlg::OnComboBox )
    EVT_COMBOBOX( idVerList, SimpleFwDlg::OnComboBox )
    EVT_COMBOBOX( idTableList, SimpleFwDlg::OnComboBox )

    EVT_BUTTON( idOpenWebToolBtn, SimpleFwDlg::OnButton )

END_EVENT_TABLE()

SimpleFwDlg::SimpleFwDlg(wxWindow * parent,const wxString &title)
{
    Create(parent, wxID_ANY, title);

    FwDesc::Init();
}

SimpleFwDlg::SimpleFwDlg(wxWindow * parent, int type)
{
    //ctor
    if(type == VERIFY){
        Create(parent, wxID_ANY, _T("Firmware verify"));
    }else{
        Create(parent, wxID_ANY, _T("Firmware upload"));
    }

    FwDesc::Init();
}

SimpleFwDlg::~SimpleFwDlg()
{
    //dtor
    //Logger::logf(LOG_MSG,_T("SimpleFwDlg::~SimpleFwDlg()"));

    PatchDesc * cltpd = m_cltPatchMgr->GetPatch();
    PatchDesc * airpd = m_airPatchMgr->GetPatch();

    if( cltpd ) {
        wxFileName fn(cltpd->GetPath());

        wxString name = fn.GetName();
        saveCfg(_T("last_patch_") + m_cltPatchMgr->PatchExt, name);
        //ConfigHelper::saveValue( _T("/dialogcontentlayouts/SimpleFwDlg"), _T("last_patch_") + m_cltPatchMgr->PatchExt, name , CONFIG_TYPE_LAYOUT);
    }

    if( airpd ){
        wxFileName fn(airpd->GetPath());

        wxString name = fn.GetName();
        saveCfg(_T("last_patch_") + m_airPatchMgr->PatchExt, name);
        //ConfigHelper::saveValue( _T("/dialogcontentlayouts/SimpleFwDlg"), _T("last_patch_") + m_airPatchMgr->PatchExt, name , CONFIG_TYPE_LAYOUT);
    }

    FwDesc::cleanup();
}

void SimpleFwDlg::saveCfg(const wxString& key, const wxString& value)
{
        ConfigBase* configBase = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT);
        configBase->setPath(_T("/dialogcontentlayouts/SimpleFwDlg"));
        configBase->write(key, value);
        configBase->flush();
        configBase->save();
}

void SimpleFwDlg::OnFwWebToolClose(wxCloseEvent& evt)
{
    evt.Skip(1);

    wxString vercomboval = m_verCombo->GetStringSelection();

    WX_CLEAR_ARRAY(m_fwList[m_devCombo->GetValue()]);

    ReadFwDir();

    FillVerCombo( m_devCombo->GetValue() );

    if( !m_verCombo->SetStringSelection( vercomboval )) return;

    FillTableCombo( GetFwPath() );
}

void SimpleFwDlg::OnButton(wxCommandEvent& evt)
{
    wxWindow* win = FwWebToolHelper::OpenFwWebTool(this);

    win->Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(SimpleFwDlg::OnFwWebToolClose), NULL, this);
}

void SimpleFwDlg::Init( const FirmwareInfo *fi)
{

    Init();
    if(!fi->isValid()) return;

    int dev = fi->getDevice();

    Logger::logf(LOG_MSG,_T("dev=%i"),dev);

    m_devCombo->Clear();

    switch(dev){
        case ECUInfo::DEVICE_ROUND:
            if(m_fwList.find(DEVSTR_ROUND) != m_fwList.end())
                m_devCombo->Append( DEVSTR_ROUND );
            break;

        case ECUInfo::DEVICE_V3:
            if(m_fwList.find(DEVSTR_V3) != m_fwList.end())
                m_devCombo->Append( DEVSTR_V3 );
            break;
    }

    m_devCombo->SetSelection(0);

    FillVerCombo( m_devCombo->GetValue() );

    if( !m_verCombo->SetStringSelection( fi->getFirmwareVersionAsString() )) return;

    FillTableCombo( GetFwPath() );

}

void SimpleFwDlg::Init(Project * prj)
{

    FirmwareVersion fv1;
    fv1.major = 1;
    fv1.minor = 1;
    fv1.build = 50;

    m_fwVerLimitations[_T("tablevar")] = fv1;
    m_fwVerLimitations[_T("clt")] = fv1;

    fv1.major = 1;
    fv1.minor = 1;
    fv1.build = 55;

    m_fwVerLimitations[_T("air")] = fv1;

    wxArrayString as;

    m_devCombo = new wxComboBox(this, idDevList, _T(""), wxDefaultPosition, wxDefaultSize, as, wxCB_READONLY);
    m_verCombo = new wxComboBox(this, idVerList, _T(""), wxDefaultPosition, wxDefaultSize, as, wxCB_READONLY);
    m_tableCombo = new wxComboBox(this, idTableList, _T(""), wxDefaultPosition, wxDefaultSize, as, wxCB_READONLY);



/*
    wxButton * okbutt = new wxButton(this, wxID_OK,_T("OK"));
    wxButton * cnbutt = new wxButton(this, wxID_CANCEL,_T("Cancel"));
*/

    wxString txt1 = _T("Important:");
    wxString txt2 = _T(" Turn off fuel pump,");
    txt2 += _T(" remove injector connectors,");
    wxString txt3 = _T(" remove ignition coil fuses");
    txt3 += _T(" or connectors!");

    wxString txt4 = _T(" Make sure supply is stable!");
    wxString txt5 = _T(" After firmware upgrade, upload some config.");
    wxString txt6 = _T(" Validate and review config!");

    wxStaticText* importantTxt1 =  new wxStaticText(this, wxID_ANY,txt1);
    importantTxt1->SetForegroundColour(wxColour(255,0,0) );
    wxStaticText* importantTxt2 =  new wxStaticText(this, wxID_ANY,txt2);
    importantTxt2->SetForegroundColour(wxColour(255,0,0) );
    wxStaticText* importantTxt3 =  new wxStaticText(this, wxID_ANY,txt3);
    importantTxt3->SetForegroundColour(wxColour(255,0,0) );

     wxStaticText* importantTxt4 =  new wxStaticText(this, wxID_ANY,txt4);
    importantTxt4->SetForegroundColour(wxColour(255,0,0) );
     wxStaticText* importantTxt5 =  new wxStaticText(this, wxID_ANY,txt5);
    importantTxt5->SetForegroundColour(wxColour(255,0,0) );
     wxStaticText* importantTxt6 =  new wxStaticText(this, wxID_ANY,txt6);
    importantTxt6->SetForegroundColour(wxColour(255,0,0) );



    m_mainSizer = new wxBoxSizer(wxVERTICAL);


    //importantTxt->Center();

    //importantTxt->Set


    wxFlexGridSizer * topsizer = new wxFlexGridSizer(2);


    topsizer->Add( new wxStaticText(this, wxID_ANY, _T("Device:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    topsizer->Add( m_devCombo, 1, wxALL | wxEXPAND, 5);
    topsizer->Add( new wxStaticText(this, wxID_ANY, _T("Version:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    topsizer->Add( m_verCombo, 1, wxALL | wxEXPAND, 5);
    topsizer->Add( new wxStaticText(this, wxID_ANY, _T("Type:")), 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    topsizer->Add( m_tableCombo,1 , wxALL | wxEXPAND, 5);


    m_mainSizer->Add(topsizer, 0, 0 | wxALIGN_CENTER_HORIZONTAL ,0);

    m_mainSizer->Add( new wxButton(this, idOpenWebToolBtn, _T("Firmware Web tool")), 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5 );

    m_firstStatLine = new wxStaticLine(this);

    m_mainSizer->Add( m_firstStatLine, 1, wxEXPAND | wxALL, 5);


    m_patchSizer = new wxGridBagSizer();

    m_mainSizer->Add(m_patchSizer, 0, wxEXPAND ,0);

    m_mainSizer->Add( new wxStaticLine(this), 1, wxEXPAND | wxALL, 5);
    m_mainSizer->Add( importantTxt1 );
    m_mainSizer->Add( importantTxt2 );
    m_mainSizer->Add( importantTxt3 );

    m_mainSizer->Add( importantTxt4 );
    m_mainSizer->Add( importantTxt5 );
    m_mainSizer->Add( importantTxt6 );



    FillDlg(prj);

    InitFwPanel(prj);

    m_mainSizer->Add( CreateButtonSizer( wxOK | wxCANCEL  ), 0, wxALL | wxALIGN_CENTER_HORIZONTAL,5 );

	SetSizer(m_mainSizer);
	m_mainSizer->Fit(this);
	m_mainSizer->SetSizeHints(this);


}

void SimpleFwDlg::ReadFwDir()
{

    wxDir dir( DirHelper::GetPath( dPROJECTS ) );
    wxString tmp;
    bool res = false;

    FwPath *fw;

    for( res = dir.GetFirst( &tmp , wxEmptyString, wxDIR_DIRS ) ; res ; res = dir.GetNext(&tmp) ){

        fw = new FwPath();

        bool parseres = fw->ParseDir( tmp, &m_fwList );

        if( !parseres )
            delete fw;

    }


    for( FwPathHash::iterator it = m_fwList.begin() ; it != m_fwList.end(); it++){
        m_devCombo->Append( it->first );
    }


}

void SimpleFwDlg::FillDlg(Project * prj)
{
    ReadFwDir();
    ReadPatchDir(_T("matfactor"), _T("airXfactor"), _T("air"), m_airList);
    ReadPatchDir(_T("coolantfactor"), _T("cltfactor"), _T("clt"), m_cltList);

    m_airPatchMgr = new PatchManagger();
    m_cltPatchMgr = new PatchManagger();

    m_airPatchMgr->PatchExt = _T("air");
    m_cltPatchMgr->PatchExt = _T("clt");

    int row = m_patchSizer->GetRows();
    m_patchSizer->Add( new wxStaticText(this, wxID_ANY, _T("Temp")), wxGBPosition(row,1),  wxGBSpan(1,1), wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    m_patchSizer->Add( new wxStaticText(this, wxID_ANY, _T("NTC/PTC")), wxGBPosition(row,2),  wxGBSpan(1,1), wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    m_patchSizer->Add( new wxStaticText(this, wxID_ANY, _T("RRef")), wxGBPosition(row,3),  wxGBSpan(1,1), wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    m_patchSizer->Add( new wxStaticText(this, wxID_ANY, _T("ARef")), wxGBPosition(row,4),  wxGBSpan(1,1), wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
    m_patchSizer->SetRows( row+1 );


    row = m_patchSizer->GetRows();
    m_patchSizer->Add( new wxStaticText(this, wxID_ANY, _T("Air patch files")), wxGBPosition(row,0),  wxGBSpan(1,2), wxALIGN_BOTTOM | wxTOP | wxLEFT, 5);
    m_patchSizer->SetRows( row+1 );
    m_airPatchMgr->Init(this, m_patchSizer, &m_airList, prj);

    row = m_patchSizer->GetRows();
    m_patchSizer->Add( new wxStaticText(this, wxID_ANY, _T("Coolant patch files")), wxGBPosition(row,0),  wxGBSpan(1,2), wxALIGN_BOTTOM | wxTOP | wxLEFT, 5);
    m_patchSizer->SetRows( row+1 );
    m_cltPatchMgr->Init(this, m_patchSizer, &m_cltList, prj);




}

void SimpleFwDlg::InitFwPanel(Project * prj)
{
    if(!prj) return;

    wxFileName fn( prj->Path );
    //wxFileName fn( _T("firmware_") + prj->getFirmwareInfo().getFirmwareVersionAsString() );

    wxArrayString dirs = fn.GetDirs();

    FwPath * fw = new FwPath();

    //bool res = fw->ParseDir( dirs[dirs.GetCount()-1] );
    bool res = fw->ParseDir( _T("firmware_") + prj->getFirmwareInfo().getFirmwareVersionAsString() );


    if( !res )  {
        delete fw;
        return;
    }

    m_devCombo->SetStringSelection( fw->GetDeviceStr() );

    FillVerCombo( fw->GetDeviceStr(), false );

    m_verCombo->SetStringSelection( fw->GetTitle() );

    FillTableCombo( GetFwPath( m_devCombo->GetValue(), m_verCombo->GetValue() ) );

    wxArrayString types;
    types.Add(_T("firmware"));
    FirmwareConfigItemArray arr = prj->CreateFirmwareConfigItems(types);

    if( arr.GetCount() < 2 ) return;


    //Logger::logf(LOG_MSG,_T("value=")+arr[1]->value);

    if( fw->GetTable() == -1 )
        m_tableCombo->Show(false);


    else if( arr[1]->value.Find(_T("16x14")) == wxNOT_FOUND && fw->GetTable() >= 0 ) {
        if( prj->getFirmwareInfo().getFirmwareVersion() >= m_fwVerLimitations[_T("tablevar")] )
            m_tableCombo->SetStringSelection(_T("PS2"));
        else
            m_tableCombo->SetStringSelection(_T("12x14"));

    } else if( arr[1]->value.Find(_T("16x14")) != wxNOT_FOUND && fw->GetTable() >= 0 ) {
        if( prj->getFirmwareInfo().getFirmwareVersion() >= m_fwVerLimitations[_T("tablevar")] )
            m_tableCombo->SetStringSelection(_T("non-PS2"));
        else
            m_tableCombo->SetStringSelection(_T("16x14"));

    }




/*
    wxArrayString types;
    types.Add(_T("firmware"));
    FirmwareConfigItemArray arr = prj->CreateFirmwareConfigItems(types);

    if( arr.GetCount() < 2 ) return;

    wxFileName fn( prj->Path + DirHelper::Separ() +  arr[1]->value );

    if( !fn.FileExists() ) return;

    //wxString

    if( fn.GetExt() == _T("hex") ) {


    } else if( fn.GetExt() == _T("chex") ) {



    } else
        return;
*/



}

void SimpleFwDlg::ReadPatchDir(const wxString &dir, const wxString &prefix, const wxString &ext, PatchDescArray &arr)
{


    wxDir dirh( DirHelper::GetPath() + dir );
    wxString tmp;
    bool res = false;

    PatchDesc * patch = NULL;


    for( res = dirh.GetFirst( &tmp , wxEmptyString, wxDIR_FILES ) ; res ; res = dirh.GetNext(&tmp) ){
        if( tmp.Find( prefix ) != 0 ) continue;

        wxFileName fn( DirHelper::GetPath() + dir + DirHelper::Separ() + tmp );

        patch = new PatchDesc();

        bool res = patch->ParseFile( fn, dir );

        if( !res ) {
            delete patch;
            continue;
        }

        arr.Add( patch );

        //Logger::logf(LOG_MSG,datas[0] + datas[1] + datas[2]);

    }

}

int FwPath::Title_Compare_function_Desc(FwPath **A,FwPath **B)
{
    return ((*B)->GetTitle()).CmpNoCase((*A)->GetTitle());
}


void SimpleFwDlg::getFirmwareConfigItemArray(FirmwareConfigItemArray &arr)
{
    wxString devstr = m_devCombo->GetValue();

    FwPathArray fwarr = m_fwList[devstr];

    wxString ctitle = m_verCombo->GetValue();

    FwPath * fw = NULL;

    for(uint8_t a=0; a < fwarr.GetCount(); a++){
        if( fwarr[a]->GetTitle() != ctitle ) continue;

        fw = fwarr[a];

    }

    if(!fw) return;

    wxString basepath = fw->GetPath();

    if( fw->GetPath16() != wxEmptyString ) {

        if( !(m_tableCombo->GetValue().CmpNoCase(_T("16x14"))) || !(m_tableCombo->GetValue().CmpNoCase(_T("non-PS2"))) ){
            basepath = fw->GetPath16();
        }

    }

    Logger::logf(LOG_MSG,_T("basepath=")+basepath);

    arr.Clear();

    FirmwareConfigItem *defitem = new FirmwareConfigItem();

    defitem->type = _T("base");
    defitem->name = _T("Base");
    defitem->value = basepath;
    defitem->parentName = _T("Base");
    defitem->isFile = false;
    defitem->mandatory = true;
    defitem->validate = true;
    defitem->active = true;

    arr.Add( defitem );

    FirmwareConfigItem *item = new FirmwareConfigItem();

    item->type = _T("firmware");
    item->name = _T("Hex file");
    item->value = fw->GetFilename();
    item->parentName = _T("Base");
    item->isFile = true;
    item->mandatory = true;
    item->validate = true;
    item->active = true;
    item->parent = defitem;

    Logger::logf(LOG_MSG,_T("path=")+fw->GetFilename());

    arr.Add( item );

    if( FwDesc::FwDescriptorHash[devstr][_T("ispatch")].GetBool() == false ) return;


    PatchDesc * cltpd = m_cltPatchMgr->GetPatch();
    PatchDesc * airpd = m_airPatchMgr->GetPatch();

    if( cltpd ){

        item = new FirmwareConfigItem();

        item->type = _T("firmware_patch");
        item->name = _T("Coolant file");
        item->value = cltpd->GetPath();
        Logger::logf(LOG_MSG,_T("clt item->value=")+item->value);
        //item->parentName = _T("Base");
        item->isFile = true;
        item->mandatory = true;
        item->validate = true;
        item->active = true;
        //item->parent = defitem;

        arr.Add( item );

    }

    if( airpd ){

        item = new FirmwareConfigItem();

        item->type = _T("firmware_patch");
        item->name = _T("Air file");
        item->value = airpd->GetPath();
        //item->parentName = _T("Base");
        item->isFile = true;
        item->mandatory = true;
        item->validate = true;
        item->active = true;
        //item->parent = defitem;

        arr.Add( item );

    }

}

void SimpleFwDlg::FillVerCombo(const wxString &devstr, bool isFillTable)
{
    m_verCombo->Clear();
    m_tableCombo->Clear();

    m_verCombo->Show( true );

    FwPathArray arr = m_fwList[devstr];

    uint8_t a = 0;

    wxArrayString titles;

    for(; a < arr.GetCount(); a++){


        titles.Add( arr[a]->GetTitle() );
    }

    titles.Sort(true);

    m_verCombo->Append( titles );

    if( a == 1 && arr[0]->GetTitle() == wxEmptyString ) {
        m_verCombo->Show( false );
        m_tableCombo->Show( false );
        return;
    }

    m_verCombo->Select( 0 );

    if(isFillTable && m_fwList[devstr].GetCount() > 0 )
        FillTableCombo( m_fwList[devstr].Item( m_fwList[devstr].GetCount()-1 ) );


    bool ispatch = FwDesc::FwDescriptorHash[devstr][_T("ispatch")].GetBool();

    m_mainSizer->Show( m_patchSizer, ispatch );
    m_mainSizer->Show( m_firstStatLine, ispatch );


    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

}

void SimpleFwDlg::FillTableCombo(FwPath * fw)
{
    m_tableCombo->Clear();

    FirmwareVersion v;
    FirmwareInfo::extractFirmwareVersion( m_verCombo->GetValue(), v );


    int tbl = fw->GetTable();

    Logger::logf(LOG_MSG,_T("SimpleFwDlg::FillTableCombo table=%i"),tbl);

    Logger::logf(LOG_MSG,_T("fwpath = ")+fw->GetPath());

    switch( tbl ){
        case -1:
            m_tableCombo->Show(false);
            break;
        case 1:
            if( v >= m_fwVerLimitations[_T("tablevar")] )
                m_tableCombo->Append(_T("non-PS2"));
            else
                m_tableCombo->Append(_T("16X14"));

        case 0:
            if( v >= m_fwVerLimitations[_T("tablevar")] )
                m_tableCombo->Append(_T("PS2"));
            else
                m_tableCombo->Append(_T("12X12"));
            m_tableCombo->Show(true);
            break;

    }

    m_tableCombo->Select( 0 );

    if( v >= m_fwVerLimitations[_T("clt")] ) {
        m_cltPatchMgr->SetRestrictState( RS_CELSIUS );
    } else {
        m_cltPatchMgr->SetRestrictState( RS_FAHRENHEIT );
    }

    if( v >= m_fwVerLimitations[_T("air")] ) {
        m_airPatchMgr->SetRestrictState( RS_CELSIUS );
    } else {
        m_airPatchMgr->SetRestrictState( RS_FAHRENHEIT );
    }

}

FwPath * SimpleFwDlg::GetFwPath(wxString devstr, wxString fwverstr)
{
    if( devstr == wxEmptyString )
        devstr = m_devCombo->GetValue();

    if( fwverstr == wxEmptyString )
        fwverstr = m_verCombo->GetValue();


    FirmwareVersion fwver;

    FirmwareInfo::extractFirmwareVersion( fwverstr, fwver );

    FwPathArray arr = m_fwList[ devstr ];

    FwPath * fw = NULL;

    for( uint16_t a=0; a<arr.GetCount() ; a++){
        if( fwver == arr[a]->GetVer() ){
            fw = arr[a];
            break;
        }
    }

    if(!fw)
        return NULL;

    short ct = 0;

    if( fw->GetPath16() != wxEmptyString )
        if( m_tableCombo->GetValue() == _T("16x14") || m_tableCombo->GetValue() == _T("non-PS2")  )
            ct = 1;

    fw->SetCurrType( ct );

    return fw;
}

void SimpleFwDlg::OnComboBox(wxCommandEvent &evt)
{
//    Logger::logf(LOG_MSG,_T("SimpleFwDlg::OnComboBox id=%i"),evt.GetId());

    switch( evt.GetId() ) {

        case idDevList:
            FillVerCombo(m_devCombo->GetValue());
            break;

        case idVerList: {

            FwPathArray& arr = m_fwList[m_devCombo->GetValue()];

            /// because the fw list in the combobox has reverse order
            int idx = arr.GetCount() - evt.GetInt() - 1;

            if(idx >= 0 && idx < (int)arr.GetCount())

            FillTableCombo( arr[idx] );

            }break;

        case idTableList:

            break;
    }
}


/**************************************************************************************************
                                            FwPath class
***************************************************************************************************/


bool FwPath::operator==( const FwPath& fp ) const
{
    if( 1
        //&& fp.Ver      == Ver
        && fp.Title    == Title
        && fp.Path     == Path
        && fp.Path16   == Path16
        && fp.Table    == Table
        && fp.Filename == Filename
        )
        return true;

    else
        return false;

}


bool FwPath::operator!=( const FwPath& fp ) const
{
    if( (*this) == fp )
        return false;
    else
        return true;
}


bool FwPath::ParseDir(const wxString &tmp, FwPathHash * hash)
{

    wxString v3dirprefix = FwDesc::FwDescriptorHash[DEVSTR_V3][_T("dir_prefix")].GetString(); //_T("firmware_");
    wxString roundprefix = FwDesc::FwDescriptorHash[DEVSTR_ROUND][_T("dir_prefix")].GetString(); //_T("round_");


    if( tmp.Find( v3dirprefix ) != wxNOT_FOUND ) {

        DeviceStr = DEVSTR_V3;

        Filename = FwDesc::FwDescriptorHash[DeviceStr][_T("filename")].GetString(); //_T("vems.hex");

        wxString fn16 = DirHelper::GetPath( dPROJECTS ) + tmp + DirHelper::Separ() + _T("my_make.16x14") + DirHelper::Separ();

        bool is16 = wxFileName::FileExists( fn16 + Filename );

//        Logger::logf(LOG_MSG,_T("check:")+fn16 + Filename);

        if(is16)
            Path16 = fn16;

        Path = DirHelper::GetPath( dPROJECTS ) + tmp + DirHelper::Separ();

        Table = is16;
        Title = tmp.Mid( v3dirprefix.Len() );
        FirmwareInfo::extractFirmwareVersion( Title, Ver );

//        Logger::logf(LOG_MSG,_T("Table=%i"),Table);

        if(hash) (*hash)[DeviceStr].Add(this);


    } else if( tmp.Find( roundprefix ) != wxNOT_FOUND ) {

        DeviceStr = DEVSTR_ROUND;

        Path = DirHelper::GetPath( dPROJECTS ) + tmp + DirHelper::Separ();
        Filename = FwDesc::FwDescriptorHash[DeviceStr][_T("filename")].GetString();//_T("round.chex");
        Table = -1;

        Title = tmp.Mid( roundprefix.Len() );
        FirmwareInfo::extractFirmwareVersion( Title, Ver );


        if(hash) (*hash)[DeviceStr].Add(this);
    } else
        return false;


    return true;

}


/**************************************************************************************************
                                        PatchDesc class
***************************************************************************************************/

bool PatchDesc::operator==( const PatchDesc& pd ) const
{
    if( 1
        && pd.ARef == ARef
        && pd.RRef == RRef
        && pd.Coec == Coec
        && pd.isCelsius == isCelsius
        //&& pd.Ext == Ext
        )

        return true;

    else
        return false;

}


bool PatchDesc::operator!=( const PatchDesc& pd ) const
{
    if( (*this) == pd )
        return false;
    else
        return true;
}

bool PatchDesc::ParseFileNameStr( const wxString &filename )
{

    wxArrayString datas = wxStringTokenize( filename , _T("_"));

    if( datas.GetCount() < 3 ) return false;

    if( datas[0].Find( _T("ptc") ) == wxNOT_FOUND ){

        RRef = datas[1];
        ARef = datas[2];
        Coec = NTCSTR;

    } else {

        Coec = PTCSTR;

        ARef = datas[1].Mid(4);
        RRef = datas[2];

    }

    if( datas.GetCount() == 4 && datas[3] == _T("Celsius") )
        isCelsius = true;

    return true;
}

bool PatchDesc::ParseFile( wxFileName fn, const wxString &dir )
{
    m_valid = false;

    if(!fn.FileExists()) return false;

    if( !ParseFileNameStr(fn.GetName()) ) return false;

    Ext = fn.GetExt() ;

    Path = dir + DirHelper::Separ() + fn.GetFullName();
    //Path = fn.GetFullPath();

    m_valid = true;

    return true;
}

/**************************************************************************************************
                                       PatchUiDesc class
***************************************************************************************************/

PatchUiDesc::PatchUiDesc()
{

}

void PatchUiDesc::Init(wxWindow * parent, wxGridBagSizer * sizer, int row)
{
    wxArrayString as;

    wxSize ps = wxSize(55,-1);

    TempCB = new wxComboBox(parent, idTemp, _T(""), wxDefaultPosition, ps, as, wxCB_READONLY);
    CoecCB = new wxComboBox(parent, idCoec, _T(""), wxDefaultPosition, ps, as, wxCB_READONLY);
    RRefCB = new wxComboBox(parent, idRRef, _T(""), wxDefaultPosition, ps, as, wxCB_READONLY | wxCB_SORT);
    ARefCB = new wxComboBox(parent, idARef, _T(""), wxDefaultPosition, ps, as, wxCB_READONLY | wxCB_SORT);


    CoecCB->Connect(idCoec,  wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( PatchUiDesc::OnComboBox ), NULL, this);
    RRefCB->Connect(idRRef,  wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( PatchUiDesc::OnComboBox ), NULL, this);
    //ARefCB->Connect(idARef,  wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( PatchUiDesc::OnComboBox ), NULL, this);
    TempCB->Connect(idTemp,  wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( PatchUiDesc::OnComboBox ), NULL, this);

    sizer->Add(TempCB, wxGBPosition(row, 1), wxDefaultSpan, wxALL, 5);
    sizer->Add(CoecCB, wxGBPosition(row, 2), wxDefaultSpan, wxALL, 5);
    sizer->Add(RRefCB, wxGBPosition(row, 3), wxDefaultSpan, wxALL, 5);
    sizer->Add(ARefCB, wxGBPosition(row, 4), wxDefaultSpan, wxALL, 5);


    m_isActive = false;

    //TempCB->Append( _T("F°") );
    //TempCB->Append( _T("C°") );

//    TempCB->Select(0);


}

void PatchUiDesc::CheckComboCnt(wxComboBox * cb)
{
    if(!m_isActive) return;

    if( cb->GetCount() > 1 ) {
        cb->Enable(true);
    } else {
        cb->Enable(false);
    }
}

void PatchUiDesc::FillCombos( int id )
{

    wxString tmp;

    switch(id){

        case idTemp:

            tmp = CoecCB->GetValue();

            CoecCB->Clear();

            for(uint16_t a = 0; a<PatchArr->GetCount(); a++){
                PatchDesc * pd = PatchArr->Item(a);
                if( TempCB->GetValue() == pd->GetTempStr()  )
                    AddPatchDetail( pd->GetCoec(), CoecCB );
            }

            if( CoecCB->FindString( tmp ) == -1)
                CoecCB->Select( 0 );
            else
                CoecCB->SetStringSelection( tmp );

            CheckComboCnt(CoecCB);


        case idCoec:

            tmp = RRefCB->GetValue();

            //Logger::logf(LOG_MSG,_T("PatchUiDesc::FillCombos idCoec RRefCB=%p, RRefCB tmp=")+tmp,RRefCB);

            RRefCB->Clear();

            for(uint16_t a = 0; a<PatchArr->GetCount(); a++){
                PatchDesc * pd = PatchArr->Item(a);
                if( TempCB->GetValue() == pd->GetTempStr() && CoecCB->GetValue() == pd->GetCoec() )
                    AddPatchDetail( pd->GetRRef(), RRefCB );
            }

            if( RRefCB->FindString( tmp ) == -1)
                RRefCB->Select( 0 );
            else
                RRefCB->SetStringSelection( tmp );

            //Logger::logf(LOG_MSG,_T("PatchUiDesc::FillCombos idCoec RRefCB val=")+RRefCB->GetValue());

            //break;



        case idRRef:

            tmp = ARefCB->GetValue();

            ARefCB->Clear();

            for(uint16_t a = 0; a<PatchArr->GetCount(); a++){
                PatchDesc * pd = PatchArr->Item(a);
                if( TempCB->GetValue() == pd->GetTempStr() && CoecCB->GetValue() == pd->GetCoec() && RRefCB->GetValue() == pd->GetRRef() )
                    AddPatchDetail( pd->GetARef(), ARefCB );
            }
            ARefCB->Select( 0 );
            if( ARefCB->FindString( tmp ) == -1)
                ARefCB->Select( 0 );
            else
                ARefCB->SetStringSelection( tmp );


/*
        case idARef:

            tmp = RRefCB->GetValue();

            RRefCB->Clear();

            for(uint16_t a = 0; a<PatchArr->GetCount(); a++){
                PatchDesc * pd = PatchArr->Item(a);
                if( TempCB->GetValue() == pd->GetTempStr() && CoecCB->GetValue() == pd->GetCoec() && ARefCB->GetValue() == pd->GetARef() )
                    AddPatchDetail( pd->GetRRef(), RRefCB );
            }
            RRefCB->Select( 0 );
            if( RRefCB->FindString( tmp ) == -1)
                RRefCB->Select( 0 );
            else
                RRefCB->SetStringSelection( tmp );
*/

            break;

    }


    CheckComboCnt(TempCB);

}

void PatchUiDesc::OnComboBox( wxCommandEvent &evt )
{

    FillCombos(evt.GetId());

}

bool PatchUiDesc::AddPatchDetail(const wxString &det, wxComboBox *cb)
{
    if( cb->FindString( det ) != wxNOT_FOUND )
        return false;

    cb->Append( det );
    return true;

}

void PatchUiDesc::EnableCB(bool enable)
{
    CoecCB->Enable( enable );
    RRefCB->Enable( enable );
    ARefCB->Enable( enable );
    TempCB->Enable( enable );
    m_isActive = enable;
    CheckComboCnt(CoecCB);
    CheckComboCnt(TempCB);
}

bool PatchUiDesc::isEnabled()
{
    /*
    if( 1
    && CoecCB->IsEnabled()
    && RRefCB->IsEnabled()
    && ARefCB->IsEnabled()
    && TempCB->IsEnabled() )
        return true;

    else
        return false;
        */

    return m_isActive;
}

void PatchUiDesc::InitTemp()
{

    for( uint16_t a = 0; a < PatchArr->GetCount(); a++)
        AddPatchDetail( PatchArr->Item(a)->GetTempStr() , TempCB);

    TempCB->Select( 0 );
    FillCombos( idTemp );

}

bool PatchUiDesc::SetValues( const PatchDesc &pd )
{

    if( !pd.IsValid() ) return false;

    bool res1, res2, res3, res4;

    for( uint16_t a=0; a< PatchArr->GetCount(); a++) {

        if( pd != *(PatchArr->Item(a)) ) continue;

        res1 = TempCB->SetStringSelection( pd.GetTempStr() );
        FillCombos( idTemp );

        res2 = CoecCB->SetStringSelection( pd.GetCoec() );
        FillCombos( idCoec );

        res3 = RRefCB->SetStringSelection( pd.GetRRef() );
        FillCombos( idRRef );

        res4 = ARefCB->SetStringSelection( pd.GetARef() );

        return true;
    }

    return false;
}

void PatchUiDesc::SetRestrictStateC( RestrictState state )
{

    TempCB->Clear();

    if( state & RS_FAHRENHEIT )
        TempCB->Append( _T("F°") );

    if( state & RS_CELSIUS )
        TempCB->Append( _T("C°") );

    TempCB->Select(0);

    FillCombos( idTemp );
}



PatchDesc * PatchUiDesc::GetPatch()
{
    if( !isEnabled() ) return NULL;

    PatchDesc pd(TempCB->GetValue(), CoecCB->GetValue(), ARefCB->GetValue(), RRefCB->GetValue() );

    for( uint16_t a=0; a < PatchArr->GetCount(); a++){
        if( pd == *(PatchArr->Item(a)) ) {
            return PatchArr->Item(a);
        }
    }

    return NULL;

}

/**************************************************************************************************
                                      PatchManagger class
***************************************************************************************************/

PatchManagger::PatchManagger()
{

}

void PatchManagger::SetRestrictState( RestrictState state )
{
 //   Logger::logf(LOG_MSG,_T("PatchManagger::SetRestrictState %i"),state);

    for( Int2PatchUiDescHash::iterator it = m_patchLines.begin(); it != m_patchLines.end(); it++){
        PatchUiDesc * pud = it->second;
        if(pud) pud->SetRestrictStateC( state );
    }

}

int PatchManagger::AddLine(wxWindow * parent, wxGridBagSizer * sizer, PatchDescArray * patcharr, const PatchDesc &pd)
{

    int row = sizer->GetRows();

    int id = wxNewId();

    wxRadioButton * RB = new wxRadioButton(parent, id, _T("patch"));
    sizer->Add(RB, wxGBPosition(row, 0), wxDefaultSpan, wxALL, 5);
    RB->Connect(wxEVT_COMMAND_RADIOBUTTON_SELECTED, wxCommandEventHandler( PatchManagger::OnRadioButton ),NULL, this);

    PatchUiDesc * pud = new PatchUiDesc();
    pud->Init(parent, sizer, row);
    pud->PatchArr = patcharr;
    pud->InitTemp();
    pud->EnableCB( false );

    pud->SetValues( pd );

    m_patchLines[id] = pud;

    sizer->SetRows( row+1 );

    return id;

}

void PatchManagger::Init(wxWindow * parent, wxGridBagSizer * sizer, PatchDescArray * patcharr, Project * prj)
{
    PatchArr = patcharr;

    int row = sizer->GetRows();

    int id = wxNewId();
    wxRadioButton * noneRB = new wxRadioButton(parent, id, _T("none"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
    m_patchLines[id] = NULL;

    noneRB->Connect(wxEVT_COMMAND_RADIOBUTTON_SELECTED, wxCommandEventHandler( PatchManagger::OnRadioButton ),NULL, this);
    sizer->Add(noneRB, wxGBPosition(row, 0), wxDefaultSpan, wxALL, 5);

    sizer->SetRows( row+1 );

    if(prj){
        wxArrayString types;
        types.Add(_T("firmware_patch"));
        FirmwareConfigItemArray arr = prj->CreateFirmwareConfigItems(types);

        //Logger::logf(LOG_MSG,_T("PatchManagger::Init fwic=%i"),arr.GetCount());

        for( uint8_t a=0; a < arr.GetCount(); a++){
            wxFileName fn( DirHelper::GetPath() + arr[a]->value);

            if( fn.GetExt() != PatchExt ) continue;

            PatchDesc pd;

            bool res = pd.ParseFile( fn, _T("") );

            if( !res ) continue;

            AddLine(parent, sizer, patcharr, pd);

        }

    } else {
        PatchDesc &pd = GetLastUsedPatch();
        //int id = AddLine(parent, sizer, patcharr, pd);
        AddLine(parent, sizer, patcharr, pd);
    }

}

PatchDesc& PatchManagger::GetLastUsedPatch()
{
    ConfigBase* configBase = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT);
    wxString fn = configBase->getValue(_T("/dialogcontentlayouts/SimpleFwDlg"), _T("last_patch_") + PatchExt, _T(""));

    //wxString fn = ConfigHelper::getValue( _T("/dialogcontentlayouts/SimpleFwDlg"), _T("last_patch_") + PatchExt, _T(""), CONFIG_TYPE_LAYOUT);
    //Logger::logf(LOG_MSG,_T("fn=")+fn);

    PatchDesc * pd = new PatchDesc();

    if( pd->ParseFileNameStr( fn ) ) {

        for( size_t a=0; a<PatchArr->GetCount(); a++){
            if( *(PatchArr->Item(a)) == (*pd) ) {
                return *(PatchArr->Item(a));
            }
        }

        return *(PatchArr->Item(0));

    } else {

        return *(PatchArr->Item(0));

    }

}

void PatchManagger::OnRadioButton( wxCommandEvent &evt )
{
//    Logger::logf(LOG_MSG,_T("PatchManagger::OnRadioButton id=%i"),evt.GetId());

    int id = evt.GetId();

    if( m_patchLines.find( id ) == m_patchLines.end() ) return;

    for( Int2PatchUiDescHash::iterator it = m_patchLines.begin(); it != m_patchLines.end(); it++){
        if( it->second && it->second->isEnabled() ) {
            it->second->EnableCB( false );
            break;
        }
    }

    if( m_patchLines[id] != NULL )
        m_patchLines[id]->EnableCB( true );

}


PatchDesc * PatchManagger::GetPatch()
{

    for( Int2PatchUiDescHash::iterator it = m_patchLines.begin(); it != m_patchLines.end(); it++){
        PatchUiDesc * pud = it->second;
        if(!pud) continue;
        if( pud->isEnabled() )
            return pud->GetPatch();
    }

    return NULL;
}

void FwDesc::Init()
{
    if( FwDescriptorHash.size() != 0 ) return;

    FwDescriptorHash[DEVSTR_V3][_T("devstr")] = DEVSTR_V3;
    FwDescriptorHash[DEVSTR_V3][_T("dir_prefix")] = _T("firmware_");
    FwDescriptorHash[DEVSTR_V3][_T("ispatch")] = true;
    FwDescriptorHash[DEVSTR_V3][_T("filename")] = _T("vems.hex");

    FwDescriptorHash[DEVSTR_ROUND][_T("devstr")] = DEVSTR_ROUND;
    FwDescriptorHash[DEVSTR_ROUND][_T("dir_prefix")] = _T("round_");
    FwDescriptorHash[DEVSTR_ROUND][_T("ispatch")] = false;
    FwDescriptorHash[DEVSTR_ROUND][_T("filename")] = _T("round.chex");
}

void FwDesc::cleanup()
{
    FwDescriptorHash.clear();
}
