#include "Symbol.h"


SymbolGroup::SymbolGroup( const wxString groupName, int groupOffset )
{
    name    = groupName;
    offset  = groupOffset;
}

Symbol::Symbol()
    : m_alias (_T(""))
    , m_units (_T(""))
    , m_const (false)
    , m_pageNo( 0)
    , m_offset(-1)
    , m_sizeOf( 1)
    , m_signed(false)
    , m_scale (1.0)
    , m_trans (0.0)
    , m_digits (0)
    , m_lo (0.0) //no range?
    , m_hi (0.0) //no range?
    , m_xOrder(true)
    , m_order (0)
    , m_bitLo (-1)
    , m_bitHi (-1)
    , m_bitOfs( 0)
    , m_expr ( _T("") )
    , m_group( NULL )
    , m_channel ( -1 )
    , m_exprFromRaw ( _T("") )
    , m_exprToRaw ( _T("") )
    , m_validationExpr ( _T("") )
{
    m_name = _T("");
    m_isValid = false;
    m_shape[0] = m_shape[1] = 1;
    m_inputType = -1;
}
/*
Symbol::Symbol(const Symbol& s)
{
    m_title     = s.m_title;
    m_gaugeType = s.m_gaugeType;
    m_name      = s.m_gaugeType;
    m_alias     = s.m_alias;
    m_units     = s.units;
    m_const     = s.m_const;
    m_pageNo    = s.m_pageNo;
    m_offset    = s.m_offset;
    int         m_sizeOf; // Number of bytes in one value, 1, 2 or 4 are valid.
    bool        m_signed; // Is data signed or unsigned?

    //--  User-units formatting information.  -----------------------------------
    double      m_scale;   // Scale of raw value.
    double      m_trans;   // Translation of raw value.
    int         m_digits;  // Display digits to right of decimal.
    //double      m_round;   // Fudge factor based upon digits count.
    double      m_lo;
    double      m_hi; // Limit values.
    //long        m_loRaw, m_hiRaw; // Limits in internal format.

    //--  Data for array types.  ------------------------------------------------
    bool        m_xOrder;   // Is table in x- or y-order?
    int         m_order;    // Tensor order, 0=scalar.
    int         m_shape[2]; // Size in up to two dimensions.

    //--  Data for bit fields.  -------------------------------------------------
    int         m_bitLo;    // Starting bit in a byte-base bitfield, zero-based.
    int         m_bitHi;    // Ending bit in bitfield.
    int         m_bitOfs;   // Offset for user value when extracted from bits.
                            // For instance, MS-I stores nCylinders in 0:3 as 1..16,
                            // so valueUser returns 0+1==1 for the first value.
    wxArrayString   m_bitFields;
    wxString        m_expr;
    wxString        m_exprReverse;      // reverse expression for const expressions
    wxString        m_target;       // reverse expression target for const expression
    SymbolGroup*    m_group;
    int             m_channel;

    bool    m_isValid;         //use this in the firmware config upload
    int     m_inputType;
}
*/

void Symbol::setBasic( const wxString& name, const wxString& units,bool _const, int pageNo, int offset )
{
    m_name      = name;
    m_units     = units;
    m_const     = _const;
    m_pageNo    = pageNo;
    m_offset    = offset;
}

void Symbol::setFormat(double scale, double trans, double lo, double hi, int digits)
{
    m_scale     = scale;
    m_trans     = trans;
    m_digits    = digits;
    m_lo        = lo;
    m_hi        = hi;
}

void Symbol::setType(bool isSigned, int sizeOf)
{
    m_signed = isSigned;
    m_sizeOf = sizeOf;
}

void Symbol::setArray(bool xOrder, int order, int dim1, int dim2)
{
    m_xOrder    = xOrder;
    m_order     = order;
    m_shape[0]  = dim1;
    m_shape[1]  = dim2;
}

void Symbol::setBit(int bitLo, int bitHi, int bitOfs, wxArrayString bitFields)
{
    m_bitLo = bitLo;
    m_bitHi = bitHi;
    m_bitOfs = bitOfs;
    m_bitFields = bitFields;
}

void Symbol::setExpr( const wxString& name, const wxString& expr)
{
        m_name = name;
        m_expr = expr;
}

int Symbol::getOffset() const
{
    if( m_group )
    {
        return m_offset;
        //return m_offset + m_group->offset;
    }
    else
    {
        return m_offset;
    }
}

Symbol::~Symbol()
{
    //dtor
}

wxString Symbol::basicToString()
{
    return wxString::Format(_T("Name : %s ; ")
                            _T("Units : %s ;")
                            _T("Const : %d ;")
                            _T("PageNo : %d ;")
                            _T("Offset : %d ; ")
                            _T("SizeOf : %d ; ")
                            _T("Signed : %d ; "),
                            m_name.c_str(),
                            m_units.c_str(),
                            m_const,
                            m_pageNo,
                            m_offset,
                            m_sizeOf,
                            m_signed);
}

wxString Symbol::formatToString()
{
    return wxString::Format(_T("Scale : %f ; ")
                            _T("Trans : %f ; ")
                            _T("Lo : %f ; ")
                            _T("Hi : %f ; ")
                            _T("Digits : %d ; "),
                            m_scale,
                            m_trans,
                            m_lo,
                            m_hi,
                            m_digits);
}

wxString Symbol::bitToString()
{
    wxString str;
    str.Append(wxString::Format(_T("BitLo : %d ; ")
                                _T("BitHi : %d ; ")
                                _T("BitOfs : %d ; "),
                                m_bitLo,
                                m_bitHi,
                                m_bitOfs));

    str.Append(_T("BitFields : "));

    for(size_t i = 0; i<m_bitFields.GetCount(); i++)
    {
        str.Append( m_bitFields.Item(i) + _T(", ") );
    }

    return str;
}

wxString Symbol::arrayToString()
{
    return wxString::Format(_T("xOrder : %d ; ")
                            _T("order : %d ; ")
                            _T("shape[0] : %d ; ")
                            _T("shpae[1] : %d ; " ),
                            m_xOrder,
                            m_order,
                            m_shape[0],
                            m_shape[1]);
}

wxString Symbol::exprToString()
{
    return wxString::Format(_T("name: %s, ")
                            _T("expr: %s"),
                            m_name.c_str(),
                            m_expr.c_str());
}

wxString Symbol::toString()
{
    wxString str;

    if(m_expr != _T("")){
        return exprToString();
    }

    str.Append(basicToString());

    if( m_bitLo != -1) //bit
    {
        str.Append( bitToString() );
        return str;
    }

    str.Append(formatToString());

    if(m_order != 0 ) //array
    {
        str.Append( arrayToString() );
        return str;
    }

    return str; //scalar
}

wxString Symbol::getType()
{
    if(m_bitLo != -1)   { return _T("BIT"); }
    if(m_order != 0)    { return _T("ARRAY"); }
    if(m_expr != _T("")){ return _T("EXPR"); }
    return _T("SCALAR");
}

Symbol* Symbol::createArray( const wxString& name, int x, int y, int page, int offset )
{
    Symbol* s = new Symbol();
    s->setBasic( name, wxEmptyString, true,  page, offset );
    int dimension = ( y == 1 ? 1 : 2 );
    s->setArray( true, dimension, x, y );

    return s;
}

Symbol* Symbol::createScalar( const wxString& name, int page, int offset, int type )
{
    Symbol* s = new Symbol();

    s->setBasic( name, wxEmptyString, true,  page, offset );

    if ( type != -1 )
    {
        s->setInputType( type );
    }

    return s;
}

bool Symbol::hasGauge() {

    if( m_gaugeType != wxEmptyString && m_title != wxEmptyString )
        return true;
    else
        return false;

}

void Symbol::setExpressionsForTransformations( const wxString& fromRaw, const wxString& toRaw )
{
    m_exprFromRaw   = fromRaw;
    m_exprToRaw     = toRaw;
}

bool Symbol::isPecialArray(const wxString& rawName, wxString& name, long& row) const
{
    if(rawName.Length() == 2){                     /// h0... h5 tables
        if( rawName.Mid(1).ToLong(&row) ){
            name = rawName.Mid(0,1);
            return true;
        }
    }
    return false;
}

bool Symbol::isVemsCfgArray() const
{
    //Logger::logf(LOG_MSG, _T("SymbolMgr::isConfigArraySymbol() symbol: ")+name);
    if( !isArray() ) return false;
    if ( m_name.Length() == 1 )                           /// j,l,n, etc tables
    {
        //Logger::logf(LOG_MSG, _T("SymbolMgr::isConfigArraySymbol() 1 symbol: ")+name);
        return true;
    }
    else if ( m_name.BeforeFirst('_').Length() == 1 )     /// j_r, j_k, etc tables
    {
       //Logger::logf(LOG_MSG, _T("SymbolMgr::isConfigArraySymbol() 2 symbol: ")+name);
        return true;
    }
    else if(m_name.Length() == 2){                     /// h0... h5 tables
        long idx;
        if( m_name.Mid(1).ToLong(&idx) ){
            return true;
        }

    }
/*
     /// h table hack it is handled as separate tables
     /// the main reason of this hack is that in this case there is no need to modify old ini files.
    if( (name.Len() == 2) && name.StartsWith(_T("h"))  ){
         Logger::logf(LOG_MSG, _T("SymbolMgr::isConfigArraySymbol() 3 symbol: ")+name);
      return true;
    }
*/
    return false;
}

bool Symbol::isVemsCfgArrayAxis() const
{
    wxString axisName = m_name.AfterFirst('_');
    if ( axisName == _T("r") ) return true;
    if ( axisName == _T("k") ) return true;
    return false;
}

int Symbol::getBitMask() const
{
    int mask = 0;

    if ( m_bitLo < 0 || m_bitHi < 0 )
        return 255;

    for ( int i = m_bitLo; i<= m_bitHi; ++i )
    {
        mask += 1 << i;
    }

    return mask;
}

/*
wxString Symbol::getVemsCfgName()
{
    if( !isArray() ) return m_name;
    if(m_name.Length() != 2) return m_name;

     long idx
        if( m_name.Mid(1).ToLong(&idx) ){
            return true;
        }
}
*/
