#include "UserLog.h"

#include "UserLogWindow.h"
#include "common/VemsMessage.h"
#include "common/VemsMessageHashHelper.h"

UserLogWindow * UserLog::m_lw = NULL;
UserLogLineSize UserLog::m_ulDefSize;
MessageContainerArray UserLog::m_messageQueue;

UserLog::UserLog()
{
    //ctor
}

UserLog::~UserLog()
{
    //dtor
}


void UserLog::Init( UserLogWindow * lw, UserLogLineSize defsize )
{
    m_lw = lw;
    m_ulDefSize = defsize;


    for(size_t a=0;a<m_messageQueue.GetCount();a++){
        MessageContainer * mc = m_messageQueue[a];
        vLog( *(mc->msg), mc->size, mc->pol);
        delete mc;
    }

    m_messageQueue.Clear();
}

void UserLog::uLog(UserLogType type, const wxString& str, UserLogLineSize size, ShowPolicy pol )
{
    if(!m_lw)
        return;

    if(size == ULOGLS_PREDEFINED)
        size = m_ulDefSize;

    m_lw->PutLog( type, str, size, pol);
}

void UserLog::vLog(const wxString &vmsgname, UserLogLineSize size, ShowPolicy pol )
{
    vLog( VemsMessageHashHelper::GetVemsMsg( vmsgname ), size, pol);
}

void UserLog::vLog(const VemsMessage& vmsg, UserLogLineSize size, ShowPolicy pol )
{
    if(!m_lw) {
//        m_messageQueue.Add( new MessageContainer(vmsg,size,pol) );
        VemsMessage* message = new VemsMessage( vmsg );
        m_messageQueue.Add( new MessageContainer(message,size,pol) );
        return;

    }

    if(size == ULOGLS_PREDEFINED)
        size = m_ulDefSize;

    UserLogType type = ULOG_INFO;

    switch( vmsg.getType() ){
        case VMT_WARNING :
            type = ULOG_WARNING;
            break;

        case VMT_ERROR :
            type = ULOG_ERROR;
            break;

        case VMT_INFO :
        case VMT_UNKNOWN :
            type = ULOG_INFO;
            break;
    }

    m_lw->PutLog( type, vmsg.getHtmlMsg(), size, pol);
}


void UserLog::rLog(const wxString& rawstr)
{
    if(!m_lw)
        return;

    m_lw->PutRawLog( rawstr );

}

ObserverBase* UserLog::GetObserver()
{
    return m_lw;
};

void UserLog::ClearLogWindow()
{
    if(m_lw)
        m_lw->ClearWin();
}
