#include "aggHelper.h"
#include <wx/arrimpl.cpp>

agg::rgba Agg::convertToRGBA( const wxColour& c )
{
    double r = (double)c.Red() / 255;
    double g = (double)c.Green() / 255;
    double b = (double)c.Blue() / 255;
    double a = (double)c.Alpha() / 255;
    return agg::rgba( r, g, b, a );
}

wxColor Agg::convertToColor( const agg::rgba& c )
{
    unsigned char r = (unsigned char)( c.r * 255 );
    unsigned char g = (unsigned char)( c.g * 255 );
    unsigned char b = (unsigned char)( c.b * 255 );
    unsigned char a = (unsigned char)( c.a * 255 );
    return wxColor( r, g, b, a );
}

wxAggObjectStack::wxAggObjectStack()
{
}

wxAggObjectStack::~wxAggObjectStack()
{
    WX_CLEAR_ARRAY( m_stack );
}

void wxAggObjectStack::push( wxAggObject* object )
{
    m_stack.Add( object );
}

wxAggObject* wxAggObjectStack::top() throw (wxAggObjectStackException)
{
    if ( m_stack.GetCount() < 1 )
        throw wxAggObjectStackException();
    else
        return m_stack.Last();
}


wxAggObject* wxAggObjectStack::pop() throw (wxAggObjectStackException)
{
    if ( m_stack.GetCount() < 1 )
        throw wxAggObjectStackException();
    else
    {
        wxAggObject* obj = m_stack[ m_stack.GetCount() - 1];
        m_stack.RemoveAt( m_stack.GetCount() - 1 );
        return obj;
    }
}

bool wxAggObjectStack::isEmpty() const
{
    return ( m_stack.GetCount() < 1 );
}

void wxAggObjectStack::clear()
{
    m_stack.Clear();
}
