#ifndef AIMCOMMUNICATIONCONFIG_H
#define AIMCOMMUNICATIONCONFIG_H

#include <wx/wx.h>
#include <wx/animate.h>
#include "../serialconfig.h"
#include "../communication/clients/AIMDetector.h"
#include "../events/CommunicationEvent.h"

class VemsTuneController;
class wxAnimationCtrl;
class AIMDeviceController;
class InputEvent;
class AIMDetectorEvent;

class AIMCommunicationConfig : public wxFrame
{
    public:
        AIMCommunicationConfig( wxWindow* parent );
        ~AIMCommunicationConfig();

    private:
        AIMDeviceController*    m_aimcontroller;

        wxComboBox*             m_comports;
        wxComboBox*             m_baudRate;
        wxComboBox*             m_enabled;

        wxEvtHandler*           m_evtHandler;

        SerialConfig            m_serialConfig;

        wxButton                *m_testButton, *m_saveButton, *m_closeButton;

        bool                    m_found;
        bool                    m_saved;
        bool                    m_wasECUOnline;

        wxArrayString           m_baudRateS;

        wxBitmap                m_noaimbmp;
        wxBitmap                m_aimdatabmp;
        wxBitmap                m_aimunknown;
        wxBitmap                m_aimtesting;

        wxStaticBitmap*         m_is_aim_bmp;
        wxAnimationCtrl*        m_animctrl;
        wxTimer                 m_timer;
        wxTimer                 m_porttimer;
        wxString                m_animFName;

        bool                    m_aimreceived;
        bool                    m_evtConnected;

        wxArrayString           m_availPorts;

    private:

        void        buildGUI();
        void        getAvailablePorts();

        void        stopController();

        void        OnSave( wxCommandEvent& event );
        void        OnTest( wxCommandEvent& event );
        void        OnClose( wxCommandEvent& event );

        void        OnCommEvent( CommunicationEvent& event );

        void        OnAIMEnabledCombo( wxCommandEvent& event );

        void        OnComboBox( wxCommandEvent& event );

        void        OnTimer( wxTimerEvent& event );

        void        OnSIPRInput( InputEvent& event );

        void        OnAIMDetectorEvent( AIMDetectorEvent& event );
        void        OnPortTimer( wxTimerEvent& event );

        inline void setTestButton( bool enable );
        inline void setAimBmp( bool incomingdata );

        static int  sortFunction( const wxString& first, const wxString& second );

    private:

        DECLARE_EVENT_TABLE()
};

#endif // AIMCOMMUNICATIONCONFIG_H
