#include "AIMCustomValuesEditorPanel.h"

#include <wx/wxsqlite3.h>
#include "../logger.h"

AIMCustomValuesEditorPanel::AIMCustomValuesEditorPanel( wxWindow * parent, int type, wxSQLite3Database* db ) : wxPanel( parent )
{
    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    m_type = type;
    m_db = db;

    getDBSettings();

    wxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    // predefined settings
    wxSizer* pdefsizer = new wxBoxSizer( wxHORIZONTAL );

    wxStaticText* txt = new wxStaticText( this, -1, _T("Predefined settings (for AIM Custom)") );
    m_predefinedSCombo = new wxComboBox( this, idPredefinedSettings, _T(""), wxDefaultPosition, wxDefaultSize, m_predefChansA, wxCB_DROPDOWN | wxCB_READONLY );
    Connect( idPredefinedSettings, wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onComboboxChanged) );

    pdefsizer->Add( txt, 0, wxEXPAND | wxALL, 2 );
    pdefsizer->Add( m_predefinedSCombo, 0, wxEXPAND | wxALL, 2 );

    sizer->Add( pdefsizer, 0, wxEXPAND | wxALL, 3 );

    // advanced settings
    txt = new wxStaticText( this, -1, _T("Advanced settings") );
    wxFont font = txt->GetFont(); font.SetWeight( wxFONTWEIGHT_BOLD ); txt->SetFont( font );
    sizer->Add( txt, 0, wxEXPAND | wxALL, 3 );
    txt = new wxStaticText( this, -1, _T("(Change these values if the the predefined\nsettings are not suitable for you)") );
    sizer->Add( txt, 0, wxEXPAND | wxALL, 3 );

    ///

    wxTextValidator validator( wxFILTER_NUMERIC );


    ///-----------------
    wxSizer* settings = new wxFlexGridSizer( 3 );
    txt = new wxStaticText( this, -1, _T("AIM Channel ID") );
    m_channelId = new wxTextCtrl( this, idAimChId, _T(""), wxDefaultPosition, wxDefaultSize, 0, validator );
        settings->Add( txt, 0, wxEXPAND | wxALL, 2 ); settings->Add( m_channelId, 0, wxEXPAND | wxALL, 2 );
        Connect( idAimChId, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onTextCtrlChanged) );
        m_channelId->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler( AIMCustomValuesEditorPanel::onKillFocus), NULL, this );

    m_predefChans = new wxComboBox( this, idPredefinedChannels, _T(""), wxDefaultPosition, wxDefaultSize, m_predefChansA, wxCB_DROPDOWN | wxCB_READONLY );
        settings->Add( m_predefChans, 0, wxEXPAND | wxALL, 2 );
        Connect( idPredefinedChannels, wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onComboboxChanged) );
    ///-----------------
    txt = new wxStaticText( this, -1, _T("Multiplicator") );
    m_multiplicator = new wxTextCtrl( this, idMultip, _T(""), wxDefaultPosition,wxDefaultSize, 0, validator );
        settings->Add( txt, 0, wxEXPAND | wxALL, 2 ); settings->Add( m_multiplicator, 0, wxEXPAND | wxALL, 2 );
        Connect( idMultip, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onTextCtrlChanged) );
        m_multiplicator->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler( AIMCustomValuesEditorPanel::onKillFocus), NULL, this );
    //txt = new wxStaticText( this, -1, _T(" %") );
    txt = new wxStaticText( this, -1, _T("") );
        settings->Add( txt, 0, wxEXPAND | wxALL, 2 );
    ///-----------------
    txt = new wxStaticText( this, -1, _T("Add") );
    m_translation = new wxTextCtrl( this, idTransl, _T(""), wxDefaultPosition, wxDefaultSize, 0, validator );
        settings->Add( txt, 0, wxEXPAND | wxALL, 2 ); settings->Add( m_translation, 0, wxEXPAND | wxALL, 2 );
        Connect( idTransl, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onTextCtrlChanged) );
        m_translation->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler( AIMCustomValuesEditorPanel::onKillFocus), NULL, this );
    //txt = new wxStaticText( this, -1, _T("-8192") );
    txt = new wxStaticText( this, -1, _T("") );
    settings->Add( txt, 0, wxEXPAND | wxALL, 2 );
    ///-----------------
    txt = new wxStaticText( this, -1, _T("Dot position") );
    m_dotpos = new wxComboBox( this, idDotPosId, m_dotposS[0], wxDefaultPosition,wxDefaultSize, m_dotposS, wxCB_DROPDOWN | wxCB_READONLY );
        settings->Add( txt, 0, wxEXPAND | wxALL, 2 ); settings->Add( m_dotpos, 0, wxEXPAND | wxALL, 2 );
        Connect( idDotPosId, wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onComboboxChanged) );
        settings->AddSpacer( 2 );
    ///-----------------
    txt = new wxStaticText( this, -1, _T("Conversion") ); txt->Disable();
    m_conversion = new wxTextCtrl( this, idConvers, _T(""), wxDefaultPosition,wxDefaultSize );
        settings->Add( txt, 0, wxEXPAND | wxALL, 2 ); settings->Add( m_conversion, 0, wxEXPAND | wxALL, 2 );
        Connect( idConvers, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( AIMCustomValuesEditorPanel::onTextCtrlChanged) );
        settings->AddSpacer( 2 );
    m_conversion->Disable();

    ///

    sizer->Add( settings, 0, wxEXPAND | wxALL, 3 );

    SetSizer( sizer );
    sizer->Fit( this );
}

CustomSettings AIMCustomValuesEditorPanel::getCustomSettings()
{
    CustomSettings s;

    s.aimchId = m_channelId->GetValue();

    s.add = m_translation->GetValue();

    s.dotSelection = m_dotpos->GetSelection();

    if ( m_type == 0 && s.dotSelection == 2 ) s.dotSelection = 3;

    s.multiplicator = m_multiplicator->GetValue();

    return s;
}

void AIMCustomValuesEditorPanel::setCustomSettings( CustomSettings& settings )
{
    if ( settings.aimchId == _T("-") )
    {
        m_predefChans->SetValue(_T(" ") );
        m_predefinedSCombo->SetValue( _T(" ") );
    }
    else
    {
        StringHash::iterator it = m_name2chanHash.find( settings.aimchId );
        if ( it != m_name2chanHash.end() )
            m_predefChans->SetValue( it->first );
    }
    m_channelId->ChangeValue( settings.aimchId );
    m_multiplicator->ChangeValue( settings.multiplicator );
    m_translation->ChangeValue( settings.add );
    if ( settings.conversion != wxEmptyString )
        m_conversion->ChangeValue( settings.conversion );

    if ( settings.dot != _T("") )
        m_dotpos->SetValue( settings.dot );
    else
        m_dotpos->SetSelection( settings.dotSelection );
    //m_dotpos->SetValue( settings.dot );

    m_predefChans->SetValue( checkChannelID() );

    m_customSettings = settings;
}

void AIMCustomValuesEditorPanel::onComboboxChanged( wxCommandEvent& event )
{
    int id = event.GetId();

    wxString typeS = ( m_type == 0 ) ? _T("top") : _T("bottom");

    if ( id == idDotPosId )
    {
        m_customSettings.dotSelection = m_dotpos->GetSelection();
        m_customSettings.dot = m_dotpos->GetValue();

        if ( m_type == 0 )
        {
            if ( m_dotpos->GetSelection() == 2 )
                m_customSettings.dotSelection = 3;
        }

        observerNotify( typeS );

        m_predefinedSCombo->SetValue( _T(" ") );
    }

    if ( id == idPredefinedChannels )
    {
        wxString selection = m_predefChans->GetValue();

        if ( selection == _T(" ") )
            return;

        m_customSettings.aimchId = m_name2chanHash[selection];
        m_channelId->SetValue( m_customSettings.aimchId );

        observerNotify( typeS );

        m_predefinedSCombo->SetValue( _T(" ") );
    }

    if ( id == idPredefinedSettings )
    {
        int selection = m_predefinedSCombo->GetSelection();

        if ( selection == 0 )
            return;

        --selection; // empty value is 0.

        m_customSettings = m_predefinedSettings[selection];
        m_channelId->SetValue( m_customSettings.aimchId );
        m_multiplicator->SetValue( m_customSettings.multiplicator );
        m_translation->SetValue( m_customSettings.add );
        m_conversion->SetValue( m_customSettings.conversion );
        if ( m_type == 0 )
            m_dotpos->SetValue( m_customSettings.dotTop );
        else
            m_dotpos->SetValue( m_customSettings.dotBottom );

        m_predefChans->SetValue( m_predefinedSCombo->GetValue() );

        observerNotify( typeS );
    }

}

void AIMCustomValuesEditorPanel::onTextCtrlChanged( wxCommandEvent& event )
{
    int id = event.GetId();

    wxString val;

    switch ( id )
    {
        case idAimChId:
        {
            val = m_channelId->GetValue();

            long d = 0; val.ToLong( &d );

            minmaxcheck( d, 0L, 255L, m_channelId, _T("%d") );

            m_customSettings.aimchId = wxString::Format( _T("%d"), d );

            // observerNotify( 0, m_type );
            break;
        }

        case idMultip:
        {
            val = m_multiplicator->GetValue();

            double d = 0.0; val.ToDouble( &d );

            minmaxcheck( d, 0.0, 2.0, m_multiplicator, _T("%.1f") );

            m_customSettings.multiplicator = wxString::Format( _T("%.1f"), d );

            // observerNotify( 0, m_type );
            break;
        }

        case idTransl:
        {
            val = m_translation->GetValue();

            double d  = 0.0; val.ToDouble( &d );

            minmaxcheck( d, -8192.0, 57343.0, m_translation, _T("%.1f") );

            m_customSettings.add = wxString::Format( _T("%.1f"), d );

            // observerNotify( 0, m_type );
            break;
        }

        case idConvers:
            m_customSettings.conversion = m_conversion->GetValue();
            observerNotify( 0, m_type );
            break;

        default:
            break;
    }
}

void AIMCustomValuesEditorPanel::onKillFocus( wxFocusEvent& event )
{
    int id = event.GetId();

    if ( id == m_channelId->GetId() )
    {
        m_predefinedSCombo->SetValue( _T(" ") );

        m_predefChans->SetValue( checkChannelID() );

        observerNotify( m_type, m_channelId );
    }

    if ( id == m_multiplicator->GetId() )
    {
        m_predefinedSCombo->SetValue( _T(" ") );
        observerNotify( m_type, m_multiplicator );
    }

    if ( id == m_translation->GetId() )
    {
        m_predefinedSCombo->SetValue( _T(" ") );
        observerNotify( m_type, m_translation );
    }

}

void AIMCustomValuesEditorPanel::getDBSettings()
{
    m_dotposS.Clear();

    wxString query = wxString::Format( _T("SELECT * FROM aimdotposition WHERE isbottom=%d ORDER BY value"), m_type );
    wxSQLite3ResultSet res = m_db->ExecuteQuery( query );
    while ( res.NextRow() )
    {
        wxString str = res.GetString(_T("title"));
        if ( str == _T("INVALID") )
            continue;
        m_dotposS.Add( str );
    }

    m_name2chanHash.clear();
    m_predefChansA.Clear();
    m_predefChansA.Add(_T(" "));

    int chans = 0;
    query = _T("SELECT COUNT(*) as num FROM aimchannels");
    res = m_db->ExecuteQuery( query );
    while ( res.NextRow() )
    {
        chans = res.GetInt( _T("num") );
    }


    m_predefinedSettings = new CustomSettings[chans];

    query = _T("SELECT * FROM aimchannels ORDER BY channelname");
    res = m_db->ExecuteQuery( query );

    int i = 0;

    while ( res.NextRow() )
    {
        wxString name = res.GetString( _T("channelname") );
        wxString channelid = res.GetString( _T("channelid") );
        wxString a = res.GetString( _T("a") );
        wxString b = res.GetString( _T("b") );
        wxString conv = res.GetString( _T("conversion") );

        wxString dott = res.GetString( _T("dottop") );
        wxString dotb = res.GetString( _T("dotbottom") );

        m_name2chanHash[name] = channelid;
        m_predefChansA.Add( name );

        m_predefinedSettings[i].aimchId = channelid;
        m_predefinedSettings[i].add = b;
        m_predefinedSettings[i].multiplicator = a;
        m_predefinedSettings[i].conversion = conv;
        m_predefinedSettings[i].dotTop = dott;
        m_predefinedSettings[i].dotBottom = dotb;
        ++i;
    }
}

wxString AIMCustomValuesEditorPanel::checkChannelID()
{
    StringHash::iterator it = m_name2chanHash.begin();

    const wxString& chId = m_channelId->GetValue();

    for ( ; it != m_name2chanHash.end(); ++it )
    {
        if ( it->second == chId )
            return it->first;
    }

    return _T(" ");
}

template<class T>
void AIMCustomValuesEditorPanel::minmaxcheck( T& val, T minval, T maxval, wxTextCtrl* ctrl, wxString format )
{
    if ( val < minval )
    {
        val = minval;
        ctrl->SetValue( wxString::Format( format, val ) );
        return;
    }
    if ( val > maxval )
    {
        val = maxval;
        ctrl->SetValue( wxString::Format( format, val ) );
        return;
    }
}
