#ifndef ROUNDDEMOMODESETTINGS_H
#define ROUNDDEMOMODESETTINGS_H

#include <wx/wx.h>

#include "../stdatamanager/STDataViewerBase.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../vemsdialog/StaticViewersController.h"
#include "../vemsdialog/StaticControlPanel.h"
#include "../Symbol.h"
#include "LEDDisplay.h"
#include "../common/types/wxRegionArray.h"

//---
extern const wxEventType wxEVT_LEDCLICK;

#define EVT_LEDCLICK(id, func) \
    wx__DECLARE_EVT1(wxEVT_LEDCLICK, id, LedClickEventHandler(func))

class LedClickEvent : public wxEvent
{
    public:
        LedClickEvent(int id = 0) : wxEvent( id, wxEVT_LEDCLICK ) {}
        virtual wxEvent *Clone() const { return new LedClickEvent(*this); }
        int lednumber;
        int segmentnumber;
    public:
        DECLARE_DYNAMIC_CLASS(LedClickEvent)
};

typedef void (wxEvtHandler::*LedClickEventFunction)(LedClickEvent&);

#define LedClickEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(LedClickEventFunction, &func)

//---
class RoundDemoModeLedDisplay : public wxPanel
{
    public:
        RoundDemoModeLedDisplay( wxWindow* parent, int id, const wxPoint& pos, const wxSize& size, int topcolor = LEDDisplay::RED_LED, int bottomcolor = LEDDisplay::RED_LED);
        ~RoundDemoModeLedDisplay();

    public:
        // display: 0-2: top, 3-6, bottom, value: -1, forced, 0-255 segmentvalue
        void        setSegments( int display, int value );
        void        redraw();

    private:
        LEDDisplay*     m_leds[7];
        int             m_topcolor, m_bottomcolor; // see LEDDisplay
        int             m_ledvalues[7];
        wxPoint**       m_regions[7];
        wxEvtHandler*   m_evtHandler;

    private:
        void        OnPaint( wxPaintEvent& event );
        void        OnClick( wxMouseEvent& event );

        DECLARE_EVENT_TABLE()

};

class SymbolMgr;

class RoundDemoModeSettings : public wxPanel, public STDataViewerBase
{
    public:
        RoundDemoModeSettings( wxWindow* parent, STDataMgrBase* stMgr, SymbolTable* symbols, SymbolMgr* symbolMgr );
        ~RoundDemoModeSettings();

    public:
        // stdataviewerbase virtual methods
        void            update();
        void            requestRead() {}
        void            updateModel();
        void            requestWrite();

        StaticViewersController*  getController() { return m_statvcontroller; }

    protected:

    private:
        SymbolTable *               m_symbols;
        SymbolMgr*                  m_symbolMgr;
        StaticViewersController *   m_statvcontroller;
        StaticControlPanel *        m_statcpanel;

        RoundDemoModeLedDisplay*    m_displayPanel;

        int                         m_demomodeVal;

        wxCheckBox*                 m_forceOutput[7];
        int                         m_foIds[7];

    private:

        void        buildGUI( wxWindow* parent );

        void        addConfigletPanel( wxSizer* sizer );

        // segment value<->segment value mappers
        int         segmentMapper( int value, bool isTranslationA );
        int         reverseSegmentMapper( int value, bool isTranslationA );

        // event handlers
        void        OnLEDClick( LedClickEvent& event );
        void        OnCheckBox( wxCommandEvent& event );
        void        OnButton( wxCommandEvent& event );
        void        OnClose( wxCloseEvent& event );

        DECLARE_EVENT_TABLE()
};

#endif // ROUNDDEMOMODESETTINGS_H
