#ifndef BCLABEL_H
#define BCLABEL_H

#include <wx/wx.h>
#include "../common/typedef.h"

#include "BitmapCanvasWidgetBase.h"

#include "BitmapCanvasSystemSettings.h"

class TextDesc
{
    public:

        TextDesc(const wxString& text = wxEmptyString,
                 uint8_t level = 0,
                 int align = wxALIGN_LEFT,
                 double height = BitmapCanvasSystemSettings::GetDouble(BCBS_TEXT_HEIGHT),
                 const wxRealPoint& pos = wxRealPoint(0,0),
                 const wxColor& color = BitmapCanvasSystemSettings::GetColor(BCBS_TEXT_COLOR),
                 double weight = BitmapCanvasSystemSettings::GetDouble(BCBS_TEXT_WEIGHT),
                 double angle = 0.0,
                 const wxString& fontfile = BitmapCanvasSystemSettings::GetString(BCBS_FONT_FILE),
                 long style = 0 )

//                 double height = 12.0,
//                 const wxRealPoint& pos = wxRealPoint(0,0),
//                 const wxColor& color = wxColor(0,0,0,255),
//                 double weight = 0.0,
//                 double angle = 0.0,
//                 const wxString& fontfile = _T("arial.ttf") )
                 {
            Text = text;
            Level = level;
            Align = align;
            Color = color;
            Height = height;
            Weight = weight;
            Angle = angle;
            Fontfile = fontfile;
            Position = pos;
            Style = style;
        };

        wxString    Text;
        uint8_t     Level;
        int         Align;
        wxColor     Color;
        double      Weight;
        double      Height;
        double      Angle;
        wxString    Fontfile;
        wxRealPoint Position;
        long        Style;

};

enum TextDescStyles {
    bcLabel_UNDERLINED  = 0x01,
    bcLabel_STRIKE      = 0x02,
    bcLabel_GRAVEN      = 0x04,
    bcLabel_SHADOW      = 0x08
};

enum bcLabelStyles {
    BCLS_AUTO_SIZE      = 0x01,
    BCLS_DROPSHADOW     = 0x02,
    BCLS_TRANSPARENT_BG = 0x04,

    BCLS_DEFAULT = BCLS_AUTO_SIZE | BCLS_TRANSPARENT_BG
};

extern const TextDesc DefaultTextDesc;

WX_DECLARE_HASH_MAP( uint8_t, TextDesc, wxIntegerHash, wxIntegerEqual, TextDescHash);

class bcLabel: public BitmapCanvasWidgetBase
{
    DECLARE_DYNAMIC_CLASS(bcLabel);

    public:
        bcLabel();
        bcLabel(BitmapCanvasParentBase* parent, int id = wxID_ANY, const wxString& text = wxEmptyString, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = BCLS_DEFAULT, bool isPreDraw = true);
        bcLabel(BitmapCanvasParentBase* parent, int id, const TextDesc& desc, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = BCLS_DEFAULT, bool isPreDraw = true);
        bcLabel(BitmapCanvasParentBase* parent, const wxString& text, wxVariantIntHash& properties, bool isPreDraw = true);
        virtual ~bcLabel();

        virtual void Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties);

        virtual bool Focusable();

        void SetLabel(TextDesc desc);
        void SetLabel(const wxString& label, uint8_t level = 0);
        void WriteLabel(TextDesc desc);
        void WriteLabel(const wxString& label, uint8_t level = 0);
        void ChangeLabel(TextDesc desc);
        void ChangeLabel(const wxString& label, uint8_t level = 0);

        wxString GetLabel(uint8_t level = 0);
        TextDesc GetLabelDesc(uint8_t level = 0);

        virtual void DrawWidget();

        wxRect GetTextBox(int level = -1);
        wxRect GetTextBox(TextDesc& desc);
    protected:

        TextDescHash    m_labels;

    private:


};

#endif // BCLABEL_H
