#include "bcProgressBar.h"

#include "../agg/wxAgg2.h"

#include "../logger.h"
#include "../wxHSLColor.h"

#include "../DirHelper.h"
#include "BitmapCanvasSystemSettings.h"

BEGIN_EVENT_TABLE(bcProgressBar,BitmapCanvasWidgetBase)

    EVT_TIMER(idTimer1,bcProgressBar::OnTimer)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(bcProgressBar, BitmapCanvasWidgetBase);

bcProgressBar::bcProgressBar(BitmapCanvasParentBase* parent, int id, const wxPoint& pos, const wxSize& size, long style)
    :BitmapCanvasWidgetBase(parent,id,pos,size,bcDefaultZindex,style)
    ,m_timer(this,idTimer1)
{
    //ctor
    if( size == wxDefaultSize ) m_properties[BCWB_SIZE] << wxRealSize(100,18);



    m_segmentOffset = 0;

    SetProperty(BCPB_ANIM_INTERVAL,30L);

    SetProperty(BCPB_MIN_VALUE,0.0);
    SetProperty(BCPB_MAX_VALUE,100.0);
    SetProperty(BCPB_CURR_VALUE,0.0);

    SetProperty(BCPB_TEXT_FORMAT, _T("%0.1f"));


    m_style = style;

    DrawWidget();
}

bcProgressBar::~bcProgressBar()
{
    //dtor

}
/// TODO: rethink and implement
void bcProgressBar::Create(BitmapCanvasParentBase* parent, wxVariantIntHash& properties)
{

}

void bcProgressBar::SetAnimInterval(long milliseconds)
{
    if( !(m_style & BCPBS_INFINITY) ) return;

    SetProperty(BCPB_ANIM_INTERVAL,milliseconds);
    StartAnim();
}

void bcProgressBar::StartAnim()
{
    if( m_style & BCPBS_INFINITY ) m_timer.Start(GetAnimInterval());
}

void bcProgressBar::StopAnim()
{
    m_timer.Stop();
}

bool bcProgressBar::IsPlay()
{
    return m_timer.IsRunning();
}

void bcProgressBar::SetStepPx(double step)
{
    if( fabs(GetPropDouble(BCPB_SEGMENT_STEPPX)-step) < 0.0001 ) return;

    SetProperty(BCPB_SEGMENT_STEPPX,step);

    Refresh();
}

double bcProgressBar::GetStepPx()
{
    return GetPropDouble(BCPB_SEGMENT_STEPPX);
}

void bcProgressBar::OnTimer(wxTimerEvent& evt)
{
    if( !(m_style & BCPBS_INFINITY) )  {
        StopAnim();
        return;
    }

    double step = GetStepPx();

    m_segmentOffset += fabs(step) < 0.0001 ? (GetSize().h/13.0) : step;

    wxRealSize size = GetSize();

    int infMax = GetSize().Ceil().y;

    if(m_segmentOffset >= infMax*2)
        m_segmentOffset = 0;

    Refresh();
    Update(false);
}

void bcProgressBar::SetValue(double value)
{
    if( fabs(GetPropDouble(BCPB_CURR_VALUE)-value) < 0.0001 ) return;

    SetProperty(BCPB_CURR_VALUE,value);

    Refresh();
}

double bcProgressBar::GetValue()
{
    return GetPropDouble(BCPB_CURR_VALUE);
}

void bcProgressBar::SetMinValue(double value)
{
    if( fabs(GetPropDouble(BCPB_MIN_VALUE)-value) < 0.0001 ) return;

    SetProperty(BCPB_MIN_VALUE,value);

    Refresh();
}

double bcProgressBar::GetMinValue()
{
    return GetPropDouble(BCPB_MIN_VALUE);
}

void bcProgressBar::SetMaxValue(double value)
{
    if( fabs(GetPropDouble(BCPB_MAX_VALUE)-value) < 0.0001 ) return;

    SetProperty(BCPB_MAX_VALUE,value);

    Refresh();
}

double bcProgressBar::GetMaxValue()
{
    return GetPropDouble(BCPB_MAX_VALUE);
}

void bcProgressBar::DrawWidget()
{
    wxColor bgcolor = GetSysColor(BCBS_BACKGROUND_COLOR);

    wxRealSize size = GetSize();

    wxAgg2 aggCanvas(this,bgcolor);

    if(!aggCanvas.isOk()) return;

    double border_width = GetSysDouble(BCBS_BORDER_WIDTH);
    double border_radius = GetSysDouble(BCBS_BORDER_RADIUS);

    wxAggPen* pen = aggCanvas.getPen();
    wxAggBrush* brush = aggCanvas.getBrush();

    wxRealRect bbox(border_width/2,border_width/2,size.w-border_width,size.h-border_width);
    wxColor bar_color = GetSysColor(BCBS_DYNAMIC_CONTENT_COLOR);

    bool hasRoundedBorder = true;
    bool hasGlass = false;

    pen->SetStyle(wxAGGPEN_TRANSPARENT);
    brush->SetStyle(wxAGGBRUSH_GRADIENT);
    wxHSLColor bghsl(bgcolor);
    bghsl.OffsetLum(-0.3);
    brush->SetColor(bghsl.ToRGB());
    brush->SetEndColor(bgcolor);
    brush->SetGradientOrientation( 90 );

    if(hasRoundedBorder) {
        aggCanvas.drawRoundedRectangle(bbox,border_radius);
    } else {
        aggCanvas.drawRectangle(bbox);
    }

    if( m_style & BCPBS_INFINITY ) {

        hasRoundedBorder = false;
        //hasGlass = true;

        wxHSLColor c2(bar_color);
        c2.OffsetLum(-0.2);
        pen->SetStyle(wxAGGPEN_TRANSPARENT);
        brush->SetStyle(wxAGGBRUSH_GRADIENT);
        brush->SetColor(bar_color);
        brush->SetEndColor(c2.ToRGB());

        int segment_count = (int)ceil(size.w / size.h);

        double segment_size = size.h;

        double startX = m_segmentOffset;

        for(int s = -2; s < segment_count; s+=2){
            RealPointArray poly;
            poly.Add( wxRealPoint( startX+s*segment_size, segment_size ) );
            poly.Add( wxRealPoint( startX+s*segment_size+segment_size, segment_size ) );
            poly.Add( wxRealPoint( startX+s*segment_size+2*segment_size, 0 ) );
            poly.Add( wxRealPoint( startX+s*segment_size+segment_size, 0 ) );

            aggCanvas.drawPolygon(poly);
        }

    } else {

        double curval = GetValue();
        double minval = GetMinValue();
        double maxval = GetMaxValue();

        double ratio = (curval - minval) / (maxval - minval);

        wxHSLColor c2(bar_color);
        c2.OffsetLum(-0.2);
        pen->SetStyle(wxAGGPEN_TRANSPARENT);
        brush->SetStyle(wxAGGBRUSH_GRADIENT);
        brush->SetColor(bar_color);
        brush->SetEndColor(c2.ToRGB());


        wxRealRect bar_rect = bbox;
        bar_rect.w *= ratio;

        aggCanvas.drawRoundedRectangle(bar_rect,border_radius);

        wxString text = wxEmptyString;

        if(m_style & BCPBS_SHOW_TITLE) {
            text += GetPropString(BCPB_TITLE);
        }

        if(m_style & BCPBS_SHOW_TEXT) {

            wxString tf = GetPropString(BCPB_TEXT_FORMAT);
            text += wxString::Format(_T("%.1f"),curval);
        }

        if(text.Len()) {

            double font_size = ceil(size.h * 0.65);

            wxString fontfile = DirHelper::GetPath(dFONTS) + GetSysString(BCBS_FONT_FILE);
            if(aggCanvas.loadTTFont(fontfile, font_size, 0 ) == false) return;
            pen->SetStyle(wxAGGPEN_SOLID);

            wxRealRect textbox = aggCanvas.getTTFTextBound( text );
            wxPoint text_pos = wxRealPointEx( (bbox.w-textbox.w)/2, (bbox.h-textbox.h)/2 + textbox.h*0.8 ).Ceil();

            pen->SetColor( GetSysColor(BCBS_TEXT_COLOR) );
            aggCanvas.drawTTFText( text, text_pos.x+1, text_pos.y+1 );
            pen->SetColor( GetSysColor(BCBS_TEXT_GRAVEN_COLOR) );
            aggCanvas.drawTTFText( text, text_pos.x, text_pos.y );

        }
    }

    if(hasGlass) {

        wxColor glass1(255,255,255,190);
        wxColor glass2(255,255,255, 80);

        pen->SetStyle(wxAGGPEN_TRANSPARENT);
        brush->SetStyle(wxAGGBRUSH_GRADIENT);
        brush->SetColor(glass1);
        brush->SetEndColor(glass2);
        brush->SetGradientOrientation( 90 );

        wxRealRect glassEffectBox(bbox);
        glassEffectBox.h /= 2;

        aggCanvas.drawRectangle( glassEffectBox );
    }

    if(border_width > 0.5) {
        pen->SetStyle(wxAGGPEN_SOLID);
        brush->SetStyle(wxAGGBRUSH_TRANSPARENT);

        pen->SetColor( GetSysColor(BCBS_BORDER_COLOR) );

        if(hasRoundedBorder)
            aggCanvas.drawRoundedRectangle(bbox,border_radius);
        else
            aggCanvas.drawRectangle(bbox);

    }


    render(aggCanvas);
}

