#include "bcProgressDialog.h"

#include "bcProgressBar.h"
#include "bcWrapper.h"
#include "bcButton.h"
#include "bcLabel.h"

#include "../logger.h"

#define UNKNOWNSTR _T("Unknown")


BEGIN_EVENT_TABLE(bcProgressDialog,bcParentDialog)

    EVT_BUTTON(idButtonSkip, bcProgressDialog::OnButton)
    EVT_BUTTON(idButtonAbort, bcProgressDialog::OnButton)

    EVT_TIMER(idTimer,bcProgressDialog::OnTimer)

END_EVENT_TABLE()

bcProgressDialog::bcProgressDialog(const wxString& title, const wxString& message, int maximum, wxWindow * parent, int style)
    :bcParentDialog(parent, wxID_ANY, title)
    ,m_timer(this,idTimer)
{
    //ctor
    m_style = style;
    m_max = maximum;
    m_value = 0;
    m_msg = message;

    m_finished = false;

    m_elapsedLabel = NULL;
    m_estimatedLabel = NULL;
    m_remainingLabel = NULL;

    m_skipButton = NULL;
    m_abortButton = NULL;

    m_isAborted = false;
    m_isSkipped = false;

    m_parentTop = wxGetTopLevelParent( parent );

    BuildGUI();

    m_watch.Start();

    if ( m_style & wxPD_APP_MODAL )
    {
        m_winDisabler = new wxWindowDisabler(this);
    }
    else
    {
        if ( m_parentTop )
            m_parentTop->Disable();
        m_winDisabler = NULL;
    }

    Show();
    Enable();
    m_timer.Start(100);
}

bcProgressDialog::~bcProgressDialog()
{
    //dtor
    m_timer.Stop();

    if( m_bar->GetStyle() & BCPBS_INFINITY )
    {
        m_bar->StopAnim();
    }


    ReenableOtherWindows();
}

void bcProgressDialog::OnTimer(wxTimerEvent& evt)
{
    if( m_bar->GetStyle() & BCPBS_INFINITY ){
        if(!m_bar->IsPlay()) m_bar->StartAnim();
    }
    UpdateTimes();
}

void bcProgressDialog::BuildGUI()
{
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    int padding = 10;

    m_msgLabel = new bcLabel(this, wxID_ANY, m_msg);

    mainsizer->Add( new bcWrapper(m_msgLabel), 0, wxALL, padding );

    m_bar = new bcProgressBar(this, wxID_ANY, wxDefaultPosition, wxSize(200,20));

    mainsizer->Add( new bcWrapper(m_bar), 0, wxALL | wxEXPAND, padding );

    wxFlexGridSizer* timessizer = new wxFlexGridSizer(2);

    if(m_style & wxPD_ELAPSED_TIME) {
        bcLabel* timelabel = new bcLabel(this, wxID_ANY, _T("Elapsed time :"));
        timessizer->Add(new bcWrapper(timelabel), 0, wxALL | wxALIGN_RIGHT, padding/2 );
        m_elapsedLabel = new bcLabel(this, wxID_ANY, UNKNOWNSTR);
        timessizer->Add(new bcWrapper(m_elapsedLabel), 0, wxALL | wxALIGN_LEFT, padding/2 );
    }

    if(m_style & wxPD_REMAINING_TIME) {
        bcLabel* timelabel = new bcLabel(this, wxID_ANY, _T("Remaining time :"));
        timessizer->Add(new bcWrapper(timelabel), 0, wxALL | wxALIGN_RIGHT, padding/2 );
        m_remainingLabel = new bcLabel(this, wxID_ANY, UNKNOWNSTR);
        timessizer->Add(new bcWrapper(m_remainingLabel), 0, wxALL | wxALIGN_LEFT, padding/2 );
    }

    if(m_style & wxPD_ESTIMATED_TIME) {
        bcLabel* timelabel = new bcLabel(this, wxID_ANY, _T("Estimated time :"));
        timessizer->Add(new bcWrapper(timelabel), 0, wxALL | wxALIGN_RIGHT, padding/2 );
        m_estimatedLabel = new bcLabel(this, wxID_ANY, UNKNOWNSTR);
        timessizer->Add(new bcWrapper(m_estimatedLabel), 0, wxALL | wxALIGN_LEFT, padding/2 );
    }

    mainsizer->Add(timessizer,0,wxEXPAND);

    wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);

    if(m_style & wxPD_CAN_ABORT) {
        m_abortButton = new bcButton(this, idButtonAbort, _T("Cancel"));
        buttonSizer->Add( new bcWrapper(m_abortButton), 0, wxALL, padding );
    }
    if(m_style & wxPD_CAN_SKIP) {
        m_skipButton = new bcButton(this, idButtonSkip, _T("Skip"));
        buttonSizer->Add( new bcWrapper(m_skipButton), 0, wxALL, padding );
    }

    #ifdef __WXMSW__
    EnableCloseButton( false );
    #endif

    mainsizer->Add(buttonSizer, 0, wxALIGN_CENTER_HORIZONTAL);

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);
}

void bcProgressDialog::UpdateMessage(const wxString& msg)
{
    if(m_msgLabel->GetLabel() != msg) {
//        m_msgLabel->WriteLabel(msg);
//        m_msgLabel->Update();
        m_msgLabel->SetLabel(msg);
    }
}

wxString bcProgressDialog::FormatTime(long millisescs)
{
    int hours = millisescs / 1000 / 3600;
    int mins = millisescs / 1000 / 60 - hours * 60;
    int secs = millisescs / 1000 - mins * 60 - hours * 3600;
    //int msecs =  millisescs - secs * 1000 - mins * 60 * 1000 - hours * 3600 * 1000;

    //return wxString::Format(_T("%i:%02i:%02i.%03i"),hours,mins,secs,msecs);
    return wxString::Format(_T("%i:%02i:%02i"),hours,mins,secs);
}

bool bcProgressDialog::Show( bool show )
{
    if( !show )
    {
        ReenableOtherWindows();
    }

    return bcParentDialog::Show( show );
}

void bcProgressDialog::UpdateTimes()
{
    double position = (double)m_value / (double)m_max;

    long currTime = m_watch.Time();
    long estTime = position > 0.001 ? (int)((double)currTime / position) : 0;
    long remTime = estTime ? (estTime - currTime) : 0;

    if(m_style & wxPD_ELAPSED_TIME) {
        m_elapsedLabel->SetLabel( FormatTime(currTime) );
    }

    if(m_style & wxPD_ESTIMATED_TIME) {
        if(estTime) m_elapsedLabel->SetLabel( FormatTime(estTime) );
    }

    if(m_style & wxPD_REMAINING_TIME) {
        if(remTime) m_elapsedLabel->SetLabel( FormatTime(remTime) );
    }

}

void bcProgressDialog::OnButton(wxCommandEvent& event)
{
    switch(event.GetId()){
        case idButtonAbort: m_isAborted = true; break;
        case idButtonSkip: m_isSkipped = true; break;
    }
}

bool bcProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
{
    if ( m_finished )
        return true;

    wxYieldIfNeeded();

    m_bar->DelStyleFlag( BCPBS_INFINITY );

    if(m_bar->IsPlay()) m_bar->StopAnim();

    m_bar->SetValue(value);

    if(newmsg != wxEmptyString) m_msgLabel->SetLabel(newmsg);

    UpdateTimes();

    if(skip) *skip = m_isSkipped;

    if(m_isAborted) return false;

    if(m_isSkipped) return true;

    if ( value >= m_max )
    {
        m_finished = true;
        #ifdef __WXMSW__
        EnableCloseButton();
        #endif
        if( !( m_style & wxPD_AUTO_HIDE) )
        {
            wxYieldIfNeeded() ;
            (void)ShowModal();
        }
        else
        {
            ReenableOtherWindows();
            Hide();
        }
    }

    return true;
}

bool bcProgressDialog::Pulse(const wxString& newmsg, bool *skip)
{
    wxYieldIfNeeded();

    m_bar->SetStyleFlag( BCPBS_INFINITY );

    if(!m_bar->IsPlay()) m_bar->StartAnim();

    if(newmsg != wxEmptyString) m_msgLabel->SetLabel(newmsg);

    UpdateTimes();

    if(skip) *skip = m_isSkipped;

    if(m_isAborted) return false;

    if(m_isSkipped) return true;

    return true;
}

void bcProgressDialog::ReenableOtherWindows()
{
    if ( m_style & wxPD_APP_MODAL )
    {
        delete m_winDisabler;
        m_winDisabler = (wxWindowDisabler *) NULL;
    }
    else
    {
        if ( m_parentTop )
        {
            m_parentTop->Enable();
        }

    }
}
