#include "bcWrapper.h"


#include "BitmapCanvasWidgetBase.h"
#include "BitmapCanvasParentBase.h"
#include "../wxVariantHelper.h"

IMPLEMENT_DYNAMIC_CLASS(bcWrapper, wxWindow);

bcWrapper::bcWrapper(BitmapCanvasWidgetBase* widget)
{
    //ctor
    m_widget = widget;

    m_widget->SetWindowWrapper(this);
}

bcWrapper::~bcWrapper()
{
    //dtor
}

void bcWrapper::DoGetPosition( int *x, int *y ) const
{
    wxPoint p = m_widget->GetPosition().Ceil();

    if(x) *x = p.x;
    if(y) *y = p.y;
}

void bcWrapper::DoGetSize( int *width, int *height ) const
{
    wxSize s = m_widget->GetSize().Ceil();

    if(width) *width = s.x;
    if(height) *height = s.y;
}

/// TODO: implement "sizeFlags" ( note: the DoMoveWindow func calls DoSetSize )
void bcWrapper::DoSetSize(int x, int y, int width, int height, int sizeFlags)
{
    wxPoint pos = m_widget->GetPosition().Ceil();
    wxSize size = m_widget->GetSize().Ceil();

    BitmapCanvasParentBase* parent = m_widget->GetParent();

    parent->AddRefreshRect(m_widget->GetRect());

    m_widget->SetProperty( BCWB_SIZE, wxVariantHelper::ToVariant(wxRealSize(width,height)), false );
    m_widget->SetProperty( BCWB_POSITION, wxVariantHelper::ToVariant(wxRealPointEx(x,y)), false );

    bool needredraw = false;
    bool needrefresh = false;

    if(pos.x != x || pos.y != y) needrefresh = true;

    if(size.x != width || size.y != height) {
        needrefresh = true;
        needredraw = true;
    }

    if(needrefresh)
        m_widget->Refresh(needredraw);
}

void bcWrapper::DoSetClientSize(int width, int height)
{
    wxSize size = m_widget->GetSize().Ceil();

    BitmapCanvasParentBase* parent = m_widget->GetParent();

    parent->AddRefreshRect(m_widget->GetRect());

    m_widget->SetProperty( BCWB_SIZE, wxVariantHelper::ToVariant(wxRealSize(width,height)), false );

    bool needredraw = false;

    if(size.x != width || size.y != height) needredraw = true;

    if(needredraw)
        m_widget->Refresh(true);

}

void bcWrapper::DoMoveWindow(int x, int y, int width, int height)
{
    DoSetSize(x,y,width,height);
}

bool bcWrapper::Show( bool show )
{
    m_widget->Show(show);
    return true;
}

bool bcWrapper::Enable( bool enable )
{
    m_widget->Enable(enable);
    return true;
}

bool bcWrapper::IsShown() const
{
    return m_widget->IsShown();
}
bool bcWrapper::IsEnabled() const
{
    return m_widget->IsEnabled();
}
