#ifndef GAUGEFRAME_H
#define GAUGEFRAME_H

#include "../common/types/wxVariantHash.h"

#include "../gaugedesc/GaugeProperties.h"
#include "../Observable.h"

class MTConfigData;
class DoubleHash;
class RTDataMgrFacade;
class RTDataInputSIPR;
class STDataMgrFacade;
class STDataModelSerial;
class STDataModelSerial;
class LogHistoryMgr;
class GaugeDescManager;
class GaugeGroupManager;
//class GaugeCarrierFrame;
class GaugeCarrierPanel;
// class UserDialogManager;
class UserMenuRefresher;
class FirmwareInfo;
class SymbolTable;
class VemsDeviceController;
class InputEvent;
class ObserverBase;
class SIPRFileLogger;
class SymbolMgr;
class AIMDeviceController;

class GaugeFrame : public wxFrame, public Observable
{
    public:
        GaugeFrame( wxWindow *win, MTConfigData * mtconfigdata, VemsDeviceController* controller, wxArrayString& requiredAddrs, const wxString& layoutname = _T("gauges.xml") );
        GaugeFrame( wxWindow *win, MTConfigData * mtconfigdata, const wxString& layoutname = _T("aimgauges.xml") );
        GaugeFrame( wxWindow* win, const wxString& layoutname = _T("gauges.xml") );
        ~GaugeFrame();

    public:
        void    setOnlineMode( bool online );
        void    loadLogFile( const wxString& filename );

        void    refreshIdx( int i );
        void    saveGauges( const wxString& layoutname = _T("") );

    private:
        bool                    m_aimmode;
        wxString                m_layoutname;
        wxWindow*               m_parent;
        bool                    m_onlineMode;

        SymbolMgr*              m_symbolMgr;

        MTConfigData*           m_mtConfigData;
        DoubleHash*             m_rtViewerModel;
        RTDataMgrFacade*        m_rtMgr;
        RTDataInputSIPR*        m_inputSIPR;
        STDataMgrFacade*        m_stMgr;
        STDataModelSerial*      m_myModelSerial;
        STDataModelSerial*      m_deviceModel;
        LogHistoryMgr*          m_logHistoryMgr;
        GaugeDescManager*       m_gaugeDescMgr;
        GaugeGroupManager *     m_gaugeGroupManager;
        //GaugeCarrierFrame*      m_carrier;
        GaugeCarrierPanel*      m_carrier;

        GaugePropertiesArray    m_gprops;
        SymbolTable*            m_symbolTable;

        VemsDeviceController*   m_controller;

        bool                    m_evtDisconnected;

        SIPRFileLogger*         m_siprFileLogger;

        wxStopWatch             m_sw;
        unsigned int            m_packetCounter;

        wxMenu*                 m_popupmenu;

    private:
        // handling gauges layout
        void        loadGaugesLayout();
        void        saveGaugesLayout( wxString layoutname = _T("") );

        // initialization
        void        init();
        void        initGauges();

        //
        void        addGauges();
        void        checkEnginetSection();
        void        checkETimeSection();

        void        onSIPREvent( InputEvent& siprevent );

        void        onClose( wxCloseEvent& event );

        void        onMouseRightClick( wxMouseEvent& event );
        void        onMenu( wxCommandEvent& event );

        void        startLog();
        void        stopLog();

        void        updateStatusBar();

        DECLARE_EVENT_TABLE()
};

#endif // GAUGEFRAME_H
