#include "ParserHelper.h"
#include "../mtconfigdata.h"
#include "../Math.h"

#include "SimpleIniParser.h"
#include "../fileparser/ExtendedHeaderParser.h"
#include "../fileparser/MapFileParser.h"
#include "../fileparser/V3iniParser.h"
#include "../common/types/wxVariantHashHash.h"

ParserHelper::ParserHelper( const wxString& firmwareDir, MTConfigData* mtConfigData )
{
    m_firmwareDir = firmwareDir;
    m_mtConfigData = mtConfigData;

    m_symbolMgr = &(m_mtConfigData->getSymbolMgr());
    m_symbolTable = &(m_mtConfigData->getSymbolTable());
}

ParserHelper::~ParserHelper()
{
    //dtor
}

void ParserHelper::doParsing()
{
    wxString fname;

    // ini
    fname = m_firmwareDir + _T("\\my_make.16x14\\extras.ini");

    // parseIni( fname );

    fname = m_firmwareDir + _T("\\my_make.16x14\\vems.map");

    if ( !parseMap( fname ) )
        LOGP( _T("MAP file parse error!") )

    fname = m_firmwareDir + _T("\\my_make.16x14\\global.h");

    parseGlobalH( fname );

    fname = m_firmwareDir + _T("\\my_make.16x14\\eventqueue.h");

    parseEventqueueH( fname );

    // add engine_t symbols
    checkEnginetSection();

    // add etime_t symbols
    checkETimeSection();

}

void ParserHelper::parseIni( const wxString& fname )
{
    // parse additional ini
    /// TODO
    m_symbolTable = &(m_mtConfigData->getSymbolTable());
    SimpleIniParser parser( m_symbolTable );
    LOGP( _T("Parsing extra INI file: ") + fname )
    parser.parse( fname );
}

bool ParserHelper::parseMap( const wxString& fname )
{
    // parse map file
    LOGP( _T("Parsing MAP file..."))
    MapFileParser mparser;
    //fname = DirHelper::GetPath(dPROGRAMROOT) + _T("commmodel\\") + fwStr + _T("\\my_make.16x14\\vems.map");
    LOGP( _T("MAP file: ") + fname )
    mparser.parse( fname  );
    LOGP( _T("Parsing MAP file finished"))

    // get engine address
    MapSymbolHash mapHash = mparser.getMapSymbols();
    MapSymbolHash::iterator it = mapHash.find( _T("engine") );
    if ( it == mapHash.end() )
    {
        LOGP( _T("Engine struct not found in map file!") )
        return false;
    }
    else
    {
        m_engineAddr = it->second.address;
    }

    // get etime address
    it = mapHash.find( _T("etime") );
    if ( it == mapHash.end() )
    {
        LOGP( _T("Etime struct not found in map ") )
        return false;
    }
    else
    {
        m_etimeAddr = it->second.address;
    }

    // get config & config_t address
    it = mapHash.find( _T("config") );
    if ( it == mapHash.end() )
    {
        LOGP( _T("config struct not found in map ") )
        return false;
    }
    else
    {
        m_cfgAddr = it->second.address;
    }
    it = mapHash.find( _T("config_ext") );
    if ( it == mapHash.end() )
    {
        LOGP( _T("config_ext struct not found in map ") )
        return false;
    }
    else
    {
        m_cfgExtAddr = it->second.address;
    }
    return true;
}

void ParserHelper::parseGlobalH( const wxString& fname )
{
    // parse global.h & mymake
    LOGP( _T("Parsing global.h..."))

    wxVariantHashHash sections;

    // get engine_t
    wxVariantHash sect;
    sect[_T("page")] = 255L;
    sect[_T("prefix")] = true;
    sect[_T("address")] = (long) m_engineAddr;
    sections[_T("engine_t")] = sect;

    /* config_t + config_ext_t - if needed */
    /*
    sect[_T("page")] = 5L;
    sect[_T("prefix")] = true;
    sect[_T("address")] = (long) m_cfgAddr;
    sections[_T("config_t")] = sect;

    sect[_T("page")] = 9L;
    sect[_T("prefix")] = true;
    sect[_T("address")] = (long) m_cfgExtAddr;
    sections[_T("config_ext_t")] = sect;
    */

    ExtendedHeaderParser globalhparser( m_symbolMgr, sections, true );

    globalhparser.parse( fname );

    LOGP( _T("Parsing global.h finished"))
}

void ParserHelper::parseEventqueueH( const wxString& fname )
{
    LOGP( _T("Parsing eventqueue.h...") )

    wxVariantHashHash sections;

    wxVariantHash sect;

    sect[_T("page")] = 255L;
    sect[_T("prefix")] = true;
    sect[_T("address")] = (long) m_etimeAddr;
    sections[_T("etime_t")] = sect;

    ExtendedHeaderParser eqhparser( m_symbolMgr, sections, true );

    eqhparser.parse( fname );

    LOGP( _T("Parsing eventqueue.h finished") )
}

void ParserHelper::checkETimeSection()
{
    Logger::logf( LOG_MSG, _T("etime addr = %x"), m_etimeAddr );

    SymbolTable::iterator it = m_symbolTable->begin();

    wxArrayString addr; IntArray reqAddr;

    for ( ; it != m_symbolTable->end(); ++it )
    {
        if ( it -> first.StartsWith( _T("etime_t") ) )
        {
            Symbol * s = it->second;
            long symaddress = s->getOffset();

            wxString hexaddr = wxString::Format( _T("%04x"), symaddress );
            hexaddr = hexaddr.Mid( 1, 2 );

            if ( addr.Index(hexaddr) == wxNOT_FOUND )
            {
                addr.Add( hexaddr );
            }
        }
    }

    addr.Sort();

    for ( size_t i = 0; i < addr.GetCount(); ++i )
    {
        int iAddr = Math::hex2int( addr[i] );
        if ( reqAddr.Index( iAddr - 1 ) == wxNOT_FOUND && reqAddr.Index( iAddr + 1 ) )
            reqAddr.Add( iAddr );
    }

    for ( size_t i = 0; i < reqAddr.GetCount(); ++i )
    {
        wxString str = wxString::Format( _T("%02x"), reqAddr[i] );
        m_requiredAddrs.Add( str );
        Logger::logf(LOG_MSG, _T("required mdx addresses:")+str );

    }

    Logger::logf(LOG_MSG, _T("checkEventQsection, Total offs=%d, m_req.count()=%d"), addr.GetCount(), m_requiredAddrs.GetCount() );
    m_requiredAddrs.Sort();
}

void ParserHelper::checkEnginetSection()
{
    m_requiredAddrs.Clear();

    Logger::logf(LOG_MSG, _T("engine adr=%x"), m_engineAddr);

    SymbolTable::iterator it = m_symbolTable->begin();

    wxArrayString addr; IntArray reqAddr;

    for ( ; it != m_symbolTable->end(); ++it )
    {
        if ( it->first.StartsWith(_T("engine_t") ) )
        {
            Symbol* s = it->second;
            long symaddress = s->getOffset();// m_engineAddr;

            wxString hexaddr = wxString::Format(_T("%04x"), symaddress );
            hexaddr = hexaddr.Mid(1,2);

            if ( addr.Index(hexaddr) == wxNOT_FOUND )
            {
                addr.Add( hexaddr );
            }
        }
    }

    addr.Sort();

    for ( size_t i = 0; i < addr.GetCount(); ++i )
    {
        int iAddr = Math::hex2int( addr[i] );
        if ( reqAddr.Index( iAddr - 1 ) == wxNOT_FOUND && reqAddr.Index( iAddr + 1 ) )
            reqAddr.Add( iAddr );
    }

    for ( size_t i = 0; i < reqAddr.GetCount(); ++i )
    {
        wxString str = wxString::Format( _T("%02x"), reqAddr[i] );
        m_requiredAddrs.Add( str );
    }

    Logger::logf(LOG_MSG, _T("checkEnginetSection, Total offs=%d, m_req.count()=%d"), addr.GetCount(), m_requiredAddrs.GetCount() );
    m_requiredAddrs.Sort();
}
