#include "HelpUtil.h"

#include "../DisplayHelper.h"

#include <wx/artprov.h>
#include <wx/html/helpctrl.h>

wxHtmlHelpController* HelpUtil::s_dialogHelpController  = new wxHtmlHelpController( wxHF_DEFAULT_STYLE | wxHF_EMBEDDED );

wxHtmlHelpController* HelpUtil::s_mainHelpController    = new wxHtmlHelpController( wxHF_DEFAULT_STYLE | wxHF_OPEN_FILES );

bool HelpUtil::m_helpshown = false;

void HelpUtil::addBook( const wxString& book )
{
    s_mainHelpController->AddBook( book );
    s_dialogHelpController->AddBook( book );
}

void HelpUtil::addBookToDialog( const wxString& book )
{
    s_dialogHelpController->AddBook( book );
}

void HelpUtil::addBookToHelp( const wxString& book )
{
    s_mainHelpController->AddBook( book );
}

void HelpUtil::showHelp( const wxString& helpTitle )
{
    s_mainHelpController->Display( helpTitle );
}

bool HelpUtil::showDialogHelp( wxWindow* dialog, const wxString& helpTitle )
{
    wxString title = dialog->GetLabel() + _T(" Help");

    wxFrame* helpWindow = (wxFrame*)wxWindow::FindWindowByLabel( title, dialog );

    if ( helpWindow )
    {
        helpWindow->Disconnect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( HelpUtil::OnHelpWinClose ) );
        helpWindow->Destroy();
        m_helpshown = false;
        return false;
    }

    wxRect rect = calculateHelpWindowRect( dialog->GetRect() );

    helpWindow = new wxFrame( dialog, wxID_ANY, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT );


    helpWindow->SetIcon( wxArtProvider::GetIcon( wxART_QUESTION ) );
    helpWindow->SetSize( rect );

    wxHtmlHelpWindow* htmlHelpWindow = new wxHtmlHelpWindow(    helpWindow, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                                                                wxTAB_TRAVERSAL| wxNO_BORDER, wxHF_OPEN_FILES );

    s_dialogHelpController->SetHelpWindow( htmlHelpWindow );

    s_dialogHelpController->Display( helpTitle );

    helpWindow->Show();

    helpWindow->Connect( wxEVT_CLOSE_WINDOW, wxCloseEventHandler( HelpUtil::OnHelpWinClose ) );

    dialog->SetFocus();

    m_helpshown = true;
    return true;
}


//wxRect HelpUtil::calculateHelpWindowRect( const wxRect& parentRect )
//{
//    wxRect rect = parentRect;
//
//    rect.x += parentRect.width;
//
//    wxRect screenRect( DisplayHelper::GetDisplaySize() );
//
//    if ( rect.GetRight() <= screenRect.GetRight() ) return rect;
//
//    rect.x = parentRect.x - parentRect.width;
//
//    if ( rect.GetLeft() >= screenRect.GetLeft() ) return rect;
//
//    rect.x = parentRect.x + parentRect.width;
//
//    return rect;
//}

wxRect HelpUtil::calculateHelpWindowRect( const wxRect& parentRect )
{
    wxRect rect = parentRect;
    wxRect screenRect( DisplayHelper::GetDisplaySize() );
    wxSize minSize( screenRect.width / 3, screenRect.height / 2 );

    /// check min size
    if ( rect.width < minSize.GetWidth() ) {
        rect.width = minSize.GetWidth();
    }

    if ( rect.height < minSize.GetHeight() ) {
        rect.height = minSize.GetHeight();
    }

    /// set x
    rect.x += parentRect.width;
    // doesn't fit right ( default )
    if ( rect.GetRight() > screenRect.GetRight() )  {
        rect.x = parentRect.x - rect.width;
    }

    // doesn't fit left
    if ( rect.GetLeft() < screenRect.GetLeft() ) {
        rect.x = parentRect.x + 10;
    }

    // dosen't fit screen, show on center
    if ( ( rect.GetRight() > screenRect.GetRight() ) || ( rect.GetLeft() < screenRect.GetLeft() ) ) {
        rect.x = ( screenRect.width - rect.width ) / 2 ;
    }

    /// set y
    // dosen't fit screen, show on center
    if ( ( rect.GetTop() < screenRect.GetTop() ) || ( rect.GetBottom() > screenRect.GetBottom() ) ) {
        rect.y = ( screenRect.height - rect.height ) / 2 ;
    }

    return rect;
}


void HelpUtil::OnHelpWinClose( wxCloseEvent& e )
{
    m_helpshown = false;
    e.Skip();
}

