#include "MenuLinkHelper.h"

#include "../mtconfigdata.h"
#include "../expression/RPNParser.h"
#include "../logger.h"

MenuLinkHelper::MenuLinkHelper()
{
    //ctor
}

MenuLinkHelper::~MenuLinkHelper()
{
    //dtor
}

wxString MenuLinkHelper::getLink(const wxString& href, const wxString& name, const wxString& symbolToMark )
{
    wxArrayString strA;

    if ( symbolToMark != wxEmptyString )
    {
        strA.Add( symbolToMark );
}

    return getLink( href, name, strA );
}

wxString MenuLinkHelper::getLink(const wxString& href, const wxString& name, const wxArrayString& symbolsToMark )
{
    wxString target = wxEmptyString;

    size_t symNo = symbolsToMark.GetCount();
    if ( symNo > 0 )
    {
        for ( size_t i = 0; i < symNo; ++i )
        {
            target += symbolsToMark[i] + _T(",");
        }
        target.RemoveLast( 1 );
    }

    wxString link = _T("<a href=\"")+href+ _T("\"");

    if ( !target.IsEmpty() )
        link += _T(" target=\"") + target + _T("\"");

    link += _T(">")+ name +_T("</a>");
    return link;
}

wxString MenuLinkHelper::getHtmlTable( const wxArrayString& missingScalar, MTConfigData& mtConfigData)
{
    wxString menuLinks;
    for(size_t i = 0; i < missingScalar.GetCount(); ++i){
        wxString symbolName = missingScalar.Item(i);
        wxString menuLink = MenuLinkHelper::getHtmlFromVariable(mtConfigData, symbolName);
        //Logger::logf(LOG_MSG, _T("MenuLinkHelper::getHtmlTable() ") + symbolName +_T(": ")+ menuLink);
        //m_uploadDetails.Append( menuLink);
        menuLinks += menuLink;
    }
    return menuLinks;
}

wxString MenuLinkHelper::getHtmlFromVariable(MTConfigData& mtconfigData, const wxString& variable)
{
    wxString htmlMsg;

    //SymbolTable& cfgSymbolTable = mtconfigData.getCfgSymbolTable();
    // size_t size = cfgSymbolTable.size();
    Symbol* symbol = mtconfigData.getCfgSymbol(variable);
    if(!symbol){
        Logger::logf( LOG_MSG, _T("MenuLinkHelper::show() symbol NOT found") );
        //htmlMsg +=wxString::Format(_T("VariableName not found: %d <br>"),size);
        return wxEmptyString;
    }

    Logger::logf( LOG_MSG, _T("MenuLinkHelper::show() symbol found: ") + symbol->getName() );
    const StringSet* dialogNames = mtconfigData.getDialogNamesBySymbol(*symbol);
    if(!dialogNames){
        Logger::logf( LOG_MSG, _T("MenuLinkHelper::show() dialog names NOT found for symbol: ")+ symbol->getName() );
        //htmlMsg +=wxString::Format(_T("Dialog name not found:<br>"));
        return wxEmptyString;
    }
    StringSet::const_iterator dialogIT = dialogNames->begin();
    // size_t dialogSize = dialogNames->size();
    //if(dialogSize > 0){
    //    htmlMsg +=_T("DialogName: ");
    //}
    htmlMsg += _T("Please check <b>")+ variable +_T("</b> in menu(s):<br>");
    for(; dialogIT != dialogNames->end(); ++dialogIT )
    {
        wxString dialogId = *dialogIT;
        Logger::logf( LOG_MSG, _T("MenuLinkHelper::show() dialogName: ")+ dialogId );
        //htmlMsg += dialogId + _T(",");
        wxString dialogName = mtconfigData.getMenuAndDialogNameByDialogId(dialogId);
        htmlMsg += getLink(dialogId, dialogName, symbol->getName() );
        htmlMsg +=_T("<br>");
        // do something useful with key
    }
    htmlMsg +=_T("<br>");
    return htmlMsg;
}

wxString MenuLinkHelper::getHtmlFromExpression(MTConfigData& mtconfigData, const wxString& expression)
{
    RPNParser rpn;
    StringHashSet variablesSet = rpn.getVariables(expression);
    StringHashSet::iterator it;
    wxString htmlMsg;
    for( it = variablesSet.begin(); it != variablesSet.end(); ++it )
    {
        wxString variable = *it;
        htmlMsg += getHtmlFromVariable(mtconfigData, variable);

    }
    return htmlMsg;
}
