#ifndef VEMSMATRIXHELPER_H
#define VEMSMATRIXHELPER_H

#include "VemsMatrix.h"
#include "../logger.h"

struct MatrixSelection
{
    int top;
    int left;
    int bottom;
    int right;
/*
    MatrixSelection( size_t top, size_t left, size_t bottom, size_t right )
    {
        this->top    = top;
        this->left   = left;
        this->bottom = bottom;
        this->right  = right;
    }
    */
};

class VemsMatrixHelper
{
    public:
        static void print( const VemsMatrix<uint8_t>& m );
        static void print( const VemsMatrix<double>& m );
        /// convert raw data to user data
        /// requirement : matrices size match
        static bool convert( const VemsMatrix<uint8_t>& from, VemsMatrix<double>& to, double scale, double translate );
        static bool convert( const VemsMatrix<double>& from, VemsMatrix<uint8_t>& to, double scale, double translate );
        static bool convert( const VemsMatrix<uint8_t>& from, VemsMatrix<double>& to, const wxString& expr );
        static bool convert( const VemsMatrix<int>& from, VemsMatrix<double>& to );
        static bool convertLambda( VemsMatrix<uint8_t>& m );
        static void interpolate(    VemsMatrix<double>* data,
                                    const VemsMatrix<double>& row,
                                    const VemsMatrix<double>& col,
                                    const MatrixSelection& s );

        static double interpolate( const VemsMatrix<double>& row, const VemsMatrix<double>& col, double x );

        static bool resample1D( const VemsMatrix<double>& oldX, const VemsMatrix<double>& newX, VemsMatrix<double>* table );
        static bool resample1D(  const VemsMatrix<uint8_t>& oldX, const VemsMatrix<uint8_t>& newX, VemsMatrix<uint8_t>* table );

        static void interpolate_col(    VemsMatrix<double>* data,
                                        const VemsMatrix<int>& cols,
                                        size_t col );

        static void interpolate_row(    VemsMatrix<double>* data,
                                        const VemsMatrix<int>& rows,
                                        size_t row );
        static double interpolate3D(    const VemsMatrix<double>& col,
                                        const VemsMatrix<double>& row,
                                        const VemsMatrix<double>& table,
                                        double x, double y );

        template <class T>
        static bool resize( const VemsMatrix<T>& from, VemsMatrix<T>& to );
        static bool resample2D( const VemsMatrix<double>& oldX,
                                const VemsMatrix<double>& oldY,
                                const VemsMatrix<double>& newX,
                                const VemsMatrix<double>& newY,
                                VemsMatrix<double>* table );
        static bool resample2D( const VemsMatrix<uint8_t>& oldX,
                                const VemsMatrix<uint8_t>& oldY,
                                const VemsMatrix<uint8_t>& newX,
                                const VemsMatrix<uint8_t>& newY,
                                VemsMatrix<uint8_t>* table );

    private :
        ~VemsMatrixHelper() {}

        static double getLeftNeighbour( double x, const VemsMatrix<double>& oldA, int& oldIdx );
        static double getRightNeighbour( double x, const VemsMatrix<double>& oldA, int& oldIdx );
};

///-------------------------------------------------------------------template functions
/**
*/
template <class T>
bool VemsMatrixHelper::resize( const VemsMatrix<T>& from, VemsMatrix<T>& to )
{
    // if sizes match then copy data
    if ( from.size() == to.size() ) // copy?
    {
        for ( size_t i = 0; i < from.size(); ++i )
        {
            to.set( i, from.get(i) );
//            Logger::logf(   LOG_MSG, _T("VemsMatrixHelper::resize() - idx: %d, from: %f, to: %f"),
//                            i, from.get(i), to.get(i) );
        }
    }
    // if from lesser than to
    else if ( from.size() < to.size() )
    {
        for ( size_t i = 0; i < from.rows(); ++i )
        {
            for ( size_t j = 0; j < from.cols(); ++j )
            {
                to.set( i, j, from.get( i, j ) );
            }
        }
    }
    // if from greater than to
    else
    {
        double deltaCol = (double)from.cols() / (double)to.cols();
        double deltaRow = (double)from.rows() / (double)to.rows();

        double rowIdx = 0.0;
        double colIdx = 0.0;

        for ( size_t i = 0; i < to.rows(); ++i )
        {
            rowIdx = i * deltaRow;

            if ( i == to.rows() - 1 ) { rowIdx = from.rows() - 1; }

            for ( size_t j = 0; j < to.cols(); ++j )
            {
                colIdx = j * deltaCol;
                if ( j == to.cols() - 1 ) { colIdx = from.cols() - 1; }

                to.set( i, j, from.get( (size_t)rowIdx, (size_t)colIdx ) );
            }
        }
    }

    return true;
}


#endif // VEMSMATRIXHELPER_H
