#ifndef COMMUNICATIONCONTROLLER2_H
#define COMMUNICATIONCONTROLLER2_H


#include "../common/ProcessBase.h"
#include "../common/typedef.h"

#include "CommunicationController2BufferSizes.h"
#include <wx/timer.h>


class CommunicationBase;
class CommunicationControllerClient;
class CommunicationControllerErrorHandler;

class CommunicationController2 : public ProcessBase
{
    public:
        enum ErrorCode { OK=0, READ_ERROR, WRITE_ERROR, CANNOT_OPEN_PORT, UNKNOWN_ERROR };

    public:
        CommunicationController2();
        CommunicationController2( wxEvtHandler* eventHandler, CommunicationControllerClient* client, CommunicationBase* commBase );
        virtual ~CommunicationController2();

        /// call only if thread isn't running
        void registerClient( CommunicationControllerClient* client );
        void setCommunicationBase( CommunicationBase* commBase );
        void setEventHandler( wxEvtHandler* eventHandler );

        bool process();
        void start( int priority = WXTHREAD_DEFAULT_PRIORITY );
        void stop();
        void reset();

        bool error() const;
        ErrorCode lastError() const;

        void clear( size_t readCount, size_t sleepBeforeClear );

    private:
        CommunicationControllerErrorHandler*    m_errorHandler;
        CommunicationControllerClient*          m_client;
        CommunicationBase*                      m_commBase;
        wxEvtHandler*                           m_eventHandler;

        uint8_t                         m_rxBuff[RX_BUFFER_SIZE];
        uint8_t                         m_txBuff[TX_BUFFER_SIZE];


        ErrorCode   m_errorCode;
        bool        m_isStopped;
        size_t      m_sleepBeforeClearing;
        size_t      m_clearingReadCount;

        void finished();

        friend class CommunicationControllerErrorHandler;
};

class CommunicationControllerErrorHandler : public wxEvtHandler
{
    public:
        CommunicationControllerErrorHandler( CommunicationController2* parent );
        virtual ~CommunicationControllerErrorHandler();

        /// handled in the CommunicationController2 thread
        /// send CommEvent, try to close CommunicationBase, send ControllerError event
        void handleError();

    private:
        CommunicationController2*   m_parent;
        wxTimer                     m_errorTimer;
        bool                        m_isPortClosed;

        /// handled in the MainThread
        void OnClosingPort( wxCommandEvent& event );
        void OnPortClosed( wxCommandEvent& event );
        void OnErrorTimer( wxTimerEvent& event );
};

#endif // COMMUNICATIONCONTROLLER2_H
