#ifndef COMMUNICATIONCONTROLLERCLIENT_H
#define COMMUNICATIONCONTROLLERCLIENT_H


#include "CommunicationTimer.h"
#include "../common/typedef.h"
#include "../vemsprotocol/CommandRequestBase.h"


class CommunicationControllerClient : public CommandRequestBase
{
    public:
        CommunicationControllerClient()
        {
            m_baseTimeout = 0;
            m_timeout = 0;
            m_interWriteDelay = 0;
            m_sleep = 1;
//            m_isTimedOut = false;
        }

        virtual ~CommunicationControllerClient() {}

        /** CommunicationController call this method first
            in process()
            if it returns false thred stop
        */
        virtual bool startProcess() { return true; }

        virtual void write( uint8_t* buffer, int& size ) = 0;
        virtual void read( uint8_t* buffer, int size ) = 0;
        virtual void finish() = 0;
        virtual void request( VemsCommand* command ) {}
        virtual void reset() {}

        void setTimeout( size_t timeout )
        {
            m_timeout = timeout;
        }

        void setBaseTimeout( size_t timeout )
        {
            m_baseTimeout = timeout;
        }

        void setInterWriteDelay( size_t interWriteDelay )
        {
            m_interWriteDelay = interWriteDelay;
        }

        virtual bool isFinished() = 0;

        //virtual size_t getInterWriteDelay() = 0;

        virtual size_t getInterWriteDelay()
        {
            return m_interWriteDelay;
        };

        size_t getBaseTimeout() const
        {
            return m_baseTimeout;
        };

        virtual size_t getTimeout() const
        {
            return m_timeout;
        };

        virtual void setSleepTime( size_t t )
        {
            m_sleep = t;
        }

        virtual size_t getSleepTime() const { return m_sleep; }


    protected:
        size_t m_baseTimeout;
        size_t m_timeout;
        size_t m_interWriteDelay;
        size_t m_sleep;

        CommunicationTimer  m_timeoutTimer;
        CommunicationTimer  m_clearingTimer;

        wxDateTime  m_startTime;
};

#endif // COMMUNICATIONCONTROLLERCLIENT_H
