#include "CommunicationLogger.h"
#include "../logger.h"

int idLogTimer = wxNewId();

CommunicationLogger::CommunicationLogger( CommunicationBase* commBase, const wxString& logFileName )
    : m_timer ( this, idLogTimer )
{
    m_commBase      = commBase;
    m_logFileName   = logFileName;
    m_logFile       = new wxTextFile( m_logFileName );

    if ( m_logFile->Exists() ) {
        m_logFile->Open();
        //m_logFile->Clear();
    } else {
        m_logFile->Create();
    }

    m_maxLineLength = 128;

    Connect( idLogTimer, wxEVT_TIMER, wxTimerEventHandler( CommunicationLogger::OnTimer ), NULL, this );
//    m_timer.Start(100);
}

CommunicationLogger::~CommunicationLogger()
{
    delete m_commBase;

    if ( m_logFile ) {
        LOG
        m_logFile->Write();
        m_logFile->Close();
        delete m_logFile;
    }
}

int CommunicationLogger::SendString(const wxString& str)
{
    return m_commBase->SendString( str );
}

int CommunicationLogger::SendString(const uint8_t* chr)
{
    return m_commBase->SendString( chr );
}

int CommunicationLogger::SendString(const uint8_t* str, size_t len)
{
    if ( !m_readBuffer.IsEmpty() ) {
        addReadBuffer();
    }

    wxString sentData ;

    for( size_t i = 0; i < len; ++i ) {
        sentData += getLogString( str[i] );
    }

    m_logFile->AddLine( _T("[W] - ") + sentData );
    m_logFile->AddLine( _T("") );

    return m_commBase->SendString( str,len );
}

int CommunicationLogger::sendStringBlock(uint8_t* value, size_t len, size_t blockLen)
{
    return m_commBase->sendStringBlock( value, len, blockLen );
}

/**

*/
int CommunicationLogger::Read(uint8_t* str, size_t len)
{
    int receivedBytes = m_commBase->Read( str, len );

    wxString readData ;

    for( size_t i = 0; i < len; ++i ) {
        readData = getLogString( str[i] );

        if ( ( m_readBuffer.Length() + readData.Length() ) < m_maxLineLength ) {
            m_readBuffer += readData;
        } else {
            m_logFile->AddLine( _T("[R] - ") + m_readBuffer );
            m_logFile->AddLine( _T("") );
            m_readBuffer.Clear();
        }
    }

    return receivedBytes;
}

int CommunicationLogger::ReadString(uint8_t* str, size_t len)
{
    return m_commBase->ReadString( str, len );
}

int CommunicationLogger::ReadString(uint8_t* data, size_t len, int maxTryCount)
{
    return m_commBase->ReadString( data, len, maxTryCount );
}

void CommunicationLogger::setBaudRate( int baudRate )
{
    return m_commBase->setBaudRate( baudRate );
}

int CommunicationLogger::getBaudRate()
{
    return m_commBase->getBaudRate();
}

void CommunicationLogger::setPortName( const wxString& portname )
{
    m_commBase->setPortName( portname );
}

wxString CommunicationLogger::getPortName() const
{
    return m_commBase->getPortName();
}

int CommunicationLogger::sendNumber(uint32_t number, int length)
{
    return m_commBase->sendNumber( number, length );
}

int CommunicationLogger::sendNumber(uint16_t number)
{
    return m_commBase->sendNumber( number );
}

int CommunicationLogger::open()
{
    return m_commBase->open();
}

void CommunicationLogger::close()
{
    m_logFile->AddLine( _T("[CLOSED]") );
    m_logFile->Write();
    m_commBase->close();
}

int CommunicationLogger::clear()
{
    return m_commBase->clear();
}

bool CommunicationLogger::IsOpen()
{
    return m_commBase->IsOpen();
}

void CommunicationLogger::addReadBuffer()
{
    m_logFile->AddLine( _T("[R] - ") + m_readBuffer );
    m_logFile->AddLine( _T("") );
    m_readBuffer.Clear();
}


wxString CommunicationLogger::getLogString( uint8_t ch )
{
    //non-printable ascii char
    if ( ch < 32 ) {
        return wxString::Format( _T("<%02X>"), ch );
    } else {
        return wxString::Format( _T("%c"), ch );
    }
}

void CommunicationLogger::OnTimer(wxTimerEvent& event)
{
    m_logFile->Write();
}

