#ifndef BAUDRATESELECTOR_H
#define BAUDRATESELECTOR_H


#include "DeviceModeSelectorListener.h"
#include "PortScannerListener.h"
#include "TimeoutListener.h"
#include "../CommunicationControllerClient.h"
#include "../../vemsprotocol/ECUInfo.h"
#include "../../common/types/IntArray.h"

class StringRecognizer;
class CommunicationBase;
class StringRecognizer;
class IntArray;
class BaudRateSelectorListener;
class TimeoutListener;


class BaudRateSelector : public CommunicationControllerClient
    , public DeviceModeSelectorListener
    , public PortScannerListener
    , public TimeoutListener
{
    public:
        BaudRateSelector(   BaudRateSelectorListener* listener,
                            TimeoutListener* timeoutListener,
                            CommunicationBase* commBase,
                            const IntArray& baudrates,
                            ECUInfo::EcuMode deviceMode,
                            size_t currentBaudrate,
                            size_t targetBaudrate );

        virtual ~BaudRateSelector();

        /// CommunicationControllerClient methods
        bool startProcess();
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

        /// DeviceModeSelectorListener methods
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );

        /// PortScannerListener methods
        void OnDeviceFound( PortScannerEvent& event );
        void OnDeviceNotFound();

        /// TimeoutListener methods
        void OnTimeout();

    private:
        enum State {
                        CLIENT_MODE,
                        SEND_CHANGE_BAUD_RATE,
                        WAIT_BAUD_RATE_CHANGED,
                        FINISHED,
                        TIMEDOUT, };

        enum { TIMEOUT = 1000 };

    private:
        BaudRateSelectorListener*   m_listener;
        TimeoutListener*            m_timeoutListener;
        CommunicationBase*  m_commBase;
        ECUInfo::EcuMode    m_deviceMode;
        size_t              m_currentBaudrate;
        size_t              m_targetBaudrate;

        State               m_state;

        size_t              m_timeoutCount;
        CommunicationControllerClient*  m_client;

        StringRecognizer* m_baudrateChangedRecognizer;

        bool m_isSelectManMode;
        IntArray m_baudrates;

    private:
        void sendClientMode( uint8_t* buffer, int& size );
        void waitClientMode( uint8_t* buffer, int size );
        void sendChangeBaudRate( uint8_t* buffer, int& size );
        void waitBaudRateChanged( uint8_t* buffer, int size );
};

#endif // BAUDRATESELECTOR_H
