#include "ConfigSaver2.h"
#include "../../vemsprotocol/SIPRTypes.h"
#include "../../vemsprotocol/SIPRPacket.h"
#include "../../events/InputEvent.h"
#include "../../events/CommunicationEvent.h"
#include "../../logger.h"

#include <wx/sstream.h>


ConfigSaver2::ConfigSaver2()
{
    m_pOutputStream     = NULL;
    m_pProgressDialog   = NULL;
    m_pFile             = NULL;
    m_strFileName       = _T("");
    m_strHeader         = _T("");
    m_bShowProgress     = true;
    m_iLineCount        = 0;
}

ConfigSaver2::~ConfigSaver2()
{
    close();
}

void ConfigSaver2::close()
{
    m_iLineCount = 0;

    if ( m_pOutputStream )
    {
        m_pOutputStream->Close();
        delete m_pOutputStream;
        m_pOutputStream = NULL;
    }

    if ( m_pFile )
    {
        m_pFile->Close();
        delete m_pFile;
        m_pFile = NULL;
    }

    if ( m_pProgressDialog )
    {
        m_pProgressDialog->Destroy();
        m_pProgressDialog = NULL;
    }
}

void ConfigSaver2::deleteFile()
{
    ::wxRemoveFile( m_strFileName );
}

void ConfigSaver2::write( uint8_t* data, uint8_t size )
{
    if ( !m_pOutputStream ) {
        /// TODO - handle error
        return;
    }

    if ( !m_pOutputStream->IsOk() ) {
        /// TODO - handle error
        return;
    }

    m_pOutputStream->Write( data, size );

    wxString eol = _T("\n");
    wxStringInputStream ss( eol );

    m_pOutputStream->Write( ss );
}


void ConfigSaver2::init( const wxString& fileName, const wxString& header  )
{
    m_strFileName   = fileName;
    m_strHeader     = header;

    m_pFile = new wxFile( m_strFileName, wxFile::write_append );

    if ( !m_pFile->IsOpened() ) {
        return;
    }

    m_pOutputStream = new wxFileOutputStream( *m_pFile );

    if ( !m_pOutputStream->IsOk() ) {
        return;
    }

    if ( !m_strHeader.IsEmpty() ) {
        m_pOutputStream->Write( m_strHeader.mb_str(), m_strHeader.Length() );
    }

    if ( m_bShowProgress )
    {
        m_pProgressDialog = new wxProgressDialog(   _T("Download config in progress!"),
                                                    _T("Line:"),
                                                    100,
                                                    NULL,
                                                    wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH |
                                                    wxPD_ELAPSED_TIME | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME );
                                                    //wxPD_AUTO_HIDE | wxPD_APP_MODAL );
    }
}


void ConfigSaver2::onInputEvent( InputEvent& event )
{
    SIPRPacket packet = event.getPacket();

    switch ( packet.type() )
    {
        case SIPR_CONFIG_LINE:
        {
            write( packet.data(), packet.size() - 1 );
            if ( m_pProgressDialog )
            {
                m_pProgressDialog->Pulse( wxString::Format( _T("Line: %d"), ++m_iLineCount ) );
            }
        } break;
        case SIPR_TABLES_END:
//        case SIPR_CONFIG_ERROR:
        {
            close();
            wxString msg = _T("Firmware config has been saved to: ") + m_strFileName;
            LOGP( msg );

        } break;
    }

    event.Skip();
}
