#ifndef FIRMWAREVERIFIER_H
#define FIRMWAREVERIFIER_H

#include "../CommunicationControllerClient.h"
#include "../../ntcfinder.h"

class FirmwareVerifierListener;
class TimeoutListener;
class FirmwareData;

class FirmwareVerifier : public CommunicationControllerClient
{
    public:
        FirmwareVerifier(   FirmwareVerifierListener* verifierListener,
                            TimeoutListener* timeoutListener,
                            FirmwareData* fwData );

        virtual ~FirmwareVerifier();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

    private:
        enum State {    SEND_PAGE_ADDRESS,
                        WAIT_PAGE_ADDRESS_OK,
                        SEND_GET_CHECKSUM_CMD,
                        WAIT_CHECKSUM,
                        FINISHED };

        enum { MAX_PAGE_ERROR_COUNT = 15 };
        enum { CHECKSUM_LENGTH = 4 };

        enum Result {   FIRMWARE_IDENTICAL,
                        FIRMWARE_DIFFERENT,
                        TIMEOUT };

        FirmwareVerifierListener*   m_verifierListener;
        TimeoutListener*            m_timeoutListener;
        FirmwareData*               m_firmwareData;

        State                       m_state;
        Result                      m_result;

//        size_t                      m_timeoutCount;
        size_t                      m_pageNum;
        size_t                      m_currentPage;
        size_t                      m_pageErrorCount;

        uint8_t                     m_rxBuffer[16];
        size_t                      m_rxDataLength;

        NTCFinder                   m_ntcfinder;

    private:
        void sendPageAddress( uint8_t* buffer, int& size );
        void waitPageAddressOk( uint8_t* buffer, int size );
        void sendGetChecksumCmd( uint8_t* buffer, int& size );
        void waitChecksum( uint8_t* buffer, int size );

        void sendFirmwareVerifierEvent( int eventType, const wxString& patchfilename = wxEmptyString );
};

#endif // FIRMWAREVERIFIER_H
