#ifndef MEGATUNESYNCER_H
#define MEGATUNESYNCER_H

#include <wx/wx.h>

#include "DeviceDetectorListener.h"
#include "../CommunicationControllerClient.h"
#include "../../FirmwareVersion.h"
#include "../../common/typedef.h"
#include "../../vemsprotocol/SIPRPacket.h"
#include "../../vemsprotocol/ECUInfo.h"
#include "../../triggerlog/TriggerFrameFSM.h"

class SIPRDescriptor;
class MegatuneConfig;
class MegatuneSyncerListener;
class TimeoutListener;

class MegatuneSyncer : public CommunicationControllerClient, public DeviceDetectorListener
{
    public:
        MegatuneSyncer( MegatuneSyncerListener* listener,
                        TimeoutListener* timeoutListener,
                        SIPRDescriptor* descriptor,
                        MegatuneConfig* config );

        virtual ~MegatuneSyncer();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();
        bool isTimedOut();

        /// DeviceDetectorListener methods
        void OnDeviceFound( ECUInfo::Device device, ECUInfo::EcuMode mode );
        void OnDeviceNotFound();

    private:
        enum State
        {
            REQUEST_RT,
            WAIT_RT,
            REQUEST_PAGE,
            WAIT_PAGE,
            CHECK_DEVICE,
            WAIT_CHECK_DEVICE,
            CLEAR,
            TIMEDOUT,
            FINISHED,
        };

        enum { TIMEOUT = 256 };
        enum { MAX_CLEAR_COUNT = 100 };
        enum { MS_RX_BUFFER_SIZE = 512 };

        const size_t RT_LEN_TRY_COUNT_MAX;
        const size_t CRC_CHECK_TRY_COUNT_MAX;

        uint8_t     m_rxBuffer[MS_RX_BUFFER_SIZE];
        int         m_rxDataLength;
        int         m_detectedLength;
        int         m_expectedRxSize;
        size_t      m_timeoutCount;
        size_t      m_clearCount;

        int         m_realtimeLength;
        wxString    m_realtimeCommand;

        size_t      m_currentPageIdx;
        int         m_currentPageSize;

        State       m_state;

        bool        m_isFirstRequest;

        size_t      m_detectCount;
        size_t      m_maxDetectCount;

        MegatuneSyncerListener*             m_listener;
        TimeoutListener*                    m_timeoutListener;
        SIPRDescriptor*                     m_descriptor;
        CommunicationControllerClient*      m_client;


        SIPRPacketList*     m_pages;
        MegatuneConfig*     m_config;
        int                 m_page;

        ECUInfo::Device     m_device;
        ECUInfo::EcuMode    m_deviceMode;

        size_t  m_rtLenTryCount;
        size_t  m_crcCheckTryCount;

        TriggerFrameFSMCRC    m_triggerFrameFSM;

        void requestRt( uint8_t* buffer, int& size );
        void waitRt( uint8_t* buffer, int size );
        void waitRtTriggerFrame( uint8_t* buffer, int size );
        void waitRtCrc( uint8_t* buffer, int size );
        void waitRtLength( uint8_t* buffer, int size );
        void waitRtLengthFirst( uint8_t* buffer, int size );
        void waitRtLengthSecond( uint8_t* buffer, int size );


        void requestPage( uint8_t* buffer, int& size );
        void waitPage( uint8_t* buffer, int size );
        void waitPageCrc( uint8_t* buffer, int size );
        void waitPageLength( uint8_t* buffer, int size );
        void waitPageFirst( uint8_t* buffer, int size );
        void waitPageSecond( uint8_t* buffer, int size );
        void checkDevice( uint8_t* buffer, int& size  );
        void waitCheckDevice( uint8_t* buffer, int size );
        void clear( uint8_t* buffer, int size );

        void sendPagePacket();
        bool checkRtCrc();
        bool isTriggerFrame();
        bool checkCrc( uint8_t* buffer, size_t length );

};

#endif // MEGATUNESYNCER_H
