#ifndef PASSWORDCHECKER_H
#define PASSWORDCHECKER_H

#include "../CommunicationControllerClient.h"

class PasswordCheckerListener;
class RegExRecognizer;
class StringRecognizer;

class PasswordChecker : public CommunicationControllerClient
{
    public:
        PasswordChecker( PasswordCheckerListener* listener );
        virtual ~PasswordChecker();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

    private:
        enum State { SEND_P, SEND_PW, WAIT_P, WAIT_PW, FINISHED };

        enum Result {   NO_PASSWORD_PROTECTION,
                        PASSWORD_ACCEPTED,
                        INVALID_PASSWORD,
                        PASSWORD_NOT_FOUND,
                        TIMEOUT };

        PasswordCheckerListener*    m_listener;
        State                       m_state;
        Result                      m_result;
        RegExRecognizer*            m_recognizerP;
        StringRecognizer*           m_recognizerPW;
        wxString                    m_password;

        size_t                      m_timeoutCount;

    private:
        void sendP( uint8_t* buffer, int& size );
        void sendPw( uint8_t* buffer, int& size );
        void waitP( uint8_t* buffer, int size );
        void waitPw( uint8_t* buffer, int size );

        wxString findPassword( const wxString& serialNumber );
};

#endif // PASSWORDCHECKER_H
