#ifndef PORTSCANNER_H
#define PORTSCANNER_H


#include "DeviceDetectorListener.h"
#include "../CommunicationControllerClient.h"
#include "../../common/types/IntArray.h"


class PortScannerListener;
class CommunicationBase;


class PortScanner : public CommunicationControllerClient
    , public DeviceDetectorListener
{
    public:
        PortScanner(    PortScannerListener* listener,
                        CommunicationBase* commBase,
                        const IntArray& baudrates,
                        size_t preferredBaudrate = 0 );

        virtual ~PortScanner();

        ///CommunicationControllerClient methods
        bool startProcess();
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();
        void reset();

        /// DeviceDetectorListener methods
        void OnDeviceFound( ECUInfo::Device device, ECUInfo::EcuMode deviceMode );
        void OnDeviceNotFound();

    private:
        enum State { START_PROCESS, WRITE, READ, DEVICE_FOUND, DEVICE_NOT_FOUND };

        CommunicationControllerClient*  m_commClient;

        PortScannerListener*    m_listener;
        CommunicationBase*      m_commBase;

        IntArray    m_baudrates;

        size_t      m_currBaudrateIdx;
        State       m_state;

        ECUInfo::Device     m_device;
        ECUInfo::EcuMode    m_mode;

    private:
        void registerDeviceDetector();
};

#endif // PORTSCANNER_H
