#include "RoundFirmwareInfoFetcher.h"
#include "FirmwareInfoFetcherListener.h"
#include "../../vemsprotocol/recognizers/ResponseRecognizer.h"
#include "../../vemsprotocol/recognizers/LengthRecognizer.h"
#include "../../vemsprotocol/recognizers/SerialRecognizer.h"
#include "../../vemsprotocol/recognizers/FirmwareVersionRecognizer.h"
#include "../../common/TxData.h"
#include "../../logger.h"
#include "../../FirmwareInfo.h"
#include "../../Math.h"
#include "../../FirmwareInfo.h"

RoundFirmwareInfoFetcher::RoundFirmwareInfoFetcher( FirmwareInfoFetcherListener* listener )
{
    m_listener      = listener;
    m_firmwareInfo  = new FirmwareInfo();;

    changeStateToClearing();
    m_nextState = SEND;
    m_isTimedout    = false;
    m_clearingTime  = 150;

    ///TODO
    size_t timeout = 0;
    timeout += TIMEOUT;

    LOGP( wxString::Format( _T("timeout: %d"), timeout ) );

    TxData* request = new TxData( _T("Man.") );
    LengthRecognizer* lengthRecognizer = new LengthRecognizer( 8 );
    ResponseRecognizer* responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("mci.") );
    //SerialRecognizer* serialRecognizer = new SerialRecognizer( 12 );
    lengthRecognizer = new LengthRecognizer( 12 );
    responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("bye.") );
    lengthRecognizer = new LengthRecognizer( 5 );
    responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("S.") );
    lengthRecognizer = new LengthRecognizer( 22 );
    responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    m_currentRecognizerIdx = 0;
}

RoundFirmwareInfoFetcher::~RoundFirmwareInfoFetcher()
{
    delete m_firmwareInfo;
	WX_CLEAR_ARRAY(m_recognizers);
}


void RoundFirmwareInfoFetcher::finish()
{
//    if ( m_isTimedout ) {
//        m_clientEventHandler->handleEvent( CommunicationClientEvent::TIMEOUT );
//    } else {
//        m_clientEventHandler->handleEvent( CommunicationClientEvent::FIRMWARE_INFO_FETCHED );
//    }
    m_listener->OnFirmwareInfoFetched( *m_firmwareInfo );
}

void RoundFirmwareInfoFetcher::read(uint8_t* buffer, int size)
{
    switch ( m_state )
    {
        case CLEAR:
            clear( buffer, size );
            break;

        case WAIT:
        {
            ResponseRecognizer* recognizer = m_recognizers[m_currentRecognizerIdx];

            if ( size == 0 )
            {
                wxThread::Sleep( m_sleep );
                recognizer->timeout();

                if ( recognizer->isTimedOut() ) {
                    switch ( m_currentRecognizerIdx )
                    {
                        case MAN:
                            ++m_currentRecognizerIdx;
                            changeStateToClearing();
                            break;

                        default:
                            m_isTimedout = true;
                            m_state = END;
                            break;
                    }
                }
            }
            else if ( size > 0 )
            {
                for ( int i = 0; i < size; ++i )
                {
                    if ( recognizer->feed( buffer[i] ) )
                    {
                        switch ( m_currentRecognizerIdx )
                        {
                            case SERIAL_NUMBER:
                            {
                                TxData* response = recognizer->getReceivedData();

                                wxString serialString = response->toString();
                                serialString = serialString.BeforeFirst( _T('\0') ) + _T("_");

                                if ( response->size >= 12 ) {
                                    uint8_t* buffer = (uint8_t*)&response->data[8];

                                    uint32_t serial =  Math::getNumber( buffer, 4 );
                                    serial -= 0x01010100;
                                    serialString += wxString::Format( _T("%u"), serial );
                                }

                                m_firmwareInfo->setSerialNumber( serialString );

                                delete response;

                            } break;

                            case SIGNATURE:
                            {
                                TxData* response = recognizer->getReceivedData();

                                wxString signature = response->toString();
                                m_firmwareInfo->setSignature( signature );
                                wxString version = signature.AfterLast( _T('v') );
                                m_firmwareInfo->setFirmwareVersion( version );

                                delete response;
                            } break;
                        }



                        if ( m_currentRecognizerIdx < m_recognizers.GetCount() - 1 )
                        {
                            ++m_currentRecognizerIdx;

                        } else {
                            m_nextState = END;
//                            m_state = END;
                        }

                        changeStateToClearing();
                    }
                }
            }
        } break;

        default:
            break;
    }
}

void RoundFirmwareInfoFetcher::write(uint8_t* buffer, int& size)
{
    switch ( m_state )
    {
        case SEND:
        {
            ResponseRecognizer* recognizer = m_recognizers[m_currentRecognizerIdx];

            TxData* command = recognizer->getRequest();
            size = command->size;
            memcpy( buffer, command->data, command->size );
            m_state = WAIT;
        } break;

        default:
            break;
    }
}

bool RoundFirmwareInfoFetcher::isFinished()
{
    return m_state == END;
}

void RoundFirmwareInfoFetcher::changeStateToClearing()
{
    m_state = CLEAR;
    m_clearingStartTime = wxDateTime::UNow();
}

void RoundFirmwareInfoFetcher::clear( uint8_t* buffer, int size )
{
    wxDateTime now = wxDateTime::UNow();
    wxTimeSpan timeSpan = now.Subtract( m_clearingStartTime );
    long elapsedTime = timeSpan.GetMilliseconds().ToLong();

    if ( elapsedTime >= m_clearingTime ) {
        m_state = m_nextState;//SEND;
    }

    if  ( size == 0 ) {
        wxThread::Sleep(m_sleep);
    }
}
