#ifndef SDCARDFILEDOWNLOADER_H
#define SDCARDFILEDOWNLOADER_H

#include <wx/wx.h>
#include "../CommunicationControllerClient.h"
#include "../../common/MMCFilesDesc.h"
#include "../../Interruptable.h"

class SDCardFileDownloader : public CommunicationControllerClient, public Interruptable
{
    public:
        SDCardFileDownloader( const MMCFilesDesc& desc, wxEvtHandler* evthandler, unsigned int blockCount = 60 );
        ~SDCardFileDownloader();

        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();

        bool isFinished();

        void Abort();

    protected:

    private:
        enum State {
            TIMEOUT,

            GET_STATUS,
            CHECK_STATUS,
            STATUS_FAILED,

            GET_SECTOR,
            CHECK_SECTOR,
            FINISHED,
            ABORTED
        };

        // read sector: maximum try count
        enum { MAX_TRY_COUNT = 10, TIMEOUT_VAL = 150 };

        unsigned int    m_blockCount;

        State           m_state;
        bool            m_nextBlock;
        unsigned char*  m_buffer;
        size_t          m_buflen;

        size_t          m_tryCount;

        MMCFilesDesc    m_desc;

        uint32_t        m_endIdx, m_blockIdx;

        wxEvtHandler*   m_evthandler;
        unsigned short  m_timeoutCounter;
        int             m_lastSecCountInBlock;

        void            getStatus( uint8_t* buffer, int& size );
        void            checkStatus( uint8_t* buffer, int size );

        void            readSector( uint8_t* buffer, int size );
        bool            isNextLog( uint8_t* data );

        unsigned char*  m_data;
        unsigned int    m_datalen;

        size_t          m_msscounter;
};

#endif // SDCARDFILEDOWNLOADER_H
