#ifndef SDCARDV3LOGGER_H
#define SDCARDV3LOGGER_H


#include "../CommunicationControllerClient.h"


enum SDLogCommand { SD_START_LOG, SD_STOP_LOG, SD_ERASE_CARD, SD_GET_SD_FIRMWARE };

class FirmwareInfo;


/// ------------------------------------------------------
/// start / stop logging, erase sdcard on v3
class SDCardV3Logger : public CommunicationControllerClient
{
    public:
        SDCardV3Logger( SDLogCommand command, wxEvtHandler * evthandler = NULL );
        virtual ~SDCardV3Logger();

    public:

        /// virtual methods of CommunicationControllerClient
        void        write( uint8_t* buffer, int& size );
        void        read( uint8_t* buffer, int size );
        void        finish();
        bool        isFinished();

    private:
        /// states
        enum State {
            INVALID,            // invalid
            IDLE,               // idle

            SEND_GET_FWVER,     // send get firmware version cmd
            CHECK_GET_FWVER,    // get fw version cmd

            SEND_START,         // send start cmd
            SEND_STOP,          // send stop cmd
            SEND_ERASE,         // send erase cmd
            SEND_STATUS,        // send status cmd

            SEND_GETBLOCK0,     // send mgs cmd
            WAIT_AFTER_ERASE,   // wait after msc sent

            SEND_ERASE_STATUS,  // get status after erase
            CHECK_ERASE_STATUS,

            CHECK_STOP,         // check status after stop
            CHECK_START,        // check status after start
            CHECK_GETBLOCK0,    // check status before getblock

            TIMEOUT,            // timeout
            FINISHED            // finished
        };

        /// iMAX_TRY_ERASECOUNT: max. # of sent msc
        /// iMAX_TRY_READCOUNT: max # of sent msg
        /// iTIMEOUT: timeout in ms
        enum { iMAX_TRY_ERASECOUNT = 5, iMAX_TRY_READCOUNT = 8, iTIMEOUT = 200 };

        // current command ( start/stop/erase )
        SDLogCommand                m_command;

        // current state
        State                       m_state;

        // an event handler (send sdcardevent)
        wxEvtHandler*               m_evthandler;

        // counter for counting mss after start command
        unsigned int                m_checkStatCounter;

        // counter for timeout
        unsigned int                m_timeoutCtr;

        // counter for sent msg
        unsigned int                m_tryCount;

        // counter for sent mss (before msg commands sent after erase)
        unsigned int                m_mssCounter;

        // buffer store incoming data
        uint8_t*                    m_buffer;
        size_t                      m_buffersize;

        // counter for send msc
        size_t                      m_checkSectTryCount;

        // a simple stopwatch
        wxStopWatch                 m_sp;

        // FW of the V3
        FirmwareInfo*               m_fwInfo;

    private:
        // mark SDStaticInfo as dirty ( see SDStaticInfo in SDCardFileSearch.h)
        void                setSDInfoDirty();

        // send a start command & check status
        void                sendStart( uint8_t* buffer, int & size );
        void                checkStart( uint8_t* buffer, int size );

        // send stop command & check status
        void                sendStop( uint8_t* buffer, int& size );
        void                checkStop( uint8_t* buffer, int size );

        void                sendStatus( uint8_t* buffer, int& size );

        // send erase command, send mss and get first block to check whether card is empty
        void                sendErase( uint8_t* buffer, int& size );

        void                sendEraseStatus( uint8_t* buffer, int& size );
        void                checkEraseStatus( uint8_t* buffer, int size );

        void                sendGetBlock0( uint8_t* buffer, int& size );
        void                checkGetBlock0( uint8_t* buffer, int size );

        void                sendGetFwVersion( uint8_t* buffer, int& size );
        void                checkGetFwVersion( uint8_t* buffer, int size );

        void                waitAfterErase();

        // send an event to m_evthandler
        void                sendEvent( int eventType, int intvalue = -1 );
};

#endif // SDCARDV3LOGGER_H
