#ifndef TRIGGERLOGREADER_H
#define TRIGGERLOGREADER_H


#include "../CommunicationControllerClient.h"


class TimeoutListener;


class TriggerLogReader : public CommunicationControllerClient
{
    public:
        TriggerLogReader(   TimeoutListener* timeoutListener,
                            wxEvtHandler* eventHandler );

        virtual ~TriggerLogReader();

        void stop();

        /// CommunicationControllerClient methods
        void write( uint8_t* buffer, int& size );
        void read( uint8_t* buffer, int size );
        void finish();
        bool isFinished();

    private:
        enum State
        {
            SEND_START_CMD,
            WAIT_LOG,
            SEND_STOP_CMD,
            FINISHED,
            TIMEDOUT,
            SEND_MAN_MDE40,
            SEND_MXC,
            WAIT_MXC,
            STOPPED,
        };

        enum { BUFFER_SIZE = 128 };

        TimeoutListener*                    m_timeoutListener;
        wxEvtHandler*                       m_eventHandler;
        State                               m_state;

        size_t                              m_rxDataLength;
        size_t                              m_timeoutCount;
        size_t                              m_mde40counter;

        uint8_t                             m_rxBuffer[BUFFER_SIZE];

        uint8_t                             m_mxcbuff[32];
        uint8_t                             m_mxcsize;
        size_t                              m_mxctimeout;

    private:
        void sendStartCmd( uint8_t* buffer, int& size );
        void waitLog( uint8_t* buffer, int size );

        void sendMxc( uint8_t* buffer, int& size );
        void waitMxc( uint8_t* buffer, int size );

        void sendStopCmd( uint8_t* buffer, int& size );
        void sendRestart( uint8_t* buffer, int& size );
        void sendEvent(size_t size);

        bool checkAIM( size_t size );
        bool isValidAIM( uint8_t* buffer );
};


#endif // TRIGGERLOGREADER_H
