#include "V3FirmwareInfoFetcher.h"
#include "FirmwareInfoFetcherListener.h"
#include "TimeoutListener.h"
#include "../CommunicationClientEventHandler.h"
#include "../../vemsprotocol/recognizers/ResponseRecognizer.h"
#include "../../vemsprotocol/recognizers/LengthRecognizer.h"
#include "../../vemsprotocol/recognizers/RangeRecognizer.h"
#include "../../vemsprotocol/recognizers/SerialRecognizer.h"
#include "../../vemsprotocol/recognizers/FirmwareVersionRecognizer.h"
#include "../../common/TxData.h"
#include "../../logger.h"
#include "../../FirmwareInfo.h"


V3FirmwareInfoFetcher::V3FirmwareInfoFetcher( FirmwareInfoFetcherListener* listener, TimeoutListener* timeoutListener )
{
    m_listener = listener;
    m_timeoutListener = timeoutListener;
    m_firmwareInfo  = new FirmwareInfo();

    m_state         = CLEAR;
    m_isTimedout    = false;

    ///TODO
    size_t timeout  = 0;
    timeout += TIMEOUT;

    LOGP( wxString::Format( _T("timeout: %d"), timeout ) );

    TxData* request = new TxData( _T("Man") );
    LengthRecognizer* lengthRecognizer = new LengthRecognizer( 8 );
    ResponseRecognizer* responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("mdv") );
    lengthRecognizer = new LengthRecognizer( 16 );
    responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("mdV") );
    FirmwareVersionRecognizer* versionRecognizer = new FirmwareVersionRecognizer();
    responseRecognizer = new ResponseRecognizer( 0, request, versionRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("mci") );
    SerialRecognizer* serialRecognizer = new SerialRecognizer( 12 );
    responseRecognizer = new ResponseRecognizer( 0, request, serialRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("bye") );
    lengthRecognizer = new LengthRecognizer( 5 );
    responseRecognizer = new ResponseRecognizer( 0, request, lengthRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("Q") );
    RangeRecognizer* rangeRecognizer = new RangeRecognizer( 23, 26 );
    responseRecognizer = new ResponseRecognizer( 0, request, rangeRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    request = new TxData( _T("S") );
    rangeRecognizer = new RangeRecognizer( 24, 26 );
    responseRecognizer = new ResponseRecognizer( 0, request, rangeRecognizer, timeout );
    m_recognizers.Add( responseRecognizer );

    m_currentRecognizerIdx = 0;
}

V3FirmwareInfoFetcher::~V3FirmwareInfoFetcher()
{
    delete m_firmwareInfo;
	WX_CLEAR_ARRAY(m_recognizers);
}


void V3FirmwareInfoFetcher::finish()
{
    if ( m_state == FIRMWAREINFO_FETCHED ) {
        m_listener->OnFirmwareInfoFetched( *m_firmwareInfo );
    } else {
        m_timeoutListener->OnTimeout();
    }
}

void V3FirmwareInfoFetcher::read(uint8_t* buffer, int size)
{
    switch ( m_state )
    {
        case CLEAR:
        {
            if ( size == 0 ) {
                m_state = SEND;
            }
        } break;

        case WAIT:
        {
            ResponseRecognizer* recognizer = m_recognizers[m_currentRecognizerIdx];

            if ( size == 0 )
            {
                wxThread::Sleep( m_sleep );
                recognizer->timeout();

                if ( recognizer->isTimedOut() ) {
                    switch ( m_currentRecognizerIdx )
                    {
                        case MAN:
                            ++m_currentRecognizerIdx;
                            m_state = CLEAR;
                            break;

                            case SIGNATURE:
                            {
                                TxData* response = recognizer->getReceivedData();
                                if ( response->size > 0 ) {
                                    m_firmwareInfo->setSignature( response->toString() );
                                    delete response;
                                    ++m_currentRecognizerIdx;
                                    m_state = CLEAR;
                                } else {
//                                    m_isTimedout = true;
                                    m_state = TIMEDOUT;
                                }
                            } break;

                            case OPTIONS:
                            {
                                LOG
                                TxData* response = recognizer->getReceivedData();
                                if ( response->size > 0 ) {
                                    LOGP( wxString::Format( _T("response->size: %d"), response->size ) );
                                    m_firmwareInfo->setFirmwareOptions( response->toString() );
                                    delete response;
                                    m_state = CLEAR;
                                } else {
                                    LOG
//                                    m_isTimedout = true;
                                    m_state = TIMEDOUT;
                                }
                            } break;

                        default:
//                            m_isTimedout = true;
                            m_state = TIMEDOUT;
                            break;
                    }
                }
            }
            else if ( size > 0 )
            {
                for ( int i = 0; i < size; ++i )
                {
                    if ( recognizer->feed( buffer[i] ) )
                    {
                        switch ( m_currentRecognizerIdx )
                        {
                            case CTIME:
                            {
                                TxData* response = recognizer->getReceivedData();
                                m_firmwareInfo->setCompilationTime( response->toString() );
                                delete response;
	                        } break;

                            case FIRMWARE_VERSION:
                            {
                                TxData* response = recognizer->getReceivedData();
                                m_firmwareInfo->setFirmwareVersion( response->toString() );
                                delete response;
                                LOG
                            } break;

                            case SERIAL_NUMBER:
                            {
                                TxData* response = recognizer->getReceivedData();
                                m_firmwareInfo->setSerialNumber( response->toString() );
                                delete response;
                            } break;

                            case SIGNATURE:
                            {
                                TxData* response = recognizer->getReceivedData();
                                m_firmwareInfo->setSignature( response->toString() );
                                delete response;
                            } break;

                            case OPTIONS:
                            {
                                TxData* response = recognizer->getReceivedData();
                                LOGP( wxString::Format( _T("response->size: %d"), response->size ) );
                                m_firmwareInfo->setFirmwareOptions( response->toString() );
                                delete response;
                            } break;

                        }

                        if ( m_currentRecognizerIdx < m_recognizers.GetCount() - 1 )
                        {
                            LOG
                            ++m_currentRecognizerIdx;
                            m_state = CLEAR;
                            wxThread::Sleep( 10 );
                        } else {
                            m_state = FIRMWAREINFO_FETCHED;
                        }
                    }
                }
            }
        } break;

        default:
            break;
    }
}

void V3FirmwareInfoFetcher::write(uint8_t* buffer, int& size)
{
    switch ( m_state )
    {
        case SEND:
        {
            ResponseRecognizer* recognizer = m_recognizers[m_currentRecognizerIdx];

            TxData* command = recognizer->getRequest();
            size = command->size;
            memcpy( buffer, command->data, command->size );
            m_state = WAIT;
        } break;

        default:
            break;
    }
}

bool V3FirmwareInfoFetcher::isFinished()
{
    return ( ( m_state == TIMEDOUT ) || ( m_state == FIRMWAREINFO_FETCHED ) );
}
