#include "ConfigMgrCommunication.h"
#include "CommunicationConfig.h"
#include "../ConfigBase.h"
#include "../logger.h"


ConfigMgrCommunication::ConfigMgrCommunication(ConfigBase* configBase)
    : ConfigMgrBase(configBase)
{

}

ConfigMgrCommunication::~ConfigMgrCommunication()
{

}

void ConfigMgrCommunication::load(wxObject* obj)
{
    CommunicationConfig* config = dynamic_cast<CommunicationConfig*>( obj );

    if ( !config ) {
        LOGP( _T("cast failed!"));
        return;
    }

    wxString path = _T("/");
    wxString port = m_configBase->getValue( path, _T("port"), _T("COM2") );
    wxString timeout = m_configBase->getValue( path, _T("timeout"), _T("10") );
    wxString rtErrorCount = m_configBase->getValue( path, _T("rterrorcount"), _T("3") );
    wxString interWriteDelay = m_configBase->getValue( path, _T("interWriteDelay"), _T("0") );
    wxString preferredBaudrate = m_configBase->getValue( path, _T("preferredBaudrate"), _T("19200") );
    wxString fwUploadBaudrate = m_configBase->getValue( path, _T("firmwareUploadBaudrate"), _T("19200") );

    path = _T("/sdcard/");
    wxString sdcardBaudrate = m_configBase->getValue( path, _T("baudrate"), _T("115200") );

//    path = _T("/devices/");
//    wxString protocol = m_configBase->getValue( path, _T("protocol"), _T("SIPR") );
//    wxString readRealtime = m_configBase->getValue( path, _T("read_realtime"), _T("SIPR_RT_MT_EXT") );

    path = _T("/communication/");
    wxString mode = m_configBase->getValue( path, _T("mode"), _T("SERIAL") );

    path = _T("/communication/tcp/");
    wxString ip = m_configBase->getValue( path, _T("ip"), _T("") );
    wxString tcpTimeout = m_configBase->getValue( path, _T("timeout"), _T("0") );

    path = _T("/communication/serial/");
    wxString serialBaudrate = m_configBase->getValue( path, _T("baudrate"), _T("19200") );

    config->fwUploadBaudrate = wxAtoi( fwUploadBaudrate );
    config->port = port;
    config->timeout = wxAtoi(timeout);
    config->rtErrorCount = wxAtoi(rtErrorCount);
    config->interWriteDelay = wxAtoi(interWriteDelay);
    config->preferredBaudrate = wxAtoi(preferredBaudrate);

    config->sdcardBaudrate =  wxAtoi(sdcardBaudrate);

//    config->protocol =  protocol;
//    config->readRealtime =  readRealtime;

    config->ip = ip;
    config->tcpTimeout = wxAtoi(tcpTimeout);;

    config->baudrate = wxAtoi( serialBaudrate );

    config->mode = mode;
}

void ConfigMgrCommunication::save(wxObject* obj)
{
    CommunicationConfig* config = dynamic_cast<CommunicationConfig*>( obj );

    if ( !config ) {
        LOGP( _T("cast failed!"));
        return;
    }

    m_configBase->setPath( _T("/") );

    m_configBase->write( _T("port"), config->port );
    m_configBase->write( _T("timeout"), wxString::Format(_T("%i"), config->timeout ) );
    m_configBase->write( _T("rterrorcount"), wxString::Format(_T("%i"), config->rtErrorCount) );
    m_configBase->write( _T("interWriteDelay"), wxString::Format(_T("%i"), config->interWriteDelay) );
    m_configBase->write( _T("preferredBaudrate"), wxString::Format(_T("%i"), config->preferredBaudrate) );
    m_configBase->write( _T("firmwareUploadBaudrate"), wxString::Format(_T("%d"), config->fwUploadBaudrate ) );

    m_configBase->setPath( _T("/sdcard/") );
    m_configBase->write( _T("baudrate"), wxString::Format(_T("%i"), config->sdcardBaudrate) );

//    m_configBase->setPath( _T("/devices/") );
//    m_configBase->write( _T("protocol"), config->protocol );
//    m_configBase->write( _T("read_realtime"), config->readRealtime );

    m_configBase->setPath( _T("/communication/") );
    m_configBase->write( _T("mode"), config->mode );

    m_configBase->setPath( _T("/communication/tcp/") );
    m_configBase->write( _T("ip"), config->ip );
    m_configBase->write( _T("timeout"), wxString::Format(_T("%i"), config->tcpTimeout) );

    m_configBase->setPath( _T("/communication/serial/") );
    m_configBase->write( _T("baudrate"), wxString::Format(_T("%i"), config->baudrate) );

    m_configBase->flush();
    m_configBase->save();
}

