#include "ConfigMgrMainGui.h"

#include "../ConfigBase.h"
#include "MainGuiConfig.h"
#include "../vemsprotocol/ECUInfo.h"

#include "../logger.h"

ConfigMgrMainGui::ConfigMgrMainGui(ConfigBase* configBase)
:ConfigMgrBase(configBase)
{
    //ctor
}

ConfigMgrMainGui::~ConfigMgrMainGui()
{
    //dtor
}

void ConfigMgrMainGui::load(wxObject* obj)
{
    Logger::log(LOG_MSG,_T("ConfigMgrMainGui::load START 10"));
    MainGuiConfig* config = dynamic_cast<MainGuiConfig*>( obj );
    if(!config){
         Logger::log(LOG_MSG,_T("ConfigMgrMainGui::load START 20 cast failed"));
        return;
    }
    Logger::log(LOG_MSG,_T("ConfigMgrMainGui::load START 30"));

    wxString path = _T("/layout/maingui");
    wxString xStr = m_configBase->getValue(path, _T("x"), _T("50") );
    wxString yStr = m_configBase->getValue(path, _T("y"), _T("50") );
    wxString widthStr = m_configBase->getValue(path, _T("width"), _T("500") );
    wxString heightStr = m_configBase->getValue(path, _T("height"), _T("100") );
    wxString maximizedStr = m_configBase->getValue(path, _T("maximized"), _T("0") );
    wxString fullscreenStr = m_configBase->getValue(path, _T("fullscreen"), _T("0") );

    config->m_x = wxAtoi(xStr);
    config->m_y = wxAtoi(yStr);
    config->m_width = wxAtoi(widthStr);
    config->m_height = wxAtoi(heightStr);
    config->m_isMaximized = wxAtoi(maximizedStr);
    config->m_isFullScreen = wxAtoi(fullscreenStr);

    if(config->m_device == -1) return;

    path = getPerspectivePath( config->m_device );
    config->m_perspective = m_configBase->getValue(path, _T("perspective"), wxEmptyString );


}

void ConfigMgrMainGui::save(wxObject* obj)
{
    MainGuiConfig* config = dynamic_cast<MainGuiConfig*>( obj );
    if(!config) return;

    m_configBase->setPath( _T("/layout/maingui") );

    m_configBase->write( _T("fullscreen"), wxString::Format(_T("%i"), config->m_isFullScreen) );
    if( !config->m_isFullScreen ){
        m_configBase->write( _T("maximized"), wxString::Format(_T("%i"), config->m_isMaximized) );
        if( !config->m_isMaximized ) {
            m_configBase->write( _T("x"), wxString::Format(_T("%i"), config->m_x) );
            m_configBase->write( _T("y"), wxString::Format(_T("%i"), config->m_y) );
            m_configBase->write( _T("width"), wxString::Format(_T("%i"), config->m_width) );
            m_configBase->write( _T("height"), wxString::Format(_T("%i"), config->m_height) );
        }
    }
//    if ( config.m_mdiHeight != -1 ) {
//        m_configBase->Write(_T("mdi_win_height"), wxString::Format(_T("%i"), config.m_mdiHeight) );
//    }

    m_configBase->setPath(  getPerspectivePath( config->m_device ) );

    if(config->m_perspective != wxEmptyString)
    m_configBase->write( _T("perspective"), config->m_perspective );

    m_configBase->flush();
    m_configBase->save();

}

wxString ConfigMgrMainGui::getPerspectivePath( int device )
{
    switch ( device )
    {
        case ECUInfo::DEVICE_V3:
            return _T("/layout/v3");
        case ECUInfo::DEVICE_ROUND:
            return _T("/layout/round");
        default:
            return _T("/layout/default");
    }
}
