#ifndef CONTROLLERDESCRIPTOR_H
#define CONTROLLERDESCRIPTOR_H

#include <wx/dynarray.h>
#include <wx/arrstr.h>

WX_DEFINE_ARRAY_INT(int, ArrayInt);

enum cdValues
{
    cdIWD, cdWB, cdBE, cdNP, cdBC, cdPS, cdPA, cdPAD, cdPRC, cdPVW, cdPCW
};

class ControllerDescriptor
{
    public:
        ControllerDescriptor();
        virtual ~ControllerDescriptor();

        void setInterWriteDelay(int iwd){ interWriteDelay = iwd; }
        int getInterWriteDelay(){ return interWriteDelay; }
        void setWriteBlocks(bool wb){ writeBlocks = wb; }
        bool getWriteBlocks(){ return writeBlocks; }
        void setBigEnd(bool be){ bigEnd = be; }
        bool getBigEnd(){ return bigEnd; }
        void setNPages(int np) { nPages = np; }
        int getNPages() const { return nPages; }
        void setBurnCommand(wxString bc){ burnCommand = bc; }
        wxString getBurnCommand() const { return burnCommand; }
        void setPageSize(const ArrayInt& ps);
        int getPageSize(int pageNo) const;
        void setPageActivate(const ArrayInt& pa){ pageActivate = pa; }
        int getPageActivate(int pageNo) const;
        int getPageIdx(int page) const;
        void setPageReadCommand(wxArrayString prc) { pageReadCommand = prc; }
        wxString getPageReadCommand(int pageNo) const;
        void setPageValueWrite(const wxArrayString& pvw) { pageValueWrite = pvw; }
        wxString getPageValueWrite(int pageNo) const;
        void setPageChunkWrite(const wxArrayString& pcw) { pageChunkWrite = pcw; }
        wxString getPageWriteCommand(int pageNo) const;
        void        setOchGetCommand( const wxString& command ) { m_ochGetCommand = command; }
        wxString    getOchGetCommand() const { return m_ochGetCommand; }

        void clear();

        //test functions
        wxString pageSizeToString();
        wxString pageActivateToString();
        wxString pageReadCommandToString();
        wxString pageValueWriteToString();
        wxString pageChunkWriteToString();

    protected:
    private:
        int     interWriteDelay;
        bool    writeBlocks;
        bool    bigEnd;
        int     nPages;
        wxString burnCommand;
        int pageActivationDelay;
        // output channels
        wxString        m_ochGetCommand;

        ArrayInt pageSize;
        //wxArrayString pageActivate;
        ArrayInt        pageActivate;
        wxArrayString   pageReadCommand;
        wxArrayString   pageValueWrite;
        wxArrayString   pageChunkWrite;


};

#endif // CONTROLLERDESCRIPTOR_H
