#include "BubbleStatusBar.h"

#include "../agg/wxAgg.h"

#define ptrdel(ptr) \
    if( (ptr) != NULL ) { delete (ptr); (ptr) = NULL; }


BubbleStateDesc DefaultBubbleState = BubbleStateDesc(wxColor(0,0,0,255),wxColor(0,0,0,0),wxColor(0,0,0,255));

BubbleStatusBar::BubbleStatusBar()
{
    m_parent = NULL;
    m_padding = 1;
    m_spacing = 2;
}

BubbleStatusBar::~BubbleStatusBar()
{
    WX_CLEAR_ARRAY( m_bubbles );
}

StatusBubble* BubbleStatusBar::AddBubble(const wxString& symbol, int align, BubbleStateDescHash statedescs)
{
    StatusBubble * bubble = new StatusBubble(symbol, m_rect.height-2*m_padding, align, this );

    bubble->SetStateDescriptors(statedescs);

    m_bubbles.Add( bubble );

    if( m_parent ) m_parent->Refresh(false);

    return bubble;
}

StatusBubble* BubbleStatusBar::InsertBubble(int idx, const wxString& symbol, int align, BubbleStateDescHash statedescs)
{
    StatusBubble * bubble = new StatusBubble(symbol, m_rect.height-2*m_padding, align, this );

    bubble->SetStateDescriptors(statedescs);

    m_bubbles.Insert( bubble, idx );

    if( m_parent ) m_parent->Refresh(false);

    return bubble;
}

bool BubbleStatusBar::RemoveBubble(size_t idx)
{
    if( idx < 0 || idx >= m_bubbles.GetCount() ) return false;

    delete m_bubbles[idx];
    m_bubbles.RemoveAt(idx);

    if( m_parent ) m_parent->Refresh(false);

    return true;
}

bool BubbleStatusBar::RemoveBubble(const wxString& symbol)
{
    for( size_t a=0; a< m_bubbles.GetCount(); a++ ){

        if( m_bubbles[a]->GetName() == symbol ){
            delete m_bubbles[a];
            m_bubbles.RemoveAt(a);
            return true;
        }

    }

    if( m_parent ) m_parent->Refresh(false);

    return false;
}

void BubbleStatusBar::DrawStatusBar(wxDC& dc)
{
    int lx = m_rect.GetLeft() + m_padding;
    int rx = m_rect.GetRight() - m_padding;
    //int cx = (rx-lx)/2+lx;
    //int cw = 0;

    for( size_t a = 0; a < m_bubbles.GetCount(); a++ ) {

        StatusBubble * bubble = m_bubbles[a];

        wxBitmap * bmp = bubble->GetBitmap();

        if(!bmp) continue;

        int targetx = 0;

        switch( bubble->GetAlign() ){
            case wxALIGN_LEFT:
                targetx = lx;
                lx += bubble->GetWidth() + m_spacing;
                break;
            case wxALIGN_RIGHT:
                targetx = rx - bubble->GetWidth();
                rx = targetx - m_spacing;
                break;
            /*case wxALIGN_CENTER:
                cw += bubble->GetWidth();

                break;  */
            default:
                break;
        }


        dc.DrawBitmap( *bmp, targetx, m_rect.GetTop()+m_padding, false );

    }

}

void BubbleStatusBar::ReDrawBubbles()
{
    for( size_t a = 0; a < m_bubbles.GetCount(); a++ ) {

        StatusBubble * bubble = m_bubbles[a];

        bubble->SetHeight(m_rect.height-2*m_padding);

        bubble->Draw();
    }
}

bool BubbleStatusBar::SetBubbleState(const wxString& symbol, const wxString& state, bool refreshParent)
{
    for( size_t a = 0; a < m_bubbles.GetCount(); a++ ) {
        if( m_bubbles[a]->GetName() == symbol ) {
            m_bubbles[a]->SetState(state,refreshParent);
            return true;
        }
    }
    return false;
}

bool BubbleStatusBar::SetBubbleText(const wxString& symbol, const wxString& text, bool refreshParent)
{
    for( size_t a = 0; a < m_bubbles.GetCount(); a++ ) {
        if( m_bubbles[a]->GetName() == symbol ) {
            m_bubbles[a]->SetText(text,refreshParent);
            return true;
        }
    }
    return false;
}

bool BubbleStatusBar::SetBubbleStateAndText(const wxString& symbol, const wxString& state,const wxString& text, bool refreshParent)
{
    for( size_t a = 0; a < m_bubbles.GetCount(); a++ ) {
        if( m_bubbles[a]->GetName() == symbol ) {
            m_bubbles[a]->SetStateAndText(state,text,refreshParent);
            return true;
        }
    }
    return false;
}

StatusBubble::StatusBubble(const wxString& name, int height, int align, BubbleStatusBar* mgr)
{
    m_bitmap = NULL;
    m_text = _T("");
    m_height = height;
    m_align = align;
    m_name = name;
    m_width = 0;
    m_mgr = mgr;
}

StatusBubble::~StatusBubble()
{
    ptrdel(m_bitmap);
}

BubbleStateDesc& StatusBubble::GetStateDesc(const wxString& state)
{
    if( m_stateDescs.find(state) == m_stateDescs.end() )
        return DefaultBubbleState;
    else
        return m_stateDescs[state];
}

int StatusBubble::Draw()
{
    int h = m_height;

    BubbleStateDesc desc = GetStateDesc(m_state);

    double border_size = desc.GetBorderSize();

    double padding = border_size + 1;

    double fontheight = (m_height - 2.0*padding) / 1.6;

    double sw = wxAgg::getTextWidth(m_text,fontheight);

    int w = (int)(sw + 3.0*padding);

    wxAgg aggCanvas( w , h );
    aggCanvas.clear( wxColor(0,0,0,0) );

    aggCanvas.setPen( desc.GetBorderColor() );
    aggCanvas.setBrush( desc.GetBackgroundColor() );

    //aggCanvas.drawFilledRoundedRectangle(bs/2.0,bs/2.0,w-bs,h-bs, border, bs);
    aggCanvas.drawFilledRoundedRectangle(border_size/2.0,border_size/2.0,w-border_size,h-border_size, desc.GetRoundedSize(), border_size);

    aggCanvas.setPen( desc.GetForegroundColor() );
    aggCanvas.drawText(m_text,padding*1.5, h-2*padding, -1, fontheight, 1);

    ptrdel(m_bitmap);

    m_bitmap = aggCanvas.renderToBitmap();

    m_width = w;

    return w;
}

void StatusBubble::SetText(const wxString& text, bool refreshParent)
{
    m_text = text;
    Draw();
}

bool StatusBubble::SetState(const wxString& state, bool refreshParent)
{
    if( m_stateDescs.find(state) == m_stateDescs.end() ) return false;

    m_state = state;
    Draw();

    return true;
}

bool StatusBubble::SetStateAndText(const wxString& state,const wxString& text, bool refreshParent)
{
    if( m_stateDescs.find(state) == m_stateDescs.end() ) return false;

    m_state = state;
    m_text = text;

    Draw();

    return true;
}
