#include "ColorSetupPanel.h"

#include <wx/colordlg.h>

#include "../DrawHelper.h"
#include "../wxVariantHelper.h"
#include "wxTextCtrlNullEx.h"
#include "../logger.h"

BEGIN_EVENT_TABLE(ColorSetupPanel, wxPanelWithCtrlExBase)

    EVT_BUTTON( idColorButt, ColorSetupPanel::OnButton )

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(ColorSetupPanel,wxPanelWithCtrlExBase);

ColorSetupPanel::ColorSetupPanel(wxWindow* parent, const wxColor& color, int deafultcolorstrstyle)
{
    //ctor
    m_colorStrStyle = deafultcolorstrstyle;

    Create(parent, wxID_ANY);

    wxString colorstr;

    if( color != wxNullColour ) colorstr = DrawHelper::GetColorAsString(color);

    m_textCtrl = new wxTextCtrl(this, wxID_ANY, colorstr );

    CreateValidator( m_textCtrl, colorstr );

    wxBitmapButton * dButt = new wxBitmapButton(this, idColorButt, DrawHelper::GetBitmap(_T("colorize16c.png")) );
    dButt->SetToolTip(_T("Open color dialog"));

    wxBoxSizer* mainsizer = new wxBoxSizer(wxHORIZONTAL);

    mainsizer->Add( m_textCtrl, 1, wxALL | wxALIGN_CENTER_VERTICAL, 1 );
    mainsizer->Add( dButt, 0, wxALL | wxALIGN_CENTER_VERTICAL, 1 );

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);


    m_textCtrl->Connect(wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(ColorSetupPanel::OnUpdated), NULL, this );
    m_textCtrl->Connect(wxEVT_SET_FOCUS, wxFocusEventHandler(ColorSetupPanel::OnSetFocus), NULL, this );
    m_textCtrl->Connect(wxEVT_KILL_FOCUS, wxFocusEventHandler(ColorSetupPanel::OnKillFocus), NULL, this );
}

ColorSetupPanel::~ColorSetupPanel()
{
    //dtor
}

void ColorSetupPanel::SetValue(const wxVariant& value)
{
    SetColor( wxVariantHelper::GetColor( value ) );
}

wxVariant ColorSetupPanel::GetValue()
{
    wxVariant v;
    v << GetColor();
    return v;
}

void ColorSetupPanel::OnUpdated(wxCommandEvent& evt)
{
    evt.Skip();
    DoOnUpdated();
}

void ColorSetupPanel::OnSetFocus(wxFocusEvent& evt)
{
    evt.Skip();
    DoSetFocus();
}

void ColorSetupPanel::OnKillFocus(wxFocusEvent& evt)
{
    evt.Skip();
    DoKillFocus();
}

void ColorSetupPanel::OnButton(wxCommandEvent& evt)
{
    wxColourData cd;
    wxColor c = DrawHelper::ParseExtColorStr( m_textCtrl->GetValue() );

    cd.SetColour( c );

    wxColourDialog dlg(this,&cd);

    if(dlg.ShowModal() == wxID_OK) {
        wxColor nc = dlg.GetColourData().GetColour();
        DrawHelper::SetColorA(nc,c.Alpha());
        SetColor(nc);
    }

}

void ColorSetupPanel::SetColor(const wxColor& color)
{
    m_textCtrl->SetValue( DrawHelper::GetColorAsString(color,m_colorStrStyle) );
}

void ColorSetupPanel::SetColor(const wxString& color)
{
    m_textCtrl->SetValue( color );
}

wxColor ColorSetupPanel::GetColor()
{
    return DrawHelper::ParseExtColorStr( m_textCtrl->GetValue() );
}

wxString ColorSetupPanel::GetColorStr()
{
    return m_textCtrl->GetValue();
}
