#include "wxBitmapToggleButton.h"

#include <wx/tooltip.h>

#include "../logger.h"

BEGIN_EVENT_TABLE(wxBitmapToggleButton, wxToggleButton)

    EVT_LEFT_DOWN(wxBitmapToggleButton::OnLeftDown)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(wxBitmapToggleButton, wxToggleButton)

wxBitmapToggleButton::wxBitmapToggleButton(wxWindow* parent, wxWindowID id, const wxBitmap &bmp, int groupid, int border, const wxPoint& pos, const wxSize& size, long style, const wxValidator& val, const wxString& name)
{
    //ctor
    m_bmpBorder = wxPoint(border,border);
    wxSize rsize = size;
    if(bmp.IsOk())
        rsize = wxSize( bmp.GetWidth()+m_bmpBorder.x*2, bmp.GetHeight()+m_bmpBorder.y*2 );

    Create(parent,id,_T(""),pos,rsize,style,val,name);

    //m_bmp = bmp.GetSubBitmap(wxRect(0, 0, bmp.GetWidth(), bmp.GetHeight()));

    m_bitmapControl = new wxStaticBitmapEx(this,-1,bmp,m_bmpBorder);

    m_grpID = groupid;
}

wxBitmapToggleButton::~wxBitmapToggleButton()
{
    //dtor
}

void wxBitmapToggleButton::OnToggle(wxCommandEvent &evt)
{
    LayoutBitmap();
    evt.Skip();
}

void wxBitmapToggleButton::Init()
{

#if !defined(__WINCE_STANDARDSDK__)
    m_bitmapControl->SetToolTip( GetToolTip()->GetTip() );
#endif

    Connect(wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, wxCommandEventHandler( wxBitmapToggleButton::OnToggle ));
}

void wxBitmapToggleButton::SetValueEx(bool isdown)
{
    SetValue(isdown);
    LayoutBitmap();
}

void wxBitmapToggleButton::LayoutBitmap(bool isnegate)
{
    bool sem = GetValue();

    if(isnegate)
        sem = !sem;

    if(sem)
        m_bitmapControl->Move(m_bmpBorder+wxPoint(1,1));
    else
        m_bitmapControl->Move(m_bmpBorder);


    if( m_grpID == -1 || !GetValue() || !GetParent() ) return;

    wxWindowList list = GetParent()->GetChildren();

    for( size_t a=0; a<list.GetCount(); a++ ){
        wxWindow * win = list.Item(a)->GetData();

        if( win == this) continue;

        if( win->IsKindOf(CLASSINFO(wxBitmapToggleButton)) ){
            wxBitmapToggleButton * btb = (wxBitmapToggleButton*)win;

            if( btb->GetGroupID() != m_grpID ) continue;
            if( !btb->GetValue() ) continue;

            btb->SetValueEx( false );
        }
    }

}

void wxBitmapToggleButton::OnLeftDown(wxMouseEvent &evt)
{
    if( evt.GetEventObject() == m_bitmapControl ) {
        SetValue( !GetValue() );
        LayoutBitmap();

        wxCommandEvent evt;
        evt.SetEventType(wxEVT_COMMAND_TOGGLEBUTTON_CLICKED);
        evt.SetEventObject(this);
        evt.SetId( GetId() );
        GetParent()->ProcessEvent(evt);
    } else {
        LayoutBitmap(true);
    }

    evt.Skip();
}


/****************************************************
                wxStaticBitmapEx
*/

BEGIN_EVENT_TABLE(wxStaticBitmapEx,wxStaticBitmap)
    EVT_MOUSE_EVENTS(wxStaticBitmapEx::OnAllMouseEvents)
END_EVENT_TABLE()

void wxStaticBitmapEx::OnAllMouseEvents(wxMouseEvent& evt)
{
    evt.m_x += GetPosition().x;
    evt.m_y += GetPosition().y;

    if(GetParent())
        GetParent()->ProcessEvent(evt);
}
