#include "evaluation.h"
#include "logger.h"

wxString boolToString(bool b)
{
    if(b){
        return _T("true");
    }else{
        return _T("false");
    }
}

Evaluation::Evaluation()
{
    curr_tok = PRINT;
    //(*table)[_T("pi")] = 3.1415926535897932385;
    //(*table)[_T("e")] = 2.7182818284590452354;
}

Evaluation::~Evaluation()
{
    //dtor
}

double Evaluation::calculate(wxInputStream* in){
    input = in;
    double result = 0.0;
    while(input->CanRead()){
        try{
            get_token();
            if(curr_tok == END) break;
            if(curr_tok == PRINT) continue;
            result =  expr(false);
            //wxLogMessage( wxString::Format(_T("%f \n"),expr(false)));
           // Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::calculate() result: %f"), result ) );
        }catch(Error::ZeroDivide){
            //wxLogError(_T("attempt to divide by zero \n"));
//            wxLogMessage(_T("attempt to divide by zero \n"));
            skip();
        }catch(Error::SyntaxError e){
            //wxLogError(wxString::Format(_T("syntax error: %s \n"), e.p.c_str()));
//            wxLogMessage(wxString::Format(_T("syntax error: %s \n"), e.p.c_str()));
            skip();
        }
    }

     //Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::calculate() return: %f"), result ) );
    //return no_of_errors;
    return result;
}

double Evaluation::calculate(const wxString& expr)
{
    return calculate(new wxStringInputStream(expr));
}

double Evaluation::expr(bool get)
{
    //TEST
    //wxLogMessage(wxString::Format(_T("expr(%s)"), boolToString(get).c_str()));

    double left = bitOr(get);
    //Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::expr() left: %f"), left ) );

    for(;;){
        switch(curr_tok){
        case QUERY:{
            double d1 = expr(true);
            if(curr_tok != COLON){
                throw Error::SyntaxError(_T(": expected"));
            }
            double d2 = expr(true);
            if(left != 0){
                return d1;
            }else{
                return d2;
            }
        }
        default:{
            return left;
        }
        }
    }
}

double Evaluation::bitOr(bool get)
{
    double left = bitAnd(get);

    for(;;){
        switch(curr_tok){
        case BIT_OR:{
            int i1 = (int)left;
            int i2 = (int)(logical(true));
            left = i1 | i2;
            break;
        }
        default:{
            return left;
        }
        }
    }
}

double Evaluation::bitAnd(bool get)
{
    double left = logical(get);

    for(;;){
        switch(curr_tok){
        case BIT_AND:{
            int i1 = (int)left;
            int i2 = (int)(logical(true));
            left = i1 & i2;
            break;
        }
        default:{
            return left;
        }
        }
    }
}

double Evaluation::logical(bool get)
{
    //TEST
    //wxLogMessage(wxString::Format(_T("logical(%s)"), boolToString(get).c_str()));

    double left = shift(get);

    for(;;){
        switch(curr_tok){
        case LT:{
            double d1 = left;
            double d2 = shift(true);
            if( d1 > d2 ){
                left = 1;
            }else{
                left = 0;
            }
            break;
        }
        case ST:{
            double d1 = left;
            double d2 = shift(true);
            if( d1 < d2 ){
                left = 1;
            }else{
                left = 0;
            }
            break;
        }
        default:{
            return left;
        }
        }
    }
}

double Evaluation::shift(bool get)
{
    //TEST
    //wxLogMessage(wxString::Format(_T("shift(%s)"), boolToString(get).c_str()));

    double left = addition(get);

    for(;;){
        switch(curr_tok){
        case LSHIFT:{
            int i1 = (int)left;
            int i2 = (int)(addition(true));
            left = i1 << i2;
            break;
        }
        case RSHIFT:{
            int i1 = (int)left;
            int i2 = (int)(addition(true));
            left = i1 >> i2;
            break;
        }
        default:{
            return left;
        }
        }
    }
}

double Evaluation::addition(bool get)
{
    //TEST
    //wxLogMessage(wxString::Format(_T("addition(%s)"), boolToString(get).c_str()));

    double left = term(get);

    for(;;){
        switch(curr_tok){
        case PLUS:{
            left += term(true);
            break;
        }
        case MINUS:{
            left -= term(true);
            break;
        }
        default:{
             //Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::addition return() Value: %f"), left ) );
            return left;
        }
        }
    }
}

double Evaluation::term(bool get){
    //TEST
    //wxLogMessage(wxString::Format(_T("term(%s)"), boolToString(get).c_str()));

    double left = prim(get);
    //Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::term() Left Value: %f"), left ) );
    for(;;){
        switch(curr_tok){
        case MUL:{
            left *= prim(true);
            //Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::term() Value: %f"), left ) );
            break;
        }
        case DIV:{
            if(double d = prim(true)){
                left /= d;
                break;
            }
            throw Error::ZeroDivide();
        }
        default:{
            //Logger::log(LOG_MSG, wxString::Format(_T("Evaluation::term return() Value: %f"), left ) );
            return left;
        }
        }
    }
}

double Evaluation::prim(bool get){
    //TEST
    //wxLogMessage(wxString::Format(_T("prim(%s)"), boolToString(get).c_str()));

    if(get) get_token();

    switch(curr_tok){
    case NUMBER:{
        double v = number_value;
        get_token();
        return v;
    }
    case NAME:{
        if(!table) { throw Error::SyntaxError(_T("table is not set!)")); }
        if(table->find(string_value) == table->end() ){
            Logger::logf(LOG_MSG, _T("Evaluation::prim() variable not found in hash table!!! ") + string_value);
        }
        double v = (*table)[string_value];
        //Logger::logf(LOG_MSG, _T("Evaluation::prim() NAME: ") +string_value +_T(" Value: %f"), v  );
        if(get_token() == ASSIGN) v = expr(true);
        return v;
    }
    case MINUS:{
        return -prim(true);
    }
    case FUNC:{
        /*
        if(functionTable.find(string_value) == functionTable.end() ){
            throw Error::SyntaxError(wxString::Format(_T("function %s not found!"),string_value.c_str()));
        }
        double e = ((PF)functionTable[string_value])(expr(true));
        if(curr_tok != RP) throw Error::SyntaxError(_T(") expected (FUNC)"));
        get_token();
        return e;
        */
       // Logger::logf(LOG_MSG, _T("Evaluation::prim() FUNC ") + string_value);

        if(functionBaseHash.find(string_value) == functionBaseHash.end() ){
            throw Error::SyntaxError(wxString::Format(_T("function %s not found!"),string_value.c_str()));
        }
       // Logger::logf(LOG_MSG, _T("Evaluation::prim() FUNC found in table hash"));

        FunctionBase *tmp = functionBaseHash[string_value];
        FunctionBase *fb = tmp->Create();
        while(curr_tok != RP){
            get_token();
            if(curr_tok == STRING){
                fb->addParam(string_value);
               //  Logger::logf(LOG_MSG, _T("Evaluation::prim() in FUNC ") + string_value);
                 get_token();
            }else{
                double v = expr(false);
                fb->addParam(v);
               //  Logger::logf(LOG_MSG, _T("Evaluation::prim() in FUNC value: %f"), v);
            }
            // curr_tok should be COMMA
//            if(curr_tok == COMMA){ Logger::logf(LOG_MSG,_T("COMMA expected! OK")); } ;
        }
        //Logger::logf(LOG_MSG, _T("Evaluation::prim() in FUNC END") );
        double v = fb->exec();
        delete fb;
        get_token();
        return v;
    }
    case LP:{
        double e = expr(true);
        if(curr_tok != RP) throw Error::SyntaxError(_T(") expected"));
        get_token();
        return e;
    }
    default:{
        throw Error::SyntaxError(_T("primary expected"));
    }
    }
}

Evaluation::Token_value Evaluation::get_token()
{
    char ch = 0;
//    char tmpC;
    do{
        if(!input->CanRead()) return curr_tok = END;
        ch = input->Peek();
        if(isspace(ch)){
             ch = input->GetC();
        }
    }while(ch != '\n' && isspace(ch));

    //TEST
    //wxLogMessage(wxString::Format(_T("get_token() -> ch = %c"),ch));

    switch(ch){
    case '\0': //sor vege!!!
    case ';':
    case '\n':
        ch = input->GetC();
        return curr_tok = PRINT;
    case '*':
    case '/':
    case '+':
    case '-':
    case '(':
    case ')':
    case '=':
    case ',':
    case '?':
    case ':':
    case '&':
    case '|':
        ch = input->GetC();
        return curr_tok = Token_value(ch);
    case '>':
        ch = input->GetC();
        if(input->Peek() == '>'){
            ch = input->GetC();
            return curr_tok = RSHIFT;
        }
        return curr_tok = Token_value(ch);
    case '<':
        ch = input->GetC();
        if(input->Peek() == '<'){
            ch = input->GetC();
            return curr_tok = LSHIFT;
        }
        return curr_tok = Token_value(ch);
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
    case '.':
        number_value = getNumber();
        return curr_tok = NUMBER;
    case '"':
        ch = input->GetC();
        string_value = getName();
        // Logger::logf(LOG_MSG, _T("Evaluation::prim() in get_token \": name ") + string_value);
        ch = input->GetC();
        //if(tmpC = )
        if( ch == RP ){
            return curr_tok = RP;
        }
        if( ch != '"' ){
            Logger::logf(LOG_MSG, _T("Evaluation::prim() in get_token \": c=%c"), ch);
        }
        return curr_tok = STRING;
    default:
        if(isalpha(ch)){
            string_value = getName();
            ch = input->Peek();
            if(ch == '('){
                ch = input->GetC();
                return curr_tok = FUNC;
            }else{
                   return curr_tok = NAME;
            }
        }
        throw Error::SyntaxError(_T("bad token"));
    }
}

void Evaluation::skip()
{
    no_of_errors++;

    while(input->CanRead()){
        char ch;
        ch = input->GetC();

        switch(ch){
        case '\n':
        case ';':
            input->GetC();
            return;
        }

    }
}

double Evaluation::getNumber()
{
    char ch = input->Peek();
    wxString number;

    while(isdigit(ch) || ch == '.'){
        number += input->GetC();
        ch = input->Peek();
    }

    double d = 0.0; //NAN?
    number.ToDouble(&d);

    return d;
}

wxString Evaluation::getName()
{
    char ch = input->Peek();
    wxString name;

    while(isalnum(ch) || ch == '_' || ch == '.' || ch=='[' || ch == ']' ){
        name += input->GetC();
        ch = input->Peek();
    }
    //Logger::log(LOG_MSG, _T("Evaluation::getName(): ") + name);
    return name;
}

void Evaluation::addFunction(const wxString& name, FunctionBase* func)
{
    functionBaseHash[name] = func;
}

