#ifndef EVALUATION_H
#define EVALUATION_H

#include <wx/sstream.h>
#include <wx/txtstrm.h>
#include "functionbase.h"
#include "common/types/DoubleHash.h"

WX_DECLARE_STRING_HASH_MAP(FunctionBase *, FunctionBaseHash);

namespace Error{
    struct ZeroDivide{};

    struct SyntaxError{
        wxString p;
        SyntaxError(const wxString& q){ p=q; }
    };
}

class Evaluation
{
    public:
        Evaluation();
        virtual ~Evaluation();
        double  calculate(wxInputStream* in);
        double  calculate(const wxString& expr);
        void    setHash(DoubleHash* dh) { table = dh; }
        void    addFunction(const wxString& name, FunctionBase* func);
    protected:
    private:
        enum Token_value{
            NAME,       FUNC,       NUMBER,     END,    STRING,
            LSHIFT,     RSHIFT,
            PLUS='+',   MINUS='-',  MUL='*',    DIV='/',
            PRINT=';',  ASSIGN='=', LP='(',     RP=')', COMMA=',',
            QUERY='?',  COLON=':',  ST='<',     LT='>',
            BIT_AND='&',BIT_OR='|'
       };

        wxInputStream*      input;
        DoubleHash*         table;
        FunctionBaseHash    functionBaseHash;
        double              number_value;
        wxString            string_value;
        Token_value         curr_tok;
        int                 no_of_errors;


        double              expr(bool get);
        double              bitOr(bool get);
        double              bitAnd(bool get);
        double              logical(bool get);
        double              shift(bool get);
        double              addition(bool get);
        double              term(bool get);
        double              prim(bool get);
        Token_value         get_token();
        void                skip();
        double              getNumber();
        wxString            getName();
};

#endif // EVALUATION_H
