#ifndef SERIALPORTSCANNEREVENT_H
#define SERIALPORTSCANNEREVENT_H


#include <wx/event.h>


#include "../vemsprotocol/DeviceInfo.h"


extern const wxEventType wxEVT_SERIAL_PORT_SCANNER;    /// without this MinGW gives undifened referenced linker error


#define EVT_SERIAL_PORT_SCANNER(id, func) \
    wx__DECLARE_EVT1(wxEVT_SERIAL_PORT_SCANNER, id, SerialPortScannerEventHandler(func))


class SerialPortScannerEvent : public wxEvent
{
    public:
        enum { SPS_EVT_DEVICE_FOUND, SPS_EVT_DEVICE_NOT_FOUND, SPS_EVT_CANNOT_OPEN_PORT };

    public:
        SerialPortScannerEvent( int id = 0 ) : wxEvent( id, wxEVT_SERIAL_PORT_SCANNER ) {}
        virtual wxEvent *Clone() const { return new SerialPortScannerEvent(*this); }
        DECLARE_DYNAMIC_CLASS(SerialPortScannerEvent)

        void setDeviceInfo( const DeviceInfo& deviceInfo ) { m_deviceInfo = deviceInfo; }
        void setDevice( ECUInfo::Device device ) { m_deviceInfo.device = device; }
        void setDeviceMode( ECUInfo::EcuMode deviceMode ) { m_deviceInfo.deviceMode = deviceMode; }
        void setPortName( const wxString& portName ) { m_deviceInfo.setPortName( portName ); }
        void setBaudrate( size_t baudrate ) { m_deviceInfo.setBaudrate( baudrate ); }
        void setScanningFinished( bool finished ) { m_isScanningFinished = finished; }
        void setNumPorts( size_t numPorts ) { m_numPorts = numPorts; }
        void setNumBauds( size_t numBauds ) { m_numBauds = numBauds; }
        void setCurrPortIdx( size_t currPortIdx ) { m_currPortIdx = currPortIdx; }
        void setCurrBaudIdx( size_t currBaudIdx ) { m_currBaudIdx = currBaudIdx; }
        void setNextPortName( const wxString& nextPortName ) { m_nextPortName = nextPortName; }

        DeviceInfo getDeviceInfo() const { return m_deviceInfo; }
        ECUInfo::Device     getDevice() const { return m_deviceInfo.device; }
        ECUInfo::EcuMode    getDeviceMode() const { return m_deviceInfo.deviceMode; }

        wxString    getPortName() const { return m_deviceInfo.getPortName(); }
        size_t      getBaudrate() const { return m_deviceInfo.getBaudrate(); }
        bool        isScanningFinished() const { return m_isScanningFinished; }
        size_t getNumPorts() const { return m_numPorts; }
        size_t getNumBauds() const { return m_numBauds; }
        size_t getCurrPortIdx() const { return m_currPortIdx; }
        size_t getCurrBaudIdx() const { return m_currBaudIdx; }
        wxString getNextPortName() const { return m_nextPortName; }

        bool isV3() const { return m_deviceInfo.isV3(); }
        bool isRound() const { return m_deviceInfo.isRound(); }

        bool isBootMode() const { return m_deviceInfo.isBootMode(); }
        bool isManMode() const { return m_deviceInfo.isManMode(); }
        bool isMegatuneMode() const { return m_deviceInfo.isMegatuneMode(); }

    private:
        DeviceInfo  m_deviceInfo;
        bool        m_isScanningFinished;
        size_t      m_numPorts;
        size_t      m_numBauds;
        size_t      m_currPortIdx;
        size_t      m_currBaudIdx;
        wxString    m_nextPortName;

};


typedef void (wxEvtHandler::*SerialPortScannerEventFunction)(SerialPortScannerEvent&);


#define SerialPortScannerEventHandler(func) \
    (wxObjectEventFunction)(wxEventFunction)wxStaticCastEvent(SerialPortScannerEventFunction, &func)


#endif // SERIALPORTSCANNEREVENT_H
