#include "ExpressionFactory.h"
#include "ExpressionNumber.h"
#include "ExpressionFvPlus.h"
#include "ExpressionFvMinus.h"
#include "ExpressionFvMul.h"
#include "ExpressionFvDiv.h"
#include "ExpressionVariable.h"
#include "ExpressionFvAnd.h"
#include "ExpressionFvOr.h"
#include "ExpressionFvXor.h"
#include "ExpressionFvShiftR.h"
#include "ExpressionFvShiftL.h"
#include "ExpressionFvNot.h"
#include "ExpressionFvNegate.h"
#include "ExpressionLP.h"
#include "ExpressionRP.h"
#include "ExpressionComma.h"
#include "ExpressionColon.h"
#include "ExpressionFvTable.h"
#include "ExpressionString.h"
#include "ExpressionFvConditional.h"
#include "ExpressionFvST.h"
#include "ExpressionFvSTOrEqual.h"
#include "ExpressionFvLT.h"
#include "ExpressionFvLTOrEqual.h"
#include "ExpressionFvNotEqualTo.h"
#include "ExpressionFvEqualTo.h"
#include "ExpressionFvFloor.h"
#include "ExpressionFvSum.h"
#include "ExpressionFvGetGPSDistance.h"
#include "ExpressionFvGetGPSCourse.h"
#include "ExpressionFvInterpolate.h"
#include "ExpressionFvInterpolate2D.h"
#include "ExpressionFvInterpolate3D.h"
#include "ExpressionFvSwitchCase8.h"
#include "StringUtil.h"
#include "ExpressionBase.h"
#include "../logger.h"

ExpressionBase* ExpressionFactory::create( const wxString& token, VariableData* data  )
{
        double d;

     // fv:
        if( token == _T("+")){ return new ExpressionFvPlus() ; }
        else if( token == _T("-")){ return new ExpressionFvMinus(); }
        else if( token == _T("*")){ return new ExpressionFvMul(); }
    //  else if( token == _T("**")){ return new ExpressionFvPowerOf(); } power of candidate
        else if( token == _T("/")){ return new ExpressionFvDiv(); }

        else if( token == _T(">>")){return new ExpressionFvShiftR();}
        else if( token == _T("<<")){return new ExpressionFvShiftL();}

        else if( token == _T("!")){return new ExpressionFvNot();}
        else if( token == _T("~")){return new ExpressionFvNegate();}
        else if( ( token == _T("&") ) || ( token == _T("&&") ) ){ return new ExpressionFvAnd(); }
        else if( ( token == _T("|") ) || ( token == _T("||") ) ){ return new ExpressionFvOr(); }
        else if( token == _T("^")){return new ExpressionFvXor();}

        //logical operators
        else if( token == _T("?") ) { return new ExpressionFvConditional(); }
        else if( token == _T("<") ) { return new ExpressionFvST(); }
        else if( token == _T("<=") ) { return new ExpressionFvSTOrEqual(); }
        else if( token == _T(">") ) { return new ExpressionFvLT(); }
        else if( token == _T(">=") ) { return new ExpressionFvLTOrEqual(); }
        else if( token == _T("==") ) { return new ExpressionFvEqualTo(); }
        else if( token == _T("!=") ) { return new ExpressionFvNotEqualTo(); }

        //
        else if( token == _T("(")){ return new ExpressionLP(); }
        else if( token == _T(")")){ return new ExpressionRP(); }
        else if( token == _T(",") ) { return new ExpressionComma(); }
        else if( token == _T(":") ) { return new ExpressionColon(); }
        //functions
        else if( token == _T("table") ) { return new ExpressionFvTable(); }
        else if( token == _T("floor") ) { return new ExpressionFvFloor(); }
        else if( token == _T("sum") ) { return new ExpressionFvSum(); }
        else if( token == _T("getGpsDistance") ) { return new ExpressionFvGetGPSDistance(); }
        else if( token == _T("getGpsCourse") ) { return new ExpressionFvGetGPSCourse(); }
        else if( token == _T("interpolate") ) { return new ExpressionFvInterpolate(); }
        else if( token == _T("interpolate2d") ) { return new ExpressionFvInterpolate2D(); }
        else if( token == _T("interpolate3d") ) { return new ExpressionFvInterpolate3D(); }
        else if( token == _T("switchCase8") ) { return new ExpressionFvSwitchCase8(); }

        // user defined fv or var:

        //else if( isAlphaNum( token) ){}

        else if( StringUtil::isAlpha( token[0] ) ){ return new ExpressionVariable( token, data ); }
      //  else if( token.EndsWith(_T("(")){m_expressionA.Add(new ExpressionFvUserDefined() );}
        //number
        else if( token.ToDouble(&d)){ return new ExpressionNumber(d); }
        else if( token.StartsWith( _T("\"") ) && token.EndsWith( _T("\"") ) )
        {
            wxString strVar = token.Mid( 1, token.length()-2 );
            return new ExpressionString( strVar );

        }

        //String only user defined fv can use string as param!!
//        else if( token.StartsWith(_T("\"")) ){m_expressionA.Add(new ExpressionString(token));}

    return NULL;
}

