#include "ConfigCriteriaParser.h"

#include <wx/xml/xml.h>

#include "../DirHelper.h"
#include "../Symbol.h"
#include "../Utils.h"
#include "../firmwareconfig/ConfigCriteriaData.h"

//TODO verify expressions exist

bool ConfigCriteriaParser::parse(ConfigCriteriaData& data, const wxString& fileName)
{
    //m_criteriaData = *data;
    wxXmlDocument doc;

    if( !doc.Load( DirHelper::GetPath( dCONFIG ) + fileName,
        _T("UTF-8"),wxXMLDOC_KEEP_WHITESPACE_NODES) )
        return false;


    if(doc.GetRoot()->GetName() != _T("ConfigCriteria")){
        return false;
    }


    wxXmlNode* ConfigNodeRoot = doc.GetRoot();

    wxXmlNode* ConfigNode = ConfigNodeRoot->GetChildren();


    while(ConfigNode){
        if(ConfigNode->GetName() == _T("Variables")){
            parseItem(ConfigNode, data.variableHash );
        }else if(ConfigNode->GetName() == _T("Validate")){
            parseRules(ConfigNode, data.criteriaArray );
        }else if(ConfigNode->GetName() == _T("ReplaceRules")){
            parseItem(ConfigNode, data.replaceHash );
        }
        ConfigNode = ConfigNode->GetNext();
    }

    return true;
}

void ConfigCriteriaParser::parseRules(wxXmlNode* node, ConfigCriteriaArray& criteriaArray)
{
    //wxString DefaultExp = node->GetPropVal(_T("expression"), wxEmptyString);
    wxXmlNode* criteriaNode = node->GetChildren();

    while(criteriaNode){
        if(criteriaNode->GetName() == _T("Rule")){
            wxString ruleKey = criteriaNode->GetPropVal(_T("key"), wxEmptyString);
            wxString variable = criteriaNode->GetPropVal(_T("var"), wxEmptyString);
            wxString errorMsg = criteriaNode->GetPropVal(_T("errorMsg"), wxEmptyString);
            wxString warningMsg = criteriaNode->GetPropVal(_T("warningMsg"), wxEmptyString);
            wxString typeStr = criteriaNode->GetPropVal(_T("type"), wxEmptyString);

            wxArrayString vars;
            if ( variable.IsEmpty() == false )
            {
                variable.Replace(_T(" "), _T(""), true );
                Utils::tokenizeToArray( variable, _T(","), vars );
            }

            ConfigCriteria::ConfigCriteriaType type;
            if(typeStr == _T("ALL_ERROR") ){
                type = ConfigCriteria::CC_RULE_ALL_ERROR;
            }else{
                type = ConfigCriteria::CC_RULE_FIRST_MATCH;
            }
            ConfigCriteria *cc = new ConfigCriteria(ruleKey, errorMsg, warningMsg, type, vars );
            parseRuleItem(criteriaNode, cc);
            criteriaArray.Add( cc );
        }

        criteriaNode = criteriaNode->GetNext();
    }
}

void ConfigCriteriaParser::parseRuleItem(wxXmlNode* node, ConfigCriteria *cc)
{
    wxXmlNode* itemNode = node->GetChildren();

    while(itemNode){
        wxString nodeName =  itemNode->GetName();

        if(nodeName == _T("ErrorDetail")){
             wxXmlNode* childNode = itemNode->GetChildren();
             wxString htmlErrorMsg = childNode->GetContent();
             cc->setHtmlErrorMsg(htmlErrorMsg);
        }

        if( nodeName != _T("Item")){
            itemNode = itemNode->GetNext();
            continue;
        }

        wxString strType = itemNode->GetPropVal(_T("type"), wxEmptyString);
        wxString msg = itemNode->GetPropVal(_T("msg"), wxEmptyString);

        wxString vars = itemNode->GetPropVal( _T("var"), wxEmptyString );

        wxArrayString varArr;
        if ( vars.IsEmpty() == false )
        {
            vars.Replace(_T(" "), _T(""), true );
            Utils::tokenizeToArray( vars, _T(","), varArr );
        }
        else
            varArr = cc->getVariables();

        int type = -1;
        if ( strType == _T("ok") ) { type = CCR_TYPE_OK; }
        else if ( strType == _T("warning") ) { type = CCR_TYPE_WARNING; }
        else if ( strType == _T("error") ) { type = CCR_TYPE_ERROR; }
        else { return; }

        //wxString minValue = node->GetPropVal(_T("minValue"), wxEmptyString);
        //wxString maxValue = node->GetPropVal(_T("maxValue"), wxEmptyString);
        //wxString expresion = node->GetPropVal(_T("expression"), wxEmptyString);

        //double dminValue, dmaxValue;

        //maxValue.ToDouble(&dmaxValue);
        //minValue.ToDouble(&dminValue);

        wxXmlNode* childNode = itemNode->GetChildren();
        wxString expression = childNode->GetContent();

        ConfigCriteriaRange *cr1 = new ConfigCriteriaRange(type, expression, msg, varArr );
        cc->addRange( cr1 );

        itemNode = itemNode->GetNext();
    }
}

void ConfigCriteriaParser::parseItem(wxXmlNode* node, SymbolTable& symbolHash )
{
    wxXmlNode* childNode = node->GetChildren();
    while(childNode){
        if(childNode->GetName() == _T("Item")){
            fillSymbolhash(childNode, symbolHash);
        }
        childNode = childNode->GetNext();
    }
}

void ConfigCriteriaParser::fillSymbolhash(wxXmlNode* node, SymbolTable& symbolHash )
{
    wxString key = node->GetPropVal(_T("key"), wxEmptyString);
	if(key == wxEmptyString) return;

    //wxString newKey = node->GetPropVal(_T("newkey"), wxEmptyString);
    wxXmlNode* childNode = node->GetChildren();
    wxString exp = childNode->GetContent();

    Symbol* symbol = new Symbol();
    symbol->setExpr(key,exp);

    symbolHash[key] = symbol;
}

