#ifndef TRIGGER_H
#define TRIGGER_H

#include "../common/typedef.h"
#include "../common/types/StringHash.h"
#include "../common/types/DoubleHash.h"

class FirmwareTriggerConfig
{
    public:
        uint16_t adv;

        uint8_t config11;     // c11, bit7-4:ncyl-1 bit3:TBI bit2:2stroke bit0:0=mpx4115/1=mpx4250
        uint16_t ign_tdcdelay;
        uint8_t trigger_tooth;
        uint16_t toothw16;
        uint8_t tooth_wheel;          // tooth wheel, number of teeth on the wheel
        uint8_t another_trigger_tooth;
        uint8_t primary_trigger;
        #define trigger_multitooth_Nminus1 4   // if 1, then N-1 wheel (0: N-2)
        #define ptrig_coil     1

        uint8_t secondary_trigger;    // trigger2, bit0 0:falling/1:rising, bit1 0:enable/1:disable, bit2 1:filtering, bit3
        #define strig_cam_sync   4       // if 1, then enable cam sync, else measure alien ignition advance
};

class Engine
{
    public:
    uint32_t rpm_period;
    uint8_t ignum;	// http://tiki.vems.hu/tiki/tiki-index.php?page=IgnitionTiming
	uint16_t dwell;		// in 4 usec resolution. (This could be 64 usec resolution and uint8_t?)
	uint16_t trig2spark;	// time from trigger to spark [4 usec] (NOT rpm_period * trig2spark / 4096 !)
	uint16_t proposed_trig2spark; // 4 usec (not angle "factor" like earlier)
	uint8_t spark_phase;	// actually toothcnt relative to the reference tooth for given cyl
	uint8_t proposed_spark_phase;
	uint8_t dwell_phase;
	uint8_t proposed_dwell_phase;
	uint16_t trig2dwell;	// time from trigger to dwell [4 usec]
	uint16_t proposed_trig2dwell;  // 4 usec
};

class TriggerMap
{
        public:
            StringHash mapHash;
            DoubleHash doubleAHash;
            DoubleHash doubleBHash;
            DoubleHash doubleCHash;
};

class Trigger
{
    public:
        Trigger();
        virtual ~Trigger();
        void triggertooth(uint16_t adv, const FirmwareTriggerConfig& config, Engine& engine);

    protected:
    private:
        FirmwareTriggerConfig config;
        Engine engine;
        uint8_t modulo_down( uint8_t tooth_cnt, uint8_t modulus );
        /**static inline */uint8_t audi135( uint8_t toothw );
        /**static inline */uint8_t audi142( uint8_t toothw );
        /**static inline */uint8_t avoid_tooth(uint8_t tooth, uint8_t catt );
        uint16_t delaydeg_rescale(uint16_t delaydeg, uint8_t toothw );
        uint16_t qdegree_to_4usec( uint16_t quarterdegrees, uint8_t toothw );
        /**static inline */uint32_t mult16_8(uint16_t a, uint8_t b);
        uint16_t IGN_BALANCE;
        #define RPM_TOO_LOW 0xFFFF
        #define INVALID_TRIGGER_TOOTH 0x80;

        uint16_t degree2time(uint16_t d, uint8_t magic_const );
        void cli();
        void sei();
        /////////// FROM GLOBAL.H///////////////////////////////////////////
        /**static inline */uint8_t get_total_tooth();
        ////////////// ARITM.H
//        uint32_t Trigger::mult16_8(uint16_t a, uint8_t b);
        /**static inline */uint16_t periodscaler_to_4usec( uint16_t rpmperiodscaler, uint32_t engine_rpm_period );
};



#endif // TRIGGER_H
