#include "ConfigSaver2.h"


#include "../vemsprotocol/SIPRTypes.h"
#include "../vemsprotocol/SIPRPacket.h"
#include "../vemsprotocol/CommunicationController.h"
#include "../events/InputEvent.h"
#include "../logger.h"
#include "../UserLog.h"


#include <wx/sstream.h>


ConfigSaver::ConfigSaver(  CommunicationController* controller, CommunicationDispatcher* dispatcher )
    : CommunicationClientBase( controller, dispatcher )
{
    m_pOutputStream     = NULL;
    m_pProgressDialog   = NULL;
    m_pFile             = NULL;
    m_strFileName       = wxEmptyString;
    m_bShowProgress     = false;
    m_iLineCount        = 0;
}


ConfigSaver::~ConfigSaver()
{
    close();
}


void ConfigSaver::close()
{
    m_iLineCount = 0;

    if ( m_pOutputStream )
    {
        m_pOutputStream->Close();
        delete m_pOutputStream;
        m_pOutputStream = NULL;
    }

    if ( m_pFile )
    {
        m_pFile->Close();
        delete m_pFile;
        m_pFile = NULL;
    }

    if ( m_pProgressDialog )
    {
        m_pProgressDialog->Destroy();
        m_pProgressDialog = NULL;
    }
}


void ConfigSaver::write( uint8_t* data, uint8_t size )
{
    if ( m_pOutputStream )
    {
        if ( m_pOutputStream->IsOk() )
        {
            m_pOutputStream->Write( data, size );

            wxString eol = _T("\n");
            wxStringInputStream ss( eol );

            m_pOutputStream->Write( ss );

            //wxString s( (const char*)data, wxConvISO8859_1, (size_t)size );
            //Logger::logf( LOG_MSG, _T("-----ConfigSaver::write() - ") + s );
        }
    }
}


bool ConfigSaver::init( const wxString& file, const wxString& header, bool showProgress )
{
    close();

    if ( file.IsEmpty() )
    {
        return false; //???
    }

    m_strFileName   = file;
    m_bShowProgress = showProgress;
    m_strHeader     = header;

    return true;
}


void ConfigSaver::process()
{
    m_pFile = new wxFile( m_strFileName, wxFile::write_append );

    if ( !m_pFile->IsOpened() )
    {
        return;
    }

    m_pOutputStream = new wxFileOutputStream( *m_pFile );

    if ( !m_pOutputStream->IsOk() )
    {
        return;
    }

    if ( !m_strHeader.IsEmpty() )
    {
        //wxStringInputStream headerStream( m_strHeader );
        //(*m_pOutputStream) << headerStream;
        m_pOutputStream->Write( m_strHeader.mb_str(), m_strHeader.Length() );
    }

    if ( m_bShowProgress )
    {
        m_pProgressDialog =
            new wxProgressDialog(   _T("Download config in progress!"),
                                    _T("Line:"),
                                    100,
                                    NULL,
                                    wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH |
                                    wxPD_ELAPSED_TIME | wxPD_ESTIMATED_TIME | wxPD_REMAINING_TIME );
                                    //wxPD_AUTO_HIDE | wxPD_APP_MODAL );

    }
    m_pController->save();
    m_pController->request( new SetStateCommand( ECUInfo::MAN ) );
    m_pController->request( new DownloadAllCommand() );
}


void ConfigSaver::OnSIPR( InputEvent& event )
{
    //Logger::logf( LOG_MSG, _T("ConfigSaver::OnSIPR()") );

    SIPRPacket packet = event.getPacket();

    switch ( packet.type() )
    {
        case SIPR_CONFIG_LINE:
        {
            write( packet.data(), packet.size() - 1 );
            if ( m_pProgressDialog )
            {
                m_pProgressDialog->Pulse( wxString::Format( _T("Line: %d"), ++m_iLineCount ) );
            }
        } break;
        case SIPR_TABLES_END:
        case SIPR_CONFIG_ERROR:
        {
            //m_pController->request( new SetStateCommand( ECUInfo::MEGATUNE ) );
            m_pController->restore();
            if ( m_pController->isReadSensors() )
            {
                m_pController->request( new EnableReadRealTimeCommand( true ) );
            }
            close();
            stop();
            wxString msg = _T("Firmware config has been saved to: ") + m_strFileName;
            UserLog::uLog( ULOG_INFO, msg );


        } break;
    }

    event.Skip();
}
