#include "ConfigValidator.h"

#include "../MTConfigHelper.h"

#include "../firmwareconfig/ConfigValidatorMgr.h"
#include "../fileparser/ConfigCriteriaParser.h"

#include "../vemsprotocol/ECUInfo.h"
#include "../FirmwareInfo.h"
#include "../logger.h"

ConfigValidator::ConfigValidator()
{
    m_oldDevice = -1;
    check_parseFile();
}

void ConfigValidator::validate( STDataModelBase* stDataModel )
{
    // parse criterias if necessary ( device changed )
    check_parseFile();
    MTConfigData* mtConfigData = MTConfigHelper::getMTConfigData();
    ConfigValidatorMgr cfValidator( &m_criteriaData, mtConfigData, stDataModel, m_fwInfo );
    StringHash symbols;
    cfValidator.validate( symbols );
    cfValidator.checkInvalidBits( symbols );
    for ( size_t i = 0; i < m_viewers.GetCount(); ++i )
    {
        m_viewers[i]->notify( symbols );
    }
}


void ConfigValidator::check_parseFile()
{
    int device = -1;

    if ( m_fwInfo != NULL )
        device = m_fwInfo->getDevice();

    if ( m_oldDevice == device )
        return;

    // parse it
    wxString file;

    switch ( device )
    {
        case ECUInfo::DEVICE_V3:
            file = _T("ConfigCriteria.xml");
            break;

        case ECUInfo::DEVICE_ROUND:
            file = _T("Round-ConfigCriteria.xml");
            break;

        default:
            break;
    }

    ConfigCriteriaParser::parse( m_criteriaData, file );

    m_oldDevice = device;
}
