#include "FirmwareConfigDifferences.h"


bool FirmwareConfigDifferences::isEmpty() const
{
    return ( missingScalars.IsEmpty() && extraScalars.IsEmpty() &&
             differentScalars.IsEmpty() && missingTables.IsEmpty() &&
             extraTables.IsEmpty() && differentTables.IsEmpty() );
}


void FirmwareConfigDifferences::clear()
{
    missingScalars.Clear();
    filteredScalars.Clear();
    extraScalars.Clear();
    differentScalars.Clear();
    missingTables.Clear();
    extraTables.Clear();
    differentTables.Clear();
}


wxString FirmwareConfigDifferences::summary( const wxString& eol ) const
{
    wxString result;

    if ( !missingScalars.IsEmpty() )
    {
        result += wxString::Format( _T("Found %d missing scalar(s)!"),
            missingScalars.GetCount() ) + eol;
    }

    if ( !filteredScalars.IsEmpty() )
    {
        result += wxString::Format( _T("Found %d intentionally filtered scalar(s)!"),
            filteredScalars.GetCount() ) + eol;
    }

    if ( !extraScalars.IsEmpty() )
    {
        result += wxString::Format( _T("Found %u extra scalar(s)!"),
            extraScalars.GetCount() ) + eol;
    }

    if ( !differentScalars.IsEmpty() )
    {
        result += wxString::Format( _T("Found %u different scalar value(s)!"),
            differentScalars.GetCount() ) + eol;
    }

    if( !missingTables.IsEmpty() )
    {
        result += wxString::Format( _T("Found %u missing tables(s)!"),
            missingTables.GetCount() ) + eol;
    }

    if ( !extraTables.IsEmpty() )
    {
        result += wxString::Format( _T("Found %u extra tables(s)!"),
            extraTables.GetCount() ) + eol;
    }

    if ( !differentTables.IsEmpty() )
    {
        result += wxString::Format( _T("Found %u different table value(s)!"),
                differentTables.GetCount() ) + eol;
    }

    return result;
}


wxString FirmwareConfigDifferences::details( const wxString& tab, const wxString& eol ) const
{
    wxString result;

    if ( !missingScalars.IsEmpty() )
    {
        result += _T("Missing scalar(s) found!") + eol;

        for ( size_t i = 0; i < missingScalars.GetCount(); ++i )
        {
            result += tab + missingScalars[ i ] + eol;
        }
    }

     if ( !filteredScalars.IsEmpty() )
    {
        result += _T("Intentionally filtered scalar(s) found!") + eol;

        for ( size_t i = 0; i < filteredScalars.GetCount(); ++i )
        {
            result += tab + filteredScalars[ i ] + eol;
        }
    }

    if ( !extraScalars.IsEmpty() )
    {
        result += _T("Extra scalar(s) found!") + eol;

        for ( size_t i = 0; i < extraScalars.GetCount(); ++i )
        {
            result += tab + extraScalars[ i ] + eol;
        }
    }

    if ( !differentScalars.IsEmpty() )
    {
        result += _T("Different scalar value(s) found! ") + eol;

        for ( size_t i = 0; i < differentScalars.GetCount(); ++i )
        {
            result += tab + differentScalars[ i ] + eol;
        }
    }

    if( !missingTables.IsEmpty() )
    {
        result += _T("Missing tables(s) found! ") + eol;

        for ( size_t i = 0; i < missingTables.GetCount(); ++i )
        {
            result += tab + missingTables[ i ] + eol;
        }
    }

    if ( !extraTables.IsEmpty() )
    {
        result += _T("Extra tables(s) found! ") + eol;

        for ( size_t i = 0; i < extraTables.GetCount(); ++i )
        {
            result += tab + extraTables[ i ] + eol;
        }
    }

    if ( !differentTables.IsEmpty() )
    {
        result += _T("Different table value(s) found! ") + eol;

        for ( size_t i = 0; i < differentTables.GetCount(); ++i )
        {
            result += tab + differentTables[ i ] + eol;
        }
    }

    return result;
}


///todo - remove
wxString FirmwareConfigDifferences::getSummary() const
{
    return summary( _T("\n") );
}

///todo - remove
wxString FirmwareConfigDifferences::getDetails() const
{
    return details( _T("\t\t"), _T("\n") );
}

