#ifndef FIRMWARECONFIGLOADER_H
#define FIRMWARECONFIGLOADER_H

#include "FirmwareConfigData.h"
#include "../FirmwareInfo.h"
#include "../Symbol.h"
#include "../mtconfigdata.h"
#include "../common/VemsMatrix.h"

class STDataModelBase;

class FirmwareConfigLoader
{
    public:
        FirmwareConfigLoader();
        virtual ~FirmwareConfigLoader();

        void setDestinationFwInfo(FirmwareInfo* ecuFwInfo);
        void setMTConfigData(MTConfigData* mtConfigData);

        bool parse(const wxArrayString& configFiles);
        FirmwareInfo getCfgFwInfo(){ return m_cfgFwInfo; }

        void load(STDataModelBase* modelBase);
        void loadScalars(STDataModelBase* modelBase);
        void loadTables(STDataModelBase* modelBase);

        void calculateTables();
        const SymbolTable& getSymbolTable(){ return m_cfgSymbolH; }
        FirmwareConfigData& getFirmwareConfigData(){return *m_fcData; }
        VemsMatrix<double>* getUserTable(  const wxString& tableName );
        static bool hasCommonAxis(const wxString& xAxisName);

        bool isAlienConfig();
    private:
    //STDataModelBase*    m_modelBase;
    //FirmwareInfo*       m_fwInfo;

    SymbolTable         m_cfgSymbolH;
    //FirmwareConfigData  m_cfgData;
    FirmwareInfo        m_cfgFwInfo;
    FirmwareInfo*       m_currFwInfo;
    MTConfigData        m_mtConfigData;
    MTConfigData*       m_currMTConfigData;
    FirmwareConfigData* m_fcData;

    bool parseCfg(const wxArrayString& configFiles);
    void parseHeader();
    bool parseIni();

    VemsMatrix<uint8_t>* getBinMatrix(VemsMatrix<uint8_t>* oldBin, size_t neededSize);

    VemsMatrix<uint8_t>* getXAxis(FirmwareConfigData& fcData, const wxString& name);
    VemsMatrix<uint8_t>* getYAxis(FirmwareConfigData& fcData, const wxString& name);

    Symbol getXBinSymbol(const SymbolTable& neededTables, const wxString& name);
    Symbol getYBinSymbol(const SymbolTable& neededTables, const wxString& name);

    bool hasRpmKpaHeader(const wxString& name);

    void addTable2Config(const wxString& key, VemsMatrix<uint8_t>* matrix, FirmwareConfigData& fcData);
    void tableAxisHack();

    bool isMSQ(const wxArrayString& configFiles);
    bool parseMSQ(const wxArrayString& configFiles);

};

#endif // FIRMWARECONFIGLOADER_H
