#ifndef FIRMWAREMANAGER_H
#define FIRMWAREMANAGER_H


#include "FirmwareConfigData.h"
#include "FirmwareConfigDifferences.h"
#include "../Symbol.h"
#include "../stdatamanager/STDataModelSerial.h"
#include "../common/types/IntToIntSetHash.h"

class Symbol;
class SymbolTable;
class CommandRequestBase;
class FirmwareConfigParser;
class UserDialogBase;
class FirmwareInfo;
class FirmwareConfigLoader;

class FWConfigUploadManager
{
    public:
                                    FWConfigUploadManager(UserDialogBase* userDialog, FirmwareInfo* ecuFwInfo);
                                    ~FWConfigUploadManager();

        void                        clear();
        bool                        createUploadConfig( const wxArrayString& configFiles );
        void                        createUploadData( const STDataModelBase& deviceModel, bool removeSameBytes = true );
        void                        uploadConfig(CommandRequestBase* controller, int currentConfigType, int uploadConfigType );
        void                        verifyConfig();

        bool                        parseConfigFromFiles( const wxArrayString& configFiles );
        void                        applyFilter(    const StringSet& filter,
                                                    wxArrayString& missing );

        bool                        isFile12x12() { return m_fcDataFile->is12x12(); }
        void                        processConfigLine( const wxString& line );
        FirmwareConfigDifferences*  compare();
//        size_t                      getDirtyCount();
        void                        updateModel();
        void                        updateECU( CommandRequestBase* controller, int currentConfigType, int uploadConfigType );

        wxString                    menuLinks;          //
        wxArrayString               parsedTables;       // parsed tables from file
        FirmwareConfigDifferences   configsVsHeader;    // result of the compare config and header

        FirmwareConfigDifferences*  compareConfig( STDataModelBase* model );
        size_t                      getUploadCount() const { return m_uploadCount; }


    private:
        enum { UMS_WAIT, UMS_WAIT_DATA };

    private:
        FirmwareConfigParser*       m_pConfigParser;
        FirmwareConfigData*         m_fcDataFile;
        FirmwareConfigData          m_fcDataECU;
        STDataModelSerial           m_model;
        SymbolTable                 m_cfgSymbolTable;
        FirmwareInfo*               m_ecuFwInfo;

        int                         m_iState;
        UserDialogBase*             m_userDialog;
        FirmwareConfigLoader*       m_firmwareConfigLoader;
        IntToIntSetHash             m_uploadData;
        size_t                      m_uploadCount;

    private:

        //void                        setTablesOffsetAndSize();
        void                        updateModel( const SymbolTable& symbolTable );
//        void                        updateECU( CommandRequestBase* controller );
        //void                        createHeaderSymbol( const SymbolTable& symbolTable, const wxString& headerName );
        void                        setRawData(Symbol* ptrSymbol, uint8_t data, int page, int offset);
        void                        setVariableData(VariableDataDH& vd, const FirmwareConfigData& fcd);
//        void                        writeAllDirty( const STDataModelBase& model, CommandRequestBase* controller );
        void                        loadDefaultConfig(const wxArrayString& missingScalars, const wxArrayString& missingTables);
        bool                        validateV3Config();
        bool                        validateRoundConfig();
        void                        safetyConfigSetup();
        void                        requestAdvancedBurn( CommandRequestBase* controller, int currentConfigType, int uploadConfigType );
        void                        filter(const wxString& file);
        void                        writeUploadData( CommandRequestBase* controller );
};


#endif // FIRMWAREMANAGER_H
