#include "GUIDescGUIItem.h"

#include "GUIDescStructItem.h"
#include "../customwidgets/wxTextCtrlNullEx.h"
#include "../customwidgets/wxComboBoxNullEx.h"
#include "../customwidgets/wxPanelWithCtrlExBase.h"

#include "../DrawHelper.h"
#include "../logger.h"


#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(GUIDescGUIItemArray);

GUIDescGUIItem::GUIDescGUIItem(const wxString& name, int type, wxWindow* ctrl, wxButton* dialogButton)
{
    Type = type;
    Ctrl = ctrl;
    DialogButton = dialogButton;
    Name = name;
};

wxVariant GUIDescGUIItem::GetValue(wxVariant& defval)
{
    wxClassInfo * ci = Ctrl->GetClassInfo();
    wxString name = ci->GetClassName();

    if( Ctrl->IsKindOf( CLASSINFO(wxTextCtrlNullEx) ) ) {

        wxTextCtrlNullEx * ctrl = wxDynamicCast(Ctrl, wxTextCtrlNullEx);
        if(ctrl->IsNull()) return defval;

        switch(Type){
            case GDSI_COLOR:{
                wxColor c = DrawHelper::ParseExtColorStr(ctrl->GetValue());
                wxVariant res;
                res << c;
                return res;
                }break;

            case GDSI_NUMBER:{
//                long d = defval.GetLong();
//                if(!ctrl->GetValue().ToLong(&d)) return defval;
                double d = defval.GetDouble();
                if(!ctrl->GetValue().ToDouble(&d)) return defval;
                return d;
                break;

            }case GDSI_TEXT:
                return ctrl->GetValue();
                break;
        }

    } else if( Ctrl->IsKindOf( CLASSINFO(wxComboBoxNullEx) ) ) {
        wxComboBoxNullEx * ctrl = wxDynamicCast(Ctrl, wxComboBoxNullEx);
        wxVariant data;
        data = ctrl->GetValue();
        return data;

    } else if( Ctrl->IsKindOf( CLASSINFO(wxPanelWithCtrlExBase) ) ) {
        wxPanelWithCtrlExBase * panel = wxDynamicCast(Ctrl, wxPanelWithCtrlExBase);
        return panel->GetValue();

    } else {

        return defval;
    }

    return defval;
}

void GUIDescGUIItem::SetValue(wxVariant& val)
{
    wxClassInfo * ci = Ctrl->GetClassInfo();
    wxString name = ci->GetClassName();

    if( name == _T("wxTextCtrlNullEx") )  {
        wxTextCtrlNullEx * ctrl = wxDynamicCast(Ctrl, wxTextCtrlNullEx);
        wxString valstr;
        if(val.GetType() == _T("wxColour")) {
            wxColor c;
            c << val;
            valstr = DrawHelper::GetColorAsString(c);
        } else {
            valstr = val.MakeString();
        }

        //ctrl->ChangeValueEx(valstr);
        ctrl->SetValue(valstr);

    } else if( name == _T("wxComboBoxNullEx") )  {
        wxComboBoxNullEx * ctrl = wxDynamicCast(Ctrl, wxComboBoxNullEx);
        /*ctrl->SetStringSelection(val.GetString());*/
        Logger::logf(LOG_MSG,_T("GUIDescGUIItem::SetValue=")+val.GetString());
        ctrl->SetValue(val.GetString());

    } else if( name == _T("wxPanelWithCtrlExBase") )  {
        wxPanelWithCtrlExBase * panel = wxDynamicCast(Ctrl, wxPanelWithCtrlExBase);
        panel->SetValue( val );
    } else {

        return;
    }
}

bool GUIDescGUIItem::IsNull()
{
    ControlColorContentValidator * cccv = dynamic_cast<ControlColorContentValidator*>(Ctrl);

    if( cccv ) return cccv->IsNull();

    return false;
}

void GUIDescGUIItem::ResetChange()
{
    ControlColorContentValidator * cccv = dynamic_cast<ControlColorContentValidator*>(Ctrl);

    if( cccv ) cccv->ResetChange();
}

bool GUIDescGUIItem::IsChanged()
{
    ControlColorContentValidator * cccv = dynamic_cast<ControlColorContentValidator*>(Ctrl);

    if( cccv ) return cccv->IsChanged();

    return false;
}
