#include "GaugeDescManagerXML.h"

#include <wx/xml/xml.h>
//#include "../MTConfigHelper.h"
//#include "../mtconfigdata.h"
#include "../DirHelper.h"
#include "../common/typedef.h"

GaugeDescManagerXML::GaugeDescManagerXML()
{
    //ctor
}

GaugeDescManagerXML::~GaugeDescManagerXML()
{
    //dtor
}

bool GaugeDescManagerXML::SaveGaugeDescriptor(VEMSGaugeDesc * desc)
{
    SaveGaugeDescriptors( m_descFilename );
    return true;
}

bool GaugeDescManagerXML::SaveGaugeDescriptors(const wxString &filename)
{
    wxXmlDocument doc;

    wxXmlNode * parent = new wxXmlNode(wxXML_ELEMENT_NODE, _T("GDs"));

    doc.SetRoot(parent);

    const VEMSGaugeDescArray * arr = &m_gaugeTable;

    for(uint16_t a=0;a<arr->GetCount();a++) {

        wxXmlNode * node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("GaugeDesc"));

        VEMSGaugeDesc * gauge = arr->Item(a);

        ///string
        wxString numf = gauge->GetGaugeNumberFormat();
        node->AddProperty( _T("name"), gauge->GetName() ) ;                                         //Name
        node->AddProperty( _T("type"), gauge->GetType() ) ;                                         //Var
        node->AddProperty( _T("unit"), gauge->GetUnit() ) ;                                         //Units
        node->AddProperty( _T("numformat"), numf ) ;                                                //numformat
        //gauge->GetTitle();

        ///double
        node->AddProperty( _T("Lo"), wxString::Format(numf, gauge->GetMinValue() ) );               //Lo
        node->AddProperty( _T("Hi"), wxString::Format(numf, gauge->GetMaxValue() ) );               //Hi

        for(uint16_t b=0; b < gauge->coloredA.GetCount();b++){
            wxXmlNode * colornode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("Area"));

            colornode->AddProperty( _T("min"), wxString::Format(numf,gauge->coloredA[b]->minValue));
            colornode->AddProperty( _T("max"), wxString::Format(numf,gauge->coloredA[b]->maxValue));
            colornode->AddProperty( _T("colorname"), gauge->coloredA[b]->ColorName );

            node->AddChild(colornode);
        }


        ///int
        node->AddProperty( _T("MajorDiv"), wxString::Format(_T("%i"), gauge->GetMajorDivisions() ) );   //majordiv
        node->AddProperty( _T("MinorDiv"), wxString::Format(_T("%i"), gauge->GetMinorDivisions() ) );   //minordiv
        node->AddProperty( _T("scale"), wxString::Format(_T("%i"), gauge->GetUnitsScaleFactor() ) );    //scale
        node->AddProperty( _T("StartAngle"), wxString::Format(_T("%i"), gauge->GetStartAngle() ) );
        node->AddProperty( _T("EndAngle"), wxString::Format(_T("%i"), gauge->GetEndAngle() ) );
        node->AddProperty( _T("VD"), wxString::Format(_T("%i"), gauge->GetVDecimals() ) );              //vd
        node->AddProperty( _T("LD"), wxString::Format(_T("%i"), gauge->GetLDecimals() ) );              //ld

        node->AddProperty( _T("TitleAngle"), wxString::Format(_T("%i"), gauge->GetTitleAngle() ) );
        node->AddProperty( _T("TitlePos"), wxString::Format(_T("%i"), gauge->GetTitlePos() ) );
        node->AddProperty( _T("UnitAngle"), wxString::Format(_T("%i"), gauge->GetUnitAngle() ) );
        node->AddProperty( _T("UnitPos"), wxString::Format(_T("%i"), gauge->GetUnitPos() ) );
        node->AddProperty( _T("ValueAngle"), wxString::Format(_T("%i"), gauge->GetValAngle() ) );
        node->AddProperty( _T("ValuePos"), wxString::Format(_T("%i"), gauge->GetValPos() ) );

        node->AddProperty( _T("UnitTextHeight") , wxString::Format(_T("%0.3f"), gauge->Properties[_T("UnitTextHeight")].GetDouble()  ) );
        node->AddProperty( _T("TitleTextHeight"), wxString::Format(_T("%0.3f"), gauge->Properties[_T("TitleTextHeight")].GetDouble() ) );
        node->AddProperty( _T("ValueTextHeight"), wxString::Format(_T("%0.3f"), gauge->Properties[_T("ValueTextHeight")].GetDouble() ) );

        node->AddProperty( _T("TickTextRadius"), wxString::Format(_T("%0.3f"), gauge->Properties[_T("TickTextRadius")].GetDouble() ) );
        node->AddProperty( _T("TickTextHeight"), wxString::Format(_T("%0.3f"), gauge->Properties[_T("TickTextHeight")].GetDouble() ) );

        parent->AddChild(node);

    }

    doc.Save( filename );

    return true;
}

bool GaugeDescManagerXML::AddGaugeDescriptor(VEMSGaugeDesc * desc)
{
    m_gaugeTable.Add( desc );
    SaveGaugeDescriptors( m_descFilename );
    return true;
}

bool GaugeDescManagerXML::LoadDescriptors(const wxString &filename)
{

    wxXmlDocument doc;

    //doc.Load( DirHelper::GetPath(dCONFIG) + _T("GaugeDescriptors.xml") );
    doc.Load( filename );

    m_descFilename = filename;

    wxXmlNode *root = doc.GetRoot();

    if( root->GetName() != _T("GDs")) return false;

    VEMSGaugeDesc* gauge;
    double tmp1 = 0.0, tmp2 = 0.0, tmp5 = 0.0, Lo, Hi;
    long tmp3 = 0, tmp4 = 0;


    for( wxXmlNode * gdnode = root->GetChildren() ; gdnode ; gdnode = gdnode->GetNext())
    {

        if( gdnode->GetName() != _T("GaugeDesc") ) continue;

        gauge = new VEMSGaugeDesc();

        wxString numf = gdnode->GetPropVal(_T("numformat"),_T("%i"));

        gdnode->GetPropVal(_T("Lo"),_T("0.0")).ToDouble(&( Lo ));
        gdnode->GetPropVal(_T("Hi"),_T("0.0")).ToDouble(&( Hi ));
        gauge->SetMinValue( Lo );
        gauge->SetMaxValue( Hi );

        for( wxXmlNode * areanode = gdnode->GetChildren() ; areanode ; areanode = areanode->GetNext()) {
            if( areanode->GetName() != _T("Area") ) continue;

            areanode->GetPropVal(_T("min"),_T("0")).ToDouble(&tmp1);
            areanode->GetPropVal(_T("max"),_T("0")).ToDouble(&tmp2);
            wxString cname = areanode->GetPropVal(_T("colorname"),_T("0"));

            if( tmp1 >= tmp2 ) continue;

            if( tmp1 < Lo || tmp2 < Lo || tmp1 > Hi || tmp2 > Hi ) continue;

            gauge->coloredA.Add(new ColoredArea(tmp1,tmp2,cname));

        }

        //gauge->SetGaugeNumberFormat(numf);
        gauge->SetGaugeNumberFormat( numf );

        gauge->SetName( gdnode->GetPropVal(_T("name"),wxEmptyString) );

        gauge->SetType( gdnode->GetPropVal(_T("type"),wxEmptyString) );

        //gauge->SetSymbolName( gauge->m_type;

        //gauge->m_title = gauge->m_type;

        gauge->SetUnit( gdnode->GetPropVal(_T("unit"),wxEmptyString) );



        gdnode->GetPropVal(_T("LD"),_T("0")).ToLong(&( tmp3 ));
        gdnode->GetPropVal(_T("VD"),_T("0")).ToLong(&( tmp4 ));
        gauge->SetLDecimals( tmp3 );
        gauge->SetVDecimals( tmp4 );

        gdnode->GetPropVal(_T("MinorDiv"),_T("0")).ToLong(&( tmp3 ));
        gdnode->GetPropVal(_T("MajorDiv"),_T("0")).ToLong(&( tmp4 ));
        gauge->SetMinorDivisions( tmp3 );
        gauge->SetMajorDivisions( tmp4 );

        gdnode->GetPropVal(_T("scale"),_T("0")).ToLong(&( tmp3 ));
        gauge->SetUnitsScaleFactor( tmp3 );

        gdnode->GetPropVal(_T("StartAngle"),_T("0")).ToLong(&( tmp3 ));
        gdnode->GetPropVal(_T("EndAngle"),_T("0")).ToLong(&( tmp4 ));
        gauge->SetStartAngle( tmp3 );
        gauge->SetEndAngle( tmp4 );


        gdnode->GetPropVal(_T("TitleAngle"),_T("0")).ToLong(&( tmp3 ));
        gdnode->GetPropVal(_T("TitlePos"),_T("0")).ToLong(&( tmp4 ));
        gauge->Properties[_T("TitleAngle")] = tmp3;
        gauge->Properties[_T("TitlePos")] = tmp4;

        gdnode->GetPropVal(_T("UnitAngle"),_T("0")).ToLong(&( tmp3 ));
        gdnode->GetPropVal(_T("UnitPos"),_T("0")).ToLong(&( tmp4 ));
        gauge->Properties[_T("UnitAngle")] = tmp3;
        gauge->Properties[_T("UnitPos")] = tmp4;

        gdnode->GetPropVal(_T("ValueAngle"),_T("0")).ToLong(&( tmp3 ));
        gdnode->GetPropVal(_T("ValuePos"),_T("0")).ToLong(&( tmp4 ));
        gauge->Properties[_T("ValueAngle")] = tmp3;
        gauge->Properties[_T("ValuePos")] = tmp4;



        gdnode->GetPropVal(_T("UnitTextHeight"),_T("0.1")).ToDouble( &tmp1 );
        gdnode->GetPropVal(_T("TitleTextHeight"),_T("0.08")).ToDouble( &tmp2 );
        gdnode->GetPropVal(_T("ValueTextHeight"),_T("0.1")).ToDouble( &tmp5 );
        gauge->Properties[_T("UnitTextHeight")] = tmp1;
        gauge->Properties[_T("TitleTextHeight")] = tmp2;
        gauge->Properties[_T("ValueTextHeight")] = tmp5;


        gdnode->GetPropVal(_T("TickTextRadius"),_T("0.65")).ToDouble( &tmp1 );
        gdnode->GetPropVal(_T("TickTextHeight"),_T("0.08")).ToDouble( &tmp2 );

        gauge->Properties[_T("TickTextRadius")] = tmp1;
        gauge->Properties[_T("TickTextHeight")] = tmp2;


        m_gaugeTable.Add( gauge );


    }

    m_gaugeTable.Sort( & (VEMSGaugeDesc::compare_function) );

    LoadGaugeSymbols( DirHelper::GetPath(dCONFIG) + _T("GaugeSymbols.xml") );

    return true;
}

void GaugeDescManagerXML::LoadGaugeSymbols(const wxString &filename)
{
    wxXmlDocument doc;

    doc.Load( filename );

    wxXmlNode *root = doc.GetRoot();

    if( root->GetName() != _T("Symbols")) return;

    for( wxXmlNode * gdnode = root->GetChildren() ; gdnode ; gdnode = gdnode->GetNext())
    {

        if( gdnode->GetName() != _T("Symbol") ) continue;

        SymbolGaugeDesc * sg = new SymbolGaugeDesc();

        sg->Name = gdnode->GetPropVal(_T("name"),wxEmptyString);
        sg->Type = gdnode->GetPropVal(_T("type"),wxEmptyString);
        sg->Title = gdnode->GetPropVal(_T("title"),wxEmptyString);

        m_symbolDesc[sg->Name] = sg;

    }
}

short GaugeDescManagerXML::ManageColoredArea( short command, ColoredArea * ca, int desc_id )
{
    return -1;
}
/*
wxString GaugeDescManagerXML::GetGaugeSymbolType(const wxString &symbolname)
{

    SymbolTable &table = MTConfigHelper::getMTConfigData()->getSymbolTable();
    if( table.find( symbolname ) != table.end() )
        return table[symbolname]->getGaugeType();

    return wxEmptyString;
}

wxString GaugeDescManagerXML::GetGaugeSymbolTitle(const wxString &symbolname)
{
    SymbolTable &table = MTConfigHelper::getMTConfigData()->getSymbolTable();
    if( table.find( symbolname ) != table.end() )
        return table[symbolname]->getTitle();

    return wxEmptyString;
}
*/
