#ifndef OUTPUTCHANNELSQL_H
#define OUTPUTCHANNELSQL_H

#include <wx/wxsqlite3.h>

#include "../common/types/wxObjectArray.h"
#include "../common/types/IntArray.h"
#include "../common/typedef.h"
#include "../logger.h"


const wxString str_invalid_value  = _T("INVALID");
const wxString str_disabled_value = _T("Disabled");
const wxString str_conflict_value = _T("CONFLICT");

class dbsymbol : public wxObject
{
public:
    wxString name;
    wxString fullname;
};

class OutputType : public wxObject
{
public:
    int channel; // upper 4 bit channel_id, lower 4 bit type_id
    OutputType() {}
    OutputType(const OutputType& ot);
    virtual ~OutputType() {
        //Logger::logf(LOG_MSG, _T("Outputtype descr"));
    }
    wxString pin;
    wxString current;
    int connector_type;
    wxString name;
    wxString title;
    wxString type;
    uint8_t getChannel_id() { uint8_t tmp = channel >> 4; if (tmp>7) tmp-=8; return tmp; }
    uint8_t getType_id() { return channel & 0x0F;}
    bool isDisabled() { return ((channel & 0x0F) == 0x0F); }
    bool isInverted() { return (channel & 128) == 128; }
};

class OutputChannelSQL
{
    public:
        OutputChannelSQL();
        virtual ~OutputChannelSQL();

        static wxObjectArray    getSymbols(bool onlyVisible = true);
        static IntArray         getChannels();
        static OutputType*      getOutputType(uint8_t channel);
        static OutputType*      getOutputType(const wxString& str);
        static wxString         getInjGroupPins();
        static bool             isValidType(int hexid);

        static bool             getIgnitionMapping(bool isDual, uint8_t h2xvalue, uint8_t& type_id, uint8_t& channel_id);
        static IntArray         getIgnitionOutputs(bool isDual);
        static int              getIgnh2x(int channel_id, int type_id, bool isDual );

    protected:
    private:
        static wxSQLite3Database m_db;
        static wxString          s_dbFilename;
        static wxString          getPin(uint8_t channel);
        static unsigned short    m_validtypes;
        static void              openDB();
        static void              getValidTypes();
};

#endif // OUTPUTCHANNELSQL_H
