#include "VemsFlagDescManager.h"

#include <wx/fileconf.h>
#include <wx/wfstream.h>


#include "../logger.h"
#include "../DirHelper.h"
#include "../DrawHelper.h"
#include "../wxVariantHelper.h"

#include "../wxSqliteHelper.h"

#define chkdb(retval) if( ! (m_db.IsOpen()) ) return retval;

VemsFlagStateDesc DefaultVemsFlagStateDesc = VemsFlagStateDesc(-1,_T("default"));
VemsFlagStateDesc DisabledVemsFlagStateDesc = VemsFlagStateDesc(-1,_T("N/A"));

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(VemsFlagDescArray);

VemsFlagDescManager::VemsFlagDescManager()
{
    //ctor
    m_nullDesc.Properties[_T("id")] = 0L;
    m_nullDesc.Properties[_T("name")] = _T("nulldesc");

    m_nullDesc.States[0] = VemsFlagStateDesc(0,_T("nulltext:0"));
    m_nullDesc.States[1] = VemsFlagStateDesc(1,_T("nulltext:1"));

    DisabledVemsFlagStateDesc.SetBackgroundColor(_T("#808080"));
    DisabledVemsFlagStateDesc.SetForegroundColor(_T("#CCCCCC"));
}

VemsFlagDescManager::~VemsFlagDescManager()
{
    //dtor
    m_db.Close();
}

VemsFlagStateDesc& VemsFlagDescManager::GetVemsFlagStateDescById(int state_id)
{
    for(VemsFlagDescHash::iterator it = m_vemsFlagDesc.begin(); it != m_vemsFlagDesc.end(); it++) {
        VemsFlagDesc& desc = it->second;
        for(VemsFlagStates::iterator sit = desc.States.begin(); sit != desc.States.end(); sit++) {
            VemsFlagStateDesc& sdesc = sit->second;
            if( sdesc.Properties[_T("id")].GetLong() == state_id ) return sdesc;
        }
    }

    return DefaultVemsFlagStateDesc;
}

VemsFlagDesc& VemsFlagDescManager::GetVemsFlagDesc(int id)
{
    for( VemsFlagDescHash::iterator it = m_vemsFlagDesc.begin(); it != m_vemsFlagDesc.end(); it++ ) {
        if( it->second.Properties[_T("id")].GetLong() == id ) return it->second;
    }

    return m_nullDesc;
}

VemsFlagDesc& VemsFlagDescManager::GetVemsFlagDesc(const wxString &name)
{
    if(m_vemsFlagDesc.find(name) == m_vemsFlagDesc.end())
        return m_nullDesc;

    return m_vemsFlagDesc[name];
}

VemsFlagStates& VemsFlagDescManager::GetVemsFlagDescStates(const wxString &name)
{
    if(m_vemsFlagDesc.find(name) == m_vemsFlagDesc.end())
        return m_nullDesc.States;

    return m_vemsFlagDesc[name].States;
}

bool VemsFlagDescManager::LoadGroups()
{
    chkdb(false);

    m_flagGroups.Clear();

    wxSQLite3ResultSet res = m_db.ExecuteQuery(_T("select * from groups order by title asc;"));

    while(res.NextRow()){

        wxVariantHash grpdata;

        wxSqliteHelper::SQLLineToVariantHash( res, grpdata );

        m_flagGroups.Add(grpdata);
    }

    return true;
}

bool VemsFlagDescManager::LoadDescriptorFromFile(const wxString& filepath)
{
    if(filepath == wxEmptyString) {
        LOGP(_T("filepath is empty"));
        return false;
    }

    if( wxSqliteHelper::CheckDBFile(filepath) == false ) return false;

    m_db.Open(filepath);

    CheckDB();

    LoadGroups();

    //wxSQLite3ResultSet res = m_db.ExecuteQuery(_T("select * from flags order by title asc;"));
    wxSQLite3ResultSet res = m_db.ExecuteQuery(_T("select f.help_file,group_id,f.id,name,f.title,autosize,g.title group_title from flags f inner join groups g on g.id = f.group_id order by group_title,UPPER(f.title) asc;"));

    m_vemsFlagDesc.clear();

    while(res.NextRow()){

        int flag_id = res.GetInt(_T("id"));

        wxString name = res.GetString(_T("name"));

        VemsFlagDesc desc(flag_id, name);

        desc.Properties[_T("autosize")] = res.GetBool(_T("autosize"));
        desc.Properties[_T("title")] = res.GetString(_T("title"));
        desc.Properties[_T("group_id")] = (long)res.GetInt(_T("group_id"));
        desc.Properties[_T("group_title")] = res.GetString(_T("group_title"));
        desc.Properties[_T("help_file")] = res.GetString(_T("help_file"));

        wxSQLite3ResultSet sres = m_db.ExecuteQuery(wxString::Format(_T("select * from states where flag_id=%i;"),flag_id));

        while(sres.NextRow()){
            VemsFlagStateDesc sdesc;
            wxSqliteHelper::SQLLineToVariantHash( sres, sdesc.Properties );

            desc.States[ sdesc.GetValue() ] = sdesc;

            //Logger::logf(LOG_MSG,_T("sdesc.Properties=%i"),desc.States[ sdesc.GetValue() ].Properties.size());
        }

        m_vemsFlagDesc[name] = desc;
    }


    m_lastDatabaseFile = filepath;

    return true;
}

bool VemsFlagDescManager::CheckDB()
{
    chkdb(false);

    ColumnDescArrayHash desc;

    ColumnDescArray arr1;
    arr1.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr1.Add( ColumnDesc(_T("name"), _T("VARCHAR(64)")) );
    arr1.Add( ColumnDesc(_T("title"), _T("VARCHAR(32)")) );
    arr1.Add( ColumnDesc(_T("autosize"), _T("TINYINT"), 0L) );
    arr1.Add( ColumnDesc(_T("group_id"), _T("INTEGER"), 1L) );
    arr1.Add( ColumnDesc(_T("help_file"), _T("VARCHAR(255)")) );
    desc[_T("flags")] = arr1;

    ColumnDescArray arr2;
    arr2.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr2.Add( ColumnDesc(_T("flag_id"), _T("INTEGER")) );
    arr2.Add( ColumnDesc(_T("value"), _T("TINYINT")) );
    arr2.Add( ColumnDesc(_T("text"), _T("VARCHAR(32)")) );
    arr2.Add( ColumnDesc(_T("fgcolor"), _T("VARCHAR(32)")) );
    arr2.Add( ColumnDesc(_T("bgcolor"), _T("VARCHAR(32)")) );
    arr2.Add( ColumnDesc(_T("fgcolor2"), _T("VARCHAR(32)")) );
    arr2.Add( ColumnDesc(_T("bgcolor2"), _T("VARCHAR(32)")) );
    arr2.Add( ColumnDesc(_T("img"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("img2"), _T("VARCHAR(64)")) );
    arr2.Add( ColumnDesc(_T("flash_time_ms"), _T("INTEGER"), 0L ) );
    desc[_T("states")] = arr2;

    ColumnDescArray arr3;
    arr3.Add( ColumnDesc(_T("id"), _T("INTEGER"), wxNullVariant, true, true ) );
    arr3.Add( ColumnDesc(_T("title"), _T("VARCHAR(32)")) );
    desc[_T("groups")] = arr3;

    wxSqliteHelper::DatabaseCheck(m_db,desc);

    return true;
}

int VemsFlagDescManager::flag_compare_function_title(VemsFlagDesc **A,VemsFlagDesc **B)
{
    wxString titleA = (*A)->Properties[_T("title")].GetString();
    wxString titleB  = (*B)->Properties[_T("title")].GetString();

    return titleA.CmpNoCase(titleB);
}

size_t VemsFlagDescManager::GetAllDesc(VemsFlagDescArray& arr)
{
    for( VemsFlagDescHash::iterator it = m_vemsFlagDesc.begin(); it != m_vemsFlagDesc.end(); it++ ) {
        arr.Add(it->second);
    }
    return arr.GetCount();
}

/*******************************************
 *          VemsFlagStateDesc class
 *******************************************/


VemsFlagStateDesc::VemsFlagStateDesc(long value, const wxString& text)
{
    wxVariantHelper::SetColor(Properties[_T("fgcolor")], wxColor(255,255,255));
    wxVariantHelper::SetColor(Properties[_T("bgcolor")], wxColor(0,0,0));

    Properties[_T("value")] = value;
    Properties[_T("text")] = text;
}

VemsFlagStateDesc::VemsFlagStateDesc()
{
    //ctor
    wxVariantHelper::SetColor(Properties[_T("fgcolor")], wxColor(255,255,255));
    wxVariantHelper::SetColor(Properties[_T("bgcolor")], wxColor(0,0,0));

    Properties[_T("value")] = 0L;
    Properties[_T("text")] = _T("text");
}

VemsFlagStateDesc::~VemsFlagStateDesc()
{
    //dtor
}

void VemsFlagStateDesc::ParseColors()
{
    GetBackgroundColor();
    GetForegroundColor();

    GetBackgroundColor2();
    GetForegroundColor2();
}

wxColor VemsFlagStateDesc::GetForegroundColor2()
{
    wxVariant& var = Properties[_T("fgcolor2")];

    if(var == wxNullVariant) return wxNullColour;

    if( var.GetType() == _T("string") ) var << DrawHelper::ParseExtColorStr( var.GetString() );

    return wxVariantHelper::GetColor(var);
}

wxColor VemsFlagStateDesc::GetBackgroundColor2()
{
    wxVariant& var = Properties[_T("bgcolor2")];

    if(var == wxNullVariant) return wxNullColour;

    if( var.GetType() == _T("string") ) var << DrawHelper::ParseExtColorStr( var.GetString() );

    return wxVariantHelper::GetColor(var);
}

bool VemsFlagStateDesc::IsFlash()
{
    return (Properties[_T("fgcolor2")] != wxNullVariant || Properties[_T("bgcolor2")] != wxNullVariant) ? true : false;
}

long VemsFlagStateDesc::GetFlashInterval()
{
    return (Properties[_T("flash_time_ms")] == wxNullVariant || Properties[_T("flash_time_ms")].GetLong() < 1) ? 500 : Properties[_T("flash_time_ms")].GetLong();
}

wxColor VemsFlagStateDesc::GetBackgroundColor()
{
    wxVariant& var = Properties[_T("bgcolor")];

    if( var.GetType() == _T("string") ) var << DrawHelper::ParseExtColorStr( var.GetString() );

    return wxVariantHelper::GetColor(var);
}

wxColor VemsFlagStateDesc::GetForegroundColor()
{
    wxVariant& var = Properties[_T("fgcolor")];

    if( var.GetType() == _T("string") ) var << DrawHelper::ParseExtColorStr( var.GetString() );

    return wxVariantHelper::GetColor(var);
}

wxString VemsFlagStateDesc::GetText()
{
    return Properties[_T("text")].GetString();
}

int VemsFlagStateDesc::GetValue()
{
    return Properties[_T("value")].GetLong();
}

bool VemsFlagStateDesc::SetBackgroundColor(const wxString &bgcstring)
{
    wxColor c;
    if(!c.Set(bgcstring))
        return false;

    wxVariantHelper::SetColor(Properties[_T("bgcolor")],c);
    return true;

}

bool VemsFlagStateDesc::SetForegroundColor(const wxString &fgcstring)
{
    wxColor c;

    if(!c.Set(fgcstring))
        return false;

    wxVariantHelper::SetColor(Properties[_T("fgcolor")],c);
    return true;

}

bool VemsFlagStateDesc::SetText(const wxString &text)
{
    Properties[_T("text")] = text;
    return true;
}

bool VemsFlagStateDesc::SetValue(long value)
{
    Properties[_T("value")] = value;
    return true;
}

bool VemsFlagStateDesc::SetValue(const wxString &value)
{
    long tmp;

    if (!value.ToLong(&tmp))
        return false;

    Properties[_T("value")] = tmp;

    return true;
}


wxBitmap VemsFlagStateDesc::GetImage()
{
    return wxVariantHelper::GetBitmap( Properties[_T("image")] );
}

bool VemsFlagStateDesc::SetImage(const wxString &imagename)
{
    /// write this function!!!

    //m_image = wxBitmap();

    return true;
}
