#include "SimpleMapParser.h"

#include <wx/regex.h>

#include "../logger.h"

SimpleMapParser::SimpleMapParser( String2IntHash* structaddresses ) : m_structaddr(structaddresses)
{
    Logger::logf( LOG_MSG, _T("Parsing map file...") );
}

SimpleMapParser::~SimpleMapParser()
{
    Logger::logf(LOG_MSG, _T("Parsing map file...finished"));
}

void SimpleMapParser::parseLine()
{
    // test: /0x0081([a-fA-F\d]{4})\s+(\S+)/
    wxRegEx regexp(_T("\\s+0x0081([a-fA-F\\d]{4})\\s+(\\S+)"), wxRE_ADVANCED );

    // test: /0x000000000081([a-fA-F\d]{4})\s+(\S+)/
    wxRegEx regexp2( _T("\\s+0x000000000081([a-fA-F\\d]{4})\\s+(\\S+)"), wxRE_ADVANCED );

    wxString addrS = wxEmptyString;
    wxString structS = wxEmptyString;

    long address;

    if ( regexp.Matches( m_line ) )
    {
        addrS = regexp.GetMatch( m_line, 1 );
        structS = regexp.GetMatch( m_line, 2 );
    }
    else if ( regexp2.Matches( m_line ) )
    {
        addrS = regexp2.GetMatch( m_line, 1 );
        structS = regexp2.GetMatch( m_line, 2 );
    }

    if ( !addrS.IsEmpty() && !structS.IsEmpty() )
    {
        // skip dummy structures
        if ( structS.Find( _T("_complement_dummy ") ) != wxNOT_FOUND )
        {
            return;
        }

        addrS = _T("0x") + addrS;

        addrS.ToLong( &address, 16 );
        (*m_structaddr)[structS] = address;
    }
}
