#ifndef VARIABLEDESCRIPTOR_H
#define VARIABLEDESCRIPTOR_H

#include <wx/wx.h>
#include <wx/dynarray.h>

enum variable_types {
    vt_uint8_t = 1, vt_uint16t, vt_struct
};

struct StructDescriptor;

struct VariableDescriptor : public wxObject {
    wxString            name;

    wxString            strType;            // eg. "uint8_t"

    bool                isElementary;       // elementary: integer types
    bool                isArray;            // is array
    bool                isStruct;           // is struct
    size_t              elemsize;
    size_t              arraySize;
    StructDescriptor    *structDescriptor;  // if it is a struct

    VariableDescriptor()
    {
        name                = wxEmptyString;
        strType             = wxEmptyString;
        isElementary        = false;
        isArray             = false;
        isStruct            = false;
        arraySize           = 0;
        structDescriptor    = NULL;
    }

    size_t              getSize() const { return (isArray) ? elemsize * arraySize : elemsize; }
};

WX_DECLARE_OBJARRAY( VariableDescriptor, VariablesArray );

struct StructDescriptor : public wxObject {
    wxString        name;
    VariablesArray  fields;

    size_t          size;

    StructDescriptor()
    {
        name = wxEmptyString;
        size = 0;
    }

    void    addVariable( VariableDescriptor& var );

    void    printStructure();
};

WX_DECLARE_OBJARRAY( StructDescriptor, StructArray );

class StructureVar {
    public:
       StructDescriptor structType;
       size_t           size; // 1 if it is not an array, >1 otherwise
};

WX_DECLARE_STRING_HASH_MAP( StructureVar, String2StructVarHash );

#endif // VARIABLEDESCRIPTOR_H
