#ifndef EXPRESSIONPARSER_H
#define EXPRESSIONPARSER_H

#include <wx/wx.h>
#include "../../common/types/DoubleHash.h"

#include "ExpressionParserHelper.h"
#include "ExpressionException.h"

class ExpressionParser
{
    public:
        ExpressionParser();
        ExpressionParser( VariableList* varlist );
        ~ExpressionParser();

    public:
        double  parse( const wxString& expression, bool showExceptions = false, bool throwExceptions = false );
        void    addConstant( const wxString& var, const double& value );

        void    reset();

        void    setVariableList( VariableList* list );
        void    createVariableList();

    private:

        enum TOKENTYPE { NOTHING = -1, DELIMITER, NUMBER, VAR, FUNCTION, UNKNOWN };
        enum OPERATORS { LOGAND, LOGOR, LOGNOT,
                         AND, OR, SHL, SHR,             // level 2
                         EQ, NEQ, LT, GT, LTE, GTE,     // level 3
                         PLUS, MINUS,                   // level 4
                         MUL, DIV, MOD, XOR };          // level 5
    private:
        char        expr[EXPR_LEN_MAX+1];     // holds the expression
        char*       e;                        // points to a character in expr

        char        token[NAME_LEN_MAX+1];    // holds the token
        TOKENTYPE   token_type;               // type of the token

        double      result;                   // holds the result of the expression

        VariableList* user_var;

        wxString    m_exprs;
        bool        m_selfCreatedVList;

    private:
        void        getToken();
        double      parse_level1();
        double      parse_level1b();
        double      parse_level2();
        double      parse_level3();
        double      parse_level4();
        double      parse_level5();
        double      parse_level8();
        double      parse_level9();
        double      parse_level10();
        double      parse_number();


        int         get_operator_id( const char op_name[] );
        double      eval_operator( const int op_id, const double &lhs, const double &rhs );
        double      eval_function( const char fn_name[], const double &value );
        double      eval_variable( const char var_name[] );


        // helpers
        inline bool isMinus ( const char c );
        inline bool isWhiteSpace( const char c );
        inline bool isDelimiter( const char c );
        inline bool isNotDelimiter( const char c );
        inline bool isAlpha( const char c );
        inline bool isDigitDot( const char c );
        inline bool isDigit( const char c );
        inline bool isHexDigit( const char c);

        int         row() const { return -1; }
        int         col() const { return e - expr - strlen( token ) + 1; }
};

#endif // EXPRESSIONPARSER_H
