#include "GPSConfigDialog2.h"

#include <wx/statline.h>
#include <wx/sstream.h>

#include "../DirHelper.h"
#include "../SerialPortHelper.h"

#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/SerialConfigMgr.h"

#include "../logger.h"
#include "../VemsTuneController.h"
#include "GPSCommunicationController.h"

#include "../common/ThreadProcess.h"
#include "../web/WebHelper.h"
#include "../ProgressBar.h"
/*
#include <wx/jsonval.h>
#include <wx/jsonreader.h>
*/
int idGPSDetectButton = wxNewId();
int idGPSSaveButton   = wxNewId();
int idGPSTestButton   = wxNewId();
int idGPSCloseButton  = wxNewId();
int idGPSCancelButton = wxNewId();

// event handlers

BEGIN_EVENT_TABLE( GPSConfigDialog2, wxDialog )

    EVT_BUTTON( idGPSDetectButton, GPSConfigDialog2::OnDetect )
    EVT_BUTTON( idGPSSaveButton, GPSConfigDialog2::OnSave )
    EVT_BUTTON( idGPSTestButton, GPSConfigDialog2::OnTest )
    EVT_BUTTON( idGPSCloseButton, GPSConfigDialog2::OnClose )
    EVT_BUTTON( idGPSCancelButton, GPSConfigDialog2::OnCancelButton )

    EVT_GPS_COMMEVT( GPSConfigDialog2::OnGPSEvent )
END_EVENT_TABLE()


GPSConfigDialog2::GPSConfigDialog2( wxWindow* parent, VemsTuneController* vtc ) : wxDialog( parent, wxID_ANY, _T("GPS Communication settings"),
    wxDefaultPosition, wxSize(450, 220), wxSTAY_ON_TOP | wxDEFAULT_DIALOG_STYLE )
{
    SetIcon( wxIcon( _T("vemsLogoIcon2") ) );

    m_vtc = vtc;
    m_gpscommcontroller = NULL;

    m_wasECUOnline = m_vtc->isECUOnline();

    // stop threads
    m_vtc->setECUOnline( false, true );
    m_vtc->setGPSOnline( false );

    // get gps config from config file
    ConfigBase* configBase = ConfigHelper2::getConfig();
    SerialConfigMgr cfgMgr( configBase, _T("/gps") );
    cfgMgr.load( &m_serialConfig );

    // get valid baudrates & convert them to string for the combobox
    IntArray baudRates = SerialPortHelper::getGPSSerialBaudRates();

    for ( size_t i = 0; i < baudRates.GetCount(); ++i )
    {
        m_baudRateS.Add( wxString::Format(_T("%d"), baudRates[i] ) );
    }

    m_saved = false;

    m_bmp.LoadFile( DirHelper::GetPath(dICON) + _T("icon_sat.png"), wxBITMAP_TYPE_PNG );

    m_contentLoader = new ThreadProcess<GPSConfigDialog2>( this );

    buildGUI();
}

GPSConfigDialog2::~GPSConfigDialog2()
{
    // delete the comm controller
    if ( m_gpscommcontroller != NULL )
    {
        delete m_gpscommcontroller;
    }

    // restart ecu if it was online
    if ( m_wasECUOnline )
         m_vtc->setECUOnline( true, true );

    // and start gps too
//    if ( m_found && m_saved && ( m_enabled->GetValue() == _T("yes") ) )
//    {
        m_vtc->setGPSOnline( true, true );
//    }
}

// create the elements of the dialog
void GPSConfigDialog2::buildGUI()
{
    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer* sizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("GPS Settings") );

    wxFlexGridSizer* fgs = new wxFlexGridSizer( 2 );

    wxStaticText* dis = new wxStaticText( this, -1, _T("GPS enabled:") );
    wxArrayString enArr; enArr.Add( _T("no") ); enArr.Add( _T("yes") );

    wxString msg;
    if ( m_serialConfig.isEnabled() )
    {
        msg = _T("yes");
    }
    else
    {
        msg = _T("no");
    }

    m_enabled = new wxComboBox( this, -1, msg, wxDefaultPosition, wxDefaultSize, enArr, wxCB_DROPDOWN | wxCB_READONLY );

    Connect( m_enabled->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, wxCommandEventHandler( GPSConfigDialog2::OnGPSEnabledCombo ) );

    fgs->Add( dis, 0, wxALL, 3 ); fgs->Add( m_enabled, 1, wxEXPAND | wxALL, 3 );

    wxStaticText* txt = new wxStaticText( this, -1, _T("COM port:") );

    wxArrayString ports;
    ports = SerialPortHelper::GetCOMPortList();
    ports.Sort( sortFunction );

    wxString selected = m_serialConfig.getPortName();

    if ( ports.Index( selected ) == wxNOT_FOUND )
        selected = wxEmptyString;

    m_comports = new wxComboBox( this, wxID_ANY, selected, wxDefaultPosition, wxDefaultSize, ports, wxCB_DROPDOWN );

    fgs->Add( txt, 0, wxALL, 3 ); fgs->Add( m_comports, 1, wxEXPAND | wxALL, 3 );

    txt = new wxStaticText( this, -1, _T("Baud rate:") );
    m_baudRate = new wxComboBox( this, wxID_ANY, m_serialConfig.getBaudRateStr(), wxDefaultPosition, wxDefaultSize, m_baudRateS,
     wxCB_DROPDOWN | wxCB_READONLY );

    fgs->Add( txt, 0, wxALL, 3 ); fgs->Add( m_baudRate, 1, wxEXPAND | wxALL, 3 );
    fgs->Add( 10, 10 );

    m_testButton = new wxButton( this, idGPSTestButton, _T("Test") );

    fgs->Add( m_testButton, 0, wxALL, 3 );
    sizer->Add( fgs, 5, wxEXPAND | wxALL, 5 );

    // -------

    wxBoxSizer* bs = new wxBoxSizer( wxVERTICAL );

    wxStaticLine* line = new wxStaticLine( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_HORIZONTAL );

    m_txtDetect = new wxStaticText( this, wxID_ANY, _T("Click on Detect to search for GPS") );
    m_progressbar = new ProgressBar( this, wxID_ANY, wxDefaultPosition, wxSize(220,20) );
    m_progressbar->SetWindowStyle( wxBORDER_SUNKEN );
    m_progressbar->SetProgressStyle( PB_PERCENTTEXT_INNER );
    m_progressbar->SetValueFormat( _T("%.1f %%") );
    m_progressbar->SetValueLimits( 0, 100.0, 1.0 );
    m_progressbar->Update( 0 );

    m_detectButton = new wxButton( this, idGPSDetectButton, _T("Detect") );

    bs->Add( m_txtDetect, 0, wxALL, 5 );
    bs->Add( line, 0, wxEXPAND | wxLEFT | wxRIGHT | wxALIGN_CENTER, 10 );
    bs->Add( m_progressbar, 0, wxALL | wxEXPAND, 5 );
    bs->Add( m_detectButton, 0, wxALL | wxALIGN_CENTER, 3 );

    wxBoxSizer* rs = new wxBoxSizer( wxHORIZONTAL );
    m_statBmp = new wxStaticBitmap( this, -1, wxNullBitmap, wxDefaultPosition, wxSize( 32, 32 ) );
    m_result = new wxRichTextCtrl( this, -1, _T(""), wxDefaultPosition, wxSize( 100, 70 ), wxRE_MULTILINE | wxRE_READONLY | wxBORDER_NONE | wxTE_BESTWRAP );
    m_result->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );

    /*wxWindowList list = this->GetChildren();
    wxWindowList::iterator it = list.begin();

    for ( ; it != list.end(); ++it )
    {
        wxTextCtrl * ctr = dynamic_cast<wxTextCtrl*>(*it);
        //wxClassInfo* inf = (*it)->GetClassInfo();

        //LOGP( inf->GetClassName())
        if ( ctr )
            ctr->ToggleWindowStyle( wxTE_MULTILINE );

    }*/


    rs->Add( m_statBmp, 0, wxALL, 5 );
    rs->Add( m_result, 1, wxEXPAND | wxALL, 5 );

    bs->Add( rs, 1, wxEXPAND | wxALL, 5 );

    wxBoxSizer* bs2 = new wxBoxSizer( wxHORIZONTAL );
    m_saveButton = new wxButton( this, idGPSSaveButton, _T("Save") );
    m_closeButton = new wxButton( this, idGPSCloseButton, _T("Close") );
    m_cancelButton = new wxButton( this, idGPSCancelButton, _T("Cancel") );
    m_cancelButton->Enable( false );

    bs2->Add( m_saveButton, 0, wxALL | wxALIGN_CENTER, 3 );
    bs2->Add( m_closeButton, 0, wxALL | wxALIGN_CENTER, 3 );
    bs2->Add( m_cancelButton, 0, wxALL | wxALIGN_CENTER, 3 );

    bs->Add( bs2, 0, wxALL | wxALIGN_CENTER, 5 );

    sizer->Add( bs, 0, wxEXPAND | wxALL, 5 );

    mainsizer->Add(sizer,0,wxALL|wxEXPAND,5);

    SetSizer( mainsizer );

    mainsizer->Fit( this );
    mainsizer->SetSizeHints( this );

    if ( !m_serialConfig.isEnabled() )
    {
        m_testButton->Enable( false );
        m_baudRate->Enable( false );
        m_comports->Enable( false );
        m_detectButton->Enable( false );
    }
}

/// EVENT HANDLERS

// on close button
void GPSConfigDialog2::OnClose( wxCommandEvent& event )
{
    EndModal( 0 );
}

// on save button
void GPSConfigDialog2::OnSave( wxCommandEvent& event )
{

    bool isEnabled = ( m_enabled->GetValue() == _T("yes") ) ? true : false;

    wxString baudrateStr = m_baudRate->GetValue();

    long baudRate;
    baudrateStr.ToLong( &baudRate );

    m_serialConfig.setEnabled( isEnabled );
    m_serialConfig.setPortName( m_comports->GetValue() );
    m_serialConfig.setBaudRate( baudRate );

    // save values to config file
    ConfigBase* configBase = ConfigHelper2::getConfig();

    SerialConfigMgr cfgMgr( configBase, _T("/gps") );
    cfgMgr.save( &m_serialConfig );

    m_saved = true;

    EndModal( 0 );
}

// select enabled yes/no
void GPSConfigDialog2::OnGPSEnabledCombo( wxCommandEvent& event )
{
    if ( m_enabled->GetValue() == _T("yes") )
    {
        m_testButton->Enable( true );
        m_baudRate->Enable( true );
        m_comports->Enable( true );
        m_detectButton->Enable( true );
    }
    else
    {
        m_testButton->Enable( false );
        m_baudRate->Enable( false );
        m_comports->Enable( false );
        m_detectButton->Enable( false );
    }
}

// on detect button
void GPSConfigDialog2::OnDetect( wxCommandEvent& event )
{
    // reset result & other info
    m_progressbar->Update( 0 );
    m_result->SetValue( _T("") );
    m_statBmp->SetBitmap( wxNullBitmap );

    if ( m_gpscommcontroller != NULL )
    {
        delete m_gpscommcontroller;
    }

    m_found = false;
    m_saved = false;

    enableButtons( false );

    // create new comm controller & start detecting
    m_gpscommcontroller = new GPSCommunicationController( this );

    m_gpscommcontroller->detectGPS();
}

// on cancel button -> abort detecting
void GPSConfigDialog2::OnCancelButton( wxCommandEvent& event )
{
    if ( m_gpscommcontroller )
    {
        m_gpscommcontroller->Abort();   // controller is Interruptable

        m_result->SetValue( _T("Waiting for abort...") );
    }
}

// on test button
void GPSConfigDialog2::OnTest( wxCommandEvent& event )
{
    long baudrate = 0;

    wxString port = m_comports->GetValue();
    wxString br = m_baudRate->GetValue();

    br.ToLong( &baudrate );

    m_result->SetValue( wxEmptyString );
    m_statBmp->SetBitmap( wxNullBitmap );

    wxString label =  _T("Testing port ") + port + _T(", baudrate = ") + br;
    m_txtDetect->SetLabel( label );

    if ( m_gpscommcontroller != NULL )
    {
        delete m_gpscommcontroller;
    }

    m_found = false;
    m_saved = false;

    enableButtons( false );

    // create nem commcontoller and test selected baudrate&port
    m_gpscommcontroller = new GPSCommunicationController( this );

    m_gpscommcontroller->testGPSPort( port, baudrate );
}

// incoming EVT_GPS_COMMEVT (from commcontroller)
void GPSConfigDialog2::OnGPSEvent( GPSCommunicationEvent& event )
{
    // detecting?
    if ( event.isSearching )
    {
        // try new port/br
        if ( event.searchstate == GPSCommunicationEvent::SEARCH_START )
        {
            wxString label = _T("Try to connect on port ") + event.portName;
            label += wxString::Format( _T(" with baudrate %d"), event.baudRate );

            m_txtDetect->SetLabel( label );

            return;
        }

        // no com port or search aborted
        if ( event.searchstate == GPSCommunicationEvent::SEARCH_ABORTED )
        {
            if ( event.state == GPSCommunicationEvent::INVALID_PORT )
            {
                m_txtDetect->SetLabel( _T("No serial port found") );
                m_result->SetValue( _T("") );
            }
            else
            {
                m_txtDetect->SetLabel( _T("Search aborted") );
                m_result->SetValue( _T("GPS not found"));
            }

            m_progressbar->Update( event.userdata );
            m_statBmp->SetBitmap( wxNullBitmap );

            enableButtons( true );

            // drop communication
            delete m_gpscommcontroller;
            m_gpscommcontroller = NULL;

            return;
        }

        // no more ports
        if ( event.searchstate == GPSCommunicationEvent::SEARCH_END )
        {
            m_txtDetect->SetLabel( _T("Search finished") );

            m_result->SetValue( _T("GPS not found") );
            m_progressbar->Update( event.userdata );

            m_statBmp->SetBitmap( wxNullBitmap );

            enableButtons( true );

            // drop communication
            delete m_gpscommcontroller;
            m_gpscommcontroller = NULL;

            return;
        }
    }
    switch ( event.state )
    {
        // time out or unavailable port
        case GPSCommunicationEvent::CANNOT_OPEN_PORT:
        {
            wxString label = _T("Can not open port ") + event.portName;
            m_txtDetect->SetLabel( label );

            m_progressbar->Update( event.userdata );
            break;
        }

        // port and/or br is invalid ( do not exist in comboboxes )
        case GPSCommunicationEvent::INVALID_PORT:
        case GPSCommunicationEvent::INVALID_BAUDRATE:
        {
            wxString label = _T("Portname and/or is invalid: ") + event.portName;
            label += wxString::Format( _T(", %d"), event.baudRate );

            m_txtDetect->SetLabel( label );

            m_progressbar->Update( event.userdata );
            break;
        }

        // port is valid, but no gps data
        case GPSCommunicationEvent::NOT_FOUND:
        {
            wxString label = _T("No valid GPS data found on port ") + event.portName;
            label += wxString::Format( _T(" (baudrate = %d)"), event.baudRate );

            m_txtDetect->SetLabel( label );

            m_progressbar->Update( event.userdata );

            m_result->SetValue( _T("") );

            m_statBmp->SetBitmap( wxNullBitmap );

            break;
        }

        // valid gps data
        case GPSCommunicationEvent::GPS_FOUND:
        {
            wxString brStr;
            brStr << event.baudRate;

            wxString label = _T("GPS found on ") + event.portName + _T(", baudrate = ") + brStr;

            m_txtDetect->SetLabel( label );

            GPSDataStruct& gpsdata = event.gpsdata;


            double latitude	= (double)gpsdata.latitude / 1000000.0;
            double floorLat = floor( latitude );

            m_gpslat = floorLat + ( latitude - floorLat ) / 60 * 100;

            double longitude	= (double)gpsdata.longnitude / 1000000.0;
            double floorLon = floor( longitude );

            m_gpslong = floorLon + ( longitude - floorLon ) / 60 * 100;

            wxString res;

            if ( gpsdata.noofsatellite < 3 )
            {
                res = _T("GPS found, but to few satellites in view");
            }
            else
            {
                res = wxString::Format( _T("GPS found.\n%c %.5f, %c %.5f"), gpsdata.latitudeindicator, m_gpslat,
                         gpsdata.longnitudeindicator, m_gpslong );

                if ( gpsdata.latitudeindicator == 'S' )
                    m_gpslat *= -1.0;
                if ( gpsdata.longnitudeindicator == 'W' )
                    m_gpslong *= -1.0;

                m_contentLoader->start();
            }


            m_result->SetValue( res );

            m_comports->SetValue( event.portName );
            m_baudRate->SetValue( brStr );

            m_statBmp->SetBitmap( m_bmp );

            m_progressbar->Update( event.userdata );

            enableButtons( true );

            // drop communication
            delete m_gpscommcontroller;
            m_gpscommcontroller = NULL;

            break;
        }

        default:
            break;
    }

    if ( event.isTesting )
    {
        enableButtons( true );
    }

}

/// other helpers

// enable or disable gui elements
void GPSConfigDialog2::enableButtons( bool enable )
{
    m_testButton->Enable( enable );
    m_detectButton->Enable( enable );
    m_saveButton->Enable( enable );
    m_closeButton->Enable( enable );
    m_cancelButton->Enable ( !enable );
    m_enabled->Enable( enable );
}

#define OSM_REVERSE_GPS 1

bool GPSConfigDialog2::process()
{
    wxString link;
    int respType = 0;

    #ifdef OSM_REVERSE_GPS
    link = wxString::Format( _T("http://nominatim.openstreetmap.org/reverse?format=json&lat=%.6f&lon=%.6f&zoom=18"), m_gpslat, m_gpslong );
    respType = 1;
    #else
    link = wxString::Format( _T("http://ws.geonames.org/findNearbyJSON?lat=%.6f&lng=%.6f" ), m_gpslat, m_gpslong );
    #endif

    wxString response = WebHelper::DownloadPage( link );

    if ( response.Length() > 0 )
    {
        parseResponse( response, respType );
    }

    return false;
}

// sort com ports
int GPSConfigDialog2::sortFunction( const wxString& first, const wxString& second )
{
    if ( first.length() != second.length() )
        return first.length() - second.length();
    else return first.CmpNoCase( second );
}

void GPSConfigDialog2::parseResponse( const wxString& respstr, int Type )
{
/*    wxJSONValue root;

    wxJSONReader reader;

    int err = reader.Parse( respstr, &root );
    if ( err > 0 )
    {
        LOGP( _T("Invalid server response (not valid JSON)") )
        return;
    }

    wxString pos = m_result->GetValue() + _T("\n");

    // geonames.org
    if ( Type == 0 )
    {
        wxJSONValue geoname = root[_T("geonames")][0];

        wxString name = geoname[_T("name")].AsString();
        wxString countryName = geoname[_T("countryName")].AsString();

        pos += name + _T(", ") + countryName;
    }
    // openstreetmaps
    else if ( Type == 1 )
    {
        pos += root[_T("display_name")].AsString();
    }

    m_result->SetValue( pos );*/
}
