#include "FileOpenDlgHelper.h"
#include "FileOpenSQL.h"
#include "../DirHelper.h"

/// FILEOPEN_ART_PROVIDER

FileOpenArtProvider::FileOpenArtProvider()
{
    wxString icon_dir = DirHelper::GetPath( dICON );
    wxBitmap folder( icon_dir + _T("icon_folder.png"), wxBITMAP_TYPE_PNG );
    wxBitmap pfolder( icon_dir + _T("icon_parent_folder.png"), wxBITMAP_TYPE_PNG );
    wxBitmap vemsfile( icon_dir + _T("default.png"), wxBITMAP_TYPE_PNG );
    wxBitmap cfgfile( icon_dir + _T("icon_nfile.png"), wxBITMAP_TYPE_PNG );

    wxBitmap iconUp( icon_dir + _T("icon_up.png"), wxBITMAP_TYPE_PNG );
    wxBitmap iconDown( icon_dir + _T("icon_down.png"), wxBITMAP_TYPE_PNG );
    wxBitmap iconAnalyze( icon_dir + _T("icon_analyze.png"), wxBITMAP_TYPE_PNG );
    wxBitmap iconSearch( icon_dir + _T("icon_search.png"), wxBITMAP_TYPE_PNG );
    wxBitmap iconMagnify( icon_dir + _T("icon-search.png"), wxBITMAP_TYPE_PNG );
    wxBitmap iconCompass( icon_dir + _T("icon_compass.png"), wxBITMAP_TYPE_PNG );
    wxBitmap iconSatellite( icon_dir + _T("icon_sat2.png"), wxBITMAP_TYPE_PNG );

    m_bitmaps[FO_GO_UP_ICON] = pfolder;
    m_bitmaps[FO_FOLDER_ICON] = folder;
    m_bitmaps[FO_VEMS_FILE] = vemsfile;
    m_bitmaps[FO_ICON_DOWN] = iconDown;
    m_bitmaps[FO_ICON_UP] = iconUp;
    m_bitmaps[FO_VEMS_CFG] = cfgfile;
    m_bitmaps[FO_ICON_ANALYZE] = iconAnalyze;
    m_bitmaps[FO_ICON_SEARCH] = iconSearch;
    m_bitmaps[FO_ICON_MAGNIFY] = iconMagnify;
    m_bitmaps[FO_ICON_GPS] = iconCompass;
    m_bitmaps[FO_ICON_SAT] = iconSatellite;
}

wxBitmap FileOpenArtProvider::CreateBitmap( const wxArtID& id, const wxArtClient& client, const wxSize& size )
{
    wxBitmap bmp = m_bitmaps[id];

    int w = size.x, h = size.y;

    if ( bmp.GetWidth() == w && bmp.GetHeight() == h )
        return bmp;

    wxImage img = bmp.ConvertToImage();
    img.Rescale( w, h, wxIMAGE_QUALITY_HIGH );

    return img;
}

/// FILEDIALOGSETTINGSDIALOG

int fdsMoveUp = wxNewId();
int fdsMoveDown = wxNewId();

BEGIN_EVENT_TABLE( FileDialogSettingsDialog, wxDialog )
    EVT_BUTTON( fdsMoveDown, FileDialogSettingsDialog::OnMoveButton )
    EVT_BUTTON( fdsMoveUp, FileDialogSettingsDialog::OnMoveButton )
END_EVENT_TABLE()

FileDialogSettingsDialog::FileDialogSettingsDialog( wxWindow* parent )
    : wxDialog( parent, wxID_ANY, _T("File dialog settings"), wxDefaultPosition, wxDefaultSize,
        wxDEFAULT_DIALOG_STYLE | wxSTAY_ON_TOP )
{
    wxBoxSizer* sizer = new wxBoxSizer( wxVERTICAL );

    wxStaticText* txt = new wxStaticText( this, -1, _T("Select visible items") );

    sizer->Add( txt, 0, wxALL, 5 );

    ColumnInfoA columns = FileDialogSettings::getColumns( false );

    m_listBox = new wxCheckListBox( this, -1, wxDefaultPosition, wxDefaultSize );

    for ( size_t i = 0; i < columns.GetCount(); ++i )
    {
        FD_ColumnInfo* ci = columns[i];
        m_listBox->Append( ci->name );
        m_listBox->Check( i, ci->visible );
    }
#ifdef __WXMSW__
    m_listBox->GetItem(0)->SetTextColour( wxColor( 100, 100, 100 ) );
    m_listBox->GetItem(0)->SetCheckable( false );
#endif
    WX_CLEAR_ARRAY( columns );

    wxButton* btn = new wxButton( this, -1, _T("Save and close") );

    wxBoxSizer* bs = new wxBoxSizer( wxHORIZONTAL );
    wxButton* mvUp = new wxButton( this, fdsMoveUp, _T("Move up") );
    wxButton* mvDw = new wxButton( this, fdsMoveDown, _T("Move down") );
    bs->Add( mvUp, 0, wxALL | wxALIGN_CENTER, 5 );
    bs->Add( mvDw, 0, wxALL | wxALIGN_CENTER, 5 );

    btn->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(FileDialogSettingsDialog::OnSave ), NULL, this );
    m_listBox->Connect( wxEVT_COMMAND_CHECKLISTBOX_TOGGLED, wxCommandEventHandler(FileDialogSettingsDialog::OnCheck ), NULL, this );

    sizer->Add( m_listBox, 0, wxEXPAND | wxALL, 5 );
    sizer->Add( bs, 0, wxEXPAND | wxALL | wxALIGN_CENTER, 5 );
    sizer->Add( btn, wxALL | wxALIGN_CENTER, 5 );

    SetSizer( sizer );
    sizer->Fit( this );
    sizer->SetSizeHints( this );
}

void FileDialogSettingsDialog::OnSave( wxCommandEvent& event )
{
    for ( size_t i = 0; i < m_listBox->GetCount(); ++i )
    {
        bool isChecked = m_listBox->IsChecked( i );
        wxString name = m_listBox->GetString( i );
        FileDialogSettings::setColumn( name, isChecked, i );
    }
    EndModal( 1 );
}

void FileDialogSettingsDialog::OnCheck( wxCommandEvent& event )
{
    int id = event.GetSelection();
    // name is always visible
    if ( m_listBox->GetString(id) == _T("Name") && m_listBox->IsChecked(id) == false )
    {
        m_listBox->Check( id, true );
        return;
    }
    event.Skip();
}

void FileDialogSettingsDialog::SwapLines( const int& idx1, const int& idx2 )
{
    wxString tmpS = m_listBox->GetString( idx2 );
    bool tmpB = m_listBox->IsChecked( idx2 );

    m_listBox->SetString( idx2, m_listBox->GetString( idx1 ) );
    m_listBox->Check( idx2, m_listBox->IsChecked( idx1 ) );

    m_listBox->SetString( idx1, tmpS );
    m_listBox->Check( idx1, tmpB );

    m_listBox->SetSelection( idx2 );
}

void FileDialogSettingsDialog::OnMoveButton( wxCommandEvent& event )
{
    int id = event.GetId();
    int selected = m_listBox->GetSelection();


    if ( id == fdsMoveUp )
    {
        if ( selected > 1 ) // Name column is always the first column
        {
            SwapLines( selected, selected -1 );
        }
    }
    else if ( id == fdsMoveDown )
    {
        if ( selected < (int)(m_listBox->GetCount()) - 1 && selected != 0 ) // selected!=0 -> Name column is always the first column
        {
            SwapLines( selected, selected + 1 );
        }
    }
}
