#include "FileOpenSQL.h"

#include <wx/regex.h>
#include <wx/filename.h>
#include <wx/dir.h>

#include "DT.h"

#include "../fileparser/FirmwareConfigParser.h"
#include "../firmwareconfig/FirmwareConfigData.h"

#include "../vemsprotocol/VemsLogHeaderParser.h"

#include "../DirHelper.h"
#include "../wxSqliteHelper.h"
#include "../logger.h"

const wxString foSQL_db = _T("vemsdetails.db");

/// FileDialogSettings
wxSQLite3Database FileDialogSettings::m_db;

/* static */
void FileDialogSettings::getSorting( uint8_t& column, bool& ascending )
{
    checkDb();
    openDb();

    wxString query = _T("SELECT * FROM defaultsort");
    try {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery( query );
        while ( rs.NextRow() )
        {
            column = rs.GetInt( _T("defcol") );
            ascending = (bool) ( rs.GetInt( _T("ascending") ) );
        }
    }
    catch ( wxSQLite3Exception e )
    {
        column = 0; ascending = 1;
    }
}

/* static */
ColumnInfoA FileDialogSettings::getColumns( bool onlyVisible )
{
    ColumnInfoA res; WX_CLEAR_ARRAY( res );

    checkDb();
    openDb();

    wxString query = _T("SELECT * FROM columns");
    if ( onlyVisible ) query += _T(" WHERE visible=1");
    query += _T(" ORDER by c_order ASC");

    try {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery( query );
        while ( rs.NextRow() )
        {
            FD_ColumnInfo* ci = new FD_ColumnInfo;

            ci->name = rs.GetString( _T("name") );
            ci->order = rs.GetInt( _T("c_order") );
            ci->visible = rs.GetBool( _T("visible") );
            ci->column_width = rs.GetDouble( _T("colwidth") );
            ci->id = rs.GetInt( _T("id") );
            res.Add( ci );
        }
    }
    catch ( wxSQLite3Exception e )
    {
    }
    return res;
}

/* static */
wxSize FileDialogSettings::getDlgSize()
{
    checkDb();
    openDb();
    wxSize size( -1, -1 );

    wxString query = _T("SELECT * FROM dlgsize");
    try {
        wxSQLite3ResultSet res = m_db.ExecuteQuery( query );

        if ( res.NextRow() )
        {
            int w = res.GetInt( _T("width") );
            int h = res.GetInt( _T("height") );
            size.Set( w, h );
        }
    }
    catch ( wxSQLite3Exception e )
    {
    }
    return size;
}

/* static */
void FileDialogSettings::setDlgSize( const wxSize& size )
{
    checkDb();
    openDb();

    wxString query = wxString::Format( _T("UPDATE dlgsize SET width=%d, height=%d"), size.GetWidth(), size.GetHeight() );
    try {
        m_db.ExecuteUpdate( query );
    }
    catch ( wxSQLite3Exception e )
    {
    }
}

/* static */
void FileDialogSettings::setColumn( const wxString& columnname, bool visible, int order )
{
    checkDb();
    openDb();

    wxString query = _T("UPDATE columns SET visible=");
    if ( visible ) query += _T("1"); else query += _T("0");
    if ( order > -1 ) query += wxString::Format( _T(", c_order=%d"), order );
    query += _T(" WHERE name='") + columnname + _T("'");

    try
    {
        m_db.ExecuteUpdate( query );
    }
    catch ( wxSQLite3Exception e )
    {
    }
}

/* static */
void FileDialogSettings::setSorting( uint8_t column, bool ascending )
{
    checkDb();
    openDb();

    wxString query = wxString::Format( _T("UPDATE defaultsort SET defcol=%d, ascending=%d"), column, (int)ascending );
    try {
        m_db.ExecuteUpdate( query );
    }
    catch ( wxSQLite3Exception e )
    {
    }
}

/* static */
void FileDialogSettings::openDb()
{
    if ( m_db.IsOpen() )
        return;

    wxString fileName = DirHelper::GetPath( dCONFIG ) + _T("filedlgsettings.db");
    m_db.Open( fileName );
}

/* static */
void FileDialogSettings::checkDb()
{
    // table 1 - columninfo
    wxArrayString columns;
    /// IMPORTANT: THIS ORDER IS FIX, ALWAY APPEND ITEMS AT THE END OF THE LIST!! (SEE ID & FILEOPENDLG.CPP)
    columns.Add(_T("Name"));
    columns.Add(_T("Size"));
    columns.Add(_T("Type"));
    columns.Add(_T("Date Modified"));
    columns.Add(_T("Firmware Version"));
    columns.Add(_T("RPM Max"));
    columns.Add(_T("MAP Max"));
    columns.Add(_T("No.of trigger errors"));
    columns.Add(_T("GPS data"));

    //
    wxVariant v = 1;
    ColumnDesc id( _T("id"), _T("INTEGER"), wxNullVariant, true, true );
    ColumnDesc name( _T("name"), _T("VARCHAR(255)") );
    ColumnDesc visible( _T("visible"), _T("TINYINT"), v );
    ColumnDesc order( _T("c_order"), _T("TINYINT") );
    ColumnDesc width( _T("colwidth"), _T("DOUBLE") );

    ColumnDescArray cols;
    cols.Add( id ); cols.Add( name ); cols.Add( visible ); cols.Add( order );
    cols.Add( width );

    // table 2 - size
    ColumnDesc w( _T("width"), _T("INTEGER"), wxNullVariant );
    ColumnDesc h( _T("height"), _T("INTEGER"), wxNullVariant );

    ColumnDescArray cols2;
    cols2.Add( w ); cols2.Add( h );

    // table 3 - default sort
    ColumnDesc defcol( _T("defcol"), _T("INTEGER"), 0L );
    ColumnDesc colorder(_T("ascending"), _T("INTEGER"), 1L);
    ColumnDescArray cols3;
    cols3.Add( defcol ); cols3.Add( colorder );


    ColumnDescArrayHash table;
    table[ _T("columns") ] = cols;
    table[ _T("dlgsize") ] = cols2;
    table[ _T("defaultsort")] = cols3;

    wxString fileName = DirHelper::GetPath( dCONFIG ) + _T("filedlgsettings.db");

    wxSqliteHelper::DatabaseCheck( fileName, table );

    // alway append at the end
    double colwidth[9] = { 0.375, 0.114, 0.136, 0.217, 0.158, 0.1, 0.1, 0.15, 0.15 };

    openDb();
    // check all columns
    for ( size_t i = 0; i < columns.GetCount(); ++i )
    {
        try {
            wxString q = _T("SELECT * FROM columns WHERE name='") + columns[i] + _T("'");
            wxSQLite3ResultSet res = m_db.ExecuteQuery( q );
            if ( res.NextRow() == false )
            {
                wxString query = _T("INSERT INTO columns(id, name, visible, c_order,colwidth) VALUES (NULL,'") + columns[i] + _T("', 1, ");
                query += wxString::Format( _T(" %d,%f)"), i, colwidth[i] );
                m_db.ExecuteUpdate( query );
            }
        }
        catch (wxSQLite3Exception e ) {}
    }
    // size
    try {
        wxSQLite3Table res = m_db.GetTable( _T("SELECT * FROM dlgsize") );
        if ( ! res.GetRowCount() )
        {
            wxString query = _T("INSERT INTO dlgsize(width,height) VALUES (-1,-1)");
            m_db.ExecuteUpdate( query );
        }
    }
    catch ( wxSQLite3Exception e ) {}

    // sorting
    try {
        wxSQLite3Table res = m_db.GetTable( _T("SELECT * FROM defaultsort") );
        if ( ! res.GetRowCount() )
        {
            wxString query = _T("INSERT INTO defaultsort(defcol,ascending) VALUES (0,1)"); // column 0, ASCending order
            m_db.ExecuteUpdate( query );
}
    }
    catch ( wxSQLite3Exception e ) {}
}

/// FileOpenSQL

FileOpenSQL::FileOpenSQL()
{
}

FileOpenSQL::FileOpenSQL( const wxString& path )
{
    m_path = path;

    if ( path.EndsWith(_T("\\")) || path.EndsWith(_T("/")) )
        m_path = m_path.RemoveLast();

//    if ( enumFiles() == false )
//        Logger::logf( LOG_MSG,_T("FileOpenSQL ctor() - directory does not contains cfg/log file") );

    enumFiles();
}

void FileOpenSQL::setPath( const wxString& path )
{
    m_path = path;
    if ( path.EndsWith(_T("\\")) || path.EndsWith(_T("/")) )
        m_path = m_path.RemoveLast();

//    if ( enumFiles() == false )
//        Logger::logf( LOG_MSG,_T("FileOpenSQL setPath() - directory does not contains cfg/log file") );

    enumFiles();
}

void FileOpenSQL::openDb()
{
    if ( m_db.IsOpen() )
        return;

    wxString fileName = m_path + DirHelper::Separ() + foSQL_db;
    m_db.Open( fileName );
}

void FileOpenSQL::checkDb()
{
    ColumnDesc id( _T("id"), _T("INTEGER"), wxNullVariant, true, true );
    ColumnDesc fname( _T("fname"), _T("VARCHAR(255)") );
    ColumnDesc fsize( _T("fsize"), _T("INTEGER") );
    ColumnDesc ftype( _T("ftype"), _T("VARCHAR(255)") );
    ColumnDesc vmajor( _T("fw_major"), _T("INTEGER") );
    ColumnDesc vminor( _T("fw_minor"), _T("INTEGER") );
    ColumnDesc vbuild( _T("fw_build"), _T("INTEGER") );
    ColumnDesc fdate( _T("fdate"), _T("INTEGER") );
    ColumnDesc rpmmax( _T("rpmmax"), _T("INTEGER") );
    ColumnDesc mapmax( _T("mapmax"), _T("INTEGER") );
    ColumnDesc trigerr( _T("trigerr"), _T("INTEGER") );
    ColumnDesc gpsdata( _T("hasGPS"), _T("TINYINT") );

    ColumnDescArray cols; cols.Add( id ); cols.Add( fname ); cols.Add( fsize ); cols.Add( fdate );
    cols.Add( ftype ); cols.Add( vmajor ); cols.Add( vminor ); cols.Add( vbuild );
    cols.Add( rpmmax ); cols.Add( mapmax ); cols.Add( trigerr ); cols.Add( gpsdata );

    ColumnDescArrayHash table;
    table[ _T("files") ] = cols;

    wxString fileName = m_path + DirHelper::Separ() + foSQL_db;

    wxSqliteHelper::DatabaseCheck( fileName, table );
}

bool FileOpenSQL::enumFiles()
{
    wxArrayString filesA;

    // get config and log files
    wxDir::GetAllFiles( m_path, &filesA, _T("*.vemslog"), wxDIR_FILES | wxDIR_HIDDEN );
    wxDir::GetAllFiles( m_path, &filesA, _T("*.triggerlog"), wxDIR_FILES | wxDIR_HIDDEN );
    wxDir::GetAllFiles( m_path, &filesA, _T("*.vemscfg"), wxDIR_FILES | wxDIR_HIDDEN );

    // do not create vemsdetails.db when selected directory does not contain any log or cfg file
    if ( filesA.GetCount() == 0 )
    {
        observerNotify( 100 );
        return false;
    }

    // check database integrity & open it
    checkDb();
    openDb();

    size_t totalFiles = filesA.GetCount();

    for ( size_t i = 0; i < totalFiles; ++i )
    {
        wxFileName fname( filesA[i] );

        if ( fileExists( fname.GetFullName() ) )
        {
            continue;
        }
        else
        {
            wxString fwver;

            wxString ext = fname.GetExt();
            wxString ftype;
            wxString extl = ext.Lower();

            if ( extl == _T("vemslog") )
                ftype = _T("VEMSLOG file");
            if ( extl == _T("vemscfg") )
                ftype = _T("VEMSCFG file");
            if ( extl == _T("triggerlog") )
                ftype = _T("VEMS trigger log");

            wxDateTime dt;
            if ( fname.GetTimes( NULL, &dt, NULL ) == false )
            {
                //dt = fname.GetModificationTime();
            }

            unsigned long fdate = dt.GetTicks();

            if ( extl == _T("vemslog") || extl == _T("triggerlog") )
            {
                wxFile f;
                f.Open( fname.GetFullPath(), wxFile::read );
                VemsLogHeaderParser parser( f );
                StringHash hh = parser.getHeaderHash();
                fwver = hh[_T("Version")];
            }
            else
            {
                FirmwareConfigData fwdata;
                FirmwareConfigParser parser( &fwdata );
                parser.parse( fname.GetFullPath() );
                fwver = fwdata.getFWVersion();
            }

            unsigned int fwMatch = 0;

            wxRegEx reg(_T("([0-9]+)\\.([0-9]+)\\.([0-9]+)"));
            wxRegEx reg2(_T("([0-9]+)\\.([0-9]+)"));

            //LOGP( fwver )

            if ( reg.Matches( fwver ) ) fwMatch = 3;
            else if ( reg2.Matches( fwver ) ) fwMatch = 2;

            if ( fwMatch == 3 || fwMatch == 2 )
            {
                wxString strMaj, strMin, strBld;

                if ( fwMatch == 3 )
                {
                    strMaj = reg.GetMatch( fwver, 1 );
                    strMin = reg.GetMatch( fwver, 2 );
                    strBld = reg.GetMatch( fwver, 3 );
                }
                if ( fwMatch == 2 )
                {
                    strMaj = reg.GetMatch( fwver, 1 );
                    strMin = reg.GetMatch( fwver, 2 );
                    strBld = _T("0");
                }

                long fwmaj, fwmin, fwbuild;
                strMaj.ToLong( &fwmaj );
                strMin.ToLong( &fwmin );
                strBld.ToLong( &fwbuild );

                wxULongLong fsizeU = fname.GetSize();
                unsigned long fsize = fsizeU.ToULong();

                // create query
                wxString query = _T("INSERT INTO files(fname, fsize, ftype, fdate, fw_major, fw_minor, fw_build) VALUES");
                query += _T("('") + fname.GetFullName() + wxString::Format( _T("', %d, "), fsize );
                query += _T("'") + ftype + _T("',") + wxString::Format( _T("%ld, %d, %d, %d)"), fdate, fwmaj, fwmin, fwbuild );

                m_db.ExecuteUpdate( query );
            }


            int percent = (int)((float) ( i + 1 ) * 100.0 / (float) ( totalFiles ));
            observerNotify( percent );
        }
    }
    observerNotify( 100 );

    return true;
}

bool FileOpenSQL::fileExists( const wxString& file )
{

    wxString query;

    try {
        query = _T("SELECT * FROM files WHERE fname = '") + file + _T("'");

        wxSQLite3ResultSet res = m_db.ExecuteQuery( query );

        if ( res.NextRow() )
            return true;
        else
            return false;
    }
    catch ( wxSQLite3Exception e )
    {
        return false;
    }
}

FileInfoA FileOpenSQL::getFiles( int files, FO_order orderby, bool ascending )
{
    FileInfoA result;
    WX_CLEAR_ARRAY( result );

    wxString selector = _T(" WHERE ftype = ");

    switch ( files )
    {
        case VF_VEMS_CFG:
            selector += _T("'VEMSCFG file'");
            break;

        case VF_VEMS_LOG:
            selector += _T("'VEMSLOG file'");
            break;

        case VF_VEMS_TLOG:
            selector += _T("'VEMS trigger log'");
            break;

        case VF_VEMS_ALL:
        default:
            selector = _T("");
            break;
    }

    wxString oby = _T(" ORDER BY ");
    switch ( orderby )
    {
        case FO_OB_DATE: oby += _T("fdate"); break;
        case FO_OB_SIZE: oby += _T("fsize"); break;
        case FO_OB_TYPE: oby += _T("ftype"); break;
        case FO_OB_FIRMWARE:
            {
                if ( !ascending ) oby += _T("fw_major desc, fw_minor desc, fw_build");
                else oby += _T("fw_major asc, fw_minor asc, fw_build"); break;
            }
        case FO_OB_RPM: oby += _T("rpmmax"); break;
        case FO_OB_MAP: oby += _T("mapmax"); break;
        case FO_OB_TRIGERR: oby += _T("trigerr"); break;
        case FO_OB_GPSDATA: oby += _T("hasGPS"); break;
        case FO_OB_FNAME:
        default: oby += _T("fname");
    }
    if ( ascending ) oby += _T(" ASC");
    else oby += _T(" DESC");

    wxString query = _T("SELECT * FROM files") + selector + oby;
    try {
        wxSQLite3ResultSet res = m_db.ExecuteQuery( query );

        while ( res.NextRow() )
        {
            // does it exist?
            wxString fname = res.GetAsString( _T("fname") );
            wxString tfname = m_path + DirHelper::Separ() + fname;

            if ( ! wxFileExists( tfname ) )
            {
                removeFile( fname );
                continue;
            }

            FileInfo* info = new FileInfo;
            info->fileName = res.GetAsString( _T("fname") );
            info->fileSize = res.GetInt( _T("fsize") );
            info->fileType = res.GetAsString( _T("ftype") );
            info->fileDate = res.GetInt( _T("fdate") );
            info->fw_major = res.GetInt( _T("fw_major") );
            info->fw_minor = res.GetInt( _T("fw_minor") );
            info->fw_build = res.GetInt( _T("fw_build") );
            info->mapMax = ( res.IsNull( _T("mapmax") ) ) ? -1 : res.GetInt( _T("mapmax") );
            info->rpmMax = ( res.IsNull( _T("rpmmax") ) ) ? -1 : res.GetInt( _T("rpmmax") );
            info->hasGPSData = ( res.IsNull( _T("hasGPS") ) ) ? -1 : res.GetInt( _T("hasGPS") );

            if ( res.IsNull( _T("trigerr") ) )
                info->triggerErr = -1;
            else
                info->triggerErr = res.GetInt( _T("trigerr") );

            result.Add( info );
        }

        return result;
    }
    catch ( wxSQLite3Exception e )
    {
        return result;
    }
}

void FileOpenSQL::removeFile( const wxString& filename )
{
    try {
        wxString query = _T("DELETE FROM files WHERE fname = '") + filename + _T("'");
        m_db.ExecuteUpdate( query );
    }
    catch ( wxSQLite3Exception e )
    {
    }
}

/* static */
FileInfoA FileOpenSQL::searchFiles( const SDHash& search_criteria )
{
    wxArrayString subdirs;
    DT traverser( subdirs, true );
    wxDir dir( DirHelper::GetPath( dWORKINGDIR ) );
    dir.Traverse( traverser );

    FileInfoA results;
    wxSQLite3Database db;

    wxString query;

    if ( search_criteria.size() < 1 )
        query = _T("SELECT * FROM files");
    else
    {
        query = _T("SELECT * FROM files WHERE");
        SDHash::const_iterator it = search_criteria.begin();

        for ( ; it != search_criteria.end(); ++ it )
        {
            wxString colName = it->first;
            SearchDescr* sd = it->second;

            if ( colName == _T("hasGPS") )
            {
                if ( sd->minval == 1 || sd->maxval == 1 ) query += _T(" (hasGPS=1) AND");
                else if ( sd->minval == 0 && sd->maxval == 0 ) query  += _T(" (hasGPS=0) AND");
            }
            else if ( colName == _T("firmware") )
            {
                if ( (sd->minval == sd->maxval) && ( sd->minval != -1 ) )
                    query += wxString::Format( _T(" ( fw_major * 1000 + fw_minor * 100 + fw_build = %d ) AND"), sd->minval );
                else
                {
                    if ( sd->minval != -1 )
                    {
                        query += wxString::Format(_T(" ( fw_major * 1000 + fw_minor * 100 + fw_build >= %d ) AND"), sd->minval );
                    }
                    if ( sd->maxval != -1 )
                    {
                        query += wxString::Format(_T(" ( fw_major * 1000 + fw_minor * 100 + fw_build <= %d ) AND"), sd->maxval );
                    }
                }
            }
            else
            {
                if ( (sd->minval == sd->maxval) && ( sd->minval != -1 ))
                    query += _T(" (") + colName + wxString::Format( _T(" = %d) AND"), sd->minval );
                else
                {
                    if ( sd->minval != -1 )
                    {
                        query += _T(" (") + colName + wxString::Format( _T(" >= %d) AND"), sd->minval );
                    }
                    if ( sd->maxval != -1 )
                    {
                        query += _T(" (") + colName + wxString::Format( _T(" <= %d) AND"), sd->maxval );
                    }
                }
            }
        }
        query = query.RemoveLast( 4 ); // remove ending AND
    }
    query += _T(" ORDER BY fname ASC");

    Logger::logf( LOG_MSG, _T("Query=") + query );

    for ( size_t i = 0; i < subdirs.GetCount(); ++i )
    {
        wxString dbName = subdirs[i] + DirHelper::Separ() + foSQL_db;
        if ( ! wxFileExists( dbName ) )
            continue;

        db.Open( dbName );

        try {
            wxSQLite3ResultSet res = db.ExecuteQuery( query );
            while ( res.NextRow() )
            {
                // does it exist?
                wxString fname = res.GetAsString( _T("fname") );
                wxString tfname = subdirs[i] + DirHelper::Separ() + fname;

                if ( ! wxFileExists( tfname ) )
                {
                    continue;
                }

                FileInfo* info = new FileInfo;
                info->fileName = subdirs[i] + DirHelper::Separ() + res.GetAsString( _T("fname") );
                info->fileSize = res.GetInt( _T("fsize") );
                info->fileType = res.GetAsString( _T("ftype") );
                info->fileDate = res.GetInt( _T("fdate") );
                info->fw_major = res.GetInt( _T("fw_major") );
                info->fw_minor = res.GetInt( _T("fw_minor") );
                info->fw_build = res.GetInt( _T("fw_build") );
                info->mapMax = ( res.IsNull( _T("mapmax") ) ) ? -1 : res.GetInt( _T("mapmax") );
                info->rpmMax = ( res.IsNull( _T("rpmmax") ) ) ? -1 : res.GetInt( _T("rpmmax") );
                info->hasGPSData = res.GetInt( _T("hasGPS") );

                if ( res.IsNull( _T("trigerr") ) )
                    info->triggerErr = -1;
                else
                    info->triggerErr = res.GetInt( _T("trigerr") );

                results.Add( info );
            }
        }
        catch ( wxSQLite3Exception e ) {}
        db.Close();
    }

    return results;
}


/// FileAnalyzeSQL
wxString FileAnalyzeSQL::m_path = _T("");
wxSQLite3Database FileAnalyzeSQL::m_db;

/* static */
void FileAnalyzeSQL::openDb()
{
    m_db.Close();
    wxString fileName = m_path + DirHelper::Separ() + foSQL_db;
    m_db.Open( fileName );
}

/* static */
void FileAnalyzeSQL::checkDb()
{
    ColumnDesc id( _T("id"), _T("INTEGER"), wxNullVariant, true, true );
    ColumnDesc fname( _T("fname"), _T("VARCHAR(255)") );
    ColumnDesc fsize( _T("fsize"), _T("INTEGER") );
    ColumnDesc ftype( _T("ftype"), _T("VARCHAR(255)") );
    ColumnDesc vmajor( _T("fw_major"), _T("INTEGER") );
    ColumnDesc vminor( _T("fw_minor"), _T("INTEGER") );
    ColumnDesc vbuild( _T("fw_build"), _T("INTEGER") );
    ColumnDesc fdate( _T("fdate"), _T("INTEGER") );
    ColumnDesc rpmmax( _T("rpmmax"), _T("INTEGER") );
    ColumnDesc mapmax( _T("mapmax"), _T("INTEGER") );
    ColumnDesc trigerr( _T("trigerr"), _T("INTEGER") );
    ColumnDesc gpsdata( _T("hasGPS"), _T("TINYINT") );

    ColumnDescArray cols; cols.Add( id ); cols.Add( fname ); cols.Add( fsize ); cols.Add( fdate );
    cols.Add( ftype ); cols.Add( vmajor ); cols.Add( vminor ); cols.Add( vbuild );
    cols.Add( rpmmax ); cols.Add( mapmax ); cols.Add( trigerr ); cols.Add( gpsdata );

    ColumnDescArrayHash table;
    table[ _T("files") ] = cols;

    wxString fileName = m_path + DirHelper::Separ() + foSQL_db;

    wxSqliteHelper::DatabaseCheck( fileName, table );
}

/* static */
bool FileAnalyzeSQL::fileExists( const wxString& file )
{
    wxString query;

    try {
        query = _T("SELECT * FROM files WHERE fname = '") + file + _T("'");

        wxSQLite3ResultSet res = m_db.ExecuteQuery( query );

        if ( res.NextRow() )
            return true;
        else
            return false;
    }
    catch ( wxSQLite3Exception e )
    {
        return false;
    }
}

/* static */
bool FileAnalyzeSQL::addData( const wxString& filename, FileInfo info )
{
    #ifdef WIN32
    wxChar separ = '\\';
    #else
    wxChar separ = '/';
    #endif
    m_path = filename.BeforeLast( separ );

    checkDb();
    openDb();

    wxString short_fname = filename.AfterLast( separ );

    wxString query;

    if ( fileExists( short_fname ) )
    {
        // only update rpmmax, mapmax, triggererror, gpsdata
        int gpsdata = ( info.hasGPSData ) ? 1 : 0;
        query = wxString::Format( _T("UPDATE files SET rpmmax=%d, mapmax=%d, trigerr=%d, hasGPS=%d "), info.rpmMax, info.mapMax, info.triggerErr, gpsdata );
        query += _T("WHERE fname = '") + short_fname + _T("'");
    }
    else
    {
        //
        wxFileName fname( filename );

        wxString fwver;
        wxString ext = fname.GetExt();
        wxString ftype;
        wxString extl = ext.Lower();

        if ( info.fileType.IsEmpty() )
        {
            if ( extl == _T("vemslog") ) ftype = _T("VEMSLOG file");
        }
        else
            ftype = info.fileType;
        if ( ftype == wxEmptyString ) return false;

        wxDateTime dt;
        if ( fname.GetTimes( NULL, NULL, &dt ) == false )
            dt = fname.GetModificationTime();
        unsigned long fdate = dt.GetTicks();

        long fwmaj = 0, fwmin = 0, fwbuild = 0;

        if ( info.fw_major > 0 || info.fw_minor > 0 || info.fw_build > 0 )
        {
            fwmaj = info.fw_major; fwmin = info.fw_minor; fwbuild = info.fw_build;
        }
        else
        {
            wxFile f;
            f.Open( fname.GetFullPath(), wxFile::read );
            VemsLogHeaderParser parser( f );
            StringHash hh = parser.getHeaderHash();
            fwver = hh[_T("Version")];

            LOGP( fwver )

            unsigned int fwMatch = 0;

            wxRegEx reg(_T("([0-9]+)\\.([0-9]+)\\.([0-9]+)"));
            wxRegEx reg2(_T("([0-9]+)\\.([0-9]+)"));

            if ( reg.Matches( fwver ) ) fwMatch = 3;
            else if ( reg2.Matches( fwver ) ) fwMatch = 2;

            if ( fwMatch == 3 || fwMatch == 2 )
            {
                wxString strMaj, strMin, strBld;

                if ( fwMatch == 3 )
                {
                    strMaj = reg.GetMatch( fwver, 1 );
                    strMin = reg.GetMatch( fwver, 2 );
                    strBld = reg.GetMatch( fwver, 3 );
                }
                if ( fwMatch == 2 )
                {
                    strMaj = reg.GetMatch( fwver, 1 );
                    strMin = reg.GetMatch( fwver, 2 );
                    strBld = _T("0");
                }

                strMaj.ToLong( &fwmaj );
                strMin.ToLong( &fwmin );
                strBld.ToLong( &fwbuild );
            }
        }

        wxULongLong fsizeU = fname.GetSize();

        if ( fsizeU == wxInvalidSize )
            return false;
        unsigned long fsize = fsizeU.ToULong();

        int gpsdata = ( info.hasGPSData ) ? 1 : 0;

        query = _T("INSERT INTO files(fname, fsize, ftype, fdate, fw_major, fw_minor, fw_build,rpmmax, mapmax,trigerr, hasGPS) VALUES");
        query += _T("('") + fname.GetFullName() + wxString::Format( _T("', %d, "), fsize );
        query += _T("'") + ftype + _T("',") + wxString::Format( _T("%ld, %d, %d, %d,"), fdate, fwmaj, fwmin, fwbuild );
        query += wxString::Format( _T("%d,%d,%d,%d)"), info.rpmMax, info.mapMax, info.triggerErr, gpsdata );
    }

    try
    {
        m_db.ExecuteUpdate( query );
        return true;
    }
    catch ( wxSQLite3Exception err )
    {
        return false;
    }
    return false;
}

/* static */
bool FileAnalyzeSQL::isAnalyzed( const wxString& filename )
{
    #ifdef WIN32
    wxChar separ = '\\';
    #else
    wxChar separ = '/';
    #endif
    m_path = filename.BeforeLast( separ );

    checkDb();
    openDb();

    wxString short_fname = filename.AfterLast( separ );

    wxString query;

    if ( fileExists( short_fname ) )
    {
        wxString query = _T("SELECT rpmmax, mapmax, trigerr, hasGPS FROM files WHERE fname = '") + short_fname + _T("'");
        try {
            wxSQLite3ResultSet res = m_db.ExecuteQuery( query );
            bool analyzed = !( res.IsNull( _T("rpmmax") ) || res.IsNull( _T("mapmax") ) || res.IsNull( _T("trigerr") ) || res.IsNull( _T("hasGPS") ) );
            return analyzed;
        }
        catch ( wxSQLite3Exception e )
        {
            return false;
        }
    }
    else
        return false;
}

/* static */
void FileAnalyzeSQL::updateGPSdata( const wxString& filename, bool hasGPS )
{
    int hasgps = ( hasGPS ) ? 1 : 0;

    #ifdef WIN32
    wxChar separ = '\\';
    #else
    wxChar separ = '/';
    #endif
    m_path = filename.BeforeLast( separ );

    checkDb();
    openDb();

    wxString short_fname = filename.AfterLast( separ );

    wxString query = wxString::Format( _T("UPDATE files SET hasGPS = %d WHERE fname='"), hasgps );
    query += short_fname + _T("'");

    try
    {
        m_db.ExecuteUpdate( query );
    }
    catch ( wxSQLite3Exception e )
    {
    }
}
