#include "FirmwareVerifyDialog.h"
#include "../logger.h"
#include "../vemsprotocol/VemsDeviceControllerBase.h"
#include "../vemsprotocol/FirmwareData.h"
#include "../vemsprotocol/FirmwareVerifyController.h"
#include "../events/VemsDeviceControllerEvent.h"
#include "../events/CommunicationEvent.h"
#include "../events/FirmwareVerifyEvent.h"
#include "../configmanager/ConfigHelper2.h"
#include "../configmanager/ConfigMgrCommunication.h"
#include "../configmanager/CommunicationConfig.h"


IMPLEMENT_CLASS( FirmwareVerifyDialog, wxProgressDialog )


FirmwareVerifyDialog::FirmwareVerifyDialog( wxWindow* parent,
                                            VemsDeviceControllerBase* deviceController,
                                            FirmwareData* firmwareData )
    : wxProgressDialog( _T(""),
                        _T("Page index: "),
                        firmwareData->getPages(),
                        parent,
                        //wxPD_CAN_ABORT |
                        wxPD_APP_MODAL |
                        wxPD_AUTO_HIDE |
                        wxPD_ELAPSED_TIME |
                        wxPD_SMOOTH |
                        wxPD_ESTIMATED_TIME |
                        wxPD_REMAINING_TIME )
{
    m_deviceController = deviceController;
    m_deviceController->stop();

    m_firmwareVerifyController = new FirmwareVerifyController();

    wxString deviceStr = m_deviceController->getDeviceAsString();
    wxString title = _T("ECU ( ") + deviceStr + _T(" ) flash verify progress!");

    SetTitle( title );

    /// FirmwareVerifyEvent
    m_firmwareVerifyController->Connect( FirmwareVerifyEvent::PAGE_VERIFIED, wxEVT_FIRMWARE_VERIFY, FirmwareVerifyEventHandler( FirmwareVerifyDialog::onFirmwarePageVerified ), NULL, this );
    m_firmwareVerifyController->Connect( FirmwareVerifyEvent::SUCCEEDED, wxEVT_FIRMWARE_VERIFY, FirmwareVerifyEventHandler( FirmwareVerifyDialog::onFirmwareVerifingSucceeded ), NULL, this );
    m_firmwareVerifyController->Connect( FirmwareVerifyEvent::FAILED, wxEVT_FIRMWARE_VERIFY, FirmwareVerifyEventHandler( FirmwareVerifyDialog::onFirmwareVerifingFailed ), NULL, this );

    /// VemsDeviceControllerEvent handlers
    Connect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( FirmwareVerifyDialog::onDeviceDisconnected ), NULL, this );

    Connect( CommunicationEvent::COMMUNICATION_ERROR, wxEVT_COMMUNICATION, CommunicationEventHandler( FirmwareVerifyDialog::onCommunicationEvent ), NULL, this );

    m_pagesCount    = firmwareData->getPages();
    m_message       = wxString::Format( _T("Page index: %d / "), m_pagesCount );

    CommunicationConfig commConfig;
    ConfigBase* cfg = ConfigHelper2::getConfig(ConfigHelper2::CONFIG_MAIN);
    ConfigMgrCommunication cfgMgr(cfg);
    cfgMgr.load(&commConfig);

    SetSize( 300, 200 );
    SetFocus();

//    m_deviceController->verifyFirmware( firmwareData );
    DeviceInfo deviceInfo = m_deviceController->getDeviceInfo();
    m_firmwareVerifyController->start( deviceInfo, firmwareData, commConfig.fwUploadBaudrate );
}

FirmwareVerifyDialog::~FirmwareVerifyDialog()
{
    cleanup();
}

void FirmwareVerifyDialog::cleanup()
{
    LOG
    if ( m_firmwareVerifyController ) {
        LOG
        m_firmwareVerifyController->stop();
LOG
        /// FirmwareVerifyEvent
        m_firmwareVerifyController->Disconnect( FirmwareVerifyEvent::PAGE_VERIFIED, wxEVT_FIRMWARE_VERIFY, FirmwareVerifyEventHandler( FirmwareVerifyDialog::onFirmwarePageVerified ), NULL, this );
        m_firmwareVerifyController->Disconnect( FirmwareVerifyEvent::SUCCEEDED, wxEVT_FIRMWARE_VERIFY, FirmwareVerifyEventHandler( FirmwareVerifyDialog::onFirmwareVerifingSucceeded ), NULL, this );
        m_firmwareVerifyController->Disconnect( FirmwareVerifyEvent::FAILED, wxEVT_FIRMWARE_VERIFY, FirmwareVerifyEventHandler( FirmwareVerifyDialog::onFirmwareVerifingFailed ), NULL, this );

        /// VemsDeviceControllerEvent handlers
        Disconnect( VDC_EVENT_DEVICE_DISCONNECTED, wxEVT_VEMS_DEVICE_CONTROLLER, VemsDeviceControllerEventHandler( FirmwareVerifyDialog::onDeviceDisconnected ), NULL, this );
        Disconnect( CommunicationEvent::COMMUNICATION_ERROR, wxEVT_COMMUNICATION, CommunicationEventHandler( FirmwareVerifyDialog::onCommunicationEvent ), NULL, this );

        delete m_firmwareVerifyController;
        m_firmwareVerifyController = NULL;

        LOG
    }
}

void FirmwareVerifyDialog::onFirmwarePageVerified( FirmwareVerifyEvent& event )
{
    size_t page = event.getFirmwarePageIdx();

    wxString message = m_message + wxString::Format( _T("%d"), page );
    Update( page, message );
}

void FirmwareVerifyDialog::onFirmwareVerifingSucceeded( FirmwareVerifyEvent& event )
{
    LOG
    Update( m_pagesCount );

    wxMessageDialog dialog( GetParent(), _T("Firmware verify was successful!"),
                            _T("Firmware verify"), wxOK | wxICON_INFORMATION, wxDefaultPosition );
    if ( dialog.ShowModal() == wxID_OK ) {
        cleanup();
//        m_deviceController->resume();
        Destroy();
    }
}

void FirmwareVerifyDialog::onFirmwareVerifingFailed( FirmwareVerifyEvent& event )
{
    Update( m_pagesCount );

    wxMessageDialog dialog( GetParent(), _T("Firmware verify was failed!"),
                            _T("Firmware verify"), wxOK | wxICON_ERROR, wxDefaultPosition );
    if ( dialog.ShowModal() == wxID_OK ) {
        cleanup();
//        m_deviceController->resume();
        Destroy();
    }
}

void FirmwareVerifyDialog::onDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    Update( m_pagesCount );

    wxMessageDialog dialog( GetParent(), _T("Device disconnected!"),
                            _T("Firmware verify"), wxOK | wxICON_ERROR, wxDefaultPosition );
    if ( dialog.ShowModal() == wxID_OK ) {
        cleanup();
        Destroy();
    }
}

void FirmwareVerifyDialog::onCommunicationEvent( CommunicationEvent& event )
{
    Update( m_pagesCount );

    wxMessageDialog dialog( GetParent(), _T("Communication error!"),
                            _T("Communication error"), wxOK | wxICON_ERROR, wxDefaultPosition );
    if ( dialog.ShowModal() == wxID_OK ) {
        cleanup();
        Destroy();
    }
}
