/**
    \brief :
                path    : config file path ( /path/ )
                entry   : entry name
*/
#ifndef GENERALCONFIGFRAME_H
#define GENERALCONFIGFRAME_H
#include <wx/wx.h>
#include "../common/typedef.h"
#include "../common/types/BoolHash.h"

class IConfigCtrl
{
    public:
        IConfigCtrl() {}
        virtual ~IConfigCtrl() {}

        virtual void setEntryValue( const wxString& value ) = 0;
        virtual wxString getEntryValue() = 0;

        void setEntryName( const wxString& name ) { m_entryName = name; }
        void setPath( const wxString& path ) { m_path = path; }
        void setSubPath( const wxString& subPath ) { m_subPath = subPath; }
        void setDefaultPath( const wxString& defaultPath ) { m_defaultPath = defaultPath; }
        wxString getEntryName() { return m_entryName; }
        wxString getPath() { return m_path; }
        wxString getSubPath() { return m_subPath; }
        wxString getDefaultPath() { return m_defaultPath; }
    private:
        wxString m_entryName;   //full entry path
        wxString m_path;
        wxString m_subPath;
        wxString m_defaultPath;
};

WX_DEFINE_ARRAY( IConfigCtrl*, IConfigCtrlA );

class wxConfigTextCtrl : public wxTextCtrl, public IConfigCtrl
{
    public:
        wxConfigTextCtrl( wxWindow* parent,
                          wxWindowID id,
                          const wxString& value = _T(""),
                          const wxPoint& pos = wxDefaultPosition,
                          const wxSize& size = wxDefaultSize,
                          long style = 0,
                          const wxValidator& validator = wxDefaultValidator,
                          const wxString& name = wxTextCtrlNameStr )
                           : wxTextCtrl( parent,id, value, pos, size, style, validator, name ) {}
        virtual ~wxConfigTextCtrl() {}

        void setEntryValue( const wxString& value ) { this->SetValue( value ); }
        wxString getEntryValue() { return this->GetValue(); }
    protected:
    private:
};

class wxConfigComboBox : public wxComboBox, public IConfigCtrl
{
    public:
        wxConfigComboBox( wxWindow* parent,
                          wxWindowID id,
                          const wxString& value,
                          const wxPoint& pos,
                          const wxSize& size,
                          const wxArrayString& choices,
                          long style = 0,
                          const wxValidator& validator = wxDefaultValidator,
                          const wxString& name = wxComboBoxNameStr )
                           : wxComboBox( parent,id, value, pos, size, choices, style, validator, name ) {}
        virtual ~wxConfigComboBox() {}

        void setEntryValue( const wxString& value ) { this->SetValue( value ); }
        wxString getEntryValue() { return this->GetValue(); }
    protected:
    private:
};

class ConfigBase;

const wxString gcfDEFAULT_WARNING_TEXT = _T("Not applied until VemsTune restart!");

WX_DEFINE_ARRAY( wxConfigTextCtrl*, wxConfigTextCtrlA );

class GeneralConfigFrame : public wxFrame
{
    public:
        GeneralConfigFrame( wxWindow* parent, wxWindowID id, const wxString& title, ConfigBase* config );
        virtual ~GeneralConfigFrame();

        void addGroup( const wxString& label = wxEmptyString );
        void selectMainGroup();
        void addComboBox( const wxString& path,
                          const wxString& label = wxEmptyString,
                          const wxString& defaultValue = wxEmptyString );

        void addTextCtrl( const wxString& entry,
                          const wxString& defaultPath = wxEmptyString,
                          const wxString& comboPath = wxEmptyString,
                          const wxString& comboSubPath = wxEmptyString );

        void addTextCtrlGroup( const wxString& path,
                               const wxString& defaultPath = wxEmptyString,
                               const wxString& comboPath = wxEmptyString,
                               const wxString& comboSubPath = wxEmptyString );
        void addTextCtrlGroup( const BoolHash& boolHash);
        void addEntrySelectComboBox( const wxString& entry,
                                     const wxArrayString choices,
                                     const wxString& defaultPath = wxEmptyString,
                                     const wxString& comboPath = wxEmptyString,
                                     const wxString& comboSubPath = wxEmptyString );

        void addWarningText( const wxString& warningText = gcfDEFAULT_WARNING_TEXT );
        void addSaveButton( wxAlignment align = wxALIGN_RIGHT );
    protected:
    private:
        IConfigCtrlA        m_viewers;      //combo box viewer array
        IConfigCtrlA        m_textCtrlA;    //added textctrl's array for saving
        wxComboBox*         m_combo;        //group selection combo box
        ConfigBase*         m_configBase;
        wxPanel*            m_mainPanel;
        wxSizer*            m_currentSizer;

        void initGUI();
        void setOptimalSize();
        void OnComboBox( wxCommandEvent& event );
        void OnSaveButton( wxCommandEvent& event );
};

#endif // GENERALCONFIGFRAME_H
