#ifndef OUTPUTCHANNELS_H
#define OUTPUTCHANNELS_H

#include <wx/wx.h>
#include <wx/tglbtn.h>
#include <wx/wxsqlite3.h>
#include "../vemsdialog/EconosealPanel.h"
#include "../gaugedesc/OutputChannelSQL.h"
#include "../stdatamanager/STDataViewerBase.h"
#include "../vemsdialog/StaticViewersController.h"
#include "../vemsdialog/StaticControlPanel.h"
#include "../Symbol.h"
#include "../common/types/String2IntHash.h"
#include "../common/types/Int2StringHash.h"

class STDataMgrBase;
class SymbolTable;
class CommandRequestBase;

WX_DECLARE_STRING_HASH_MAP ( int, OTHash );

class OutputChannelsCfg : public wxPanel, public STDataViewerBase
{
    public:
        OutputChannelsCfg(wxWindow * parent, STDataMgrBase * stdata_mgr, CommandRequestBase* requestMgr, SymbolTable * symbols, wxArrayString& symbolsToMark );
        virtual ~OutputChannelsCfg();
        StaticViewersController*  getController() { return m_statvcontroller; }

    protected:
    private:
        wxArrayString       m_symbolsToMark;

        // Symbol              m_symbol;
        SymbolTable *       m_symbols;
        CommandRequestBase* m_requestMgr;
        wxFrame *           m_frame;

        wxObjectArray       m_dbsymbols;

        String2IntHash      m_out2channel;
        Int2StringHash      m_channel2str;

        // small trick: do not erase the value of Idle solenoid channel in case of dual solenoid
        int                 m_idleSolenoidChannelNo;

        uint8_t             m_injgroupused;

        wxArrayString       m_injgrouppins;

        wxComboBox*         m_testingMode;
        wxToggleButton**    m_testButtons;
        IntArray            m_testBIds;
        wxCheckBox**        m_testCheckBoxes;
        IntArray            m_cboxIds;
        IntArray            m_oldHexvalues;
        IntArray            m_oldHexIdsForTesting;

        wxStaticText**      m_headerText;

        wxStaticText**      m_labels;
        wxRadioButton**     m_showButtons;
        wxTextCtrl**        m_HexIds;
        wxCheckBox**        m_inverted;
        wxArrayString       m_oldselections;
        IntArray            m_channels;
        wxComboBox**        m_outputtypes;
        bool*               m_isconflicted;

        wxStaticBitmap**    m_conflictBMP;
        wxBitmap*           m_warningbmp;

        EconosealPanel*     m_p36,* m_p18;
        StaticViewersController * m_statvcontroller;
        StaticControlPanel * m_statcpanel;
        wxArrayString       m_outputtypestrings;
        wxArrayString       m_ignusedchannels;
        OTHash              m_hex2indexHash;

        OutputType**        m_outputTypeCache;
        wxString            m_injgrouppinsStr;


        bool                m_invalidvaluesfound;
        bool                m_conflictfound;

        size_t     m_num_combos;

        void    getAllOutputtypes();
        void    buildGUI(wxWindow* parent);
        void    onComboBox(wxCommandEvent& event);
        void    onRadioButton(wxCommandEvent& event);
        void    onCheckbox(wxCommandEvent& event);
        void    onTestingMode( wxCommandEvent& event );
        void    showPins(int ecpanel, const wxString& str);
        void    showPins( int channel );
        void    cleanUp();
        void    getInjGroupPins();
        // void    checkIgnitionOutputPins();

        // void    refreshCombos(unsigned short comboid);
        // bool    checkConflict(unsigned short id1, unsigned short id2);

        // bool    isValidType(int channel);

        void    cacheOT();
        OutputType* getOTFromCache( int val );

        void    update();

        void    requestRead() {}
        void    updateModel();
        void    requestWrite();

        void    onClose(wxCloseEvent& evt);
        void    onTestButton( wxCommandEvent& event );
        void    onTestCheckBox( wxCommandEvent& event );

        void    SwitchOffAllTests();

        DECLARE_EVENT_TABLE()

};

#endif // OUTPUTCHANNELS_H
