#include "PasswordProtectionDialog.h"
#include "../SymbolMgr.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../rtdatamanager/RTDataMgrFacade.h"
#include "../vemsprotocol/PasswordHelper.h"

#include "../logger.h"
#include "../DirHelper.h"

#include <wx/statline.h>

int PPD_PASSWORD_ID = wxNewId();
int PPD_PASSWORD_LABEL_ID = wxNewId();
int PPD_PASSWORD_CHECK_ID = wxNewId();
int PPD_MESSAGE_LABEL_ID = wxNewId();


BEGIN_EVENT_TABLE( PasswordProtectionDialog, wxDialog )
    EVT_BUTTON( wxOK, PasswordProtectionDialog::OnButton )
    EVT_BUTTON( wxCANCEL, PasswordProtectionDialog::OnButton )
    EVT_TEXT_ENTER( PPD_PASSWORD_ID, PasswordProtectionDialog::OnEnter)
END_EVENT_TABLE()

PasswordProtectionDialog::PasswordProtectionDialog( wxWindow* parent, STDataMgrBase* staticController
    , SymbolMgr* symbolManager, const wxString& serialNumber )
    : wxDialog( parent, wxID_ANY, _T("Password Protecion Dialog"), wxDefaultPosition, wxDefaultSize )
{
    m_staticController  = staticController;
    m_symbolManager     = symbolManager;
    m_symbolName        = _T("pw_comm_lock0");
    m_serialNumber      = serialNumber;

    buildGUI();
    refreshGUI();
}

PasswordProtectionDialog::~PasswordProtectionDialog()
{

}

void PasswordProtectionDialog::buildGUI()
{
    wxBoxSizer* mainSizer = new wxBoxSizer( wxVERTICAL );

    wxString serialString = _T("Serial Number: ") + m_serialNumber;
    wxStaticText* serialText = new wxStaticText( this, wxID_ANY, serialString, wxDefaultPosition, wxDefaultSize );
    mainSizer->Add( serialText, 0, wxALL, 4 );

//    wxStaticText* passwordText = new wxStaticText( this, PPD_PASSWORD_LABEL_ID, _T("Password:"), wxDefaultPosition, wxDefaultSize );
//    wxTextCtrl* passwordCtrl = new wxTextCtrl( this, PPD_PASSWORD_ID, _T(""), wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD | wxTE_PROCESS_ENTER );
//
//    wxBoxSizer* passwordSizer = new wxBoxSizer( wxHORIZONTAL );
//    passwordSizer->Add( passwordText, 0, 0, 0 );
//    passwordSizer->Add( passwordCtrl, 0, 0, 0 );
//
//    mainSizer->Add( passwordSizer, 0, wxALL, 8 );

    wxString passwordFilePath = DirHelper::GetPath( dCONFIG ) + _T("password.txt");

    wxStaticText* passwordLocText = new wxStaticText( this, wxID_ANY, _T("Password file location: ") + passwordFilePath,
        wxDefaultPosition, wxDefaultSize );
    mainSizer->Add( passwordLocText, 0, wxALL, 4 );

    wxStaticText* messageText = new wxStaticText( this, PPD_MESSAGE_LABEL_ID, _T(""), wxDefaultPosition, wxDefaultSize );
    mainSizer->Add( messageText, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 4 );

    wxCheckBox* passwordEnabledCheckBox = new wxCheckBox( this, PPD_PASSWORD_CHECK_ID, _T("Enable Password Protection") );
    mainSizer->Add( passwordEnabledCheckBox, 0, wxALL, 8 );

    mainSizer->Add( new wxStaticLine(this), 0, wxEXPAND | wxALL, 5 );

    wxString warningString = _T("Restart ECU after change this setting!");
    wxStaticText* warningText = new wxStaticText( this, wxID_ANY, warningString, wxDefaultPosition, wxDefaultSize );
    warningText->SetForegroundColour( *wxRED );
    mainSizer->Add( warningText, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 8 );

    wxButton* okButton = new wxButton( this, wxOK, _T("Ok") );
    wxButton* cancelButton = new wxButton( this, wxCANCEL, _T("Cancel") );

    wxBoxSizer* buttonSizer = new wxBoxSizer( wxHORIZONTAL );
    buttonSizer->Add( okButton, 0, wxALL , 5 );
    buttonSizer->Add( cancelButton, 0, wxALL , 5 );

    mainSizer->Add( buttonSizer, 0, wxALL | wxEXPAND | wxALIGN_RIGHT , 8 );

    SetSizer( mainSizer );
    mainSizer->Fit( this );
    mainSizer->SetSizeHints( this );
}

void PasswordProtectionDialog::refreshGUI()
{
//    bool isPasswordEnabled = true;
    bool isCheckBoxEnabled = false;
    bool isCheckBoxChecked = false;

//    wxString password = PasswordHelper::getPasswordFromFile( m_serialNumber );
//
//    if ( !password.IsEmpty() ) {
//        isPasswordEnabled = false;
//        isCheckBoxEnabled = true;
//    }

    int checkResult = PasswordHelper::checkPassword( m_serialNumber );

    isCheckBoxEnabled = ( checkResult == PasswordHelper::PASSWORD_ACCEPTED );

    Symbol* protectionSymbol = m_symbolManager->getSymbol( m_symbolName );

    if ( protectionSymbol ) {
        double symbolValue = m_staticController->getDataFromViewerModel( *protectionSymbol );
        isCheckBoxChecked = (int)symbolValue == 222;
        Logger::logf( LOG_MSG, _T("symbolValue: %f"), symbolValue );
    } else {
        isCheckBoxEnabled = false;
    }

    wxCheckBox* passwordEnabledCheckBox = (wxCheckBox*) FindWindow(PPD_PASSWORD_CHECK_ID);
    passwordEnabledCheckBox->Enable( isCheckBoxEnabled );
    passwordEnabledCheckBox->SetValue( isCheckBoxChecked );

    wxStaticText* messageText = (wxStaticText*) FindWindow(PPD_MESSAGE_LABEL_ID);

    switch ( checkResult )
    {
        case PasswordHelper::PASSWORD_ACCEPTED:
            messageText->SetLabel( _T("Password Accepted!") );
            messageText->SetForegroundColour( wxColour( 0, 128, 0) );
            break;

        case PasswordHelper::PASSWORD_FILE_NOT_FOUND:
            messageText->SetLabel( _T("Password File not found!") );
            messageText->SetForegroundColour( *wxRED );
            break;

        case PasswordHelper::PASSWORD_NOT_FOUND:
            messageText->SetLabel( _T("Password not found for this serial: ") + m_serialNumber + _T("!")  );
            messageText->SetForegroundColour( *wxRED );
            break;

        case PasswordHelper::INVALID_PASSWORD:
            messageText->SetLabel( _T("Invalid Password!") );
            messageText->SetForegroundColour( *wxRED );
            break;
    }

    Fit();


//    wxTextCtrl* passwordCtrl = (wxTextCtrl*) FindWindow(PPD_PASSWORD_ID);
//    passwordCtrl->Enable( isPasswordEnabled );
//
//    wxStaticText* passwordLabel = (wxStaticText*) FindWindow(PPD_PASSWORD_LABEL_ID);
//    passwordLabel->Enable( isPasswordEnabled );
}

void PasswordProtectionDialog::OnButton( wxCommandEvent& event )
{
    int eventId = event.GetId();

    if ( eventId == wxOK ) {
        wxCheckBox* passwordEnabledCheckBox = (wxCheckBox*) FindWindow(PPD_PASSWORD_CHECK_ID);
        bool isCheckBoxEnabled = passwordEnabledCheckBox->IsEnabled();

        if ( isCheckBoxEnabled ) {
            Symbol* protectionSymbol = m_symbolManager->getSymbol( m_symbolName );

            if ( protectionSymbol ) {
                bool isCheckBoxChecked = passwordEnabledCheckBox->IsChecked();
                double symbolValue = isCheckBoxChecked ? 222.0 : 0.0;
                m_staticController->setModelData( *protectionSymbol, 0, 0, symbolValue );
                m_staticController->requestWrite( *protectionSymbol, 0, 0 );
                m_staticController->requestBurn();
                m_staticController->updateRegisteredViewers();
            }
        }
    }

    EndModal( eventId );
}

void PasswordProtectionDialog::OnEnter( wxCommandEvent& event )
{
//    wxTextCtrl* passwordCtrl = (wxTextCtrl*) FindWindow(PPD_PASSWORD_ID);
//    wxStaticText* passwordLabel = (wxStaticText*) FindWindow(PPD_PASSWORD_LABEL_ID);
//
//    wxString password = passwordCtrl->GetValue();
//
//    if ( PasswordHelper::isPasswordFormatAccepted( password ) && PasswordHelper::isPasswordAccepted( password ) ) {
//        wxCheckBox* passwordEnabledCheckBox = (wxCheckBox*) FindWindow(PPD_PASSWORD_CHECK_ID);
//        passwordEnabledCheckBox->Enable( true );
//        passwordCtrl->Enable( false );
//        passwordLabel->Enable( false );
//    } else {
//        wxMessageBox( _T("Invalid Password!") );
//    }
}
