#include "hashcompare.h"

#include "logger.h"

HashCompare::HashCompare()
{
    //ctor
}

HashCompare::~HashCompare()
{
    //dtor
}

CompareResult* HashCompare::compare(String2Uint32Hash& h1, String2Uint32Hash& h2, uint32_t mask)
{
//    size_t h1S = h1.size();
    size_t h2S = h2.size();

    CompareResult * cr = new CompareResult();
    cr->size = h2S;

    String2Uint32Hash::iterator it;
    for ( it = h2.begin(); it != h2.end(); ++it )
    {
        wxString key = it->first;
        uint32_t value = it->second;

        if( h1.find( key) == h1.end() )
        {
            // key not found
            Logger::log(LOG_MSG,_T("HashCompare::compare() Key not found: ") + key);
            cr->addMissingKey(key);
            continue;
        }
        uint32_t value1 = h1[key];
        int res = compareValue(key, value1, value);
        if(res == COMPARE_DIFF){
            Logger::log(LOG_MSG,_T("HashCompare::compare() Value is different in key: ") + key);
            cr->addDiffValueKey(key);
        }
    }
    return cr;
}

CompareResult* HashCompare::compare(StringHash& h1, StringHash& h2)
{
//    size_t h1S = h1.size();
    size_t h2S = h2.size();

    CompareResult * cr = new CompareResult();
    cr->size = h2S;

    StringHash::iterator it;
    for ( it = h2.begin(); it != h2.end(); ++it )
    {
        wxString key = it->first, value = it->second;

        if( h1.find( key) == h1.end() )
        {
            // key not found
                Logger::log(LOG_MSG,_T("HashCompare::compare() Key not found: ") + key);
                cr->addMissingKey(key); /// missing from h1
                continue;
        }
        wxString value1 = h1[key];
        int res = compareValue(key, value1, value);
        if(res == COMPARE_DIFF){
            Logger::log(LOG_MSG,_T("HashCompare::compare() Value is different in key: ") + key );
            cr->addDiffValueKey(key);
        }


    }
    return cr;
}

int HashCompare::compareValue(const wxString& key, const wxString& v1, const wxString& v2)
{
    if( v1.CmpNoCase(v2) == 0){
           return COMPARE_EQUAL;
    }
    if(key.StartsWith(_T("j")) ){

        if(v1.Mid(0,2) == v2.Mid(0,2) ){ return COMPARE_NEAR;} //--> 0x4280 0x427F
    }
    return COMPARE_DIFF;

}

int HashCompare::compareValue(const wxString& key, uint32_t v1, uint32_t v2)
{
         if( (v1) == v2 ){
            return COMPARE_EQUAL;
         }
        if(key.StartsWith(_T("j")) ){

            uint8_t v1Norm = norm(v1);
            uint8_t v2Norm = norm(v2);
            //return COMAPRE_DIFF;
            if( v1Norm == v2Norm){
                return COMPARE_EQUAL;
            }
            if( abs(v1Norm-v2Norm) <3){ return COMPARE_NEAR;}

        }

        if( (v1) != v2 ){
            return COMPARE_DIFF;
        }

    return 0;
}
uint8_t HashCompare::norm(uint32_t v1)
{
    if(v1 > 255) return (uint8_t)(v1>>8);
    return (uint8_t)v1;
}
