#include "wxHotKeyManager.h"

#include <wx/keybinder.h>
#include <wx/wxsqlite3.h>
#include "../common/VemsMessageHashHelper.h"
#include "../sqlite/wxSQLite3DbManager.h"
#include "../logger.h"
#include "../UserLog.h"
#include "../DirHelper.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(HotKeyDescArray);

Int2StringHash wxHotKeyManager::m_numpadKeycodes = wxHotKeyManager::InitNumpadKeyCodes();

wxHotKeyManager::wxHotKeyManager(const wxString &dbfile)
{
    //ctor
    m_dbFile = dbfile;
    ParseConfigFile();

    InitNumpadKeyCodes();
}

wxHotKeyManager::wxHotKeyManager(wxObject *baseClassWithRTTI)
{
    wxString classname = baseClassWithRTTI->GetClassInfo()->GetClassName();

    wxString fn = DirHelper::GetPath(dHOTKEYS) + _T("profiles.db");
    wxString fnt = DirHelper::GetPath(dHOTKEYS) + _T("targets.db");

    wxSQLite3Database* db = NULL;

    if( !wxFileExists( fn ) ){
        StringHash dbe_hash;
        dbe_hash[_T("filename")] = fn;
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("db_file_not_exists"),&dbe_hash) );
        return;
    }

    if( !wxFileExists( fnt ) ){
        StringHash dbe_hash;
        dbe_hash[_T("filename")] = fnt;
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("db_file_not_exists"),&dbe_hash) );
        return;
    }

    try {
        db = wxSQLite3DbManager::Open( fn );

        if ( !db )
            return;

        wxString qs;

        try {
                qs = _T("attach database '") + fnt + _T("' as targetdb");
                db->ExecuteQuery( qs );
        }
        catch ( wxSQLite3Exception& e )
        {
            //
        }
        catch(...)
        {
            LOGP( _T("unknown exception caught"))
        }

        qs = _T("select filename,targets.symboltablefile from profiles inner join targets on \
                        profiles.target_id = targets.id and targets.class_symbol='") + classname + _T("' and profiles.isuse = 1;");

            wxSQLite3ResultSet res = db->ExecuteQuery(qs);

        if(res.IsNull(0)) {
            UserLog::uLog(ULOG_ERROR, _T("Cant find any hotkey profile this class."));
            return;
        }

        m_eventsFile = DirHelper::GetPath(dHOTKEYS) + res.GetString(1);

        m_dbFile = DirHelper::GetPath(dHOTKEYS) + res.GetString(0);

        ParseConfigFile();
        InitNumpadKeyCodes();
    }
    catch(wxSQLite3Exception &ex) {
        UserLog::uLog(ULOG_ERROR, ex.GetMessage());
        return;
    }
    catch(...)
    {
        LOGP(_T("something weird"))
    }
}

wxHotKeyManager::~wxHotKeyManager()
{
    //dtor
}

void wxHotKeyManager::GetHotKeyList(HotKeyDescArray& arr, wxArrayString* excludesymbols)
{
    wxSQLite3Database* db = NULL;

    try {
        db = wxSQLite3DbManager::Open(m_dbFile);

        if ( !db )
            return;

        wxString qs = _T("attach database '") + m_eventsFile + _T("' as eventsdb");
        try {
            db->ExecuteQuery( qs );
        }
        catch ( wxSQLite3Exception& e )
        {
            // nop
        }
        catch (...)
        {
            LOGP(_T("unknow exception caught"))
        }

        wxSQLite3ResultSet res = db->ExecuteQuery(_T("select symbol,title,keystring from hotkeys inner join hotkeyevent on hotkeys.event_id=hotkeyevent.event_id order by title asc;"));

    while(res.NextRow()){

        if( excludesymbols && excludesymbols->Index(res.GetString(0)) != wxNOT_FOUND ) continue;

        HotKeyDesc hk;

        hk.HotKey = res.GetString(2);
        hk.Title = res.GetString(1);

        arr.Add( hk );

    }
}
    catch (wxSQLite3Exception& e)
    {
        UserLog::uLog(ULOG_ERROR, e.GetMessage());
    }
}

wxString wxHotKeyManager::GetHotKeyStringBySymbol(const wxString &eventsymbol)
{
    wxString ks;

    for( StringHash::iterator it = m_hotKeyToEventStrings.begin(); it != m_hotKeyToEventStrings.end(); it++ ){
        if( it->second == eventsymbol )
            ks += it->first + _T(", ");
    }

    ks.RemoveLast(2);

    return ks;
}

wxString wxHotKeyManager::GetHotKeyTitleBySymbol(const wxString &eventsymbol)
{
    wxSQLite3Database* db;

    try {
        db = wxSQLite3DbManager::Open(m_eventsFile);

        if ( !db )
            return wxEmptyString;

        wxString qs = _T("select title from hotkeyevent where symbol='") + eventsymbol + _T("'");

        wxSQLite3ResultSet res = db->ExecuteQuery( qs );

    return res.GetString(0);
}
    catch ( wxSQLite3Exception& e )
    {
        UserLog::uLog(ULOG_ERROR, e.GetMessage());
        return wxEmptyString;
    }
    catch(...)
    {
        LOGP( _T("unknown exception caught"))
        return wxEmptyString;
    }
}


void wxHotKeyManager::ParseConfigFile(const wxString &dbfile)
{
    if(dbfile != wxEmptyString)
        m_dbFile = dbfile;

    if( !wxFileExists( m_dbFile ) ){
        StringHash dbe_hash;
        dbe_hash[_T("filename")] = m_dbFile;
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("db_file_not_exists"),&dbe_hash) );
        return;
    }

    if( !wxFileExists( m_eventsFile ) ){
        StringHash dbe_hash;
        dbe_hash[_T("filename")] = m_eventsFile;
        UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("db_file_not_exists"),&dbe_hash) );
        return;
    }


    wxSQLite3Database* db = NULL ;

    try {
        db = wxSQLite3DbManager::Open(m_dbFile);

        if ( !db )
        return;

        if( !db->TableExists(_T("hotkeys")) )
        {
            db->ExecuteUpdate(_T("CREATE TABLE IF NOT EXISTS [hotkeys] ([id] INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT,[event_id] INTEGER  NULL,[keystring] VARCHAR(64)  NULL);"));
    }

        try {
            wxString qs = _T("attach database '") + m_eventsFile + _T("' as eventsdb");
            db->ExecuteQuery(qs);
    }
        catch ( wxSQLite3Exception& e )
        {
            LOGP( e.GetMessage() )
        }
        catch(...)
        {
            LOGP( _T("Something weird") )
        }

        wxSQLite3ResultSet res = db->ExecuteQuery(_T("select symbol,keystring from hotkeys inner join hotkeyevent on hotkeys.event_id=hotkeyevent.event_id;"));

    while(res.NextRow()){
        m_hotKeyToEventStrings[res.GetString(1)] = res.GetString(0);
    }

    } catch(wxSQLite3Exception &ex) {
        UserLog::uLog(ULOG_ERROR, ex.GetMessage());
        return;
}
    catch (...)
    {
        LOGP( _T("unknown exception caught"))
    }
}

void wxHotKeyManager::RefreshHotKeys()
{
    m_hotKeyToEventStrings.clear();

    ParseConfigFile();
}

wxString wxHotKeyManager::GetEvent(wxKeyEvent &evt, const wxString& removeFromKeyStroke)
{
    wxString key = wxHotKeyManager::NumPadKeyStrokeToString(evt.GetKeyCode());

    if(key == wxEmptyString) {
        key = wxKeyBind::KeyCodeToString(evt.GetKeyCode());
    }

    if(key == wxEmptyString)
        return wxEmptyString;

    wxString keystroke = wxKeyBind::KeyModifierToString(wxKeyBind::GetKeyModifier(evt)) + key;

    if( removeFromKeyStroke != wxEmptyString )
        keystroke.Replace( removeFromKeyStroke, _T(""), true );

    //Logger::logf(LOG_MSG,_T("keystroke=")+keystroke);

//    wxString key = wxKeyBind::GetKeyStrokeString(evt);

    if(m_hotKeyToEventStrings.find(keystroke) == m_hotKeyToEventStrings.end())
        return wxEmptyString;

    return m_hotKeyToEventStrings[keystroke];
}

wxString wxHotKeyManager::NumPadKeyStrokeToString(int keycode)
{
    if( m_numpadKeycodes.find(keycode) == m_numpadKeycodes.end() ) return wxEmptyString;

    return m_numpadKeycodes[keycode];
}

Int2StringHash wxHotKeyManager::InitNumpadKeyCodes()
{
    Int2StringHash hash;

    hash[WXK_NUMPAD_SPACE] = _T("NumPad Space");
    hash[WXK_NUMPAD_TAB] = _T("NumPad Tab");
    hash[WXK_NUMPAD_ENTER] = _T("NumPad Enter");
    hash[WXK_NUMPAD_F1] = _T("NumPad F1");
    hash[WXK_NUMPAD_F2] = _T("NumPad F2");
    hash[WXK_NUMPAD_F3] = _T("NumPad F3");
    hash[WXK_NUMPAD_F4] = _T("NumPad F4");
    hash[WXK_NUMPAD_HOME] = _T("NumPad Home");
    hash[WXK_NUMPAD_LEFT] = _T("NumPad Left");
    hash[WXK_NUMPAD_UP] = _T("NumPad Up");
    hash[WXK_NUMPAD_RIGHT] = _T("NumPad Right");
    hash[WXK_NUMPAD_DOWN] = _T("NumPad Down");
    hash[WXK_NUMPAD_PAGEUP] = _T("NumPad PageUp");
    hash[WXK_NUMPAD_PAGEDOWN] = _T("NumPad PageDown");
    hash[WXK_NUMPAD_END] = _T("NumPad End");
    hash[WXK_NUMPAD_BEGIN] = _T("NumPad Begin");
    hash[WXK_NUMPAD_INSERT] = _T("NumPad Insert");
    hash[WXK_NUMPAD_DELETE] = _T("NumPad Delete");
    hash[WXK_NUMPAD_EQUAL] = _T("NumPad Equal");
    hash[WXK_NUMPAD_MULTIPLY] = _T("NumPad Multiply");
    hash[WXK_NUMPAD_ADD] = _T("NumPad Add");
    hash[WXK_NUMPAD_SEPARATOR] = _T("NumPad Separator");
    hash[WXK_NUMPAD_SUBTRACT] = _T("NumPad Subtract");
    hash[WXK_NUMPAD_DECIMAL] = _T("NumPad Decimal");
    hash[WXK_NUMPAD_DIVIDE] = _T("NumPad Divide");

    for(int a=WXK_NUMPAD0;a<=WXK_NUMPAD9;a++){
        hash[a] = wxString::Format(_T("NumPad %i"),a-WXK_NUMPAD0);
    }

    return hash;
}
