#ifndef LOGHISTORYBASE_H
#define LOGHISTORYBASE_H

#include "../common/types/DoubleHashArray.h"
#include "../common/types/IntArray.h"
#include "../common/typedef.h"
#include "../Observable.h"
#include "../ObserverBase.h"

#include "../rtdatamanager/RTDataMgrBase.h"
#include "../stdatamanager/STDataMgrBase.h"

#include "../vemsprotocol/VEMSLogBuffer.h"

//class RTDataMgrBase;
//class STDataMgrBase;
class SymbolMgr;
class SIPRPacket;
class LogFilter;

const wxString strLogIdxPos = _T("rtidx");
/*
enum{
    MODE_OFFLIE,
    MODE_ONLINE,
    MODE_LOCKED
};
*/

WX_DEFINE_ARRAY(LogFilter*, LogFilterA);

enum { LH_MARKER_USER = 1, LH_MARKER_STATIC = 2 };

class LogHistoryBase: public Observable, public ObserverBase
{
    public:
        LogHistoryBase( RTDataMgrBase* rtMgr, STDataMgrBase* stMgr, SymbolMgr* symbolMgr );
        virtual ~LogHistoryBase();

        virtual void getHistory( const IntArray& idxA, DoubleHashArray& viewerData ) = 0;
        virtual void getHistory( size_t startIdx, size_t endIdx, DoubleHashArray& viewerData ) = 0;
        virtual void emptyHistory() = 0;
        virtual void analyzeHistory() = 0;

        virtual bool hasGPSdata() = 0;
        virtual void getGPSData( DoubleHashArray& gpsdata ) = 0;

        virtual void getMarkers( IntArray& idxA, uint8_t markerType ) = 0;
        virtual size_t getMdxFrameMarkersCount() = 0;
        virtual void getMdxSiprPackets( size_t idx, SIPRPacket*** packets, unsigned int& numberOfPacketsInFrame ) = 0;
        virtual void getMdxSiprIndices( size_t idx, IntArray& indices ) = 0;

        DoubleHash* getData(size_t idx);

        virtual size_t getPacket( size_t idx, SIPRPacket** packet ) = 0;

        virtual size_t getHistoryCount( size_t pType = IDX_RTDATA_NORMAL ) = 0;

        void registerLogFilter( LogFilter* logfilter );
        void unregisterLogFilter( LogFilter* logfilter );
        virtual void processFilter( size_t startIdx, size_t endIdx ) = 0;
        virtual void setPacketDelay( long delay ) = 0;

        virtual void refresh( size_t idx, bool notifySlider = true ) = 0;
        virtual void process( const SIPRPacket& p) = 0;

        virtual void clear() = 0;
        virtual wxString getLogFileName() = 0;

        virtual bool loadHistoryFromLog(const wxString& filename) = 0;
        virtual StringHash  getHeaderHash() = 0;
        virtual void refreshLastPosition() = 0;

        RTDataMgrBase* getRTDataMgr() { return m_rtMgr; }
        STDataMgrBase* getSTDataMgr() { return m_stMgr; }
        SymbolMgr*     getSymbolMgr() const { return m_symbolMgr; };

        void StopProcessing() { m_isStopProcessFlag = true; }

    protected:
        RTDataMgrBase* m_rtMgr;
        STDataMgrBase* m_stMgr;
        SymbolMgr*     m_symbolMgr;
        LogFilterA     m_logfilterA;

        double         m_rt_time;
        int            m_rt_idx;
        bool           m_isStopProcessFlag;

    private:
};

#endif // LOGHISTORYBASE_H
