#include "QuickAnalyze.h"
#include "LogHistoryBase.h"
#include "../rtdatamanager/RTDataMgrBase.h"
#include "../rtdatamanager/RTDataMgrFacade.h"
#include "../logger.h"
#include "../gui/FileOpenSQL.h"


QuickAnalyze::QuickAnalyze( LogHistoryMgr* loghistory, wxString filename )
{
    m_filename = filename;
    m_loghistory = new LogHistoryMgr( *loghistory );

    //m_rtDataMgr = loghistory->getRTDataMgr();

    m_rpmMax = 0;
    m_mapMax = 0;
    m_triggerError = 0;
    m_isTriggerError = false;

    m_isCancelled = false;
    m_isRunning = false;

    m_datas.Clear();

    m_analyzer = new RTDataAnalyzer();
    m_loghistory->getRTDataMgr()->RegisterViewer( m_analyzer );
    m_analyzer->SetDataMgr(m_loghistory->getRTDataMgr());
    //require();
}

void QuickAnalyze::set(LogHistoryMgr* loghistory, wxString filename)
{
    m_loghistory->getRTDataMgr()->UnRegisterViewer( m_analyzer );
    delete m_analyzer;
    delete m_loghistory;

    m_filename = filename;
    m_loghistory = new LogHistoryMgr( *loghistory );

    //m_rtDataMgr = loghistory->getRTDataMgr();

    m_rpmMax = 0;
    m_mapMax = 0;
    m_triggerError = 0;
    m_isTriggerError = false;

    m_isCancelled = false;
    m_isRunning = false;

    m_datas.Clear();

    m_analyzer = new RTDataAnalyzer();
    m_loghistory->getRTDataMgr()->RegisterViewer( m_analyzer );
    m_analyzer->SetDataMgr(m_loghistory->getRTDataMgr());
}

QuickAnalyze::~QuickAnalyze()
{
    m_loghistory->getRTDataMgr()->UnRegisterViewer( m_analyzer );
    delete m_analyzer;
    delete m_loghistory;
}

void QuickAnalyze::start()
{
    setDisabled( false );

    size_t hist_size = m_loghistory->getHistoryCount();
    if ( hist_size < 1 || hist_size == (size_t)-1 )
    {
        setDisabled( true );
        m_isRunning = false;

        FileInfo info;
        info.rpmMax = 0;
        info.mapMax = 0;
        info.triggerErr = 0;
        info.hasGPSData = 0;

        FileAnalyzeSQL::addData( m_filename, info );

        observerNotify( _T("qaFinished") );
        return;
    }

    ProcessBase::start( (WXTHREAD_MIN_PRIORITY + WXTHREAD_DEFAULT_PRIORITY) / 2 );
}

bool QuickAnalyze::process()
{
    if ( m_isDisabled )
        return false;

    if ( m_isCancelled )
        return false;

    if ( m_isRunning )
        return false;

    m_isRunning = true;

    m_loghistory->analyzeHistory();

    DoubleHash* a_values = m_analyzer->getValues();

    m_rpmMax = (size_t)((*a_values)[_T("rpmMax")]);
    m_mapMax = (size_t)((*a_values)[_T("mapMax")]);
    m_triggerError = (size_t)((*a_values)[_T("triggerError")]);
    m_hasGPSdata = m_loghistory->hasGPSdata();

    // Logger::logf(LOG_MSG, _T("QuickAnalyze::process() ended"));
    return true;
}

void QuickAnalyze::stop()
{
    m_isCancelled = true;
    m_isRunning = false;

    setDisabled( true );

    ProcessBase::stop();

    observerNotify( QA_CANCELLED );
}

void QuickAnalyze::reset()
{
}

void QuickAnalyze::finished()
{
    m_isRunning = false;
    setDisabled( true );

    if ( m_isCancelled ) ;
    // update db
    else
    {
        /// TODO
        // observerNotify( QA_FINISHED );

        FileInfo info;
        info.fw_major = info.fw_minor = info.fw_build = 0;
        info.rpmMax = m_rpmMax;
        info.mapMax = m_mapMax;
        info.triggerErr = m_triggerError;
        info.hasGPSData = m_hasGPSdata;

        FileAnalyzeSQL::addData( m_filename, info );

        observerNotify( _T("qaFinished") );
    }
}

bool QuickAnalyze::isAnalyzed( const wxString& filename )
{
    return FileAnalyzeSQL::isAnalyzed( filename );
}
