#ifndef SEARCHDIALOG_H
#define SEARCHDIALOG_H

#include "LogHistorySearch.h"
#include "../ObserverBase.h"

class LogHistoryBase;
class SearchDialogHelp;
class LogHistoryProcessFilterThread;

/// search in the loghistory with any expression string
class SearchDialog: public wxDialog, public ObserverBase
{
    public:
        SearchDialog(wxWindow* parent, LogHistoryBase* loghistory, const wxString& title = _T("Search"), wxArrayString searchHistory = wxArrayString(), wxArrayString symbolList = wxArrayString() );
        virtual ~SearchDialog();

        void SetSearchSection(const Section& section);
        void SetSearchString(const wxString& str);

        SectionArray& GetResult() { return m_resultSections; }
        wxArrayString& GetSearchHistory() { return m_searchHistory; }

        void InsertSymbol(const wxString& symbolname);

        virtual void notify( int i );
        virtual void notify(const wxString& str);


    protected:

        void BuildGUI();

        void OnComboEnter(wxCommandEvent& evt);
        void OnComboText(wxCommandEvent& evt);
        void OnCheck(wxCommandEvent& evt);
        void OnButton(wxCommandEvent& evt);
        void OnMenu(wxCommandEvent& evt);
        void OnClose(wxCloseEvent& evt);
        void OnKeyDown(wxKeyEvent& evt);

        void OnSearchBoxKeyDown(wxKeyEvent& evt);
        void OnSearchBoxKeyUp(wxKeyEvent& evt);
        void OnMove(wxMoveEvent& evt);

        void OnSymbolCompDClick(wxCommandEvent& evt);
        void OnSymbolCompKeyDown(wxKeyEvent& evt);


        void Search();

        void EnablePanel(bool isEnable);

        void StopSearch(bool isCancel = true, int retcode = wxID_CANCEL);
        void EndSearch();

        void EnableSectionCtrls(bool isEnable);

    private:

        wxString SearchPartialSymbolName();
        void SymbolCompletion(const wxString& symbol);

        void SetSymbolCompletionFramePos();

        void OnInternalCommand(wxCommandEvent& evt);
        void OnHelpClosed(wxCloseEvent& evt);

        enum {
            idComboSearch,
            idisSearchBetweenSectionCB,
            idButtonSearch,
            idButtonCancel,
            idButtonClose,
            idButtonHelp,
            idInternalProcess,
            idInternalEndProcess,
            idSymbolCompList,
            idMenuClose
        };

        LogHistoryBase* m_logMgr;

        wxArrayString   m_searchHistory;
        wxArrayString   m_symbolList;

        wxComboBox*     m_searchCB;

        wxTextCtrl*     m_beginIdxCtrl;
        wxTextCtrl*     m_endIdxCtrl;
        wxCheckBox*     m_isSection;

        Section         m_limitSection;

        SectionArray    m_resultSections;

        wxGauge*        m_progressBar;

        LogHistoryProcessFilterThread*  m_searchProcesser;
        LogHistorySearch*       m_filter;

        wxWindow*       m_progressStaticBox;

        bool            m_isCancelledSearch;

        int             m_retCode;
        int             m_lastInstertPoint;

        wxListBox*      m_symbolCompletionList;
        wxFrame*        m_symbolCompletionFrame;

        SearchDialogHelp* m_helpPanel;

        DECLARE_EVENT_TABLE();
};

class SearchDialogHelp: public wxFrame
{
    public:
        SearchDialogHelp(SearchDialog* parent, const wxArrayString& symbolList);
        virtual ~SearchDialogHelp();

    protected:

        void OnEnter(wxCommandEvent& evt);
        void OnListKeyDown(wxKeyEvent& evt);

        void InsertSymbol();

    private:

        SearchDialog*   m_parent;

        wxListBox*  m_symbolList;
        wxComboBox* m_expressionCtrl;

};


#endif // SEARCHDIALOG_H
