#include "SearchResultDialog.h"

#include <wx/imaglist.h>

#include "../logger.h"
#include "../events/GeneralEvent.h"

#include "../wxListCtrlHelper.h"
#include "../DrawHelper.h"

BEGIN_EVENT_TABLE(SearchResultDialog, wxFrame)

    EVT_LIST_ITEM_ACTIVATED(idResultList, SearchResultDialog::OnListActivated)
    EVT_LIST_COL_CLICK(idResultList, SearchResultDialog::OnListHeaderClick)

END_EVENT_TABLE()

SearchResultDialog::SearchResultDialog(wxWindow* parent, int id, SectionArray& result, const wxString& title)
{
    //ctor
    m_handler = NULL;
    Create(parent, id, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    int h = (20+result.GetCount()*15);

    wxSize size(400,h > 400 ? 400 : h );

    m_resultList = new wxListCtrl(this,idResultList,wxDefaultPosition,size, wxLC_REPORT | wxLC_SINGLE_SEL);

    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer* label = new wxStaticBoxSizer(wxVERTICAL,this,_T("Result list"));

    label->Add(m_resultList, 1, wxALL | wxEXPAND, 5);

    mainsizer->Add(label,1,wxALL | wxEXPAND, 5);

    InitList();

    for(size_t a=0; a<result.GetCount();a++){
        AddLine(result[a]);
    }

    m_sectionList = result;

    SetSizer(mainsizer);
    mainsizer->SetSizeHints(this);

    m_lastOrderedCol = 0;

    wxListCtrlHelper::SortColoumData(m_resultList, m_lastOrderedCol, true, true);
}

SearchResultDialog::~SearchResultDialog()
{
    //dtor
}

wxString SearchResultDialog::GetListCtrlCellText(wxListCtrl * lc, long item, int col)
{
    wxListItem info;
    info.SetColumn(col);
    info.SetId(item);
    info.SetMask(wxLIST_MASK_TEXT);

    lc->GetItem(info);

    return info.GetText();
}

void SearchResultDialog::OnListActivated(wxListEvent& evt)
{
    long item = evt.GetIndex();

    long sidx,eidx;

    GetListCtrlCellText(m_resultList,item,0).ToLong(&sidx);
    GetListCtrlCellText(m_resultList,item,1).ToLong(&eidx);

    //Logger::logf(LOG_MSG,_T("s=%i,e=%i"),sidx,eidx);

    if(m_handler) {
        GeneralEvent evt( GetId() );
        wxVariantHash hash;
        hash[_T("start_idx")] = sidx;
        hash[_T("end_idx")] = eidx;

        evt.SetVariantHash(hash);
        m_handler->AddPendingEvent(evt);
    }
}

void SearchResultDialog::InitList()
{
    int w = m_resultList->GetSize().x-20;

    wxListItem info1;
    wxListItem info2;
    wxListItem info3;
    wxListItem info4;

    info1.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT  );
    info2.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT  );
    info3.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT  );
    info4.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT  );

    info1.SetWidth(w/4);
    info1.SetAlign(wxLIST_FORMAT_LEFT);
    info1.SetText(_T("Start"));
    info1.SetData( 1 );
    //info1.SetImage( IMG_UP );

    info2.SetWidth(w/4);
    info2.SetAlign(wxLIST_FORMAT_LEFT);
    info2.SetText(_T("End"));
    info2.SetData( 1 );

    info3.SetWidth(w/4);
    info3.SetAlign(wxLIST_FORMAT_LEFT);
    info3.SetText(_T("Length"));
    info3.SetData( 1 );

    info4.SetWidth(w/4);
    info4.SetAlign(wxLIST_FORMAT_CENTER);
    info4.SetText(_T("Note"));
    info4.SetData( 1 );

    m_resultList->InsertColumn(0,info1);
    m_resultList->InsertColumn(1,info2);
    m_resultList->InsertColumn(2,info3);
    m_resultList->InsertColumn(3,info4);

    wxImageList *imglist = new wxImageList(16,16,true);

    wxBitmap emptyBmp(16,16,4);

    imglist->Add( emptyBmp );
    imglist->Add( DrawHelper::GetBitmap(_T("icon_up.png")) );
    imglist->Add( DrawHelper::GetBitmap(_T("icon_down.png")) );

    m_resultList->AssignImageList( imglist, wxIMAGE_LIST_SMALL );

}

void SearchResultDialog::AddLine(Section& sect)
{
    wxListItem item0, item1, item2, item3;

    int i = m_resultList->GetItemCount();

    item0.SetId( i );
    item1.SetId( i );
    item2.SetId( i );
    item3.SetId( i );

    item0.SetColumn(0);
    item1.SetColumn(1);
    item2.SetColumn(2);
    item3.SetColumn(3);

    item0.SetMask( wxLIST_MASK_TEXT );
    item1.SetMask( wxLIST_MASK_TEXT );
    item2.SetMask( wxLIST_MASK_TEXT );
    item3.SetMask( wxLIST_MASK_TEXT );

    size_t len = sect.endIdx-sect.beginIdx;

    item0.SetText(wxString::Format(_T("%i"),sect.beginIdx));
    item1.SetText(wxString::Format(_T("%i"),sect.endIdx));
    item2.SetText(wxString::Format(_T("%i"),len));
    item3.SetText(len <= 2 ? _T("very short!") : _T(""));


    long tmp = m_resultList->InsertItem(item0);
    m_resultList->SetItemData(tmp, i);
    m_resultList->SetItem(item1);
    m_resultList->SetItem(item2);
    m_resultList->SetItem(item3);


}

void SearchResultDialog::OnListHeaderClick(wxListEvent &evt)
{
    //Logger::logf(LOG_MSG,_T("OnListHeaderClick"));

    int col = evt.GetColumn();

    if(col == 3) return;

    wxListCtrl * lc;

    bool isAsc = true;

    switch(evt.GetId()){

        case idResultList:
            lc = m_resultList;
            isAsc = m_listColOrders[col];
            m_listColOrders[col] = !isAsc;
            break;

        default :
            return;
    }

    wxListCtrlHelper::SortColoumData(lc, col, isAsc, true);

    wxListItem info;
    lc->GetColumn(col,info);

    //Logger::logf(LOG_MSG,_T("m_lastOrderedCol=%i, col=%i"),m_lastOrderedCol,col);

    if(m_lastOrderedCol != col) {
        wxListItem prev_info;
        lc->GetColumn(m_lastOrderedCol,prev_info);
        info.SetImage( IMG_NULL );
        lc->SetColumn(m_lastOrderedCol, info);
    }

    info.SetImage( isAsc ? IMG_UP : IMG_DOWN );

    lc->SetColumn(col, info);

    m_lastOrderedCol = col;
}
