#ifndef MAINFRAME_H
#define MAINFRAME_H



#include <wx/frame.h>
#include <wx/intl.h>
#include <wx/listbox.h>
#include <wx/settings.h>
#include <wx/sizer.h>
#include <wx/xrc/xmlres.h>
#include <wx/taskbar.h>
#include <wx/textfile.h>
#include <wx/wx.h>
#include <wx/numdlg.h>
#include <wx/progdlg.h>
#include <wx/aui/tabmdi.h>

#include "common/types/StringHash.h"
#include "common/typedef.h"
#include "FileDropManagerBase.h"
#include "FileDropTargetBase.h"
#include "VemsTuneInfo.h"

#include "events/VemsGUICommandEvent.h"

#define MDI_FRAME 0
#define AUI_MDI_FRAME 1


#if MDI_FRAME
    #define MAINFRAME_BASECLASS wxMDIParentFrame
#elif AUI_MDI_FRAME
    #define MAINFRAME_BASECLASS wxAuiMDIParentFrame
#else
    #define MAINFRAME_BASECLASS wxFrame
#endif


class WindowReg;
class VemsTaskBarIcon;
class FrameLayoutHandler;
class RTDataMgrBase;
class GaugeGroupManager;
class MainPanel;
class STDataMgrBase;
class RTDataInputBase;
class SFloodProcess;
class FirmwareMgrFrame;
class VemsTuneController;
class UserDialogManager;
class ProjectManager;
class Project;
class InputEvent;
class ProgressBarObserver;
class wxHotKeyProfileManager;
class UserMenuRefresher;
class VTFilesUpdater;
class ECUEvent;
class TriggerLogManager;
class TriggerLogGraph;
class FirmwareUploadedEvent;
class ConfigSwitchState;

/// refactor communication
class VemsDeviceControllerEvent;
class DeviceControllerProgressBarManager;
class ConfigDownloadDirector;
class FirmwareConfigManager;
class FirmwareDirector;
class SDCardDirector;
class CommunicationEvent;
class VemsTuneEventDispatcher;

class SDCardEvent;
/// refactor communication

enum StatusBarFieldId
{
    SB_MENU_HELP,
    SB_FIRMWARE_VERSION,
    SB_DATA_INPUT,
    SB_CONFIG_VERSION,
    SB_UPDATE_FREQUENCY,
};

class MainFrame: public MAINFRAME_BASECLASS, public FileDropManagerBase, public FileDropTargetBase
{
    DECLARE_CLASS(MainFrame)
	public:
        MainFrame(const wxChar *title, int x, int y, int width, int height, const StringHash& params );
		virtual ~MainFrame();

        void TimerMgr();
        void SetRootDir(const wxString &dir);

        GaugeGroupManager* getGaugeGroupManager(){ return m_gaugeGroupManager;}
        void OnOpenVemsCfg( const wxString& filename = _T("showdialog") );
        void OnOpenVemsLog( const wxString& filename = _T("showdialog") );


        void SetFullScreen(bool isFull);
        void ToggleFullScreen();
        void loadLastLogFile();

        void ChangePage(int dir);
        void UpdateAUI();

        virtual bool AcceptDrop();
        virtual bool DropFiles( const wxArrayString& files );

	protected:

		void OnQuit(wxCommandEvent& event);
		void OnAbout(wxCommandEvent& event);
		void OnGPSConfig(wxCommandEvent& event);
        void OnSelectGaugeInput(wxCommandEvent& event);
        void OnConfigSwitchUpload( wxCommandEvent& event );
        void OnLogTest(wxCommandEvent& event);
		void OnSoundConfig(wxCommandEvent& event);
		void OnLogAnalyze(wxCommandEvent& event);

        void OnNewProject(wxCommandEvent& event);

        void OnSetDefWorkingDir(wxCommandEvent & event);
        void OnRecentECU(wxCommandEvent & event);


        void OnLoadRecentProject(wxCommandEvent& event);

       // void OnUploadTables(wxCommandEvent& event);
       // void OnFetchTables(wxCommandEvent& event);
        //void OnGenTable(wxCommandEvent& event);
        void OnSession(wxCommandEvent& event);



        void OnSelectBaseDir(wxCommandEvent& event);
        void OnSelectHexFile(wxCommandEvent& event);
		void OnSelectMakeConfigGlobal(wxCommandEvent& event);
		void OnSelectMakeConfigTxt(wxCommandEvent& event);
		//void OnGenConfig(wxCommandEvent& event);
		void OnGenTrigger(wxCommandEvent& event);
        void OnTriggerConfig(wxCommandEvent& event);
        void OnMenuResetLayout(wxCommandEvent& event);

        void OnDlgTest(wxCommandEvent& event);
        void OnRacesAnalyzer(wxCommandEvent& event);
        void OnMixtureTableView(wxCommandEvent& event);
        void OnWinCETest(wxCommandEvent& event);
        void OnWinCETest2(wxCommandEvent& event);
        void OnWinCETest3(wxCommandEvent& event);
        void OnWinCETestTimer( wxTimerEvent&event );
        void OnDefWorkDirDBReset(wxCommandEvent& event);
        void OnDevOutChannel(wxCommandEvent& evt);
        void OnRTDataStat(wxCommandEvent& event);
        void OnAddGauge(wxCommandEvent& event);
        void OnResetLayout(wxCommandEvent& event);
        void OnResetToolBars(wxCommandEvent& event);
        void OnFullScreenMenu(wxCommandEvent& event);

        void OnMenuLoggerScreen(wxCommandEvent& event);

        void OnGGB(wxCommandEvent& event);

        void OnGGFloatBehav(wxCommandEvent& event);

        void OnAddSymbol(wxCommandEvent& event);
        void OnGaugeGroupEditor(wxCommandEvent& event);
        void OnMultiGraphProfiles(wxCommandEvent& event);
        void OnVemsFlagDescEditor(wxCommandEvent& event);

        void OnClose(wxCloseEvent& event);
        void OnGoToState( wxCommandEvent& event );

        void OnFWUploadedEvent( FirmwareUploadedEvent& event );

		wxString SetSerialDialog();

		void SetSoundDialog();
		int Execute(wxString& command, wxArrayString& output, wxArrayString& errors);


        void LogSetupInit();

        void OnActivate(wxActivateEvent& evt);

        void OnRawConvert(wxCommandEvent& event);

        void SavePerspective();

        void OnIdle(wxIdleEvent &event);
        void OnEditConfig( wxCommandEvent& event );
        void OnPasswordProtection( wxCommandEvent& event );
        void OnTorqueConfig( wxCommandEvent& event );
        void OnIniFileConfig( wxCommandEvent& event );
        void OnGaugeGroupTest( wxCommandEvent& event );
        void OnProtocolConfig( wxCommandEvent& event );
        void OnECUConfig( wxCommandEvent & event );

		static wxListBox* ListBox1;



        void OnKeyDown(wxKeyEvent &evt);

        void OnSetFocus(wxFocusEvent& evt);

        void OnIconize(wxIconizeEvent &evt);

        void OnGUICommand(VemsGUICommandEvent& evt);

        void OnMenuBarOpen(wxMenuEvent& evt);
        void OnMenuBarClose(wxMenuEvent& evt);

        void GenerateMenu(int id);

	private:
        void CreateProjectRecentSubMenu();
        void SortRecentProjects(Project *prj = NULL);

        void CreateRecentECUsMenu();
        void EnableSelectECUMenus(bool enabled);

        // ftype: 1 - only recent log menu, 2 - only recent cfg menu, 3 - both
        void CreateRecentFilesMenu( unsigned char fType );

        ProjectManager  *m_prjMgr;

        wxMenu* m_prjRecentMenu;
        wxMenu* m_setDefDirMenu;
        wxMenu* m_RecentECUMenu;
        wxMenu* m_recentLogsMenu;
        wxMenu* m_recentCfgsMenu;
        wxMenu* m_fileMenu;
        wxMenu* m_testMenu;

        FrameLayoutHandler *m_frameLayoutHandler;

        FirmwareMgrFrame *m_fwMgrFrame;

        VTFilesUpdater*   m_VTFilesUpdater;

        bool              m_fwuploadedsuccess;
        bool              m_roundCfgSaved;

        wxString V3GUI_VERSION;

        wxString m_rootDir;

        wxString m_pathMakeConfigTxt;
        wxString m_pathMakeConfigGlobal;
        wxString m_baseDir;
        wxBoxSizer * topSizer;
        uint16_t    m_mmcBackTime;



//        wxTimer m_timer;

        VemsTuneController* m_vemsTuneController;
        RTDataMgrBase*      m_rtDataMgr;
        STDataMgrBase*    m_stController;

        TriggerLogManager*  m_tlogmanager;
        TriggerLogGraph*    m_tlogGraph;


        GaugeGroupManager* m_gaugeGroupManager;


        wxTextFile* m_DataLogFile;
        bool m_timerToogle;

        void OnTimer(wxTimerEvent& event);
        void SetLogging();
        void test();

        wxFrame *LogSetFrame;
        wxBoxSizer *BoxSizerLS1;
        wxBoxSizer *BoxSizerLS2;
        wxRadioButton *RadioHighLog;
        wxRadioButton *RadioLowLog;
        wxStaticText *StaticText1;
        wxButton *ButtonLogStart;
        wxButton *ButtonLogStop;

        VemsTaskBarIcon *m_TaskBarIcon;
        WindowReg *wreg;
        wxString m_projectCaption;

        wxMenuItem *m_menuItemSelectInputNone;
        MainPanel* m_panel;
        wxMenu* m_firmwareMenu;
        wxMenu* m_projectMenu;
        //SFloodThread* m_floodThread;
        SFloodProcess* m_sflood;
        wxMenu*     m_selectInputMenu; //select input popup menu
        UserDialogManager* m_userDialogManager;
        ProgressBarObserver* m_pProgressBarSaveSD;

        wxHotKeyProfileManager  * m_hotKeyProfileMgr;

        // Lookup test
        uint8_t MAPCorrTable[8][8];
        uint8_t maxRPM; // 0..maxRPM is index 0..7
        uint8_t maxMAP; // [2kPa] 50..maxMAP is index 0..7

        void TableInit(void);
        uint8_t GetMAPCorrValue (int rpm, int map);

        bool                    m_isGroupVisible;
        StringHash              m_params;       //command line params
        bool                    m_isAfterStart; //true when first activate otherwise false
        int                     m_iSaveSDLogBackTime;
        bool                    m_bSaveSD;
        bool                    m_bGaugesStartup;
        UserMenuRefresher*          m_pUserMenuRefresher;

        int                     m_prevDevice;
        int                     m_rtStatWinID;
        bool                    m_isInecuNameAskLoop;
        ConfigSwitchState*      m_configSwitchStateViewer;

        ConfigDownloadDirector* m_configDownloadDirector;
        FirmwareConfigManager*  m_pFirmwareConfigManager;
        FirmwareDirector*       m_firmwareDirector;
        SDCardDirector*         m_sdCardDirector;

        VemsTuneEventDispatcher*  m_eventDispatcher;
        bool                    m_uploadFirmware; // upload firmware after config download
        bool                    m_projectAction; // call action from projetc menu

        VemsTuneInfo            m_VemsTuneInfo;
        bool                    m_isConfigValidationNeeded;

        void checkCommPort();
        void loadCurrentProject();
        bool showNoProjectDialog();
        void createMenu();
        void openVemsCfg( const wxString& filename = _T("showdialog") );

        void setStatusBarFieldText( StatusBarFieldId fieldId, const wxString& fieldText = wxEmptyString );
        void clearStatusBar();

        void showWebTool();
        void downloadConfig( bool showFileDialog );
        void initFrameLayout();


        void OnSFlood( wxCommandEvent& event );
        void OnMMCLog( wxCommandEvent& event );
        void OnHelpTopics( wxCommandEvent& event );
        void OnConvertLastLog( wxCommandEvent& event );
        void OnOpenLastLog( wxCommandEvent& event );
        void OnConvertAllLog( wxCommandEvent& event );
        void ExportMSQ( wxCommandEvent& event );
        void StartSparktTester( wxCommandEvent& event );
        void OnEnvironmentConfig( wxCommandEvent& event );
        void OnTableEditor3DPreferences( wxCommandEvent& event );
        void OnWindowCreate(wxWindowCreateEvent& event);

        void OnSelectRecentLog( wxCommandEvent& event );
        void OnSelectRecentConfig( wxCommandEvent& event );

        void OnAddECUDir(wxCommandEvent& event);
        void OnSelectECUFromDb(wxCommandEvent& event);

        void OnFileOpenDlg( wxCommandEvent& event );
        void OnTriggerLog( wxCommandEvent& event );

        /// Offline menu functions
        void OnSaveVemsCfg( wxCommandEvent & event );
        bool savevemscfg( bool showError = true );

        /// Firmware menu functions
        void OnFirmwareInfo(wxCommandEvent& event);
        void OnFirmwareUpload( wxCommandEvent& event );
        void OnFirmwareVerify( wxCommandEvent& event );
        void OnConfigUpload( wxCommandEvent& event );
        void OnFilteredConfigUpload( wxCommandEvent& event );
        void OnConfigVerify( wxCommandEvent& event );
        void OnConfigDownload( wxCommandEvent& event );
        void OnFirmwareWeb( wxCommandEvent& event );
        void OnClearDebug(wxCommandEvent& event);

        /// Project menu functions
        void OnMenuProject(wxCommandEvent& event);
        void OnProjectSettings(wxCommandEvent& event);
        void OnMenuNewProject(wxCommandEvent &evt);
        void OnProjectUpload( wxCommandEvent& event );
        void OnProjectFirmwareUpload( wxCommandEvent& event );
        void OnProjectFirmwareVerify( wxCommandEvent& event );
        void OnProjectConfigUpload( wxCommandEvent& event );
        void OnProjectConfigVerify( wxCommandEvent& event );
        void OnDeviceCannotConnect( VemsDeviceControllerEvent& event );

        /// Preferences
        void OnVTFilesUpdaterSettings( wxCommandEvent& event );

/// refactor communication
        DeviceControllerProgressBarManager*  m_progressBarManager;

        void OnCannotOpenPort( VemsDeviceControllerEvent& event );
        void OnDeviceFound( VemsDeviceControllerEvent& event );
        void OnDeviceNotFound( VemsDeviceControllerEvent& event );
        void OnDeviceConnected( VemsDeviceControllerEvent& event );
        void OnDeviceDisconnected( VemsDeviceControllerEvent& event );
        void OnDeviceReconnected( VemsDeviceControllerEvent& event );
        void OnDeviceControllerStopped( VemsDeviceControllerEvent& event );
        void OnDeviceResumed( VemsDeviceControllerEvent& event );
        void OnDeviceProtected( VemsDeviceControllerEvent& event );

        void OnCommunicationError( CommunicationEvent& event );

        void refreshLogDir();
        void RebuildGUI();

/// refactor communication

        void OnProperIniFileNotFound( wxEvent& event );
        void OnTestSendEvent( wxCommandEvent& event );
        void OnTestConfigParsing(  wxCommandEvent& event );
        void OnSiprTest(  wxCommandEvent& event );
        void OnDeviceConfig( wxCommandEvent& event );
        void OnAIMTest( wxCommandEvent& event );
        void OnLookUp (wxCommandEvent& event ) ;
        void OnSIPRInput( InputEvent& event );


        void OnVemsLogStartNew(wxCommandEvent &);
        void OnVemsLogAddMarker(wxCommandEvent &);
        void OnRefreshUserMenu( wxCommandEvent& event );
        void OnAdvFrame(wxCommandEvent& event);
        void OnConfigCheckerTest( wxCommandEvent& event );

        void OnDevIgnOutput( wxCommandEvent& event );
        void OnIgnLock(wxCommandEvent& event);
        void OnIACLock(wxCommandEvent& event);
        void OnIACControl(wxCommandEvent& event);
        void OnTachoLock(wxCommandEvent& event);

        //
        void OnCalcModelAnalyzis(wxCommandEvent& event);

        void OnGPSSettings( wxCommandEvent& event );
        void FirmwareConfigDownloaded( wxCommandEvent& event );

        void OnVemsTuneInfo( wxCommandEvent& event );

        void OnSaveEepromCfg( wxCommandEvent& event );

        void validateConfig();

        void OnFirmwareConfigUploaded( wxCommandEvent& event );

		DECLARE_EVENT_TABLE()
};

#endif

