#ifndef MTCONFIGDATA_H
#define MTCONFIGDATA_H

#include "Symbol.h"
#include "SymbolMgr.h"
#include "vemsgauge/vemsgaugedesc.h"
#include "vemsgauge/VemsTableDesc.h"
#include "vemsgauge/ExpressionGridDesc.h"
#include "vemsdialog/TblMapDesc.h"
#include "vemsdialog/VemsCurveDialogDesc.h"
#include "vemsdialog/SuperCurveDescriptor.h"
#include "vemsdialog/InterpolationWidgetDescriptor.h"
#include "dag/Graph.h"
#include "vemsprotocol/SIPRDescriptor.h"
#include "stdatamanager/UserDataDesc.h"

#include "common/types/Int2Int2StringSetHash.h"
#include "common/types/StringHash.h"
#include "common/types/BoolHash.h"

#include <wx/string.h>


class UserMenuList;
class UserDialogList;

class DataLogEntry
{
    public:
        wxString m_name;
        wxString m_label;
        wxString m_type;
        wxString m_format;

        DataLogEntry(wxString name, wxString label, wxString type, wxString format);
};


WX_DEFINE_ARRAY(DataLogEntry*, DataLogEntryArray);

class MTConfigData
{
    public:
        ExpressionGridDescArray         m_expressionGridDescriptors;
        InterpolationWidgetDescH        m_interpolationWidgets;
        UserDataDescArray               m_userDataDescA;
        TblMapDescHash                  m_tableMappers;
        StringHash                      m_iniHelps;

    public:
        MTConfigData();
        virtual ~MTConfigData();

        //TEST
        SymbolTable&            getSymbolTable() { return m_symbolMgr.m_allSymbolTable; }
        SymbolMgr&              getSymbolMgr() { return m_symbolMgr; }

        //Symbol table functions
        bool                    isValid(){ return m_isValid; }
        void                    setValid(bool v){ m_isValid = v; }

        void                    addSymbol(Symbol* symbol);
        Symbol*                 getSymbol(const wxString& symbolName);
        Symbol*                 getCfgSymbol(const wxString& symbolName);
        int                     getSymbolPage(const wxString& symbolName);
        wxString                getSymbolType(const wxString& symbolName);

        void                    addGroup( const wxString& name );
        void                    setGroupOffset( const wxString& name, int offset );
        SymbolGroup*            getGroup( const wxString& groupName);





        void                    addDataLogEntry(wxString name, wxString label, wxString type, wxString Format);
        DataLogEntryArray*      getDataLogEntries() { return &m_DataLogEntryArray; }


        //user menu
        UserMenuList*           getUserMenuList() { return m_UserMenuList; }
        UserDialogList*         getUserDialogList() { return m_UserDialogList; }

        //table editor
        VemsTableDescH*         getTables() { return &m_tables; }

        const Graph&            getDAG() const { return m_symbolMgr.m_dag; }
        VemsCurveDialogDescH&   getCurveDialogs() { return m_curveDialogs; }
        SuperCurveDescriptorH&  getSuperCurveDialogs() { return m_superCurveDialogs; }
        void                    setSignature( const wxString& signature ) { m_signature = signature; }
        wxString                getSignature() const { return m_signature; }
        void                    clear();

        void                    setFileVersion( const wxString s ) { m_fileVersion = s; }
        wxString                getFileVersion() const { return m_fileVersion; }


        SIPRDescriptor*         getSIPRDescriptor() { return &m_SIPRDescriptor; }


        const StringSet*        getDialogNamesBySymbol(const Symbol& symbol) const;
        void                    fillDialogNamesBySymbol(const wxString& currentDialog, const wxString& symbolName);
        void                    fillMenuAndDialogNamesByDialogId(const wxString& dialogId, const wxString& name );
        wxString                getMenuAndDialogNameByDialogId(const wxString& id) const;
        void                    addParam( const wxString& name, bool value ) { m_params[name] = value; }
        const BoolHash&         getParams() { return m_params; }
        bool                    isRawLambdaUsed();
        ExpressionGridDesc*     getExpressionGridDescriptor( const wxString& name );

    private:
        SymbolMgr               m_symbolMgr;

        VemsTableDescH          m_tables;
        VemsCurveDialogDescH    m_curveDialogs;
        SuperCurveDescriptorH   m_superCurveDialogs;
        DataLogEntryArray       m_DataLogEntryArray;
        UserMenuList*           m_UserMenuList;
        UserDialogList*         m_UserDialogList;
        wxString                m_signature;        //[megatune] siganture = ...
        wxString                m_fileVersion;      // parsed file verison ( get from filename )
        SIPRDescriptor          m_SIPRDescriptor;

        BoolHash                m_params;

        ///StringSet;
        ///Int2StringSetHash;	key = offset, value: StringSet
        ///Int2Int2StringSetHash; key = pageNo, value: Int2Int2StringSet
        Int2Int2StringSetHash   m_dialogSetByPageAndOffset;

        Int2Int2Int2StringSetHash m_dialogSetByPageAndOffsetAndMask;

        StringHash              m_menuAndDialogNamesH;
        bool                    m_isValid;
};

#endif // MTCONFIGDATA_H
