#include "ProjectManager.h"
#include "../logger.h"
#include "../firmwareconfigitem.h"
#include "../DirHelper.h"
#include "../wxXmlHelper.h"
#include "../UserLog.h"

#include <wx/filename.h>
#include "../common/typedef.h"

ProjectManager::ProjectManager()
{
    m_currPrj = NULL;
    m_mainStatusBar = NULL;
    //ctor
    m_recentsFile = DirHelper::GetPath(dCONFIG) + _T("RecentProjects.xml");

    Logger::logf(LOG_MSG,_T("ProjectManager 10"));
    LoadRecents();
    Logger::logf(LOG_MSG,_T("ProjectManager 20"));
}

ProjectManager::~ProjectManager()
{
    //dtor
}


Project* ProjectManager::getProject(const wxString& filename)
{
    //    Logger::logf(LOG_MSG,_T("ProjectManager::LoadProjectFile ")+filename);

    wxXmlDocument prjXml;

    if(!prjXml.Load(filename, _T("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES)) return NULL;

    //Logger::logf(LOG_MSG,_T("m_prjXml.GetRoot()->GetName()")+prjXml.GetRoot()->GetName());

    if(prjXml.GetRoot()->GetName() != _T("VemsTuneProjectFile")) return NULL;

    //wxXmlNode* detailNode = prjXml.GetRoot()->GetChildren();

    Project *prj = new Project();
/*
    while( detailNode )
    {
        prj->Details[detailNode->GetName()] = detailNode;
        detailNode = detailNode->GetNext();
    }
*/
    //prjXml.GetRoot()

    prj->Doc  = prjXml;

    prj->Path = filename;

    prj->Set();

    Logger::logf(LOG_MSG,_T("ProjectManager::getProject END"));

    return prj;
}

Project* ProjectManager::AddProject(const wxString& filename)
{
    Logger::logf(LOG_MSG,_T("filename=")+filename);

    Project* prj = ProjectManager::getProject(filename);
    if(prj == NULL) return NULL;
    m_projects.Add(prj);
    return prj;
}

Project * ProjectManager::GetProjectByName(const wxString& name)
{

    for (uint8_t a=0; a < m_projects.GetCount(); a++ ){
        if(m_projects[a]->Name == name)
            return m_projects[a];
    }

    return NULL;

}

Project * ProjectManager::GetProjectByPath(const wxString& path)
{

    for (uint8_t a=0; a < m_projects.GetCount(); a++ ){
        if(m_projects[a]->Path == path)
            return m_projects[a];
    }

    return NULL;

}

Project * ProjectManager::GetProjectByMenuID(int id)
{

    for (uint8_t a=0; a < m_projects.GetCount(); a++ ){
        if(m_projects[a]->MenuId == id)
            return m_projects[a];
    }

    return NULL;

}

void ProjectManager::SetCurrPrj(Project * prj)
{
    Logger::logf(LOG_MSG,_T("ProjectManager::SetCurrPrj p= %p"),prj);


    m_currPrj = prj;

    MoveProject(prj,0);

    Logger::logf(LOG_MSG,_T("ProjectManager::SetCurrPrj 2"));

    UpdateStatusBar();


    Logger::logf(LOG_MSG,_T("ProjectManager::SetCurrPrj 3"));
    //RefreshMenus();

    if(!prj) return;



    SaveRecents();

    Logger::logf(LOG_MSG,_T("ProjectManager::SetCurrPrj 4"));

}

void ProjectManager::RefreshMenus()
{
    bool show = false;

    if(m_currPrj) show = true;

    for (uint8_t a=0; a < m_menuRefreshers.GetCount();a++ ){
        MenuRefresher * rm = m_menuRefreshers[a];
        for (uint8_t b=0; b < rm->IDs.GetCount();b++ )
            rm->Menu->Enable(rm->IDs[b],show);
    }




}

void ProjectManager::LoadRecents()
{
    wxString recsfname = m_recentsFile;

    wxXmlDocument doc;

    if(!doc.Load( recsfname, _T("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES))  return;

    wxXmlNode* root = doc.GetRoot();

    if(root->GetName() != _T("RecentProjects")) return;

    for(wxXmlNode* projects = root->GetChildren(); projects; projects = projects->GetNext() )    {

        if( projects->GetName() == _T("File") )
        {


            wxString prjPath = projects->GetPropVal(_T("path"),wxEmptyString);
            wxString Name = GetProjectName(prjPath);
            if(Name == wxEmptyString) continue;

            AddProject(prjPath);
        }


    }


}

bool ProjectManager::MoveProject(Project * prj, int idx)
{

    int fidx = m_projects.Index(prj);

    if(fidx == wxNOT_FOUND) return false;

    m_projects.RemoveAt(fidx);

    m_projects.Insert(prj,idx);

    return true;

}


void ProjectManager::SaveRecents()
{
    if(m_projects.GetCount() < 1) return;

    wxXmlNode* root = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, _T("RecentProjects"));

    wxXmlNode* childnode = new wxXmlNode(root, wxXML_ELEMENT_NODE, _T("File"));;

    root->SetChildren(childnode);

    for(short a = m_projects.GetCount()-1; a >= 0 ; a--){
        Logger::logf(LOG_MSG,_T("%i"),a);

        if(!childnode) {
            childnode = new wxXmlNode(root, wxXML_ELEMENT_NODE, _T("File"));
            root->SetNext(childnode);
        }

        childnode->AddProperty(_T("path"), m_projects[a]->Path);

//        Logger::logf(LOG_MSG,_T("ProjectManager::SaveRecents() m_projects[%i]->Name = %s"),a,m_projects[a]->Name.c_str());

        childnode = NULL;

    }

    wxXmlDocument doc;
    doc.SetRoot(root);
    doc.Save(m_recentsFile);

}

wxString ProjectManager::GetProjectName(const wxString& path)
{

    wxXmlDocument doc;

    if(!doc.Load( path, _T("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES)) {
        //Logger::logf(LOG_MSG,_T("ProjectChooserFrame::GetProjectName"));
      //  if(!doc.Load( DirHelper::GetPath(dCONFIG)+path, _T("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES))
        return wxEmptyString;
    }

    wxXmlNode* root = doc.GetRoot();

    if( root->GetName() != _T("VemsTuneProjectFile") ) return wxEmptyString;

    wxXmlNode * prjnode = wxXmlHelper::GetNodeByName(_T("ProjectSettings"),root);

    wxString Name;

    if(prjnode){

        wxXmlNode * childNode = wxXmlHelper::GetNodeByName(_T("set"),prjnode);

        Name = childNode->GetPropVal(_T("name"),wxEmptyString);

    }

    return Name;

}


void ProjectManager::UpdateStatusBar()
{

    if(m_mainStatusBar == NULL) return;

    wxString str;

    if(m_currPrj)
        str = m_currPrj->Name;
    else
        str = _T("none");

    m_mainStatusBar->SetStatusText(_T("Project: ") + str , 2);

}

bool ProjectManager::InsertProject(const wxString& filename,int idx)
{
    Project * prj = getProject(filename);
    if(prj == NULL)  return false;

    InsertProject(prj,idx);
    return true;
}

bool ProjectManager::InsertProject(Project * prj,int idx)
{
    m_projects.Insert(prj,idx);
    return true;
}



/*********************************************************
 *                   Project functions
 *********************************************************/


FirmwareConfigItemArray Project::CreateFirmwareConfigItems(wxArrayString &types)
{


    FirmwareConfigItemArray arr;

    FirmwareConfigItem *defitem = new FirmwareConfigItem();

    wxFileName fn(Path);

    defitem->type = _T("base");
    defitem->name = _T("Base");
    defitem->value = fn.GetPath();
    defitem->parentName = _T("Base");
    defitem->isFile = false;
    defitem->mandatory = true;
    defitem->validate = true;
    defitem->active = true;


    arr.Add(defitem);

    FirmwareConfigItem *item;



    if(types.Index(_T("header")) != wxNOT_FOUND) {
        item = new FirmwareConfigItem();


        item->type = _T("header");
        item->name = _T("Global header");
        item->value = m_fwInfo.getHeaderFile();
        item->isFile = true;
        item->mandatory = true;
        item->validate = true;
        item->active = false;

        arr.Add(item);

        types.Remove(_T("header"));
    }


    if(types.Index(_T("firmware")) != wxNOT_FOUND) {
        item = new FirmwareConfigItem();


        item->type = _T("firmware");
        item->name = _T("Hex file");
        if(Type == 1)
            item->value = _T("my_make.16x14/");

        item->value += _T("vems.hex");

        item->isFile = true;
        item->mandatory = true;
        item->validate = true;
        item->active = false;

        arr.Add(item);

        types.Remove(_T("firmware"));
    }


    wxXmlNode* detailNode = Doc.GetRoot()->GetChildren();

    if(Doc.GetRoot()->GetName() != _T("VemsTuneProjectFile")) return arr;

    while( detailNode )
    {

        for(wxXmlNode * itemNode = detailNode->GetChildren(); itemNode ;itemNode = itemNode->GetNext() )
        {

            if(itemNode->GetName() == _T("Group")) {

                wxString grp = itemNode->GetPropVal(_T("name"),wxEmptyString);

                for(wxXmlNode * itemSubNode = itemNode->GetChildren(); itemSubNode ;itemSubNode = itemSubNode->GetNext() ) {

                    item = FillFwCfgItem(itemSubNode, types, defitem);

                    if(!item) continue;

                    item->grpName = grp;

                    arr.Add(item);
                //    Logger::logf(LOG_MSG,_T("+++++++++++ name=%s, value=%s, status=%i"),item->name.c_str(),item->value.c_str(),item->status);
                }

            } else if(itemNode->GetName() == _T("File")) {

                item = FillFwCfgItem(itemNode, types, defitem);

                if(!item) continue;

                arr.Add(item);
              //  Logger::logf(LOG_MSG,_T("++++++!!!!! name=%s, value=%s, status=%i"),item->name.c_str(),item->value.c_str(),item->status);

            }




        }

        detailNode = detailNode->GetNext();
    }
    return arr;

}

bool Project::AddFirmwareConfigItem(FirmwareConfigItem *beforeitem, FirmwareConfigItem *item)
{


    Logger::logf(LOG_MSG,_T("beforeitem->status = %i"),beforeitem->status);
    Logger::logf(LOG_MSG,_T("beforeitem->name = ")+beforeitem->name + _T(", item->name = ") + item->name);

    wxXmlNode * beforenode = wxXmlHelper::FindChildNodeByProp(Doc.GetRoot(), _T("name") , beforeitem->name);

//    <File type="config" name="727cc Inj." path="aan/inj_727cc.txt" parent="Base" active="0"/>
    if(!beforenode) return false;

   // return false;

    wxXmlNode * parent = beforenode->GetParent();

    wxXmlNode * newitem = new wxXmlNode(wxXML_ELEMENT_NODE, beforenode->GetName());

    newitem->AddProperty(_T("type"),item->type);
    newitem->AddProperty(_T("name"),item->name);
    newitem->AddProperty(_T("path"),item->value);
    newitem->AddProperty(_T("parent"),item->parentName);
    newitem->AddProperty(_T("active"),wxString::Format(_T("%i"),item->active));

//    return parent->InsertChildAfter(newitem, beforenode);

    parent->AddChild(newitem);

    return true;
}

bool Project::RemoveFirmwareConfigItem(FirmwareConfigItem *item)
{

    wxXmlNode * itemnode = wxXmlHelper::FindChildNodeByProp(Doc.GetRoot(), _T("name") , item->name);

    if(!itemnode) return false;

    wxXmlNode * parent = itemnode->GetParent();

    if(!parent) return false;

    return parent->RemoveChild(itemnode);

}

bool Project::ModFirmwareConfigItem(FirmwareConfigItem *item)
{

    wxXmlNode * itemnode = wxXmlHelper::FindChildNodeByProp(Doc.GetRoot(), _T("name") , item->name);

    if(!itemnode) return false;

    SetItemNodeProperties(itemnode, item->value, item->active);

    return true;

    /*
    wxXmlNode* detailNode = Doc.GetRoot()->GetChildren();

    while( detailNode )
    {

        for(wxXmlNode * itemNode = detailNode->GetChildren(); itemNode ;itemNode = itemNode->GetNext() )
        {

            if(item->name == itemNode->GetPropVal(_T("name"),wxEmptyString)) {
                SetItemNodeProperties(itemNode, item->value, item->active);
                return true;
            }

        }

        detailNode = detailNode->GetNext();
    }

    return false;

    */
}

bool Project::SaveFirmwareConfigItem(FirmwareConfigItem *item)
{
    return false;
}

bool Project::SaveConfig(const FirmwareConfigItemArray  &arr)
{

    FirmwareConfigItem *item = NULL;

    for(uint8_t a=0; a < arr.GetCount(); a++){

        item = arr[a];

        //Logger::logf(LOG_MSG,_T("P:S name=%s, value=%s, status=%i, idx=%i"),item->name.c_str(),item->value.c_str(),item->status,a);

        switch(item->status) {

            case FirmwareConfigItem::FWIS_NEW:
                if(a > 0)
                    AddFirmwareConfigItem(arr[a-1],item);
                break;

            case FirmwareConfigItem::FWIS_VIRGIN:
                break;

            case FirmwareConfigItem::FWIS_REMOVED:
                RemoveFirmwareConfigItem(item);
                break;

            case FirmwareConfigItem::FWIS_MODIFIED:
                ModFirmwareConfigItem(item);
                break;

            default:
                break;
        }


    }

    return Doc.Save(Path, wxXML_NO_INDENTATION);

};



bool Project::SetItemNodeProperties(wxXmlNode* itemNode, const wxString& value, bool isActive )
{
    wxXmlProperty* properties = itemNode->GetProperties();

    char a=0;

    while( properties )
    {
        wxString propName = properties->GetName();

        if( propName == _T("path") )
        {
            properties->SetValue( value );
            a++;
        }
        else if( propName == _T("active") )
        {
            properties->SetValue( wxString::Format( _T("%i"), isActive ) );
            a++;
        }

        if(a==2) return true;

        properties = properties->GetNext();
    }

    return false;

}



void Project::SaveSettings()
{
    Doc.Save(Path, wxXML_NO_INDENTATION);
}

void Project::SetFwVers(const wxString& str)
{

    m_fwInfo.setFirmwareVersion(str);
    //FirmwareInfo::extractFirmwareVersion( str, FwVersion);

}

void Project::Set()
{

    wxXmlNode * prjnode = wxXmlHelper::GetNodeByName(_T("ProjectSettings"),Doc.GetRoot());


    if(prjnode){

        wxXmlNode * childNode = wxXmlHelper::GetNodeByName(_T("set"),prjnode);

        if(!childNode) {
            UserLog::uLog(ULOG_ERROR, _T("<b>Wrong project file!</b>"));
            return;
        }

        Name = childNode->GetPropVal(_T("name"),wxEmptyString);
        SetFwVers(childNode->GetPropVal(_T("firmwareversion"),_T("0.0.0")));
        childNode->GetPropVal(_T("type"),_T("0")).ToLong(&Type);

    } else {

        Name = Doc.GetRoot()->GetPropVal(_T("name"),wxEmptyString);
        SetFwVers( Doc.GetRoot()->GetPropVal(_T("firmwareversion"),_T("0.0.0")));
        Doc.GetRoot()->GetPropVal(_T("type"),_T("0")).ToLong(&Type);
    }

}


/**
 * \brief Get active config files from project
 */
wxArrayString Project::GetConfigFiles()
{
    wxXmlNode* detailNode = Doc.GetRoot()->GetChildren();

    wxArrayString configFiles;

    if ( Doc.GetRoot()->GetName() != _T("VemsTuneProjectFile") ) { return configFiles; }

    wxFileName fn( Path );
    wxString base = fn.GetPath( wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR );

    while( detailNode )
    {
        for( wxXmlNode* itemNode = detailNode->GetChildren(); itemNode ;itemNode = itemNode->GetNext() )
        {
            if ( itemNode->GetPropVal( _T("type"), wxEmptyString ) != _T("config") ) { continue; }

            if ( itemNode->GetPropVal( _T("active"), wxEmptyString ) != _T("1") ) { continue; }

            wxString fileName;

            if ( itemNode->GetPropVal( _T("parent"), wxEmptyString) == _T("Base") )
            {
                fileName.Append( base );
            }

            fileName.Append( itemNode->GetPropVal( _T("path"), wxEmptyString ) );

            configFiles.Add( fileName );
        }

        detailNode = detailNode->GetNext();
    }

    return configFiles;
}

/**
 * \brief Get global.h from project
 */
wxString Project::GetConfigIniFile()
{
    /*
    wxXmlNode* detailNode = Doc.GetRoot()->GetChildren();

    wxString iniFile;

    if ( Doc.GetRoot()->GetName() != _T("VemsTuneProjectFile") ) { return iniFile; }

    wxFileName fn( Path );
    wxString base = fn.GetPath( wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR );

    while( detailNode )
    {
        for( wxXmlNode* itemNode = detailNode->GetChildren(); itemNode ;itemNode = itemNode->GetNext() )
        {
            if ( itemNode->GetPropVal( _T("type"), wxEmptyString ) != _T("header") ) { continue; }

            if ( itemNode->GetPropVal( _T("parent"), wxEmptyString) == _T("Base") )
            {
                iniFile.Append( base );
            }

            iniFile.Append( itemNode->GetPropVal( _T("path"), wxEmptyString ) );

            return iniFile;
        }

        detailNode = detailNode->GetNext();
    }
*/
    return m_fwInfo.getHeaderFile();
}

FirmwareConfigItem * Project::FillFwCfgItem(wxXmlNode *itemNode, wxArrayString &types, FirmwareConfigItem *defitem)
{

    int idx = types.Index(itemNode->GetPropVal(_T("type"),wxEmptyString));
    if( idx == wxNOT_FOUND ) return NULL;

    FirmwareConfigItem * item = new FirmwareConfigItem();

    long tmp;

    item->name = itemNode->GetPropVal(_T("name"),wxEmptyString);
    item->value = itemNode->GetPropVal(_T("path"),wxEmptyString);
    item->parentName = itemNode->GetPropVal(_T("parent"),wxEmptyString);

    if(item->parentName == _T("Base")){
        item->parent = defitem;
    }

    item->type = types[idx];

    itemNode->GetPropVal(_T("active"),_T("0")).ToLong(&tmp);
    item->active = tmp!=0?true:false;

    itemNode->GetPropVal(_T("isFile"),_T("1")).ToLong(&tmp);
    item->isFile = tmp!=0?true:false;

    itemNode->GetPropVal(_T("mandatory"),_T("0")).ToLong(&tmp);
    item->mandatory = tmp!=0?true:false;

    itemNode->GetPropVal(_T("validate"),_T("1")).ToLong(&tmp);
    item->validate = tmp!=0?true:false;

    return item;

}
