#include "ProjectWizard.h"

#include <wx/statline.h>
#include <wx/xml/xml.h>

#include "../UserLog.h"
#include "../DirHelper.h"
#include "../SimpleFwDlg.h"

BEGIN_EVENT_TABLE(ProjectWizard, wxDialog)

    EVT_BUTTON(idBtnFwDlg, ProjectWizard::OnButton)
    EVT_BUTTON(idBtnSaveProject,ProjectWizard::OnButton)
    EVT_BUTTON(idBtnClose,ProjectWizard::OnButton)
    EVT_BUTTON(idBtnSaveAndLoad,ProjectWizard::OnButton)


END_EVENT_TABLE()

ProjectWizard::ProjectWizard(wxWindow * parent, int id, const wxString &title)
{
    //ctor
    Create(parent, id, title);

    m_projectName = new wxTextCtrl(this, wxID_ANY, _T(""));


    m_fwDlg = new SimpleFwDlg(this, _T("Select a firmware and patches for the project"));
    m_fwDlg->Init();

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer * namesizer = new wxBoxSizer(wxHORIZONTAL);

    namesizer->Add( new wxStaticText(this, -1, _T("Project name:")),0,wxALIGN_CENTER_VERTICAL|wxALL,5);

    namesizer->Add( m_projectName, 1, wxALL | wxEXPAND, 5);

    mainsizer->Add( namesizer, 0, wxEXPAND);

    mainsizer->Add( new wxStaticLine(this,wxID_ANY,wxDefaultPosition, wxSize(200,-1)), 0, wxALL | wxEXPAND, 5 );

    mainsizer->Add( new wxButton(this,idBtnFwDlg,_T("Firmware and patch selector")),0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    mainsizer->Add( new wxStaticLine(this,wxID_ANY,wxDefaultPosition, wxSize(200,-1)), 0, wxALL | wxEXPAND, 5 );

    wxBoxSizer * btnsizer = new wxBoxSizer(wxHORIZONTAL);



    btnsizer->Add( new wxButton(this,idBtnSaveProject ,_T("Save project")),0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);
    btnsizer->Add( new wxButton(this,idBtnClose ,_T("Cancel")),0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);
    btnsizer->Add( new wxButton(this,idBtnSaveAndLoad ,_T("Save and Load")) ,0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);


    mainsizer->Add( btnsizer );

	SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);

    m_projectPath =  wxEmptyString;
}

ProjectWizard::~ProjectWizard()
{
    //dtor
}

bool ProjectWizard::SetupAndSaveProject()
{

    if( m_configItemA.GetCount() == 0 ){
        UserLog::uLog(ULOG_ERROR, _T("Firmware not found! Choose a firmware."));
        return false;
    }

    FwPath * fwp = m_fwDlg->GetFwPath();

    if(!fwp) {
        UserLog::uLog(ULOG_ERROR, _T("Firmware not found!"));
        return false;
    }

    if( m_projectName->GetValue().Len() == 0 ){
        UserLog::uLog(ULOG_ERROR, _T("The project name field is empty!"));
        return false;
    }

    wxFileDialog dlg(this,_T("Choose a file"), DirHelper::GetPath(), _T(""), _T("*.vemsprj"), wxFD_SAVE );

    if( dlg.ShowModal() != wxID_OK ) return false;

    wxXmlDocument doc;

    wxXmlNode * root = new wxXmlNode(wxXML_ELEMENT_NODE, _T("VemsTuneProjectFile"));

    doc.SetRoot(root);

    /// ProjectSettings node

    wxXmlNode * node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("ProjectSettings"));

    wxXmlNode * setnode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("set"));

    setnode->AddProperty( _T("name"), m_projectName->GetValue() );
    setnode->AddProperty( _T("firmwareversion"), fwp->GetVer().toString() );
    setnode->AddProperty( _T("type"), wxString::Format(_T("%i"),fwp->GetCurrType() ));

    node->AddChild(setnode);

    root->AddChild(node);

    /// ConfigFiles node

    node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("ConfigFiles"));
    root->AddChild(node);


    /// ConfigFiles node

    node = new wxXmlNode(wxXML_ELEMENT_NODE, _T("FirmwareFiles"));


    for(uint8_t a=0;a<m_configItemA.GetCount();a++){
        if( m_configItemA[a]->type == _T("firmware_patch") ) {
            wxXmlNode * itemnode = new wxXmlNode(wxXML_ELEMENT_NODE, _T("File"));
            itemnode->AddProperty( _T("type"), _T("firmware_patch") );
            itemnode->AddProperty( _T("name"),  m_configItemA[a]->name );
            itemnode->AddProperty( _T("path"),  m_configItemA[a]->value );
            itemnode->AddProperty( _T("active"), _T("1") );
            node->AddChild(itemnode);
        }

    }


    root->AddChild(node);

    bool res = doc.Save( dlg.GetPath() );

    if(res) {
        UserLog::uLog(ULOG_OK, _T("<b>New project created and file saved.</b></font><br>File: ") + dlg.GetPath() );
        m_projectPath = dlg.GetPath();
    }
    else
        UserLog::uLog(ULOG_ERROR, _T("Could not saved the project file.</font><br>File: ") + dlg.GetPath() );

    return res;
}

void ProjectWizard::OnButton(wxCommandEvent &evt)
{
    switch( evt.GetId() ){
        case idBtnFwDlg:
            SetupFirmware();
            break;

        case idBtnSaveProject:
            if(SetupAndSaveProject())
                EndModal(wxID_OK);
            break;

        case idBtnClose:
            EndModal(wxID_CANCEL);
            break;

        case idBtnSaveAndLoad:
            if(SetupAndSaveProject())
                EndModal(wxID_APPLY);
            break;



    }
}

void ProjectWizard::SetupFirmware()
{
    if( m_fwDlg->ShowModal() != wxID_OK  ) return;

    m_fwDlg->getFirmwareConfigItemArray( m_configItemA );
}

