#include "RingBufferSync.h"

#include <wx/thread.h>

#include "RingBuffer.h"
#include "SyncObj.h"
#include "../logger.h"


RingBufferSync::RingBufferSync(RingBuffer* rb, SyncObj* syncObj)
{
    //ctor
    m_ringBuffer = rb;
    m_syncObj = NULL;
    m_isNotEmpty = syncObj->m_isNotEmpty;
    m_isNotFull =  syncObj->m_isNotFull;
}

RingBufferSync::RingBufferSync(int bufferCount, int bufferSize)
{
    m_ringBuffer = new RingBuffer(bufferCount, bufferSize);
    m_syncObj =  new SyncObj();

    m_isNotEmpty = m_syncObj->m_isNotEmpty;
    m_isNotFull =  m_syncObj->m_isNotFull;
}

RingBufferSync::~RingBufferSync()
{
    //dtor
    if(m_syncObj){
        delete m_syncObj;
        delete m_ringBuffer;
    }
}

/// Producer will use it.
void RingBufferSync::waitRingBufferNotFull()
{
    while( m_ringBuffer->isFull() ) {
            //Logger::logf(LOG_MSG, _T("RingBufferSync::waitRingBufferNotFull() isFull() - WAITING "));
            m_isNotFull->Wait(); // signaled by writer;
            //Logger::logf(LOG_MSG, _T("RingBufferSync::waitRingBufferNotFull() isFull() - RELEASED"));
    }
}
void RingBufferSync::nextHeadAndBroadcast()
{
    bool isEmpty = m_ringBuffer->isEmpty();
    m_ringBuffer->nextHead();
    if(isEmpty) m_isNotEmpty->Broadcast();
}

/// Consumer will use it.
void RingBufferSync::waitRingBufferNotEmpty()
{
    while( m_ringBuffer->isEmpty() )
    {
    //        Logger::logf(LOG_MSG, _T("RingBufferReaderThread::Entry() isEmpty() "));
        m_isNotEmpty->Wait(); // signaled by writer;
      //      Logger::logf(LOG_MSG, _T("RingBufferReaderThread::Entry() isEmpty() released"));
    }
}

void RingBufferSync::nextTailAndBroadcast()
{
    bool isFull = m_ringBuffer->isFull();
    m_ringBuffer->nextTail();
    if(isFull) m_isNotFull->Broadcast();
}
