#include "SFloodProcess.h"

#include "../common/FSMString.h"
#include "../CommunicationBase.h"
#include "../logger.h"

SFloodProcess::SFloodProcess(CommunicationBase* commBase)
{
    //ctor
    m_commBase = commBase;
    m_fsm = new FSMString(_T("AVR2"));
    m_txState = TX_S;
}

SFloodProcess::~SFloodProcess()
{
    //dtor
    delete m_fsm;
}

void SFloodProcess::start()
{
    m_fsm->init();
    this->ProcessBase::start();
    Logger::logf( LOG_MSG, _T("SFloodProcess::start()") );
}

void SFloodProcess::stop()
{
    this->ProcessBase::stop();
    m_commBase->close();
}


bool SFloodProcess::process()
{
    writeBytes();
    return readBytes()!=0?true:false;
}

void SFloodProcess::reset()
{

}

void SFloodProcess::finished()
{
    /// finished with ok or timeout!
    /// check again
    if(m_fsm->isEnded() ){
        /// successfully ended
        Logger::logf( LOG_MSG, _T("SFloodProcess::finished() FOUND BOOT!") );
        observerNotify( 1 );
    }else{
        /// timeout occured
        observerNotify( 0 );
    }
    Logger::logf( LOG_MSG, _T("SFloodProcess::finished() END") );
}




/// Must be non blocking (invoked from thread)
int SFloodProcess::readBytes()
{
    //Logger::logf( LOG_MSG, _T("CommunicationController::readBytes()") );
    int rxCount = 1;
    int receivedBytes = m_commBase->Read(m_rxBuff, rxCount);
    if(receivedBytes == 0){

    }else{
        m_fsm->rx(m_rxBuff[0]);
        if(m_fsm->isEnded() ){
            return 0;       /// will stop the thread
        }
    }
    return 1;
//
//    if( isTimeout() ){
//        /// notify observers
//    }
}

/// Must be non blocking (invoked from thread)
//int SFloodProcess::readBytes()
//{
//    //Logger::logf( LOG_MSG, _T("CommunicationController::readBytes()") );
//    int rxCount = 1;
//    int receivedBytes = 0;
//    int sumBytes = 0;
//    do{
//        receivedBytes = m_commBase->Read(m_rxBuff, rxCount);
//        sumBytes += receivedBytes;
//    }while(receivedBytes);
//
//    return sumBytes;
//}

/// Must be non blocking (invoked from thread)
int SFloodProcess::writeBytes()
{
    //Logger::logf( LOG_MSG, _T("CommunicationController::writeBytes()") );
    int txCount = getTx(m_txBuff);
   //Logger::logf( LOG_MSG, _T("CommunicationController::writeBytes() - %d"), txCount );
    if ( txCount > 0 ){
        m_commBase->SendString(m_txBuff, txCount);
        //Logger::logf( LOG_MSG, _T("SFloodProcess::writeBytes() Send S!") );
    }
    //}
    return txCount;
}


int SFloodProcess::getTx(uint8_t* m_txBuff)
{
    switch(m_txState){
        case TX_S:
            m_txBuff[0] = 'S';
            return 1;
        case TX_WAIT:
            return 0;
    }
    return 0;
}
