#include "RTDataCSVWriter.h"

#include <wx/textfile.h>

#include "../mtconfigdata.h"
#include "../RealTimeDesc.h"
#include "../common/CSVConverter.h"

#include "../logger.h"

RTDataCSVWriter::RTDataCSVWriter(const DataLogEntryArray* entries)
{
    //ctor
    m_entries = entries;
    m_file = new wxFile();

    m_expression = wxEmptyString;
    m_filter = wxEmptyString;
    m_isSymbolsRegistered = false;
}

RTDataCSVWriter::~RTDataCSVWriter()
{
    //dtor
    UnRegisterViewer();
    m_file->Close();
}

void RTDataCSVWriter::closeFile()
{
    m_file->Close();
}

void RTDataCSVWriter::require()
{
    if( !m_isSymbolsRegistered )
    {
        for(size_t i=0; i<m_entries->GetCount(); i++)
        {
            m_rtDataMgr->registerSymbol( m_entries->Item(i)->m_name );
        }

        m_isSymbolsRegistered = true;
    }
    requireChanged();
}

void RTDataCSVWriter::unregisterSymbols()
{
    if( m_isSymbolsRegistered )
    {
        for( size_t i = 0; i<m_entries->GetCount(); i++ )
        {
            m_rtDataMgr->unregisterSymbol( m_entries->Item(i)->m_name );
        }

        m_isSymbolsRegistered = false;
    }
}

/** @brief set
  *
  * @todo: document this function
  */
void RTDataCSVWriter::setFileName(const wxString& filename)
{
    m_fileName = filename;
}

/** @brief writeHeaders
  *
  * @todo: document this function
  */
bool RTDataCSVWriter::writeHeaders()
{
    if(!m_file->Open(m_fileName, wxFile::write )) return false;

    ///wxString line = CSVConverter::getHeader(*m_entries);

    //Logger::logf(LOG_MSG,_T("RTDataLogWriter::writeHeaders() columns:%i"),  m_entries->GetCount() );
    wxString header = _T("#VemsTune\n");

    for(size_t i=0; i<m_entries->GetCount(); i++){
        header += m_entries->Item(i)->m_label + _T("\t");
    }

    header += _T("\n\0");

    //m_file->AddLine(header);
    m_file->Write(header);
//    init();
    return true;
}





void  RTDataCSVWriter::processData(const DoubleHash& data, bool passedFilter)
{
    //Logger::logf(LOG_MSG,_T("RTDataCSVWriter::processData() %f"), (*data)[_T("time")] );
    wxString lines = CSVConverter::getLine(*m_entries, data);
    m_file->Write(lines +_T("\n") );
    #if defined (__WINCE_STANDARDSDK__)
        m_file->Flush();
    #endif

}
