#include "RTDataInputGPS.h"

#include "../CommunicationBase.h"
#include "../SerialException.h"
#include "../DirHelper.h"
#include "../GPSData.h"
#include "../Math.h"
#include "../logger.h"
#include "../vemsprotocol/GpsProtocol.h"

RTDataInputGPS::RTDataInputGPS(CommunicationBase* commBase, GpsProtocol* pb)
{
    //ctor
    m_commBase = commBase;
    m_gps = pb;
    m_state         = GPSInfo::NOT_CONNECTED;

    m_fileOutputStream  = NULL;
    m_gpsDataBuffer = new char[ 2*256 ];
    m_gpsDataIdx    = 0;
    m_recording     = false;
    m_isLog         = false;

}

RTDataInputGPS::~RTDataInputGPS()
{
    //dtor
    Logger::logf( LOG_MSG, _T("RTDataInputGPS::RTDataInputGPS() DTOR") );
    delete[] m_gpsDataBuffer;
    delete m_fileOutputStream;
}

int RTDataInputGPS::updateInputModel()
{
    m_state = getGPSData();
    if(m_state == GPSInfo::DATA_RECEIVED){
        m_state = processGPSData(); /// Set m_state to connected when valid record found
    }
    return m_state;
}

GPSInfo::State RTDataInputGPS::getGPSData()
{
  //Logger::logf( LOG_MSG, _T("RTDataInputGPS::getGPSData() - START") );
  //  waitRingBufferNotFull();

  //  char* m_buff = m_ringBuffer->getHeadPointer(); //not increment the ringbuffer head index

    int result = -1;

    try{
        result = m_commBase->ReadString( &m_buff, 1 );
    }catch( SerialException& se ){
        //Logger::logf( LOG_MSG, _T("RTDataInputGPS::getGPSData() - SerialException : %s"), se.toString().c_str() );
        return GPSInfo::NOT_CONNECTED;
    }catch (...){
        //Logger::logf( LOG_MSG, _T("RTDataInputGPS::getGPSData() - Unknown SerialException") );
    }
  //  Logger::logf( LOG_MSG, _T("RTDataInputGPS::getGPSData() - 1") );
    if( result <= 0 )
    {
        return GPSInfo::NO_GPS_DATA;
    }

/// Write to file!!
    //if( m_isLog)
    //{
/*
        if( !m_fileOutputStream )
        {
            wxString fileName = DirHelper::CreateNewFileName( _T("rawGPSlog2.txt"), DirHelper::GetPath(dLOG), true, true );
            m_fileOutputStream = new wxFileOutputStream( fileName );
        }

        m_gpsDataBuffer[ m_gpsDataIdx ] = m_buff;
        ++m_gpsDataIdx;

        if( m_gpsDataIdx == 256 ){
            m_fileOutputStream->Write( m_gpsDataBuffer, m_gpsDataIdx );
            m_gpsDataIdx=0;
        }

    //}
*/

    return GPSInfo::DATA_RECEIVED;
}

GPSInfo::State RTDataInputGPS::processGPSData()
{
    int status = m_gps->rx(&m_buff, 1);
    //Logger::logf( LOG_MSG, _T("RTDataInputGPS::processGPSData(): %c"), m_buff );
    switch( status )
    {
        case NMEAParse::GPS_FORMAT_GPRMC:
        case NMEAParse::GPS_FORMAT_GPGGA:
              //m_rawLogger->captureEcuBlock();
              break;
        case END :
            //Logger::logf( LOG_MSG, _T("RTDataInputGPS::processGPSData() END") );
            //packData();
            //m_gpsData->set( m_packedGpsData );

            return GPSInfo::CONNECTED;
            break;
    }
    return m_state;
}

void RTDataInputGPS::enableLog( bool enable )
{
    Logger::logf( LOG_MSG, _T("RTDataInputGPS::enableLog()") );
    if( m_fileOutputStream ){
    Logger::logf( LOG_MSG, _T("RTDataInputGPS::enableLog() delete") );
        //delete m_fileOutputStream;
        //m_fileOutputStream = NULL;
    }
    m_isLog = enable;
}
