#include "RTDataInputRawLog.h"

#include "../Utils.h"
#include "../logger.h"
#include "../RawLogHelper.h"
#include "../SymbolMgr.h"


RTDataInputRawLog::RTDataInputRawLog( SymbolMgr* mt, DoubleHash* viewerModel )
    :RTDataRawInputBase( mt, viewerModel )
    ,m_blockIdx (0)
    ,m_isValid (false)
{
    m_dataBuffer    = NULL;
    m_gpsDataLength = 0;
    m_gpsBlockIdx   = 0;
}

RTDataInputRawLog::~RTDataInputRawLog()
{
    m_rawFile.Close();
    delete[] m_dataBuffer;
}

void RTDataInputRawLog::reset()
{

    //Logger::logf(LOG_MSG,_T("RTDataInputRawLog::reset()"));

    m_rawFile.Seek(HEADERBLOCKSIZE);

    m_blockIdx = 0;

}

/** @brief getType
  *
  * @todo: document this function
  */
RTDataInputType RTDataInputRawLog::getType()
{
    return INPUT_TYPE_RAWFILE;
}

/** @brief refresh
  *
  * @todo: document this function
  */
int RTDataInputRawLog::updateViewerModel()
{
    //Logger::logf(LOG_MSG,_T("RTDataInputRawLog::updateViewerModel()"));
    fillSymbolValues();
    return 0;
}


int RTDataInputRawLog::updateInputModel()
{
    if( !m_isValid ) return -1;

    if( m_blocksize - m_blockIdx <= ( m_datalen + m_gpsDataLength ) )
    {
        int res = m_rawFile.Read(m_blockBuffer,m_blocksize);
        if(res != m_blocksize) return -2;
        m_blockIdx = 0;
    }

    memcpy( m_dataBuffer, &(m_blockBuffer[ m_blockIdx ]), m_datalen );

    if( m_gpsDataLength > 0 )
    {
        memcpy( &(m_dataBuffer[ m_datalen ]), &(m_blockBuffer[ m_gpsBlockIdx ]), m_gpsDataLength );
    }

    setRTDataBuffer( m_dataBuffer, m_datalen + m_gpsDataLength);

    //setRTDataBuffer( &(m_blockBuffer[m_blockIdx]) );

    m_blockIdx += m_datalen;

    //Logger::logf(LOG_MSG,_T("RTDataInputRawLog::updateInputModel() m_blockIdx = %i"),m_blockIdx);
    return 0;
}

/** @brief isExist
  *
  * @todo: document this function
  */
bool RTDataInputRawLog::isExist(const wxString& symbolname)
{
    return true;
}

/** @brief isValid
  *
  * @todo: document this function
  */
bool RTDataInputRawLog::isValid()
{
    return m_isValid;
}


bool RTDataInputRawLog::SetInputFile(const wxString &filename)
{
    m_fileName = filename;
    reset();
    return true;
}


int RTDataInputRawLog::ParseHeader()
{
    m_headerHash.clear();

    if( !m_rawFile.Open(m_fileName,wxFile::read) )
        return -1;

    if( m_rawFile.Read(m_headerBuffer,HEADERBLOCKSIZE) != HEADERBLOCKSIZE )
        return -1;

    wxString tmp = wxString::FromAscii(m_headerBuffer);
    wxArrayString tmparray;

    Utils::tokenizeToArray( tmp, _T("\n"), tmparray);

    for(uint16_t a=0; a<tmparray.GetCount();a++) {
        //Logger::logf(LOG_MSG,tmparray.Item(a));
        if( tmparray.Item(a).StartsWith(_T("#"))) continue;
        wxArrayString tmpitemarray;
        Utils::tokenizeToArray( tmparray.Item(a), _T("="), tmpitemarray);
        m_headerHash[tmpitemarray.Item(0)] = tmpitemarray.Item(1);
    }

    if( m_headerHash.find(RawLogHelper::blockSize) == m_headerHash.end() ) return -10;
    if( m_headerHash.find(RawLogHelper::dataLen) == m_headerHash.end() ) return -11;

    m_headerHash[RawLogHelper::blockSize].ToLong(&m_blocksize);
    m_headerHash[RawLogHelper::dataLen].ToLong(&m_datalen);
    m_headerHash[ RawLogHelper::gpsDataLen ].ToLong( &m_gpsDataLength );

    if(m_datalen < 1.0) return -12;
    m_RTDataPerBlock =  (m_blocksize - m_gpsDataLength)/m_datalen;
    Logger::logf(LOG_MSG,_T("RTDataInputRawLog::ParseHeader() RTDataPerBlock: %i"), m_RTDataPerBlock);

    m_blockBuffer = new char[m_blocksize];
    m_rawFile.Read(m_blockBuffer,m_blocksize);

    m_isValid = true;

    m_dataBuffer = new uint8_t[ m_blocksize ];


    int remains = m_blocksize % m_datalen;

    if( remains < m_gpsDataLength )
    {
        m_gpsBlockIdx = m_blocksize - m_datalen - remains;
    }
    else
    {
        m_gpsBlockIdx = m_blocksize - remains;
    }

    m_symbolMgr->setGroupOffset( _T("GPSData"), m_datalen );

    return 0;

}


/** @brief GetFileSize
  *
  * @todo: document this function
  */
unsigned int RTDataInputRawLog::GetFileSize()
{
    return m_rawFile.Length();
}

/** @brief GetFilePos
  *
  * @todo: document this function
  */
unsigned int RTDataInputRawLog::GetFilePos()
{
    return m_rawFile.Tell();
}

long RTDataInputRawLog::getRTDataPerBlock()
{
    return m_RTDataPerBlock;
}

wxString RTDataInputRawLog::getHeaderValue(const wxString& key) const
{
    StringHash::const_iterator IT = m_headerHash.find(key);
    if(IT == m_headerHash.end() ) return wxEmptyString;
    return IT->second;
}
