#include "RTDataInputRingBufferRawFile.h"

#include "../Utils.h"
#include "../logger.h"
#include "../RawLogHelper.h"
#include "../SymbolMgr.h"

#define BUFFER_SIZE 256

RTDataInputRingBufferRawFile::RTDataInputRingBufferRawFile(RingBuffer * rb, SymbolMgr * mt, SyncObj* syncObj, DoubleHash* viewerModel )
: RTDataInputRingBufferSync(rb, mt,  syncObj, viewerModel )
{
    m_blockIdx = 0;
    m_isValid = false;
    m_datalen = 0;
    m_gpsDataLength = 0;
    m_gpsBlockIdx   = 0;
    m_dataBuffer    = new uint8_t[BUFFER_SIZE];

    m_seekIdx = 0;
    isSettedSeek = false;
}

RTDataInputRingBufferRawFile::~RTDataInputRingBufferRawFile()
{
    //dtor
    m_rawFile.Close();

}

void RTDataInputRingBufferRawFile::reset()
{

    //Logger::logf(LOG_MSG,_T("RTDataInputRawLog::reset()"));

    m_rawFile.Seek(HEADERBLOCKSIZE);

    m_blockIdx = 0;

}

/** @brief getType
  *
  * @todo: document this function
  */
RTDataInputType RTDataInputRingBufferRawFile::getType()
{
    return INPUT_TYPE_RINGBUFFER_RAWFILE;
}


int RTDataInputRingBufferRawFile::updateInputModel()
{
    waitRingBufferNotFull();

    if( !m_isValid ) return -1;

    if(m_blocksize - m_blockIdx <= (m_datalen + m_gpsDataLength) ){

        if(isSettedSeek) {
            m_rawFile.Seek( m_seekIdx * m_blocksize, wxFromStart);
            isSettedSeek = false;
        }

        int res = m_rawFile.Read(m_blockBuffer,m_blocksize);
        if(res != m_blocksize){
           reset();
            return -2;
        }
        m_blockIdx = 0;
    }

    memcpy( m_dataBuffer, &(m_blockBuffer[ m_blockIdx ]), m_datalen );

    if( m_gpsDataLength > 0 )
    {
        memcpy( &(m_dataBuffer[ m_datalen ]), &(m_blockBuffer[ m_gpsBlockIdx ]), m_gpsDataLength );
    }

    setRTDataBuffer( m_dataBuffer, BUFFER_SIZE );

    //setRTDataBuffer(&(m_blockBuffer[m_blockIdx]));

    char* m_RTBuff = m_ringBuffer->getHeadPointer();
    //memcpy(m_RTBuff, m_rtDataBuffer, m_datalen);
    memcpy(m_RTBuff, m_dataBuffer, m_datalen + m_gpsDataLength );

    m_blockIdx += m_datalen;

    //Logger::logf(LOG_MSG,_T("RTDataInputRawLog::updateInputModel() m_blockIdx = %i"),m_blockIdx);

    if( isActive() && !m_ringBuffer->isFull() ){
        m_ringBuffer->nextHead();
        m_isNotEmpty->Broadcast();
       // Logger::logf(LOG_MSG,_T("RTDataInputRawLog::updateInputModel() nextHead %p"), m_isNotEmpty);
    }

    if( m_prevFilePos != m_rawFile.Tell())
        this->observerNotify(m_rawFile.Tell(), m_rawFile.Length());

    m_prevFilePos = m_rawFile.Tell();

    return 0;
}

/** @brief isExist
  *
  * @todo: document this function
  */
bool RTDataInputRingBufferRawFile::isExist(const wxString& symbolname)
{
    return true;
}

/** @brief isValid
  *
  * @todo: document this function
  */
bool RTDataInputRingBufferRawFile::isValid()
{
    return m_isValid;
}


bool RTDataInputRingBufferRawFile::SetInputFile(const wxString &filename)
{
    m_fileName = filename;

    return true;
}


int RTDataInputRingBufferRawFile::ParseHeader()
{
    m_headerHash.clear();

    if( !m_rawFile.Open(m_fileName,wxFile::read) )
        return -1;

    if( m_rawFile.Read(m_headerBuffer,HEADERBLOCKSIZE) != HEADERBLOCKSIZE )
        return -1;

    wxString tmp = wxString::FromAscii(m_headerBuffer);
    wxArrayString tmparray;

    Utils::tokenizeToArray( tmp, _T("\n"), tmparray);

    for(uint16_t a=0; a<tmparray.GetCount();a++) {
        //Logger::logf(LOG_MSG,tmparray.Item(a));
        if( tmparray.Item(a).StartsWith(_T("#"))) continue;
        wxArrayString tmpitemarray;
        Utils::tokenizeToArray( tmparray.Item(a), _T("="), tmpitemarray);
        m_headerHash[tmpitemarray.Item(0)] = tmpitemarray.Item(1);
    }

    if( m_headerHash.find(RawLogHelper::blockSize) == m_headerHash.end() ) return -10;
    if( m_headerHash.find(RawLogHelper::dataLen) == m_headerHash.end() ) return -11;

    m_headerHash[RawLogHelper::blockSize].ToLong(&m_blocksize);
    m_headerHash[RawLogHelper::dataLen].ToLong(&m_datalen);

    if( m_headerHash.find(RawLogHelper::gpsDataLen) == m_headerHash.end() ){
        m_gpsDataLength = 0;
    }else{
        m_headerHash[RawLogHelper::gpsDataLen].ToLong(&m_gpsDataLength);
    }

    m_blockBuffer = new char[m_blocksize];
    m_rawFile.Read(m_blockBuffer,m_blocksize);

    m_isValid = true;

    int remains = m_blocksize % m_datalen;

    if( remains < m_gpsDataLength )
    {
        m_gpsBlockIdx = m_blocksize - m_datalen - remains;
    }
    else
    {
        m_gpsBlockIdx = m_blocksize - remains;
    }

    m_symbolMgr->setGroupOffset( _T("GPSData"), m_datalen );

    this->observerNotify(1);

    return 0;

}


/** @brief GetFileSize
  *
  * @todo: document this function
  */
unsigned int RTDataInputRingBufferRawFile::GetFileSize()
{
    return m_rawFile.Length();
}

/** @brief GetFilePos
  *
  * @todo: document this function
  */
unsigned int RTDataInputRingBufferRawFile::GetFilePos()
{
    return m_rawFile.Tell();
}

void RTDataInputRingBufferRawFile::SetFilePos(double percent)
{
    uint16_t fullblock = m_rawFile.Length() / m_blocksize;

    m_seekIdx = (int)(fullblock * percent);

    isSettedSeek = true;

}

wxString RTDataInputRingBufferRawFile::getHeaderValue(const wxString& key) const
{
     StringHash::const_iterator IT = m_headerHash.find(key);
    if(IT == m_headerHash.end() ) return wxEmptyString;
    return IT->second;
}

