#include "RTSymbolViewer.h"
#include "../SymbolMgr.h"
#include "../Symbol.h"
#include "../logger.h"


RTSymbolViewer::RTSymbolViewer( const wxString& symbolName, SymbolMgr* symbolManager )
{
    m_symbolManager = symbolManager;
    m_symbolName = symbolName;
    m_isSymbolValid = isValid();
    m_isSymbolRegistered = false;
    m_symbolValue = 0.0;
    m_isSymbolUpdated = false;
}


RTSymbolViewer::~RTSymbolViewer()
{
}

void RTSymbolViewer::require()
{
    if( !m_isSymbolRegistered && m_isSymbolValid )
    {
        m_rtDataMgr->registerSymbol( m_symbolName );
        m_isSymbolRegistered = true;
    } else if ( m_isSymbolRegistered && !m_isSymbolValid ) {
        m_rtDataMgr->unregisterSymbol( m_symbolName );
        m_isSymbolRegistered = false;
    }
}

void RTSymbolViewer::refresh( DoubleHash* table )
{
    if ( !m_isSymbolRegistered || !m_isSymbolValid ) return;

    DoubleHash::const_iterator it = table->find( m_symbolName );

    if( it == table->end() ) { return; }

    m_symbolValue = it->second;

    m_isSymbolUpdated = true;

}

void RTSymbolViewer::unregisterSymbols()
{
    if( m_isSymbolRegistered )
    {
        m_rtDataMgr->unregisterSymbol( m_symbolName );
        m_isSymbolRegistered = false;
    }
}

void RTSymbolViewer::configUpdated()
{
    LOGP( wxString::Format( _T("m_isSymbolValid: %d"), m_isSymbolValid ) );

    if ( m_isSymbolValid == isSymbolValid() ) {
        return;
    }

    m_isSymbolValid = isSymbolValid();
    m_isSymbolUpdated = false;
    requireChanged();
    LOGP( wxString::Format( _T("m_isSymbolValid: %d"), m_isSymbolValid ) );
}

bool RTSymbolViewer::isSymbolValid()
{
    Symbol* sym = m_symbolManager->getSymbol( m_symbolName );

    if ( !sym ) {
        LOGP( _T("Symbol ") + m_symbolName + _T(" doesn't exist!") );
        return false;
    } else {
        if ( sym->isConst() ) {
            LOGP( _T("Symbol ") + m_symbolName + _T(" isn't realtime!") );
            return false;
        } else {
            return true;
        }
    }
}
