#include "AccSetMatrix.h"

#include <boost/accumulators/accumulators.hpp>
#include <boost/accumulators/statistics/stats.hpp>
#include <boost/accumulators/statistics/mean.hpp>
#include <boost/accumulators/statistics/max.hpp>
#include <boost/accumulators/statistics/min.hpp>
#include <boost/accumulators/statistics/count.hpp>
#include <boost/accumulators/statistics/variance.hpp>
#include <boost/accumulators/statistics/density.hpp>

using namespace boost::accumulators;

/// SimpleIntMatrix

SimpleIntMatrix::SimpleIntMatrix()
{
    m_rows = 0; m_cols = 0; m_data = NULL;
}

SimpleIntMatrix::SimpleIntMatrix(size_t rows, size_t cols)
{
    setDims( rows, cols );
}

SimpleIntMatrix::~SimpleIntMatrix()
{
    if ( m_data ) {
        delete m_data; m_data = NULL;
    }
}

void SimpleIntMatrix::setDims(size_t rows, size_t cols)
{
    m_rows = rows; m_cols = cols; m_data = NULL;
    if ( m_rows && m_cols )
    {
        m_data = new int[m_cols*m_rows];
        for ( size_t i = 0; i < m_cols*m_rows; ++i) m_data[i] = 0;
    }
}

void SimpleIntMatrix::setData(size_t index, int value)
{
    if ( m_data ) m_data[index] = value;
}

int& SimpleIntMatrix::operator[] (size_t index) throw(int)
{
    if ( m_data ) return m_data[ index ];
    throw -1;
}

int SimpleIntMatrix::sumOfElements()
{
    int sum = 0;
    if ( m_data )
        for ( size_t i = 0; i < m_cols *m_rows; ++i ) sum+=m_data[i];
    return sum;
}

void SimpleIntMatrix::clear()
{
    if ( m_data )
        for ( size_t i = 0; i < m_cols *m_rows; ++i ) m_data[i] = 0;
}


/// AccSetMatrix

AccSetMatrix::AccSetMatrix(size_t rows, size_t cols)
{
    m_rows = rows; m_cols = cols;
    m_data = NULL;

    if ( m_rows && m_cols )
    {
        m_data = new AccSetT*[m_rows*m_cols];
        for ( size_t i = 0; i < m_cols * m_rows; ++i )
            m_data[i] = new AccSetT(tag::density::num_bins = 30, tag::density::cache_size = 50);
    }
}

AccSetMatrix::~AccSetMatrix()
{
    if ( m_data != NULL )
    {
        for ( size_t i = 0; i < m_cols * m_rows; ++i )
            delete m_data[i];
        delete [] m_data;
    }
}

AccSetT* AccSetMatrix::getData(size_t row, size_t col)
{
    if ( m_data == NULL ) return NULL;
    else return m_data[ row * m_cols + col];
}

void AccSetMatrix::reinit_cell( size_t row, size_t col )
{
    if ( m_data == NULL || ( row >= m_rows ) || ( col >= m_cols ) )
    {
        return;
    }

    size_t idx = row * m_cols + col;
    delete m_data[idx];

    m_data[idx] = new AccSetT(tag::density::num_bins = 30, tag::density::cache_size = 50);
}

AccSetT* AccSetMatrix::getData (size_t index)
{
    if ( m_data == NULL ) return NULL;
    else return m_data[ index ];
}

AccSetT* AccSetMatrix::operator[] (size_t index)
{
    if ( m_data == NULL ) return NULL;
    else return m_data[ index ];
}

/// end of AccSetMatrix
