#include "RTCellInfo.h"

#include <wx/dcbuffer.h>
#include <wx/settings.h>

#include "../logger.h"
#include "RTDataStatMath.h"

BEGIN_EVENT_TABLE( RTCellInfo, wxPanel )
    EVT_PAINT ( RTCellInfo::OnPaint )
    EVT_RIGHT_DOWN ( RTCellInfo::OnRightClick )
END_EVENT_TABLE()

extern wxString strNA;

RTCellInfo::RTCellInfo(wxWindow* parent, wxWindowID id, wxPoint pos, wxSize size)
    : wxPanel(parent, id, pos, size)
{
    m_parent = parent;
    m_checked = false;
    m_count = 0;
    m_filteredOut = 0;
    m_min = 0;
    m_max = 0;
    m_mean = 0;
    m_variance = 0;

    m_unf_count = 0;
    m_unf_min = 0;
    m_unf_max = 0;
    m_unf_mean = 0;
    m_unf_variance = 0;

    m_cellbgcolor = wxColor(255,255,255);
    m_celltextcolor = wxColor( 0, 0, 0 );
    m_size = size;
    SetToolTip(_T("Top-left: min. value in the cell\nTop-right: max. value in the cell\nCenter: mean\n")
        _T("Bottom-left: # of values (# of filtered out values)\nBottom-right: variance\n\nRIGHT CLICK: Display histogram") );

    m_distr_win = new RTDataStatDistributionWin( m_parent, wxDefaultPosition, wxSize(400,400) );
}

RTCellInfo::~RTCellInfo()
{
}

void RTCellInfo::setCellInfo(int count, int filteredOut, int outliers, double minval, double maxval, double ve_mean, double vecorr_mean, double variance)
{
    m_count = count;
    m_filteredOut = filteredOut;
    m_min = minval;
    m_max = maxval;
    m_mean = ve_mean;
    m_mean2 = vecorr_mean;
    m_variance = variance;
    m_outliers = outliers;
    Refresh();
}

void RTCellInfo::setCellInfo(int count, int filteredOut, int outliers, double minval, double maxval, double mean, double variance)
{
    m_count = count;
    m_filteredOut = filteredOut;
    m_min = minval;
    m_max = maxval;
    m_mean = mean;
    m_mean2 = 0;
    m_variance = variance;
    m_outliers = outliers;
    Refresh();
}

void RTCellInfo::setUnfilteredInfo( int count, double minval, double maxval, double mean, double sd )
{
    m_unf_count = count;
    m_unf_min = minval;
    m_unf_max = maxval;
    m_unf_mean = mean;
    m_unf_variance = sd;
    Refresh();
}

void RTCellInfo::setCellBgColor( wxColor color )
{
    m_cellbgcolor = color;
    Refresh();
}

void RTCellInfo::setCellTextColor( wxColor color )
{
    m_celltextcolor = color;
    Refresh();
}

void RTCellInfo::OnPaint( wxPaintEvent& evt )
{
    wxBufferedPaintDC dc(this);
    dc.SetBackground( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ));
    dc.Clear();

    // N/A?
    dc.SetPen( wxColor( 160, 160, 160 ));

    if ( m_count == 0 )
    {
        dc.SetBrush( *wxWHITE );
        dc.DrawRectangle(0,0, m_size.GetWidth(), m_size.GetHeight() );
        wxFont font(48, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD );
        dc.SetFont(font);
        dc.SetTextForeground(wxColor(160,160,160));
        wxSize s = dc.GetTextExtent( strNA );
        dc.DrawText( strNA, (m_size.x-s.x)/2, (m_size.y-s.y)/2);
        return;
    }
    else
    {
        if ( m_checked )
        {
            dc.SetBrush ( wxColor( 0, 128, 0) );
            dc.DrawRectangle(0,0, m_size.GetWidth(), m_size.GetHeight() );

            wxFont font(36, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD );
            dc.SetFont(font);
            // dc.SetTextForeground(*wxBLACK);
            dc.SetTextForeground( m_celltextcolor );

            wxString outstr = _T("Fixed");
            wxSize s = dc.GetTextExtent( outstr );
            dc.DrawText( outstr, (m_size.x - s.x) / 2, (m_size.y - s.y) / 2 );
            return;
        }

        dc.SetBrush( m_cellbgcolor );
        dc.DrawRectangle(0,0, m_size.GetWidth(), m_size.GetHeight() );

        wxFont font(12, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD );
        wxFont smallfont(8, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL );
        dc.SetFont(font);
        // dc.SetTextForeground(*wxBLACK);
        dc.SetTextForeground( m_celltextcolor );
        wxString outstr;
        // min
        outstr = wxString::Format(_T("%.2f"), m_min );
        dc.DrawText(outstr, 5, 5 );
        dc.SetFont(smallfont); dc.DrawText(_T("min"),5, 18); dc.SetFont(font);
        // max
        outstr = wxString::Format(_T("%.2f"), m_max );
        wxSize s = dc.GetTextExtent( outstr );
        dc.DrawText( outstr, m_size.x - 5 - s.x, 5);
        dc.SetFont(smallfont); dc.DrawText(_T("max"), m_size.x-24, 18); dc.SetFont(font);
        // count
        outstr = wxString::Format(_T("%d (%d / %d)"), m_count, m_filteredOut, m_outliers);
        s = dc.GetTextExtent( outstr );
        dc.DrawText( outstr, 5, m_size.y - s.y - 5);
        dc.SetFont(smallfont); dc.DrawText(_T("# (dropped / outliers)"), 5, m_size.y - s.y - 16); dc.SetFont(font);
        // variance
        outstr = wxString::Format(_T("%.2f"), m_variance);
        s = dc.GetTextExtent( outstr );
        dc.DrawText( outstr, m_size.x - 5 - s.x, m_size.y - s.y - 5);
        dc.SetFont(smallfont); dc.DrawText(_T("var"), m_size.x-24, m_size.y - s.y - 16); dc.SetFont(font);

        // mean
        font.SetPointSize(32);
        dc.SetFont(font);

        if ( m_mean2 != 0 )
        {
            outstr = wxString::Format(_T("%.0f (%.0f)"), m_mean, m_mean2 );
        }
        else
        {
            outstr = wxString::Format(_T("%.2f"), m_mean);
        }
        s = dc.GetTextExtent( outstr );
        dc.DrawText( outstr, (m_size.x-s.x)/2, (m_size.y-s.y)/2);
    }
    evt.Skip();
}

void RTCellInfo::OnRightClick( wxMouseEvent& event )
{
    if ( m_unf_count == 0 )
    {
        event.Skip();
        return;
    }

    showHistogramWindow();
}

void RTCellInfo::setHistogram( double minval, double maxval, const DoubleArray& data )
{
    if ( m_distr_win )
        m_distr_win->SetHistogram( minval, maxval, data );
}

void RTCellInfo::drawGraphs()
{
    if ( m_distr_win == NULL )
        return;
    double mmin = ( m_min < m_unf_min ) ? m_min : m_unf_min;
    double mmax = ( m_max > m_unf_max ) ? m_max : m_unf_max;

    double* filt_data = NULL, *unf_data = NULL;

    double _mmax1 = RTDataStatMath::NormalDistribution( m_mean, m_variance, mmin, mmax, 300, &filt_data );
    double _mmax2 = RTDataStatMath::NormalDistribution( m_unf_mean, m_unf_variance, mmin, mmax, 300, &unf_data );

    if ( _mmax1 == 0 && _mmax2 == 0 )
    {
        m_distr_win->ClearCurves();
        m_distr_win->DrawEmpty();
        return;
    }

    // double ymax = ( _mmax1 > _mmax2 ) ? _mmax1 : _mmax2;

    double step = ( mmax - mmin ) / 300.0;
    double start = mmin;

    RealPointArray carr1, carr2;

    for ( int j = 0; j < 300; ++j )
    {
        if ( filt_data != NULL )
        {
            wxRealPoint p1( start, filt_data[j]);
            carr1.Add( p1 );
        }
        else carr1.Add( wxRealPoint(0,0) );

        if ( filt_data != NULL )
        {
            wxRealPoint p2( start, unf_data[j]);
            carr2.Add( p2 );
        }
         else carr2.Add( wxRealPoint(0,0) );

        start = start + step;
    }

    m_distr_win->ClearCurves();

    if ( filt_data != NULL )
    {
        m_distr_win->AddCurve( carr1, _T("With filtering"), wxColor( 0, 102, 153 ) );
        delete[] filt_data;
    }

    if ( unf_data != NULL )
    {
        m_distr_win->AddCurve( carr2, _T("Without filtering"), wxColor( 255, 0, 0, 255 ) );
        delete[] unf_data;
    }

    m_distr_win->SetMinMax( mmin, mmax );

    m_distr_win->DrawGraphs();
}

void RTCellInfo::showHistogramWindow()
{
    if ( m_distr_win == NULL )
        return;

    if ( m_distr_win->IsShown() == false || m_distr_win->IsShownOnScreen() == false )
    {
        m_distr_win->Show();
    }
}
