#ifndef RTDATAAQUISITION_H
#define RTDATAAQUISITION_H

#include <wx/wx.h>
#include <wx/grid.h>

#include "../common/types/DoubleHash.h"
#include "../common/types/DoubleArray.h"
#include "../common/VemsMatrix.h"
#include "../rtdatamanager/RTDataViewerBase.h"
#include "../ObserverBase.h"
#include "../Symbol.h"
#include "../SymbolMgr.h"

class VemsTuneController;
class LogHistoryBase;
class VemsTableDesc;
class STDataMgrBase;
class RTDataStat;

class RTDataAquisition : public wxFrame, public RTDataViewerBase, public ObserverBase
{
    public:
        RTDataAquisition( wxWindow* parent, VemsTuneController* vtc, LogHistoryBase* loghistory, VemsTableDesc* tableDesc );
        virtual ~RTDataAquisition();

        void    refresh( DoubleHash* table );
        void    require();
        void    unregisterSymbols();

        void    SetValueIdx( int idx ) { }
        void    SetValue( double value ) { }

        void    notify( int i, void* ptr );

    protected:

    private:

        bool                    m_isstarted;

        wxWindow*               m_parent;

        VemsTuneController*     m_vtc;
        LogHistoryBase*         m_loghistory;
        VemsTableDesc*          m_tableDesc;

        STDataMgrBase*          m_stDataMgr;

        Symbol*                 m_xBinSymbol;
        Symbol*                 m_yBinSymbol;
        Symbol*                 m_xRTSymbol;
        Symbol*                 m_yRTSymbol;

        DoubleArray             m_HorizLimits, m_VertLimits;

        SymbolMgr*              m_symbolMgr;

        wxButton*               m_startButton;
        wxButton*               m_stopButton;
        wxButton*               m_clearButton;
        wxButton*               m_analyzeButton;
        wxTextCtrl*             m_minDataTxt;
        wxGrid*                 m_table;

        RTDataStat*             m_statFrame;
        bool                    m_tableClosed;

        long                    m_minData;

        unsigned int            m_startIndex;
        unsigned int            m_endIndex;

        unsigned int            m_cols, m_rows;

        wxArrayString           m_registeredSymbolNames;
        wxArrayString           m_xbins, m_ybins;
        bool                    m_symreg;

        int                     m_xmin, m_xmax, m_ymin, m_ymax;

        VemsMatrix<unsigned>*   m_matrix;

        void        buildGUI( );
        void        createGrid();
        void        calcLimits();

        void        OnStartStop( wxCommandEvent& event );

        void        OnAnalyzeButton( wxCommandEvent& event );
        void        OnClearButton( wxCommandEvent& event );

        void        SendLogMarker( bool start );

        void        updateTable( int xSym, int ySym );

        DECLARE_EVENT_TABLE()
};

#endif // RTDATAAQUISITION_H
