#ifndef RTDATASTATDISTRIBUTIONWIN_H
#define RTDATASTATDISTRIBUTIONWIN_H

#include <wx/wx.h>
#include <wx/dynarray.h>

#include "../common/types/DoubleArray.h"
#include "../common/types/RealPointArray.h"
#include "../common/types/IntArray.h"
#include "../common/types/wxColourArray.h"

WX_DECLARE_OBJARRAY( RealPointArray, RealPointArrayA );
WX_DEFINE_ARRAY_LONG( unsigned long, ULongArray );

class RTPlotter;

class RTDataStatDistributionWin : public wxFrame
{
    public:
        RTDataStatDistributionWin( wxWindow *parent, const wxPoint& pos, const wxSize& size );
        ~RTDataStatDistributionWin();

        void SetMinMax( double minV, double maxV ) { m_minV = minV; m_maxV = maxV; }
        void SetYAxisMax( double ymax );

		void AddCurve( RealPointArray curve, const wxString & curvename, wxColour color = wxColour(_T("RED")) );
        void ClearCurves();

        void DrawEmpty();

        void SetHistogram( double minval, double maxval, const DoubleArray& data = DoubleArray() );

        void DrawGraphs();

    protected:

    private:

        bool                m_hasData;
        unsigned            m_startCIdx;

        wxColourArr         m_curveColors;
        RealPointArrayA     m_curves;
        wxArrayString       m_curvenames;

        double              m_minV, m_maxV;
        double              m_ymax;

        double              m_histMin, m_histMax;
        DoubleArray         m_histData;
        unsigned            m_histCount;

        RTPlotter*          m_plotter;

        void OnPaint( wxPaintEvent& event );
        void OnMouseMove( wxMouseEvent& event );
        void ReDraw( wxSizeEvent& event );
        void OnClose( wxCloseEvent& event );

        DECLARE_EVENT_TABLE()
};

#endif // RTDATASTATDISTRIBUTIONWIN_H
