#ifndef RTDATASTATMATH_H
#define RTDATASTATMATH_H

#include <limits>
#include <wx/dc.h>

typedef double(*dblFunc)(double);
typedef double(*dbl3dFunc)(double, double);

WX_DECLARE_STRING_HASH_MAP( dblFunc, FunctionA );

#ifndef _MSC_VER
	template <bool> struct static_assert;
	template <> struct static_assert<true> { };
#endif

template<typename T>
inline bool is_NaN(T const& x) {
    static_cast<void>(sizeof(static_assert<std::numeric_limits<T>::has_quiet_NaN>));
    return std::numeric_limits<T>::has_quiet_NaN and (x != x);
}

template <typename T>
inline bool is_inf(T const& x) {
    static_cast<void>(sizeof(static_assert<std::numeric_limits<T>::has_infinity>));
    return x == std::numeric_limits<T>::infinity() or x == -std::numeric_limits<T>::infinity();
}


class DblHistogram
{
    public:
        DblHistogram();
        DblHistogram( const double& start, const double& end, const unsigned int& bins );
        DblHistogram( const DblHistogram& hist );
        ~DblHistogram();

        DblHistogram& operator = ( const DblHistogram& hist );

        void         setProps( const double& start, const double& end, const unsigned int& bins );
        void         getMinMax( double* hmin, double* hmax, double *hbins = NULL );

        void         addValue( double value );
        unsigned int getCount();

        const unsigned int* getFrequencies();

    private:
        double          m_start;
        double          m_end;
        double          m_nb_int;
        unsigned int    m_bins;
        unsigned int*   m_freq;
};

class RTDataStatMath
{
    public:
        static double NormalDistribution( double mean, double sigma, double minval, double maxval,
            int bins, double** data );

        static void RandomNumbersUniform( unsigned count, double minval, double maxval, double** data );
        static void RandomNumbersNormal( unsigned count, double mean, double sigma, double** data );

        static void writeToCSV( const wxString& filename, double minval, double maxval,
            int datasize, double* data );

    protected:

    private:
};

#endif // RTDATASTATMATH_H
