#ifndef RTDATASTATTABLE_H
#define RTDATASTATTABLE_H

#include <wx/dcbuffer.h>
#include <wx/wx.h>
#include "../Observable.h"
#include "AccSetMatrix.h"
#include "../stdatamanager/STDataViewerBase.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../common/types/DoubleArray.h"
#include "RTDataStatHelper.h"

class GaugeDescManager;

const int idSelectCell  = 21;
const int idMultipleSel = 31;
const int idFixCell     = 33;
const int idSelectFeat  = 42;


class RTDataStatTable : public wxPanel, public Observable, public STDataViewerBase
{
    public:
        RTDataStatTable(wxWindow* parent, int width, int height,  const wxArrayString& xbins, const wxArrayString& ybins, const wxString& reqsymbol, bool tuneMode = false);
        ~RTDataStatTable();

        void        setBgColor(wxColour color);

        void        selectStatisticalFeature(uint8_t feature);
        uint8_t     getStatisticalFeature() { return m_statisticalfeat; }
        void        resizeTo(int width, int height);
        void        setMatrix(AccSetMatrix* matrix);
        void        setSymbol(wxString symbolname) { m_symbolname = symbolname; }
        void        setMinCount(size_t min = 0, bool refresh = false);
        wxColor     GetCellColor(double value);

        void        setSelection( cell topLeft, cell bottomRight );
        void        forceNumericFormat( const wxString& format ) { m_forcedNumFormat = format; }

        void        setCustomColors(DoubleArray limits, DoubleArray colors);
        void        resetColorGradients();

        // tune
        void        checkCell( int row, int col );
        bool        isChecked( int row, int col );

        void        updateTable( const wxString& tableName, wxPoint cursorPosition );
        void        update() {}

        void        requestRead() {}
        void        updateModel() {}
        void        requestWrite() {}

    protected:
    private:
        wxString            m_forcedNumFormat;
        bool                m_returnfocus;
        bool                m_customcolors;
        bool                m_tuneMode;

        size_t              m_mincount;
        size_t              m_maxcount;
        uint8_t             m_cols;
        uint8_t             m_rows;
        size_t              m_width;
        size_t              m_height;
        wxArrayString       m_xbins, m_ybins;
        wxColor             m_bgColor;
        AccSetMatrix*       m_matrix;
        SimpleIntMatrix     m_FixedCells;
        uint8_t             m_statisticalfeat;

        int                 m_cellw, m_cellh;
        wxRect              m_cellsize;
        bool                m_calculated;
        double              m_tblmin, m_tblmax;
        GaugeDescManager*   m_descmgr;
        wxString            m_dStr;
        wxString            m_symbolname;
        wxString            m_numformat;
        GradientColorArray  m_gradientColorArray;

        wxMenu*             m_popupmenu;

        cell                m_highlightedCell;
        cell                m_selectedCell;

        // multiple cell selection
        bool                m_isSelecting;
        cell                m_startcell;
        cell                m_endcell;

        // colors
        wxColor             m_cellTextColor;
        wxColor             m_3dinnerColor;
        wxColor             m_3douterColor;
        wxColor             m_labelBkColor;

        DoubleArray         m_colorlimits;
        DoubleArray         m_customcolorsA;

        // methods

        void                Draw3DBorder(wxDC& dc, const wxRect& rc);
        void                Draw3DCells(wxDC& dc);
        void                DrawData(wxDC& dc);
        int                 CalcFontSize(wxDC& dc, wxRect rect, const wxString& str, wxFont font);
        int                 CalcCellFontSize(wxDC& dc);
        void                GetTblMinMax(wxDC& dc);
        wxRect              GetCellRect(int row, int col);

        void                SelectCell(wxDC& dc);
        wxColor             calcCustomColorCellVal(double value);

        wxColor             dbl2Color(double rgb);
        void                getSelectionCells(cell& topleft, cell& bottomright); // multiple cell selections
        bool                isInSelection(int row, int col);

        void                OnPaint( wxPaintEvent& event );
        void                OnSize( wxSizeEvent& event );
        void                OnRClick( wxMouseEvent& event );
        void                OnStatFeatSelect( wxCommandEvent& event );
        void                OnLeftButton( wxMouseEvent& event );
        void                OnKeyDown( wxKeyEvent& event );
        void                OnMenuFix( wxCommandEvent& event );
        void                OnMouse( wxMouseEvent& event );
        void                OnFocusEvent( wxFocusEvent& event );

        void                OnShowHistogram( wxCommandEvent& event );

        DECLARE_EVENT_TABLE();

};

#endif // RTDATASTATTABLE_H
