#include "RTPlotter.h"

#define _USE_MATH_DEFINES
#include <math.h>
#include <climits>

#include "../logger.h"
#include "../agg/wxAgg2.h"
#include "../DirHelper.h"
#include "../common/types/RealPointArray.h"
#include <wx/arrimpl.cpp>
#include <wx/dcbuffer.h>
#include "RTDataStatMath.h"
#include "../triggerlog/TriggerLogSettingsHelper.h"
#include "../common/types/Int2IntHash.h"

template<class T>
T min(const T& a, const T& b) { return (( a > b ) ? b : a); }
template<class T>
T max(const T& a, const T& b) { return (( a > b ) ? a : b); }


///
/// RTPlotter EXAMPLE - NOT A FULL, WORKING CODE!
/*

class RTPlotterTestFrame : public wxFrame {
public:
    RTPlotterTestFrame( wxWindow * parent );
private:
    RTPlotter* plotter;
    double hist[123];
    wxColourArr colors;

    void OnPaint(..);
    void OnMouse(..);
    void OnSize(..);

    DECLARE_EVENT_TABLE();
};

BEGIN_EVENT_TABLE( RTPlotterTestFrame, wxFrame )
    EVT_PAINT( RTPlotterTestFrame::OnPaint )
    EVT_SIZE( RTPlotterTestFrame::OnSize )
    EVT_MOTION( RTPlotterTestFrame::OnMouse )
END_EVENT_TABLE()

// create sample functions: y = f(x)
double deg2rad( double x ){ return x * M_PI / 180.0 ;}
double sin_x( double x ) { return sin(deg2rad( x )); }
double cos_x( double x ) { return cos(deg2rad( x )); }

// create parametric function (x(t),y(t))
double bflyx( double t ) { return (sin(t)*( pow(M_E, cos(t)) - 2*cos(4*t)-pow(sin(t/12),5)));}
double bflyy( double t ) { return (cos(t)*( pow(M_E, cos(t)) - 2*cos(4*t)-pow(sin(t/12),5)));}

// create polar function r = f(theta)
double polarfunc( double t ) { return 1 + cos(t); }

// create 3D function z = f(x,y)
double func3d( double x, double y ) { return sin(x); }

RTPlotterTestFrame::RTPlotterTestFrame(wxWindow* parent)
{
    // create frame
    Create(...)
    plotter = new RTPlotter( NULL, this ); // set DC in Onpaint, this frame will handle mouse events

    // fill hist[j]
    for ( unsigned char j = 0; j < 123; ++j )
        hist[j] = rand() / RAND_MAX * 50; // between 0 and 50

    // pie chart colors
    wxColour col = wxColor( 34,67,231); colors.Add(col);
    col = wxColor( 123, 47, 89 ); colors.Add(col);
    col = wxColor( 89,203, 111); colors.Add(col);
    col = wxColor( 145, 34,124); colors.Add(col);
}

void RTPlotterTestFrame::OnPaint( wxPaintEvent& event )
{
    wxBufferedPaintDC dc(this);
    dc.Clear();

    plotter->SetDC( &dc ); // !!important

    int w, h; GetSize( &w, &h );

    // set the size of the plot & create the canvas
    plotter->SetViewPortSize( w, h );
    plotter->CreateCanvas();

    //
    // -> draw the histogram: 123 data, xmin = 0, xmax = 50, y-axis shows the frequencies
    //
    plotter->DrawHistogram( 123, hist, 0, 50, false );

    //
    // or draw functions
    //
    plotter->AddFunction( _T("sinx"), sin_x );
    plotter->AddFunction( _T("cosx"), cos_x );
    plotter->SetViewPortLimits( -180, 180, -1, 1); // limits of sin&cos is -1 / 1
    // draw the function sin(x) with red color and fill it
    plotter->AddCurve( _T("sinx"), -180, 180, wxColor(255,0,0), _T("y = sin(x)"),
        true, wxColor(255,0,0,128));
    // draw the function cos(x) with blue, do not fill it
    plotter->AddCurve( _T("cosx"), -180, 180, wxColor(255,0,0), _T("y = cos(x)") );
    // set axis labels and title
    plotter->SetAxisLabels( _T("x"), _T("y = sin(x) and y = cos(x)") );
    plotter->SetPlotTitle( _T("Sample") );
    plotter->DrawCurves( true ); // if false, it will not erase background (eg. in case of a previosly drawn histogram)

    //
    // or draw a pie chart
    //
    arr.Add( 10 ); arr.Add( 20 ); arr.Add( 30 ); arr.Add( 40 );
    plotter->SetPlotTitle( _T("Sample pie chart") );
    plotter->DrawPiechart( arr, colors );

    //
    // or draw a parametric plot
    //
    plotter->SetViewPortLimits( -2, 2, -2, 2 );
    plotter->DrawParametricPlot( bflyx, bflyy, 0, 200, 2200, wxColor(255,0,0,128), _T("Butterfly") );

    //
    // or draw a polar plot
    //
    plotter->SetAxisLabels( _T("θ"), _T("r(θ) = 1 + cos(θ)"));
    plotter->SetPlotTitle( _T("Sample polar plot") );
    plotter->DrawPolarPlot( polarfunc, 0, 2*M_PI, 1000 );

    //
    // or display a 3D function
    //
    plotter->SetFillColor( wxColor(102,153,204,208) );
    plotter->Render3DFunc( func3d, -20, 20, -20, 20 );
}

// lets see mouse events - how to draw numerical values on plot
void RTPlotterTestFrame::OnMouse( wxMouseEvent& event )
{
    // in case of a 2D function
    int mx = event.GetX(), my = event.GetY();
    if ( plotter != NULL )
    {
        double xval, yval;
         // is the mouse cursor in the plot area
        if ( plotter->GetXYValue( mx, my, xval, yval ) )
        {
            SetCursor( wxCURSOR_CROSS );
            plotter->SetMousePos( event.GetPosition() ); // it will call RTPlotterTestFrame::Refresh
        }
        else SetCursor(wxCURSOR_ARROW);
    }

    // in case of a 3D function - set camera position
    if ( plotter != NULL )
    {
        double xval, yval;
         // is the mouse cursor in the plot area
        if ( plotter->GetXYValue( mx, my, xval, yval ) )
        {
            int w,h;
            GetSize(&w,&h);
            // calc camera position
            int cx=..., cy=..., cz=...;
            plotter->SetCameraPos( cx, cy, cz );
        }
    }
}

*/

/// END OF EXAMPLE

class SurfaceRenderer {
    public:
        SurfaceRenderer( double cX, double cY, double cZ, int xs0, int ys0, int w, int h );

        void SetLimits( wxRealPoint bottomLeft, wxRealPoint topRight );
        void SetLimits( double xmin, double ymin, double xmax, double ymax );
        void SetDensity( double density );
        void SetFunction( dbl3dFunc func );

        void RenderSurface( wxAgg2* canvas );

    private:
        wxRealPoint m_startPoint;
        wxRealPoint m_endPoint;
        wxColour    m_penColor;
        double      m_density;

        // transformation coeff.

        double      m_sD;
        double      m_sf, m_cf, m_st, m_ct;
        double      m_r, m_a, m_b, m_c, m_d;

        // z = f(x,y)
        dbl3dFunc   m_func;

        void calcTransfCoeff( double cX, double cY, double cZ, int xs0, int ys0, int w, int h );
        wxRealPoint projectPoint( double x, double y, double z );
        wxColour calcGradient( unsigned i );
};

Curve::Curve()
{
    color = wxColor( 0, 0, 0 );
    fillcolor = wxColor( 255, 0, 0, 128 );
    linewidth = 1.0;
    copied = false;
    fillcurve = false;
}

Curve::Curve( const Curve& c )
{
    n = c.n;
    xmin = c.xmin;
    xmax = c.xmax;
    ymin = c.ymin;
    ymax = c.ymax;
    color = c.color;
    linewidth = c.linewidth;
    name = c.name;
    descr = c.descr;

    x = new double[n];
    y = new double[n];

    memcpy( x, c.x, n * sizeof(double) );
    memcpy( y, c.y, n * sizeof(double) );

    fillcurve = c.fillcurve;
    fillcolor = c.fillcolor;

    copied = true;

}

Curve::~Curve()
{
    if ( copied )
    {
        delete[] x;
        delete[] y;
    }
}

FunctionA RTPlotter::m_functions;

WX_DEFINE_OBJARRAY( CurveA );

RTPlotter::RTPlotter( wxDC* dc, wxWindow* parent )
{
    m_dc = dc;
    m_parent = parent;

    m_width = 0;
    m_height = 0;
    m_xpos = 0;
    m_ypos = 0;

    m_minx = 0.0;
    m_miny = 0.0;
    m_maxx = 0.0;
    m_maxy = 0.0;

    m_linewidth = 1.0;
    m_linecolor = wxColor( 0, 0, 0 );
    m_bgcolor = wxColor( 255, 255, 255 );
    m_fillcolor = wxColor( 255, 255, 255 );
    m_divcolor = wxColor( 200, 200, 200 );

    m_selectionColor = wxColor( 217, 228, 241, 128 );

    m_hor_div = 10;
    m_vert_div = 8;

    m_aggcanvas = NULL;
    m_displayValues = true;
    m_mousepos.x = -1; m_mousepos.y = -1;

    m_xaxislabel = _T("");
    m_yaxislabel = _T("");
    m_plottitle = _T("");

    m_epsilon = 0.00001;

    m_coordSytem = CS_NONE;

    m_drawXValues = m_drawYValues = true;

    m_xvalFormat = _T("%.3g");
    m_yvalFormat = _T("%.3g");

    m_selectFrom = m_selectTo = -1;

    m_extraText = wxEmptyString;
}

RTPlotter::~RTPlotter()
{
    ClearCurves();
    DeleteCanvas();
}

void RTPlotter::SetExtraText( const wxString& text )
{
    m_extraText = text;
}

void RTPlotter::SetViewPortSize( unsigned w, unsigned h )
{
    m_width = w;
    m_height = h;
}

void RTPlotter::SetViewPortPosition( unsigned x, unsigned y )
{
    m_xpos = x;
    m_ypos = y;
}

void RTPlotter::SetViewPortLimits( float minx, float maxx, float miny, float maxy )
{
    m_minx = minx;
    m_miny = miny;
    m_maxx = maxx;
    m_maxy = maxy;
}

void RTPlotter::SetAxisDivision( int vert, int hor )
{
    m_hor_div = ( hor >= 1 )? hor : 1;
    m_vert_div = ( vert >= 1 ) ? vert : 1;
}

void RTPlotter::SetAxisLabels( const wxString& xlabel, const wxString& ylabel )
{
    m_xaxislabel = xlabel;
    m_yaxislabel = ylabel;
}

void RTPlotter::SetPlotTitle( const wxString& title )
{
    m_plottitle = title;
}

void RTPlotter::SetSelection( double from, double to )
{
    if ( from < to )
    {
        m_selectFrom = from; m_selectTo = to;
    }
    else
    {
        m_selectFrom = to; m_selectTo = from;
    }
}

void RTPlotter::GetRectSize()
{
    m_rectSize.h = m_height - 3 * m_textHeight - 20;

    double extraw = -1;

    if ( m_coordSytem == CS_CARTHESIAN )
    {
        double vert_step = (m_maxy - m_miny) / (double) m_vert_div;
        if ( m_maxx * m_maxy < 0 )
        {
            double val = 0;
            while ( val >= m_miny )
            {
                wxString str = wxString::Format(_T("%.5g"), val );
                double w1 = m_aggcanvas->getTTFTextBound( str ).w; if ( w1 > extraw ) extraw = w1;
                val -= vert_step;
            }
            val = vert_step;
            while ( val <= m_maxy )
            {
                wxString str = wxString::Format(_T("%.5g"), val );
                double w1 = m_aggcanvas->getTTFTextBound( str ).w; if ( w1 > extraw ) extraw = w1;
                val += vert_step;
            }
        }
        else
        {
            double val = m_miny;
            while ( val <= m_maxy )
            {
                wxString str = wxString::Format(_T("%.5g"), val );
                double w1 = m_aggcanvas->getTTFTextBound( str ).w; if ( w1 > extraw ) extraw = w1;
                val += vert_step;
            }
        }
    }
    else
    {
        extraw = m_textHeight * 0.8;
    }

    m_rectSize.w = m_width - 1.7 * extraw - 20;
    m_rectSize.x = extraw * 1.7;


    m_rectSize.y = m_textHeight + 10;
}

void RTPlotter::DrawAxes()
{
    if ( m_maxx == m_minx || m_maxy == m_miny )
        return;

    m_aggcanvas->getBrush()->SetStyle( wxAGGBRUSH_SOLID );
    m_aggcanvas->setBrush( m_bgcolor );
    m_aggcanvas->setPen( m_linecolor );

    // m_aggcanvas->drawFilledRectangle( m_rectSize.x, m_rectSize.y, m_rectSize.w, m_rectSize.h, m_linewidth );
    m_aggcanvas->setPenWidth( m_linewidth );
    m_aggcanvas->drawRectangle( m_rectSize.x, m_rectSize.y, m_rectSize.w, m_rectSize.h );

    m_aggcanvas->setPen( m_divcolor );

    m_aggcanvas->setTransparentBrush();

    double pattern[2]; pattern[0] = 3.0; pattern[1] = 3.0;

    double posXA = -1, posYA = -1;

    // draw abscissa & ordinate
    if ( m_maxx * m_minx < 0 )
    {
        posYA = fabs(m_minx) / ( m_maxx - m_minx ) * m_rectSize.w + m_rectSize.x;
        m_aggcanvas->drawLine( posYA, m_rectSize.y, posYA, m_rectSize.y + m_rectSize.h - 1 );


        if ( m_drawYValues )
        {
            m_aggcanvas->setPen( m_linecolor );
            wxString str = _T("0");
            wxRealRect r = m_aggcanvas->getTTFTextBound( str );
            m_aggcanvas->drawTTFText( posYA - r.w/2, m_rectSize.y + m_rectSize.h + m_textHeight/2, str );
        }
    }
    if ( m_maxy * m_miny < 0 )
    {
        posXA = fabs(m_maxy) / ( m_maxy - m_miny ) * m_rectSize.h + m_rectSize.y;
        m_aggcanvas->setPen( m_divcolor );
        m_aggcanvas->drawLine( m_rectSize.x, posXA, m_rectSize.x + m_rectSize.w - 1, posXA );

        if ( m_drawXValues )
        {
            wxString str = _T("0");
            m_aggcanvas->setPen( m_linecolor );
            wxRealRect r = m_aggcanvas->getTTFTextBound( str );
            m_aggcanvas->drawTTFText( m_rectSize.x - 1.2*r.w, posXA + r.h/2, str );
        }
    }

    double vert_step = m_rectSize.h / (double) m_vert_div;
    double hor_step = m_rectSize.w / (double) m_hor_div;

    // draw vertical division lines
    if ( m_vert_div > 1 )
    {
        if ( posXA != -1 )
        {
            double pos = posXA - vert_step;
            while ( pos >= m_rectSize.y )
            {
                if ( pos != m_rectSize.y )
                {
                    double x1 = m_rectSize.x, x2 = m_rectSize.x + m_rectSize.w - 1;
                    m_aggcanvas->setPen( m_divcolor );
                    m_aggcanvas->saveState();
                    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_USER_DASH );
                    m_aggcanvas->getPen()->SetDashes( 1, pattern );
                    m_aggcanvas->setPenWidth( 1.0 );
                    //m_aggcanvas->drawDashedLine( x1, pos, x2, pos, 1.0, 1, pattern );
                    m_aggcanvas->drawLine( x1, pos, x2, pos );
                    m_aggcanvas->restoreState();
                }

                if ( m_drawYValues )
                {
                    if ( ( posXA - m_rectSize.y ) != 0 )
                    {
                        double val = ( posXA - pos ) / ( posXA - m_rectSize.y ) * m_maxy;
                        if ( fabs(val) < m_epsilon ) val = 0;
                        wxString str = wxString::Format(m_yvalFormat,val);
                        m_aggcanvas->setPen( m_linecolor );
                        wxRealRect r = m_aggcanvas->getTTFTextBound( str );
                        m_aggcanvas->drawTTFText( m_rectSize.x - 1.2*r.w, pos + r.h/2, str );
                    }
                }

                pos -= vert_step;
            }
            pos = posXA + vert_step;
            while ( pos <= m_rectSize.y + m_rectSize.h )
            {
                if ( pos <  m_rectSize.y + m_rectSize.h )
                {
                    double x1 = m_rectSize.x, x2 = m_rectSize.x + m_rectSize.w - 1;
                    m_aggcanvas->setPen( m_divcolor );
                    m_aggcanvas->saveState();
                    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_USER_DASH );
                    m_aggcanvas->getPen()->SetDashes( 1, pattern );
                    m_aggcanvas->setPenWidth( 1.0 );
                    //m_aggcanvas->drawDashedLine( x1, pos, x2, pos, 1.0, 1, pattern );
                    m_aggcanvas->drawLine( x1, pos, x2, pos );
                    m_aggcanvas->restoreState();
                }

                // wrong??
                // double val = ( pos - posXA ) / ( posXA - m_rectSize.y )* m_miny;

                if ( m_drawYValues )
                {
                    if ( ( m_rectSize.h - posXA + m_rectSize.y ) != 0 )
                    {
                        double val = ( pos - posXA ) / ( m_rectSize.h - posXA + m_rectSize.y ) * m_miny;
                        if ( fabs(val) < m_epsilon ) val = 0;
                        wxString str = wxString::Format(m_yvalFormat,val);
                        m_aggcanvas->setPen( m_linecolor );
                        wxRealRect r = m_aggcanvas->getTTFTextBound( str );
                        m_aggcanvas->drawTTFText( m_rectSize.x - 1.2*r.w, pos + r.h/2, str );
                    }
                }
                pos += vert_step;
            }
        }
        else
        {
            double pos = m_rectSize.y;
            while ( pos < m_rectSize.y + m_rectSize.h + 1 )
            {
                if ( pos < m_rectSize.y + m_rectSize.h && pos != m_rectSize.y )
                {
                    double x1 = m_rectSize.x, x2 = m_rectSize.x + m_rectSize.w - 1;
                    m_aggcanvas->setPen( m_divcolor );

                    m_aggcanvas->saveState();
                    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_USER_DASH );
                    m_aggcanvas->getPen()->SetDashes( 1, pattern );
                    m_aggcanvas->setPenWidth( 1.0 );
                    //m_aggcanvas->drawDashedLine( x1, pos, x2, pos, 1.0, 1, pattern );
                    m_aggcanvas->drawLine( x1, pos, x2, pos );
                    m_aggcanvas->restoreState();
                }

                if ( m_drawYValues && ( m_rectSize.h != 0 ) )
                {
                    double val = m_maxy - (( pos - m_rectSize.y ) / ( m_rectSize.h )* (m_maxy-m_miny));
                    if ( fabs(val) < m_epsilon ) val = 0;
                    wxString str = wxString::Format(m_yvalFormat,val);
                    m_aggcanvas->setPen( m_linecolor );
                    wxRealRect r = m_aggcanvas->getTTFTextBound( str );
                    m_aggcanvas->drawTTFText( m_rectSize.x - 1.2*r.w, pos + r.h/2, str );
                }
                pos += vert_step;
            }
        }
    }

    if ( m_hor_div > 1 )
    {
        if ( posYA != -1 )
        {
            double pos = posYA - hor_step;
            while ( pos >= m_rectSize.x )
            {
                if ( pos != m_rectSize.x )
                {
                    double y1 = m_rectSize.y, y2 = m_rectSize.y + m_rectSize.h - 1;

                    m_aggcanvas->setPen( m_divcolor );

                    m_aggcanvas->saveState();
                    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_USER_DASH );
                    m_aggcanvas->getPen()->SetDashes( 1, pattern );
                    m_aggcanvas->setPenWidth( 1.0 );
                    //m_aggcanvas->drawDashedLine( pos, y1, pos, y2, 1.0, 1, pattern );
                    m_aggcanvas->drawLine( pos, y1, pos, y2 );
                    m_aggcanvas->restoreState();
                }

                if ( m_drawXValues )
                {
                    if ( ( posYA - m_rectSize.x ) != 0 )
                    {
                        double val = ( posYA - pos ) /  ( posYA - m_rectSize.x ) * m_minx;
                        if ( fabs(val) < m_epsilon ) val = 0;
                        wxString str = wxString::Format(m_xvalFormat, val );
                        m_aggcanvas->setPen( m_linecolor );
                        wxRealRect r = m_aggcanvas->getTTFTextBound( str );
                        m_aggcanvas->drawTTFText( pos - r.w/2, m_rectSize.y + m_rectSize.h + r.h, str );
                    }
                }

                pos -= hor_step;
            }
            pos = posYA + hor_step;
            while ( pos <= m_rectSize.x + m_rectSize.w )
            {
                if ( pos < m_rectSize.x + m_rectSize.w )
                {
                    double y1 = m_rectSize.y, y2 = m_rectSize.y + m_rectSize.h - 1;
                    m_aggcanvas->setPen( m_divcolor );
                    m_aggcanvas->saveState();
                    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_USER_DASH );
                    m_aggcanvas->getPen()->SetDashes( 1, pattern );
                    m_aggcanvas->setPenWidth( 1.0 );
                    //m_aggcanvas->drawDashedLine( pos, y1, pos, y2, 1.0, 1, pattern );
                    m_aggcanvas->drawLine( pos, y1, pos, y2 );
                    m_aggcanvas->restoreState();
                }

                if ( m_drawXValues )
                {
                    if ( ( m_rectSize.w - posYA + m_rectSize.x ) != 0 )
                    {
                        double val = ( pos - posYA ) / ( m_rectSize.w - posYA + m_rectSize.x ) * m_maxx;
                        if ( fabs(val) < m_epsilon ) val = 0;
                        wxString str = wxString::Format( m_xvalFormat, val );
                        m_aggcanvas->setPen( m_linecolor );
                        wxRealRect r = m_aggcanvas->getTTFTextBound( str );
                        m_aggcanvas->drawTTFText( pos - r.w/2, m_rectSize.y + m_rectSize.h + r.h, str );
                    }
                }

                pos += hor_step;
            }
        }
        else
        {
            double pos = m_rectSize.x + hor_step;
            while ( pos < m_rectSize.x + m_rectSize.w + 1 )
            {
                if ( pos < m_rectSize.x + m_rectSize.w && pos != m_rectSize.x )
                {
                    double y1 = m_rectSize.y, y2 = m_rectSize.y + m_rectSize.h - 1;
                    m_aggcanvas->setPen( m_divcolor );
                    m_aggcanvas->saveState();
                    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_USER_DASH );
                    m_aggcanvas->getPen()->SetDashes( 1, pattern );
                    m_aggcanvas->setPenWidth( 1.0 );
                    //m_aggcanvas->drawDashedLine( pos, y1, pos, y2, 1.0, 1, pattern );
                    m_aggcanvas->drawLine( pos, y1, pos, y2 );
                    m_aggcanvas->restoreState();
                }

                if ( m_drawXValues && ( m_rectSize.w != 0 ) )
                {
                    double val = m_minx + ( pos - m_rectSize.x ) / ( m_rectSize.w ) * ( m_maxx - m_minx );
                    if ( fabs(val) < m_epsilon ) val = 0;
                    wxString str = wxString::Format( m_xvalFormat, val );
                    m_aggcanvas->setPen( m_linecolor );
                    wxRealRect r = m_aggcanvas->getTTFTextBound( str );
                    m_aggcanvas->drawTTFText( pos - r.w/2, m_rectSize.y + m_rectSize.h + r.h, str );
                }

                pos += hor_step;
            }
        }
    }

    // draw labels
    if ( m_xaxislabel != wxEmptyString && m_drawXValues )
    {
        double xw = (m_aggcanvas->getTTFTextBound( m_xaxislabel, 0 )).w;
        m_aggcanvas->drawTTFText( m_rectSize.x + (m_rectSize.w - xw) / 2, m_rectSize.y + m_rectSize.h + m_textHeight, m_xaxislabel );
    }

    if ( m_yaxislabel != wxEmptyString && m_drawYValues )
    {
        double yh = (m_aggcanvas->getTTFTextBound( m_yaxislabel, 270)).h;
        m_aggcanvas->drawTTFText( m_textHeight/2, m_rectSize.y + ( m_rectSize.h - yh) / 2, m_yaxislabel, 270 );
    }

    // draw title
    if ( m_plottitle != wxEmptyString )
    {
        double tw = (m_aggcanvas->getTTFTextBound( m_plottitle, 0)).w;
        m_aggcanvas->drawTTFText( m_rectSize.x + ( m_rectSize.w - tw ) / 2, m_textHeight, m_plottitle );
    }

    m_coordSytem = CS_CARTHESIAN;
}

void RTPlotter::AddFunction( const wxString& funcname, dblFunc f )
{
    m_functions[funcname] = f;
}

void RTPlotter::AddCurve( Curve c )
{
    m_curves.Add( c );
}

void RTPlotter::ClearCurves()
{
    m_curves.Clear();
}

void RTPlotter::AddCurve( const wxString& funcname, double xmin, double xmax, wxColour color, const wxString& descr, bool fillcurve, wxColour fillColor )
{
    if ( !funcExists( funcname ) )
        return;

    dblFunc f = m_functions[funcname];

    GetRectSize();
/*    if ( m_rectSize.w < 1 )
        return;*/

    unsigned n = (unsigned)m_rectSize.w;
    if ( n < 2 )
        return;

    double step = ( xmax - xmin ) / (double)(n - 1);

    double* x = new double[n];
    double* y = new double[n];

    double xs = xmin;
    double ymin = 9E35, ymax = -9E35;

    for ( unsigned i = 0; i < n; ++i )
    {
        x[i] = xs;
        y[i] = f( xs );
        xs += step;
        if ( y[i] < ymin ) ymin = y[i];
        if ( y[i] > ymax ) ymax = y[i];
    }

    Curve c;
    c.name = funcname;
    c.xmin = xmin; c.xmax = xmax;
    c.x = x;
    c.y = y;
    c.ymin = ymin; c.ymax = ymax;
    c.n = n;
    c.descr = ( descr == wxEmptyString ) ? funcname : descr;
    c.fillcurve = fillcurve;

    if ( color == wxNullColour )
        c.color = m_linecolor;
    else
        c.color = color;

    if ( fillColor == wxNullColour )
        c.fillcolor = m_fillcolor;
    else
        c.fillcolor = fillColor;

    SetViewPortLimits( xmin, xmax, ymin, ymax );
    AddCurve( c );

    delete[] x;
    delete[] y;
}

void RTPlotter::CreateCanvas( double mintextHeight )
{
    if ( m_aggcanvas )
        delete m_aggcanvas;
    m_aggcanvas = new wxAgg2( m_width, m_height );

    if ( mintextHeight < 0 )
    {
        m_textHeight = m_height / 20.0;
    }
    else
    {
        double c_textheight = m_height / 20.0;
        if ( mintextHeight > c_textheight )
        {
            m_textHeight = 2 * mintextHeight;
        }
        else m_textHeight = c_textheight;
    }

    m_aggcanvas->loadTTFont( DirHelper::GetPath(dFONTS) + _T("arial.ttf"), m_textHeight / 2, 0 );
}

void RTPlotter::DrawSelection()
{
    m_aggcanvas->saveState();

    wxAggBrush brush( m_selectionColor, wxAGGBRUSH_SOLID );
    m_aggcanvas->setBrush( brush );
    m_aggcanvas->setPen( m_selectionColor );

    double xfrom = ( m_selectFrom - m_minx) * m_rectSize.w / ( m_maxx - m_minx ) + m_rectSize.x;
    double xlen = ( m_selectTo - m_minx) * m_rectSize.w / ( m_maxx - m_minx ) + m_rectSize.x - xfrom;

    m_aggcanvas->drawRectangle( xfrom, m_rectSize.y, xlen, m_rectSize.h );

    m_aggcanvas->restoreState();
}

void RTPlotter::Render()
{
    if ( m_dc == NULL )
        return;

    // draw selection
    if ( m_selectFrom >= 0 && m_selectTo >= 0 && m_selectFrom >= m_minx && m_selectTo <= m_maxx )
        DrawSelection();

    if ( m_displayValues )
    {
        RenderValues();
    }

    wxBitmap* canvasbmp = m_aggcanvas->renderToBitmap();
    m_dc->DrawBitmap( *canvasbmp, m_xpos, m_ypos, true );

    delete canvasbmp;
}

void RTPlotter::RenderToMemoryDC( wxDC* dc )
{
    if ( dc == NULL )
        return;

    // draw selection
    if ( m_selectFrom >= 0 && m_selectTo >= 0 && m_selectFrom <= m_minx && m_selectTo >= m_maxx )
        DrawSelection();

    wxBitmap* canvasbmp = m_aggcanvas->renderToBitmap();
    dc->DrawBitmap( *canvasbmp, 0, 0, true );

    delete canvasbmp;
}


void RTPlotter::DeleteCanvas()
{
    if ( m_aggcanvas != NULL )
    {
        delete m_aggcanvas;
        m_aggcanvas = NULL;
    }
}

void RTPlotter::DrawCurves( bool eraseBg, bool autoscaleaxes )
{
    m_displayValues = true;
    if ( autoscaleaxes )
    {
        double maxx = -9E25, maxy = -9E25;
        double minx = 9E25, miny = 9E25;

        for ( size_t i = 0; i < m_curves.GetCount(); ++i )
        {
            Curve c = m_curves[i];
            if ( c.xmax > maxx ) maxx = c.xmax;
            if ( c.ymax > maxy ) maxy = c.ymax;
            if ( c.xmin < minx ) minx = c.xmin;
            if ( c.ymin < miny ) miny = c.ymin;
        }

        if ( fabs(minx) < m_epsilon ) minx = 0;
        if ( fabs(miny) < m_epsilon ) miny = 0;

        SetViewPortLimits( minx, maxx, miny, maxy );
    }

    if ( m_minx == m_maxx || m_miny == m_maxy )
        return;
    if ( m_width == 0 || m_height == 0 )
        return;
    if ( m_rectSize.w < 1 || m_rectSize.h < 1 )
        return;
    if ( eraseBg )
    {
        CreateCanvas();
        for ( unsigned n = 0; n < m_curves.GetCount(); ++ n )
        {
            Curve c = m_curves[n];
            if ( c.xmax > m_maxx ) m_maxx = c.xmax;
            if ( c.xmin < m_minx ) m_minx = c.xmin;
            if ( c.ymax > m_maxy ) m_maxy = c.ymax;
            if ( c.ymin < m_miny ) m_miny = c.ymin;
        }
        DrawAxes();
    }

    m_aggcanvas->setPen( wxColor( 160,0,0) );

    /// draw on DC

    for ( unsigned n = 0; n < m_curves.GetCount(); ++n )
    {
        Curve c = m_curves[n];

        double* x = c.x;
        double* y = c.y;

        RealPointArray points;
        for ( size_t i = 0; i < c.n; ++i )
        {
            wxRealPoint p;
            p.x = ( x[i] - m_minx ) / ( m_maxx - m_minx ) * m_rectSize.w + m_rectSize.x;
            p.y = ( m_maxy - y[i] ) / ( m_maxy - m_miny ) * m_rectSize.h + m_rectSize.y;

            if ( p.x < m_rectSize.x ) p.x = m_rectSize.x;
            if ( p.x > m_rectSize.x + m_rectSize.w ) p.x = m_rectSize.x + m_rectSize.w;
            if ( p.y < m_rectSize.y ) p.y = m_rectSize.y;
            if ( p.y > m_rectSize.y + m_rectSize.h ) p.y = m_rectSize.y + m_rectSize.h;

            points.Add( p );
        }

        m_aggcanvas->setPen( c.color );

        if ( c.fillcurve == false )
        {
            m_aggcanvas->saveState();
            m_aggcanvas->setPenWidth( c.linewidth );
            m_aggcanvas->setTransparentBrush();
            m_aggcanvas->drawLine( points );
            m_aggcanvas->restoreState();
        }
        else
        {
            m_aggcanvas->saveState();
            m_aggcanvas->setPenWidth( c.linewidth );
            m_aggcanvas->setBrush( c.fillcolor );
            m_aggcanvas->drawPolygon( points );
            m_aggcanvas->restoreState();
        }

        m_aggcanvas->drawTTFText( m_rectSize.x + 4, m_rectSize.y + (n+1) * m_textHeight, c.descr );
    }

    m_coordSytem = CS_CARTHESIAN;

    Render();
}

void RTPlotter::AddCurve( const RealPointArray& points, wxColour color, const wxString& name, bool fillcurve, wxColour fillColor )
{
    unsigned int n = points.GetCount();

    if ( n < 1 )
        return;

    double *x = new double[n];
    double *y = new double[n];

    for ( unsigned int i = 0; i < n; ++i )
    {
        x[i] = points[i].x;
        y[i] = points[i].y;
    }

    AddCurve( n, x, y, color, name, fillcurve, fillColor );

    delete[] x;
    delete[] y;

}

void RTPlotter::AddCurve( unsigned n, double *x, double *y, wxColour color, const wxString& name, bool fillcurve, wxColour fillColor )
{
    if ( n < 1 || x == NULL || y == NULL )
        return;

    /*
    GetRectSize();

    if ( m_rectSize.w < 1 )
        return;*/

    double xmin = 9E35, xmax = -9E35;
    double ymin = 9E35, ymax = -9E35;
    double* xc = new double[n];
    double* yc = new double[n];

    for ( unsigned i = 0; i < n; ++i )
    {
        xc[i] = x[i];
        yc[i] = y[i];
        if ( xc[i] > xmax ) xmax = xc[i];
        if ( xc[i] < xmin ) xmin = xc[i];
        if ( yc[i] < ymin ) ymin = yc[i];
        if ( yc[i] > ymax ) ymax = yc[i];
    }

    Curve c;
    c.name = name;
    c.descr = name;
    c.xmin = xmin; c.ymin = ymin;
    c.xmax = xmax; c.xmax = xmax;
    c.n = n;
    c.x = xc;
    c.y = yc;
    c.fillcurve = fillcurve;

    if ( color == wxNullColour )
        c.color = m_linecolor;
    else
        c.color = color;

    if ( fillColor == wxNullColour )
        c.fillcolor = m_fillcolor;
    else
        c.fillcolor = fillColor;

    // SetViewPortLimits( xmin, xmax, ymin, ymax );
    AddCurve( c );

    delete[] xc;
    delete[] yc;
}

void RTPlotter::DrawHistogram( const DoubleArray& values, double xmin, double xmax, unsigned int nbins, bool relative, bool isMax1 )
{
    unsigned n = values.GetCount();

    if ( n < 1 )
        return;

    double* dvals = new double[n];

    for ( unsigned i = 0; i < n; ++i )
        dvals[i] = values[i];

    DrawHistogram( n, dvals, xmin, xmax, nbins, relative, isMax1 );

    delete[] dvals;
}

void RTPlotter::DrawHistogram( unsigned n, double* values, double xmin, double xmax, unsigned nbins, bool relative, bool isMax1 )
{
    m_displayValues = false;
    if ( n < 1 || values == NULL || xmin >= xmax || nbins < 1 )
        return;

    double maxy = 0.0;
    // calculate histogram
    unsigned int* freq = new unsigned int[nbins];

    for ( unsigned i = 0; i < nbins; ++i )
        freq[i] = 0;

    double nbintv = (double) nbins / ( xmax - xmin );

    for ( unsigned i = 0; i < n; ++i )
    {
        unsigned p = (unsigned int)(( values[i] - xmin ) * nbintv );

        if ( p <= nbins )
        {
            if ( p == nbins ) p = nbins - 1;
            ++freq[p];
            if ( freq[p] > maxy ) maxy = (double)freq[p];
        }
    }

    m_minx = xmin; m_maxx = xmax;
    m_miny = 0;

    if ( relative )
    {
        m_maxy = maxy / n;
        if ( isMax1 ) m_maxy = 1.0;
    }
    else
        m_maxy = maxy;

    //
    GetRectSize();

    DrawAxes();

    double hist_b_step = m_rectSize.w / (double) nbins;

    double xstart = m_rectSize.x + 1;

    m_aggcanvas->setPen( m_linecolor );
    m_aggcanvas->getBrush()->SetStyle( wxAGGBRUSH_SOLID );
    m_aggcanvas->setBrush( m_fillcolor );
    m_aggcanvas->setPenWidth( m_linewidth );

    for ( unsigned i = 0; i < nbins; ++ i )
    {
        double h;

        if ( relative )
        {
            if ( isMax1 )
            {
                h = (double) freq[i]/n * m_rectSize.h;
            }
            else
            {
                h = (double)freq[i]/maxy * m_rectSize.h;
            }
        }
        else h = (double)freq[i]/ maxy * m_rectSize.h;

        double w = hist_b_step - 1;


        /*m_aggcanvas->drawFilledRectangle( xstart, ( m_rectSize.y + m_rectSize.h - h), w, h,
            m_linewidth );*/
        m_aggcanvas->drawRectangle( xstart, ( m_rectSize.y + m_rectSize.h - h ), w, h );
        xstart += hist_b_step;
    }

    Render();

    delete[] freq;
}

#include "../triggerlog/TriggerLogSettingsHelper.h"

void RTPlotter::DrawTriggers( Int2DoubleArrayHash& triggers, double xmin, double xmax, const Int2IntHash& triggerFrames )
{
    // # of triggertypes
    size_t t_num = triggers.size();

    double* tmax = new double[t_num];
    int* tkeys = new int[t_num];

    for ( size_t i = 0; i < t_num; ++i )
    {
        tmax[i] = 0.;
        tkeys[i] = 0;
    }

    m_xvalFormat = _T("%.3f");
    m_yvalFormat = _T("%.3f");

    Int2DoubleArrayHash::const_iterator it = triggers.begin();

    bool hasmax = false;

    int i = 0;
    for ( ; it != triggers.end(); ++it )
    {
        tkeys[i] = it->first;
        const DoubleArray& arr = it->second;
        tmax[i] = ( arr.GetCount() ) ? arr[arr.GetCount() - 1] : 0;

        hasmax |= ( tmax[i] > 0 );
        ++i;
    }

    if ( ! hasmax )
    {
        m_aggcanvas->saveState();
        m_aggcanvas->setPen( TriggerLogSettingsHelper::tLog_text );
        m_aggcanvas->setFontHeight( 54 );
        m_aggcanvas->drawTTFText(_T("No data to display"), 50, 260 );

        m_aggcanvas->restoreState();

        Render();

        delete[] tmax;
        delete[] tkeys;
        return;
    }

    bool drawxval = m_drawXValues, drawyval = m_drawYValues;

    m_vert_div = 4;

    m_drawXValues = true; m_drawYValues = false;

    m_miny = 0;
    m_maxy = 6.0;
    m_minx = xmin;
    m_maxx = xmax;

    if ( m_maxx == m_minx ){
        delete[] tkeys;
        delete[] tmax;
        return;
    }

    GetRectSize();
    DrawAxes();

    Int2wxColorHash& triggerColors = TriggerLogSettingsHelper::tLogColors;

    for ( size_t j = 0; j < t_num; ++j )
    {
        int event_id = tkeys[j];

        if ( TriggerLogSettingsHelper::tLogEventsOrder.Index( event_id ) == wxNOT_FOUND )
        {
            continue;
        }
        if ( triggerColors.find( event_id ) == triggerColors.end() )
        {
            continue;
        }

        m_aggcanvas->setPen( triggerColors[event_id] );
        m_aggcanvas->setPenWidth( 2.0 );

        double x = m_rectSize.x;
        double y = m_rectSize.y + ( m_rectSize.h / 4 ) * 3;

        int key = tkeys[j];
        const DoubleArray& arr = triggers[key];

        double offset = -TriggerLogSettingsHelper::tLogVerticalOffsets[event_id]; // flip vertical

        y += offset;

        for ( size_t i = 0; i < arr.GetCount(); ++i )
        {
            double val = arr[i];

            if ( val >= m_minx && val <= m_maxx )
            {
                double xto = ( arr[i] - m_minx ) * m_rectSize.w / ( m_maxx - m_minx ) + m_rectSize.x;

                double yto = y;

                if ( fabs ( x - xto ) >= 1 )
                {
                    m_aggcanvas->drawLine( x, y, xto, y );
                    yto = m_rectSize.y + ( m_rectSize.h / 4 ) + offset ;
                    m_aggcanvas->drawLine( xto, y, xto, yto );
                    x = xto;
                }
            }
        }

        if ( tmax[j] >= xmax )
            m_aggcanvas->drawLine( x, y, m_rectSize.x + m_rectSize.w, y );
    }

    double oldFW = m_aggcanvas->getFontWidth();

    Int2StringHash& tlognames = TriggerLogSettingsHelper::tLogNames;

    size_t counter = 0;

    for ( size_t j = 0; j < t_num; ++j )
    {
        int event_id = tkeys[j];

        if ( tlognames.find( event_id ) == tlognames.end() )
        {
            continue;
        }

        counter++;

        m_aggcanvas->setPen( triggerColors[event_id] );

        m_aggcanvas->setFontWidth( 15 );

        wxString trStat = tlognames[event_id];

        Int2IntHash::const_iterator iit = triggerFrames.find( event_id );
        if ( iit != triggerFrames.end() )
        {
            size_t trCount = iit->second;
            trStat += wxString::Format( _T(" (%d)"), trCount );
        }
        else
        {
            trStat += _T(" (?)");
        }

        m_aggcanvas->drawTTFText( trStat , m_rectSize.x + 10, m_rectSize.y + 15 * counter );
    }

    m_aggcanvas->setFontWidth( oldFW );
    m_aggcanvas->setPenWidth( 1.0 );

    Render();

    m_drawXValues = drawxval;
    m_drawYValues = drawyval;

    delete[] tkeys;
    delete[] tmax;
}

/*void RTPlotter::DrawTriggers( const DoubleArray& primary, const DoubleArray& secondary, const wxColour& primColor, const wxColour& secColor, double xmin, double xmax )
{
    double pMax, sMax;
    m_xvalFormat = _T("%.3f");
    m_yvalFormat = _T("%.3f");
    pMax = ( primary.GetCount() ) ? primary[primary.GetCount()-1] : 0;
    sMax = ( secondary.GetCount() ) ? secondary[secondary.GetCount()-1] : 0;

    if ( pMax == 0 && sMax == 0 )
        return;

    bool drawxval = m_drawXValues, drawyval = m_drawYValues;

    m_vert_div = 4;

    m_drawXValues = true; m_drawYValues = false;

    m_miny = 0;
    m_maxy = 6.0;
    m_minx = xmin;
    m_maxx = xmax;

    if ( m_maxx == m_minx )
        return;

    GetRectSize();
    DrawAxes();

    m_aggcanvas->setPen( primColor );
    m_aggcanvas->setPenWidth( 2.0 );

    double x = m_rectSize.x;
    double y = m_rectSize.y + ( m_rectSize.h / 4 ) * 3;

    for ( size_t i = 0; i < primary.GetCount(); ++i )
    {
        double val = primary[i];
        if ( val >= m_minx && val <= m_maxx )
        {
            double xto = (primary[i] - m_minx) * m_rectSize.w / ( m_maxx - m_minx ) + m_rectSize.x;
            double yto = y;
            if ( fabs( x - xto ) >= 1 )
            {
                m_aggcanvas->drawLine( x, y, xto, y );
                yto = m_rectSize.y + ( m_rectSize.h / 4 );
                m_aggcanvas->drawLine( xto, y, xto, yto );
                x = xto;
            }
        }
    }
    if ( pMax >= xmax )
        m_aggcanvas->drawLine( x, y, m_rectSize.x + m_rectSize.w, y );

    m_aggcanvas->setPen( secColor );

    x = m_rectSize.x;
    y = m_rectSize.y + m_rectSize.h - 1;
    for ( size_t i = 0; i < secondary.GetCount(); ++i )
    {
        double val = secondary[i];
        if ( val >= m_minx && val <= m_maxx )
        {
            double xto = (secondary[i] - m_minx) * m_rectSize.w / ( m_maxx - m_minx ) + m_rectSize.x;
            double yto = y;

            if ( fabs( x - xto ) >= 1 )
            {
                m_aggcanvas->drawLine( x, y, xto, y );
                yto = m_rectSize.y + ( m_rectSize.h / 4 ) * 2;
                m_aggcanvas->drawLine( xto, y, xto, yto );
                x = xto;
            }
        }
    }

    if ( sMax >= xmax )
        m_aggcanvas->drawLine( x, y, m_rectSize.x + m_rectSize.w, y );

    m_aggcanvas->setPen( primColor );

    double oldFW = m_aggcanvas->getFontWidth();

    m_aggcanvas->setFontWidth( 15 );


    m_aggcanvas->drawTTFText( _T("Primary trigger"), m_rectSize.x + 10, m_rectSize.y + 15 );

    m_aggcanvas->setPen( secColor );
    m_aggcanvas->drawTTFText( _T("Secondary trigger"), m_rectSize.x + 10, m_rectSize.y + 30 );

    m_aggcanvas->setFontWidth( oldFW );

    m_aggcanvas->setPenWidth( 1.0 );

    Render();

    m_drawXValues = drawxval;
    m_drawYValues = drawyval;
}*/

void RTPlotter::DrawParametricPlot( dblFunc xt, dblFunc yt, double tmin, double tmax,
    unsigned points, wxColour color, const wxString& descr, bool fillcurve, wxColour fillColor )
{
    if ( tmax <= tmin || points < 1 )
        return;

    double xmax = -9E25, ymax = -9E25;
    double xmin = 9E25, ymin = 9E25;

    double steps = ( tmax - tmin ) / (double) (points-1);

    double* x = new double[points];
    double* y = new double[points];

    double t = tmin;
    for ( unsigned i = 0; i < points; ++i )
    {
        x[i] = xt(t); y[i] = yt(t);
        if ( x[i] > xmax ) xmax = x[i];
        if ( x[i] < xmin ) xmin = x[i];
        if ( y[i] > ymax ) ymax = y[i];
        if ( y[i] < ymin ) ymin = y[i];

        t += steps;
    }

    Curve c;
    c.linewidth = m_linewidth;
    c.n = points; c.x = x; c.y = y;
    c.name = descr; c.descr = descr;
    c.xmax = xmax; c.xmin = xmin;
    c.ymax = ymax; c.ymin = ymin;
    c.fillcurve = fillcurve;
    if ( color == wxNullColour )
        c.color = m_linecolor;
    else
        c.color = color;

    if ( fillColor == wxNullColour )
        c.fillcolor = m_fillcolor;
    else
        c.fillcolor = fillColor;

    SetViewPortLimits( xmin, xmax, ymin, ymax );
    GetRectSize();
    AddCurve( c );

    delete[] x;
    delete[] y;
}

bool RTPlotter::GetXYValue( unsigned scrx, unsigned scry, double& x, double& y)
{
    if ( m_coordSytem == CS_NONE )
    {
        x = 0; y = 0;
        return false;
    }

    GetRectSize();

    scrx -= m_xpos;
    scry -= m_ypos;

    if ( scrx < m_rectSize.x || scrx > m_rectSize.x + m_rectSize.w )
        return false;

    if ( scry < m_rectSize.y || scry > m_rectSize.y + m_rectSize.h )
        return false;

    if ( m_coordSytem == CS_CARTHESIAN )
    {
        x = m_minx + ( scrx - m_rectSize.x ) / m_rectSize.w * ( m_maxx - m_minx );
        y = m_maxy - ( scry - m_rectSize.y ) / m_rectSize.h * ( m_maxy - m_miny );
    }

    if ( m_coordSytem == CS_POLAR )
    {
        double xt = m_minx + ( scrx - m_rectSize.x ) / m_rectSize.w * ( m_maxx - m_minx );
        double posXA = fabs(m_maxy) / ( m_maxy - m_miny ) * m_rectSize.h + m_rectSize.y;
        double yt = ( posXA - scry  ) / m_rectSize.w * ( m_maxy - m_miny );

        double th = atan2(yt,xt);
        if ( th < 0 ) th += 2 * M_PI;
        y = th * 180.0 / M_PI;
        x = xt;
    }

    return true;
}

void RTPlotter::SetMousePos( wxPoint mousepos )
{
    m_mousepos = mousepos;

    if ( m_parent != NULL )
        m_parent->Refresh( false );
}

void RTPlotter::SetCameraPos( double x, double y, double z )
{
    m_cx = x;
    m_cy = y;
    m_cz = z;

    if ( m_parent != NULL )
        m_parent->Refresh( false );
}

void RTPlotter::RenderValues()
{
    double xval, yval;

    if ( m_coordSytem == CS_NONE )
        return;

    if ( m_mousepos.x < 0 || m_mousepos.y < 0 )
        return;

    if ( GetXYValue( m_mousepos.x, m_mousepos.y, xval, yval ) )
    {
        wxString str;
        switch ( m_coordSytem )
        {
            case CS_POLAR: str = wxString::Format( _T("r = %.3g, θ = %.1f°"), xval, yval );
                            break;

            case CS_CARTHESIAN:
            default:
                if ( m_drawXValues && m_drawYValues )
                    str = wxString::Format( _T("x = ") + m_xvalFormat + _T(", y = "), + m_yvalFormat, xval, yval );
                else if ( m_drawXValues )
                    str = wxString::Format( _T("x = ") + m_xvalFormat, xval );
                else if ( m_drawYValues )
                    str = wxString::Format( _T("y = ") + m_yvalFormat, yval );

                if ( m_selectFrom >= 0 && m_selectTo > m_selectFrom )
                    str += wxString::Format( _T("\nSelection:") + m_xvalFormat, (m_selectTo - m_selectFrom ) );

                if ( m_extraText.IsEmpty() == false )
                    str += _T("\n") + m_extraText;
                break;
        }

        if ( str == wxEmptyString )
            return;

        wxRealRect r = m_aggcanvas->getTTFTextBound( str, 0 );

        wxColor col = wxColor( 255, 255, 59, 143 );
        m_aggcanvas->getBrush()->SetStyle( wxAGGBRUSH_SOLID );
        m_aggcanvas->setBrush( col );
        m_aggcanvas->setPen( wxColor(0,0,0) );

        double x,y,w,h;

        double t = ( m_rectSize.w / m_hor_div ) * 2;
        w = (r.w + 2 > t) ? ( r.w + 2 ) : t;
        h = r.h + 2;

        y = m_rectSize.y + 4;
        x = m_rectSize.x + m_rectSize.w - w - 4;

        m_aggcanvas->drawRoundedRectangle( x, y, w, h, 1.0 );
        m_aggcanvas->drawTTFText( x + 2, y  + m_textHeight/2, str );
    }
}

void RTPlotter::DrawPiechart( DoubleArray& values, const wxColourArr& colors, const wxArrayString& names )
{
    m_displayValues = false;

    m_coordSytem = CS_NONE;

    unsigned valuesCnt = values.GetCount();
    unsigned namesCnt = names.GetCount();
    unsigned colorsCnt = colors.GetCount();

    if ( colorsCnt < valuesCnt )
        return;

    wxArrayString legend;
    if ( namesCnt > 0 )
    {
        legend = names;
    }
    for ( size_t i = namesCnt; i < valuesCnt; ++i )
        legend.Add( wxString::Format(_T("Data %d"), i+1) );

    if ( valuesCnt < 1 )
        return;

    if ( m_width == 0 || m_height == 0 )
        return;

    SetViewPortLimits( 0, m_width-1, 0, m_height-1 );

    double total = 0;

    for ( size_t i = 0; i < valuesCnt; ++i )
        total += values[i];

    CreateCanvas();

    // create shadow
    wxColourArr shadowCol;
    for ( size_t i = 0; i < colorsCnt; ++i )
    {
        unsigned r = colors[i].Red(); if ( r > 100 ) r -= 100; else r = 0;
        unsigned g = colors[i].Green(); if ( g > 100 ) g -= 100; else g = 0;
        unsigned b = colors[i].Blue(); if ( b > 100 ) b -= 100; else b = 0;
        wxColour col = wxColor( r, g, b );
        shadowCol.Add( col );
    }
    //
    double startA = 0 * M_PI / 180.0;

    double* start = new double[valuesCnt], *end = new double[valuesCnt];
    double val = 0;
    for ( size_t i = 0; i < valuesCnt; ++i )
    {
        val += values[i];
        double endA = floor((val/total)*360.0);
        start[i] = M_PI * startA / 180.0;
        end[i] = M_PI * endA / 180.0;

        startA = endA;
    }

    double centerx = m_width / 2, centery = m_height / 2;
    double dX = (m_width)/3.5, dY = (m_height)/3.5;
    double shadowHeight = 15.0;

    for ( size_t i = (size_t) (centery + shadowHeight); i > (size_t)centery; --i )
    {
        for ( unsigned j = 0; j < valuesCnt; ++j )
        {
            if ( start[j] != end[j] )
            {
                m_aggcanvas->getBrush()->SetStyle( wxAGGBRUSH_SOLID );
                m_aggcanvas->setBrush( shadowCol[j] );
                m_aggcanvas->setPen( shadowCol[j] );
                m_aggcanvas->drawEllipse( centerx, i, dX, dY, start[j], end[j]-start[j] );
            }
        }
    }

    for ( unsigned j = 0; j < valuesCnt; ++j )
    {
        if ( start[j] != end[j] )
        {
            m_aggcanvas->getBrush()->SetStyle( wxAGGBRUSH_SOLID );
            m_aggcanvas->setBrush( colors[j] );
            m_aggcanvas->setPen( colors[j] );
            m_aggcanvas->drawEllipse( centerx, centery, dX, dY, start[j], end[j]-start[j]);
        }
    }

    delete[] start; delete[] end;

    // draw legend
    m_aggcanvas->setBrush( wxColor( 250, 250, 250, 250) );
    m_aggcanvas->setPen( wxColor(0,0,0) );

    double tw = 0;
    double  ofh = m_aggcanvas->getFontHeight();
    m_aggcanvas->setFontHeight( ofh * 1.2 );
    for ( size_t i = 0; i < legend.GetCount(); ++i )
    {
        legend[i] += wxString::Format(_T("(%.3f, %.2f %%)"), values[i], (100.0*values[i]/total));
        double w = m_aggcanvas->getTTFTextBound( legend[i], 0 ).w;
        if ( w > tw ) tw = w;
    }
    tw *= 1.3;

    m_aggcanvas->setPenWidth( 1.2 );
    m_aggcanvas->drawRoundedRectangle( 5, 5, tw, (m_textHeight+2.0) * legend.GetCount(), 5 );

    for ( size_t i = 0; i < legend.GetCount(); ++i )
    {
        m_aggcanvas->setPen( colors[i] );
        m_aggcanvas->drawTTFText( 8, 5 + (m_textHeight)*(i+1), legend[i] );
    }
    m_aggcanvas->setFontHeight( ofh );

    // draw title
    m_aggcanvas->setPen( m_linecolor );
    m_aggcanvas->setFontHeight( 14 );
    double w = m_aggcanvas->getTTFTextBound( m_plottitle, 0 ).w;
    m_aggcanvas->drawTTFText( ( m_width - w )/2, m_textHeight, m_plottitle );
    m_aggcanvas->setFontHeight( m_textHeight/2 );

    Render();
}

void RTPlotter::DrawPolarPlot( dblFunc polarFunc, double tmin, double tmax, unsigned points )
{
    m_coordSytem = CS_POLAR;

    double xmin = 9E25, xmax = -9E25;
    double ymin = 9E25, ymax = -9E25;

    if ( points < 1 )
        return;

    double* x = new double[points];
    double* y = new double[points];

    double step = (tmax - tmin) / ( points - 1 );

    for ( unsigned i = 0; i < points; ++i )
    {
        double th = tmin + i * step;
        double r = polarFunc( th );

        x[i] = r * cos( th );
        y[i] = r * sin( th );
        if ( x[i] < xmin ) xmin = x[i];
        if ( y[i] < ymin ) ymin = y[i];
        if ( x[i] > xmax ) xmax = x[i];
        if ( y[i] > ymax ) ymax = y[i];
    }

    double xw = (fabs(xmin) > fabs(xmax)) ? fabs(xmin) : fabs(xmax);
    double yw = (fabs(ymin) > fabs(ymax)) ? fabs(ymin) : fabs(ymax);

    double w = ( xw > yw ) ? xw : yw;

    SetViewPortLimits( -w, w, -w, w );
    CreateCanvas();

    GetRectSize();

    // create polar coordinate system
    m_aggcanvas->getBrush()->SetStyle( wxAGGBRUSH_SOLID );
    m_aggcanvas->setBrush( m_bgcolor );
    m_aggcanvas->setPen( m_linecolor );
    m_aggcanvas->setPenWidth( m_linewidth );
    m_aggcanvas->drawRectangle( m_rectSize.x, m_rectSize.y, m_rectSize.w, m_rectSize.h );

    m_aggcanvas->setPen( wxColor(200,200,200) );

    double pattern[2]; pattern[0] = 3.0; pattern[1] = 3.0;

    double posXA = -1, posYA = -1;

    // draw abscissa & ordinate
    m_aggcanvas->setPenWidth( 1.0 );
    if ( m_maxx * m_minx < 0 )
    {
        posYA = fabs(m_minx) / ( m_maxx - m_minx ) * m_rectSize.w + m_rectSize.x;
        m_aggcanvas->drawLine( posYA, m_rectSize.y, posYA, m_rectSize.y + m_rectSize.h - 1 );
    }
    if ( m_maxy * m_miny < 0 )
    {
        posXA = fabs(m_maxy) / ( m_maxy - m_miny ) * m_rectSize.h + m_rectSize.y;
        m_aggcanvas->setPen( wxColor(200,200,200) );
        m_aggcanvas->drawLine( m_rectSize.x, posXA, m_rectSize.x + m_rectSize.w - 1, posXA );
    }

    step = m_rectSize.w / m_hor_div;

    double w2 = m_rectSize.w/2.0, h2 = m_rectSize.h / 2.0;
    double radmin = min( w2, h2 );

    double rad = step;

    m_aggcanvas->setPen( wxColor(180,180,180) );

    wxRect r( (int)m_rectSize.x, (int)m_rectSize.y, (int)m_rectSize.w, (int)m_rectSize.h );

    m_aggcanvas->setClippingRect( r );

    unsigned ctr = 1;

    float textrot = 15.0 / 180.0 * M_PI;

    m_aggcanvas->setPenWidth( 1.0 );
    while ( rad <= w2 + step|| rad <= h2 + step )
    {
        m_aggcanvas->drawCircle( posYA, posXA, rad );

        double r = m_maxx / m_hor_div * 2 * ctr++;

        wxString str = wxString::Format( _T("%.2g"), r );
        double tw = m_aggcanvas->getTTFTextBound( str, 15 ).w;
        m_aggcanvas->drawTTFText( posYA + rad * cos( textrot ) - tw / 2, posXA + rad * sin( textrot ), str, 15.0 );

        rad += step;
    }
    double radmax = rad - step;

    m_aggcanvas->saveState();

    for ( int j = 0; j < 360; j += 30 )
    {
        wxString str = wxString::Format(_T("%d°"), j);

        double xe = radmax * cos( j * M_PI / 180.0 );
        double ye = radmax * sin( j * M_PI / 180.0 );

        double xe1 = radmin * cos( j * M_PI / 180.0 );
        double ye1 = radmin * sin( j * M_PI / 180.0 );

        m_aggcanvas->drawTTFText( posYA + xe1, posXA - ye1, str );

        if ( j % 90 == 0 )
            continue;
        m_aggcanvas->getPen()->SetStyle( wxAGGPEN_SHORT_DASH );
        m_aggcanvas->drawLine( posYA, posXA, posYA + xe, posXA - ye );
    }
    m_aggcanvas->restoreState();

    // draw the curve
    m_aggcanvas->setPen( m_linecolor );

    RealPointArray curvepoints;
    for ( size_t i = 0; i < points; ++i )
    {
        wxRealPoint p;
        p.x = ( x[i] - m_minx ) / ( m_maxx - m_minx ) * m_rectSize.w + m_rectSize.x;
        p.y = posXA + ( y[i] ) / ( m_maxy - m_miny ) * m_rectSize.w;

        curvepoints.Add( p );
    }

    m_aggcanvas->getPen()->SetStyle( wxAGGPEN_SOLID );
    m_aggcanvas->setPenWidth( m_linewidth );
    m_aggcanvas->drawLine( curvepoints );

    m_aggcanvas->clearClipping();

    // draw labels
    if ( m_xaxislabel != wxEmptyString )
    {
        double xw = (m_aggcanvas->getTTFTextBound( m_xaxislabel, 0 )).w;
        m_aggcanvas->drawTTFText( m_rectSize.x + (m_rectSize.w - xw) / 2, m_rectSize.y + m_rectSize.h + m_textHeight, m_xaxislabel );
    }

    if ( m_yaxislabel != wxEmptyString )
    {
        double yh = (m_aggcanvas->getTTFTextBound( m_yaxislabel, 270)).h;
        m_aggcanvas->drawTTFText( m_textHeight/2, m_rectSize.y + ( m_rectSize.h - yh) / 2, m_yaxislabel, 270 );
    }

    // draw title
    if ( m_plottitle != wxEmptyString )
    {
        double tw = (m_aggcanvas->getTTFTextBound( m_plottitle, 0)).w;
        m_aggcanvas->drawTTFText( m_rectSize.x + ( m_rectSize.w - tw ) / 2, m_textHeight, m_plottitle );
    }

    delete[] x;
    delete[] y;

    Render();
}

void RTPlotter::Render3DFunc(dbl3dFunc func, double xmin, double xmax, double ymin, double ymax )
{
    m_coordSytem = CS_3D;

    CreateCanvas();
    GetRectSize();
    m_cz = 40;
    SurfaceRenderer rend(m_cx, m_cy, m_cz, 0, 0, m_width, m_height);
    rend.SetFunction( func );
    rend.SetLimits( xmin, ymin, xmax, ymax );
    rend.RenderSurface( m_aggcanvas );

    Render();
}

void RTPlotter::DrawText( const wxString& text, unsigned int x, unsigned int y, double fontSize )
{

    if ( x < 0 || y < 0 || x >= m_width || y >= m_height )
    {
        return;
    }

    if ( fontSize > 0 )
    {
        double oldFS = m_aggcanvas->getFontHeight();
        m_aggcanvas->setFontHeight( fontSize );
        m_aggcanvas->drawTTFText( x, y, text );
        m_aggcanvas->setFontHeight( oldFS );
    }
    else
    {
        m_aggcanvas->drawTTFText( x, y, text );
    }
}

/// 3D plotting

//
SurfaceRenderer::SurfaceRenderer( double cX, double cY, double cZ, int xs0, int ys0, int w, int h )
{
    m_density = 0.5;
    m_penColor = wxColor(0,0,0);
    calcTransfCoeff( cX, cY, cZ, xs0, ys0, w, h );
    m_func = NULL;
}

void SurfaceRenderer::SetLimits( wxRealPoint bottomLeft, wxRealPoint topRight )
{
    m_startPoint = bottomLeft;
    m_endPoint = topRight;
}

void SurfaceRenderer::SetLimits( double xmin, double ymin, double xmax, double ymax )
{
    m_startPoint = wxRealPoint( xmin, ymin );
    m_endPoint = wxRealPoint( xmax, ymax );
}

void SurfaceRenderer::SetFunction( dbl3dFunc func )
{
    m_func = func;
}

void SurfaceRenderer::SetDensity( double density )
{
    m_density = density;
}

void SurfaceRenderer::RenderSurface( wxAgg2* canvas )
{
    if ( m_func == NULL || canvas == NULL )
        return;

    // create brushes
    wxColourArr brushes;

    for ( size_t i = 0; i < 16*4; ++i )
    {
        unsigned char r,g,b;
        r = 255; g = ( i > 10 ) ? ( i * 4 + 1 ) : 4*i; b = 0;
        brushes.Add( wxColor( r, g, b ) );
    }

    double z1, z2;
    double xi = m_startPoint.x, yi;
    double  minZ = std::numeric_limits<double>::max(),
            maxZ = - std::numeric_limits<double>::max();

    int cols = (int)( ( ( m_endPoint.x - m_startPoint.x ) / m_density + 1 ) ),
        rows = (int)( ( ( m_endPoint.y - m_startPoint.y ) / m_density + 1 ) );

    double* mesh = new double[cols*rows];

    wxRealPoint* meshF = new wxRealPoint[cols*rows];

    for ( int x = 0; x < cols; x++ )
    {
        yi = m_startPoint.y;
        for ( int y = 0; y < rows; y++ )
        {
            double zz = m_func( xi, yi );
            mesh[x + cols * y] = zz;
            meshF[x + cols * y] = projectPoint( xi, yi, zz );
            yi += m_density;

            if ( minZ > zz ) minZ = zz;
            if ( maxZ < zz ) maxZ = zz;
        }

        xi += m_density;
    }

    double cc = ( maxZ - minZ ) / ( 16 * 4 - 1.0 );
    if ( cc == 0 ) cc = 1;

    for ( int x = 0; x < cols - 1; x++ )
    {
        for ( int y = 0; y < rows - 1; y++ )
        {
            z1 = mesh[x + y * cols];
            z2 = mesh[x +  (y + 1)*cols];
            RealPointArray arr;
            arr.Add( meshF[x + y*cols] );
            arr.Add( meshF[x + (y + 1)*cols] );
            arr.Add( meshF[x + 1 + (y + 1)*cols] );
            arr.Add( meshF[x + 1 + y*cols] );

            canvas->setBrush( brushes[(int)( ( ( z1 + z2 ) / 2.0 - minZ ) / cc)] );
            canvas->setPen( m_penColor );

            canvas->drawPolygon( arr );
        }
    }

    delete[] mesh; delete[] meshF;
}


void SurfaceRenderer::calcTransfCoeff( double cX, double cY, double cZ, int xs0, int ys0, int w, int h )
{
    double r1, a;

    double wP = w * 0.0257 / 72.0, hP = h * 0.0257 / 72.0;

    r1 = cX * cX + cY * cY; a = sqrtf( r1 ); m_r = sqrtf( r1 + cZ * cZ );

    if ( a != 0 ) { m_cf = cX / a; m_sf = cY / a; }
    else { m_cf = 1; m_sf = 0; }

    m_st = a / m_r;  m_ct = cZ / m_r;

    m_a = w / wP;  m_b = xs0 + m_a * wP / 2.0;
    m_c = -(double)h / hP; m_d = ys0 - m_c * hP / 2.0;

    m_sD = 0.5;
}

wxRealPoint SurfaceRenderer::projectPoint( double x, double y, double z )
{
    double xn, yn, zn;

    xn = -m_sf * x + m_cf * y;
    yn = -m_cf * m_ct * x - m_sf * m_ct * y + m_st * z;
    zn = -m_cf * m_st * x - m_sf * m_st * y - m_ct * z + m_r;

    if (zn == 0) zn = 0.01;

    return wxRealPoint( (float)( m_a * xn * m_sD / zn + m_b ), (float)( m_c * yn * m_sD / zn + m_d ) );
}

wxColour SurfaceRenderer::calcGradient( unsigned i )
{
    /// TODO, if neccessary
    return wxNullColour;
}

