#ifndef TABLEPANEL_H
#define TABLEPANEL_H

#include <wx/wx.h>
#include "../common/VemsMatrix.h"
#include "../common/types/DoubleArray.h"
#include "RTDataStatHelper.h"

class TablePanelBase : public wxPanel
{
    public:
        TablePanelBase( wxWindow* parent, int width, int height, const wxArrayString& xbins, const wxArrayString& ybins, bool showHeader );
        virtual ~TablePanelBase();

    public:
        void        setCustomColors( DoubleArray limits, DoubleArray colors );
        void        resetColorGradients();
        void        setBgColor( wxColor color ) { m_bgColor = color; }
        wxColor     GetCellColor( double value );

        void        setMinCount( const size_t& count ) { m_mincount = count; }

        void        setMatrix( VemsMatrix<double>* matrix );
        VemsMatrix<double>* getMatrix();

        void        setCountMatrix( VemsMatrix<double>* matrix );
        VemsMatrix<double>* getCountMatrix();

        void        setNumberFormat( const wxString& numformat ) { m_numformat = numformat; }

        void        setShowValues( bool show ) { m_showValues = show; }

    protected:
        bool                m_showValues;
        size_t              m_mincount;
        wxString            m_numformat;
        size_t              m_width;
        size_t              m_height;
        size_t              m_rows;
        size_t              m_cols;
        bool                m_showHeader;
        wxArrayString       m_xbins;
        wxArrayString       m_ybins;
        VemsMatrix<double>* m_matrix;
        VemsMatrix<double>* m_countmatrix;

        int                 m_cellw, m_cellh;
        wxRect              m_cellsize;
        wxColor             m_bgColor;

        GradientColorArray  m_gradientColorArray;

        // colors
        wxColor             m_cellTextColor;
        wxColor             m_3dinnerColor;
        wxColor             m_3douterColor;
        wxColor             m_labelBkColor;

        DoubleArray         m_colorlimits;
        DoubleArray         m_customcolorsA;
        bool                m_customcolors;

        double              m_tblmin, m_tblmax;
        bool                m_calculated;

    protected:
        virtual void    onPaint( wxPaintEvent& event );
        virtual void    onSize( wxSizeEvent& event );

        virtual void    onLeftClick( wxMouseEvent& event ) = 0;
        virtual void    onRightClick( wxMouseEvent& event ) = 0;
        virtual void    onMouse( wxMouseEvent& event ) = 0;
        virtual void    onKeyDown( wxKeyEvent& event ) = 0;

        void            Draw3DBorder( wxDC& dc, const wxRect& rc );
        void            Draw3DCells(wxDC& dc);
        virtual void    DrawData(wxDC& dc);
        virtual int     CalcFontSize( wxDC& dc, wxRect rect, const wxString& str, wxFont font );
        virtual int     CalcCellFontSize( wxDC& dc );
        virtual void    GetTblMinMax( wxDC& dc );
        virtual inline wxRect  GetCellRect( int row, int col );
        virtual void    resizeTo( int width, int height );

        wxColor         dbl2Color( double rgb );
        wxColor         calcCustomColorCellVal( double value );


    private:

        DECLARE_EVENT_TABLE()
};

class TablePanel : public TablePanelBase
{
    public:
        TablePanel( wxWindow* parent, int width, int height, const wxArrayString& xbins, const wxArrayString& ybins, bool showHeader );
        virtual ~TablePanel();

    protected:
        void    onLeftClick( wxMouseEvent& event );
        void    onRightClick( wxMouseEvent& event );
        void    onMouse( wxMouseEvent& event );
        void    onKeyDown( wxKeyEvent& event );

    private:
        DECLARE_EVENT_TABLE()
};

#endif // TABLEPANEL_H
