#include "serialconfig.h"

#include <wx/wx.h>
#include "UnicodeHelper.h"
#include "logger.h"

IMPLEMENT_DYNAMIC_CLASS(SerialConfig, wxObject)

SerialConfig::SerialConfig()
{
    m_portNameC = NULL;
    m_baudRate = 0;
}

SerialConfig::~SerialConfig()
{
	delete m_portNameC;
	m_portNameC = NULL;
}

SerialConfig::SerialConfig( const SerialConfig& config )
{
	m_portNameC = NULL;

    setPortName( config.getPortName() );
    setBaudRate( config.getBaudRate() );
    setEnabled( config.isEnabled() );
}

SerialConfig& SerialConfig::operator=( const SerialConfig& config )
{
    setPortName( config.getPortName() );
    setBaudRate( config.getBaudRate() );
    setEnabled( config.isEnabled() );

    return *this;
}

wxString SerialConfig::getPortName() const
{
    return m_portName;
}

const char*
SerialConfig::getPortNameC()
{
    return m_portNameC;
}



void SerialConfig::setPortName(const wxString& portName)
{
    m_portName = portName;
#ifdef __WXMSW__
    wxString tmpS = portName.Lower();
#else
    wxString tmpS = m_portName;
#endif
    if ( tmpS.StartsWith( _T("com1") ) )
    {
        if ( tmpS.IsSameAs(_T("com1"),false) == false )
        {
            tmpS = _T("\\\\.\\") + tmpS;
        }
    }

    //if(m_portNameC != NULL) delete m_portNameC;
		if(m_portNameC != NULL){
		     delete m_portNameC;
		     m_portNameC = NULL;
		}
        m_portNameC = UnicodeHelper::U2C(tmpS);
}

long SerialConfig::getBaudRate() const
{
    return m_baudRate;
}

wxString SerialConfig::getBaudRateStr() const
{
    return wxString::Format(_T("%i"), m_baudRate);
}

void SerialConfig::setBaudRate(long baudRate)
{
    m_baudRate = baudRate;
}
