#ifndef RESULTHISTOGRAM_H
#define RESULTHISTOGRAM_H

#include "../common/typedef.h"
#include "../common/types/DoubleHash.h"
/*
Histogram example:

ResultHistogram( 2, 10, 30);

The 10-30 range will be split to 2 equal parts: 10-20 and 20-30.
2 extra bins will be created (2+2=4 in this example), for the value<minValue  and  value>maxValue
value  0-10      10-20    20-30    30-inf
idx      0         1         2       3

Values are uint16_t so negative does not really make sense.
idx is 0 .. m_len+1
histogram[0] and histogram[m_len+1] are "collector" bins:
histogram[0] is used for value < m_minValue
histogram[m_len+1] is used for value > m_maxValue

*/

class ResultHistogram
{
    public:
        ResultHistogram(uint16_t histogram_elements = 21, uint16_t minValue = 6, uint16_t maxValue = 48 );
        virtual ~ResultHistogram();
        uint32_t getSum();
        uint32_t vectorMult( uint16_t mult ); // multiplies all bins with mult >> 16
        uint16_t getIdx(uint16_t value); // returns idx of the bin where value belongs
        void increment(uint16_t value); // adds +0x1000 to the bin where value belongs
        uint16_t getBin(uint16_t idx); // threshold for bin
        void vectorPrint();
        void vectorPrint(DoubleHash *tableH);
    protected:
    private:
        uint16_t m_len;
        // histogram[0] is used for value < m_minValue
        uint32_t * m_histogram; // m_len + 2 bins total. so max idx is mlen+1
        uint16_t m_minValue;
        uint16_t m_maxValue;
        uint32_t m_interval; // interval between bins (m_minvalue - m_maxvalue) / m_len   *65536

        uint32_t m_sum; // sum of histogram[i] >> 16
};

#endif // RESULTHISTOGRAM_H
