#ifndef SIGNALPROCESSTHREAD_H
#define SIGNALPROCESSTHREAD_H

#include <wx/file.h>

//#include "../ringbuffer/RingBuffer.h"
#include "TriggerDetect.h"
#include "ResultHistogram.h"
#include "../vemsprotocol/Sipr.h"


class RingBuffer;
class RTDataMgrBase;
class RTSignalViewerBase;
class Histogram;

//WX_DEFINE_ARRAY( RTSignalViewerBase*, SignalViewers);

#define SUBARU_CAMSYNC  1
#define LANCIA_COSWORTH  2

class SignalProcessThread
{
    public:
        SignalProcessThread( RingBuffer *rbp, RTDataMgrBase *mgr, DoubleHash* viewerModel );
        virtual ~SignalProcessThread();

        //void    setRTDataMgrBase(RTDataMgrBase *mgr);
        //void*   Entry();
        uint16_t getRPM();
        void processBlock(short * sampleblock, int samplePerChannel, int channel);
        void processPulseTrainDataBlock(SIPRPacket pulsePacket);
        void setTriggerType(const wxString& type);
        TriggerDetect * getSparkPeaks()
        {
            return &sparkPeaks;
        }

        ResultHistogram *getSparkPhaseHistogram()
        {
                return sparkPhaseHistogram;
        }

    private:

        uint8_t m_pinstateold;//from pulsetrain
        uint8_t m_seqPulsetrainOld;//from pulsetrain

        uint32_t m_sampleIdxold;
        uint32_t m_sampleIdx; // permanent index; not zeroed for every processBlock(), +=1 for every sample(pair: L+R)
        uint32_t m_SecTrigPulse;
        uint32_t m_SecTrigPulse2;
        uint32_t m_PrevSecTrigPulse;
        uint32_t m_SecTrigPeriodSampleCount;
        uint32_t m_PrevSecTrigPeriodSampleCount;
        uint32_t m_SectrigPulseCount;   // counts sectrig pulses

        uint32_t m_DataFallingEdge;
        uint32_t m_DataFallingEdgeold;
        uint32_t m_DataRisingEdge;
        uint32_t m_DataFallingEdge2;
        uint32_t m_DataFallingEdgeold2;
        uint32_t m_DataRisingEdge2;

         // need to remember these ? will be added to histograms anyway....
        uint16_t m_dwellPhase;  // samplecount (not degrees)
        uint16_t m_dwellPhase2;  // samplecount (not degrees)
        uint16_t m_dwellLenSamples;
        uint16_t m_dwellLenSamples2;
        uint16_t m_SparkPhase;  // samplecount (not degrees)
        uint16_t m_SparkPhase2;  // samplecount (not degrees)ch2
        uint32_t m_SparkCount;  // counts detected sparks

        // helper variables:
        uint32_t m_rpm_constant; // (720 << 12 ) / m_SecTrigPeriodSampleCount  (only calculated when necessary)

        uint16_t m_primValueBeforeFallingedge;
        uint16_t m_prevsampleaverage;
        uint32_t test;
        wxString writeData;


        int16_t m_buffer[10];

        bool m_isRisingEdge;
        bool m_isRisingEdge2;

        uint8_t m_triggerType;
        uint16_t m_ShortGapCount;
        uint32_t m_PrevSecTrigPulseShort;
        uint32_t m_SecTrigPulse2Short;
        uint32_t m_SecTrigPeriodSampleCountShort;
        uint32_t m_PrevSecTrigPeriodSampleCountShort;

        uint16_t m_prevSpark;
        uint16_t m_prevSpark2;

        uint16_t m_prevDwell;
        uint16_t m_prevDwell2;

        RingBuffer*     m_ringBuffer;
        RTDataMgrBase*  m_mgrBase;

        // trigger detect for input wav signals
        TriggerDetect primtrig;
        TriggerDetect localprimtrig;
        TriggerDetect sectrig;

         // for the calculated dwell and spark we also calculate peak, so a gauge can display them
        TriggerDetect dwellPeaks;    // detect peaks and average of the results
        TriggerDetect sparkPeaks;    // detect peaks and average of the results

        Histogram *m_histo;

        ResultHistogram *sparkPhaseHistogram;
        ResultHistogram *dwellTimeHistogram;

        DoubleHash* m_viewerModel;

        //debug wave
        wxFile * m_debugWav;


        void        consumeRecordedBlocks();
        uint16_t    getSparkP();   // calculates spark phase (degrees) from m_rpm_constant
        uint16_t    getSparkP2();
        uint16_t    getDwellP();
        uint16_t    getDwellP2();

        uint32_t    circleOfLifeTest( void );
        uint32_t    otherDummyTest( void );
        void setSymbolsAtPrimeTrigger();
        void setSymbolsAtSecTrigger();
        void setSymbolSparkPeaks();
        void dataFallingEdge();
        void dataRisingEdge();
        void dataFallingEdge2();
        void dataRisingEdge2();
        void secTrigFallingEdge();
        void secTrigRisingEdge();

        //void NotifySignalViewers();
};

#endif // SIGNALPROCESSTHREAD_H
