#include "timingtester.h"

#include <wx/file.h>

#define WAV_SPS 96000

TimingTester::TimingTester():
 m_wavOffset (44)
,m_channelcount (2)
,m_samplesize (2)
,m_sampleTime(1/WAV_SPS)
{
    //ctor
#define SAMPLES_PER_BLOCK 256
// to get the idx relative to buf, use (absidx & SAMPLE_OFFSET_MASK)
#define SAMPLE_OFFSET_MASK (SAMPLES_PER_BLOCK-1)

// if 4 consequtive samples are of same nature, we consider that real (not noise)
#define SIGNAL_LENGTH 4

    channelState[LEFT] = LOW;
    channelState[RIGHT] = LOW;

#define THRESHOLD_AMPLITUDE 1000
#define refChannel RIGHT

    channelLowThr[LEFT] = 32768-THRESHOLD_AMPLITUDE;
    channelLowThr[RIGHT] = 32768-THRESHOLD_AMPLITUDE;

    channelHighThr[LEFT] = 32768+THRESHOLD_AMPLITUDE;
    channelHighThr[RIGHT] = 32768+THRESHOLD_AMPLITUDE;

    channelHighCnt[LEFT] = 0;
    channelHighCnt[RIGHT] = 0;
    channelLowCnt[LEFT] = 0;
    channelLowCnt[RIGHT] = 0;

    risingEdgeIndex[LEFT] = 0;
    risingEdgeIndex[RIGHT] = 0;

    fallingEdgeIndex[LEFT] = 0;
    fallingEdgeIndex[RIGHT] = 0;

    // must be absolute (-1 means not relevant)
    // highStartIdx[LEFT] = -1;
    // highStartIdx[RIGHT] = -1;
}

TimingTester::~TimingTester()
{
    //dtor
}

/** find edges and prints results from the whole buf

buf:
sampleStartIdx: relative to buf
channel
absidx: index absolute (from file-begin), needed for print. TODO: long
*/
int TimingTester::processSamples( char* buf, int sampleCnt,  int channel, int absidx )
{
    int i = absidx & SAMPLE_OFFSET_MASK;
    long blockIdx0 = absidx - i;

    int idx = 0;
    while( idx < sampleCnt ){
        //char * startBuf = buf + (sampleCount-sampleCnt);
        if( channelState[channel] == LOW ){ // looking for rising edge
            absidx = getRisingEdge(buf, blockIdx0 + idx, sampleCnt, channel);
            if(absidx > -1){
                idx = absidx & SAMPLE_OFFSET_MASK;
                if( channel == refChannel ){ // refchannel rising edge
                    rpm_period = absidx - risingEdgeIndex[refChannel];
                    rpm = (120 * WAV_SPS) / rpm_period ; // the number of camrotations in 120 seconds
                    fprintf(stderr, "---- refChannel RISING edge, RPM_period:%d  RPM_ref:%d\n", rpm_period, rpm);
                }
                fprintf(stderr, "Channel %d Rising edge found idx: %d\n",channel, absidx);
                risingEdgeIndex[channel] = absidx;
            }
        } else { // channelState[channel] == HIGH, so looking for falling edge
            absidx = getFallingEdge(buf, blockIdx0 + idx, sampleCnt, channel);
            if(absidx > -1){
                idx = absidx & SAMPLE_OFFSET_MASK;
                long samples_since = absidx - risingEdgeIndex[refChannel];
                int angle = (720 * samples_since) / rpm_period ;
                fprintf(stderr, "Channel %d Falling edge found idx: %d, angle: %d\n",channel, absidx, angle);
                fallingEdgeIndex[channel] = absidx;
            }
        }
        idx++;
    }
    return 0;
}

void TimingTester::init()
{
  /*
    WavHelper wh = new WavHelper();
    wh->OpenFile(_T("c:\\del.wav"));
    long sampelCount wh->getSampleCount();
    int value wh->getSample(long index, int channel);
    wh->close();
  */
    wxFile * wavFile = new wxFile();
    //wxChar * filename = _T("c:\\del.wav");
    wxChar * filename = _T("C:\\tmp\\f72\\firmware_1.0.72\\c001_180-1740_30_15_1000_1000.wav");

    bool res = wavFile->Open(filename,wxFile::read);
    if( ! res ){
            fprintf(stderr, "wavFile->Open() returned false, sorry...\n");
            return;
    }
    long fileLength = wavFile->Length();
    fprintf(stderr, "File Length: %d\n", fileLength);

    char buf[4096];
    int sampleCount = SAMPLES_PER_BLOCK; // 1024 bytes
    wavFile->Seek(m_wavOffset, wxFromStart); //file header
    fileLength -= m_wavOffset; // cheat, we neglect wav header

    /* TODO: debug... pretend that file is shorter */
    int TEST_IDX = 3990000;
    if( fileLength > TEST_IDX ) fileLength = TEST_IDX;

    bool bailout = false;

    long totalSampleCnt = fileLength / (m_channelcount * m_samplesize);
    long currSampleCnt = 0;
    fprintf(stderr, "Total Sample Count: %d\n", totalSampleCnt);

    while( currSampleCnt <= totalSampleCnt || bailout){
        // fprintf(stderr, "Curr Sample count: %d\n", currSampleCnt );
        int size;
            // Get SOME samples into the buffer
            int bufSize = sampleCount * (m_channelcount * m_samplesize);

                size = wavFile->Read(buf, bufSize);
                if( size < 0) { bailout = true; continue;}

                if( size != bufSize){
                    fprintf(stderr, "size=%d (while trying to read %d)\n", size, bufSize );
                    bailout = true;
                }

            sampleCount = size / (m_channelcount * m_samplesize);

            // Process ALL (samplesleft) samples from buf, from beginning:
            processSamples( buf, sampleCount, refChannel, currSampleCnt );
            // Process ALL (samplesright) samples from buf, from beginning:
            processSamples( buf, sampleCount, 1 ^ refChannel, currSampleCnt );

            currSampleCnt += sampleCount;
    }

     wavFile->Close();
}

/** channel : 0,1
* index: sample index
*/
uint16_t TimingTester::getSample(char * buf, int index, int channel)
{
        int offset = 0;

        int idx = offset + (index * m_channelcount + channel) * m_samplesize;

        uint8_t high = (uint8_t) buf[idx+1] ;
        uint8_t low =  (uint8_t) buf[idx];

        uint16_t sample =  (high<<8) + low + 32768;
        return sample;
}

/**
return absolute index of rising edge if found; else -1;
*/
long TimingTester::getRisingEdge(char *buf, long absidx, int sampleCnt, int channel)
{
    if(channelState[channel] != LOW) { return -1; }

    int highCount = channelHighCnt[channel]; // late writeback (before return)

    int i = absidx & SAMPLE_OFFSET_MASK;
    long blockIdx0 = absidx - i;
    for( ; i<sampleCnt; i++){
            int sampleValue = getSample( buf, i, channel);
            if( sampleValue < channelLowThr[channel] ){
                // we could compare against channelHighThr[channel]
                highCount = 0;
            } else if( sampleValue >= channelHighThr[channel] ){
                highCount++;
                if(highCount >= SIGNAL_LENGTH ){
                    channelState[channel] = HIGH;
                    channelLowCnt[channel] = 0;
                    channelHighCnt[channel] = highCount;
                    long highIndex = blockIdx0 + i;
                    return highIndex;
                }
            } // do nothing in the midlane
    }
    // save, even if less than SIGNAL_LENGTH
    channelHighCnt[channel] = highCount;
    return -1;
}

long TimingTester::getFallingEdge(char *buf, long absidx, int sampleCnt, int channel)
{
 if(channelState[channel] != HIGH) { return -1; }

    int lowCount = channelLowCnt[channel]; // late writeback (before return);
    int i = absidx & SAMPLE_OFFSET_MASK;
    long blockIdx0 = absidx - i;
    for( ; i<sampleCnt; i++){
            int sampleValue = getSample( buf, i, channel);
            if( sampleValue > channelHighThr[channel] ){
                lowCount = 0;
            } else if( sampleValue <= channelLowThr[channel] ){
                lowCount++;
                if(lowCount >= SIGNAL_LENGTH ){
                    channelState[channel] = LOW;
                    channelHighCnt[channel] = 0;
                    channelLowCnt[channel] = lowCount;
                    long lowIndex = blockIdx0 + i;
                    return lowIndex;
                }
            }
    }

  // save, even if less than SIGNAL_LENGTH
    channelLowCnt[channel] = lowCount;
    return -1;
}

    /*
uint16_t TimingTester::getTreshold(buf, buffSize, LEFT)
{
    for(int i=0; i<buffSize; i++){

        if(state == HIGH ){
                if( sample < LOW_TRESHOLD){


                }

        }
    }

}

*/
