#include "STDataMgrBase.h"
#include "STDataViewerBase.h"
#include "STDataModelBase.h"
#include "VemsEvent.h"
#include "../logger.h"
#include "../expression/RPNParser.h"


STDataMgrBase::STDataMgrBase() : wxEvtHandler()
{
    m_viewers       = NULL;
    m_tableViewers  = NULL;
    m_stDataModel   = NULL;
    m_deviceModel   = NULL;
    m_configValidator = NULL;

    m_isOnline          = false;
    m_isConnected       = false;
    m_isRefreshDeviceModelOnly = false;
}

STDataMgrBase::~STDataMgrBase()
{
    WX_CLEAR_HASH_MAP( StringToPosition, m_positionsOfTables );
}


void STDataMgrBase::RegisterViewer(STDataViewerBase * viewer)
{
    m_viewers->insert( viewer );

    viewer->SetDataMgr(this);
}

void STDataMgrBase::RegisterTableViewer( STDataViewerBase* viewer )
{
    m_tableViewers->insert( viewer );
    viewer->SetDataMgr(this);
}

void STDataMgrBase::UnRegisterViewer( STDataViewerBase * viewer )
{
    m_viewers->erase( viewer );
    m_tableViewers->erase( viewer );
}

void STDataMgrBase::UnregisterAll(bool del)
{
    //STDataViewerBase* viewer;
    //STDataViewerBaseList::const_iterator it = m_viewers->begin();
    //for(; it != m_viewers->end(); it++ )
    //{
    //    viewer = (*it);
    //    UnRegisterViewer( viewer );
    //    if(del) delete viewer;
    //}
	if ( del )
	{
	    WX_CLEAR_HASH_SET( STDataViewerBaseSet, *m_viewers );
		//WX_CLEAR_LIST( STDataViewerBaseList, *m_viewers );
	}
	else
	{
		m_viewers->clear();
		m_tableViewers->clear();
	}
}

void STDataMgrBase::refreshViewerModel()
{
    m_deviceModel->copyTo( m_stDataModel );
    clearBurningQueue();
    clearWritingQueue();
}

int STDataMgrBase::getRegisteredCount()
{
    return m_viewers->size();
}


bool STDataMgrBase::wasDataWritten( const Symbol& symbol, size_t row, size_t col )
{

    const double EPSILON = 0.000001;

    double viewerData = m_stDataModel->getViewerData( symbol, row, col );
    double deviceData = m_deviceModel->getViewerData( symbol, row, col );

    return fabs( viewerData - deviceData ) < EPSILON;
}


bool STDataMgrBase::wasDataWritten( const Symbol& symbol, size_t offset )
{

    const double EPSILON = 0.000001;

    double viewerData = m_stDataModel->getViewerDataByIndex( symbol, offset );
    double deviceData = m_deviceModel->getViewerDataByIndex( symbol, offset );

    return fabs( viewerData - deviceData ) < EPSILON;
}


bool STDataMgrBase::wasDataWritten( const wxString& symbolName, size_t offset )
{
    const double EPSILON = 0.000001;

    double viewerData = m_stDataModel->getViewerDataByIndex( symbolName, offset );
    double deviceData = m_deviceModel->getViewerDataByIndex( symbolName, offset );

    return fabs( viewerData - deviceData ) < EPSILON;
}


double STDataMgrBase::calculate( const wxString& expr )
{
    RPNParser rpn;
    rpn.setVariableData( m_stDataModel );
    double result = rpn.calculate( expr );
    return result;
}


double STDataMgrBase::getDataFromViewerModel( const Symbol& symbol, size_t row, size_t col )
{
    return m_stDataModel->getViewerData( symbol, row, col );
}


double STDataMgrBase::getDataFromDeviceModel( const Symbol& symbol, size_t row, size_t col )
{
    return m_deviceModel->getViewerData( symbol, row, col );
}


void STDataMgrBase::setTablePosition( const wxString& tableName, const wxPoint& position )
{
   if ( !m_positionsOfTables[tableName] ) { m_positionsOfTables[tableName] = new wxPoint(); }

    *m_positionsOfTables[tableName] = position;
}


wxPoint STDataMgrBase::getTablePosition( const wxString& tableName ) const
{
    StringToPosition::const_iterator it = m_positionsOfTables.find( tableName );

    if ( it == m_positionsOfTables.end() ) return wxPoint( -1, -1 );

    return *it->second;
}

void STDataMgrBase::updateTableViewers( STDataViewerBase* sender, const wxString& tableName, wxPoint position )
{


}

void STDataMgrBase::clearBurningQueue()
{
    StringToIntSetH::iterator it = m_waitingForBurn.begin();

    for ( ; it != m_waitingForBurn.end(); ++it )
    {
        IntSet* set = it->second;
        set->clear();
        delete set;
    }

    m_waitingForBurn.clear();
}

void STDataMgrBase::clearWritingQueue()
{
    StringToIntSetH::iterator it = m_waitingForWrite.begin();

    for ( ; it != m_waitingForWrite.end(); ++it )
    {
        IntSet* set = it->second;
        set->clear();
        delete set;
    }

    m_waitingForWrite.clear();
}
