#ifndef STDATAMODELBASE_H
#define STDATAMODELBASE_H

#include "../common/typedef.h"
#include "../common/VemsMatrix.h"
#include "../expression/VariableData.h"

#include <wx/string.h>

class SIPRPacket;
class Symbol;
class STDataIOBase;
class ModelFilter;

class STDataModelBase : public VariableData,  public wxObject
{
    DECLARE_ABSTRACT_CLASS(STDataModelBase)

    public:
        enum { USER, USER_FORCE_WRITE, ECU };
    public:
        STDataModelBase() {}
        virtual ~STDataModelBase() {}

        virtual void    setRawData( const wxString& symbolName, uint8_t value ) = 0;
        virtual void    setRawData( uint8_t value, int page, int offset, int source = USER ) = 0;
        virtual void    setRawData(const Symbol& alienSymbol, VemsMatrix<uint8_t>* table) = 0;

        virtual uint8_t getRawData( int page, int offset ) const = 0;
        virtual uint8_t getRawData( const wxString& symbolName ) = 0;
        virtual uint8_t getRawData( const Symbol& alienSymbol ) const = 0;
        virtual uint8_t* getRawBuffer(int pageNo) = 0;

        virtual void    process( const SIPRPacket& packet, const ModelFilter* filter = NULL ) = 0;
        virtual void    writeAllDirty( STDataIOBase* iob ) = 0;
        virtual void    readAllDirty( STDataIOBase* iob ) = 0;

        virtual bool    isPageDirty( int page ) const               = 0;
        virtual bool    isDataDirty( int page, int offset ) const   = 0;
        virtual bool    isPageValid( int page ) const  = 0;
        virtual int     getPageCount() const = 0;
        virtual int     getPageSize( int page ) const  = 0;
        virtual void    clear() = 0;

        virtual double  getViewerData( const wxString& symbolName, size_t, size_t col) =0;
        virtual double  getViewerData( const Symbol& symbol, size_t row = 0, size_t col = 0 ) = 0 ;
        virtual double  getViewerDataByIndex( const Symbol& symbol, size_t index = 0) = 0;
        virtual double  getViewerDataByIndex( const wxString& symbolName, size_t index = 0 ) = 0;
        virtual void    setModelData( const Symbol& symbol, size_t row, size_t col, double data ) = 0;
        virtual void    writeViewerData( const Symbol& symbol, STDataIOBase* iob, size_t row = 0, size_t col = 0 ) = 0;
        virtual void    setRawModelData( const Symbol& symbol, size_t row, size_t col, uint8_t data ) = 0;
        virtual uint8_t getRawModelData( const Symbol& symbol, size_t row, size_t col ) = 0;

        virtual void    copyPageTo( size_t page, STDataModelBase* target ) = 0;
        virtual void    copyTo( STDataModelBase* target ) = 0;
        virtual size_t  getDirtyCount() = 0;
};


#endif // STDATAMODELBASE_H
