#ifndef STDATAMODELSERIAL_H
#define STDATAMODELSERIAL_H

#include "STDataModelBase.h"
#include "ModelFilter.h"
#include "../common/VemsMatrix.h"
#include "../common/types/Int2CharRefHash.h"
#include "../expression/RPNParser.h"

class SymbolTable;
class SIPRPacket;
class SIPRDescriptor;
class STDataIOBase;

WX_DEFINE_ARRAY( IntSet* ,IntSetsArray );

class STDataModelSerial : public STDataModelBase
{
    public:
        STDataModelSerial( const SymbolTable& symbols, const SIPRDescriptor* siprDesc = NULL );
        virtual ~STDataModelSerial();

        void init();

        void    setRawData( const wxString& symbolName, uint8_t value );
        void    setRawData( uint8_t value, int page, int offset, int source = USER );          // set dirty too
        void    setRawData(const Symbol& symbol, VemsMatrix<uint8_t>* table);

        uint8_t getRawData( int page, int offset ) const;
        uint8_t getRawDataBak( int page, int offset ) const;
        uint8_t getRawData( const wxString& symbolName );
        uint8_t getRawData( const Symbol& alienSymbol ) const;
        uint8_t* getRawBuffer(int pageNo);

        bool                getValue( const wxString& name, double* value, IntSet* dirty = NULL );
        bool                getValue( const wxString& name, VemsMatrix<double>& data );
        void                process( const SIPRPacket& packet, const ModelFilter* filter = NULL );
        void                writeAllDirty( STDataIOBase* iob );
        void                readAllDirty( STDataIOBase* iob );

        bool                isPageDirty( int page ) const { return ( m_dirtyPages.find( page ) != m_dirtyPages.end() ); }
        bool                isDataDirty( int page, int offset ) const;
        bool                isPageValid( int page ) const;
        int                 getPageCount() const;
        int                 getPageSize( int page ) const;
        void                clear();

        double              getViewerData( const wxString& symbolName, size_t row, size_t col );
        double              getViewerData( const Symbol& symbol, size_t row = 0, size_t col = 0 );
        double              getViewerDataByIndex( const Symbol& symbol, size_t index = 0 );
        double              getViewerDataByIndex( const wxString& symbolName, size_t index = 0 );
        void                setModelData( const Symbol& symbol, size_t row, size_t col, double data );
        void                writeViewerData( const Symbol& symbol, STDataIOBase* iob, size_t row = 0, size_t col = 0 );
        void                setRawModelData( const Symbol& symbol, size_t row, size_t col, uint8_t data );
        uint8_t             getRawModelData( const Symbol& symbol, size_t row, size_t col );

        void                copyPageTo( size_t page, STDataModelBase* target );
        void                copyTo( STDataModelBase* target );
        size_t              getDirtyCount();

    private:
        Int2CharRefHash     m_pageBufferHash;

        IntSet              m_dirtyPages;               // contains pages that different from from ECU
        IntSetsArray        m_dirtyOffsets;             // contains dirty offsets by pages
        const SymbolTable&  m_symbolTable;
        const SIPRDescriptor*     m_pSIPRDescriptor;
        RPNParser           m_rpnForExpressionBasedSymbols;
        VariableDataDH      m_variablesForExpressionBasedSymbols;

    private:
        void                setBitSymbol( const Symbol& bitSymbol, double d );
        void                setScalarSymbol( const Symbol& scalarSymbol, double d );
        void                setArraySymbol( const Symbol& arraySymbol, size_t row, size_t col, double d );
        void                setExpressionSymbol( const Symbol& expressionSymbol, double d );

        wxString            getSourceAsString( int src );
        void                setScalarValue( const Symbol* scalar, double value );
        void                setScalarValue( const wxString& name, double value );
};


#endif // STDATAMODELSERIAL_H
