/** \class TbTbDevices
  *
  * \~english
  * \brief Show Devices connects state
  * \details
  *
  *
  * \~hungarian
  * \brief Eszközök kapcsolódási állapotait mutatja
  * \details
  *
  *
  */

#include "TbTbDevices.h"
#include "../DirHelper.h"
#include "../logger.h"
#include "../gui/ECUConfigPanel.h"
#include "../VemsTuneController.h"
#include "../events/VemsDeviceControllerEvent.h"

enum ToolBitmapIds
{
    TBI_DEV_V3,
    TBI_DEV_ARM,
    TBI_DEV_NONE,
    TBI_DEV_ROUND,
    TBI_DEV_NONERED
};

IMPLEMENT_DYNAMIC_CLASS(TbTbDevices,VemsToolBarToolBase)


BEGIN_EVENT_TABLE(TbTbDevices,VemsToolBarToolBase)
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_CANNOT_OPEN_PORT, TbTbDevices::OnCannotOpenPort )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_FOUND, TbTbDevices::OnDeviceConnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_NOT_FOUND, TbTbDevices::OnDeviceNotFound )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_CONNECTED, TbTbDevices::OnDeviceConnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_DISCONNECTED, TbTbDevices::OnDeviceDisconnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_RECONNECTED, TbTbDevices::OnDeviceConnected )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_STOPPED, TbTbDevices::OnDeviceControllerStopped )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_RESUMED, TbTbDevices::OnDeviceResumed )
    EVT_VEMS_DEVICE_CONTROLLER( VDC_EVENT_DEVICE_PROTECTED, TbTbDevices::OnDeviceConnected )
END_EVENT_TABLE()


TbTbDevices::TbTbDevices(int id)
        :VemsToolBarToolBase(id)
{
    m_bmpKeyListArray[TBI_DEV_V3].Add( DirHelper::GetPath(dICON) + _T("tbDeviceV3_48.png"));
    m_bmpKeyListArray[TBI_DEV_ARM].Add( DirHelper::GetPath(dICON) + _T("tbDeviceArm_48.png"));
    m_bmpKeyListArray[TBI_DEV_NONE].Add( DirHelper::GetPath(dICON) + _T("tbDeviceNone_48.png"));
    m_bmpKeyListArray[TBI_DEV_ROUND].Add( DirHelper::GetPath(dICON) + _T("tbDeviceRound_48.png"));
    m_bmpKeyListArray[TBI_DEV_NONERED].Add( DirHelper::GetPath(dICON) + _T("tbDeviceNoneRed_48.png"));

    m_vemscontroller = NULL;
}

TbTbDevices::~TbTbDevices()
{
}

void TbTbDevices::Init()
{
    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_DEV_NONE];

    AddRequest(TN_VemsController);

    m_shortHelpString = _T("Devices");
    m_longHelpString  = _T("Devices");
}

bool TbTbDevices::SendObject(int id, wxObject *object, const wxString &message)
{
    if ( id  == TN_VemsController ) {
        m_vemscontroller = dynamic_cast<VemsTuneController*> (object);
        if ( !m_vemscontroller ) {
            LOGP( _T("VemsTuneControlles dynamic_cast failed") );
        }
    }

    return true;
}

void TbTbDevices::OnToolR(wxCommandEvent &event)
{
    ECUConfigPanel::findAndShow( m_toolBar->GetParent(), m_vemscontroller );
}

void TbTbDevices::OnTool(wxCommandEvent &event)
{
    if ( !m_vemscontroller ) {
        return;
    }

    if ( m_vemscontroller->isOnline() ) {
        m_vemscontroller->setOffline();
    } else {
        m_vemscontroller->setOnline();
    }
}


void TbTbDevices::OnCannotOpenPort( VemsDeviceControllerEvent& event )
{
    SetToolBmp(m_bmpHash[TBI_DEV_NONE]);
    m_shortHelpString = _T("ECU Disabled!");

    InitHelpStrings();
}

void TbTbDevices::OnDeviceNotFound( VemsDeviceControllerEvent& event )
{
    SetToolBmp(m_bmpHash[TBI_DEV_NONERED]);
    m_shortHelpString = _T("Device: not found");

    InitHelpStrings();
}

void TbTbDevices::OnDeviceConnected( VemsDeviceControllerEvent& event )
{
    switch( event.getDevice() )
    {
        case ECUInfo::DEVICE_V3:
            SetToolBmp(m_bmpHash[TBI_DEV_V3]);
            m_shortHelpString = _T("Device: V3");
            break;

        case ECUInfo::DEVICE_ROUND:
            SetToolBmp(m_bmpHash[TBI_DEV_ROUND]);
            m_shortHelpString = _T("Device: Round");
            break;

        case ECUInfo::DEVICE_ARM:
            SetToolBmp(m_bmpHash[TBI_DEV_ARM]);
            m_shortHelpString = _T("Device: ARM");
            break;

        default:
            LOG
    }

    InitHelpStrings();
}

void TbTbDevices::OnDeviceDisconnected( VemsDeviceControllerEvent& event )
{
    SetToolBmp(m_bmpHash[TBI_DEV_NONERED]);
    m_shortHelpString = _T("Device: not found");

    InitHelpStrings();
}

void TbTbDevices::OnDeviceControllerStopped( VemsDeviceControllerEvent& event )
{
    SetToolBmp(m_bmpHash[TBI_DEV_NONE]);
    m_shortHelpString = _T("ECU Disabled!");

    InitHelpStrings();
}

void TbTbDevices::OnDeviceResumed( VemsDeviceControllerEvent& event )
{
    switch( event.getDevice() )
    {
        case ECUInfo::DEVICE_V3:
            SetToolBmp(m_bmpHash[TBI_DEV_V3]);
            m_shortHelpString = _T("Device: V3");
            break;

        case ECUInfo::DEVICE_ROUND:
            SetToolBmp(m_bmpHash[TBI_DEV_ROUND]);
            m_shortHelpString = _T("Device: Round");
            break;

        case ECUInfo::DEVICE_ARM:
            SetToolBmp(m_bmpHash[TBI_DEV_ARM]);
            m_shortHelpString = _T("Device: ARM");
            break;

        default:
            LOG
    }

    InitHelpStrings();
}
