#include "TbTbGaugeGroup.h"

#include "../DirHelper.h"
#include "../DrawHelper.h"
#include "../logger.h"
#include "../vemsgauge/GaugeGroupManager.h"
#include "../FrameLayoutHandler.h"

#include "../common/FunctionMeter.h"

#include <wx/regex.h>

/** \class TbTbGaugeGroup
  *
  * \~english
  * \brief
  * \details
  *
  *
  * \~hungarian
  * \brief
  * \details
  *
  *
  */

enum ToolBitmapIds
{

    TBI_GG_ON,
    TBI_GG_OFF
};

IMPLEMENT_DYNAMIC_CLASS(TbTbGaugeGroup,VemsToolBarToolBase)

TbTbGaugeGroup::TbTbGaugeGroup(int id, FrameLayoutHandler* handler)
    :VemsToolBarToolBase(id)
    ,m_groupIDtmp(-1)
    ,m_gaugeGroupManager(NULL)
{

    m_bmpKeyListArray[TBI_GG_ON].Add( DirHelper::GetPath(dICON) + _T("tbGaugeGroup48-on.png"));
    m_bmpKeyListArray[TBI_GG_OFF].Add( DirHelper::GetPath(dICON) + _T("tbGaugeGroup48-off.png"));

    m_popupMenu = new wxMenu();

    //m_popupMenu->Append
    m_handler = handler;
}

TbTbGaugeGroup::~TbTbGaugeGroup()
{
    //dtor
}

void TbTbGaugeGroup::SetGroup(const GaugeGroupDesc& desc)
{
    if( !m_gaugeGroupManager ) return;

    bool isShowned = false;

    if( m_ToolBmpId == TBI_GG_ON ) {
        m_gaugeGroupManager->hideGroup( m_groupInfo.GetName() );
        isShowned = true;
    }

    m_groupInfo = desc;

    InitBmpRes();

    DrawIconText();

    if( isShowned ){
        m_gaugeGroupManager->showGroup( m_groupInfo.GetName() );
    } else {
        SetToolBmp( TBI_GG_OFF );
    }

    ApplyLabel();
    InitHelpStrings();

}

void TbTbGaugeGroup::ChangeGroup(const GaugeGroupDesc& desc)
{
    SetGroup(desc);

    if(desc.GetId() > 0 && m_handler)
        m_handler->SetToolBarIconParam(wxString::Format(_T("%i"),desc.GetId()), m_itemId);

}

void TbTbGaugeGroup::OnMenu(wxCommandEvent &evt)
{

    int id = evt.GetId();

    if( id == idMenuAddNew ){

        Logger::logf(LOG_MSG,_T("TbTbGaugeGroup::OnMenu add"));

        VemsToolBarToolBase * tbt = m_handler->AddNewToolBarTool( m_toolBar, _T("Gauge Groups"), _T("GaugeGroup"), -1, _T("Main"));

        if(tbt->GetClassInfo()->GetClassName() == _T("TbTbGaugeGroup") )
            ((TbTbGaugeGroup*)tbt)->InitGaugeGroupMgr( m_gaugeGroupManager );

    } else if( id == idMenuRemove ){

        Logger::logf(LOG_MSG,_T("TbTbGaugeGroup::OnMenu rem"));

        m_handler->RemoveToolBarTool( m_toolBar, _T("Gauge Groups"), GetId(),  GetItemId() );

    } else if( id == idMenuStick ){
        StickGroup();
    } else {
        ChangeGroup( m_gaugeGroupManager->GetGroupInfo(_T("name"),m_gMenu->GetLabelText(id)) );
    }

}

void TbTbGaugeGroup::StickGroup()
{
    if(!m_gaugeGroupManager) return;
    m_gaugeGroupManager->setStickGroup( m_groupInfo.GetName(), m_popupMenu->IsChecked(idMenuStick) );
}

void TbTbGaugeGroup::InitGGMenu()
{
    if(!m_gaugeGroupManager) return;

    wxMenu * gmenu = m_gaugeGroupManager->GetMenu();

    wxMenuItemList list = gmenu->GetMenuItems();

    m_gMenu = new wxMenu();

    int id;

    for(uint16_t a=0; a<list.GetCount();a++){
        wxMenuItem * mitem = list.Item(a)->GetData();
        if( mitem->GetSubMenu() != NULL ) continue;
        id = wxNewId();
        m_gMenu->AppendRadioItem( id, mitem->GetItemLabel() );
        m_toolBar->Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( TbTbGaugeGroup::OnMenu ) , NULL, this );

        if( mitem->GetItemLabelText() == m_groupInfo.GetName() )
            m_gMenu->Check( id, true );

    }

    idMenuAddNew = wxNewId();
    idMenuRemove = wxNewId();

    idMenuStick = wxNewId();

    m_popupMenu->AppendSubMenu( m_gMenu, _T("Select a group") );
    m_popupMenu->AppendSeparator();
    m_popupMenu->AppendCheckItem(idMenuStick, _T("Stick the group"));
    m_popupMenu->Check( idMenuStick, m_gaugeGroupManager->isStickedGroup( m_groupInfo.GetName() ) );

    m_popupMenu->AppendSeparator();
    m_popupMenu->Append(idMenuAddNew, _T("Add new icon"));
    m_popupMenu->Append(idMenuRemove, _T("Remove icon"));

    m_toolBar->Connect( idMenuStick,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( TbTbGaugeGroup::OnMenu ) , NULL, this );

    m_toolBar->Connect( idMenuRemove,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( TbTbGaugeGroup::OnMenu ), NULL, this  );
    m_toolBar->Connect( idMenuAddNew,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( TbTbGaugeGroup::OnMenu ), NULL, this  );

}

void TbTbGaugeGroup::SetGroupInfo(const GaugeGroupDesc& desc)
{
    m_groupInfo = desc;
}

void TbTbGaugeGroup::OnTool(wxCommandEvent &event)
{
    Logger::logf(LOG_MSG,_T("TbTbGaugeGroup::OnTool"));


    if(!m_gaugeGroupManager) return;

    Logger::logf(LOG_MSG,_T("TbTbGaugeGroup::OnTool 2"));

    wxMenu * gmenu = m_gaugeGroupManager->GetMenu();


    int id = gmenu->FindItem( m_groupInfo.GetName() );

    short res = m_gaugeGroupManager->showGroup( m_groupInfo.GetName() );

    switch(res){
        case 1:
            SetToolBmp( TBI_GG_ON );
            gmenu->Check(id, true);
            break;

        case -1:
            SetToolBmp( TBI_GG_OFF );
            gmenu->Check(id, false);
            break;

        case -2:
            break;
    }


    Logger::logf(LOG_MSG,_T("res = %i"),res);

}

void TbTbGaugeGroup::OnToolR(wxCommandEvent &event)
{
    m_toolBar->PopupMenu( m_popupMenu );

}

void TbTbGaugeGroup::OnToolEnter(wxCommandEvent &event)
{
}

void TbTbGaugeGroup::Init()
{

    InitBmpRes();

    m_defaultBitmap = m_bmpHash[TBI_GG_OFF];
    m_inactiveBitmap = m_bmpHash[TBI_GG_OFF];

    AddRequest(TN_GaugeGroupManager);
    AddRequest(TN_GGEventHide);
    AddRequest(TN_GGEventShow);



    //DrawIconText();

}

void TbTbGaugeGroup::DrawIconText()
{


    wxRegEx ex;
    ex.Compile(_T(".+\\((.+)\\).*"));

    wxString label = m_groupInfo.GetShortName();

    label.Replace(_T(" "),_T("\n"));


    wxSize tsize = DrawHelper::GetTinyTextExtent( label );

    wxSize fsize = m_toolBar->GetToolBitmapSize();

    //Logger::logf(LOG_MSG,_T("tsize.y=%i, label=")+label,tsize.y);

    wxPoint tpoint;
    tpoint.y = fsize.y/2 - tsize.y/2;
    if( tsize.x > fsize.x )
        tpoint.x = 1;
    else
        tpoint.x = fsize.x/2-tsize.x/2;

    wxPoint tp2 = tpoint;
    tp2.x++;tp2.y++;

    DrawHelper::DrawTinyText( m_bmpHash[ TBI_GG_OFF ], label, tp2, wxColor(0,0,0), true);
    DrawHelper::DrawTinyText( m_bmpHash[ TBI_GG_OFF ], label, tpoint, wxColor(255,255,255), true);

    DrawHelper::DrawTinyText( m_bmpHash[ TBI_GG_ON ], label, tp2, wxColor(255,255,255), true);
    DrawHelper::DrawTinyText( m_bmpHash[ TBI_GG_ON ], label, tpoint, wxColor(0,0,0), true);

}

void TbTbGaugeGroup::InitGaugeGroupMgr(GaugeGroupManager* mgr)
{

    m_gaugeGroupManager = mgr;

//    SetGroup( m_gaugeGroupManager->GetGroupInfo(m_groupIDtmp) );

    if( !m_gaugeGroupManager ) return;

    m_groupInfo = m_gaugeGroupManager->GetGroupInfo(m_groupIDtmp);

    InitBmpRes();

    DrawIconText();

    SetToolBmp( TBI_GG_OFF );

    ApplyLabel();
    InitHelpStrings();

    m_groupIDtmp = -1;

    InitGGMenu();

}

bool TbTbGaugeGroup::SendObject(int id, wxObject *object, const wxString &message)
{

    switch(id) {

        case TN_GaugeGroupManager:
            InitGaugeGroupMgr((GaugeGroupManager*)object);
            break;

        case TN_GGEventShow:
            if( message == m_groupInfo.GetName() )
                SetToolBmp( TBI_GG_ON );
            break;

        case TN_GGEventHide:
            if( message == m_groupInfo.GetName() )
                SetToolBmp( TBI_GG_OFF );
            break;

        default:
            break;
    }
    return true;
}

void TbTbGaugeGroup::ApplyLabel()
{
    m_longHelpString  = _T("Show/hide ") + m_groupInfo.GetName();

    m_shortHelpString = m_groupInfo.GetName() + _T("\nHotkey: ") +
                        ( m_groupInfo.GetHotKey() == wxEmptyString ? _T("None") : m_groupInfo.GetHotKey() );
}



