#include "TbTbSerialMode.h"

#include "../DirHelper.h"
#include "../logger.h"

#include "../VemsTuneController.h"
#include "../common/ProcessBase.h"


/** \class TbTbSerialMode
  *
  * \~english
  * \brief Show and set the serial state
  * \details
  *
  *
  * \~hungarian
  * \brief Megmutatja és beállítható soros port módja
  * \details
  *
  *
  */


enum ToolBitmapIds
{
    TBI_SM_ONLINE,
    TBI_SM_OFFLINE
};

IMPLEMENT_DYNAMIC_CLASS(TbTbSerialMode,VemsToolBarToolBase)

TbTbSerialMode::TbTbSerialMode(int id)
            :VemsToolBarToolBase(id)
            ,m_eventHandler (NULL)
            ,m_commControlerBase(NULL)
            ,m_ConnMenu (NULL)

{
    m_bmpKeyListArray[TBI_SM_ONLINE].Add( DirHelper::GetPath(dICON) + _T("tbSerialModeOn48.png") );
    m_bmpKeyListArray[TBI_SM_ONLINE].Add( DirHelper::GetPath(dICON) + _T("tbSerialModeOn.png") );
    m_bmpKeyListArray[TBI_SM_OFFLINE].Add( DirHelper::GetPath(dICON) + _T("tbSerialModeOff48.png") );
    m_bmpKeyListArray[TBI_SM_OFFLINE].Add( DirHelper::GetPath(dICON) + _T("tbSerialModeOff.png") );
}

TbTbSerialMode::~TbTbSerialMode()
{
    if ( m_eventHandler ) {
        m_eventHandler->Disconnect( wxEVT_ECU_COMM, ECUCommEventHandler( TbTbSerialMode::OnECUCommState ),NULL, this);
    }
}


void TbTbSerialMode::OnTool(wxCommandEvent &event)
{

    if(m_commControlerBase == NULL) return;

    bool isRunning = m_commControlerBase->isRunning();
    Logger::logf(LOG_MSG,_T("TbTbDevices::OnTool() isRunning: %u"), isRunning);
    if( isRunning ){
        m_commControlerBase->stop();
    }else{
        m_commControlerBase->start();
    }
    InitHelpStrings();
}

void TbTbSerialMode::OnECUCommState( ECUCommEvent& event )
{

    if( event.state == ECUCommEvent::ONLINE )
    {
        m_shortHelpString = _T("Set offline");
        SetToolBmp(TBI_SM_ONLINE);
    }
    else
    {
        m_shortHelpString = _T("Set online");
        SetToolBmp(TBI_SM_OFFLINE);
    }
    if(m_ConnMenu == NULL){
         event.Skip();
         return;
    }
    m_ConnMenu->Check(m_ConnMenu->FindItem(_T("Online")), m_state);
    m_ConnMenu->Check(m_ConnMenu->FindItem(_T("Offline")), !m_state);

    event.Skip();
}

void TbTbSerialMode::Init()
{
    InitBmpRes();
    //InitXpmRes();

    AddRequest(TN_ConnMenu);
    AddRequest(TN_ConMess);
    AddRequest(TN_wxEventHandler);
    AddRequest(TN_commControllerECU);


    m_shortHelpString = _T("Set offline");
    m_longHelpString  = _T("Set serial mode: Online/offline");

    m_defaultBitmap = m_bmpHash[TBI_SM_ONLINE];


}

bool TbTbSerialMode::SendObject(int id, wxObject *object, const wxString &message)
{
    switch(id) {
        case TN_wxEventHandler:
            m_eventHandler = dynamic_cast<wxEvtHandler*> (object);
            if(!m_eventHandler){
                Logger::logf(LOG_MSG,_T("TbTbDevices::SendObject() dynamic_cast failed!"));
                return false;
            }
            m_eventHandler->Connect( wxEVT_ECU_COMM, ECUCommEventHandler( TbTbSerialMode::OnECUCommState ),NULL, this);
            break;
        case TN_commControllerECU:
            m_commControlerBase = dynamic_cast<ProcessBase*> (object);
            if(! m_commControlerBase){
                Logger::logf(LOG_MSG,_T("TbTbDevices::SendObject() ProcessBase dynamic_cast failed!"));
            }
            break;
        case TN_ConnMenu:
            m_ConnMenu = (wxMenu*)object;
            break;
        case TN_ConMess:
            m_shortHelpString = message;
            InitHelpStrings();
            break;
        default:
            break;

    }
    return true;
}
