#ifndef TRIGGERFRAME_H
#define TRIGGERFRAME_H

#include "../common/typedef.h"
#include "../common/types/IntArray.h"
#include "../common/types/DoubleArray.h"
#include "../common/types/Int2IntHash.h"
#include "../common/types/Int2DoubleArrayHash.h"

class TriggerFrame
{
    public:
        TriggerFrame();

    public:
        enum type {
            CRC_ONLY = 1,
            SEC_TRIG = 2,
            CAMSHAFT_EX =3,
            EVENT_4,
            EVENT_5,
            ROAD_SPEED1,
            ROAD_SPEED2,
            EVENT_8,
            EVENT_9,
            EVENT_10,
            EVENT_11,
            EVENT_12,
            EVENT_13,
            EVENT_14,
            EVENT_15,
            EVENT_EEPROM_READ,

            EVENT_REALTIME = 32,

            EVENT_DATA_TIMESTAMP_1 = 64,

            ///------------------
            /// NOTE!!!
            // if you add new events here then alter TriggerLogSettingsHelper.cpp too
            // see setDefaults() in triggerlog/TriggerLogSettingsHelper.cpp
            ///------------------

            ///
            LAST_OK_ID,
            ///
            UNSUPPORTED_FORMAT  = 257,
            TRFRAME_SMALL_SIZE  = 258,
            TRFRAME_CRC_ERROR   = 259,
            TRFRAME_EMPTY_FRAME = 260,
        };

        IntArray    m_primaryA16;

        uint16_t    m_Trigger; // time of the Nth trigger (N>1)
        uint16_t    m_data;

        uint8_t     m_type; /// see: enum type
        uint16_t    m_crc; /// incoming crc
        uint16_t    m_calc_crc; /// calculated crc

    public:
        int         addFrame(const IntArray& frame);
        bool        checkCRC();
        uint16_t    getLastPRTrigger();

        void        convertFrame2Time( Int2DoubleArrayHash& triggerTimes, int& clockbase, int& prevTime );

    /// DEPRECATED - only prim&sectrig ( if something goes wrong)
    // void convertFrame2Time(DoubleArray& primaryTime, DoubleArray& secondaryTime, int& clockbase, int& prevTime);
};

WX_DEFINE_ARRAY( TriggerFrame*, TriggerFrameArray );


#endif // TRIGGERFRAME_H
