#ifndef UNIDLG2TEST_H
#define UNIDLG2TEST_H

#include <wx/wxprec.h>


#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/statbmp.h>
#include <wx/button.h>
#include <wx/dialog.h>
#include <wx/gbsizer.h>
#include <wx/statline.h>

#include "common/typedef.h"
#include "common/types/Int2StringHash.h"
#include "common/types/IntArrayRefHash.h"
#include "common/types/ArrayStringRefHash.h"
#include "common/types/String2IntHash.h"
#include "common/types/StringHash.h"
#include "firmwareconfigitem.h"


WX_DECLARE_STRING_HASH_MAP(wxFlexGridSizer *, SizerHash);
WX_DECLARE_STRING_HASH_MAP(wxBoxSizer *, wxBoxSizerHash);
WX_DECLARE_STRING_HASH_MAP(wxGridBagSizer *, wxGBxSizerHash);
WX_DECLARE_HASH_MAP( int, FirmwareConfigItem *, wxIntegerHash, wxIntegerEqual, Int2FirmwareConfigItem);

class ProjectConfigSaver;

class Unidlg2Test: public wxDialog
{
	public:

		Unidlg2Test(wxWindow* parent,wxWindowID id = -1);
		virtual ~Unidlg2Test();

		//(*Identifiers(Unidlg2Test)
		static const long ID_STATICTEXT1;
		static const long ID_TEXTCTRL1;
		static const long ID_STATICBITMAP1;
		static const long ID_CHECKBOX1;
		static const long ID_BUTTONFILE;
		//*)
		void addBox(const wxString& boxName);
        //void addLine(const wxString& lineName);
        void addLine(FirmwareConfigItem * fwItem);
        void setDefaultParent(FirmwareConfigItem *dp);
        void sessionSelect();
        void addCaption();
        void openSession(const wxString& name );
        void setDialogType(const wxString& dtype );
        void setButtonOK(wxString& name);
        /// Should be removed
        FirmwareConfigItemArray* getConfigItemArray();

        wxString getHeaderFile();
        wxArrayString getConfigFiles();
        wxArrayString getDownloadConfigFiles();

        wxArrayString& getTypes() { return typesA; };

        void loadProject(FirmwareConfigItemArray configItemA);

        void setProjectConfigSaver(ProjectConfigSaver * prjcfgsvr) { m_prjCfgSaver = prjcfgsvr; };

        static wxString v3FileDialog(wxString & message, wxString& defaultDir, wxString & filter, bool multiSel);
	protected:

		void OnButton(wxCommandEvent &evt);


		//wxStaticBitmap* StaticBitmap1;
		//wxButton* FileSelectButton;
		//wxStaticText* StaticText1;
		//wxCheckBox* CheckBox1;
		//wxGridSizer* GridSizer1;
		//wxTextCtrl* TextCtrl1;
		//wxStaticBoxSizer* StaticBoxSizer1;

		Int2StringHash onBrowseHash;
		Int2StringHash onDeleteHash;

		wxString m_currBoxName;
        wxFlexGridSizer* m_currGridSizer;
        wxGridBagSizer* m_currGBSizer;


        wxBoxSizer* m_topSizer;
        int m_boxCount;
        //int boxRowCount;

        wxBitmap errorBitMap;
        wxBitmap tickBitMap;
        wxBitmap deleteLineBitMap;
        wxBitmap addLineBitMap;


        void AddTextChangeEvent(int id);
        void AddCheckBoxEvent(int id);
        void AddButtonEvent(int id);
        void AddDeleteEvent(int id);
        void AddRadioButtonEvent(int id);

        void OnTextChange(wxCommandEvent& event);
        void OnCheckBox(wxCommandEvent& event);
        void OnRadioButton(wxCommandEvent& event);
        void OnBrowse(wxCommandEvent& event);
        void OnDelete(wxCommandEvent& event);
        void OnAddLine(wxCommandEvent& event);
        void OnSessionLoad(wxCommandEvent& event);
        void OnSessionDelete(wxCommandEvent& event);
        void OnSessionSave(wxCommandEvent& event);

        void deleteLine(int itemRowIndex, int sizerRowIndex, const wxString& actualBoxName);
        void addNewLine();
        void refreshAllBitmap();
        //void validateButton();
        void addDialogButtons();
        void deleteBox(const wxString & name);
        wxString getSessionComboValue();
        void addRowEventId(int id);
        wxArrayString* getLineNameArray();

        wxWindow * getTextWindowByName(const wxString& wName);
        FirmwareConfigItem * getConfigItemByWindowName(const wxString& wName);
        void autoSelectCheckBox(const wxString& wName);
        wxWindow * getCheckBoxWindowByName(const wxString& wName);


        void fillDlg();
        wxArrayString getFiles(const wxString& type);

	private:

        wxStaticLine* addStaticLine(wxGridBagSizer * currLineSizer, int style, uint8_t lenght, int align, wxGBPosition pos);
        void addStaticCaption(wxGridBagSizer * currLineSizer, uint8_t lenght, const wxString &caption, int rowidx);
        wxPoint GetGroupBorders(wxGridBagSizer * actualSizer, int row, int col);
        wxString SetRadioButton(int id, bool set);
        void DeleteStaticCaptionLine(wxGridBagSizer * actualSizer, wxGBPosition pos);
        void DeleteStaticLine(wxGridBagSizer * actualSizer, wxGBPosition pos);


        enum {
            idSaveProject
        };

        //wxGridSizer* getActualSizer(const wxString& actualBoxName);
        SizerHash sizerHash;
        wxGBxSizerHash  m_sizerHash;
        IntArrayRefHash deleteLineHash;

        ArrayStringRefHash lineNameArrayHash;

        Int2FirmwareConfigItem id2FirmwareConfigItem;
        //IntArray deleteLineArray;
        FirmwareConfigItem * defaultParent; /// default Parent idx would be better

        String2IntHash boxRowCountHash;
        //wxString currBoxName;

        FirmwareConfigItemArray m_configItemArray;

        StringHash type2BoxName;
        wxArrayString typesA;
        bool isAddNewLine;
        wxString m_buttonOK;

        ProjectConfigSaver *m_prjCfgSaver;

        bool m_isInGroup;
        wxString m_currGrp;
        uint8_t m_grpBeginIdx;

		DECLARE_EVENT_TABLE()

        String2IntHash  m_rbLastIDs;




};

#endif
