#include "EconosealPanel.h"

#include <wx/dcbuffer.h>

#include "../logger.h"
#include "../DirHelper.h"
//#include "../common/FunctionMeter.h"

BEGIN_EVENT_TABLE(EconosealPanel,wxPanel)

    EVT_PAINT(EconosealPanel::OnPaint)

END_EVENT_TABLE();

EconosealPanel::EconosealPanel(wxWindow *parent, size_t pincount, size_t linecnt,int id, const wxPoint& pos)
{
    //ctor
    wxImage im(DirHelper::GetPath(dGRAPHICS) + wxString::Format(_T("econoseal%i.png"),pincount) ,wxBITMAP_TYPE_PNG );

    wxSize size(im.GetWidth(),im.GetHeight());

    Create(parent, id, pos, size);
    SetInitialSize( size );

    m_backGround = new wxBitmap( im );
/*
    wxImage imb(size.x,size.y);
    imb.SetAlpha();

    for(size_t a=0;a<size.x;a++)
        for(size_t b=0;b<size.y;b++)
            imb.SetAlpha(a,b,0);

    m_pinNumbersBmp = new wxBitmap( imb );
*/

    m_pinBmp = NULL;

    m_aggCanvas = new wxAgg( size.x, size.y );

    m_showPinNumbers = true;

    m_pinCount = pincount;
    m_lineCount = linecnt;

    Init();
}

EconosealPanel::~EconosealPanel()
{
    //dtor
}

void EconosealPanel::OnPaint(wxPaintEvent &evt)
{
    wxBufferedPaintDC dc(this);

    dc.SetBackground(wxBrush( wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE) ));
    dc.Clear();

    dc.DrawBitmap( *m_pinBmp, 0, 0, true);
    dc.DrawBitmap( *m_backGround, 0, 0, true);

    if(m_showPinNumbers) {
        //dc.DrawBitmap( *m_pinNumbersBmp, 0, 0, false);
        wxFont font = dc.GetFont();
        font.SetPointSize(9);
        dc.SetFont(font);

        for( size_t a=0;a< m_pinCenters.GetCount();a++) {
            wxString text = wxString::Format(_T("%i"), a+1);
            wxSize ne = dc.GetTextExtent( text );
            dc.DrawText( text, (int)(m_pinCenters[a].x-ne.x/2.0), (int)(m_pinCenters[a].y-ne.y/2.0));
        }
    }

}

void EconosealPanel::Init()
{
    m_pinRadius = 10.0;
    m_pinDistance = 28.0;

    size_t colcnt = m_pinCount/m_lineCount;

    double x = 0;
    double y = m_pinDistance;

    m_aggCanvas->clear( wxColor(0,0,0,0) );
    m_aggCanvas->setPen( wxColor(0,0,0) );
/*
    wxMemoryDC dc;

    dc.SelectObject(*m_pinNumbersBmp);
    //  dc.Clear();
    dc.SetTextForeground(wxColor(0,0,0));

    wxFont font = dc.GetFont();
    font.SetPointSize(10);
    dc.SetFont(font);
*/
    for(size_t a=0; a< m_lineCount;a++){
        x = m_pinDistance + a%2 * m_pinDistance/2.0;
        for(size_t b=0; b< colcnt; b++){
            m_pinCenters.Add(wxRealPoint(x,y));
            m_aggCanvas->drawCircle( x,y, m_pinRadius, 1.1);
/*
            wxString text = wxString::Format(_T("%i"), (a*colcnt)+b+1 );
            wxSize ne = dc.GetTextExtent( text );
            dc.DrawText( text, (int)(x-ne.x/2.0), (int)(y-ne.y/2.0));
*/
            x += m_pinDistance;
        }
        y += m_pinRadius * 2.0;
    }

    m_pinBmp = m_aggCanvas->renderToBitmap();
/*
    dc.SelectObject( wxNullBitmap );
    m_pinNumbersBmp->SaveFile(_T("f.png"),wxBITMAP_TYPE_PNG);
*/
}

void EconosealPanel::SetPinColor(size_t pin, const wxColor &color, bool refresh)
{
    pin--;
    if(pin > m_pinCenters.GetCount()) return;

    wxMemoryDC dc(*m_pinBmp);

    m_aggCanvas->clear( wxColor(0,0,0,0) );

    m_aggCanvas->setBrush( color );

    m_aggCanvas->drawFilledCircle( m_pinCenters[pin], m_pinRadius - 0.5 );

    m_aggCanvas->render( dc );

    dc.SelectObject(wxNullBitmap);

    if ( refresh )
        Refresh(false);
}

void EconosealPanel::ClearPin(size_t pin)
{
    SetPinColor( pin, wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
}

void EconosealPanel::ClearAllPin()
{
    for( size_t a=1;a<= m_pinCenters.GetCount();a++)
        SetPinColor( a, wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE), false);
    Refresh(false);
}

void EconosealPanel::ShowPinNumbers(bool show)
{
    m_showPinNumbers = show;

    Refresh( false );
}
