#include "ExpressionGridWidget.h"

#include <wx/grid.h>
#include "../logger.h"

ExpressionGridWidget::ExpressionGridWidget(
    STDataMgrBase* stController,
    RTDataMgrBase* rtController,
    ExpressionGridDesc* expGridDesc,
    wxWindow* parent,
    wxWindowID id,
    const wxPoint& pos,
    const wxSize& size,
    long style )
{
    m_stController = stController;
    m_rtDataMgr = rtController;
    m_expGridDesc  = expGridDesc;

    m_isSymbolsRegistered = false;

    /// TODO: fetch from cellString
    //m_rtVariables.insert(_T("rpm"));
    //m_rtVariables.insert(_T("map"));

    Create(parent, id, pos, size, style );
}

bool ExpressionGridWidget::Create( wxWindow* parent,
                                 wxWindowID id,
                                 const wxPoint& pos,
                                 const wxSize& size,
                                 long style )
{

    wxPanel::Create( parent, id, pos, size, style );
    SetSizer( new wxBoxSizer( wxHORIZONTAL ) );
    //size_t rowCount = m_expGridDesc->rows.GetCount();
    //size_t colCount = m_expGridDesc->rows.Item(0)->GetCount();
    m_rows = m_expGridDesc->cells.shape()[0];
    m_cols = m_expGridDesc->cells.shape()[1];

    m_grid = new wxGrid(this, wxID_ANY);

    m_grid->CreateGrid( m_rows, m_cols);
    m_grid->SetColLabelSize(0);
    m_grid->SetRowLabelSize(0);

    m_grid->DisableDragRowSize();
    //m_grid->SetColLabelValue(0, _T("Prop."));
    //m_grid->SetColLabelValue(1, _T("Value"));
    m_grid->EnableEditing(false);

    //StringSet allVariableA;
//    for(size_t r = 0; r < m_expGridDesc->rows.GetCount(); ++r){
//        wxArrayString* row = m_expGridDesc->rows.Item(r);
//        for(size_t c = 0; c < row->GetCount(); ++c){
//                wxString cellStr = row->Item(c);
//                m_grid->SetCellValue( r, c, cellStr);
//                size_t found = getVariable(cellStr, m_rtVariables);
//                if(found){
//                    StringSet* variables = new StringSet();
//                    getVariable(cellStr, *variables);
//                    m_variablePointA.Add( new wxPoint(r,c) );
//                    m_variableSetA.Add( variables );
//                }
//        }
//    }
    for(size_t r = 0; r < m_rows; ++r)
    {
        for(size_t c = 0; c < m_cols; ++c)
        {
                wxString cellStr = m_expGridDesc->cells[r][c].text;
                m_grid->SetCellValue( r, c, cellStr);
                size_t found = getVariable(cellStr, m_rtVariables);
                if(found){
                    StringSet* variables = new StringSet();
                    getVariable(cellStr, *variables);
                    m_variablePointA.Add( new wxPoint(r,c) );
                    m_variableSetA.Add( variables );
                }
        }
    }
    Logger::logf(LOG_MSG,_T("ExpressionGridWidget::Create: number of cells which contain variables:%d"), m_variablePointA.GetCount() );
    //fillRTVariable();
    //fillSTVariable();

    //m_grid->AutoSize();

    GetSizer()->Add( m_grid, 1, wxALL| wxEXPAND, 1 );
    //GetSizer()->Add( m_grid, 0, wxALL, 1 );
    GetSizer()->Fit( this );
    SetInitialSize();
    return true;
}

ExpressionGridWidget::~ExpressionGridWidget()
{
    //dtor
     m_rtDataMgr->UnRegisterViewer(this);

    /// TODO clear
    WX_CLEAR_ARRAY(m_variablePointA);
    WX_CLEAR_ARRAY(m_variableSetA);

    //wxPointArray m_variablePointA;
    //StringSetArray m_variableSetA;

}



/// RTDataMgrBase

void ExpressionGridWidget::refresh(DoubleHash* table)
{

    m_rtVariablesH = table;

    /// Fast
    m_grid->BeginBatch();
     for(size_t i = 0; i < m_variablePointA.GetCount(); ++i){
            wxPoint* cell = m_variablePointA.Item(i);
            //wxArrayString* row = m_expGridDesc->rows.Item(cell->x);
            //wxString cellStr = row->Item(cell->y);
            wxString cellStr = m_expGridDesc->cells[cell->x][cell->y].text;
            StringSet* varSet = m_variableSetA.Item(i);
            replaceRTVariables(*varSet, &cellStr);

            m_grid->SetCellValue(cell->x,cell->y, cellStr);
     }
      m_grid->EndBatch();

/*
    wxString rpm = _T("rpm");
    if( table->find(rpm) == table->end() ){
        return;
    }
    double value = (*table)[rpm];
    wxString rpmValue = wxString::Format(_T("%f"), value);
    m_grid->SetCellValue( 0, 1, rpmValue);

    value = (*table)[_T("map")];
    wxString map = wxString::Format(_T("%f"), value);
    m_grid->SetCellValue( 1, 1, map);
*/
/*
    /// Slow
    m_grid->BeginBatch();
    for(size_t r = 0; r < m_expGridDesc->m_rowsA.GetCount(); ++r){
        wxArrayString* row = m_expGridDesc->m_rowsA.Item(r);
        for(size_t c = 0; c < row->GetCount(); ++c){
                wxString cellStr = row->Item(c);
                replaceRTVariables(m_rtVariables, &cellStr);
                m_grid->SetCellValue( r, c, cellStr);
        }
    }
     m_grid->EndBatch();
*/
}

void ExpressionGridWidget::require()
{
    if( m_isSymbolsRegistered ) return;

    StringSet::const_iterator IT = m_rtVariables.begin();
    for(; IT != m_rtVariables.end(); ++IT){
        wxString varName = *IT;
        m_rtDataMgr->registerSymbol( varName );
    }

    m_isSymbolsRegistered = true;

}

void ExpressionGridWidget::unregisterSymbols()
{
    if( !m_isSymbolsRegistered ) return;

    StringSet::const_iterator IT = m_rtVariables.begin();
    for(; IT != m_rtVariables.end(); ++IT){
        wxString varName = *IT;
        m_rtDataMgr->unregisterSymbol( varName );
    }

}

void ExpressionGridWidget::replaceRTVariables(const StringSet& varSet, wxString* cellStr)
{

    //DoubleHash::const_iterator* IT = m_rtVariables->begin();

    double var;
    StringSet::const_iterator IT = varSet.begin();
    for(; IT != varSet.end(); ++IT){
        wxString varName = *IT;
        if( !getValue(varName, var)){ continue; }

        wxString valueStr;
        if(var < -999999.0){
            valueStr = _T("NA");
        }else{
            valueStr = wxString::Format(_T("%.3f"), var);;
        }

        cellStr->Replace( _T("${")+ varName + _T("}"), valueStr);
    }
}

bool ExpressionGridWidget::getValue(const wxString& name, double& var)
{
    DoubleHash::const_iterator IT = m_rtVariablesH->find(name);
     if( IT == m_rtVariablesH->end() ){
        return false;
    }
    var = IT->second;
    return true;
}

 size_t ExpressionGridWidget::getVariable(const wxString& str, StringSet& variables)
 {
    wxString DOLLAR = _T("$");
    wxString BRACKET_START = _T("{");
    wxString BRACKET_END = _T("}");
    size_t found = 0;
    int state = WAIT_DOLLAR;
    wxString varName;
    for(size_t i = 0; i < str.size(); ++i){
        wxChar chr = str.GetChar(i);
        switch(state){
            case WAIT_DOLLAR:
                if( DOLLAR.IsSameAs( chr ) ){
                    state = WAIT_START_BRACKET;
                }
                break;
            case WAIT_START_BRACKET:
                if( BRACKET_START.IsSameAs(chr) ){
                    state = WAIT_END_BRACKET;
                }else{
                    state = WAIT_START_BRACKET;
                }
                break;
            case WAIT_END_BRACKET:
                 if(BRACKET_END.IsSameAs(chr) ){
                    variables.insert(varName);
                    found++;
                    varName.Clear();
                    state = WAIT_DOLLAR;
                }else{
                    varName.Append(chr);
                }
                break;
        }
    }
    return found;
 }




 ExpressionGridDesc* ExpressionGridWidget::testDesc()
{
    ExpressionGridDesc* testDesc = new ExpressionGridDesc();
    ExpressionCellDescriptorMatrix& cells = testDesc->cells;

    ExpressionCellDescriptorMatrix::extent_gen extents;
    cells.resize(extents[48][8]);

    cells[0][0].text = _T("Warmup enrichment (coolant)");
    cells[0][1].text = _T("${coolant}C");
    cells[0][2].text = _T("=");
    cells[0][3].text = _T("${calc_warmup_enrich_norm}%");
    cells[0][4].text = _T("=");
    cells[0][5].text = _T("${calc_warmup_enrich_norm}%");
    cells[0][6].text = _T(" ");
    cells[0][7].text = _T(" ");

    cells[1][0].text = _T("Warmup crank/afterstart (coolant)");
    cells[1][1].text = _T("${coolant}C");
    cells[1][2].text = _T("=");
    cells[1][3].text = _T("${calc_warmup_enrich_multi}%");
    cells[1][4].text = _T("*=");
    cells[1][5].text = _T("${calc_warmup_enrich}% ");
    cells[1][6].text = _T(" ");
    cells[1][7].text = _T(" ");

    cells[2][0].text = _T("");
    cells[2][1].text = _T("");
    cells[2][2].text = _T("");
    cells[2][3].text = _T("");
    cells[2][4].text = _T("Diff:");
    cells[2][5].text = _T("${calc_warmup_enrich_diff}");
    cells[2][6].text = _T("");
    cells[2][7].text = _T("");

///////////////////////////////////////////////////////////
    cells[3][0].text = _T("Boost target curve(RPM)");
    cells[3][1].text = _T("${rpm}");
    cells[3][2].text = _T("=");
    cells[3][3].text = _T("${boost_curve_rpm}");
    cells[3][4].text = _T("=");
    cells[3][5].text = _T("${boost_curve_rpm}");
    cells[3][6].text = _T(" ");
    cells[3][7].text = _T(" ");

    cells[4][0].text = _T("Boost tps curve(TPS)");
    cells[4][1].text = _T("${throttle}%");
    cells[4][2].text = _T("=");
    cells[4][3].text = _T("${boost_corr_tps}%");
    cells[4][4].text = _T("*=");
    cells[4][5].text = _T("${boost_curve_rpm_tps}");
    cells[4][6].text = _T("note:1");

    cells[5][0].text = _T("Boost Control Target Offset");
    cells[5][1].text = _T("${boost_more_raw}");
    cells[5][2].text = _T("=");
    cells[5][3].text = _T("${boost_corr_offs}");
    cells[5][4].text = _T("+=");
    cells[5][5].text = _T("${boost_curve_rpm_tps_offs}");
    cells[5][6].text = _T(" ");

    cells[6][0].text = _T("Boost gear reduction(gear)");
    cells[6][1].text = _T("(${gear}");
    cells[6][2].text = _T("=");
    cells[6][3].text = _T("${boost_curve_gear}%");
    cells[6][4].text = _T("*=");
    cells[6][5].text = _T("${boost_curve_rpm_tps_offs_gear}");
    cells[6][6].text = _T(" ");

    cells[7][0].text = _T("Boost MAT reduction(mat)");
    cells[7][1].text = _T("${mat}");
    cells[7][2].text = _T("=");
    cells[7][3].text = _T("${boost_corr_mat}");
    cells[7][4].text = _T("-=");
    cells[7][5].text = _T("${boost_curve_rpm_tps_offs_gear_mat}");
    cells[7][6].text = _T(" ");

    cells[8][0].text = _T("Boost EGT reduction(egt1)");
    cells[8][1].text = _T("${egt1}");
    cells[8][2].text = _T("=");
    cells[8][3].text = _T("${boost_corr_egt}");
    cells[8][4].text = _T("-=");
    cells[8][5].text = _T("${boost_curve_rpm_tps_offs_gear_mat_egt}");
    cells[8][6].text = _T(" ");

    cells[9][0].text = _T("");
    cells[9][2].text = _T("");
    cells[9][3].text = _T("");
    cells[9][4].text = _T("");
    cells[9][5].text = _T("ECU Boost target:");
    cells[9][6].text = _T("${boost_tgt}");
    cells[9][7].text = _T(" ");

    cells[10][0].text = _T("");
    cells[10][1].text = _T("");
    cells[10][2].text = _T("");
    cells[10][3].text = _T("");
    cells[10][4].text = _T("Diff:");
    cells[10][5].text = _T("${boost_tgt_diff}");
    cells[10][6].text = _T(" ");

    cells[11][0].text = _T("");

///// SPARK
    cells[12][0].text = _T("spark:");

    cells[13][0].text = _T("Spark curve(RPM,MAP)");
    cells[13][1].text = _T("${rpm}");
    cells[13][2].text = _T("${map}");
    cells[13][3].text = _T("=");
    cells[13][4].text = _T("${spark_curve_rpm_map}");
    cells[13][5].text = _T("=");
    cells[13][6].text = _T("${spark_curve_rpm_map}");
    cells[13][7].text = _T(" ");

    cells[14][0].text = _T("Launch retard");
    cells[14][1].text = _T("${launch_retard}");
    cells[14][2].text = _T("");
    cells[14][3].text = _T("=");
    cells[14][4].text = _T("${spark_corr_launch}");
    cells[14][5].text = _T("-=");
    cells[14][6].text = _T("${spark_curve_rpm_map_launch}");
    cells[14][7].text = _T("note:2");

    cells[15][0].text = _T("ALS retard(tps,als_retard)");
    cells[15][1].text = _T("${throttle}");
    cells[15][2].text = _T("${als_retard}");
    cells[15][3].text = _T("=");
    cells[15][4].text = _T("${spark_corr_als}");
    cells[15][5].text = _T("=");
    cells[15][6].text = _T("${spark_curve_rpm_map_launch_als}");
    cells[15][7].text = _T("note:3 ${als_retard_active} ${als_retard_value1}");

    cells[16][0].text = _T("KNOCK Adj (knock retard)");
    cells[16][1].text = _T("${lambda_corr_knock_maxret6}");
    cells[16][2].text = _T("");
    cells[16][3].text = _T("=");
    cells[16][4].text = _T("${spark_corr_knock}");
    cells[16][5].text = _T("-=");
    cells[16][6].text = _T("${spark_curve_rpm_map_launch_als_knock}");
    cells[16][7].text = _T("note:4");

    cells[17][0].text = _T("IAC retard");
    cells[17][1].text = _T("");
    cells[17][2].text = _T("");
    cells[17][3].text = _T("=");
    cells[17][4].text = _T("??????");
    cells[17][5].text = _T("-=");
    cells[17][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac}");
    cells[17][7].text = _T("");

    cells[18][0].text = _T("Mat retard(MAT,TPS)");
    cells[18][1].text = _T("${mat}");
    cells[18][2].text = _T("${throttle}");
    cells[18][3].text = _T("=");
    cells[18][4].text = _T("${spark_corr_mat}");
    cells[18][5].text = _T("-=");
    cells[18][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat}");
    cells[18][7].text = _T("");

    cells[19][0].text = _T("Acceleration retard");
    cells[19][1].text = _T("");
    cells[19][2].text = _T("");
    cells[19][3].text = _T("=");
    cells[19][4].text = _T("??????");
    cells[19][5].text = _T("-=");
    cells[19][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat_acc}");
    cells[19][7].text = _T("");

    cells[20][0].text = _T("N2O retard");
    cells[20][1].text = _T("");
    cells[20][2].text = _T("");
    cells[20][3].text = _T("=");
    cells[20][4].text = _T("??????");
    cells[20][5].text = _T("=");
    cells[20][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat_acc_n2o}");
    cells[20][7].text = _T("note:4");

    cells[21][0].text = _T("Cold adjust(coolant,map)");
    cells[21][1].text = _T("${coolant}");
    cells[21][2].text = _T("${map}");
    cells[21][3].text = _T("=");
    cells[21][4].text = _T("${spark_corr_cold2}");
    cells[21][5].text = _T("+=");
    cells[21][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat_acc_n2o_coldAdj}");
    cells[21][7].text = _T("");

    cells[22][0].text = _T("Cranking(RPM)");
    cells[22][1].text = _T("${rpm}");
    cells[22][2].text = _T("");
    cells[22][3].text = _T("=");
    cells[22][4].text = _T("${spark_corr_crank2}");
    cells[22][5].text = _T("=");
    cells[22][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat_acc_n2o_coldAdj_crank}");
    cells[22][7].text = _T("");

    cells[23][0].text = _T("Shift Cut retard");
    cells[23][1].text = _T("${shiftcut_ret_raw_val}");
    cells[23][2].text = _T("");
    cells[23][3].text = _T("=");
    cells[23][4].text = _T("${spark_corr_shiftcut}");
    cells[23][5].text = _T("=");
    cells[23][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat_acc_n2o_coldAdj_crank_shiftCut}");
    cells[23][7].text = _T("");

    cells[24][0].text = _T("ignretard_clamp");
    cells[24][1].text = _T("");
    cells[24][2].text = _T("");
    cells[24][3].text = _T("=");
    cells[24][4].text = _T("${spark_corr_clamp}");
    cells[24][5].text = _T("=");
    cells[24][6].text = _T("${spark_curve_rpm_map_launch_als_knock_iac_mat_acc_n2o_coldAdj_crank_shiftCut_clamp}");
    cells[24][7].text = _T("");

    cells[25][0].text = _T("");
    cells[25][1].text = _T("");
    cells[25][2].text = _T("");
    cells[25][3].text = _T("");
    cells[25][4].text = _T("ECU Spark Advance:");
    cells[25][5].text = _T("${spark}");
    cells[25][6].text = _T(" ");

    cells[26][0].text = _T("");
    cells[26][1].text = _T("");
    cells[26][2].text = _T("");
    cells[26][3].text = _T("");
    cells[26][4].text = _T("Diff:");
    cells[26][5].text = _T("${spark_curve_diff}");
    cells[26][6].text = _T(" ");

///// Lambda:
    cells[27][0].text = _T("");

    cells[28][0].text = _T("Lambda:");

    cells[29][0].text = _T("Lambda Table (RPM,MAP)");
    cells[29][1].text = _T("${rpm}");
    cells[29][2].text = _T("${map}");
    cells[29][3].text = _T("=");
    cells[29][4].text = _T("${lambda_target_rpm_map}");
    cells[29][5].text = _T("=");
    cells[29][6].text = _T("${lambda_target_rpm_map}");
    cells[29][7].text = _T(" ");

    cells[30][0].text = _T("Knock correction");
    cells[30][1].text = _T("${lambda_corr_knock_maxret6}");
    cells[30][2].text = _T("");
    cells[30][3].text = _T("=");
    cells[30][4].text = _T("${lambda_corr_knock100}%");
    cells[30][5].text = _T("*=");
    cells[30][6].text = _T("${lambda_target_rpm_map_knock}");
    cells[30][7].text = _T("");

    cells[31][0].text = _T("EGT correction (egt) ");
    cells[31][1].text = _T("${egt1}");
    cells[31][2].text = _T("");
    cells[31][3].text = _T("=");
    cells[31][4].text = _T("${lambda_corr_egt100}%");
    cells[31][5].text = _T("=");
    cells[31][6].text = _T("${lambda_target_rpm_map_knock_egt}");
    cells[31][7].text = _T("");

    cells[32][0].text = _T("");
    cells[32][1].text = _T("");
    cells[32][2].text = _T("");
    cells[32][3].text = _T("");
    cells[32][4].text = _T("ECU Lambda Target:");
    cells[32][5].text = _T("${lambdatarget}");
    cells[32][6].text = _T(" ");

    cells[33][0].text = _T("");
    cells[33][1].text = _T("");
    cells[33][2].text = _T("");
    cells[33][3].text = _T("");
    cells[33][4].text = _T("Diff:");
    cells[33][5].text = _T("${lambda_target_diff}");
    cells[33][6].text = _T(" ");

///// PW

    cells[34][0].text = _T("");

    cells[35][0].text = _T("Pulse Width:");

    cells[36][0].text = _T("Gamma Enrichment ");
    cells[36][1].text = _T("");
    cells[36][2].text = _T("");
    cells[36][3].text = _T("=");
    cells[36][4].text = _T("${gammaEnrich}");
    cells[36][5].text = _T("=");
    cells[36][6].text = _T("${pulse_gammae}");
    cells[36][7].text = _T(" ");

    cells[37][0].text = _T("VE table correction(RPM,MAP)");
    cells[37][1].text = _T("${rpm}");
    cells[37][2].text = _T("${map}");
    cells[37][3].text = _T("=");
    cells[37][4].text = _T("${pulse_corr_ve}");
    cells[37][5].text = _T("*=");
    cells[37][6].text = _T("${pulse_gammae_ve}");
    cells[37][7].text = _T("");

    cells[38][0].text = _T("Lambda correction");
    cells[38][1].text = _T("");
    cells[38][2].text = _T("");
    cells[38][3].text = _T("=");
    cells[38][4].text = _T("${lambdatarget}");
    cells[38][5].text = _T("*=");
    cells[38][6].text = _T("${pulse_gammae_ve_lambda}");
    cells[38][7].text = _T("");

    cells[39][0].text = _T("throttle acceleration correction (TPS)");
    cells[39][1].text = _T("${throttle}");
    cells[39][2].text = _T("");
    cells[39][3].text = _T("=");
    cells[39][4].text = _T("${accelEnrich}");
    cells[39][5].text = _T("=");
    cells[39][6].text = _T("${pulse_gammae_ve_lambda_tps}");
    cells[39][7].text = _T("");

    cells[40][0].text = _T("fuel film");
    cells[40][1].text = _T("");
    cells[40][2].text = _T("");
    cells[40][3].text = _T("=");
    cells[40][4].text = _T("");
    cells[40][5].text = _T("+=");
    cells[40][6].text = _T("${pulse_gammae_ve_lambda_tps_film}");
    cells[40][7].text = _T("");

    cells[41][0].text = _T("req fuel");
    cells[41][1].text = _T("");
    cells[41][2].text = _T("");
    cells[41][3].text = _T("=");
    cells[41][4].text = _T("${req_fuel}");
    cells[41][5].text = _T("*=");
    cells[41][6].text = _T("${pulse_gammae_ve_lambda_tps_film_req}");
    cells[41][7].text = _T("");

    cells[42][0].text = _T("Battery correction");
    cells[42][1].text = _T("");
    cells[42][2].text = _T("");
    cells[42][3].text = _T("=");
    cells[42][4].text = _T("");
    cells[42][5].text = _T("=");
    cells[42][6].text = _T("${pulse_gammae_ve_lambda_tps_film_req_batt}");
    cells[42][7].text = _T("");

    cells[43][0].text = _T("");
    cells[43][1].text = _T("");
    cells[43][2].text = _T("");
    cells[43][3].text = _T("");
    cells[43][4].text = _T("ECU Pulse width:");
    cells[43][5].text = _T("${pulseWidth}");
    cells[43][6].text = _T(" ");

    cells[44][0].text = _T("");
    cells[44][1].text = _T("");
    cells[44][2].text = _T("");
    cells[44][3].text = _T("");
    cells[44][4].text = _T("Diff:");
    cells[44][5].text = _T("${pulse_diff}");
    cells[44][6].text = _T(" ");

    cells[45][0].text = _T("MAP:");
    cells[45][1].text = _T("${map}");
    cells[45][2].text = _T("");

    cells[46][0].text = _T("Lambda:");
    cells[46][1].text = _T("${lambda}");
    cells[46][2].text = _T("");

    cells[47][0].text = _T("Tps:");
    cells[47][1].text = _T("${throttle}");
    cells[47][2].text = _T("${tpsADC}");

    return testDesc;
}
