#include "STMapperViewer.h"


#include "../Math.h"
#include "../common/VemsMatrixHelper.h"
#include "../stdatamanager/STDataMgrBase.h"


STMapperViewer::STMapperViewer( const Symbol& s )
{
    m_symbol        = s;
    m_isAccepted    = s.isArray();

    if ( m_isAccepted )
    {
        m_data.setSize( s.getArrayRowSize(), s.getArrayColSize() );
    }
}


STMapperViewer::~STMapperViewer()
{
}


void STMapperViewer::update()
{
    for ( size_t r = 0; r < m_data.rows(); ++r )
    {
        for ( size_t c = 0; c < m_data.cols(); ++c )
        {
            double viewerData = m_stDataMgr->getDataFromViewerModel( m_symbol, r, c );
            m_data.set( r, c, viewerData );
        }
    }
}


void STMapperViewer::updateModel()
{
    for ( size_t r = 0; r < m_data.rows(); ++r )
    {
        for ( size_t c = 0; c < m_data.cols(); ++c )
        {
            double viewerData = m_data.get( r, c );

            m_stDataMgr->setModelData( m_symbol, r, c, viewerData );
        }
    }
}


void STMapperViewer::remap( const VemsMatrix<double>& oldX,
                            const VemsMatrix<double>& oldY,
                            const VemsMatrix<double>& newX,
                            const VemsMatrix<double>& newY )
{
    if ( !m_isAccepted ) { return; }

    VemsMatrixHelper::resample2D( oldX, oldY, newX, newY, &m_data );
}


wxString STMapperViewer::toString()
{
    return _T("STMapperViewer - ")  + m_symbol.getName();
}
