#ifndef STATICCONTROLPANEL_H
#define STATICCONTROLPANEL_H

#include <wx/wx.h>

#include "../stdatamanager/STDataViewerBase.h"

class StaticViewersController;

class StaticControlPanel : public wxPanel, public STDataViewerBase
{
    public:
        enum
        {
            REFRESH_BUTTON          = 1,
            WRITE_BUTTON            = 2,
            BURN_BUTTON             = 4,
            REMAP_BUTTON            = 8,
            CLOSE_BUTTON            = 16,
            SHOW_REMAP_BUTTON       = 32,
            EXPORT_BUTTON           = 64,
            IMPORT_BUTTON           = 128,
            START_CALIBRATE         = 256,
            STOP_CALIBRATE          = 512,
            PREVIOUS_DIALOG         = 1024,
            NEXT_DIALOG             = 2048,
            //DEFAULT_BUTTONS         = REFRESH_BUTTON | WRITE_BUTTON | BURN_BUTTON | CLOSE_BUTTON,
            DEFAULT_BUTTONS         = REFRESH_BUTTON | WRITE_BUTTON | BURN_BUTTON | PREVIOUS_DIALOG | NEXT_DIALOG |CLOSE_BUTTON,
            REMAPPER_BUTTONS        = REFRESH_BUTTON | BURN_BUTTON | REMAP_BUTTON | CLOSE_BUTTON,
            DEFAULT_IMPORT_BUTTONS   = DEFAULT_BUTTONS | IMPORT_BUTTON,
            REMAPPER_TABLE_BUTTONS  = DEFAULT_IMPORT_BUTTONS | SHOW_REMAP_BUTTON,
        };
    public:
        StaticControlPanel();
        StaticControlPanel( wxWindow* parent,
                            StaticViewersController* controller,
                            int buttons = DEFAULT_BUTTONS,
                            wxWindowID id = wxID_ANY,
                            const wxPoint& pos = wxDefaultPosition,
                            const wxSize& size = wxDefaultSize,
                            long style = wxTAB_TRAVERSAL );
        ~StaticControlPanel();

        bool Create( wxWindow* parent,
                     StaticViewersController* controller,
                     int buttons = DEFAULT_BUTTONS,
                     wxWindowID id = wxID_ANY,
                     const wxPoint& pos = wxDefaultPosition,
                     const wxSize& size = wxDefaultSize,
                     long style = wxTAB_TRAVERSAL );

        void OnKeyDown( wxKeyEvent& event );

        void OnStatusChanged( wxCommandEvent& event );

        ///STDataViewerBase virtual methods
        void    update();

        void    requestRead() {}
        void    updateModel() {}
        void    requestWrite() {}
        wxString toString();

        void setStaticViewersController( StaticViewersController* controller );

    private:
        StaticViewersController* m_controller;

        void setButtonHighlighted( int id, bool highlighted );
        bool CreateControls( int buttons );
        void enableButtons( bool enable = true );
        void enableButton( int id, bool enable = true );

        void OnButton( wxCommandEvent& event );

        DECLARE_EVENT_TABLE();
};

#endif // STATICCONTROLPANEL_H
