#include "StaticViewersController.h"

#include "../stdatamanager/STDataMgrBase.h"
#include "../vemsdialog/CalibrationBase.h"
#include "../logger.h"
#include "../common/HelpUtil.h"
#include "UserDialogBase.h"

bool StaticViewersController::m_helpIsShown = false;

StaticViewersController::StaticViewersController( wxWindow* helpParent, const wxString& helpId )
{
    m_calibrationBase   = NULL;
    m_helpId            = helpId;
    m_helpParent        = helpParent;
    m_dialogManager     = NULL;
    m_dialogName        = wxEmptyString;
}

StaticViewersController::~StaticViewersController()
{
    unregisterViewers();
}

void StaticViewersController::setController( STDataMgrBase* controller )
{
    m_controller = controller;
}

void StaticViewersController::unregisterViewers()
{
    for( uint32_t i = 0; i < m_ViewerBaseA.GetCount(); i++ )
    {
        m_ViewerBaseA.Item( i )->unregisterViewer();
    }
}


void StaticViewersController::refreshData()
{
    m_controller->requestReadPage();
}

void StaticViewersController::writeData()
{
    if ( !m_controller->isConnected() ) return;
    for( uint32_t i = 0; i < m_ViewerBaseA.GetCount(); i++ )
    {
        m_ViewerBaseA.Item( i )->updateModel();
        m_ViewerBaseA.Item( i )->requestWrite();
    }

    refreshData();
}

void StaticViewersController::addViewer(STDataViewerBase * viewer)
{
    m_ViewerBaseA.Add(viewer);
    m_controller->RegisterViewer(viewer);
}

void StaticViewersController::removeViewer(STDataViewerBase * viewer)
{
    m_ViewerBaseA.Remove(viewer);
    //m_controller->UnRegisterViewer(viewer);
}

void StaticViewersController::burnData()
{
    if ( !m_controller->isConnected() ) return;
    writeData();
    m_controller->requestBurn();
}


/**
 * @name    updateData
 *
 * @brief   update viwers gui from model
 *
 * @params  callSetStatus - if true ( default ) call setStatus function
*/
void StaticViewersController::updateData( bool callSetStatus )
{
    for( uint32_t i = 0; i < m_ViewerBaseA.GetCount(); i++ )
    {
        m_ViewerBaseA.Item( i )->update();
    }

    //if( callSetStatus ) { setStatus(); }
}

void StaticViewersController::requestReadAllDirty()
{
    m_controller->requestReadAllDirty();
}

void StaticViewersController::updateRegisteredViewers()
{
    m_controller->updateRegisteredViewers();
}

STDataModelBase* StaticViewersController::getModel()
{
    return m_controller->getModel();
}

void StaticViewersController::setCalibration(CalibrationBase* calibrationBase )
{
    m_calibrationBase = calibrationBase;
    m_calibrationBase->SetDataMgrObj(m_controller);
}

void StaticViewersController::startCalibration()
{
    Logger::logf(LOG_MSG, _T("StaticViewersController::startCalibration() Start Clalibration()"));
    if(m_calibrationBase){
        Logger::logf(LOG_MSG, _T("StaticViewersController::startCalibration()2 Start Clalibration()"));
        m_calibrationBase->start();
    }
}

void StaticViewersController::stopCalibration()
{
    if(m_calibrationBase) m_calibrationBase->stop();
}

void StaticViewersController::showHelp()
{
    if ( m_helpId.IsEmpty() ) return;

    if ( !m_helpParent ) return;

#ifndef __WINCE_STANDARDSDK__
	m_helpIsShown = HelpUtil::showDialogHelp( m_helpParent, m_helpId );
#endif
}

void StaticViewersController::setUserDialogManager( UserDialogBase* dialogManager )
{
    m_dialogManager = dialogManager;
}

void StaticViewersController::changeToPreviousDialog()
{
    for( uint32_t i = 0; i < m_ViewerBaseA.GetCount(); i++ )
    {
        m_ViewerBaseA.Item( i )->updateModel();
        m_ViewerBaseA.Item( i )->requestWrite();
    }

    if ( m_controller->isConnected() ) { refreshData(); }

    if ( m_dialogManager ) { m_dialogManager->changeToPreviousDialog( m_dialogName ); }
}

void StaticViewersController::changeToNextDialog()
{
    for( uint32_t i = 0; i < m_ViewerBaseA.GetCount(); i++ )
    {
        m_ViewerBaseA.Item( i )->updateModel();
        m_ViewerBaseA.Item( i )->requestWrite();
    }

    if ( m_controller->isConnected() ) { refreshData(); }

    if ( m_dialogManager ) { m_dialogManager->changeToNextDialog( m_dialogName ); }
}
