#include "SuperCurveEditor.h"
#include "SuperCurveDescriptor.h"
#include "VemsCurveDialogDesc.h"
#include "CurveEditor.h"
#include "UserDialogFrameBase.h"
#include "UserDialogBase.h"
#include "StaticControlPanel.h"
#include "../stdatamanager/STDataMgrBase.h"


SuperCurveEditor::SuperCurveEditor( UserDialogFrameBase* parent,
                                    SuperCurveDescriptor* superCurveDesc,
                                    VemsCurveDialogDescH* curveDescs,
                                    UserDialogBase* dialogManager,
                                    STDataMgrBase* staticController,
                                    RTDataMgrBase* realtimeController,
                                    SymbolMgr* symbolManager )
    : wxPanel ( parent )
{
    m_superCurveDesc        = superCurveDesc;
    m_curveDescs            = curveDescs;
    m_dialogManager         = dialogManager;
    m_realtimeController    = realtimeController;
    m_symbolManager         = symbolManager;

    SetDataMgr( staticController );

    m_curveEditor           = NULL;

    SetSizer( new wxBoxSizer( wxVERTICAL ) );

    update();

    staticController->RegisterViewer( this );
}

SuperCurveEditor::~SuperCurveEditor()
{
    unregisterViewer();
}

CurveEditor* SuperCurveEditor::getCurrentCurve()
{
    return m_curveEditor;
}

void SuperCurveEditor::update()
{
    const SuperCurveRecordArray& records = m_superCurveDesc->records;

    for ( size_t i = 0; i < records.GetCount(); ++i ) {
        const SuperCurveRecord& record = records[i];

        if ( record.expr.IsEmpty() ) {
            continue;
        }

        if ( m_stDataMgr->calculate( record.expr ) ) {
            changeToCurve( record.curveName );
            return;
        }
    }

    if ( records.GetCount() > 0 ) {
        const SuperCurveRecord& record = records.Last();
        changeToCurve( record.curveName );
    } else {
        LOG
    }
}

void SuperCurveEditor::changeToCurve( const wxString& name )
{
    if ( m_curveEditor ) {
        if ( m_curveEditor->getDialogName().IsSameAs( name ) ) {
            return;
        }
    }

    VemsCurveDialogDescH::const_iterator it =  m_curveDescs->find( name );

    if ( it != m_curveDescs->end() ) {
        VemsCurveDialogDesc* curveDesc = it->second;
        createCurve( curveDesc );
    }
}

void SuperCurveEditor::createCurve( VemsCurveDialogDesc* curveDesc )
{
    GetSizer()->Detach( m_curveEditor );

    delete m_curveEditor;

    m_curveEditor = new CurveEditor( this, curveDesc, m_symbolManager );
    m_curveEditor->init( m_stDataMgr, m_realtimeController );
    m_curveEditor->setUserDialogName( m_superCurveDesc->name );
    m_curveEditor->setUserDialogManager( m_dialogManager );

    GetSizer()->Add( m_curveEditor, 1, wxEXPAND );
    GetSizer()->Layout();

    UserDialogFrameBase* frame =  (UserDialogFrameBase*)GetParent();

    frame->setStaticViewersController( m_curveEditor );

    if ( !frame->IsShown() ) {
        frame->GetSizer()->Add( this, 1, wxEXPAND );
    }

    if ( frame->getControlPanel() ) {

    } else {
        frame->addControlPanel( m_curveEditor, StaticControlPanel::DEFAULT_IMPORT_BUTTONS );
    }

    if ( !frame->IsShown() ) {
        frame->setLayout( m_curveEditor->GetLabel() );
    } else {
        frame->SetTitle( m_curveEditor->GetLabel() );
    }

    m_curveEditor->updateData();
}



