#ifndef TABLEEDITOR_H
#define TABLEEDITOR_H

#include "wx/wx.h"
#include "StaticViewersController.h"
#include "../Symbol.h"
#include "../rtdatamanager/RTDataViewerBase.h"
#include "../stdatamanager/STDataViewerBase.h"
#include "../common/VemsMatrixHelper.h"
#include "../gaugedesc/GaugeProperties.h"
#include "../vemsgauge/VemsControlBase.h"
#include "../vemsgauge/SelectableWidgetBase.h"
#include <wx/control.h>


class FirmwareConfigData;
class FirmwareConfigLoader;
class FocusParentSinkerBase;
class GaugeDescManager;
class GaugeGroupManager;
class GroupMember;
class MTConfigData;
class RTDataMgrBase;
class TableEditor3D;
class TableEditor;
class TableEditorEvent;
class UpdateViewerEvent;
class VEMSTableEditor;
class VemsTableDesc;


/**********************************************************
*   STViewerTableRowHeader
**********************************************************/
class STViewerTableRowHeader : public STDataViewerBase
{
    public:
        STViewerTableRowHeader( TableEditor* table );

        void update();
        void requestRead();
        void updateModel();
        void updateSingleData( int row );
        void requestWriteSingleData( int row );
        void requestWrite();

        wxString getSymbolName() const { return m_symbol.getName(); }
        void setSymbol( const Symbol& symbol_in ) { m_symbol = symbol_in; }

        Symbol      getSymbol() const { return m_symbol; }
        wxString    toString();
    private:
        TableEditor*    m_table;
        Symbol          m_symbol;
};


/**********************************************************
*   STViewerTableColHeader
**********************************************************/
class STViewerTableColHeader : public STDataViewerBase
{
    public:
        STViewerTableColHeader( TableEditor* table );

        void update();
        void requestRead();
        void updateModel();
        void updateSingleData( int col );
        void requestWriteSingleData( int col );
        void requestWrite();

        wxString getSymbolName() const { return m_symbol.getName(); }
        void setSymbol( const Symbol& symbol_in ) { m_symbol = symbol_in; }

        Symbol      getSymbol() const { return m_symbol; }
        wxString    toString();

    private:
        TableEditor*    m_table;
        Symbol          m_symbol;
};


/**********************************************************
*   STViewerTable
**********************************************************/
class STViewerTable : public STDataViewerBase
{
    public:
        STViewerTable( TableEditor* table );

        void update();
        void updateTable( const wxString& tableName, wxPoint cursorPosition );

        void requestRead();
        void updateModel();
        void updateSingleData( int row, int col );
        void requestWriteSingleData( int row, int col );
        void requestWrite();

        wxString getSymbolName() const { return m_symbol.getName(); }
        void setSymbol( const Symbol& symbol_in ) { m_symbol = symbol_in; }

        Symbol      getSymbol() const { return m_symbol; }
        wxString    toString();

    private:
        TableEditor*    m_table;
        Symbol          m_symbol;
};

/**********************************************************
*   TableEditor
**********************************************************/
class TableEditor : public wxPanel, public RTDataViewerBase, public StaticViewersController, public VemsControlBase, public SelectableWidgetBase
{
    public:
        TableEditor( wxWindow* parent, GaugeDescManager * dmgr,  GaugeProperties &prop, bool showTitle = false);
         //const wxPoint &pos = wxDefaultPosition, const wxSize &size = wxDefaultSize, GaugeGroupManager* groupManager = NULL, const wxString groupName = wxEmptyString );
        ~TableEditor();

        static StaticViewersController* newInstance(
            wxWindow* parent, const VemsTableDesc& desc, STDataMgrBase* stMgr, RTDataMgrBase* rtMgr,
            MTConfigData* mtconfigdata, GaugeDescManager* gaugeDescMgr, UserDialogBase* udMgr);

        void init( const wxString& name, STDataMgrBase* stController, RTDataMgrBase* rtController, MTConfigData* mtconfigdata);

        virtual void SetNewTheme(WidgetThemeTable *theme);

        void SetValue( double value );
        void refresh( DoubleHash* table );
        //void require( DoubleHash* table);
        void require();
        void unregisterSymbols();
        void showRemapDialog();

        virtual wxRegion GetShape();
        virtual wxRegion GetBorderRegion();

        void SetValueIdx( int idx );
        void updateSTData();
        void OnUpdateValue( UpdateViewerEvent& event );

        //
        void SetCellValue( int row, int col, double value );
        void SetRowLabelValue( int row, double value );
        void AcceptModifiedRowLabelValue( int row );
        int GetRowLabelValue(int row );								// Return caption of the specified column
        void SetColLabelValue( int col, double value );
        void AcceptModifiedColLabelValue( int col );
        int GetColLabelValue(int col);								// Return caption of the specified column

        double GetCellValue( int row, int col );

        int GetNumberRows();
        int GetNumberCols();

        bool WasCellModified( int row, int col );
        void AcceptAllModifiedCells();
        void AcceptSingleModifiedCell( int row, int col );
        bool is3D() const { return m_is3D; }

        void exportToFile( const wxString& file );
        void importFromFile(  const wxString& file );
        void showTable3D();
        void change( const wxString& name );    // change table
        void        OnKeyDown( wxKeyEvent& event );
        void        OnKeyUp( wxKeyEvent& event );

        GroupMember * GetGrpMember() { return m_grpMember; };

        wxWindow    * GetTable();

        bool onDropFiles( const wxArrayString& filenames );
        void moveCursorToPosition( const wxPoint& position );
        wxString getName() const { return m_name; }


        virtual void b_refreshData() { refreshData(); };
        virtual void b_writeData() { writeData(); };
        virtual void b_burnData() { burnData(); };
        virtual void b_showTable3D() { showTable3D(); };
        virtual void b_SetFocus() { GetTable()->SetFocus(); };
        virtual void b_UnRegisterViewer() { UnRegisterViewer(); };
        virtual void b_SetSize(const wxSize& size);

        virtual void ProcessWidgetCommand(wxVariantHash& cmdHash);

        void SetFocusParentSinker(FocusParentSinkerBase* fps);

    private:
        VEMSTableEditor*        m_table;
        TableEditor3D*          m_table3D;
        STViewerTableRowHeader* m_rowHeaderViewer;
        STViewerTableColHeader* m_colHeaderViewer;
        STViewerTable*          m_tableViewer;
        RTDataMgrBase*          m_pRTController;
        wxString                m_loadHSymbolName;
        wxString                m_loadVSymbolName;
        wxString                m_loadValueSymbolName;
        wxString                m_name;
        double                  m_loadHValue;
        double                  m_loadVValue;
        double                  m_loadValue;
        bool                    m_isRTSymbolRegistered;
        bool                    m_bUseRealtimeTarget;
        GaugeGroupManager*      m_groupManager;
        wxString                m_groupName;
        bool                    m_is3D;
        GroupMember*            m_grpMember;
        GaugeProperties         m_prop;
        GaugeDescManager*       m_gaugeDescMgr;
        bool                    m_hasTitle;

    private:
        wxString                findTableNameByTitle( const wxString& title );
        void                    init( const wxString& name );
        void                    loadTable( FirmwareConfigLoader& configLoader );
        void                    loadTableWithoutHeaders( FirmwareConfigLoader& configLoader );
        void                    loadTableWithHeaders( FirmwareConfigLoader& configLoader );

        void                    OnTableEditorEvent( TableEditorEvent& event );
        void                    OnSize(wxSizeEvent &evt);
        void                    OnSetFocus( wxFocusEvent& WXUNUSED(event) );
        void                    OnKillFocus( wxFocusEvent& WXUNUSED(event) );


        MTConfigData*           m_mtConfigData;

        FocusParentSinkerBase*  m_focusSinker;

        DECLARE_EVENT_TABLE();
};

#endif // TABLEEDITOR_H
