#include "UserDialogManager.h"

#include "UserMenu.h"
#include "UserDialog.h"

#include "UserDialogFrame.h"
#include "../mtconfigdata.h"
#include "../logger.h"
#include "../Utils.h"
#include "../vemsgauge/VemsTableDesc.h"
#include "CheckBoxMatrixViewer.h"
#include "DialogsLayoutManager.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../stdatamanager/STDataModelBase.h"

#include "../customwidgets/InfoWindow.h"
#include "../firmwareconfig/ConfigValidator.h"


class StringObject : public wxObject
{
    public:
        wxString string;

        StringObject( const wxString& s )
        {
            string = s;
        }
};

int UDM_TIMER_ID = wxNewId();

IMPLEMENT_DYNAMIC_CLASS(UserDialogManager, UserDialogBase )


/** @brief UserDialogManager
  *
  * @todo: document this function
  */
UserDialogManager::UserDialogManager( wxWindow* window,
                                      MTConfigData* configData,
                                      STDataMgrBase* staticController,
                                      RTDataMgrBase* realtimeController,
                                      CommandRequestBase* requestMgr,
                                      wxMenuBar* menuBar,
                                      GaugeDescManager* gaugeDescMgr,
                                      const wxString& strFirmwareVersion,
                                      int menuBarStartPos )
    : m_mtConfigData ( configData )
    , m_staticController ( staticController )
    , m_realtimeController ( realtimeController )
    , m_requestMgr (requestMgr)
    , m_gaugeDescMgr (gaugeDescMgr)
    , m_strFirmwareVersion ( strFirmwareVersion )
    , m_timer ( this, UDM_TIMER_ID )

{
    m_currFrame = NULL;
    m_window = window;
    m_dlgLayoutMgr = new DialogsLayoutManager();
    m_dlgLayoutMgr->OpenDB();

    m_window->PushEventHandler( this );
    createMenu( menuBar, menuBarStartPos );
    m_menuBar = menuBar;
    Connect( UDM_TIMER_ID, wxEVT_TIMER, wxTimerEventHandler(UserDialogManager::OnTimer) );

    m_configValidator = new ConfigValidator;
    staticController->setConfigValidator( m_configValidator );

    //m_parentMenu = NULL;
}


UserDialogManager::UserDialogManager( wxWindow* window,
                                      MTConfigData* configData,
                                      STDataMgrBase* staticController,
                                      RTDataMgrBase* realtimeController,
                                      CommandRequestBase* requestMgr,
                                      wxMenu* parentMenu,
                                      GaugeDescManager* gaugeDescMgr,
                                      const wxString& strFirmwareVersion )
    : m_mtConfigData ( configData )
    , m_staticController ( staticController )
    , m_realtimeController ( realtimeController )
    , m_requestMgr (requestMgr)
    , m_gaugeDescMgr (gaugeDescMgr)
    , m_strFirmwareVersion ( strFirmwareVersion )
    , m_timer ( this, UDM_TIMER_ID )
{
    m_currFrame = NULL;
    m_window = window;
    m_dlgLayoutMgr = new DialogsLayoutManager();
    m_dlgLayoutMgr->OpenDB();

    m_window->PushEventHandler( this );
    createMenu( parentMenu );
    Connect( UDM_TIMER_ID, wxEVT_TIMER, wxTimerEventHandler(UserDialogManager::OnTimer) );

    m_configValidator = new ConfigValidator;
    staticController->setConfigValidator( m_configValidator );
    //m_menuBar = menuBar;
}

/** @brief ~UserDialogManager
  *
  * @todo: document this function
  */
UserDialogManager::~UserDialogManager()
{
    m_window->PopEventHandler();
}

void UserDialogManager::removeMenu( wxMenuBar* parent )
{
    //Logger::logf( LOG_MSG, _T("UserDialogManager::removeMenu() - START") );
    for ( size_t i = 0; i < m_menuTitles.GetCount(); ++i )
    {
        //Logger::logf( LOG_MSG, _T("UserDialogManager::removeMenu() - ") + m_menuTitles[i] );
        int menuIndex = parent->FindMenu( m_menuTitles[i] );
        if ( menuIndex != wxNOT_FOUND )
        {
            wxMenu* menu = parent->Remove( menuIndex );
            delete menu;
        }
    }

    m_menuTitles.Clear();
    m_IDToMenuDesc.clear();
}

void UserDialogManager::createMenu( wxMenuBar* parent, int menuBarStartPos)
{
    VemsTableDescH*             tables          = m_mtConfigData->getTables();
    VemsCurveDialogDescH&       curves          = m_mtConfigData->getCurveDialogs();
    SuperCurveDescriptorH&      superCurves     = m_mtConfigData->getSuperCurveDialogs();
    UserDialogList*             udl             = m_mtConfigData->getUserDialogList();
    UserMenuList*               uml             = m_mtConfigData->getUserMenuList();
    TblMapDescHash&             mappers         = m_mtConfigData->m_tableMappers;
    InterpolationWidgetDescH&   interpolations  = m_mtConfigData->m_interpolationWidgets;

    m_menuIdToDialogId.clear();
    m_name2Id.clear();

    if( menuBarStartPos == -1 ) menuBarStartPos = 2;

    for(size_t i=0; i < uml->getCount(); i++){
        UserMenu* um =(*uml)[i];

        if( parent->FindMenu(um->getLabel()) != wxNOT_FOUND) continue;

        wxMenu* menu = new wxMenu();

        for(size_t j=0; j<um->getCount(); j++)
        {
            UserMenuItem* umi = (*um)[j];

            if(umi->isSeparator())
            {
                menu->AppendSeparator();
            }
            else
            {
                int id = wxNewId();
                menu->Append(id, umi->getLabel(), umi->getLabel());
                Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(UserDialogManager::OnMenu) );

                wxString dialogName = umi->getDialog();

                UserDialog* ud = udl->find( dialogName );
                m_name2Id[dialogName] = id;
                m_menuIdToDialogId[id] = dialogName;
                m_menuItems.Add( id );

                //Logger::logf( LOG_MSG, _T("menuId: %d, dialogId: ") + dialogName, id );

                if( ud )
                {
                    m_IDToMenuDesc[id] = ud;
                    ///TEST
                    if(ud->getName() == _T("injOutVisual")){

                        wxObject* obj = CheckBoxMatrixViewer::getTestDesc();
                        m_IDToMenuDesc[id] = obj;
                    }
                    else if ( ud->getName().IsSameAs( _T("outputChannels") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                    }
                    else if ( ud->getName().IsSameAs( _T("ignitionOutputsVisual") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                    }
                    else if ( ud->getName().IsSameAs( _T("roundMainDisplay") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                }
                    else if ( ud->getName().IsSameAs(_T("ledDisplayDemo") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                    }
                }
                else if( tables->find( dialogName ) != tables->end() )
                {
                     m_IDToMenuDesc[id] = (*tables)[ dialogName ];
                }
                else if( curves.find( dialogName ) != curves.end() )
                {
                    m_IDToMenuDesc[id] = curves[ dialogName ];
                }
                else if( superCurves.find( dialogName ) != superCurves.end() )
                {
                    m_IDToMenuDesc[id] = superCurves[ dialogName ];
                }
                else if ( mappers.find( dialogName) != mappers.end() )
                {
                    m_IDToMenuDesc[id] = mappers[ dialogName ];
                }
                else if ( interpolations.find( dialogName ) != interpolations.end() )
                {
                    m_IDToMenuDesc[id] = interpolations[ dialogName ];
                }
                else
                {
                    menu->Enable( id, false );
                }
            }
        }

        parent->Insert( menuBarStartPos + i, menu, um->getLabel() );
        m_menuTitles.Add( um->getLabel() );
    }
}


void UserDialogManager::createMenu( wxMenu* parent )
{
    VemsTableDescH*             tables          = m_mtConfigData->getTables();
    VemsCurveDialogDescH&       curves          = m_mtConfigData->getCurveDialogs();
    UserDialogList*             udl             = m_mtConfigData->getUserDialogList();
    UserMenuList*               uml             = m_mtConfigData->getUserMenuList();
    TblMapDescHash&             mappers         = m_mtConfigData->m_tableMappers;
    InterpolationWidgetDescH&   interpolations  = m_mtConfigData->m_interpolationWidgets;

    for(size_t i=0; i < uml->getCount(); i++)
    {
        UserMenu* um =(*uml)[i];

        for(size_t j=0; j<um->getCount(); j++)
        {
            UserMenuItem* umi = (*um)[j];

            if(umi->isSeparator())
            {
                parent->AppendSeparator();
            }
            else
            {
                int id = wxNewId();
                parent->Append(id, umi->getLabel(), umi->getLabel());
                Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(UserDialogManager::OnMenu) );

                wxString dialogName = umi->getDialog();
                UserDialog* ud = udl->find( dialogName );

                if( ud )
                {
                    m_IDToMenuDesc[ id ] = ud;
                    ///TEST
                    if(ud->getName() == _T("injOutVisual") ){

                        wxObject* obj = CheckBoxMatrixViewer::getTestDesc();
                         m_IDToMenuDesc[id] = obj;
                    }
                    else if ( ud->getName().IsSameAs( _T("outputChannels") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                    }
                    else if ( ud->getName().IsSameAs( _T("ignitionOutputsVisual") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                    }
                    else if ( ud->getName().IsSameAs( _T("roundMainDisplay") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                }
                    else if ( ud->getName().IsSameAs(_T("ledDisplayDemo") ) )
                    {
                        wxObject* obj = new StringObject( ud->getName() );
                        m_IDToMenuDesc[id] = obj;
                    }
                }
                else if( tables->find( dialogName ) != tables->end() )
                {
                    m_IDToMenuDesc[ id ] = (*tables)[ dialogName ];
                }
                else if( curves.find( dialogName ) != curves.end() )
                {
                    m_IDToMenuDesc[ id ] =  curves[ dialogName ];
                }
                else if ( mappers.find( dialogName ) != mappers.end() )
                {
                    m_IDToMenuDesc[ id ] = mappers[ dialogName ];
                }
                else if ( interpolations.find( dialogName ) != interpolations.end() )
                {
                    m_IDToMenuDesc[id] = interpolations[ dialogName ];
                }
                else
                {
                    parent->Enable( id, false );
                }
            }
        }
    }
}


bool UserDialogManager::showDialog( wxObject* objDesc )
{
    bool res = false;

    res = objDesc->IsKindOf( CLASSINFO(UserDialog) );

    if(res){
        showDialog( (UserDialog*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(VemsTableDesc) );
    if(res){
        showTable( (VemsTableDesc*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(VemsCurveDialogDesc) );
    if(res){
        showCurve( (VemsCurveDialogDesc*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(SuperCurveDescriptor) );
    if(res){
        showSuperCurve( (SuperCurveDescriptor*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(InterpolationWidgetDescriptor) );
    if(res){
        showInterpolationWidget( (InterpolationWidgetDescriptor*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(TblMapDesc) );
    if(res){
        showMapper( (TblMapDesc*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(CheckBoxMatrixDesc) );
    if(res){
        show( (CheckBoxMatrixDesc*) objDesc );
        return true;
    }

    res = objDesc->IsKindOf( CLASSINFO(StringObject) );
    if(res){
        wxString dialogName = ((StringObject*)objDesc)->string;

        if ( dialogName.IsSameAs( _T("outputChannels") ) )
        {
            showOutputChannelsDialog( dialogName );
            return true;
        }

        if ( dialogName.IsSameAs( _T("ignitionOutputsVisual") ))
        {
            showIgnitionOutputsVisualDialog( dialogName );
            return true;
        }

        if ( dialogName.IsSameAs( _T("roundMainDisplay") ) )
        {
            showRoundDisplaySettingsDialog( dialogName );
    }

        if ( dialogName.IsSameAs( _T("ledDisplayDemo") ) )
        {
            showRoundDisplayDemoDialog( dialogName );
        }
    }

    return false;
}

bool UserDialogManager::showDialog( UserDialog* ud )
{
//    wxWindow w = getWindow(ud);
//    if(w) w->


    //StaticViewersController* stViewersCtrl = STViewersFactory::create();
    //m_currFrame = new  UserDialogFrameBase(m_window, stViewersCtrl, title);

    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createDialog( *ud, &(m_mtConfigData->getSymbolMgr()), m_staticController );
    m_symbolsToMark.Clear();
    return true;
}


bool UserDialogManager::showTable( VemsTableDesc* desc )
{
    m_currFrame = new UserDialogFrame( m_window ,  m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createTable( *desc, m_staticController, m_realtimeController, m_mtConfigData, m_gaugeDescMgr );
    return true;
}


bool UserDialogManager::showCurve( VemsCurveDialogDesc* desc )
{
    m_currFrame = new UserDialogFrame( m_window ,  m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createCurve( desc, m_staticController, m_realtimeController, &m_mtConfigData->getSymbolMgr() );
    return true;
}

bool UserDialogManager::showSuperCurve( SuperCurveDescriptor* desc )
{
    m_currFrame = new UserDialogFrame( m_window ,  m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createSuperCurve(  desc, &m_mtConfigData->getCurveDialogs(),
                                    m_staticController, m_realtimeController, &m_mtConfigData->getSymbolMgr() );
    return true;
}

 bool UserDialogManager::showInterpolationWidget( InterpolationWidgetDescriptor* desc )
 {
    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createInterpolationWidget( desc, m_staticController, m_realtimeController );
    return true;
 }

bool UserDialogManager::showMapper( TblMapDesc* desc )
{
    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr );
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createMapper( *desc, m_staticController, &m_mtConfigData->getSymbolTable() );
    return true;
}

bool UserDialogManager::show( CheckBoxMatrixDesc* desc )
{
    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->create( &m_mtConfigData->getSymbolMgr(), desc, m_staticController, m_requestMgr);
    return true;
}


bool UserDialogManager::showOutputChannelsDialog( const wxString& name )
{
    wxString title = _T("Output Channels");
    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createOutputChannelsDialog( title, m_staticController, m_requestMgr,
        &m_mtConfigData->getSymbolTable() );
    return true;
}

bool UserDialogManager::showIgnitionOutputsVisualDialog( const wxString& name )
{
    wxString title = _T("h[2] table - Ignition outputs");
    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->setSymbolsToMark( m_symbolsToMark );
    m_currFrame->createIgnitionOutputsVisual( title, m_staticController, &m_mtConfigData->getSymbolTable(), m_requestMgr );
    return true;
}

bool UserDialogManager::showRoundDisplaySettingsDialog( const wxString& name )
{
    FirmwareVersion fwVersion( m_strFirmwareVersion );
    wxString title = _T("Round Main Display Settings");
    m_currFrame = new UserDialogFrame( m_window, m_configValidator, this, m_dlgLayoutMgr );
    m_currFrame->createRoundDisplaySettingsDialog( title, m_staticController, m_realtimeController, &m_mtConfigData->getSymbolTable(),
        &m_mtConfigData->getSymbolMgr(), fwVersion, m_requestMgr );
    return true;
}

bool UserDialogManager::showRoundDisplayDemoDialog( const wxString& name )
{
    m_currFrame = new UserDialogFrame( m_window , m_configValidator, this, m_dlgLayoutMgr);
    m_currFrame->createRoundDisplayDemoDialog( _T("Round Display Demo Settings"), m_staticController, &m_mtConfigData->getSymbolTable(),
        &m_mtConfigData->getSymbolMgr() );
    return true;
}

/** @brief OnMenu
  *
  * @todo: document this function
  */
void UserDialogManager::OnMenu( wxCommandEvent& event )
{
    int id = event.GetId();
    //show user dialog
    showDialog(id);
}

void UserDialogManager::unActivate(int id)
{
    m_IDToActiveFrames.erase(id);

    wxString dialogId = m_menuIdToDialogId[id];
    m_activeDialogs.erase( dialogId );

    Logger::logf( LOG_MSG, _T("UserDialogManager::unActivate() - menuId: %d, dialogId: ")
                + dialogId, id );
}

bool UserDialogManager::showActiveFrame(int id)
{
    wxObjectHash::const_iterator IT = m_IDToActiveFrames.find(id);
    //if(IT != m_IDToMenuDesc.end() ){
    if(IT == m_IDToActiveFrames.end() ) return false;

    wxObject* obj = IT->second;
    wxFrame* frame = (wxFrame*)obj;
    frame->Maximize(false);
    frame->SetFocus();
    return true;
}

void UserDialogManager::showDialog(int id)
{
    if( showActiveFrame(id) ) return;

    wxObjectHash::const_iterator IT = m_IDToMenuDesc.find(id);
    if(IT == m_IDToMenuDesc.end() ){
		 return;
    }
    wxObject* descObj = IT->second;
    showDialog( descObj );
    m_currFrame->setMenuId(id);
    m_IDToActiveFrames[id] = m_currFrame;

    wxString dialogId = m_menuIdToDialogId[id];
    m_activeDialogs.insert( dialogId );

    m_currFrame->showHelp();


    Logger::logf( LOG_MSG, _T("UserDialogManager::showDialog() - menuId: %d, dialogId: ") + dialogId, id );
}

void UserDialogManager::showDialog( const wxString& menuName, const wxString& symbolsToMark )
{
    m_symbolsToMark.Clear();
    if ( symbolsToMark != _T("") )
        Utils::tokenizeToArray( const_cast<wxString&>(symbolsToMark), _T(","), m_symbolsToMark );

    String2IntHash::const_iterator IT = m_name2Id.find(menuName);
    m_dlgName = menuName;
    if(IT == m_name2Id.end() ){
         //wxMessageBox( menuName +_T(" dialog not found") );
        return;
    }
    int id = IT->second;
    showDialog(id);
}


void UserDialogManager::closeDialog( int id, bool needRestore )
{
    wxObjectHash::const_iterator IT = m_IDToActiveFrames.find(id);
    if ( IT == m_IDToActiveFrames.end() ) return;

    wxObject* obj = IT->second;
    wxFrame* frame = (wxFrame*)obj;
    m_IDToActiveFrames.erase( IT );
    frame->Destroy();

    wxString dialogId = m_menuIdToDialogId[id];

    if ( !needRestore )
    {
        //wxString dialogId = m_menuIdToDialogId[id];
        m_activeDialogs.erase( dialogId );
    }

    Logger::logf( LOG_MSG, _T("UserDialogManager::closeDialog() - menuId: %d, dialogId: ")
                + dialogId + _T(", needRestore: %d"), id, needRestore );
}

void UserDialogManager::setFirmwareVersion( const wxString& strFirmwareVersion )
{
    m_strFirmwareVersion = strFirmwareVersion;
}

void UserDialogManager::refreshMenu( wxMenuBar* menuBar )
{
    UserMenuList* uml = m_mtConfigData->getUserMenuList();

    for ( size_t i=0; i < uml->getCount(); ++i )
    {

        UserMenu* um =(*uml)[i];

        for( size_t j=0; j<um->getCount(); ++j )
        {
            UserMenuItem* umi = (*um)[j];

            if( umi->isSeparator() )
            {
                continue;
            }

            if ( umi->getExpr().IsEmpty() )
            {
                continue;
            }

            int id = menuBar->FindMenuItem( um->getLabel(), umi->getLabel() );

            if ( id == wxNOT_FOUND )
            {
                continue;
            }

            wxMenuItem* mi = menuBar->FindItem( id );

            if ( !mi )
            {
                continue;
            }

            bool enabled = true;

            /// todo parse expression
            m_rpn.setVariableData( m_staticController->getModel() );
            enabled = m_rpn.calculate( umi->getExpr() )!=0?true:false;

            mi->Enable( enabled );
        }
    }
}



void UserDialogManager::changeToPreviousDialog( const wxString& dialogName )
{
    String2IntHash::const_iterator it = m_name2Id.find( dialogName );

    if ( it == m_name2Id.end() )
    {
        Logger::logf( LOG_MSG, _T("UserDialogManager::changeToPreviousDialog() - cannot find menuitem for dialog: ") + dialogName );
        return;
    }

    int menuItemId = it->second;

    int menuItemIndex = m_menuItems.Index( menuItemId );

    if ( menuItemIndex == wxNOT_FOUND )
    {
        Logger::logf( LOG_MSG, _T("UserDialogManager::changeToPreviousDialog() - cannot find menuitem for id: %d, dialog: ")
                        + dialogName,menuItemId );
        m_dlgName = dialogName;
    }

    for ( int i = menuItemIndex - 1; i >= 0; --i )
    {
        int previousMenuItemId = m_menuItems[i];

        wxMenuItem* menuItem = m_menuBar->FindItem( previousMenuItemId );

        if ( !menuItem ) continue;
        if ( !menuItem->IsEnabled() ) continue;
        //showDialog( menuItem->GetId() );
        closeDialog( menuItemId );
        m_nextDialogId = menuItem->GetId();

        if( showActiveFrame(m_nextDialogId) ) return;

        wxObjectHash::const_iterator IT = m_IDToMenuDesc.find( m_nextDialogId );

        if(IT == m_IDToMenuDesc.end() )
        {
            InfoWindow::Display( NULL, _T("Warning"), _T("Could not find dialog before\nthe ") + dialogName + _T(" dialog"), IW_FADE_IN | IW_MOVE | IW_WARNING, 3500 );
        }
        else
        {
        m_timer.Start( 200, true );
        break;
    }

    }
}

void UserDialogManager::changeToNextDialog( const wxString& dialogName )
{
    String2IntHash::const_iterator it = m_name2Id.find( dialogName );

    if ( it == m_name2Id.end() )
    {
        Logger::logf( LOG_MSG, _T("UserDialogManager::changeToPreviousDialog() - cannot find menuitem for dialog: ") + dialogName );
        return;
    }

    int menuItemId = it->second;

    int menuItemIndex = m_menuItems.Index( menuItemId );

    if ( menuItemIndex == wxNOT_FOUND )
    {
        Logger::logf( LOG_MSG, _T("UserDialogManager::changeToPreviousDialog() - cannot find menuitem for id: %d, dialog: ")
                        + dialogName,menuItemId );
        m_dlgName = dialogName;
    }

    for ( size_t i = menuItemIndex+1; i < m_menuItems.GetCount(); ++i )
    {
        int nextMenuItemId = m_menuItems[i];

        wxMenuItem* menuItem = m_menuBar->FindItem( nextMenuItemId );

        if ( !menuItem ) continue;
        if ( !menuItem->IsEnabled() ) continue;
        closeDialog( menuItemId );
        //showDialog( menuItem->GetId() );
        m_nextDialogId = menuItem->GetId();

        if( showActiveFrame(m_nextDialogId) ) return;

        wxObjectHash::const_iterator IT = m_IDToMenuDesc.find( m_nextDialogId );

        if(IT == m_IDToMenuDesc.end() )
        {
            InfoWindow::Display( NULL, _T("Warning"), _T("Could not find dialog after the ") + dialogName + _T(" dialog"), IW_FADE_IN | IW_MOVE | IW_WARNING, 3500 );
        }
        else
        {
        m_timer.Start( 200, true );
        break;
    }
    }
}

void UserDialogManager::closeAllDialog()
{
    Logger::logf( LOG_MSG, _T("UserDialogManager::closeAllDialog()") );

    StringSet::const_iterator itDialogId = m_activeDialogs.begin();

    for ( ; itDialogId != m_activeDialogs.end(); ++itDialogId )
    {
        wxString dialogId = *itDialogId;

        Logger::logf( LOG_MSG, _T("\t") + dialogId );

        String2IntHash::const_iterator itMenuId = m_name2Id.find( dialogId );

        if ( itMenuId == m_name2Id.end() ) continue;

        int menuId = itMenuId->second;

        closeDialog( menuId, true );
    }
}

void UserDialogManager::restoreDialogs()
{
    Logger::logf( LOG_MSG, _T("UserDialogManager::restoreDialogs()") );

    StringSet removedDialogs;
    StringSet::const_iterator itDialogId = m_activeDialogs.begin();

    for ( ; itDialogId != m_activeDialogs.end(); ++itDialogId )
    {
        wxString dialogId = *itDialogId;

        Logger::logf( LOG_MSG, _T("\t") + dialogId );

        String2IntHash::const_iterator itMenuId = m_name2Id.find( dialogId );

        if ( itMenuId == m_name2Id.end() )
        {
            removedDialogs.insert( dialogId );
        }
        else
        {
            int menuId = itMenuId->second;
            showDialog( menuId );
        }
    }

    StringSet::const_iterator itRemovedDialogId = removedDialogs.begin();

    for ( ; itRemovedDialogId != removedDialogs.end(); ++itRemovedDialogId )
    {
        wxString removedDialogId = *itRemovedDialogId;
        m_activeDialogs.erase( removedDialogId );
        Logger::logf( LOG_MSG, _T("UserDialogManager::restoreDialogs() - dialog removed: ") + removedDialogId );
    }
}

void UserDialogManager::OnTimer( wxTimerEvent& event )
{
    showDialog( m_nextDialogId );
}

