/**
 * @name    UserDialogManager
 *
 * @brief   Create menus from UserMenu.Assign dialog to menus.
 *          If a menuitem found in ini file [Menu] section, but belonging
 *          dialog not found, menuitem will be disabled.
*/
#ifndef USERDIALOGMANAGER_H
#define USERDIALOGMANAGER_H

#include <wx/msgdlg.h>
#include <wx/menu.h>
#include "UserDialogBase.h"
#include "../expression/RPNParser.h"

#include "../common/types/wxObjectHash.h"
#include "../common/types/String2IntHash.h"
#include "../common/types/IntArray.h"
#include "../common/types/StringSet.h"
#include "../common/types/Int2StringHash.h"

class MTConfigData;
class STDataMgrBase;
class RTDataMgrBase;
class UserDialog;
class VemsTableDesc;
class VemsCurveDialogDesc;
class SuperCurveDescriptor;
class InterpolationWidgetDescriptor;
class TblMapDesc;
class CheckBoxMatrixDesc;
class DialogsLayoutManager;
class UserDialogFrame;
class CommandRequestBase;
class GaugeDescManager;
class ConfigValidatorBase;

class UserDialogManager : /*public wxEvtHandler,*/ public UserDialogBase
{
    DECLARE_DYNAMIC_CLASS(UserDialogManager)

    public:
        UserDialogManager() {}
        UserDialogManager(  wxWindow* window,
                            MTConfigData* configData,
                            STDataMgrBase* staticController,
                            RTDataMgrBase* realtimeController,
                            CommandRequestBase* requestMgr,
                            wxMenuBar* menuBar,
                            GaugeDescManager* gaugeDescMgr,
                            const wxString& strFirmwareVersion = _T("0.0.0"),
                            int menuBarStartPos = -1);

        UserDialogManager(  wxWindow* window,
                            MTConfigData* configData,
                            STDataMgrBase* staticController,
                            RTDataMgrBase* realtimeController,
                            CommandRequestBase* requestMgr,
                            wxMenu* parentMenu,
                            GaugeDescManager* gaugeDescMgr,
                            const wxString& strFirmwareVersion = _T("0.0.0"));

        virtual ~UserDialogManager();

        void removeMenu( wxMenuBar* menuBar );
        void createMenu( wxMenuBar* menuBar, int menuBarStartPos = -1);
        void refreshMenu( wxMenuBar* menuBar );

        void showDialog( const wxString& menuName, const wxString& symbolsToMark = wxEmptyString );
        void setFirmwareVersion( const wxString& strFirmwareVersion );
        void unActivate(int id);

        void changeToPreviousDialog( const wxString& dialogName );
        void changeToNextDialog( const wxString& dialogName );
        void closeAllDialog();
        void restoreDialogs();

        //MTConfigData* GetMTConfigData() { return m_mtConfigData; }

    private:
        RPNParser               m_rpn;
        wxArrayString           m_menuTitles;
        MTConfigData*           m_mtConfigData;
        STDataMgrBase*          m_staticController;
        RTDataMgrBase*          m_realtimeController;
        CommandRequestBase*     m_requestMgr;
        GaugeDescManager*       m_gaugeDescMgr;
        wxString                m_strFirmwareVersion;
        wxObjectHash            m_IDToMenuDesc;
        String2IntHash          m_name2Id;
        Int2StringHash          m_menuIdToDialogId;
        DialogsLayoutManager*   m_dlgLayoutMgr;
        UserDialogFrame*        m_currFrame;
        wxObjectHash            m_IDToActiveFrames;
        wxMenuBar*              m_menuBar;
        IntArray                m_menuItems;
        StringSet               m_activeDialogs;
        wxTimer                 m_timer;
        int                     m_nextDialogId;

        bool                    m_forwardDir; // next/prev dialog?
        wxString                m_dlgName;
        int                     m_prevMenuId;

        ConfigValidatorBase*    m_configValidator;

        void createMenu( wxMenu* menuBar );
        void showDialog(int id);
        bool showDialog( wxObject* objDesc );
        bool showDialog( UserDialog* ud );
        bool showTable( VemsTableDesc* desc );
        bool showCurve( VemsCurveDialogDesc* desc  );
        bool showSuperCurve( SuperCurveDescriptor* desc );
        bool showInterpolationWidget( InterpolationWidgetDescriptor* desc );
        bool showMapper( TblMapDesc* desc );
        bool show(CheckBoxMatrixDesc* desc);
        bool showOutputChannelsDialog( const wxString& name );
        bool showIgnitionOutputsVisualDialog( const wxString& name );
        bool showRoundDisplaySettingsDialog(const wxString& name );
        bool showRoundDisplayDemoDialog( const wxString& name );
        bool showActiveFrame(int id);
        void closeDialog( int id, bool needRestore = false );

        void OnMenu(wxCommandEvent& event);
        void OnTimer( wxTimerEvent& event );
};

#endif // USERDIALOGMANAGER_H

