#include "UserDialogTemplate.h"

#include <wx/dcbuffer.h>
#include <wx/file.h>
#include <wx/tooltip.h>
#include "../Utils.h"

#include "UserDialog.h"
#include "ConfigletPanel.h"
#include "../common/HelpUtil.h"
#include "../common/VemsContext.h"
#include "../common/VemsMessageHashHelper.h"

#include "../DirHelper.h"
#include "../mtconfigdata.h"
//#include "../MTConfigHelper.h"
#include "../VemsNumPad.h"
#include "../logger.h"

#include "../UserLogWindow.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../stdatamanager/STDataModelBase.h"
#include "../events/RefreshUserMenuEvent.h"

#include "../firmwareconfig/FirmwareConfigLoader.h"

#include "../stdatamanager/STDataModelSerial.h"
#include "../firmwareconfig/ConfigValidatorBase.h"
#include "../common/FunctionMeter.h"

/// custom events
const wxEventType NeedRefreshEvent = wxNewEventType();


/**********************************************************
*   StaticLabel methods
**********************************************************/
BEGIN_EVENT_TABLE(StaticLabel, wxWindow)
    EVT_PAINT(StaticLabel::OnPaint)
END_EVENT_TABLE()

StaticLabel::StaticLabel( wxWindow *parent, const wxString& label, bool vertical) : wxWindow(parent, wxID_ANY, wxPoint(0, 0), wxSize(1, 1))
{
	m_isVertical = vertical;
	SetLabel( label );
}

StaticLabel::~StaticLabel() {}

void StaticLabel::OnPaint( wxPaintEvent& event )
{
	wxBufferedPaintDC dc( this );

	wxSize sz = GetClientSize();

	dc.SetBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
	dc.SetPen(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
	dc.DrawRectangle(wxPoint(0, 0), sz);

	dc.SetBackgroundMode(wxTRANSPARENT);

	const wxFont& cur_font = GetFont();

	dc.SetFont(cur_font);

	int w = 0, h = 0;
	GetTextExtent(GetLabel(), &w, &h, 0, 0, &cur_font);

	if (m_isVertical)
	{
		const int x = (sz.GetWidth() - h) / 2;
		const int y = (sz.GetHeight() - w) / 2;

		dc.DrawRotatedText(GetLabel(), x, y + w, 90.0);
	}
	else
	{
		const int x = (sz.GetWidth() - w) / 2;
		const int y = (sz.GetHeight() - h) / 2;

		dc.DrawText(GetLabel(), x, y);
	}

	event.Skip();
}


/**********************************************************
*   STViewerTextCtrl methods
**********************************************************/

BEGIN_EVENT_TABLE( STViewerTextCtrl, wxTextCtrl )
    EVT_TEXT_ENTER(wxID_ANY, STViewerTextCtrl::OnEnter)
    EVT_KILL_FOCUS( STViewerTextCtrl::OnKillFocus )
    EVT_KEY_DOWN(STViewerTextCtrl::OnKeyDown)
END_EVENT_TABLE()

STViewerTextCtrl::STViewerTextCtrl( wxWindow* parent, wxWindowID id, const wxString& value, const wxPoint& pos, const wxSize& size,
    long style, const wxValidator& validator, const wxString& name ) : wxTextCtrl( parent, id, value, pos, size, style, validator, name )
{
    m_parent = (UserDialogTemplate*)parent;
}

void STViewerTextCtrl::update()
{
    double viewerData = m_stDataMgr->getDataFromViewerModel( m_symbol, 0, 0 );

    if ( m_stDataMgr->wasDataWritten( m_symbol, 0, 0 ) )
    {
        SetForegroundColour( *wxBLACK );
    }
    else
    {
        SetForegroundColour( *wxRED );
    }

    ChangeValue( wxString::Format( m_format, viewerData ) );

    if ( !m_expr.IsEmpty() )
    {
        Enable( m_stDataMgr->calculate( m_expr )!=0?true:false );
    }

//    if ( m_parent->getConfigValidator() != NULL )
//        m_parent->getConfigValidator()->validate( m_stDataMgr->getModel() );


    Refresh();

}


void STViewerTextCtrl::requestRead()
{
    LOGP( _T("m_symbol.getName(): ") +  m_symbol.getName() );
    LOGP( wxString::Format( _T("m_symbol.getPage(): %d"), m_symbol.getPage() ) );
    m_stDataMgr->addPageNeedRefresh( m_symbol.getPage() );
}


void STViewerTextCtrl::updateModel()
{
    double data = 0.0;

    if ( GetValue().ToDouble( &data ) )
    {
        m_stDataMgr->setModelData( m_symbol, 0, 0, data );
    }

}


void STViewerTextCtrl::requestWrite()
{
    m_stDataMgr->requestWrite( m_symbol, 0, 0 );
}


void STViewerTextCtrl::setFormat( int digits_in )
{
    m_format = _T("%.") + wxString::Format( _T("%i"), digits_in ) + _T("f");
}

void STViewerTextCtrl::ReValidate()
{
    if ( m_parent->getConfigValidator() != NULL )
        m_parent->getConfigValidator()->validate( m_stDataMgr->getModel() );
}

void STViewerTextCtrl::OnEnter( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("STViewerTextCtrl::OnEnter()") );
    updateModel();
    requestWrite();
    m_stDataMgr->updateRegisteredViewers();

    ReValidate();

    Navigate();
}


void STViewerTextCtrl::OnKillFocus( wxFocusEvent& event )
{
    updateModel();
    requestWrite();
    m_stDataMgr->updateRegisteredViewers();

    ReValidate();
}

void STViewerTextCtrl::OnKeyDown( wxKeyEvent& event )
{
    int key = event.GetKeyCode();

    if ( key == WXK_TAB )
    {
        if ( event.ShiftDown() )
        {
            Navigate( wxNavigationKeyEvent::IsBackward );
            return;
        }

        updateModel();
        requestWrite();
        m_stDataMgr->updateRegisteredViewers();

        ReValidate();

        Navigate();
        return;
    }

    event.Skip();
}


wxString STViewerTextCtrl::toString()
{
    return _T("STViewerTextCtrl - symbol: ") + m_symbol.getName();
}

SymbolArray STViewerTextCtrl::getSymbols()
{
    SymbolArray symbols;
    symbols.Add( new Symbol( m_symbol ) );
    return symbols;
}

/**********************************************************
*   STViewerComboBox methods
**********************************************************/
BEGIN_EVENT_TABLE( STViewerComboBox, wxComboBox )
    EVT_COMBOBOX( wxID_ANY, STViewerComboBox::OnTextUpdated )
END_EVENT_TABLE()

//STViewerComboBox::STViewerComboBox( wxWindow* parent, wxWindowID id, const wxString& value,
//                                    const wxPoint& pos, const wxSize& size, const wxArrayString& choices, long style )
STViewerComboBox::STViewerComboBox( wxWindow* parent, const Symbol& sym, const wxString& expr )
{
    m_parent = (UserDialogTemplate*)parent;
    m_value = 0;
    m_symbol = sym;
    m_expr = expr;

    wxArrayString oldBits = sym.getBits();
    int bitCount = 1 << (sym.getBitHi() - sym.getBitLo() + 1);

    //create labels
    for( int i = 0; i < bitCount; ++i )
{
        if( i < (int)oldBits.GetCount() ) {
            m_labels.Add( oldBits.Item(i) );
        } else {
            m_labels.Add( wxString::Format( _T("%d"), i + sym.getBitOfs() ) );
        }
    }

    //create choices
    wxArrayString choices;

    for ( size_t i = 0; i < m_labels.GetCount(); ++i ) {
        if( m_labels[i] == _T("INVALID") ) continue;

        if ( choices.Index( m_labels[i] ) != wxNOT_FOUND ) continue;

        choices.Add( m_labels[i] );
        }

    Create( parent, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, choices, wxCB_READONLY );

        }

void STViewerComboBox::update()
    {
    m_value = (int)m_stDataMgr->getDataFromViewerModel( m_symbol, 0, 0 );

    if ( m_value >= 0 && m_value < m_labels.GetCount() ) {

        wxString selectedLabel = m_labels[m_value];

        if( selectedLabel.IsSameAs( _T("INVALID"), false ) ) {

            if ( FindString( selectedLabel ) == wxNOT_FOUND ) {
                Append( selectedLabel );
    }

    }

        SetValue( selectedLabel );

    } else {
        LOGP( wxString::Format( _T("Invalid value: %d"), m_value ) );
        return;
    }

    wxColor fgColor = m_stDataMgr->wasDataWritten( m_symbol, 0, 0 ) ? *wxBLACK : *wxRED;

    SetForegroundColour( fgColor );

    if ( !m_expr.IsEmpty() )
    {
        Enable( m_stDataMgr->calculate( m_expr )!=0?true:false );
    }

    Refresh();
}

void STViewerComboBox::ReValidate()
{
    if ( m_parent->getConfigValidator() != NULL )
        m_parent->getConfigValidator()->validate( m_stDataMgr->getModel() );
}

void STViewerComboBox::requestRead()
{
    LOGP( wxString::Format( _T("m_symbol.getPage(): %d"), m_symbol.getPage() ) );
    m_stDataMgr->addPageNeedRefresh( m_symbol.getPage() );
}


void STViewerComboBox::updateModel()
{
    m_stDataMgr->setModelData( m_symbol, 0, 0, m_value );
    }


void STViewerComboBox::requestWrite()
{
    m_stDataMgr->requestWrite( m_symbol, 0, 0 );
}


void STViewerComboBox::OnTextUpdated( wxCommandEvent& event )
{
    //remove INVALID label from list if VALID value selected
    if ( !GetValue().IsSameAs( _T("INVALID"), false ) ) {

        int invalidIdx = FindString( _T("INVALID") );

        if ( invalidIdx != wxNOT_FOUND ) {
            Delete( invalidIdx );
        }
    }

    m_value = m_labels.Index( GetValue() );

    updateModel();
    requestWrite();
    m_stDataMgr->updateRegisteredViewers();

    ReValidate();

    wxWindow* frame = GetGrandParent();

    if ( frame )
    {
        wxWindow* main = frame->GetParent();

        if ( main )
        {
            wxCommandEvent e( RefreshUserMenuEvent );
            main->AddPendingEvent( e );
        }
    }

    event.Skip();
}


wxString STViewerComboBox::toString()
{
    return _T("STViewerComboBox - symbol: ") + m_symbol.getName();
}

SymbolArray STViewerComboBox::getSymbols()
{
    SymbolArray symbols;
    symbols.Add( new Symbol( m_symbol ) );
    return symbols;
}

/**********************************************************
*   UserDialogTemplate methods
**********************************************************/

BEGIN_EVENT_TABLE( UserDialogTemplate, wxPanel )
    EVT_CLOSE( UserDialogTemplate::OnClose )
END_EVENT_TABLE()

wxBitmap UserDialogTemplate::m_warningIcon = wxNullBitmap;
wxBitmap UserDialogTemplate::m_errorIcon = wxNullBitmap;
wxBitmap UserDialogTemplate::m_markedIcon = wxNullBitmap;


/** @brief UserDialogTemplate
  *
  * @todo: document this function
  */
UserDialogTemplate::UserDialogTemplate()
{
    Init();
    //EnableScrolling( 0 , 1 );
    //SetScrollRate(10,5);
}

/** @brief UserDialogTemplate
  *
  * @todo: document this function
  */
UserDialogTemplate::UserDialogTemplate( wxWindow* parent,
                                        const UserDialog& desc,
                                        SymbolMgr* symbolmgr,
                                        ConfigValidatorBase* validator,
                                        wxWindowID id,
                                        const wxPoint& pos,
                                        const wxSize& size,
                                        long style
                                        )
    : StaticViewersController( parent, desc.getHelpID() )
{
    Init();
    Create( parent, desc, symbolmgr, validator, id, pos, size, style );
    //EnableScrolling( 0 , 1 );
    //SetScrollRate(10,5);
}

/** @brief UserDialogTemplate
  *
  * @todo: document this function
  */
UserDialogTemplate::~UserDialogTemplate()
{
    if ( m_validator )
        m_validator->unregisterCfgValidatorViewer( this );
}

/** @brief Init
  *
  * @todo: document this function
  */
bool UserDialogTemplate::Init()
{
    m_tablePopupMenuTitle = wxEmptyString;
    return true;
}

/** @brief Create
  *
  * @todo: document this function
  */
bool UserDialogTemplate::Create( wxWindow* parent,
                                 const UserDialog& desc,
                                 SymbolMgr* symbolmgr,
                                 ConfigValidatorBase* validator,
                                 wxWindowID id,
                                 const wxPoint& pos,
                                 const wxSize& size,
                                 long style )

{
    m_helpId        = desc.getHelpID();
    m_helpParent    = parent;
    m_validator     = validator;

    if ( m_warningIcon.IsOk() == false )
        m_warningIcon.LoadFile( DirHelper::GetPath( dICON ) + _T("ul-warning-m.png"), wxBITMAP_TYPE_PNG );
    if ( m_errorIcon.IsOk() == false )
        m_errorIcon.LoadFile( DirHelper::GetPath( dICON ) + _T("ul-error-m.png"), wxBITMAP_TYPE_PNG );
    if ( m_markedIcon.IsOk() == false )
        m_markedIcon.LoadFile( DirHelper::GetPath( dICON ) + _T("green_right_arrow_m.png"), wxBITMAP_TYPE_PNG );

    if ( m_validator )
        m_validator->registerCfgValidatorViewer( this );

    wxPanel::Create( parent, id, pos, size, style );

    m_mainSizer = new wxGridBagSizer();
    m_mainSizer->AddGrowableCol(1);

    //SetSizer( new wxBoxSizer( wxVERTICAL ) );
    SetSizer( m_mainSizer );
    CreateDialog( desc, symbolmgr );
    //SetInitialSize();

    return true;
}

StaticViewersController* UserDialogTemplate::newInstance(
    wxWindow* parent, const UserDialog& desc, UserDialogBase* udMgr, SymbolMgr* symbolMgr, STDataMgrBase* stMgr, ConfigValidatorBase* validator, wxArrayString symbolsToMark )
{
    //FunctionMeter f(_T("UserDialogTemplate::newInstance"));
    UserDialogTemplate* udt = new UserDialogTemplate();
    udt->setSymbolsToMark( symbolsToMark );
    udt->setController( stMgr );
    udt->Create( parent, desc, symbolMgr, validator );
    udt->setUserDialogName( desc.getName() );
    udt->setUserDialogManager( udMgr );
    udt->updateData();
    parent->GetSizer()->Add( udt, 1, wxEXPAND );

    udt->EnableScrolling(1,1);
    udt->SetScrollRate(10,5);

    return udt;
}

/** @brief  create UserDialog from descriptor
  *         ud must valid pointer
*/
bool UserDialogTemplate::CreateDialog( const UserDialog& desc, SymbolMgr* symbolmgr)
{
    m_symbolsOnDialog.Clear();

    for( size_t i = 0; i < desc.getCount(); ++i )
    {
        UserEditField* uef = desc[i];

        if( uef->isLabel() )
        {
            addLabel( uef->getLabel() );
            continue;
        }

        Symbol* sym = symbolmgr->getSymbol( uef->getSymbol() );

        if( sym )
        {
            m_symbolsOnDialog.Add( sym->getName() );

            if( sym->isBits() )
            {
                addComboBox( m_controller, uef->getLabel(), *sym, uef->getExpr() );
            }
            else
            {
                addEditField( m_controller, uef->getLabel(), *sym, uef->getExpr() );
            }
        }
        else
        {
            addMissingWarning( uef->getLabel(), uef->getSymbol() );
        }
    }

    addConfigletPanel( desc, symbolmgr );

    if ( m_validator )
        m_validator->validate( m_controller->getModel() );

    return true;
}

void UserDialogTemplate::addConfigletPanel( const UserDialog& desc, SymbolMgr* symbolmgr )
{
    wxString configletDir = DirHelper::GetPath( dCONFIGLETS ) + desc.getName() + DirHelper::Separ();

    if ( !::wxDirExists( configletDir ) ) return;

    ConfigletPanel* configletPanel = new ConfigletPanel( this, configletDir, symbolmgr );
    m_ViewerBaseA.Add( configletPanel );
    m_controller->RegisterViewer( configletPanel );

    int rows = m_mainSizer->GetRows();
    m_mainSizer->Add( configletPanel, wxGBPosition(rows,0), wxGBSpan( 1, 3 ) , wxEXPAND | wxALL, 3 );
    m_mainSizer->SetRows( rows+1 );
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}


void UserDialogTemplate::addMissingWarning( const wxString& label, const wxString& variable )
{
    wxStaticText* textLabel = new wxStaticText( this, wxID_ANY, label );

    int rows = m_mainSizer->GetRows();



    m_mainSizer->Add( textLabel, wxGBPosition(rows,0), wxDefaultSpan , wxEXPAND | wxALL, 3 );

    wxString warningStr = _T("variable '") + variable + _T("' not found in the ini file!!!");
    wxStaticText* warningText = new wxStaticText( this, wxID_ANY, warningStr );
    warningText->SetForegroundColour( wxColour( _T("RED") ) );

    m_mainSizer->Add( warningText, wxGBPosition(rows,1), wxDefaultSpan, wxEXPAND | wxALL, 3 );

    m_mainSizer->SetRows( rows+1 );

    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void UserDialogTemplate::addLabel( const wxString& label )
{
    int rows = m_mainSizer->GetRows();

    wxString newLabel = label;
    bool isWarning = false;

    if( newLabel.Find( _T("!") ) == 0 )
    {
        newLabel = label.AfterFirst( _T('!') );
        isWarning = true;
    }

    wxStaticText* textLabel = new wxStaticText( this, wxID_ANY, newLabel );

    wxFont font = textLabel->GetFont();
    font.SetWeight(wxFONTWEIGHT_BOLD);
    if( isWarning )
    {
        textLabel->SetForegroundColour( wxColour( _T("RED") ) );
    }
    textLabel->SetFont(font);

    m_mainSizer->Add(textLabel, wxGBPosition(rows,0), wxGBSpan(1,4), wxEXPAND|wxALL, 3);
    m_mainSizer->SetRows( rows+1 );

    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}


void UserDialogTemplate::addEditField( STDataMgrBase* controller, const wxString& label, const Symbol& symbol_in, const wxString& expr )
{
    int rows = m_mainSizer->GetRows();

    //if label begins '!' then it is a warning label
    wxString newLabel = label;
    bool isWarning = false;

    if( newLabel.Find( _T("!") ) == 0 )
    {
        newLabel = label.AfterFirst( _T('!') );
        isWarning = true;
    }

    wxStaticText* textLabel = new wxStaticText( this, wxID_ANY, newLabel );

    //warning label must be red
    if( isWarning )
    {
        textLabel->SetForegroundColour( *wxRED );
    }

    //textLabel->SetToolTip( symbol_in.getName() );
    wxString toolTip = wxString::Format( symbol_in.getName() + _T(" - max : %f, min : %f"), symbol_in.getHi(), symbol_in.getLo() );
    textLabel->SetToolTip( toolTip );

    int pos = 0;
    wxStaticBitmap* icon;

    if ( m_SymbolMarked.Index( symbol_in.getName() ) != wxNOT_FOUND )
    {
         icon = new wxStaticBitmap( this, -1, m_markedIcon );
    }
    else
        icon = new wxStaticBitmap( this, -1, wxNullBitmap );

    m_mainSizer->Add( icon, wxGBPosition(rows,pos++), wxDefaultSpan, wxLEFT | wxALL, 3 );
    m_symbol2icon[symbol_in.getName()] = icon;

    ///

    m_mainSizer->Add(textLabel,wxGBPosition(rows,pos++), wxDefaultSpan,wxEXPAND|wxALL|wxALIGN_CENTER_VERTICAL, 3);

    int id =  wxNewId();

    wxTextValidator validator( wxFILTER_NUMERIC | wxFILTER_EXCLUDE_LIST );
    wxArrayString excludes;
    excludes.Add( _T(",") );
    validator.SetExcludes( excludes );

    //STViewerTextCtrl* textField = new  STViewerTextCtrl( this, id, wxEmptyString , wxDefaultPosition, wxDefaultSize, 0, validator );
    wxSize textFieldSize = wxDefaultSize;
    textFieldSize.SetWidth( 60 );

    STViewerTextCtrl* textField = new  STViewerTextCtrl( this, id, wxEmptyString , wxDefaultPosition, textFieldSize
                                                            , wxTE_PROCESS_ENTER | wxTE_PROCESS_TAB, validator );


    textField->setFormat( symbol_in.getDigits() );
    textField->SetName( label );

    m_idToSymbol[id] = symbol_in.getName();
#if defined (__WINCE_STANDARDSDK__)
    textField->Connect( wxEVT_LEFT_DCLICK, wxMouseEventHandler(UserDialogTemplate::OnLeftUpTextField));
#endif


    //m_mainSizer->Add(textField,wxGBPosition(rows,1), wxDefaultSpan,wxEXPAND|wxALL,3);
    m_mainSizer->Add(textField,wxGBPosition(rows,pos++), wxDefaultSpan, wxALIGN_RIGHT|wxALL,3);

    //if symbol has unit display it
    wxString unit = symbol_in.getUnit();

    if( unit.Len() > 0 ) {
        wxStaticText* unitLabel = new wxStaticText( this, wxID_ANY, unit );
        m_mainSizer->Add( unitLabel, wxGBPosition(rows,pos++), wxDefaultSpan, wxLEFT | wxALL | wxALIGN_CENTER_VERTICAL, 3 );
    }

    // storage for "warning/error" icon
    icon = new wxStaticBitmap( this, -1, wxNullBitmap );
    m_mainSizer->Add( icon, wxGBPosition(rows,pos++), wxDefaultSpan, wxLEFT | wxALL, 3 );

    m_symbol2icon[symbol_in.getName()] = icon;
    m_symbol2label[symbol_in.getName()] = textLabel;
    //



    m_mainSizer->SetRows( rows+1 );

    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    textField->setSymbol( symbol_in );
    textField->setExpression( expr );

    m_ViewerBaseA.Add( textField );
    controller->RegisterViewer( textField );
}


void UserDialogTemplate::addComboBox( STDataMgrBase* controller, const wxString& label, const Symbol& symbol_in, const wxString& expr)
{
    int rows = m_mainSizer->GetRows();

    wxString newLabel = label;
    bool isWarning = false;

    if( newLabel.Find( _T("!") ) == 0 )
    {
        newLabel = label.AfterFirst( _T('!') );
        isWarning = true;
    }

    wxStaticText* textLabel = new wxStaticText(this, wxID_ANY, newLabel);

    if( isWarning )    {
        textLabel->SetForegroundColour( *wxRED );
    }

    textLabel->SetToolTip( symbol_in.getName() );

    int pos = 0;
    wxStaticBitmap* icon;

    if ( m_SymbolMarked.Index( symbol_in.getName() ) != wxNOT_FOUND )
    {
         icon = new wxStaticBitmap( this, -1, m_markedIcon );
    }
    else
        icon = new wxStaticBitmap( this, -1, wxNullBitmap );

    m_mainSizer->Add( icon, wxGBPosition(rows,pos++), wxDefaultSpan, wxLEFT | wxALL, 3 );
    m_symbol2icon[symbol_in.getName()] = icon;

    m_mainSizer->Add(textLabel,wxGBPosition(rows,pos++), wxDefaultSpan,wxEXPAND|wxALL, 3);

    STViewerComboBox* comboBox = new STViewerComboBox(  this, symbol_in, expr );

    //m_mainSizer->Add(comboBox, wxGBPosition(rows,1), wxDefaultSpan, wxEXPAND|wxALL,3);
    m_mainSizer->Add(comboBox, wxGBPosition(rows,pos++), wxDefaultSpan, wxALIGN_RIGHT | wxALL,3);

    //if symbol has unit display it
    size_t unitLength = 6;
    wxString unit = symbol_in.getUnit().Mid( 0, unitLength );

    if( unit.Len() > 0 ) {
        //Logger::logf( LOG_MSG, _T("UserDialogTemplate::addComboBox() - unit: ") + unit );
        wxStaticText* unitLabel = new wxStaticText( this, wxID_ANY, unit.Pad( unitLength - unit.length(), _T(' ') ) );
        m_mainSizer->Add( unitLabel, wxGBPosition(rows,pos++), wxDefaultSpan, wxLEFT | wxALL, 3 );
    }

    // storage for "warning/error" icon
    icon = new wxStaticBitmap( this, -1, wxNullBitmap );
    m_mainSizer->Add( icon, wxGBPosition(rows,pos++), wxDefaultSpan, wxLEFT | wxALL, 3 );

    m_symbol2icon[symbol_in.getName()] = icon;
    m_symbol2label[symbol_in.getName()] = textLabel;
    //

    m_mainSizer->SetRows( rows+1 );
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
//
//    comboBox->setSymbol( symbol_in );
//    comboBox->setExpression( expr );

    m_ViewerBaseA.Add( comboBox );
    controller->RegisterViewer( comboBox );
}


void UserDialogTemplate::OnLeftUpTextField( wxMouseEvent& event )
{
#if defined (__WINCE_STANDARDSDK__)

    wxTextCtrl *textField = ((wxTextCtrl*)event.GetEventObject());

    textField->SetFocus();
    textField->SetSelection(-1,-1);

    VemsNumPad pad(this, VNP_CAPTION);

    pad.SetValue( textField->GetValue() );

    pad.SetValueCaption( textField->GetName() );

    if( pad.ShowModal() == wxID_OK )
        textField->SetValue(pad.GetValueAsString());


    textField->SetSelection(-1,-1);

#endif
}
void UserDialogTemplate::OnClose( wxCloseEvent& event )
{
    unregisterViewers();
}

void UserDialogTemplate::loadModelFromDialog( STDataModelBase& model )
{
    for ( size_t i = 0; i < m_ViewerBaseA.GetCount(); ++i )
    {
        STDataViewerBase* viewer = m_ViewerBaseA[i];
        const SymbolArray& viewerSymbols = viewer->getSymbols();

        for ( size_t j = 0; j <  viewerSymbols.GetCount(); ++j )
        {
            const Symbol* aViewerSymbol = viewerSymbols[j];
            double data = m_controller->getDataFromViewerModel( *aViewerSymbol );
            model.setModelData( *aViewerSymbol, 0, 0, data );
        }
    }
}

void UserDialogTemplate::updateDialogFromModel( STDataModelBase& model )
{
    for ( size_t i = 0; i < m_ViewerBaseA.GetCount(); ++i )
    {
        STDataViewerBase* viewer = m_ViewerBaseA[i];
        const SymbolArray& viewerSymbols = viewer->getSymbols();

        for ( size_t j = 0; j <  viewerSymbols.GetCount(); ++j )
        {
            const Symbol* aViewerSymbol = viewerSymbols[j];
            double value = model.getViewerData( *aViewerSymbol, 0, 0 );
            m_controller->setModelData( *aViewerSymbol, 0, 0, value );
        }

        viewer->update();
    }
}

bool UserDialogTemplate::onDropFiles( const wxArrayString& filenames )
{
    size_t nFiles = filenames.GetCount();

    if ( nFiles != 1 )
    {
        return false;
    }

    wxString filename = filenames[0];

    if ( !wxFile::Exists( filename ) )
    {
        return false;
    }

    wxFileName fn( filename );

    if( (fn.GetExt() ==  _T("vemscfg") ) || fn.GetExt()  ==  _T("txt") ){
       importFromFile( filename );
    }
    return true;
}

void UserDialogTemplate::importFromFile(  const wxString& filename )
{
    SymbolTable symbols;
    STDataModelSerial  serialModel( symbols );

    wxArrayString fileA;
    fileA.Add(filename);

    loadModelFromDialog( serialModel );

    MTConfigData configData;
    FirmwareConfigLoader configLoader;

    wxObject* obj = VemsContext::getObject(_T("FirmwareInfo"));
    if(!obj){
        WX_CLEAR_HASH_MAP( SymbolTable, symbols );
        return;
    }

    FirmwareInfo* destFwInfo = (FirmwareInfo*) obj;
    if(destFwInfo->getHeaderFile() == wxEmptyString){
        /// error you must download fw
        WX_CLEAR_HASH_MAP( SymbolTable, symbols );
        showErrorDialog(*destFwInfo);
        return;
    }
    configLoader.setDestinationFwInfo((FirmwareInfo*) obj);
    configLoader.setMTConfigData( &configData );
    configLoader.parse( fileA );

    FirmwareInfo cfgInfo = configLoader.getCfgFwInfo();
    if(cfgInfo.getHeaderFile() == wxEmptyString){
        /// error you must download fw
        WX_CLEAR_HASH_MAP( SymbolTable, symbols );
        showErrorDialog(cfgInfo);
        return;
    }

    configLoader.load( &serialModel );
    updateDialogFromModel( serialModel );

    WX_CLEAR_HASH_MAP( SymbolTable, symbols );

    return;
}

void UserDialogTemplate::showErrorDialog(const FirmwareInfo& fwInfo)
{
    StringHash hash;
    hash[_T("firmware_version")] = fwInfo.getFirmwareVersionAsString();

    wxString msg = VemsMessageHashHelper::GetVemsMsg(_T("header_not_found_str"),&hash).getHtmlMsg();
    wxMessageBox( msg );
}

void UserDialogTemplate::notify( const StringHash& errors_and_warnings )
{
    // clear marks
    {
        const wxColor& bgcolor = wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE );
        symbol2IconHash::iterator icon_it = m_symbol2icon.begin();
        for ( ; icon_it != m_symbol2icon.end(); ++icon_it )
            icon_it->second->SetBitmap( wxNullBitmap );

        symbol2LabelHash::iterator label_it = m_symbol2label.begin();
        for ( ; label_it != m_symbol2label.end(); ++label_it )
            label_it->second->SetBackgroundColour( bgcolor );
        Refresh();
    }

    StringHash::const_iterator it = errors_and_warnings.begin();

    for ( ; it != errors_and_warnings.end(); ++it )
    {
        if ( m_symbolsOnDialog.Index( it->first ) != wxNOT_FOUND )
        {
            wxStaticBitmap* icon = NULL;
            symbol2IconHash::iterator icon_it = m_symbol2icon.find( it->first );

            if ( icon_it != m_symbol2icon.end() )
                icon = icon_it->second;

            if ( icon )
            {
                wxString msg = it->second;
                msg.Replace( _T("<br>"), _T("\n"), true );
                msg.Replace( _T("<b>"), _T(""), true );
                msg.Replace( _T("</b>"), _T(""), true );
                msg.Replace( _T("</a>"), _T(""), true );
                msg.Replace( _T("$MENU_LINK"), _T(""), true );
                msg.Replace( _T("  "), _T(" "), true );
                msg.Replace( _T("\t"), _T(" "), true );
                msg.Replace( _T("\r\n"), _T("\n"), true );
                msg.Replace( _T("\n\n"), _T("\n"), true );
                msg.Trim();

                if ( msg.StartsWith(_T("Error")))
                    icon->SetBitmap( m_errorIcon );
                if ( msg.StartsWith(_T("Warning") ) )
                    icon->SetBitmap( m_warningIcon );

                icon->SetToolTip( msg );
            }

            wxStaticText* label = NULL;
            symbol2LabelHash::iterator label_it = m_symbol2label.find( it->first );

            if ( label_it != m_symbol2label.end() )
                label = label_it->second;

            if ( label )
            {
                label->SetBackgroundColour( wxColor( 205, 109, 13 ) );
            }
        }
    }
}
