#ifndef USERMENU_H
#define USERMENU_H

#include <wx/string.h>
#include <wx/dynarray.h>

#include "../common/typedef.h"

/**********************************************************
*   UserMenuItem
**********************************************************/
class UserMenuItem
{
    public:
        UserMenuItem(const wxString& dialog, const wxString& label=_T(""), int pageNo=-1, const wxString& expr=_T(""));
        virtual ~UserMenuItem();

        wxString    getDialog() const { return m_dialog; }
        wxString    getLabel() const { return m_label; }
        wxString    getExpr() const { return m_expr; }
        int         getPageNo() const { return m_pageNo; }

        bool        isSeparator() const { return m_dialog == _T("std_separator"); }
    protected:
    private:
        wxString m_dialog;
        wxString m_label;
        int      m_pageNo;
        wxString m_expr;
};

/**********************************************************
*   UserMenu
**********************************************************/
WX_DECLARE_OBJARRAY(UserMenuItem, UserMenuItemArray );

class UserMenu
{
    public:
        UserMenu(const wxString& label);
        virtual ~UserMenu();

        void addUserMenuItem(const wxString& dialog, const wxString& label=_T(""), int pageNo=0, const wxString& expr=_T(""));
        UserMenuItem* operator[](size_t i);
        size_t getCount() { return m_UserMenuItems.GetCount(); }
        wxString getLabel(){ return m_label; }
        UserMenuItemArray& getItems() { return m_UserMenuItems; }

    private:
        wxString            m_label;
        UserMenuItemArray   m_UserMenuItems;
};

/**********************************************************
*   UserMenuList
**********************************************************/
WX_DECLARE_OBJARRAY(UserMenu, USER_MENUS);

class UserMenuList
{
    public:
        UserMenuList();
        virtual ~UserMenuList();

        void addUserMenu(const wxString& um);
        void addUserMenuItem(const wxString& um,const wxString& dialog, const wxString& label=_T(""), int pageNo=0, const wxString& expr=_T(""));
        size_t getCount() { return m_UserMenus.GetCount(); }
        UserMenu* operator[](size_t i);
        void clear();
    protected:
    private:
        USER_MENUS  m_UserMenus;

        int findMenu(const wxString& usermenu);
};

#endif // USERMENU_H
