#include "ArcBarGauge.h"

#include <wx/dcclient.h>
#include <wx/brush.h>
#include <wx/bitmap.h>
#include <wx/font.h>
#include <wx/fontenum.h>
#include <wx/fontmap.h>
#include <wx/image.h>
#include <wx/intl.h>
#include <wx/settings.h>
#include <wx/dcbuffer.h>

#include <math.h>

#include "../Math.h"
#include "../logger.h"
#include "../wxStaticTextEx.h"

#include "../common/FunctionMeter.h"
#include "../common/typedef.h"


BEGIN_EVENT_TABLE(ArcBarGauge,VemsWidgetBase)

    EVT_PAINT(ArcBarGauge::OnPaint)

    EVT_SIZE ( ArcBarGauge::OnSize )

    //EVT_SIZE ( ArcBarGauge::OnSize )

END_EVENT_TABLE()

ArcBarGauge::ArcBarGauge(wxWindow* parent, wxWindowID id,GaugeDescManager * gdmgr, GaugeProperties &prop )
    //const wxString& symbolName, wxPoint gPos, int width, long style, const VEMSGaugeDescArray * gaugeDescA,const WidgetThemeTableArray * themes, const wxString& currTheme)
      //  : VemsWidgetBase(parent, id,gPos,wxSize(width,width/3),style, wxPanelNameStr, gaugeDescA)
//         m_calcedRefSegments (false)
:VemsWidgetBase(parent, id, gdmgr, prop)
,m_timer(this)
{
    Logger::logf(LOG_MSG,_T("ArcBarGauge CTOR START"));


    m_CurrValue = 0;
    m_value = 0;

    prop.height = prop.width/3;

    DigitalFont = new wxFont( int(prop.width/10.6),wxSWISS,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL,false,_T("7 Segment"),wxFONTENCODING_DEFAULT);

    //CreateWidget(parent, id, gdmgr, prop);
    //gPos,wxSize(width,width/3),style, wxPanelNameStr, gaugeDescA,themes);

    m_calcedRefSegments = false;

    if( m_graphQual > GRAPH_MIDDLE) m_graphQual = GRAPH_AGG;

    m_StartAngle = 118;
    m_EndAngle = 65;

    m_AlphaPreviousValue = 64;
    m_idxValue = 0;
    m_idxPrevValue = 0;

    PaintBgSem = 1;

    m_parent = parent;

    SegmentInit();

    SetValue(0);

}

ArcBarGauge::~ArcBarGauge()
{
    //dtor
    //Logger::log(LOG_MSG,_T("ArcBarGauge DTOR"));
    for(uint8_t a=0; a < m_bmpValues.GetCount(); a++)
        delete m_bmpValues[a];

}

void ArcBarGauge::SetNewTheme(WidgetThemeTable *theme)
{
    m_theme = theme;
    UpdateSize();
}

void ArcBarGauge::CreateBmpStates()
{
    //Logger::logf(LOG_MSG,_T("ArcBarGauge::CreateBmpStates 1"));
    //Logger::logMilli(LOG_MSG,_T("ArcBarGauge::CreateBmpStates START"));
    FunctionMeter fm(_T("ArcBarGauge::CreateBmpStates()"));

    if(m_segments.GetCount() == 0) return;

    int width = GetClientSize().x;
    int height= GetClientSize().y;

    wxBitmap backBmp(width,height);

    wxMemoryDC memdc(backBmp);

    memdc.SetBackground(m_theme->BackgroundColor);
    memdc.Clear();

    PaintGaugeNumbers(memdc);
    PaintTickMarks(memdc);

    memdc.SelectObject(wxNullBitmap);

    for(uint8_t a=0; a < m_bmpValues.GetCount(); a++)
        delete m_bmpValues[a];

    m_bmpValues.Clear();

    wxBitmap *bmp;
    wxAgg aggCanvas( width,height );

    int h=0;

    for(short s = m_segments.GetCount()-1; s >= 0; s--){

        aggCanvas.clear(wxColor(0,0,0,0));

        if( s == (short)(m_segments.GetCount()-1)) {

            for(uint8_t a = 0; a < m_segments.GetCount(); a++){

                aggCanvas.setBrush( m_segments[a]->GetColor(false) );
                aggCanvas.drawPolygon( *(m_segments[a]->GetAggCoords()) );

            }
        }
        else {

            aggCanvas.setBrush( m_segments[s]->GetColor(true) );
            aggCanvas.drawPolygon( *(m_segments[s]->GetAggCoords()) );

        }

        bmp = new wxBitmap(width,height);

        memdc.SelectObject(*bmp);
        memdc.SetBackground( m_theme->BackgroundColor );
        memdc.Clear();


        if( s == (short)(m_segments.GetCount()-1) )
            memdc.DrawBitmap(backBmp,0,0,1);
        else
            memdc.DrawBitmap(*m_bmpValues[h-1],0,0,1);


        aggCanvas.render( memdc );

        memdc.SelectObject(wxNullBitmap);

        m_bmpValues.Add(bmp);

        h++;
    }





}

void ArcBarGauge::OnSize(wxSizeEvent &evt)
{
    if(m_timer.IsRunning())
        return;

    UpdateSize();
    m_timer.Start(10, true );
    evt.Skip();
}

void ArcBarGauge::SetValue(double value)
{

    if(m_gd == NULL){
        Logger::logf(LOG_MSG,_T("m_gd is NULL !!"));
        return;
    }

    m_CurrValue = value;
    m_value = value;

    if(value > m_gd->GetMaxValue()){
        m_CurrValue = m_gd->GetMaxValue();
    }

    if(value < m_gd->GetMinValue()){
        m_CurrValue = m_gd->GetMinValue();
    }

    UpdateBarValue();
    UpdateGaugeValue();

}

void ArcBarGauge::UpdateBarValue()
{
    wxRect rrect;

    double tmp = m_gd->GetMinValue();
    if (tmp < 0) tmp = 0;


    m_idxValue = (short int)((SEGSUM * (m_CurrValue/( m_gd->GetMaxValue()- tmp ))));

    if(m_idxValue >= SEGSUM) m_idxValue = SEGSUM-1;

    if(m_idxValue < 0) m_idxValue = 0;

    if(m_idxValue == m_idxPrevValue) return;

    m_idxPrevValue = m_idxValue;


    Refresh(false);

}

void ArcBarGauge::UpdateGaugeValue()
{

    if(!m_segments.GetCount()) return;

	wxClientDC dc(this);
	wxSize tsize;

	dc.SetFont(*DigitalFont);

    // get the area under the old value
    tsize = dc.GetTextExtent(sDigiValue);
    sDigiPoint = wxPoint( DigitalValueCenter.x - tsize.x/2, DigitalValueCenter.y - tsize.y );
    wxRect rectold(sDigiPoint,tsize);

    // update value
    sDigiValue = wxString::Format(m_gd->GetGaugeNumberFormat(),m_value);
    tsize = dc.GetTextExtent(sDigiValue);
    sDigiPoint = wxPoint( DigitalValueCenter.x - tsize.x/2, DigitalValueCenter.y - tsize.y );
    wxRect rectnew(sDigiPoint,tsize);

	RefreshRect(rectold.Union(rectnew),false);

//	Logger::log(LOG_MSG,_T("VemsGauge::UpdateGaugeValue() END"));
}


void ArcBarGauge::CalculateColors()
{
    if(m_segments.GetCount() == 0) return;

    int max = m_segments.GetCount()-1;

    for( uint8_t b = 0; b <= max; b++) {
        m_segments[b]->SetActiveColor( m_theme->ColorSymbols[_T("normal")] );
        m_segments[b]->SetInActiveColor( m_theme->ColorSymbols[_T("inactive")] );
    }


    for(uint16_t i = 0; i <  m_gd->coloredA.Count(); i++){

        ColoredArea * ca = m_gd->coloredA.Item(i);

        if( ca->minValue < 0 ) continue;

        if( m_theme->ColorSymbols.find(ca->ColorName) == m_theme->ColorSymbols.end() ) continue;

        uint8_t minColorIdx = (uint8_t)((ca->minValue / m_gd->GetMaxValue()) * (max+1)) ;
        uint8_t maxColorIdx = (uint8_t)((ca->maxValue / m_gd->GetMaxValue()) * (max+1)) ;

        if(i>0) minColorIdx++;

        for( uint8_t a = minColorIdx; a <= maxColorIdx; a++) {
            int t = max - a;

            if( t < 0 || t > max ) continue;

            m_segments[t]->SetActiveColor( m_theme->ColorSymbols[ ca->ColorName ] );
        }

    }

}

void ArcBarGauge::OnPaint(wxPaintEvent& WXUNUSED(event))
{

    wxBufferedPaintDC dc(this);

    dc.SetPen(*wxTRANSPARENT_PEN);
    dc.SetBackground( m_theme->BackgroundColor );
    dc.Clear();

    dc.DrawBitmap( *(m_bmpValues[m_idxValue]),0,0,1 );


    ///draw digital value
    dc.SetFont(*DigitalFont);
    dc.SetTextForeground( m_theme->ValueColor );
    dc.DrawText(sDigiValue,sDigiPoint.x,sDigiPoint.y);

    m_calcedRefSegments = false;


    DrawExtraLayer(dc);
}


void ArcBarGauge::SegmentInit()
{

    SEGSUM = 53;  /// the quantity of segment


    AlphaInc = (double)(m_StartAngle - m_EndAngle) / (double)SEGSUM;



    CreateSegments();

}

void ArcBarGauge::CalcSegments()
{

    int width = GetClientSize().x;
    double MinSegmentHeight = 2.6;
    double dsum = SEGSUM;

    double border = 0.8;

    double l_segmentWidthDeg = AlphaInc * border;
    double l_segmentHeightPix = GetSize().y/2.37;

    double SegmentHeightReducer = (l_segmentHeightPix - MinSegmentHeight) / dsum;

    m_InnerRadius = width;

    m_InnerCenter.x = width/2;
    m_InnerCenter.y = (int)(width*1.10);

    for(uint16_t a=0; a< m_segments.GetCount(); a++){

        l_segmentHeightPix -= SegmentHeightReducer;

        m_segments[a]->CalculatePoints( m_InnerCenter, m_InnerRadius, l_segmentHeightPix, l_segmentWidthDeg );

    }

}


void ArcBarGauge::CreateSegments()
{
    SegmentDesc *desc;

    for(double alpha = m_EndAngle; alpha < m_StartAngle; alpha+=AlphaInc)  {
        desc = new SegmentDesc(m_graphQual, alpha );
        desc->m_startAlpha = m_StartAngle;
        desc->m_endAlpha = m_EndAngle;
        m_segments.Add(desc);
    }

    Logger::logf(LOG_MSG,_T("ArcBarGauge::CreateSegments() AlphaInc=%0.2f,%i,%i"),AlphaInc,SEGSUM,m_segments.GetCount());

    SEGSUM = m_segments.GetCount();


}

void ArcBarGauge::PaintBackground(void)
{

}

void ArcBarGauge::PaintGaugeNumbers(wxDC &dc)
{

    dc.SetFont(wxFont(GetSize().x/30,wxSWISS,wxFONTSTYLE_NORMAL,wxFONTWEIGHT_NORMAL,false));//,_T("7 Segment"),wxFONTENCODING_DEFAULT));
	dc.SetTextForeground( m_theme->DivTextColor );
	//dc.SetTextBackground(*faceColor);

    wxPoint textp;
    double alpha=m_StartAngle;
    double dMDbuff = m_gd->GetMajorDivisions();

    double trad = m_InnerRadius*0.96;

    //Logger::log(LOG_MSG,wxString::Format(_T(""),m_gd->m_gaugeNumbers.GetCount()));

    for(uint16_t i=0; i < m_gd->m_gaugeNumbers.GetCount();i++) {

        wxString number = m_gd->m_gaugeNumbers.Item(i);
        double tmp = 0.0;
        number.ToDouble(&tmp);
        if( tmp < 0 ) {
            dMDbuff--;
            continue;
        }

        wxSize numsize = dc.GetTextExtent(number);
        double offsetx = numsize.x/2;
        double offsety = numsize.y/2;

        textp.x = (int)((m_InnerCenter.x + trad * cos(alpha*RAD_FACT) ) - offsetx);
        textp.y = (int)((m_InnerCenter.y - trad * sin(alpha*RAD_FACT) ) - offsety);

        dc.DrawText(number, textp.x, textp.y);

        alpha -= ((m_StartAngle-m_EndAngle)/dMDbuff);
    }

}



void ArcBarGauge::PaintTickMarks(wxDC &dc)
{
    wxAgg aggCanvas( GetSize().x, GetSize().y);

    double TickMarksHeight =  GetSize().x/100.0;

    double r = m_InnerRadius - (TickMarksHeight*0.9);

    aggCanvas.setPen( *wxGREEN );

   //aggCanvas.drawLine( Math::CalcCircPoint(m_StartAngle, r, m_InnerCenter), Math::CalcCircPoint(m_EndAngle, r, m_InnerCenter));

    double startrad = (m_EndAngle*RAD_FACT) + 3.09;
    double deltarad = (m_StartAngle - m_EndAngle)*RAD_FACT;

    aggCanvas.drawArc( m_InnerCenter, r, startrad, deltarad , 1.5 );


    double dMD = m_gd->GetMajorDivisions();

    for(char b=0; b<= dMD ; b++){
        double angle = m_EndAngle + b*((m_StartAngle-m_EndAngle)/dMD);


        aggCanvas.drawLine( Math::CalcCircPoint(angle, r-TickMarksHeight, m_InnerCenter), Math::CalcCircPoint(angle, r, m_InnerCenter), 1.5 );
    }


    aggCanvas.render(dc);

}



wxRegion ArcBarGauge::GetShape()
{

#if defined (__WINCE_STANDARDSDK__)

    wxRegion region = wxRegion(0,0,GetSize().x,GetSize().y);

#else

    wxRegion region = wxRegion(0,0,GetSize().x,GetSize().y);
/*
    PaintBackground();
    wxRegion region = wxRegion(*m_BarGaugeBgBitmap,wxColor(255,255,255));
    region.ConvertToBitmap().SaveFile(_T("regionarcbar.bmp"),wxBITMAP_TYPE_BMP,0);
*/

#endif

    return region;
}

void ArcBarGauge::UpdateSize()
{

    DigitalValueCenter.x = GetSize().x/2;
    DigitalValueCenter.y = GetSize().y-5;


    DigitalFont->SetPointSize((int)(GetSize().x/10.6));


    CalculateColors();

    CalcSegments();

    UpdateGaugeValue();

    Refresh(false);

    m_calcedRefSegments = false;

    CreateBmpStates();


}

void SegmentDesc::CalculatePoints(wxPoint InnerCenter, double InnerRadius, double SegmentHeightPix, double SegmentWidthDeg)
{

    switch(m_type) {

        case GRAPH_POOR: /// normal wxPoint coords
            m_coords[0].x = (int)(InnerCenter.x + InnerRadius*cos(m_alpha*RAD_FACT));
            m_coords[0].y = (int)(InnerCenter.y - InnerRadius*sin(m_alpha*RAD_FACT));

            m_coords[1].x = (int)(InnerCenter.x + (InnerRadius+SegmentHeightPix)*cos(m_alpha*RAD_FACT));
            m_coords[1].y = (int)(InnerCenter.y - (InnerRadius+SegmentHeightPix)*sin(m_alpha*RAD_FACT));

            m_coords[2].x = (int)(InnerCenter.x + (InnerRadius+SegmentHeightPix)*cos((m_alpha+SegmentWidthDeg)*RAD_FACT));
            m_coords[2].y = (int)(InnerCenter.y - (InnerRadius+SegmentHeightPix)*sin((m_alpha+SegmentWidthDeg)*RAD_FACT));

            m_coords[3].x = (int)(InnerCenter.x + InnerRadius*cos((m_alpha+SegmentWidthDeg)*RAD_FACT));
            m_coords[3].y = (int)(InnerCenter.y - InnerRadius*sin((m_alpha+SegmentWidthDeg)*RAD_FACT));

            for( uint8_t a=0; a<4; a++) {
                if( m_xMin > m_coords[a].x ) m_xMin = m_coords[a].x;
                if( m_xMax < m_coords[a].x ) m_xMax = m_coords[a].x;
                if( m_yMin > m_coords[a].y ) m_yMin = m_coords[a].y;
                if( m_yMax < m_coords[a].y ) m_yMax = m_coords[a].y;
            }

          break;


        case GRAPH_AGG:{ /// draw with agg

            m_aggCoords.Clear();

            double ccc = 0.95 + 0.04 * ( (m_startAlpha-m_alpha) / (m_startAlpha - m_endAlpha));

            wxRealPoint a,b,c,d;

            a.x = (InnerCenter.x + InnerRadius*cos(m_alpha*RAD_FACT));
            a.y = (InnerCenter.y - InnerRadius*sin(m_alpha*RAD_FACT));

            b.x = (InnerCenter.x + (InnerRadius+SegmentHeightPix)*cos(m_alpha*RAD_FACT));
            b.y = (InnerCenter.y - (InnerRadius+SegmentHeightPix)*sin(m_alpha*RAD_FACT));

            c.x = (InnerCenter.x + (InnerRadius+SegmentHeightPix*ccc)*cos((m_alpha+SegmentWidthDeg)*RAD_FACT));
            c.y = (InnerCenter.y - (InnerRadius+SegmentHeightPix*ccc)*sin((m_alpha+SegmentWidthDeg)*RAD_FACT));

            d.x = (InnerCenter.x + InnerRadius*cos((m_alpha+SegmentWidthDeg)*RAD_FACT));
            d.y = (InnerCenter.y - InnerRadius*sin((m_alpha+SegmentWidthDeg)*RAD_FACT));

            m_aggCoords.Add(a);
            m_aggCoords.Add(b);
            m_aggCoords.Add(c);
            m_aggCoords.Add(d);

            for(uint8_t i=0; i<4; i++) {
                if( m_xMin > m_aggCoords[i].x ) m_xMin = (int)(m_aggCoords[i].x);
                if( m_xMax < m_aggCoords[i].x ) m_xMax = (int)(m_aggCoords[i].x);
                if( m_yMin > m_aggCoords[i].y ) m_yMin = (int)(m_aggCoords[i].y);
                if( m_yMax < m_aggCoords[i].y ) m_yMax = (int)(m_aggCoords[i].y);
            }

            m_lineWidth = ((InnerRadius * 2.0 * M_PI) / 360.0) * SegmentWidthDeg;




            /*


            double x0 = InnerCenter.x + (InnerRadius+SegmentHeightPix)*cos((m_alpha+SegmentWidthDeg/2.0)*RAD_FACT);
            double y0 = InnerCenter.y - (InnerRadius+SegmentHeightPix)*sin((m_alpha+SegmentWidthDeg/2.0)*RAD_FACT);

            double x1 = InnerCenter.x + InnerRadius*cos((m_alpha+SegmentWidthDeg/2.0)*RAD_FACT);
            double y1 = InnerCenter.y - InnerRadius*sin((m_alpha+SegmentWidthDeg/2.0)*RAD_FACT);

            m_lineWidth = ((InnerRadius * 2.0 * M_PI) / 360.0) * SegmentWidthDeg;

            double beta = 180.0 - m_alpha;

            double diffx = (cos(beta*RAD_FACT) * (m_lineWidth / 2.0));
            double diffy = (sin(beta*RAD_FACT) * (m_lineWidth / 2.0));

            wxPoint tmp[4];

            tmp[0].x = (int)(x0 + diffx);
            tmp[0].y = (int)(y0 - diffy);

            tmp[1].x = (int)(x0 - diffx);
            tmp[1].y = (int)(y0 + diffy);

            tmp[2].x = (int)(x1 - diffx);
            tmp[2].y = (int)(y1 - diffy);

            tmp[3].x = (int)(x1 + diffx);
            tmp[3].y = (int)(y1 + diffy);

            for( c=0; c<4; c++) {
                if( m_xMin > tmp[c].x ) m_xMin = tmp[c].x;
                if( m_xMax < tmp[c].x ) m_xMax = tmp[c].x;
                if( m_yMin > tmp[c].y ) m_yMin = tmp[c].y;
                if( m_yMax < tmp[c].y ) m_yMax = tmp[c].y;
            }



            m_aggCoords.Add(wxRealPoint(x0,y0));
            m_aggCoords.Add(wxRealPoint(x1,y1));
*/
          break;}


    }


}
