#ifndef ARCBARGAUGE_H
#define ARCBARGAUGE_H

#include <wx/frame.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/dcmemory.h>
#include <wx/dcmemory.h>

#include "../rtdatamanager/RTDataViewerBase.h"
#include "../gaugevalueselector.h"
#include "VemsWidgetBase.h"
#include "vemsgaugedesc.h"


//#define SEGSUM 53

#define GRAPH_POOR      0
#define GRAPH_MIDDLE    1
#define GRAPH_RICH      2
#define GRAPH_AGG       GRAPH_MIDDLE

#include "../agg/wxAgg.h"
#include "../common/types/wxBitmapArray.h"

class SegmentDesc
{
    public:
        SegmentDesc(int type, double alpha) {
            m_alpha = alpha;
            m_type = type;
            m_refreshable = true;
            m_xMin = 1000;
            m_xMax = 0;
            m_yMin = 1000;
            m_yMax = 0;


        };
        virtual ~SegmentDesc() {};

        void CalculatePoints(wxPoint InnerCenter, double InnerRadius, double SegmentHeightPix, double SegmentWidthDeg);


        int  GetType() { return m_type; };

        void SetActiveColor(const wxColor &color) {
            m_activeColor = color;

            m_refreshable = m_prevActiveColor == m_activeColor ? 0 : 1;

            m_prevActiveColor = m_activeColor;

        }

        void SetInActiveColor(const wxColor &color) { m_inactiveColor = color; }

        wxPoint *GetNormalCoords() {
            if(m_type == GRAPH_POOR)
                return m_coords;
            else
                return NULL; }

        RealPointArray *GetAggCoords() {
            if(m_type == GRAPH_AGG)
                return &m_aggCoords;
            else
                return NULL; }

        wxColor& GetColor(bool active, bool previous = false) {
            if(active)
                return m_activeColor;
            else
                return m_inactiveColor;     }


        double GetLineWidth() {
            if(m_type == GRAPH_AGG)
                return m_lineWidth;
            else
                return 1;   }

        bool IsRefreshAble() { return m_refreshable; };

        double      m_alpha;
        bool        m_refreshable;

        wxCoord     m_xMin;
        wxCoord     m_xMax;
        wxCoord     m_yMin;
        wxCoord     m_yMax;

        double      m_startAlpha;
        double      m_endAlpha;


    private:



        wxPoint     m_coords[4];


        RealPointArray  m_aggCoords;

        double      m_lineWidth;

        int         m_type;

        wxColor     m_inactiveColor;
        wxColor     m_activeColor;
        wxColor     m_prevActiveColor;

};


WX_DEFINE_ARRAY( SegmentDesc* , DescArray );



class ArcBarGauge: public VemsWidgetBase
{
    public:
        ArcBarGauge(wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop );
        /*const wxString& symbolName = _T("rpm"), wxPoint gPos = wxDefaultPosition, int width = 120,
            long style = wxNO_BORDER, const VEMSGaugeDescArray * gaugeDescA = NULL,const WidgetThemeTableArray * themes=NULL, const wxString& currTheme = ThemesHelper::m_defaultThemeName );*/
        virtual ~ArcBarGauge();

        void OnPaint(wxPaintEvent& WXUNUSED(event));

        void SetValue(double value);
        virtual void UpdateSize();
        virtual wxRegion GetShape();
        wxRegion GetBorderRegion() { return wxRegion(); };
        void setSize(int x , int y){

        }

        void UpdateGaugeValue();
        void UpdateBarValue();
        void PaintBackground(void);
        void PaintGaugeNumbers(wxDC &dc);
        void PaintTickMarks(wxDC &dc);
        void CalculateColors();

        void SetNewTheme(WidgetThemeTable *theme);

    protected:

        DECLARE_EVENT_TABLE();

        void CreateBmpStates();

        void SegmentInit();
        void CalcSegments();
        void CreateSegments();
        void OnSize( wxSizeEvent &evt);

    private:

        wxBitmapArray m_bmpValues;
        wxMemoryDC m_backGroundMemDC;
        wxString m_selectedSymbol;
        int m_defGaugeId;
        bool PaintBgSem;
        wxPoint SegmentPoints[4];
        //TDPoint TDSegmentPoints[5];
        double SegmentWidthDeg;
        double SegmentHeightPix;
        int m_InnerRadius;
        int OuterRadius;
        wxPoint m_InnerCenter;
        wxPoint OuterCenter;

        double m_CurrValue;
        double m_AlphaValue;
        double m_AlphaPreviousValue;
        int m_StartAngle;
        int m_EndAngle;
        short m_idxValue;
        short m_idxPrevValue;

        wxFont *DigitalFont;
        wxPoint DigitalValueCenter;  //note:  horizontal: center, vertical: bottom
        wxPoint sDigiPoint;
        wxWindow *m_parent;
        wxString sDigiValue;

        int SEGSUM;
        double AlphaInc;

        double SegmentHeights[110];
        wxColor SegmentColors[110];


        //TMGraphics  *GBuffer;
        //TMGraphics  *GCover;


        DescArray   m_segments;


        bool        m_calcedRefSegments;

        wxTimer     m_timer;

};



#endif // ARCBARGAUGE_H

