#ifndef EXPRESSIONGRIDDESC_H
#define EXPRESSIONGRIDDESC_H

#include "../common/types/wxArrayStringArray.h"
#include "../common/types/Int2StringHash.h"
#include "boost/multi_array.hpp"


class ExpressionGridDescHelper
{
    public:
        static wxAlignment parseAlign( const wxString& align )
        {
            if ( align.IsSameAs( _T("LEFT"), false ) ) return wxALIGN_LEFT;
            else if ( align.IsSameAs( _T("RIGHT"), false ) ) return wxALIGN_RIGHT;
            else if ( align.IsSameAs( _T("CENTER"), false ) ) return wxALIGN_CENTER_HORIZONTAL;
            else return wxALIGN_RIGHT;
        }

    private:
        ~ExpressionGridDescHelper();
};


struct ExpressionCellLink
{
    wxString        dialogId;
    wxArrayString   fieldIds;
    wxString        enableExpr;
    wxArrayString   variables;
};

WX_DECLARE_OBJARRAY( ExpressionCellLink, ExpressionCellLinkArray );

struct ExprCellText
{
    wxString    text;
    wxString    enableExpr;
};

WX_DECLARE_OBJARRAY( ExprCellText, ExprCellTextArray );

struct ExprCellColor
{
    wxString    bgColor;
    wxString    fgColor;
    wxString    enableExpr;
};

WX_DECLARE_OBJARRAY( ExprCellColor, ExprCellColorArray );

struct ExpressionCellDescriptor
{
    ExpressionCellLinkArray 	links;
    ExprCellTextArray           texts;
    ExprCellColorArray          colors;
    wxString 	                text;
    int 		                width;
    int 		                precision;
    int 		                align;
    wxString                    help;
};

typedef boost::multi_array<ExpressionCellDescriptor, 2> ExpressionCellDescriptorMatrix;


struct ExpressionGridDesc
{
    ExpressionCellDescriptorMatrix  cells;
    Int2StringHash                  linesEnableExpr;
    wxString                        name;
    wxString                        title;
    int                             disableCol;
};

WX_DEFINE_ARRAY( ExpressionGridDesc*, ExpressionGridDescArray );


#endif // EXPRESSIONGRIDDESC_H
