#include "GaugeCarrierAdvFrame.h"

#include <wx/wx.h>

#include "../vemsdialog/CalibrationBase.h"

#include "../vemsdialog/UserDialog.h"
#include "../gaugedesc/GaugeProperties.h"
#include "../gaugedesc/GaugeDescHelper.h"
#include "../gaugedesc/GaugeDescManager.h"

#include "../vemsdialog/UserDialogTemplate.h"
#include "../vemsdialog/StaticControlPanel.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "GaugeCarrierPanel.h"
#include "../vemsgauge/GaugeType.h"
#include "../vemsgauge/GaugeGroupManager.h"


GaugeCarrierAdvFrame::GaugeCarrierAdvFrame(wxWindow* parent, wxWindowID id,
    const wxString& title, const wxPoint& pos, size_t gaugeCount, long style)
{
    //ctor
    //InitWindow( ((wxFrame*)this) );
     //wxSize fSize = wxSize(( gaugeCount +1) *240,380);

#if defined(__WINCE_STANDARDSDK__)
    Create(parent,id,title,pos,wxDefaultSize,style);
#else
    #if defined(__WXMSW__)
        //Create(parent,id,title,pos,size,style|wxFRAME_SHAPED | wxFRAME_TOOL_WINDOW  | wxFRAME_FLOAT_ON_PARENT );
        Create(parent,id,title,pos,wxDefaultSize, style | wxFRAME_FLOAT_ON_PARENT );
    #else
        Create(parent,id,title,pos,wxDefaultSize,style);
    #endif
#endif
    //this->SetSizer( new wxBoxSizer( wxHORIZONTAL ) );

    wxFlexGridSizer* FlexGridSizer1 = new wxFlexGridSizer( 0, gaugeCount+1, 0, 0 );
    FlexGridSizer1->AddGrowableCol(0,0);
    FlexGridSizer1->AddGrowableRow(0,0);
    this->SetSizer( FlexGridSizer1 );

    m_dialogSizer = new wxBoxSizer( wxVERTICAL );

    GetSizer()->Add( m_dialogSizer, 0, wxEXPAND );

    m_calibrationBase = NULL;

}

GaugeCarrierAdvFrame::~GaugeCarrierAdvFrame()
{
    //dtor
    unregisterViewer();
}

void GaugeCarrierAdvFrame::buildGUI(wxWindow* parent)
{


}

void GaugeCarrierAdvFrame::SetCalibrationBase(CalibrationBase* calibrationBase)
{
    m_calibrationBase = calibrationBase;

    m_rtDataMgr->RegisterViewer(m_calibrationBase);
    m_calibrationBase->SetDataMgr(m_rtDataMgr);

}

void GaugeCarrierAdvFrame::unregisterViewer()
{
    for( size_t i = 0; i < m_viewers.GetCount(); ++i )
    {
        m_viewers.Item( i )->UnRegisterViewer();
    }

}

void GaugeCarrierAdvFrame::SetDataMgrObj(RTDataMgrBase *rtDataMgr)
{
    m_rtDataMgr = rtDataMgr;
}

void GaugeCarrierAdvFrame::SetDataMgrObj(STDataMgrBase *stDataMgr)
{
    m_stDataMgr = stDataMgr;
}

bool GaugeCarrierAdvFrame::addDialog( UserDialog* userDialogDesc, SymbolMgr* symbolmgr)
{
    UserDialogTemplate* staticDataPanel = new UserDialogTemplate();
    staticDataPanel->setController( m_stDataMgr );
    staticDataPanel->Create( this, *userDialogDesc, symbolmgr, NULL, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
    //staticDataPanel->Create( this, userDialogDesc, wxID_ANY, wxDefaultPosition, wxDefaultSize);
    staticDataPanel->updateData();
    //staticDataPanel->setStatus();
    if(m_calibrationBase) staticDataPanel->setCalibration(m_calibrationBase);
    //m_controller = udt;
    //udt->refreshData();

    staticDataPanel->EnableScrolling(1,1);

    wxSize dialogSize = staticDataPanel->GetVirtualSize();
    m_dialogSizer->SetMinSize(dialogSize);

    //wxSizerItem* item = m_dialogSizer->Add( staticDataPanel, 1, wxEXPAND);
    m_dialogSizer->Add( staticDataPanel, 1, wxEXPAND);
    //item->SetInitSize(dialogSize);
/*
    wxString msg = _T("valami");
    wxStaticText* text = new wxStaticText( this, wxID_ANY, msg );
    text->SetForegroundColour( *wxRED );
    m_dialogSizer->Add( text, 0, wxALL, 5 );
*/
    //wxFrame* calibrationSpecificPanel = m_calibrationBase->getPanel();
    //if(calibrationSpecificPanel) m_dialogSizer->Add(calibrationSpecificPanel, 0, 0 );

    StaticControlPanel* ctrlPanel = new StaticControlPanel( this, staticDataPanel,
    StaticControlPanel::WRITE_BUTTON | StaticControlPanel::BURN_BUTTON |
    StaticControlPanel::START_CALIBRATE | StaticControlPanel::STOP_CALIBRATE);

    m_dialogSizer->Add( ctrlPanel, 0, wxEXPAND );

    GetSizer()->Fit(this);

    return true;
}

bool GaugeCarrierAdvFrame::addGauge(GaugeProperties& prop)
{

    GaugeCarrierPanel* gaugePanel = new GaugeCarrierPanel(this, wxID_ANY, _T("Text"));


//    VemsWidgetBaseFactory* vwFactory = new VemsWidgetBaseFactory();
//
//    vwFactory->SetDataMgrObj( m_rtDataMgr );
//    vwFactory->SetDataMgrObj( m_stDataMgr );
//    vwFactory->SetGaugeDescManager( GaugeDescHelper::GetGDMgr() );
//    vwFactory->SetGaugeGroupManager( GaugeDescHelper::GetGDMgr()->GetGaugeGrpMgr() );
//
//    m_gaugeGroupManager->SetFactory( vwFactory );

    gaugePanel->SetFactory( GaugeDescHelper::GetGDMgr()->GetGaugeGrpMgr()->GetFactories() );

//    gaugePanel->SetDataMgrObj(m_rtDataMgr);
  //  gaugePanel->SetDataMgrObj(m_stDataMgr);

    gaugePanel->OnlyOneGauge( true );
    gaugePanel->SetGaugeShowInit( true );
    gaugePanel->SetFloating( false );

    //VemsWidgetBase* gauge =
    wxWindow* gauge = gaugePanel->AddGauge( prop );
    if(!gauge) return false;
    //m_viewers.Add(gauge);

    gaugePanel->ShowCarrier(true);

    GetSizer()->Add( gaugePanel, 1, wxEXPAND );
    GetSizer()->Fit(this);

    return true;
}

CalibrationDesc GaugeCarrierAdvFrame::getWBO2CallDesc()
{
    CalibrationDesc desc( _T("widebandCalibration") );

    GaugeProperties& prop = desc.getGaugeProp();
    prop.symbolName = _T("stallO2");
    prop.itemID = -1;
    prop.gaugeType =  GaugeType::ROUND;

    prop.setPos(wxPoint(0,0));
    prop.setSize( wxSize(240,240) );
    prop.extraParams[_T("showBorder")] = false;

    return desc;
}
