#include "GaugeCarrierAuiMDIChildFrame.h"

#include <wx/dcbuffer.h>
#include <wx/clipbrd.h>

#include "../DisplayHelper.h"
#include "../logger.h"
#include "VemsWidgetBase.h"
#include "TableEditorPanel.h"
#include "GroupMember.h"
#include "GaugeGroupManager.h"
#include "../customwidgets/InfoPanelHelper.h"
#include "../wxSizeEx.h"
#include "../Math.h"
#include "../DirHelper.h"
#include "../common/FileDropTarget.h"
#include "GaugeSetHelper.h"
#include "../FileDropManagerBase.h"
#include "GaugeCarrierBaseDesc.h"
#include "../FileHelper.h"
#include "VemsControlBase.h"
#include "../stdatamanager/STDataViewerBase.h"
#include "GroupMemberObject.h"

#define HKEYPANELNAME _T("GaugeCarrierAuiMDIChildFramePanel")

#define USE_ADVANCED_SELECTION_MODE 1

BEGIN_EVENT_TABLE(GaugeCarrierAuiMDIChildFrame, wxAuiMDIChildFrame)

    EVT_KEY_DOWN(GaugeCarrierAuiMDIChildFrame::OnChar)

    EVT_RIGHT_DOWN(GaugeCarrierAuiMDIChildFrame::OnRightClick)
    EVT_LEFT_DOWN(GaugeCarrierAuiMDIChildFrame::OnLeftDown)

#if USE_ADVANCED_SELECTION_MODE
    EVT_LEFT_UP(GaugeCarrierAuiMDIChildFrame::OnLeftUp)
    EVT_MOTION(GaugeCarrierAuiMDIChildFrame::OnMouseMove)
#endif

    EVT_PAINT(GaugeCarrierAuiMDIChildFrame::OnPaint)
    EVT_CLOSE(GaugeCarrierAuiMDIChildFrame::OnClose)

    EVT_MENU_RANGE(idMenuExportAll,idMenuEND,GaugeCarrierAuiMDIChildFrame::OnMenu)

    EVT_KEY_DOWN(GaugeCarrierAuiMDIChildFrame::OnKeyDown)

    EVT_LEAVE_WINDOW(GaugeCarrierAuiMDIChildFrame::OnLeave)

END_EVENT_TABLE()



GaugeCarrierAuiMDIChildFrame::GaugeCarrierAuiMDIChildFrame(wxAuiMDIParentFrame * parent, int id, const wxString &pagetext, GaugeGroupManager* mgr)
    :GaugeCarrierBase (parent)
    ,wxScrollHelperNative(this)
    ,m_dropmgr(NULL)
{
    //ctor
    m_popupPos = new wxPoint();
    m_selectBoxStart = wxDefaultPosition;

    Create(parent,id,pagetext,wxDefaultPosition,wxDefaultSize, wxBORDER_SUNKEN | wxHSCROLL | wxVSCROLL );
    InitWindow(this);

    m_targetWindow = this;

    m_groupName = pagetext;

    SetBackgroundColour(wxColour(0,0,0));

    m_desc->m_hasShape = false;

    m_desc->m_graphQual = 2;
    m_desc->m_type = -1;

    m_groupMgr = mgr;

    m_dropmgr = dynamic_cast<FileDropManagerBase*>(parent);
    if(m_dropmgr) m_dropmgr->RegisterTarget(this);

    //new FileDropTarget<GaugeCarrierAuiMDIChildFrame>( this );

    EnableScrolling(1,1); /// doesn't work... todo: fix  :)
}

GaugeCarrierAuiMDIChildFrame::~GaugeCarrierAuiMDIChildFrame()
{
    //dtor
    if(m_dropmgr) m_dropmgr->UnRegisterTarget(this);
    delete m_popupPos;
}

bool GaugeCarrierAuiMDIChildFrame::AcceptDrop()
{
    if(!m_groupMgr->IsEditMode()) return false;
    if(!IsShown()) return false;

    return true;
}

bool GaugeCarrierAuiMDIChildFrame::DropFiles( const wxArrayString& files )
{
    if(!m_groupMgr->IsEditMode()) return false;

    for(size_t a=0; a < files.GetCount(); a++){

        wxFileName fn(files[a]);

        if( !fn.FileExists() ) continue;

        GaugeSetHelper::ImportGauges(files[a],m_groupMgr,this);
    }

    return true;
}

void GaugeCarrierAuiMDIChildFrame::OnLeave(wxMouseEvent& evt)
{
    //EndSelection();
}

bool GaugeCarrierAuiMDIChildFrame::onDropFiles( const wxArrayString& filenames )
{
    return true;
}

void GaugeCarrierAuiMDIChildFrame::DeleteSelectedItems(bool isConfirm)
{
    GaugePropertiesArray items;

    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        wxWindow* win = it->second;

        SelectableWidgetBase * swb = dynamic_cast<SelectableWidgetBase*>(win);

        if(!swb) continue;

        if(!swb->IsSelected()) continue;

        GroupMember* gm = dynamic_cast<GroupMember*>(win);

        if(!gm) {
            TableEditorPanel* tep = dynamic_cast<TableEditorPanel*>(win);
            if(!tep) continue;

            gm = tep->GetGroupMember();
        }

        if(!gm) continue;

        items.Add(gm->GetGaugeProperties());
    }

    if(items.GetCount() == 0) return;

    if(isConfirm) {

        wxString msg = _T("Do you really want to delete the selected ");

        msg += items.GetCount() == 1 ? _T("gauge?") : wxString::Format(_T("%i gauges?"), items.GetCount());

        if( wxMessageBox(msg,_T("Confirm"), wxYES_NO | wxICON_QUESTION ) == wxNO ) return;

    }

    m_groupMgr->StartTransaction();

    for(size_t a=0; a < items.GetCount() ; a++) {
        m_groupMgr->RemoveGauge( items[a] );
    }

    m_groupMgr->EndTransaction();
}

void GaugeCarrierAuiMDIChildFrame::OnKeyDown(wxKeyEvent& evt)
{
    evt.Skip();

    if( !m_groupMgr->IsEditMode() ) return;

    int kc = evt.GetKeyCode();

    //LOGP(wxString::Format(_T("kc=%i"),kc ));

    if(kc == WXK_DELETE) {
        DeleteSelectedItems( !evt.ShiftDown() );
    }

    if(kc == 'A' && evt.ControlDown()) {
        SelectAllItems();
    }

    if(kc == 'C' && evt.ControlDown()) {
        CopySelectedGauges();
    }

    if(kc == 'V' && evt.ControlDown()) {
        Paste();
    }

    if(kc == 'S' && evt.ControlDown()) {
        m_groupMgr->saveGroup(m_groupName);
    }

}


bool GaugeCarrierAuiMDIChildFrame::IsValidPaste()
{
    if(!wxTheClipboard->Open()) return false;

    if(!wxTheClipboard->IsSupported( wxDF_TEXT )) return false;

    wxTextDataObject data;
    wxTheClipboard->GetData( data );
    wxString gaugesetStr = data.GetText();

    wxTheClipboard->Close();

    if(gaugesetStr == wxEmptyString || gaugesetStr.Len() < 42) return false;

    return GaugeSetHelper::CheckXmlContentHeader(gaugesetStr);
}

void GaugeCarrierAuiMDIChildFrame::CopySelectedGauges()
{
    if(!wxTheClipboard->Open()) return;

    GaugePropertiesArray props;
    GetGaugesToExport(props,true);
    wxString gaugesetStr = GaugeSetHelper::ExportGauges(props);
    wxTheClipboard->SetData( new wxTextDataObject( gaugesetStr ) );

    wxTheClipboard->Close();
}

void GaugeCarrierAuiMDIChildFrame::Paste()
{
    if(!wxTheClipboard->Open()) return;

    if(!wxTheClipboard->IsSupported( wxDF_TEXT )) return;

    wxTextDataObject data;
    wxTheClipboard->GetData( data );
    wxString gaugesetStr = data.GetText();
    GaugeSetHelper::ImportGaugesFromStr(gaugesetStr, m_groupMgr, this);

    wxTheClipboard->Close();
}

void GaugeCarrierAuiMDIChildFrame::DropDeselectWidgetCommand()
{
    wxVariantHash hash;
    hash[_T("groupname")] = m_groupName;
    hash[_T("cmd")] =  _T("gauge_deselect");
	hash[_T("target_id")] =  _T("0");
	hash[_T("sender_id")] =  _T("0");

    ProcessWidgetCommand(hash);
}

void GaugeCarrierAuiMDIChildFrame::OnLeftDown(wxMouseEvent& evt)
{
    if(m_groupMgr->IsEditMode()) {
        m_selectBoxStart = evt.GetPosition();

        SetFocus();
    }

    if(!evt.ShiftDown()) {
        DropDeselectWidgetCommand();
    }
}

void GaugeCarrierAuiMDIChildFrame::SelectAllItems()
{

    if( !m_groupMgr->IsEditMode() ) return;

    wxVariantHash hash;
    hash[_T("groupname")] = m_groupName;
    hash[_T("cmd")] =  _T("gauge_select");

    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        wxWindow* win = it->second;

        SelectableWidgetBase * swb = dynamic_cast<SelectableWidgetBase*>(win);

        if(!swb) continue;

        hash[_T("target_id")] = (long)win->GetId();
        swb->ProcessWidgetCommand(hash);
    }

    Refresh(false);
}

void GaugeCarrierAuiMDIChildFrame::EndSelection()
{
    if( m_selectBoxStart == wxDefaultPosition ) return;

    if( !m_groupMgr->IsEditMode() ) return;

    wxSizeEx s( wxGetMousePosition() - GetScreenPosition() - m_selectBoxStart );

    wxRect selection( m_selectBoxStart, s );

    Math::NormalizeRect(selection);

    //Logger::logf(LOG_MSG,_T("selection x=%i,y=%i,w=%i,h=%i"),selection.x,selection.y,selection.width,selection.height);

//    if( selection.width < 1 || selection.height < 1 ) {
//        m_selectBoxStart = wxDefaultPosition;
//        return;
//    }

    wxVariantHash hash;
    hash[_T("groupname")] = m_groupName;
    hash[_T("cmd")] =  _T("gauge_select");

    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        wxWindow* win = it->second;

        SelectableWidgetBase * swb = dynamic_cast<SelectableWidgetBase*>(win);

        if(!swb) continue;

        wxRect grect = win->GetRect();

        if(selection.Intersects(grect)) {
            hash[_T("target_id")] = (long)win->GetId();
            swb->ProcessWidgetCommand(hash);
        }

    }

    m_selectBoxStart = wxDefaultPosition;
    Refresh(false);
}

void GaugeCarrierAuiMDIChildFrame::OnLeftUp(wxMouseEvent& evt)
{
    EndSelection();
}

void GaugeCarrierAuiMDIChildFrame::OnMouseMove(wxMouseEvent& evt)
{
    if( !m_groupMgr->IsEditMode() ) return;

    if( m_selectBoxStart == wxDefaultPosition ) return;

    if(!evt.LeftIsDown()) {
        EndSelection();
        return;
    }

    Refresh(false);
}

void GaugeCarrierAuiMDIChildFrame::GetGaugesToExport(GaugePropertiesArray& arr, bool onlySelected)
{
    for(wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        wxWindow* win = it->second;

        wxRect grect = win->GetRect();

        if(onlySelected) {
            SelectableWidgetBase * swb = dynamic_cast<SelectableWidgetBase*>(win);
            if(!swb) continue;
            if(!(swb->IsSelected())) continue;
        }

        GroupMember* gm = dynamic_cast<GroupMember*>(win);

        if(!gm) {
            TableEditorPanel* tep = dynamic_cast<TableEditorPanel*>(win);
            if(!tep) continue;

            grect.SetSize( tep->GetClientSize() );

            gm = tep->GetGroupMember();
        }

        if(!gm) continue;

        //grect.Offset(offset);

        GaugeProperties prop = gm->GetGaugeProperties();

        prop.setPos( grect.GetPosition() );
        prop.setSize( grect.GetSize() );

        arr.Add(prop);
    }
}

void GaugeCarrierAuiMDIChildFrame::OnMenu(wxCommandEvent& evt)
{
    wxSize s = DisplayHelper::GetDisplaySize();

    wxDateTime tnow = wxDateTime::Now();

    wxString date  = wxString::Format(_T("%i.%.2i.%.2i"),tnow.GetYear(),tnow.GetMonth()+1,tnow.GetDay());
    date += wxString::Format(_T("-%.2i.%.2i.%.2i"),tnow.GetHour(),tnow.GetMinute(),tnow.GetSecond());

    wxString file = wxString::Format(_T("Gauges-")+GetTitle()+_T("-%ix%i-")+date+_T(".xml"),s.x,s.y);

    switch(evt.GetId()) {
        case idMenuExportAll:{
                wxFileDialog dlg(this,_T("Save to file"), DirHelper::GetPath(dWORKINGDIR) , file, _T("XML files (*.xml)|*.xml"),wxFD_SAVE);
                if(dlg.ShowModal() != wxID_OK) return;
                GaugePropertiesArray props;
                GetGaugesToExport(props,false);
                GaugeSetHelper::ExportGauges(props, dlg.GetPath() );
            }break;

        case idMenuExportSelected:{
                wxFileDialog dlg(this,_T("Save to file"), DirHelper::GetPath(dWORKINGDIR) , file, _T("XML files (*.xml)|*.xml"),wxFD_SAVE);
                if(dlg.ShowModal() != wxID_OK) return;
                GaugePropertiesArray props;
                GetGaugesToExport(props,true);
                GaugeSetHelper::ExportGauges(props, dlg.GetPath() );
            }break;

        case idMenuImport:{
                wxFileDialog dlg(this,_T("Load from file"), DirHelper::GetPath(dWORKINGDIR) , wxEmptyString, _T("XML files (*.xml)|*.xml"),wxFD_DEFAULT_STYLE);
                if(dlg.ShowModal() != wxID_OK) return;
                GaugePropertiesArray props;
                GaugeSetHelper::ImportGauges(dlg.GetPath(), m_groupMgr, this );
        }break;

        case idMenuCopy:
            CopySelectedGauges();
            break;

        case idMenuPaste:
            Paste();
            break;

        case idMenuSave:
            m_groupMgr->saveGroup(m_groupName);
            break;

        case idMenuSelectAll:
            SelectAllItems();
            break;

        case idMenuDelSelected:
            DeleteSelectedItems();
            break;

        case idMenuDelSelectedNoConfirm:
            DeleteSelectedItems(false);
            break;
    }
}

void GaugeCarrierAuiMDIChildFrame::createAddSubMenu(wxMenu* parent)
{
    wxMenu* addMenu = new wxMenu;

    int id = wxNewId();
    addMenu->Append( id, _T("VemsGauge") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("VemsGauge"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Digital") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("Digital"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Text") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("Text"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Flag") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("Flag"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Graph") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("Graph"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("ArcBar") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("ArcBar"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Differ") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("Differ"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("ColumnsGauge") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("ColumnsGauge"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Multi Graph") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("MultiGraph"),-1, -1,m_popupPos  ), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Table") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("table"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Table 3D") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("table3d"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("Expression Grid") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("ExpressionGrid"),-1,-1,m_popupPos), m_groupMgr );

    id = wxNewId();
    addMenu->Append( id, _T("GPS Compass") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("Compass"),-1,-1,m_popupPos), m_groupMgr );


    id = wxNewId();
    addMenu->Append( id, _T("GPS route") );
    Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnAddGauge ),
                       new GroupMemberObject( this, m_groupName, _T("GPS route"),-1,-1,m_popupPos), m_groupMgr );


    parent->AppendSubMenu(addMenu, _T("Add new gauge"));
}

bool GaugeCarrierAuiMDIChildFrame::HasSelectedItem()
{
    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        SelectableWidgetBase * swb = dynamic_cast<SelectableWidgetBase*>(it->second);

        if(swb && swb->IsSelected()) return true;
    }

    return false;
}

void GaugeCarrierAuiMDIChildFrame::createExportSubMenu(wxMenu* parent)
{
    wxMenu* addMenu = new wxMenu;

    addMenu->Append(idMenuExportAll,_T("All"));

    if(m_groupMgr->IsEditMode()) {

        addMenu->Append(idMenuExportSelected,_T("Selected only"));

        addMenu->Enable(idMenuExportSelected,HasSelectedItem());
    }

    parent->AppendSubMenu(addMenu, _T("Export gauges to file"));
}

void GaugeCarrierAuiMDIChildFrame::createImportSubMenu(wxMenu* parent)
{
    parent->Append(idMenuImport,_T("Import gauges from file"));
}


void GaugeCarrierAuiMDIChildFrame::createCopyPasteMenu(wxMenu* parent)
{
    parent->Append(idMenuCopy,_T("Copy\tCtrl+C"));
    parent->Append(idMenuPaste,_T("Paste\tCtrl+V"));

    parent->Enable(idMenuCopy, HasSelectedItem());
    parent->Enable(idMenuPaste, IsValidPaste() );
}

void GaugeCarrierAuiMDIChildFrame::OnRightClick(wxMouseEvent& evt)
{
    if(!m_groupMgr->IsEditMode()) return;

    wxPoint pos = evt.GetPosition();

    m_popupPos->x = pos.x;
    m_popupPos->y = pos.y;

    wxMenu popupMenu;

    createAddSubMenu(&popupMenu);
    popupMenu.AppendSeparator();
    popupMenu.Append(idMenuSelectAll,_T("Select all\tCtrl+A"));
    popupMenu.Append(idMenuDelSelected,_T("Delete selected\tDelete"));
    popupMenu.Append(idMenuDelSelectedNoConfirm,_T("Delete selected (no ask)\tShift+Delete"));
    popupMenu.Enable(idMenuDelSelected, HasSelectedItem());
    popupMenu.Enable(idMenuDelSelectedNoConfirm, HasSelectedItem());
    popupMenu.AppendSeparator();
    createExportSubMenu(&popupMenu);
    createImportSubMenu(&popupMenu);
    popupMenu.AppendSeparator();
    createCopyPasteMenu(&popupMenu);
    popupMenu.AppendSeparator();
    popupMenu.Append(idMenuSave, _T("Save gauges layout\tCtrl+S"));


    PopupMenu( &popupMenu );
}

void GaugeCarrierAuiMDIChildFrame::SetFocusToItem(int id)
{
    //Logger::logf(LOG_MSG,_T("GaugeCarrierAuiMDIChildFrame::SetFocusToItem"));

    if( id == -1) {  // set focus to the last Table

        TableEditorPanel* tep = 0;

        for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

            wxString classname = it->second->GetClassInfo()->GetClassName();

            //Logger::logf(LOG_MSG,_T("m_desc->m_IDtoControls[%i]=%p > ")+classname,it->first,it->second);

            if(classname == _T("TableEditorPanel"))
                tep = (TableEditorPanel*)(it->second);

        }

        Logger::logf(LOG_MSG,_T("tep=%p"),tep);

        if( tep ) tep->GetTable()->SetFocus();

    } else {

        if(m_desc->m_IDtoControls.find(id) == m_desc->m_IDtoControls.end()) return;
        wxWindow * win = m_desc->m_IDtoControls[id];
        if( !win ) return;
        win->SetFocus();

    }

}

void GaugeCarrierAuiMDIChildFrame::ProcessWidgetCommand(wxVariantHash& cmdHash)
{
    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        SelectableWidgetBase * swb = dynamic_cast<SelectableWidgetBase*>(it->second);
        if(swb) swb->ProcessWidgetCommand(cmdHash);

        //VemsWidgetBase * v = wxDynamicCast( it->second, VemsWidgetBase ) ;
        //if(v) v->ProcessWidgetCommand(cmdHash);
    }
}

void GaugeCarrierAuiMDIChildFrame::ResetMinMax()
{
    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){
        VemsWidgetBase * v = wxDynamicCast( it->second, VemsWidgetBase ) ;
        if(v) v->ResetMinMax();
    }

}

void GaugeCarrierAuiMDIChildFrame::ShowCarrier(bool isShow)
{
    //maybe not neccessary
}

void GaugeCarrierAuiMDIChildFrame::CloseCarrier()
{

    for( wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){
        CloseGauge( it->first, false );
    }

    m_desc->m_IDtoControls.clear();

    Destroy();
}

void GaugeCarrierAuiMDIChildFrame::CloseGauge(int winid, bool eraseHash)
{
//    Logger::logf(LOG_MSG,_T("close: %i, size=%i"),winid,m_desc->m_IDtoControls.size());

    if(winid == -1 || m_desc->m_IDtoControls.find(winid) == m_desc->m_IDtoControls.end() || !m_desc->m_IDtoControls[winid] ) {
        Destroy();
        return;
    }

    wxWindow* win = m_desc->m_IDtoControls[winid];

    win->Hide();
    win->GetParent()->GetParent()->SetFocus();

    RTDataViewerBase * vwb = 0;
    TableEditorPanel* tep = 0;
    STDataViewerBase* swb = 0;

    if(vwb = dynamic_cast<RTDataViewerBase*> (win)){

        vwb->UnRegisterViewer();
    }

    if(swb = dynamic_cast<STDataViewerBase*> (win)){

        swb->unregisterViewer();

    }

    if(tep = dynamic_cast<TableEditorPanel*> (win)){

        wxWindow* t = tep->GetTable();

        VemsControlBase* vcb = dynamic_cast<VemsControlBase*>(t);

        if(vcb) vcb->b_UnRegisterViewer();
    }

    //delete win;

    if(eraseHash)
        m_desc->m_IDtoControls.erase(winid);
}

void GaugeCarrierAuiMDIChildFrame::SetEditMode(bool isedit)
{
    Refresh(false);

    InfoPanel * panel = InfoPanelHelper::GetPanel(HKEYPANELNAME);

    if( isedit && !panel ) {

        panel = new InfoPanel(GetParent(),GetHotKeyStr());
        panel->SetAlpha(180);
        panel->Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(GaugeCarrierAuiMDIChildFrame::OnInfoPanelClose), NULL, this );
        panel->SetSize(270,400);

        InfoPanelHelper::SetPanel(panel,HKEYPANELNAME);
    }

    if( panel ) {
        panel->Show(isedit);
        panel->Move( GetScreenPosition().x + GetSize().x - panel->GetSize().x, GetScreenPosition().y );
    }

    if(!isedit)
        DropDeselectWidgetCommand();
}

void GaugeCarrierAuiMDIChildFrame::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    if( m_groupMgr->IsEditMode() ) {

        wxColor cf(230,230,230);
        wxColor cb(128,128,128);

        dc.SetBackground( wxBrush(cb) );
        dc.SetBrush( wxBrush(cf) );
        dc.SetPen( wxPen(cf) );
        dc.Clear();

        int w = GetClientSize().x;
        int h = GetClientSize().y;

        int s = 10;

        for( int y=0; y<h; y+=s )
            for( int x=( (y%(2*s)) ? 0 : s ); x<w; x+=(2*s) )
                dc.DrawRectangle( x,y,s,s );

#if USE_ADVANCED_SELECTION_MODE
        if(m_selectBoxStart != wxDefaultPosition) {
            wxPoint pos = wxGetMousePosition() - GetScreenPosition();
            dc.SetBrush( *wxTRANSPARENT_BRUSH );
            wxPen pen( wxColor(255,0,0),2, wxLONG_DASH );
            dc.SetPen( pen );
            wxPoint d = pos - m_selectBoxStart;
            wxSize size(d.x,d.y);
            dc.DrawRectangle(m_selectBoxStart, size);
        }
#endif

    } else {
        dc.SetBackground( *wxBLACK_BRUSH );
        dc.Clear();
    }

//    m_handler->ResetDrawnFlag();

}

wxString GaugeCarrierAuiMDIChildFrame::GetTheme()
{
    return wxEmptyString;
}

void GaugeCarrierAuiMDIChildFrame::connectKeyDownEvent( wxWindow* window )
{
    //Logger::logf( LOG_MSG, _T("GaugeCarrierAuiMDIChildFrame::connectKeyDownEvent") );
    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_DOWN,
                         wxKeyEventHandler( GaugeCarrierAuiMDIChildFrame::OnChar ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyDownEvent( child );

            node = node->GetNext();
        }
    }
}


void GaugeCarrierAuiMDIChildFrame::OnChar( wxKeyEvent& event )
{
    int keyCode = event.GetKeyCode();

//    Logger::logf( LOG_MSG, _T("GaugeCarrierAuiMDIChildFrame::OnChar() - %c"), keyCode );

    switch( keyCode )
    {
        case WXK_ESCAPE :
        {
            GetParent()->SetFocus();
        }break;

        case 'R' :
        case 'W' :
        case 'B' :
            if( event.AltDown() ) HandleVemsControlBaseKeys(keyCode);
            else event.Skip();
            break;

//        case 'G' :
//        {
//            if ( m_control ) { m_control->b_showTable3D(); }
//        } break;
/*
        case 'T' :
        {
            wxString name = ::wxGetTextFromUser( _T(""),
                                                _T("Change table!"),
                                                _T(""),
                                                this,
                                                wxDefaultCoord,
                                                wxDefaultCoord,
                                                true );
            if ( m_table ) { m_table->change( name ); }
        } break;
*/
        default :
        {
            event.Skip();
        }
    }
}

void GaugeCarrierAuiMDIChildFrame::HandleVemsControlBaseKeys( int keyCode )
{
    for(wxWindowHashInt::iterator it = m_desc->m_IDtoControls.begin(); it != m_desc->m_IDtoControls.end(); it++ ){

        wxWindow* win = it->second;

        TableEditorPanel* tep = dynamic_cast<TableEditorPanel*>(win);

        if(!tep) continue;

        wxWindow* t = tep->GetTable();

        VemsControlBase* vcb = dynamic_cast<VemsControlBase*>(t);

        if(!vcb) continue;

        switch( keyCode )
        {
            case 'R' :
                vcb->b_refreshData();
                break;

            case 'W' :
                vcb->b_writeData();
                break;

            case 'B' :
                vcb->b_burnData();
                break;
        }
    }
}


wxString GaugeCarrierAuiMDIChildFrame::GetHotKeyStr()
{
LOG
    wxString content = FileHelper::GetFileContents( DirHelper::GetPath(dCONFIG)+_T("editmodehelp.html"));
LOG
    return content;

//    wxString str = _T("<center><b>Hotkeys in edit mode<table>");
//    str += _T("<tr><td nowrap>Move widget</td><td nowrap>Shift + Arrows</td></tr>");
//    str += _T("<tr><td nowrap>Move widget fast</td><td nowrap>Alt + Arrows</td></tr>");
//    str += _T("<tr><td nowrap>Resize widget</td><td nowrap>Shift + Ctrl + Arrows</td></tr>");
//    str += _T("<tr><td nowrap>Resize widget fast</td><td nowrap>Shift + Alt + Arrows</td></tr>");
//    //str += _T("<tr><td>Reset gauge</td><td>F11</td></tr>");
//    //str += _T("<tr><td>Reset all gauge on current group</td><td>F12</td></tr>");
//    str += _T("</table>");
//    str += _T("<font size=+1>teve</fotn>");
//    str += _T("</b></center>");
//    return str;
}

void GaugeCarrierAuiMDIChildFrame::OnInfoPanelClose(wxCloseEvent& evt)
{
    InfoPanelHelper::SetPanel(NULL,HKEYPANELNAME);
    evt.Skip();
}

void GaugeCarrierAuiMDIChildFrame::OnClose( wxCloseEvent& event )
{
    m_groupMgr->hideGroup( m_groupName );
}
