#include "GaugeCarrierFrame.h"



#include "../WindowReg.h"
#include "../ConfigBase.h"
#include "../configmanager/ConfigHelper2.h"

#include "../VemsTaskBarIcon.h"

#include "../vemstoolicon/VemsToolIconBase.h"
#include "VemsControlBase.h"

#include "SelectableWidgetBase.h"
#include "../ThemesHelper.h"
#include "GroupMember.h"

#include "../logger.h"
#if defined(__WINCE_STANDARDSDK__)
	#include <aygshell.h>
#endif

#include "GaugeCarrierBaseDesc.h"

BEGIN_EVENT_TABLE(GaugeCarrierFrame, wxFrame)
    EVT_LEFT_DCLICK(GaugeCarrierFrame::OnDoubleClick)
    EVT_LEFT_DOWN(GaugeCarrierFrame::OnLeftDown)
    EVT_LEFT_UP(GaugeCarrierFrame::OnLeftDown)
    EVT_CLOSE(GaugeCarrierFrame::OnFrameClose)
    EVT_ACTIVATE(GaugeCarrierFrame::OnActivate)
    EVT_KEY_DOWN(GaugeCarrierFrame::OnChar)

END_EVENT_TABLE()

GaugeCarrierFrame::GaugeCarrierFrame(wxWindow* parent, wxWindowID id,const wxString& title, const wxPoint& pos, const wxSize& size, long style)
    :GaugeCarrierBase (parent)
{
    //ctor
    InitWindow( ((wxFrame*)this) );

#if defined(__WINCE_STANDARDSDK__)
    Create(parent,id,title,pos,size,style);
#else
    #if defined(__WXMSW__)
        //Create(parent,id,title,pos,size,style|wxFRAME_SHAPED | wxFRAME_TOOL_WINDOW  | wxFRAME_FLOAT_ON_PARENT );
        Create(parent,id,title,pos,size,style);
    #else
        Create(parent,id,title,pos,size,style);
    #endif
#endif

    SetBackgroundColour(wxColour(0,0,0));

    m_desc->m_hasShape = true;

    ConfigBase* cfgBase = ConfigHelper2::getConfig();
    cfgBase->setPath(_T("/"));
    long tmp;
    cfgBase->read(_T("graphics_quality_factor"), &tmp, 0L);
    m_desc->m_graphQual = tmp;
    m_desc->m_type = -1;
//    m_orientation = wxVERTICAL;

    SetIcon(wxIcon(_T("vemsLogoIcon2")));
    IsUnregViewers = true;
}

GaugeCarrierFrame::~GaugeCarrierFrame()
{
    //dtor
    if(IsUnregViewers)
        unregisterViewer();
}

void GaugeCarrierFrame::ProcessWidgetCommand(wxVariantHash& cmdHash)
{
    if(!m_control) return;

    SelectableWidgetBase* selwb = dynamic_cast<SelectableWidgetBase*>(m_control);

    if(!selwb) return;

    selwb->ProcessWidgetCommand(cmdHash);
}

void GaugeCarrierFrame::SetFocusToItem(int id)
{
    if( id != -1) return;

    if(m_control)
        m_control->b_SetFocus();

    else
        m_desc->m_window->SetFocus();

}

void GaugeCarrierFrame::ShowCarrier(bool isShow)
{
    Show(isShow);
}

void GaugeCarrierFrame::OnActivate(wxActivateEvent& evt)
{
    if( m_control ) { m_control->b_SetFocus(); }

    if(m_TaskBarIconApp == NULL) return;
    if(m_desc->m_wreg == NULL) return;

   	Logger::logf(LOG_MSG,_T("GaugeCarrierFrame::OnActivate  evt.GetActive()=%i"),evt.GetActive());

	bool tmp = m_TaskBarIconApp->IsRestored();

	Logger::logf(LOG_MSG,_T("GaugeCarrierFrame::OnActivate  m_TaskBarIconApp->IsRestored()=%i"),tmp);

    if(evt.GetActive() && m_TaskBarIconApp->IsRestored() ) {

        m_TaskBarIconApp->SetRestored(false);
        m_desc->m_wreg->Maximize();

		Show(true);
    }


#if defined (__WINCE_STANDARDSDK__)
	SHFullScreen( (HWND)(GetHandle()) , SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);
#endif
    //evt.Skip();

	Logger::logf(LOG_MSG,_T("GaugeCarrierFrame::OnActivate END "));

}

void GaugeCarrierFrame::OnLeftDown(wxMouseEvent& evt)
{
#if !defined (__WINCE_STANDARDSDK__)

    wxPoint mpoint = evt.GetPosition();

    if( m_desc->m_IDtoControls.find(evt.GetId()) != m_desc->m_IDtoControls.end() )
        mpoint += m_desc->m_IDtoControls[evt.GetId()]->GetPosition();

    Logger::logf(LOG_MSG,_T("GaugeCarrierFrame::OnLeftDown(x=%i, y=%i)"),mpoint.x,mpoint.y);

    for(wxWindowHashInt::iterator iti = m_desc->m_IDtoControls.begin(); iti != m_desc->m_IDtoControls.end(); iti++) {

        if(iti->second->GetName() != _T("VemsToolIconBase")) continue;

        VemsToolIconBase *icon = (VemsToolIconBase*)(iti->second);
        wxRegion iregtmp = icon->GetShape();
        iregtmp.Offset(icon->GetPosition());

        if( iregtmp.Contains(mpoint) == wxInRegion ) {
            Logger::logf(LOG_MSG,_T("match!"));
            icon->ProcessEvent(evt);
            return;
        }

    }
    evt.Skip();

#endif
}



wxString GaugeCarrierFrame::GetTheme()
{
    if(m_control){
        ThemeAble* w = dynamic_cast<ThemeAble*>(m_control);
        if(w)
            return w->GetThemeName();
    }

        return ThemesHelper::m_defaultThemeName;
};


void GaugeCarrierFrame::OnFrameClose(wxCloseEvent& event)
{
    Logger::logf(LOG_MSG,_T("GaugeCarrierFrame::OnFrameClose"));

    GroupMember * gm = NULL;
    if(m_control) gm = dynamic_cast<GroupMember*>(m_control);
    //else if(m_table) gm = dynamic_cast<GroupMember*>(m_table);

    if(!event.CanVeto() || (gm && gm->GetItemID() == -1)) {
        Destroy();
    }

    wxCommandEvent evt(wxEVT_COMMAND_ENTER, 42);
    AddPendingEvent(evt);
}

void GaugeCarrierFrame::connectKeyDownEvent( wxWindow* window )
{
    //Logger::logf( LOG_MSG, _T("GaugeFrameBase::connectKeyDownEvent") );
    if( window )
    {
        window->Connect( wxID_ANY,
                         wxEVT_KEY_DOWN,
                         wxKeyEventHandler( GaugeCarrierFrame::OnChar ),
                         (wxObject*) NULL,
                         this );

        wxWindowListNode* node = window->GetChildren().GetFirst();

        while( node )
        {
            wxWindow* child = node->GetData();
            this->connectKeyDownEvent( child );

            node = node->GetNext();
        }
    }
}



void GaugeCarrierFrame::OnChar( wxKeyEvent& event )
{
    int keyCode = event.GetKeyCode();

    // Logger::logf( LOG_MSG, _T("GaugeCarrierFrame::OnChar() - %c"), keyCode );

    switch( keyCode )
    {
        case WXK_ESCAPE :
        {
            GetParent()->SetFocus();
        }break;
        case 'R' :
        {
            if( event.AltDown() && m_control ) { m_control->b_refreshData(); }
            else { event.Skip(); }//Logger::logf( LOG_MSG, _T("GaugeCarrierFrame::OnChar() - %c"), keyCode ); }

        }break;
        case 'W' :
        {
            if( event.AltDown() && m_control ) { m_control->b_writeData(); }
            else { event.Skip(); }
        }break;
        case 'B' :
        {
            if( event.AltDown() && m_control ) { m_control->b_burnData(); }
            else { event.Skip(); }
        }break;
        case 'G' :
        {
            if ( m_control ) { m_control->b_showTable3D(); }
        } break;
/*
        case 'T' :
        {
            wxString name = ::wxGetTextFromUser( _T(""),
                                                _T("Change table!"),
                                                _T(""),
                                                this,
                                                wxDefaultCoord,
                                                wxDefaultCoord,
                                                true );
            if ( m_table ) { m_table->change( name ); }
        } break;
    */
        default :
        {
            event.Skip();
        }
    }
}


void GaugeCarrierFrame::OnDoubleClick(wxMouseEvent& WXUNUSED(evt))
{
    Logger::logf(LOG_MSG,_T("GaugeCarrierFrame::OnDoubleClick() START"));

    if(!m_desc->m_hasShape) return;

    //region.ConvertToBitmap().SaveFile(_T("region.bmp"),wxBITMAP_TYPE_BMP,0);

    if(m_desc->m_onlyOneGauge ) {

        if( m_control )
        {
            if( m_control->HasShape() )
            {
                wxSize size = GetClientSize();
                m_control->b_SetSize(size);
                //gauge->setSize(size.GetWidth(),size.GetWidth() );
                wxRegion region;
                region.Clear();
                SetShape(region);
                m_control->SetShape(false);
                SetWindowStyleFlag(GetWindowStyle()|wxRESIZE_BORDER);
            }
            else
            {
                wxSize size = GetClientSize();
                m_control->b_SetSize(size);
                //gauge->setSize(size.GetWidth(),size.GetWidth() );
                wxRegion region = m_control->GetShape();
                m_control->SetShape(SetShape(region));
                SetWindowStyleFlag(GetWindowStyle()&~wxRESIZE_BORDER);
            }
        }

    }
    else {

        wxRegion tempreg;

        if(m_desc->m_hasShape){
            tempreg = m_desc->m_region;
            m_desc->m_hasShape = false;
        } else {
            m_desc->m_hasShape = true;
        }

        SetShape(tempreg);

    }
}

void GaugeCarrierFrame::ResetMinMax()
{
    if(!m_control) return;

    m_control->ResetMinMax();
};
