#include "GaugeGroupEditor.h"

#include <wx/filename.h>
#include <wx/gbsizer.h>
#include <wx/statline.h>

#include "GaugeGroupManager.h"
#include "../wxTextCtrlHotKey.h"
#include "../UserLog.h"
#include "../ThemesHelper.h"
#include "../logger.h"
#include "../wxSqliteHelper.h"

#include "../customwidgets/wxCheckBoxEx.h"
#include "../customwidgets/wxTextCtrlNullEx.h"
#include "../customwidgets/wxComboBoxNullEx.h"

#include "../events/EditorValueChangedEvent.h"

BEGIN_EVENT_TABLE(GaugeGroupEditor, wxFrame)

    EVT_COMBOBOX(idGGCombo, GaugeGroupEditor::OnGaugeGroup)

    EVT_BUTTON(idButtSave, GaugeGroupEditor::OnButton)
    EVT_BUTTON(idButtClose, GaugeGroupEditor::OnButton)
    EVT_BUTTON(idButtShowGrp, GaugeGroupEditor::OnButton)

    EVT_BUTTON(idButtDelGrp, GaugeGroupEditor::OnButton)
    EVT_BUTTON(idButtDuplicateGrp, GaugeGroupEditor::OnButton)


END_EVENT_TABLE()

GaugeGroupEditor::GaugeGroupEditor(wxWindow * parent, GaugeGroupManager * ggmgr)
{
    //ctor
    Create(parent, wxID_ANY, _T("GaugeGroups Editor"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE | wxSTAY_ON_TOP);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    m_ggMgr = ggmgr;


    Connect( EditorValueChangedEvent, wxCommandEventHandler(GaugeGroupEditor::OnChange)  );
}

GaugeGroupEditor::~GaugeGroupEditor()
{
    //dtor
}

void GaugeGroupEditor::GetGroupList( wxArrayString &arr )
{
    wxSQLite3ResultSet res = m_db.ExecuteQuery(_T("select name from groups order by name asc;"));

    while( res.NextRow() ){
        arr.Add( res.GetString(0) );
    }
}

void GaugeGroupEditor::Init()
{
    wxString filename = m_ggMgr->GetGroupsDBFileName();

    if(!wxSqliteHelper::Exist(filename)) return;

    m_db.Open( filename );

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer * sbs = new wxStaticBoxSizer(wxVERTICAL, this, _T(" Gauge groups: "));

    wxBoxSizer * gs = new wxBoxSizer(wxHORIZONTAL);
        wxArrayString arr;
        GetGroupList( arr );

        m_gaugeGroups = new wxComboBox(this, idGGCombo, _T(""), wxDefaultPosition, wxDefaultSize, arr, wxCB_READONLY | wxCB_SORT);
        m_gaugeGroups->Select(0);

        wxButton * butt = new wxButton(this,idButtShowGrp,_T("Show"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
        gs->Add( m_gaugeGroups, 1, wxALIGN_CENTER_VERTICAL );
        gs->Add( butt,0,wxLEFT, 5 );
        m_newGgrpCtrl = new wxTextCtrl(this, wxID_ANY,_T(""));
    sbs->Add( gs, 0, wxEXPAND|wxALL, 5 );

    wxBoxSizer * gs2 = new wxBoxSizer(wxHORIZONTAL);
        gs2->Add( m_newGgrpCtrl,1,wxALL|wxEXPAND,5);
        butt = new wxButton(this,idButtDuplicateGrp,_T("Duplicate"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
        gs2->Add( butt,0,wxALL&~wxLEFT ,5);

        gs2->Add( new wxStaticLine(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxLI_VERTICAL),0,wxALL|wxEXPAND,5);
        butt = new wxButton(this,idButtDelGrp,_T("Remove"),wxDefaultPosition,wxDefaultSize,wxBU_EXACTFIT);
        gs2->Add( butt,0,wxALL,5);
    sbs->Add( gs2, 0, wxEXPAND );


    mainsizer->Add( sbs, 0, wxEXPAND|wxALL, 5 );


    wxStaticBoxSizer * sbs2 = new wxStaticBoxSizer(wxVERTICAL, this, _T(" Group properties: "));

    wxGridBagSizer * gbs = new wxGridBagSizer();
    gbs->AddGrowableCol(1);

    m_hkeyCtrl = new wxTextCtrlHotKey(this, wxID_ANY, _T(""));
    m_hkeyCtrl->registerObserver(this);
    m_ggNameCtrl = new wxTextCtrlNullEx(this, wxID_ANY, _T(""),_T(""), wxDefaultPosition, wxSize(150,-1));
    m_ggShortNameCtrl = new wxTextCtrlNullEx(this, wxID_ANY, _T(""),_T(""), wxDefaultPosition, wxSize(150,-1));

    wxArrayString tarr;
    tarr.Add(_T(""));
    ThemesHelper::GetThemeNameList( tarr );

    m_themeList = new wxComboBoxNullEx(this, wxID_ANY, _T(""),_T(" "), wxDefaultPosition, wxDefaultSize, tarr, wxCB_READONLY | wxCB_SORT);

    m_checks[_T("sticked")] = new wxCheckBoxEx(this, wxID_ANY, _T("Sticked"));
    m_checks[_T("show_on_startup")] = new wxCheckBoxEx(this, wxID_ANY, _T("Show on startup when loaded log file"));
    m_checks[_T("show_on_startup_offline_cfg")] = new wxCheckBoxEx(this, wxID_ANY, _T("Show on startup when loaded config file"));
    m_checks[_T("show_on_startup_online")] = new wxCheckBoxEx(this, wxID_ANY, _T("Show on startup when ECU is online"));
    m_checks[_T("floated")] = new wxCheckBoxEx(this, wxID_ANY, _T("Floating"));


    m_hotkeyFeedBack = new wxStaticText(this,-1,_T(""), wxDefaultPosition, wxDefaultSize, wxBORDER_SIMPLE | wxALIGN_CENTRE | wxST_NO_AUTORESIZE);
    m_hotkeyFeedBack->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNHIGHLIGHT));
    m_hotkeyFeedBack->ClearBackground();

    int line = 0;

    gbs->Add( new wxStaticText(this, wxID_ANY, _T("Name: ")), wxGBPosition(line,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gbs->Add( m_ggNameCtrl, wxGBPosition(line,1), wxDefaultSpan, wxALL | wxEXPAND, 5);
    line++;

    gbs->Add( new wxStaticText(this, wxID_ANY, _T("Short name: ")), wxGBPosition(line,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gbs->Add( m_ggShortNameCtrl, wxGBPosition(line,1), wxDefaultSpan, wxALL | wxEXPAND, 5);
    line++;

    gbs->Add( new wxStaticText(this, wxID_ANY, _T("Theme: ")), wxGBPosition(line,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gbs->Add( m_themeList, wxGBPosition(line,1), wxDefaultSpan, wxALL | wxEXPAND, 5);
    line++;

    gbs->Add( new wxStaticText(this, wxID_ANY, _T("Hotkey: ")), wxGBPosition(line,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gbs->Add( m_hkeyCtrl, wxGBPosition(line,1), wxDefaultSpan, wxALL | wxEXPAND, 5);
    line++;

    gbs->Add( new wxStaticText(this, wxID_ANY, _T("Currently\nassigned to: ")), wxGBPosition(line,0), wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gbs->Add( m_hotkeyFeedBack, wxGBPosition(line,1), wxDefaultSpan, wxALL | wxEXPAND, 5);
    line++;


    gbs->Add( m_checks[_T("sticked")], wxGBPosition(line,0), wxGBSpan(1,2), wxALL , 5);line++;
    gbs->Add( m_checks[_T("show_on_startup")], wxGBPosition(line,0), wxGBSpan(1,2), wxALL , 5);line++;
    gbs->Add( m_checks[_T("show_on_startup_offline_cfg")], wxGBPosition(line,0), wxGBSpan(1,2), wxALL , 5);line++;
    gbs->Add( m_checks[_T("show_on_startup_online")], wxGBPosition(line,0), wxGBSpan(1,2), wxALL , 5);line++;
    gbs->Add( m_checks[_T("floated")], wxGBPosition(line,0), wxGBSpan(1,2), wxALL , 5);line++;

    m_savePropsButton = new wxButton(this, idButtSave, _T("Save properties"));

    gbs->Add( m_savePropsButton, wxGBPosition(line,0), wxGBSpan(1,2), wxALL | wxALIGN_CENTER_HORIZONTAL , 5);line++;

    sbs2->Add(gbs, 1, wxEXPAND);

    mainsizer->Add(sbs2, 0, wxEXPAND|wxALL, 5 );

    mainsizer->Add( new wxButton(this, idButtClose, _T("Close")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);

	SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);


    FillGroupProperitesBox();

    m_lastSelectedGroup = m_gaugeGroups->GetValue();
}

void GaugeGroupEditor::OnChange(wxCommandEvent& evt)
{
    m_savePropsButton->Enable(true);
}

bool GaugeGroupEditor::FillGroupProperitesBox()
{
    wxString value = m_gaugeGroups->GetValue();
    m_hkeyCtrl->SetDefaultValue(wxEmptyString);

    wxSQLite3ResultSet res = m_db.ExecuteQuery( wxString::Format(_T("select * from groups where name='%s';"), value.c_str()));

    if(res.Eof()) return false;

    m_ggNameCtrl->ChangeValueEx( res.GetString(_T("name")) );
    m_ggNameCtrl->ResetChange();

    m_ggShortNameCtrl->ChangeValueEx( res.GetString(_T("short_name")) );
    m_ggShortNameCtrl->ResetChange();

    m_themeList->SetStringSelection( res.GetString(_T("theme")) );

    wxString hk = res.GetString(_T("hotkey"));

    m_hkeyCtrl->SetValue(hk);

    for( wxCheckBoxHash::iterator it = m_checks.begin(); it != m_checks.end(); it++ ){
        it->second->SetValue( res.GetBool(it->first) );
        it->second->ResetChange(false);
    }

    m_hkeyCtrl->SetDefaultValue(hk);

    m_themeList->ResetChange();
    m_hkeyCtrl->ResetChange();

    m_savePropsButton->Enable(false);

    m_lastSelectedGroup = value;

    return true;
}

bool GaugeGroupEditor::SelectGroup(const wxString &name)
{
    bool res = m_gaugeGroups->SetStringSelection( name );

    if( res )
        FillGroupProperitesBox();

    return res;
}

void GaugeGroupEditor::OnGaugeGroup(wxCommandEvent &evt)
{
    //Logger::logf(LOG_MSG,_T("m_lastSelectedGroup=")+m_lastSelectedGroup);

    if(m_savePropsButton->IsEnabled()) {
        int res = wxMessageBox(_T("Do you want to save the changes?"),_T("Question"),wxYES_NO|wxICON_QUESTION);
        if(res == wxYES) SaveProperties(m_lastSelectedGroup);
    }

    FillGroupProperitesBox();
}

void GaugeGroupEditor::SaveProperties(const wxString& groupname)
{
    wxString qs = wxString::Format(_T("update groups set name='%s', hotkey='%s', theme='%s', short_name='%s' "),
                  m_ggNameCtrl->GetValue().c_str(), m_hkeyCtrl->GetValue().c_str(), m_themeList->GetValue().c_str(), m_ggShortNameCtrl->GetValue().c_str() );

    for( wxCheckBoxHash::iterator it = m_checks.begin(); it != m_checks.end(); it++ ){
        int val = it->second->GetValue();

        qs += wxString::Format( _T(", %s=%i"),it->first.c_str(),val);

    }

    wxString targetgrp = groupname == wxEmptyString ? m_gaugeGroups->GetStringSelection() : groupname;

    qs += wxString::Format(_T(" where name='%s';"), targetgrp.c_str()  );

    m_db.ExecuteUpdate( qs );

    if(groupname == wxEmptyString) {
    m_gaugeGroups->SetString( m_gaugeGroups->GetSelection(), m_ggNameCtrl->GetValue() );
    m_gaugeGroups->SetStringSelection( m_ggNameCtrl->GetValue() );
    } else {
        m_gaugeGroups->SetString( m_gaugeGroups->FindString(groupname) , m_ggNameCtrl->GetValue() );
    }

    m_ggMgr->reloadHotkeys();

    m_savePropsButton->Enable(false);
}

void GaugeGroupEditor::OnButton(wxCommandEvent &evt)
{
    switch( evt.GetId() ){
        case idButtSave:
            SaveProperties();
            FillGroupProperitesBox();
            break;

        case idButtClose:
            Close();
            break;

        case idButtShowGrp:
            m_ggMgr->showGroup( m_gaugeGroups->GetValue() );
            break;

        case idButtDelGrp:
            DelGroup();
            break;

        case idButtDuplicateGrp:
            DuplicateCurrentGroup();
            break;

    }
}

void GaugeGroupEditor::DuplicateCurrentGroup()
{
    if(m_savePropsButton->IsEnabled()) {
        int res = wxMessageBox(_T("Do you want to save the changes before duplicate?"),_T("Question"),wxYES_NO|wxCANCEL|wxICON_QUESTION);

        switch(res){
            case wxYES: SaveProperties(); break;
            case wxCANCEL: return;
        }

    }

    wxString grpstr = m_gaugeGroups->GetStringSelection();
    wxString ngrpstr = m_newGgrpCtrl->GetValue();

    if( ngrpstr.Len() < 1 ) {
        return;
    }

    wxSQLite3ResultSet ores = m_db.ExecuteQuery( wxString::Format(_T("select * from groups where name='%s';"),grpstr.c_str() ) );

    wxString otheme;
    int sticked=0,showons=0,showonsonline=0,floated=0,showonsoffcfg=0;

    if( ores.Eof() == false ) {
        otheme = ores.GetString(_T("theme"));
        sticked = ores.GetInt(_T("sticked"));
        showons = ores.GetInt(_T("show_on_startup"));
        showonsonline = ores.GetInt(_T("show_on_startup_online"));
        showonsoffcfg = ores.GetInt(_T("show_on_startup_offline_cfg"));
        floated = ores.GetInt(_T("floated"));
    }

    m_db.ExecuteUpdate( wxString::Format( _T("insert into groups (name,sticked,theme,show_on_startup,show_on_startup_online,floated,show_on_startup_offline_cfg) \
                            values('%s',%i,'%s',%i,%i,%i,%i);"), ngrpstr.c_str(), sticked, otheme.c_str(), showons, showonsonline, floated, showonsoffcfg ) );

    wxSQLite3Database db;
    db.Open(m_ggMgr->GetGaugesDBFileName());
    wxSQLite3ResultSet res = db.ExecuteQuery(wxString::Format( _T("select * from gauges where group_id=%i;"), ores.GetInt(_T("id")) ) );

    if( res.Eof() ) {
        //wxMessageBox(_T(""))
        //return;
    }

    int ngid = m_db.GetLastRowId().ToLong();

    db.Begin();

    while( res.NextRow() ){

        wxString qs = _T("insert into gauges (type,name,x,y,width,height,descriptor,theme,group_id) values (");

        qs += wxString::Format(_T("'%s',"), res.GetString(_T("type")).c_str() );
        qs += wxString::Format(_T("'%s',"), res.GetString(_T("name")).c_str() );
        qs += wxString::Format(_T("%i,"), res.GetInt(_T("x")) );
        qs += wxString::Format(_T("%i,"), res.GetInt(_T("y")) );
        qs += wxString::Format(_T("%i,"), res.GetInt(_T("width")) );
        qs += wxString::Format(_T("%i,"), res.GetInt(_T("height")) );
        qs += wxString::Format(_T("'%s',"), res.GetString(_T("descriptor")).c_str() );
        qs += wxString::Format(_T("'%s',"), res.GetString(_T("theme")).c_str() );
        qs += wxString::Format(_T("%i"), ngid );

        qs += _T(");");

        db.ExecuteUpdate( qs );

    }

    db.Commit();

    m_newGgrpCtrl->SetValue(_T(""));
    m_gaugeGroups->AppendString( ngrpstr );

    SelectGroup(ngrpstr);
}

void GaugeGroupEditor::DelGroup()
{
    int qres = wxMessageBox(_T("Dou you really want to delete the selected group?"),_T("Confirm"), wxYES_NO | wxICON_QUESTION);

    if(qres == wxNO) return;

    wxString grpstr = m_gaugeGroups->GetStringSelection();

    wxSQLite3ResultSet res = m_db.ExecuteQuery( wxString::Format( _T("select id from groups where name='%s';"),grpstr.c_str() ) );

    if( res.Eof() ) {
        //wxMessageBox(_T(""))
        return;
    }

    int gid = res.GetInt(0);


    m_ggMgr->hideGroup(grpstr);

    m_gaugeGroups->Delete( m_gaugeGroups->FindString(grpstr) );

    wxSQLite3Database db;
    db.Open(m_ggMgr->GetGaugesDBFileName());

    db.ExecuteUpdate( wxString::Format(_T("delete from gauges where group_id=%i;"),gid) );
    db.Close();

    m_db.ExecuteUpdate( wxString::Format(_T("delete from groups where id=%i;"),gid ) );

    m_gaugeGroups->SetSelection(0);
    FillGroupProperitesBox();

    m_ggMgr->reloadHotkeys();
}

void GaugeGroupEditor::notify( int i )
{
    if( i == 1 ) {

        wxString hk = m_hkeyCtrl->GetValue();

        wxString qs = wxString::Format( _T("select name from groups where hotkey='%s';"), hk.c_str() );

        wxSQLite3ResultSet res = m_db.ExecuteQuery( qs );

        m_hotkeyFeedBack->SetLabel( _T(" ") + res.GetString(0) + _T(" ") );

    } else if( i == 2 ) {

        m_hotkeyFeedBack->SetLabel(_T(""));
    }
}
