#include "GaugeGroupManagerXml.h"

#include "GaugeCarrierFrame.h"
#include "GaugeType.h"

#include "../confighelper.h"
#include "../common/VemsMessageHashHelper.h"
#include "../FrameLayoutHandler.h"
#include "../wxXmlHelper.h"

#include "../logger.h"
#include "../UserLog.h"

#include "../mtconfigdata.h"
#include "../DirHelper.h"
#include "../gaugedesc/GaugeDescHelper.h"

GaugeGroupManagerXml::GaugeGroupManagerXml(wxWindow* window, wxMenu* parent, MTConfigData* mtConfigData, RTDataMgrBase* rtDataMgrBase, STDataMgrBase* stDataMgrBase)
    :GaugeGroupManager( window, parent, mtConfigData, rtDataMgrBase, stDataMgrBase )
{
    //ctor
}

GaugeGroupManagerXml::~GaugeGroupManagerXml()
{
    //dtor
}

/** @brief  Get group node from Xml document by group name, if
  *         node not found return NULL
  */
wxXmlNode* GaugeGroupManagerXml::getGroupNode( const wxString& groupName )
{
    wxXmlDocument& doc = m_groupsXML;
     wxXmlNode* root = doc.GetRoot();
    if(!root) return NULL;
    if( root->GetName() != _T("GAUGE_GROUPS") ) { return NULL; }

    wxXmlNode* gaugeGroupNode = root->GetChildren();

    while( gaugeGroupNode )
    {
        if( gaugeGroupNode->GetName() == _T("GAUGE_GROUP") )
        {
            if( groupName != wxEmptyString
                && groupName == gaugeGroupNode->GetPropVal( _T("name"), wxEmptyString ) )
            {
                return gaugeGroupNode;
            }
        }

        gaugeGroupNode = gaugeGroupNode->GetNext();
    }

    return NULL;
}


/** @brief  Get group node from Xml document by group name, if
  *         node not found return NULL
  */
void GaugeGroupManagerXml::setGaugeNodeProperties( wxXmlNode* gaugeNode,
                                                const wxString& type,
                                                const wxString& name,
                                                int x,
                                                int y,
                                                int width,
                                                int height,
                                                const wxString& theme,
                                                const wxString& desc )
{
    wxXmlProperty* properties = gaugeNode->GetProperties();

    bool isHeightProp = false;

    while( properties )
    {
        wxString propName = properties->GetName();

        if( propName == _T("x") )
        {
            properties->SetValue( wxString::Format( _T("%i"), x ) );
        }
        else if( propName == _T("y") )
        {
            properties->SetValue( wxString::Format( _T("%i"), y ) );
        }
        else if( propName == _T("width") )
        {
            if( type == GaugeType::toString( GaugeType::ROUND ) ) { width += 8; }
            properties->SetValue( wxString::Format( _T("%i"), width ) );
        }
        else if( propName == _T("height") )
        {
            if( type == GaugeType::toString( GaugeType::ROUND ) ) { height += 8; }
            properties->SetValue( wxString::Format( _T("%i"), height ) );
            isHeightProp = true;
        }
        else if( propName == _T("name") )
        {
            properties->SetValue( name );
        }
        else if( propName == _T("theme") )
        {
            properties->SetValue( theme );
        }
        else if( propName == _T("desc") )
        {
            properties->SetValue( desc );
        }


        properties = properties->GetNext();
    }


    if( width != height && !isHeightProp)
        gaugeNode->AddProperty(_T("height"), wxString::Format( _T("%i"), height));

}


/** @brief Show group by name
  */
short GaugeGroupManagerXml::showGroup( const wxString& groupName )
{
    if( !m_mtConfigData->isValid() ){
        wxString msg = _T("<b>Unable to show gauge group: ") + groupName + _T("! It has 2 reasons:</b><br></font><font>");
        msg +=_T("<ul><li>ECU not connected and project is not selected!</li>");
        msg +=_T("<li>ECU connected but the firmware is older than 1.0.73!</li></ul>");
        UserLog::uLog( ULOG_WARNING, msg );
       // wxMessageBox(msg);
        return -2;
    }

    if( getGaugeGroupsBehav() == GGB_SINGLE && !m_stickedGroups[m_lastShowedGroup] && m_lastShowedGroup != wxEmptyString ) {
        short hr = hideGroup( m_lastShowedGroup );

        if( hr == 1 )
            m_menu->Check(m_menu->FindItem( m_lastShowedGroup ), false);

        if( m_lastShowedGroup == groupName && hr == 1 ){
            m_lastShowedGroup = wxEmptyString;
            m_lastShowedGroup = wxEmptyString;
            return -1;
        }
    }

    if( m_frameGroups.find( groupName ) == m_frameGroups.end() ) {
        m_frameGroups[ groupName ]= new FrameArray();
    } else {
        hideGroup( groupName );
        return -1;
    }

    wxXmlNode* groupNode = getGroupNode( groupName );

    if( !groupNode ) { return -2; }


    wxString groupTheme = groupNode->GetPropVal(_T("theme"),m_mainThemeName);

    wxString hotkey = groupNode->GetPropVal(_T("hotKey"),_T(""));

    wxXmlNodeArray removedNodes;

    for( wxXmlNode* gaugeNode = groupNode->GetChildren(); gaugeNode; gaugeNode = gaugeNode->GetNext() )
    {
        wxString name = gaugeNode->GetName();

        if( name == _T("GAUGE") )
        {
            if( gaugeNode->GetPropVal( _T("added"), _T("false") ) ==  _T("true") )
            {
                removedNodes.Add( gaugeNode );
                continue;
            }

            GaugeProperties prop;

            prop.gaugeType = GaugeType::getType(gaugeNode->GetPropVal( _T("type"), _T("VemsGauge") ));
            prop.symbolName = gaugeNode->GetPropVal( _T("name"), wxEmptyString );
            prop.themeName = gaugeNode->GetPropVal( _T("theme"), groupTheme);
            prop.descName = gaugeNode->GetPropVal( _T("desc"), _T("defaultdesc") );

            gaugeNode->GetPropVal( _T("x"), wxEmptyString ).ToLong( &prop.x );
            gaugeNode->GetPropVal( _T("y"), wxEmptyString ).ToLong( &prop.y );
            gaugeNode->GetPropVal( _T("width"), wxEmptyString ).ToLong( &prop.width );
            gaugeNode->GetPropVal( _T("height"), _T("-1") ).ToLong( &prop.height );

/*
            wxString type = gaugeNode->GetPropVal( _T("type"), _T("VemsGauge") );
            wxString name = gaugeNode->GetPropVal( _T("name"), wxEmptyString );
            wxString theme = gaugeNode->GetPropVal( _T("theme"), groupTheme);
            wxString desc = gaugeNode->GetPropVal( _T("desc"), _T("defaultdesc") );
            gaugeNode->GetPropVal( _T("x"), wxEmptyString ).ToLong( &x );
            gaugeNode->GetPropVal( _T("y"), wxEmptyString ).ToLong( &y );
            gaugeNode->GetPropVal( _T("width"), wxEmptyString ).ToLong( &gWidth );
            gaugeNode->GetPropVal( _T("height"), _T("-1") ).ToLong( &gHeight );
*/
            if( prop.height == -1 )
                prop.height = prop.width;


            GaugeCarrierFrame* gFrame;
            int id = wxNewId();

            if( prop.gaugeType == GaugeType::TABLE ) {

                gFrame = new GaugeCarrierFrame( m_window, id, prop.symbolName, prop.getPos(), wxDefaultSize,
                                             wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN );

                gFrame->SetDataMgrObj(m_rtDataMgrBase);
                gFrame->SetDataMgrObj(m_stDataMgrBase);

                gFrame->OnlyOneGauge( true );
                gFrame->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOW ) );
                //GroupMember * gm = gFrame->AddVemsTable( prop.symbolName, wxPoint( 0, 0 ), -1, true, this, groupName ); // size, width not used for tables
                gFrame->AddVemsTable( GaugeDescHelper::GetGDMgr(), prop );
            }
            else
            {
                prop.extraParams[_T("showBorder")] = true;

                StringHash hash;
                hash[_T("symbol")] = prop.symbolName;

                if(m_mtConfigData->getSymbol( prop.symbolName ) == 0) {
                    UserLog::vLog( VemsMessageHashHelper::GetVemsMsg(_T("symbol_not_found_2"),&hash) );
                    continue;
                }


                gFrame = new GaugeCarrierFrame( m_window, id, GaugeType::toString(prop.gaugeType), prop.getPos(), prop.getSize(), 0 | wxFRAME_SHAPED | wxFRAME_NO_TASKBAR );
                gFrame->SetDataMgrObj(m_rtDataMgrBase);
                gFrame->SetDataMgrObj(m_stDataMgrBase);
                gFrame->OnlyOneGauge( true );
                //gFrame->AddGauge(gauge_id, m_mtConfigData->getVEMSGaugeDescArray(),ThemesHelper::GetWidgetThemes(),  gaugeType, name, wxPoint( 0, 0 ), size.GetWidth(), size.GetHeight(), true, this, groupName,theme, desc, hotkey );
                gFrame->AddGauge( GaugeDescHelper::GetGDMgr(), prop );
            }

            gFrame->SetName( prop.symbolName );
            m_frameGroups[ groupName ]->Add( gFrame );
            m_frameToNode[ gFrame ] = gaugeNode;

            gFrame->Show();
        }
        else if( name == _T("MULTIGAUGE") )
        {
            wxString name = gaugeNode->GetPropVal( _T("name"), wxEmptyString );

            long x = -1;
            long y = -1;

            gaugeNode->GetPropVal( _T("x"), wxEmptyString ).ToLong( &x );
            gaugeNode->GetPropVal( _T("y"), wxEmptyString ).ToLong( &y );
            wxString orientation = gaugeNode->GetPropVal( _T("orientation"), wxEmptyString );

            wxXmlNode* multiGaugeNode = gaugeNode->GetChildren();

            GaugeCarrierFrame* gFrame = new GaugeCarrierFrame(    m_window, wxNewId(),
                                                //m_vemsTuneCtrl,
                                                name, wxPoint( x, y ), wxSize( 200, 200 ),
                                                0 | wxFRAME_SHAPED | wxFRAME_NO_TASKBAR );
                                                //wxMINIMIZE_BOX | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN );
            if( orientation == _T("horizontal") ) { gFrame->setOrientation( wxHORIZONTAL ); }
            gFrame->SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOW ) );
            gFrame->SetName( name );

            gFrame->SetDataMgrObj(m_rtDataMgrBase);
            gFrame->SetDataMgrObj(m_stDataMgrBase);

            while( multiGaugeNode )
            {
                wxString name = multiGaugeNode->GetName();

                if( name == _T("GAUGE") )
                {
                    wxString type = multiGaugeNode->GetPropVal( _T("type"), _T("VemsGauge") );

                    if( type == GaugeType::toString( GaugeType::FLAG ) )
                    {
                        wxString name = multiGaugeNode->GetPropVal( _T("name"), wxEmptyString );
                        gFrame->AddFlagWidget( name, this, groupName );
                    }
                }

                multiGaugeNode = multiGaugeNode->GetNext();
            }

            m_frameGroups[ groupName ]->Add( gFrame );
            m_frameToNode[ gFrame ] = gaugeNode;

            gFrame->SetInitialSize();
            gFrame->Show();
        }


    }//gaugeNode

    // remove unsaved nodes
    for( size_t i = 0; i < removedNodes.GetCount(); ++i ) { groupNode->RemoveChild( removedNodes[i] ); }
    WX_CLEAR_ARRAY( removedNodes );

    m_lastShowedGroup = groupName;

    if(m_framelh) m_framelh->SendObject( TN_GGEventShow, NULL, groupName );

    return 1;
}

void GaugeGroupManagerXml::SetTheme(int id)
{

    if( m_themeGrpHash.find(id) == m_themeGrpHash.end() ) return;

    wxXmlNode* gNode = getGroupNode( m_themeGrpHash[id]->Group );

    if( gNode->HasProp(_T("theme")) ) {

        wxXmlProperty *gprops = gNode->GetProperties();

        while( gprops ) {

            if( gprops->GetName() == _T("theme")) {
                gprops->SetValue( m_themeGrpHash[id]->Theme );
                break;
            }

            gprops = gprops->GetNext();
        }

    } else {

        gNode->AddProperty(_T("theme"),m_themeGrpHash[id]->Theme);

    }

    if( m_frameGroups.find(m_themeGrpHash[id]->Group) == m_frameGroups.end() ) return;

    int c = m_frameGroups[m_themeGrpHash[id]->Group]->GetCount();

    for(int a=0; a < c; a++){
        ThemeAble *w = m_frameGroups[m_themeGrpHash[id]->Group]->Item(a)->GetThemedObj();
        if(w)
            w->SetNewTheme(m_themeGrpHash[id]->Theme);
    }

    //Logger::logf(LOG_MSG,_T("%p"),m_themeGrpHash[id]);

}


void GaugeGroupManagerXml::showOnStartup()
{
    wxXmlDocument& doc = m_groupsXML;
    wxXmlNode* root = doc.GetRoot();
    if(!root) return;
    if( root->GetName() != _T("GAUGE_GROUPS") ) { return; }

    wxXmlNode* gaugeGroupNode = root->GetChildren();

    while( gaugeGroupNode )
    {
        if( gaugeGroupNode->GetName() == _T("GAUGE_GROUP") )
        {
            if( gaugeGroupNode->GetPropVal( _T("showOnStartup"), _T("false") ) == _T("true") )
            {
                wxString groupName = gaugeGroupNode->GetPropVal( _T("name"), wxEmptyString );
                showGroup( groupName );
                m_menu->Check( m_menu->FindItem( groupName ), true );
            }
        }

        gaugeGroupNode = gaugeGroupNode->GetNext();
    }
}


void GaugeGroupManagerXml::DoRemoveGauge(GaugeCarrierFrame* gaugeFrame)
{

    m_frameToNode[ gaugeFrame ]->AddProperty( _T("removed"), wxEmptyString );
    m_frameToNode.erase( gaugeFrame );

    Logger::logf(LOG_MSG,_T("GaugeGroupManagerXml::DoRemoveGauge"));
    gaugeFrame->Close();
}


/** @brief load
  *
  * @todo: document this function
  */
bool GaugeGroupManagerXml::load( const wxString& filename )
{
    wxXmlDocument& doc = m_groupsXML;

    m_dbfile = filename;

    if( !doc.Load( filename, _T("UTF-8"), wxXMLDOC_KEEP_WHITESPACE_NODES ) ) { return false; }

    wxXmlNode* root = doc.GetRoot();
    if(!root) return false;
    if( root->GetName() != _T("GAUGE_GROUPS" ) ) { return false; }

    m_mainThemeName = root->GetPropVal(_T("theme"), ThemesHelper::m_defaultThemeName);

    wxXmlNode* gaugeGroupNode = root->GetChildren();



    while( gaugeGroupNode )
    {
        if( gaugeGroupNode->GetName() == _T("GAUGE_GROUP") )
        {
            wxString groupName = gaugeGroupNode->GetPropVal(_T("name"),wxEmptyString);
            wxString hotKey = gaugeGroupNode->GetPropVal(_T("hotKey"),wxEmptyString);

            long ltmp;
            gaugeGroupNode->GetPropVal(_T("stick"),_T("0")).ToLong(&ltmp);
            m_stickedGroups[groupName] = ltmp;

            int id = wxNewId();
            m_menu->AppendCheckItem( id, groupName + _T("\t") + hotKey );
            Connect( id,  wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( GaugeGroupManager::OnGroup ) );

            wxString theme = gaugeGroupNode->GetPropVal(_T("theme"),m_mainThemeName);

            wxMenu *tmenu = ThemesHelper::CreateThemeMenu(ThemesHelper::GetWidgetThemes(), this, wxCommandEventHandler( GaugeGroupManager::OnTheme ),theme,groupName,&m_themeGrpHash);
            m_themeMenu->AppendSubMenu(tmenu,groupName);

        }

        gaugeGroupNode = gaugeGroupNode->GetNext();
    }//gaugeGroupNode*/



    m_menu->AppendSubMenu(m_themeMenu,_T("Group Theme"));
    return true;
}//load

void GaugeGroupManagerXml::setStickGroup(const wxString &groupname, bool isStick)
{
    m_stickedGroups[groupname] = isStick;
    //Logger::logf(LOG_MSG,groupname + _T(", %i"),m_stickedGroups[groupname]);
}


/** @brief save
  *
  * @todo: document this function
  */
bool GaugeGroupManagerXml::save( const wxString& filename )
{
    //save visible gauges properties
    FrameArrayHash::const_iterator it;
    for( it = m_frameGroups.begin(); it != m_frameGroups.end(); ++it )
    {
        FrameArray* fa = it->second;
        Logger::logf(LOG_MSG,it->first);

        wxString grptheme = getGroupTheme( it->first );

        uint32_t i;
        for( i = 0; i < fa->GetCount(); ++i )
        {
            GaugeCarrierFrame* gFrame  = fa->Item( i );
            wxXmlNode*      gNode    = m_frameToNode[ gFrame ];

            ThemeAble *th = gFrame->GetThemedObj();

            if( grptheme == gFrame->GetTheme() ) {
                gNode->DeleteProperty(_T("theme"));
                th = NULL;
            }

            if( th && th->isChanged() ) {
                if( !gNode->HasProp(_T("theme")) )
                   gNode->AddProperty(_T("theme"),gFrame->GetTheme());
            }


            if( !gFrame->isDefGaugeDesc() ) {

                if( !gNode->HasProp(_T("desc")) )
                   gNode->AddProperty(_T("desc"),gFrame->GetDesc());

            } else {
                gNode->DeleteProperty(_T("desc"));
            }

            setGaugeNodeProperties( gNode,
                                    GaugeType::toString( gFrame->getType() ),
                                    gFrame->GetName(),
                                    gFrame->GetPosition().x, gFrame->GetPosition().y,
                                    gFrame->GetSize().x,
                                    gFrame->GetSize().y,
                                    gFrame->GetTheme(),
                                    gFrame->GetDesc() );

        }
    }

    //remove marked gauges, ...
    wxXmlDocument& doc = m_groupsXML;
    wxXmlNode* root = doc.GetRoot();
    if(!root) return false;
    wxXmlNode* gaugeGroupNode = root->GetChildren();


    while( gaugeGroupNode )
    {
        bool stick = m_stickedGroups[ gaugeGroupNode->GetPropVal(_T("name"),_T("")) ];

        wxXmlHelper::SetPropVal(gaugeGroupNode, _T("stick"), wxString::Format(_T("%i"),stick), true );

        if( gaugeGroupNode->GetName() == _T("GAUGE_GROUP") )
        {
            wxXmlNode* gaugeNode = gaugeGroupNode->GetChildren();

            while( gaugeNode )
            {
                bool remove = false;
                wxXmlNode* tmpNode = NULL;

                if( gaugeNode->GetName() == _T("GAUGE") )
                {
                    wxXmlProperty*  properties = gaugeNode->GetProperties();

                    while( properties )
                    {
                        if( properties->GetName() == _T("removed") )
                        {
                            tmpNode = gaugeNode;
                            remove  = true;
                        }

                        properties = properties->GetNext();
                    }//properties
                }

                gaugeNode->DeleteProperty( _T("added") );
                gaugeNode = gaugeNode->GetNext();

                if( remove )
                {
                    gaugeGroupNode->RemoveChild( tmpNode );
                    delete tmpNode;
                }
            }//gaugeNode
        }//gaugeGroupNode

        gaugeGroupNode = gaugeGroupNode->GetNext();
    }

    //save to file
    wxString groupFile = ConfigHelper::getDeviceFile( _T("groupFile"), _T("gauges.xml") );

    if( !doc.Save( DirHelper::GetPath( dCONFIG ) + groupFile, wxXML_NO_INDENTATION ) )
    {
        wxLogError( _T("ItemGroupManager::save() - error") );
        return false;
    }

    return true;
}

void GaugeGroupManagerXml::DoAddGauge( const wxString &groupName, wxString gaugeType)
{


    if( groupName == wxEmptyString ) { return; }

    //set new gauge parameters
    wxString  name        = _T("rpm");
    if ( gaugeType.IsSameAs( _T("table") ) ) { name = _T("veTableTbl"); }
    else if ( gaugeType.IsSameAs( _T("table3d") ) ) { name = _T("veTableMap"); }

    const wxPoint   position    = wxPoint( 0,0 );
    const wxSize    size        = wxSize( 200, 200 );
    const int       id          = wxNewId();
    const wxString  theme       = ThemesHelper::m_defaultThemeName;
    const wxString  desc        = _T("defaultdesc");

    //create a new gauge
    GaugeCarrierFrame* gFrame = (GaugeCarrierFrame*)addGauge( id, groupName, gaugeType , name, position, size, theme , desc );

    //add gauge to node
    wxXmlNode* groupNode = getGroupNode( groupName );
    wxXmlNode* gaugeNode = new wxXmlNode( wxXML_ELEMENT_NODE, _T("GAUGE") );

    //handle table3d
    if ( gaugeType.IsSameAs( _T("table3d") ) ) { gaugeType= _T("table"); }

    //add properties
    if ( gaugeType != GaugeType::toString( GaugeType::ROUND ) )
    {
        gaugeNode->AddProperty( _T("type"), gaugeType );
    }
    gaugeNode->AddProperty( _T("name"), name );
    gaugeNode->AddProperty( _T("x"), wxString::Format( _T("%d"), position.x ) );
    gaugeNode->AddProperty( _T("y"), wxString::Format( _T("%d"), position.y ) );
    gaugeNode->AddProperty( _T("width"), wxString::Format( _T("%d"), size.x ) );
    gaugeNode->AddProperty( _T("height"), wxString::Format( _T("%d"), size.y ) );
    gaugeNode->AddProperty( _T("theme"), theme );
    gaugeNode->AddProperty( _T("desc"), desc );
    gaugeNode->AddProperty( _T("added"), _T("true") ); //mark new gauge

    groupNode->AddChild( gaugeNode );

    //
    m_frameToNode[ gFrame ] = gaugeNode;

    gFrame->Show();

}

