#include "MVG_DescEditor.h"

#include <wx/colordlg.h>
#include <wx/gbsizer.h>
#include <wx/artprov.h>

#include "../../gaugedesc/GaugeDescManager.h"
#include "../../DrawHelper.h"
#include "../../logger.h"
#include "../../customwidgets/wxTextCtrlNullEx.h"
#include "../../gaugedesc/VemsFlagDescManager.h"

#include "../../configmanager/ConfigMgrFrameLayout.h"
#include "../../common/FunctionMeter.h"
#include "../../wxVariantHelper.h"

#include "MultiValueGraph.h"
#include "MVG_ProfilesEditor.h"

#include "../../configmanager/ConfigHelper2.h"
#include "../../configmanager/ConfigHashHelper.h"


///temporarily...
#define USE_NEW_SYMBOL_SELECTOR 1

#if USE_NEW_SYMBOL_SELECTOR
    #include "MVG_SymbolChooserPanel.h"
#else
    #include "../../customwidgets/wxComboBoxNullEx.h"
#endif

#include "MVG_SymbolDescChooserPanel.h"

bool UseCustomtitleOnPaneltitle = false;

BEGIN_EVENT_TABLE(MVG_DescEditor,wxFrame)

    EVT_MENU_RANGE(idMenuSTART, idMenuEND, MVG_DescEditor::OnMenu)

    EVT_BUTTON(idBtnSave, MVG_DescEditor::OnButton)
    EVT_BUTTON(idBtnApply, MVG_DescEditor::OnButton)
    EVT_BUTTON(idBtnSaveAs, MVG_DescEditor::OnButton)
    EVT_BUTTON(idBtnRestore, MVG_DescEditor::OnButton)
    EVT_BUTTON(idBtnClose, MVG_DescEditor::OnButton)

    EVT_CLOSE(MVG_DescEditor::OnClose)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(MVG_DescEditor,wxFrame);

/**
    TODO:
        - Restore func is bad! // temp solution: function disabled
            - if there was appended, removed nodes, the Restore is bad. the solution maybe reopen the editor



*/

#define SETTINGS_STAY _T("stayontop")
#define SETTINGS_FLOAT _T("floatonparent")



MVG_DescEditor::MVG_DescEditor(wxWindow * parent, GaugeDescManager * gdmgr, int profileID)
{
    //ctor
    m_editorCaption = _T("Multi Graph Desc editor for profile: ");
    m_profileID = profileID;
    m_multiGraph = NULL;
    m_gdMgr = gdmgr;

    m_mgMgr = gdmgr->GetMultiGraphDescMgr();

    m_mgr = new MVG_DescManager(m_gdMgr, profileID);


    long flags = wxDEFAULT_FRAME_STYLE;

    m_settings[SETTINGS_FLOAT] = true;
    m_settings[SETTINGS_STAY] = false;
    m_settings[ SETTINGS_TOGGLE ] = m_mgr->Settings[ SETTINGS_TOGGLE ];

    ConfigHashHelper::GetValues(this,m_settings, ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT ));

    if(m_settings[ SETTINGS_FLOAT ].GetBool() == true) {
        flags |= wxFRAME_FLOAT_ON_PARENT;
    }

    if(m_settings[ SETTINGS_STAY ].GetBool() == true) {
        flags |= wxSTAY_ON_TOP;
    }

    m_mgr->Settings[ SETTINGS_TOGGLE ] = m_settings[ SETTINGS_TOGGLE ];



    Create(parent, wxID_ANY, m_editorCaption + m_mgMgr->GetMultiGraphDescProfileTitle(m_profileID), wxDefaultPosition, wxDefaultSize, flags );
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_wasRestore = false;

    m_graphDesc = m_mgMgr->GetMultiGraphDesc(m_profileID);


    m_graphDescBackup = new MultiGraphDesc(*m_graphDesc);

    Connect( EditorValueChangedEvent, wxCommandEventHandler(MVG_DescEditor::OnChange)  );

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    ConfigMgrFrameLayout cfgmgr(ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT ));
    cfgmgr.loadTo(this);

    //ConfigHelper::LoadFrameLayout(this);
}

MVG_DescEditor::~MVG_DescEditor()
{
    //dtor

    ConfigMgrFrameLayout cfgmgr(ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT ));
    cfgmgr.saveFrom(this);

    ConfigHashHelper::SaveValues(this,m_settings, ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT));

    delete m_graphDescBackup;
    delete m_mgr;
}

void MVG_DescEditor::OnClose(wxCloseEvent& evt)
{
    if(m_saveButton->IsEnabled()) {
        if(wxMessageBox(_T("Do you want to save the changes?"),_T("Warning"), wxYES_NO | wxICON_QUESTION ) != wxNO) {
            Save();
        }
    }

    evt.Skip();

    if(m_multiGraph) m_multiGraph->SetFocus();
}

void MVG_DescEditor::OnMenu(wxCommandEvent& evt)
{
    int id = evt.GetId();

    switch(id){

        case idMenuClose:
            Destroy();
            break;

        case idMenuToggle:{
            m_mgr->Settings[ SETTINGS_TOGGLE ] = evt.IsChecked();
            m_settings[ SETTINGS_TOGGLE ] = evt.IsChecked();
            }break;


        case idMenuFloat:
            m_settings[SETTINGS_FLOAT] = evt.IsChecked();
            break;

        case idMenuStay: {
            m_settings[SETTINGS_STAY] = evt.IsChecked();
            long flags = GetWindowStyleFlag();
            if(evt.IsChecked())
                flags |= wxSTAY_ON_TOP;
            else
                flags &= ~wxSTAY_ON_TOP;

            SetWindowStyle(flags);

            }break;

        case idMenuProfileTitle:{
            wxString ctitle = m_mgMgr->GetMultiGraphDescProfileTitle(m_profileID);
            wxString title = wxGetTextFromUser(_T("The current profile title:"),_T("Profile settings"), ctitle );
            if(title != wxEmptyString && title != ctitle) {
                m_mgMgr->SetMultiGraphDescProfileTitle(m_profileID, title);
                SetTitle( m_editorCaption + title );

                MVG_ProfilesEditor * pe = wxDynamicCast(GetParent(),MVG_ProfilesEditor);
                if(pe) pe->RefreshDescList();

            }
            }break;

        case idMenuAppendNode:{

            MultiGraphDescNode * node = m_mgr->AppendNode();
            if(node) {
                m_mgr->RefreshTargetGraph();
                ProcessChange( EVCE_STRUCT );
                InsertNode(node);
            }
            }break;

    }
}

void MVG_DescEditor::InitEditor(int node_id, int symbol_id)
{
    wxMenuBar * mbar = new wxMenuBar();


    MultiGraphDescNodeArray& nodearr = m_graphDesc->Cells;

    wxBoxSizer * sizer = new wxBoxSizer(wxVERTICAL);

    for(size_t a=0; a < nodearr.GetCount(); a++) {

        int cnodeid = nodearr[a].Properties[_T("id")].GetLong();

        if( node_id != 0 && node_id != cnodeid ) continue;

        int id = wxNewId();

        NodePanel * panel = new NodePanel(this, id, nodearr[a], m_mgr );

        panel->InitPanel(symbol_id);

        m_nodeList[ cnodeid ] = panel;

        sizer->Add(panel,0,wxALL|wxEXPAND, 1);

    }

    wxMenu * fmenu = new wxMenu();
    fmenu->Append(idMenuClose,_T("&Close editor"));


    wxMenu * smenu = new wxMenu();
    smenu->AppendCheckItem(idMenuToggle,_T("Toggle symbols"),_T("Toggle symbol panels"));
    smenu->AppendCheckItem(idMenuFloat,_T("Float on parent"));
    smenu->AppendCheckItem(idMenuStay,_T("Stay on top"));
    smenu->Check( idMenuStay, m_settings[SETTINGS_STAY].GetBool() );
    smenu->Check( idMenuFloat, m_settings[SETTINGS_FLOAT].GetBool() );
    smenu->Check( idMenuToggle, m_settings[SETTINGS_TOGGLE].GetBool());

    wxMenu * pmenu = new wxMenu();

    pmenu->Append(idMenuProfileTitle, _T("Set the current profile title") );
    pmenu->AppendSeparator();
    pmenu->Append(idMenuAppendNode, _T("&Append new empty node") );

    mbar->Append(fmenu,_T("&File"));
    mbar->Append(smenu,_T("&Settings"));
    mbar->Append(pmenu,_T("&Profile"));

    SetMenuBar(mbar);

    m_restoreButton = new wxButton(this, idBtnRestore, _T("Restore"));
    m_saveButton = new wxButton(this, idBtnSave, _T("Save"));
    m_applyButton = new wxButton(this, idBtnApply, _T("Apply"));
    wxButton * saveAsButton = new wxButton(this, idBtnSaveAs, _T("Save As"));

    m_restoreButton->Show(false);

    wxBoxSizer * bsizer = new wxBoxSizer(wxHORIZONTAL);

    bsizer->Add(m_applyButton,0, wxALL,5);
    bsizer->Add(m_saveButton,0, wxALL,5);
    bsizer->Add(saveAsButton,0, wxALL,5);
    bsizer->Add(m_restoreButton,0, wxALL,5);
    bsizer->Add(new wxButton(this, idBtnClose, _T("Close Editor")),0, wxALL,5);

    m_applyButton->SetToolTip(_T("Set symbol descriptor values"));
    m_saveButton->SetToolTip(_T("Save descriptor to the database"));
    saveAsButton->SetToolTip(_T("Save as current descriptor..."));
    m_restoreButton->SetToolTip(_T("Restore all modifications, refresh MultiGraph & Close Editor"));

    sizer->AddSpacer(20);


    sizer->Add(bsizer, 1,  wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL);

	SetSizer(sizer);
	sizer->Fit(this);
	sizer->SetSizeHints(this);

    m_saveButton->Enable(m_graphDesc->IsChanged);

	m_restoreButton->Enable(false);
	m_applyButton->Enable(false);
}

void MVG_DescEditor::SetGuiToModel()
{
    for( NodePanelHash::iterator it = m_nodeList.begin(); it != m_nodeList.end() ; it++ ){
        //int node_id = it->first;
        NodePanel * nodePanel = it->second;

        SymbolPanelHash& symbolPanelList = nodePanel->GetSymbolPanelHash();

        for( SymbolPanelHash::iterator nit = symbolPanelList.begin(); nit != symbolPanelList.end(); nit ++ ) {
            //int symbol_id = nit->first;
            SymbolPanel * symbolPanel = nit->second;
            GUIDescGUIItemArray& symbolDesc = symbolPanel->GetGUIDescGUIItemArray();

            wxVariantHash& mgDesc = (*symbolPanel->Desc);

            for(size_t a = 0; a < symbolDesc.GetCount(); a++) {
                GUIDescGUIItem& desc = symbolDesc[a];

                if(desc.IsChanged()) desc.ResetChange();

                if(desc.IsNull() && mgDesc.find(desc.Name) != mgDesc.end() ) {
                    mgDesc.erase(desc.Name);
                    continue;
                }

                wxVariant value = desc.GetValue();

                if( value == wxNullVariant ) continue;

                if(desc.Type == GDSI_SYMBOLNAME) {
                    wxString sname = value.GetString();
                    wxString title;
                    if(UseCustomtitleOnPaneltitle) {
                        if(mgDesc.find(_T("customtitle")) != mgDesc.end()) title = mgDesc[_T("customtitle")].GetString();
                    }

                    if(title == wxEmptyString) {
                        title = m_gdMgr->GetGaugeSymbolTitle(sname);
                        if(title == wxEmptyString) title = sname;
                    }
                    symbolPanel->SetPanelString( title );

                    wxString stype = GetSymbolType(sname);

                    if(stype == wxEmptyString)
                        mgDesc.erase(_T("type"));
                    else
                        mgDesc[_T("type")] = stype;
                }

                mgDesc[desc.Name] = value;
            }
        }
    }
}

wxString MVG_DescEditor::GetSymbolType(const wxString& name)
{
    VemsFlagDescHash& hash = m_gdMgr->GetVemsFlagDescMgr()->GetAllDesc();
    if(hash.find(name) != hash.end())
        return _T("flag");

    return wxEmptyString;
}

void MVG_DescEditor::RestoreDesc()
{

    m_mgMgr->SetMultiGraphDesc(m_graphDescBackup, m_profileID);

    m_mgr->RefreshTargetGraph();

    m_graphDesc = m_graphDescBackup;

    m_graphDescBackup = new MultiGraphDesc(*m_graphDesc);

    SetModelToGui();
}

void MVG_DescEditor::SetModelToGui()
{
    MultiGraphDescNodeArray& nodearr = m_graphDesc->Cells;

    for(size_t a=0; a < nodearr.GetCount(); a++) {

        int node_id = nodearr[a].Properties[_T("id")].GetLong();

        if( m_nodeList.find(node_id) == m_nodeList.end() ) continue;

        NodePanel * nodepanel = m_nodeList[node_id];

        if(nodepanel == NULL) continue;

        wxVariantHashArray& arr = nodearr[a].Symbols;

        SymbolPanelHash& symbolPanelList = nodepanel->GetSymbolPanelHash();

        for(size_t b=0; b < arr.GetCount(); b++ ) {

            wxVariantHash& hash = arr[b];

            int symbol_id = hash[_T("id")].GetLong();

            if( symbolPanelList.find( symbol_id ) == symbolPanelList.end() ) continue;

            SymbolPanel * symbolpanel = symbolPanelList[symbol_id];

            if( symbolpanel == NULL ) return;

            GUIDescGUIItemArray& syms = symbolpanel->GetGUIDescGUIItemArray();

            for(size_t c = 0; c < syms.GetCount(); c++ ) {

                GUIDescGUIItem& ssdesc = syms[c];

                wxString name = ssdesc.Name;

                wxVariant v = wxNullVariant;

                if( hash.find(name) != hash.end() ) v = hash[name];

                ssdesc.SetValue(v);

            }

        }

    }

}

void MVG_DescEditor::Apply()
{
    SetGuiToModel();
    m_mgr->RefreshTargetGraph();
    m_applyButton->Enable(false);
    m_restoreButton->Enable(true);
}

void MVG_DescEditor::Save()
{
    if(m_applyButton->IsEnabled())
        Apply();

    m_mgMgr->SaveMultiGraphDesc( m_profileID );
    m_saveButton->Enable(false);
    m_restoreButton->Enable(false);
    delete m_graphDescBackup;
    m_graphDescBackup = new MultiGraphDesc(*m_graphDesc);
}

void MVG_DescEditor::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idBtnApply:
            Apply();
            break;

        case idBtnSave:
            Save();
            break;

        case idBtnSaveAs:{
            wxString title = wxGetTextFromUser(_T("Save as descriptor to new profile"),_T("Profile title:"));
            if(title != wxEmptyString)
                m_mgMgr->SaveAsMultiGraphDesc( m_profileID, title );
            }break;

        case idBtnRestore:
            RestoreDesc();
            break;

        case idBtnClose:
            Close();
            break;
    }
}

void MVG_DescEditor::RemoveNodePanel(int id)
{
    m_nodeList.erase(id);
}

void MVG_DescEditor::InsertNode(MultiGraphDescNode * nodeDesc, int target, NodePanel * sPanel)
{
    int id = nodeDesc->Properties[_T("id")].GetLong();
    Logger::logf(LOG_MSG,_T("id=%i"),id);
    NodePanel * panel = new NodePanel(this, wxNewId(), *nodeDesc, m_mgr);
    panel->InitPanel();
    m_nodeList[ id ] = panel;


    bool isBefore;
    switch(target){
        case MVGD_INSERT_BEFORE: isBefore = true; break;
        case MVGD_INSERT_AFTER: isBefore = false; break;
        default: return;
    }


    wxSizer * sizer = GetSizer();
    if(!sizer) return;

    wxSizerItemList& list = sizer->GetChildren();

    if(sPanel) {
        wxSizerItemList::compatibility_iterator node = list.GetFirst();

        for(int idx = 0; node ; node = node->GetNext(), idx++ )
        {
            wxSizerItem *item = node->GetData();
            if(!item) continue;
            wxWindow * swin = item->GetWindow();
            if(!swin) continue;

            if(swin == sPanel) {
                sizer->Insert( (isBefore?idx:idx+1) ,panel,0,wxALL|wxEXPAND, 1);
                Logger::logf(LOG_MSG,_T("found!"));
                break;
            }
        }
    } else {
        sizer->Insert( list.GetCount()-2, panel,0,wxALL|wxEXPAND,1);
    }

    sizer->Layout();
    sizer->Fit(this);
    sizer->SetSizeHints(this);
    Refresh();

}

void MVG_DescEditor::ProcessChange(int type)
{
    Logger::logf(LOG_MSG,_T("MVG_DescEditor::ProcessChange type=%i"),type);

    if( type & EVCE_STRUCT ) {
        m_restoreButton->Enable(true);
    }

    if( type & EVCE_TEXT ) {
        m_applyButton->Enable(true);
    }

    m_saveButton->Enable(true);

    m_graphDesc->IsChanged = true;
}

void MVG_DescEditor::OnChange(wxCommandEvent& evt)
{
    ProcessChange(evt.GetInt());
}

/************************************************************************************************************************************************************
                     NodePanel
*/

BEGIN_EVENT_TABLE(NodePanel, wxPanel)

    EVT_RIGHT_DOWN(NodePanel::OnRightClick)
    EVT_MENU_RANGE(idPopupMenuPaste,idPopupMenuAppendSymbol, NodePanel::OnMenu)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(NodePanel,wxPanel);

NodePanel::NodePanel(wxWindow * parent, int id, MultiGraphDescNode& desc, MVG_DescManager* mgr)
{
    Create(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxBORDER_RAISED);
    Desc = &desc;
    m_nodeID = Desc->Properties[_T("id")].GetLong();
    m_nodeEditor = NULL;

    m_mgr = mgr;

    m_addMenu = new wxMenu();

    m_popupMenu = new wxMenu();
    m_popupMenu->Append(idPopupMenuPaste,_T("Paste symbol desc to new symbol"));
    m_addMenu->Append(idPopupMenuPaste,_T("Paste symbol desc to new symbol"));

    m_popupMenu->Append(idPopupMenuAppendSymbol,_T("Append an empty symbol panel"));
    m_addMenu->Append(idPopupMenuAppendSymbol,_T("Append an empty symbol panel"));

    wxMenu * insertmenu = new wxMenu();



    insertmenu->Append(idPopupMenuInsertAfter,_T("After the current node"));
    insertmenu->Append(idPopupMenuInsertBefore,_T("Before the current node"));

    m_popupMenu->AppendSeparator();
    m_addMenu->AppendSeparator();

    m_popupMenu->AppendSubMenu(insertmenu,_T("Insert new empty node..."));
    m_addMenu->AppendSubMenu(insertmenu,_T("Insert new empty node..."));

    m_popupMenu->Append(idPopupMenuRemove,_T("Remove node (and all symbols on this node)"));




    Connect( EditorValueChangedEvent, wxCommandEventHandler(NodePanel::OnChange)  );
}

NodePanel::~NodePanel()
{
    wxWindow * win = GetParent();
    MVG_DescEditor * editor = NULL;
    if(win) {
        editor = wxDynamicCast(win,MVG_DescEditor);
        if(editor) {
            editor->RemoveNodePanel( m_nodeID );
        }
    }
}

void NodePanel::OnButton(wxCommandEvent& evt)
{
    int id = evt.GetId();
    switch(id){
        case idButtonUp:
            if(m_mgr->SlideNodeTo( Desc, MVGD_SLIDE_UP )){
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                SlideTo(MVGD_SLIDE_UP);
            }
            break;

        case idButtonDown:
            if(m_mgr->SlideNodeTo( Desc, MVGD_SLIDE_DOWN )){
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                SlideTo(MVGD_SLIDE_DOWN);
            }
            break;

        case idButtonRemove:
            if(m_mgr->RemoveNode(Desc)){
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                RemoveThis();
            }
            break;

        case idButtonEdit:
            if(m_nodeEditor) m_nodeEditor->SetFocus();
            else {
                m_nodeEditor = new NodeEditPanel(this, Desc, m_mgr);
                m_nodeEditor->Show(true);
                m_nodeEditor->Connect(wxEVT_CLOSE_WINDOW, wxCloseEventHandler(NodePanel::OnCloseEditor), NULL, this );
            }
            break;

        case idButtonAdd:
            m_addMenu->Enable(idPopupMenuPaste,m_mgr->GetSymbolDescClipboard().size() < 1 ? false : true);
            PopupMenu(m_addMenu);
            break;
    }
}

/** \~english
  *
  * \~hungarian
  *
  */
void NodePanel::InitPanel(int symbol_id)
{
    //Logger::logf(LOG_MSG,_T("NodePanel::InitPanel()"));


    wxVariantHashArray& arr = Desc->Symbols;

    wxBoxSizer * sizer = new wxBoxSizer(wxHORIZONTAL);

    wxBoxSizer * navisizer = new wxBoxSizer(wxHORIZONTAL);

    wxBitmapButton * upb = new wxBitmapButton(this,  idButtonUp, wxArtProvider::GetBitmap(wxART_GO_UP) );
    wxBitmapButton * addb = new wxBitmapButton(this,  idButtonAdd, DrawHelper::GetBitmap(_T("add-16.png")) );
    wxBitmapButton * downb = new wxBitmapButton(this,  idButtonDown , wxArtProvider::GetBitmap(wxART_GO_DOWN) );
    wxBitmapButton * editb = new wxBitmapButton(this,  idButtonEdit , DrawHelper::GetBitmap(_T("edit-16.png")) );
    wxBitmapButton * removeb = new wxBitmapButton(this,  idButtonRemove, DrawHelper::GetBitmap(_T("remove-16.png")) );

    upb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton), NULL, this);
    addb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton), NULL, this);
    downb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton), NULL, this);
    editb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton), NULL, this);
    removeb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton), NULL, this);

    upb->SetToolTip(_T("Move node to up"));
    downb->SetToolTip(_T("Move node to down"));
    editb->SetToolTip(_T("Open Node edtior dialog"));
    removeb->SetToolTip(_T("Remove node (and all symbols on this node)"));

    navisizer->Add(upb, 0, wxALL | wxALIGN_CENTER_VERTICAL , 2 );
    navisizer->Add(downb, 0, wxALL | wxALIGN_CENTER_VERTICAL , 2 );
    navisizer->Add(editb, 0, wxALL | wxALIGN_CENTER_VERTICAL , 2 );
    //navisizer->Add(removeb, 0, wxALL | wxALIGN_CENTER_VERTICAL , 2 );

    sizer->Add(navisizer, 0, wxALL | wxEXPAND , 2 );

    for(size_t a=0; a < arr.GetCount(); a++ ) {

        int csymbolid = (arr[a][_T("id")]).GetLong();

        if( symbol_id != 0 && symbol_id != csymbolid ) continue;

        int id = wxNewId();

        SymbolPanel * panel = new SymbolPanel(this, id, arr[a], m_mgr );

        panel->InitPanel();

        m_symbolGuiList[ csymbolid ] = panel;

        sizer->Add(panel, 0, wxALL|wxALIGN_TOP, 2);

    }

    wxBoxSizer * t = new wxBoxSizer(wxHORIZONTAL);
    t->AddStretchSpacer(1);
    t->Add(addb, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 2 );
    t->Add(removeb, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT, 2 );
    sizer->Add(t, 1, wxALIGN_RIGHT | wxEXPAND );

	SetSizer(sizer);
	sizer->Fit(this);
	sizer->SetSizeHints(this);

}

void NodePanel::OnRightClick(wxMouseEvent& evt)
{
    bool validCB = m_mgr->GetSymbolDescClipboard().size() < 1 ? false : true;

    m_popupMenu->Enable(idPopupMenuPaste,validCB);

    PopupMenu(m_popupMenu);
}

void NodePanel::AddSymbol(wxVariantHash * hash)
{

    wxSizer * sizer = GetSizer();
    if(!sizer) return;

    SymbolPanel * panel = new SymbolPanel(this, wxNewId(), *hash, m_mgr );

    panel->InitPanel();
    m_symbolGuiList[ (*hash)[_T("id")].GetLong() ] = panel;

    wxSizerItemList& list = sizer->GetChildren();
    int c = list.GetCount();

    sizer->Insert(c-1,panel, 0, wxALL|wxALIGN_TOP, 2);

    sizer->Layout();
    sizer->Fit(this);
    sizer->SetSizeHints(this);
    Refresh();

    wxWindow * win = GetParent();
    if(!win) return;
    sizer = win->GetSizer();
    if(!sizer) return;
    sizer->Layout();
    sizer->Fit(win);
    sizer->SetSizeHints(win);
    win->Refresh();
}

void NodePanel::OnMenu(wxCommandEvent& evt)
{
    wxWindow * win = GetParent();
    MVG_DescEditor * editor = NULL;
    if(win) {
        editor = wxDynamicCast(win,MVG_DescEditor);
    }

    switch(evt.GetId()){
        case idPopupMenuPaste:{
            wxVariantHash * hash = m_mgr->AddSymbol(Desc->Properties[_T("id")].GetLong(), m_mgr->GetSymbolDescClipboard() );
            if(hash) {
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                AddSymbol(hash);
            }
            }break;

        case idPopupMenuInsertAfter:{
            MultiGraphDescNode * node = m_mgr->InsertNode(Desc,MVGD_INSERT_AFTER);
            if(node) {
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                if(editor) editor->InsertNode(node,MVGD_INSERT_AFTER,this);
            }
            }break;

        case idPopupMenuInsertBefore: {
            MultiGraphDescNode * node = m_mgr->InsertNode(Desc,MVGD_INSERT_BEFORE);
            if(node) {
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                if(editor) editor->InsertNode(node,MVGD_INSERT_BEFORE,this);
            }
            }break;

        case idPopupMenuRemove:
            if(m_mgr->RemoveNode(Desc)){
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                RemoveThis();
            }
            break;

        case idPopupMenuAppendSymbol:{
            wxVariantHash newhash;
            newhash[_T("name")] = m_mgr->GetUnusedSymbolname();
            newhash[_T("color")] << wxColor(0,0,0);
            wxVariantHash * hash = m_mgr->AddSymbol(Desc->Properties[_T("id")].GetLong(), newhash );
            if(hash) {
                m_mgr->RefreshTargetGraph();
                DropChangeEvent( EVCE_STRUCT );
                AddSymbol(hash);
            }
            }break;

    }
}


bool NodePanel::SlideTo(int target)
{
    bool isUp;
    switch(target){
        case MVGD_SLIDE_UP: isUp = true; break;
        case MVGD_SLIDE_DOWN: isUp = false; break;
        default: return false;
    }


    wxWindow * win = GetParent();
    if(!win) return false;
    wxSizer * sizer = win->GetSizer();
    if(!sizer) return false;

    wxSizerItemList& list = sizer->GetChildren();

    size_t count = list.GetCount();

    wxSizerItemList::compatibility_iterator node = list.GetFirst();

    int slidediff = isUp ? -1 : 1;
    int borderidx = isUp ? count - 3 : 0;
    int borderlim = isUp ? 0 : count - 3;

    for(int idx = 0; node ; node = node->GetNext(), idx++ )
    {
        wxSizerItem *item = node->GetData();
        if(!item) continue;
        wxWindow * swin = item->GetWindow();
        if(!swin) continue;

        if(swin == this) {
            wxSizerItem * newitem = new wxSizerItem(swin, item->GetProportion(), item->GetFlag(), item->GetBorder(), item->GetUserData() );
            sizer->Detach(idx);
            if(idx == borderlim)
                sizer->Insert( borderidx, newitem );
            else
                sizer->Insert( idx + slidediff, newitem );

            sizer->Layout();
            win->Refresh();
            return true;

        }
    }
    return false;
}

void NodePanel::RemoveThis()
{
    wxWindow * win = GetParent();
    if(!win) return;
    wxSizer * sizer = win->GetSizer();
    if(!sizer) return;


    sizer->Detach(this);
    sizer->Layout();

    if( m_mgr->Settings.find(_T("lastShowedNode")) != m_mgr->Settings.end() ){
        wxObject * obj = m_mgr->Settings[_T("lastShowedNode")].GetWxObjectPtr();
        NodePanel * np = wxDynamicCast(obj,NodePanel);
        if( np && np == this ) m_mgr->Settings.erase(_T("lastShowedNode"));
    }

    Destroy();

    sizer->Fit(win);
    sizer->SetSizeHints(win);
    win->Refresh();

}

void NodePanel::RemoveSymbolPanel(int id)
{
    m_symbolGuiList.erase(id);
}

void NodePanel::RefreshPanel()
{
    wxSizer * sizer = GetSizer();
    if(!sizer) return;

    sizer->Layout();
    sizer->Fit(this);
    sizer->SetSizeHints(this);
    Refresh();


    wxWindow * win = GetParent();
    if(!win) return;
    sizer = win->GetSizer();
    if(!sizer) return;

    sizer->Layout();
    sizer->Fit(win);
    sizer->SetSizeHints(win);
    win->Refresh();
}

void NodePanel::OnCloseEditor(wxCloseEvent& evt)
{
    m_nodeEditor = NULL;
    evt.Skip();
}

void NodePanel::OnChange(wxCommandEvent& evt)
{
    wxWindow * win = GetParent();
    if(win) win->AddPendingEvent(evt);
}

void NodePanel::DropChangeEvent(int t)
{
    wxWindow * win = GetParent();
    if(!win) return;

    wxCommandEvent event(EditorValueChangedEvent);
    event.SetInt(t);
    win->AddPendingEvent(event);
}

/************************************************************************************************************************************************************
                    SymbolPanel
*/

IMPLEMENT_DYNAMIC_CLASS(SymbolPanel,wxPanel);

SymbolPanel::SymbolPanel(wxWindow * parent, int id, wxVariantHash& desc, MVG_DescManager* mgr)
{
    Create(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxBORDER_NONE );
    Desc = &desc;

    m_symbolID = (*Desc)[_T("id")].GetLong();

    m_mgr = mgr;
    m_mainButtonStr = _T("");//_T("Show/hide ");
    m_panel = NULL;
    m_panelButton = NULL;

    Connect( EditorValueChangedEvent, wxCommandEventHandler(SymbolPanel::OnChange)  );
}

SymbolPanel::~SymbolPanel()
{
    wxWindow * win = GetParent();
    NodePanel * panel = NULL;
    if(win) {
        panel = wxDynamicCast(win,NodePanel);
        if(panel) {
            panel->RemoveSymbolPanel( m_symbolID );
        }
    }
}

void SymbolPanel::OnChange(wxCommandEvent& evt)
{
    wxWindow * win = GetParent();
    if(win) win->AddPendingEvent(evt);
}

void SymbolPanel::SetPanelString(const wxString& str)
{
    if(!m_panelButton) return;

    if( m_panelButton->GetLabel() == str ) return;

    m_panelButton->SetLabel(str);

    m_mainsizer->Layout();
    m_mainsizer->Fit(this);

    NodePanel * np = wxDynamicCast(GetParent(),NodePanel);
    if(np) np->RefreshPanel();
}

void SymbolPanel::ShowHide()
{
#if !USE_NEW_SYMBOL_SELECTOR
    for( size_t a = 0; a < m_settingsGui.GetCount(); a++ ) {
        GUIDescGUIItem desc = m_settingsGui[a];
        if(desc.Type == GDSI_SYMBOLNAME) {
            wxComboBox * box = wxDynamicCast(desc.Ctrl, wxComboBox);
            if(!box) break;
            if( box->GetCount() != 0 ) break;
            box->Append(m_mgr->GetSymbolList());
            box->SetStringSelection((*Desc)[_T("name")].GetString());
            break;
        }
    }
#endif
    wxSizerItem * citem = m_mainsizer->GetItem(1);

    m_mainsizer->Show(1,!m_mainsizer->IsShown(1));
    m_mainsizer->Layout();
    m_mainsizer->Fit(this);


    wxWindow * win = GetParent();
    NodePanel * panel = NULL;
    if(win) {
        panel = wxDynamicCast(win,NodePanel);
    }

    if( m_mgr->Settings[SETTINGS_TOGGLE].GetBool() ) { // close last showed symbol panel

        if(m_mgr->Settings.find(_T("lastShowedSymbolItem")) != m_mgr->Settings.end()) {
            wxObject * obj = m_mgr->Settings[_T("lastShowedSymbolItem")].GetWxObjectPtr();
            wxSizerItem * sitem = wxDynamicCast(obj,wxSizerItem);
            if(sitem && citem != sitem) sitem->Show(false);
        }

        if(m_mgr->Settings.find(_T("lastShowedNode")) != m_mgr->Settings.end()) { // refresh the last showed node's sizer

            wxObject * obj = m_mgr->Settings[_T("lastShowedNode")].GetWxObjectPtr();

            NodePanel * lpanel = wxDynamicCast(obj,NodePanel);

            if(lpanel && lpanel != panel) {
                wxSizer * sizer = lpanel->GetSizer();
                if(sizer) {
                    sizer->RecalcSizes();
                    sizer->Layout();
                    sizer->Fit(lpanel);
                    sizer->SetSizeHints(lpanel);
                    win->Refresh();
                }
            }
        }

    }

    if(panel)
        m_mgr->Settings[_T("lastShowedNode")] = panel;

    m_mgr->Settings[_T("lastShowedSymbolItem")] = citem;

    wxWindow * gpwin = GetGrandParent();
    wxWindow * pwin = GetParent();

    if(pwin) {
        wxSizer * sizer = pwin->GetSizer();
        if(sizer) {
            sizer->RecalcSizes();
            sizer->Layout();
            sizer->Fit(pwin);
            sizer->SetSizeHints(pwin);
            pwin->Refresh();
        }
    }

    if(gpwin) {
        wxSizer * sizer = gpwin->GetSizer();
        if(sizer) {
            sizer->RecalcSizes();
            sizer->Layout();
            sizer->Fit(gpwin);
            sizer->SetSizeHints(gpwin);
            gpwin->Refresh();
        }
    }

}

void SymbolPanel::DropChangeEvent(int t)
{
    wxWindow * win = GetParent();
    if(!win) return;

    wxCommandEvent event(EditorValueChangedEvent);
    event.SetInt(t);
    win->AddPendingEvent(event);
}

void SymbolPanel::OnButton(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( id == idShowHide && m_panel ) {
        ShowHide();
        return;
    }

    for( size_t a = 0; a < m_settingsGui.GetCount(); a++ ) {

        GUIDescGUIItem desc = m_settingsGui[a];

        if( desc.DialogButton && desc.DialogButton->GetId() == id ) { // open dialog

            switch(desc.Type) {
                case GDSI_COLOR:{
                    wxColourData cd;
                    wxColor c;
                    wxVariant d;
                    d << c;
                    c << desc.GetValue( d );
                    cd.SetColour( c );
                    wxColourDialog dlg(this,&cd);
                    if(dlg.ShowModal() == wxID_OK) {
                        wxColor nc = dlg.GetColourData().GetColour();
                        DrawHelper::SetColorA(nc,c.Alpha());
                        wxVariant r = DrawHelper::GetColorAsString(nc,wxC2S_CSS_SYNTAX);
                        desc.SetValue( r );
                    }

                }break;
            }

            return;
        }

    }


    switch(id){
        case idButtonLeft:
            if( m_mgr->SlideSymbolTo(Desc,MVGD_SLIDE_LEFT) ) {
                DropChangeEvent( EVCE_STRUCT );
                m_mgr->RefreshTargetGraph();
                SlideTo(MVGD_SLIDE_LEFT);
            }
            break;

        case idButtonRight:
            if( m_mgr->SlideSymbolTo(Desc,MVGD_SLIDE_RIGHT) ) {
                DropChangeEvent( EVCE_STRUCT );
                m_mgr->RefreshTargetGraph();
                SlideTo(MVGD_SLIDE_RIGHT);
            }
            break;

        case idButtonRemove:
            if( m_mgr->RemoveSymbolDesc( Desc ) ) {
                DropChangeEvent( EVCE_STRUCT );
                m_mgr->RefreshTargetGraph();
                RemoveThis();
            }
            break;

        case idButtonCopy:
            m_mgr->SetSymbolDescClipboard(*Desc);
            break;
    }

}

bool SymbolPanel::SetSymbolName(const wxString& name)
{
    MultiGraphDesc* mgdesc = m_mgr->GetDesc();

    if(mgdesc->FindBy( name, _T("name"), (*(Desc))[_T("id")], _T("id") ) != NULL) {
        int r = wxMessageBox(_T("The selected symbolname is alredy use.\nDo you want to use for this item?"),_T("Symbolname collision"),wxYES_NO | wxICON_QUESTION );
        if(r == wxNO) {
            return false;
        }
    }

    DropChangeEvent( EVCE_TEXT );

    return true;
}

void SymbolPanel::OnCombo(wxCommandEvent& evt)
{
    evt.Skip();
    wxString sel = evt.GetString();

    MultiGraphDesc* mgdesc = m_mgr->GetDesc();

    if(mgdesc->FindBy( sel, _T("name"), (*(Desc))[_T("id")], _T("id") ) != NULL) {
        int r = wxMessageBox(_T("The selected symbolname is alredy use.\nDo you want to use for this item?"),_T("Symbolname collision"),wxYES_NO | wxICON_QUESTION );
        if(r == wxNO) {
            wxComboBox * cb = wxDynamicCast(evt.GetEventObject(),wxComboBox);
            if(cb) cb->SetStringSelection( (*(Desc))[_T("name")].GetString() );
            return;
        }
    }

    DropChangeEvent( EVCE_TEXT );
}

/** \~english
  *
  * \~hungarian
  *
  */
void SymbolPanel::InitPanel()
{
    //FunctionMeter fn(_T("SymbolPanel::InitPanel()"));
    m_panel = this;

    GUIDescStructItemArray& descstruct = m_mgr->GetDescStruct();
    wxGridBagSizer * sizer = new wxGridBagSizer();

    wxString title;
    wxString sname = (*Desc)[_T("name")].GetString();

    if(UseCustomtitleOnPaneltitle) {
        if( Desc->find(_T("customtitle")) != Desc->end() ) title = (*Desc)[_T("customtitle")].GetString();
    }

    if(title == wxEmptyString){
        title = m_mgr->GetGaugeDescMgr()->GetGaugeSymbolTitle(sname);
        if(title == wxEmptyString) title = sname;
    }

    size_t rowc = 0;
    int border = 1;

	m_panelButton = new wxButton(this, idShowHide, m_mainButtonStr + title);// + _T(" panel"));
	m_panelButton->Connect(idShowHide,wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SymbolPanel::OnButton), NULL, this);
	wxFont font = m_panelButton->GetFont();
	font.SetWeight(wxFONTWEIGHT_BOLD);
	m_panelButton->SetFont(font);

    wxBitmapButton * removeb = new wxBitmapButton(this,  idButtonRemove, DrawHelper::GetBitmap(_T("remove-16.png")) );
    wxBitmapButton * copyb = new wxBitmapButton(this,  idButtonCopy, DrawHelper::GetBitmap(_T("copy-16.png")) );

    removeb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SymbolPanel::OnButton), NULL, this);
    copyb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SymbolPanel::OnButton), NULL, this);

    removeb->SetToolTip(_T("Remove symbol descriptor"));
    copyb->SetToolTip(_T("Copy"));


    for( size_t i = 0; i < descstruct.GetCount(); i++ ) {

        GUIDescStructItem dsitem = descstruct[i];

        if( dsitem.ShowOnEditor == false ) continue;

        wxString name = dsitem.Name;

        //int id = (*Desc)[_T("id")].GetLong();
        wxWindow * win = NULL;
        wxButton * dButt = NULL;


        wxString value = NULLSTR;
        if( Desc->find(name) != Desc->end() ) {
            value = (*Desc)[name].GetString();
        }

        switch(dsitem.Type){

#if USE_NEW_SYMBOL_SELECTOR
            case GDSI_SYMBOLNAME:{
                MVG_SymbolChooserPanel * panel = new MVG_SymbolChooserPanel(m_panel, m_mgr->GetGaugeDescMgr());
                panel->SetSymbol( Desc );
                win = panel;
                }break;
#else
            case GDSI_SYMBOLNAME:{
                int id = wxNewId();
                wxComboBoxNullEx * box = new wxComboBoxNullEx(m_panel, id, (*Desc)[name].GetString(),NULLSTR, wxDefaultPosition, wxDefaultSize,wxArrayString(), wxCB_SORT /*| wxCB_READONLY*/ );
                box->Connect(wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler(SymbolPanel::OnCombo), NULL, this );
                //box->SetStringSelection((*Desc)[name].GetString());
                win = box;
                }break;
#endif
            case GDSI_SYMBOLDESC:{
                MVG_SymbolDescChooserPanel * panel = new MVG_SymbolDescChooserPanel(m_panel, m_mgr);
                panel->SetSymbol( Desc );
                win = panel;
                }break;

            case GDSI_COLOR:{
                wxColor color;
                wxString colorstr;
                if( Desc->find(name) != Desc->end() )  {
                    color << (*Desc)[name];
                    colorstr = DrawHelper::GetColorAsString(color,wxC2S_HTML_SYNTAX);
                }
                win = new wxTextCtrlNullEx(m_panel, wxID_ANY, colorstr, NULLSTR );
                dButt = new wxBitmapButton(m_panel, wxNewId(), DrawHelper::GetBitmap(_T("colorize16c.png")) );
                dButt->SetToolTip(_T("Open color dialog"));
                //new wxButton(m_panel, wxNewId(), _T("dlg"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
                }break;

            case GDSI_NUMBER:{
                wxTextCtrlNullEx * ctrl = new wxTextCtrlNullEx(m_panel, wxID_ANY, value, NULLSTR );
                wxTextValidator validator(wxFILTER_NUMERIC);
                ctrl->SetValidator(validator);
                win = ctrl;
                }break;

            case GDSI_TEXT:{
                win = new wxTextCtrlNullEx(m_panel, wxID_ANY, value, NULLSTR);
                }break;

            default:
                continue;

        }

        wxStaticText * stext = new wxStaticText(m_panel, wxID_ANY, dsitem.Title);

        stext->SetToolTip(dsitem.HelpStr);
        win->SetToolTip(dsitem.HelpStr);

        wxFont font = stext->GetFont();
        font.SetWeight(wxFONTWEIGHT_BOLD);
        stext->SetFont(font);

        sizer->Add( stext, wxGBPosition(rowc,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL | wxALL, border);

        if(dButt == NULL){
            sizer->Add(win, wxGBPosition(rowc,1), wxDefaultSpan, wxEXPAND | wxALL, border);
        } else {
            wxBoxSizer * bs = new wxBoxSizer(wxHORIZONTAL);
            bs->Add(win,1,wxEXPAND);
            bs->Add(dButt);
            sizer->Add(bs, wxGBPosition(rowc,1), wxDefaultSpan, wxEXPAND | wxALL, border);
        }


        if(dButt) dButt->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SymbolPanel::OnButton), NULL, this);


        GUIDescGUIItem desc(name, dsitem.Type, win, dButt);

        //Logger::logf(LOG_MSG,_T("SymbolPanel::InitPanel() addline end"));

        m_settingsGui.Add(desc);
        rowc++;
    }

    wxBoxSizer * navisizer = new wxBoxSizer(wxHORIZONTAL);

    wxBitmapButton * leftb = new wxBitmapButton(this,  idButtonLeft, wxArtProvider::GetBitmap(wxART_GO_BACK) );
    wxBitmapButton * rightb = new wxBitmapButton(this,  idButtonRight , wxArtProvider::GetBitmap(wxART_GO_FORWARD) );


    leftb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SymbolPanel::OnButton), NULL, this);
    rightb->Connect(wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(SymbolPanel::OnButton), NULL, this);


    leftb->SetToolTip(_T("Move symbol to left"));
    rightb->SetToolTip(_T("Move symbol to right"));

    navisizer->Add(leftb, 1, wxALL | wxALIGN_LEFT, border );
    navisizer->Add(rightb, 1, wxALL | wxALIGN_RIGHT, border );

    sizer->Add(navisizer, wxGBPosition(rowc,0), wxGBSpan(1,2), wxEXPAND  );

	m_mainsizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer * headsizer = new wxBoxSizer(wxHORIZONTAL);

    headsizer->Add(m_panelButton,1,wxALL|wxEXPAND, 2);
    headsizer->Add(removeb,0,wxALL, 2);
    headsizer->Add(copyb,0,wxALL, 2);

	m_mainsizer->Add(headsizer, 0, wxALIGN_CENTRE_HORIZONTAL | wxBOTTOM  );
    m_mainsizer->Add(sizer, 0, wxALL , 5);

    m_mainsizer->Show(sizer,false);

	SetSizer(m_mainsizer);
	m_mainsizer->Fit(this);
//	m_mainsizer->SetSizeHints(this);

}

bool SymbolPanel::SlideTo(int target)
{
    bool isLeft;
    switch(target){
        case MVGD_SLIDE_LEFT: isLeft = true; break;
        case MVGD_SLIDE_RIGHT: isLeft = false; break;
        default: return false;
    }

    wxWindow * win = GetParent();
    if(!win) return false;
    wxSizer * sizer = win->GetSizer();
    if(!sizer) return false;
    NodePanel * np = wxDynamicCast(win,NodePanel);

    wxSizerItemList& list = sizer->GetChildren();

    size_t count = list.GetCount();

    wxSizerItemList::compatibility_iterator node = list.GetFirst();

    int slidediff = isLeft ? -1 : 1;
    int borderidx = isLeft ? count - 2 : 1;
    int borderlim = isLeft ? 1 : count - 2;

    for(int idx = 0; node ; node = node->GetNext(), idx++ )
    {
        wxSizerItem *item = node->GetData();
        if(!item) continue;
        wxWindow * swin = item->GetWindow();
        if(!swin) continue;

        if(swin == this) {
            wxSizerItem * newitem = new wxSizerItem(swin, item->GetProportion(), item->GetFlag(), item->GetBorder(), item->GetUserData() );
            sizer->Detach(idx);
            if(idx == borderlim)
                sizer->Insert( borderidx, newitem );
            else
                sizer->Insert( idx + slidediff, newitem );


            if(np) np->RefreshPanel();

            return true;
        }
    }

    return false;
}

void SymbolPanel::RemoveThis()
{
    wxWindow * win = GetParent();
    if(!win) return;
    wxSizer * sizer = win->GetSizer();
    if(!sizer) return;


    sizer->Detach(this);
    sizer->Layout();

    if( m_mgr->Settings.find(_T("lastShowedSymbolItem")) != m_mgr->Settings.end() ){
        wxObject * obj = m_mgr->Settings[_T("lastShowedSymbolItem")].GetWxObjectPtr();
        wxSizerItem * sitem = wxDynamicCast(obj,wxSizerItem);
        wxSizerItem * citem = NULL;
        if( sitem && sitem == m_mainsizer->GetItem(1) ) m_mgr->Settings[_T("lastShowedSymbolItem")] = citem;
    }

    Destroy();

//    sizer->Fit(win);
//    sizer->SetSizeHints(win);

    NodePanel * np = wxDynamicCast(win,NodePanel);
    if(np) np->RefreshPanel();

}

BEGIN_EVENT_TABLE(NodeEditPanel,wxFrame)

    EVT_BUTTON(idBtnClose, NodeEditPanel::OnButton)
    EVT_BUTTON(idBtnApply, NodeEditPanel::OnButton)

END_EVENT_TABLE()

NodeEditPanel::NodeEditPanel(NodePanel* parent,MultiGraphDescNode* nodeDesc, MVG_DescManager* mgr)
{
    m_parent = parent;
    m_nodeDesc = nodeDesc;
    m_mgr = mgr;

    Create(parent, wxID_ANY, _T("Node editor"));
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    wxFlexGridSizer * gsizer = new wxFlexGridSizer(2);
    gsizer->AddGrowableCol(1);

    double fdr = m_nodeDesc->Properties[_T("flagdisplayratio")].GetDouble();

    m_titleCtrl = new wxTextCtrl(this, wxID_ANY, m_nodeDesc->Properties[_T("title")].GetString() );
    m_flagCtrl = new wxTextCtrl(this, wxID_ANY, wxString::Format(_T("%0.2f"),fdr) );
    m_flagCtrl->SetValidator(wxTextValidator(wxFILTER_NUMERIC));

    wxBitmapButton * upb = new wxBitmapButton(this,  idButtonUp, wxArtProvider::GetBitmap(wxART_GO_UP) );
    wxBitmapButton * downb = new wxBitmapButton(this,  idButtonDown , wxArtProvider::GetBitmap(wxART_GO_DOWN) );
    wxBitmapButton * removeb = new wxBitmapButton(this,  idButtonRemove, DrawHelper::GetBitmap(_T("remove-16.png")) );

    m_parent->Connect(idButtonUp, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton));
    m_parent->Connect(idButtonDown, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton));
    m_parent->Connect(idButtonRemove, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler(NodePanel::OnButton));

    gsizer->Add(new wxStaticText(this, wxID_ANY,_T("Node title")),0,wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gsizer->Add(m_titleCtrl,0,wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, 5);

    gsizer->Add(new wxStaticText(this, wxID_ANY,_T("Flag display ratio")),0,wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gsizer->Add(m_flagCtrl,0,wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, 5);

    gsizer->Add(new wxStaticText(this, wxID_ANY,_T("Move up this node")),0,wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gsizer->Add(upb,0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    gsizer->Add(new wxStaticText(this, wxID_ANY,_T("Move down this node")),0,wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gsizer->Add(downb,0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    gsizer->Add(new wxStaticText(this, wxID_ANY,_T("Delete this node\n(and all symbol)")),0,wxALL | wxALIGN_CENTER_VERTICAL, 5);
    gsizer->Add(removeb,0,wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    wxBoxSizer * bsizer = new wxBoxSizer(wxHORIZONTAL);

    bsizer->Add(new wxButton(this, idBtnApply,_T("Apply")), 0, wxALL, 5 );
    bsizer->Add(new wxButton(this, idBtnClose,_T("Close")), 0, wxALL, 5 );

    wxBoxSizer * mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->Add(gsizer,0,wxEXPAND);
    mainsizer->Add(bsizer,0,wxALIGN_CENTER_HORIZONTAL);

	SetSizer(mainsizer);
	mainsizer->Fit(this);
	mainsizer->SetSizeHints(this);

}

void NodeEditPanel::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()) {
        case idBtnApply:{
            m_nodeDesc->Properties[_T("title")] = m_titleCtrl->GetValue();
            double tmp;
            m_flagCtrl->GetValue().ToDouble(&tmp);
            m_nodeDesc->Properties[_T("flagdisplayratio")] = tmp;
            wxWindow * win = GetParent();
            wxCommandEvent event(EditorValueChangedEvent);
            event.SetInt( EVCE_STRUCT );
            if(win) win->AddPendingEvent(event);
            m_mgr->RefreshTargetGraph();
            }break;

        case idBtnClose:
            Close();
            break;
    }
}
