#ifndef MVG_DESCEDITOR_H
#define MVG_DESCEDITOR_H

#include <wx/wx.h>
#include "../../gaugedesc/MultiGraphDesc.h"
#include "MVG_DescManager.h"

#define NULLSTR _T("NULL")

class NodeEditPanel;
class MultiValueGraph;
class GaugeDescManager;
class MultiGraphDescMgr;

class MVG_DescEditor: public wxFrame
{
    DECLARE_DYNAMIC_CLASS(MVG_DescEditor);
    public:
        MVG_DescEditor() {};
        MVG_DescEditor(wxWindow * parent, GaugeDescManager * gdmgr, int profileID);
        virtual ~MVG_DescEditor();

        void SetMultiGraph(MultiValueGraph * mg) {
            m_multiGraph = mg;
            m_mgr->SetMultiValueGraph(mg);
        };

        void InitEditor(int node_id = 0, int symbol_id = 0);

        void RemoveNodePanel(int id);

        void InsertNode(MultiGraphDescNode * nodeDesc, int target = MVGD_INSERT_AFTER, NodePanel * sPanel = NULL);

        void RestoreDesc();

        void ProcessChange(int type);

        void Apply();
        void Save();

    protected:

        void SetGuiToModel();
        void SetModelToGui();

        void OnMenu(wxCommandEvent& evt);
        void OnButton(wxCommandEvent& evt);

        void OnChange(wxCommandEvent& evt);

        void OnClose(wxCloseEvent& evt);

        wxString GetSymbolType(const wxString& name);

    private:
        wxVariantHash m_settings;

        enum {
            idMenuSTART = 2000,
            idMenuClose,
            idMenuToggle,
            idMenuProfileTitle,
            idMenuAppendNode,
            idMenuFloat,
            idMenuStay,
            idMenuEND,
            idBtnApply,
            idBtnSave,
            idBtnSaveAs,
            idBtnRestore,
            idBtnClose
        };


        MultiGraphDescManager* m_mgMgr;
        MultiValueGraph*    m_multiGraph;
        GaugeDescManager*   m_gdMgr;
        MultiGraphDesc *    m_graphDesc;
        MVG_DescManager*    m_mgr;

        wxButton *          m_restoreButton;
        wxButton *          m_saveButton;
        wxButton *          m_applyButton;

        NodePanelHash m_nodeList;

        MultiGraphDesc *    m_graphDescBackup;

        bool m_wasRestore;

        int                 m_profileID;

        wxString            m_editorCaption;



        DECLARE_EVENT_TABLE();
};


class NodePanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(NodePanel);

    public:
        NodePanel() {};
        NodePanel(wxWindow * parent, int id, MultiGraphDescNode& desc, MVG_DescManager* mgr);
        virtual ~NodePanel();

        MultiGraphDescNode * Desc;

        void InitPanel(int symbol_id = 0);

        SymbolPanelHash& GetSymbolPanelHash() { return m_symbolGuiList; }

        void RemoveSymbolPanel(int id);

        void AddSymbol(wxVariantHash * hash);

        void RefreshPanel();

        void OnButton(wxCommandEvent& evt);

    protected:

        void OnRightClick(wxMouseEvent& evt);
        void OnMenu(wxCommandEvent& evt);

        bool SlideTo(int target);
        void RemoveThis();

        void OnCloseEditor(wxCloseEvent& evt);

        void OnChange(wxCommandEvent& evt);
        void DropChangeEvent(int t);

    private:

        int m_nodeID;

        enum {
            idButtonUp,
            idButtonAdd,
            idButtonDown,
            idButtonEdit,
            idButtonRemove,
            idPopupMenuPaste,
            idPopupMenuInsertAfter,
            idPopupMenuInsertBefore,
            idPopupMenuRemove,
            idPopupMenuAppendSymbol
        };

        MVG_DescManager* m_mgr;

        SymbolPanelHash m_symbolGuiList;

        wxMenu  *   m_popupMenu;
        wxMenu  *   m_addMenu;

        NodeEditPanel * m_nodeEditor;

        DECLARE_EVENT_TABLE();


};



class SymbolPanel: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(SymbolPanel);

    public:
        SymbolPanel() {};
        SymbolPanel(wxWindow * parent, int id, wxVariantHash& desc, MVG_DescManager* mgr);
        virtual ~SymbolPanel();

        wxVariantHash * Desc;

        void InitPanel();

        GUIDescGUIItemArray& GetGUIDescGUIItemArray() { return m_settingsGui; }

        void SetPanelString(const wxString& str);

        bool SetSymbolName(const wxString& name);

    protected:
        void OnButton(wxCommandEvent& evt);
        void OnCombo(wxCommandEvent& evt);

        bool SlideTo(int target);
        void RemoveThis();

        void ShowHide();

        void OnChange(wxCommandEvent& evt);

        void DropChangeEvent(int t);

    private:

        int m_symbolID;

        enum {
            idShowHide = wxID_OPEN,
            idButtonLeft = 2000,
            idButtonRight,
            idButtonRemove,
            idButtonCopy
        };

        MVG_DescManager*        m_mgr;

        GUIDescGUIItemArray m_settingsGui;

        wxString                m_mainButtonStr;

        wxPanel*                m_panel;
        wxBoxSizer*             m_mainsizer;

        wxButton  *             m_panelButton;
};

class NodeEditPanel: public wxFrame
{
    public:
        NodeEditPanel(NodePanel* parent, MultiGraphDescNode* nodeDesc, MVG_DescManager* mgr);

    private:

        void OnButton(wxCommandEvent& evt);

        enum {
            idButtonUp,
            idButtonDown,
            idButtonEdit,
            idButtonRemove,
            idBtnApply,
            idBtnClose
        };

        NodePanel*  m_parent;
        MultiGraphDescNode* m_nodeDesc;

        wxTextCtrl* m_titleCtrl;
        wxTextCtrl* m_flagCtrl;

        MVG_DescManager *   m_mgr;

        DECLARE_EVENT_TABLE();
};


#endif // MVG_DESCEDITOR_H
