#include "MVG_ThemeEditor.h"

#include <wx/colordlg.h>
#include <wx/gbsizer.h>

#include "../../gaugedesc/GaugeDescManager.h"
#include "../../DrawHelper.h"
#include "../../logger.h"
#include "../../customwidgets/wxTextCtrlNullEx.h"
#include "../../common/FunctionMeter.h"
#include "MultiValueGraph.h"
#include "../../events/EditorValueChangedEvent.h"
#include "../../configmanager/ConfigMgrFrameLayout.h"
#include "../../customwidgets/wxTextCtrlNullEx.h"
#include "MVG_ProfilesEditor.h"

#include "../../configmanager/ConfigHelper2.h"
#include "../../configmanager/ConfigHashHelper.h"

#include "../../DisplayHelper.h"

#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(ThemeLineArray);

BEGIN_EVENT_TABLE(MVG_ThemeEditor,wxFrame)

    EVT_BUTTON(idBtnApply, MVG_ThemeEditor::OnButton)
    EVT_BUTTON(idBtnSave, MVG_ThemeEditor::OnButton)
    EVT_BUTTON(idBtnSaveAs, MVG_ThemeEditor::OnButton)
    EVT_BUTTON(idBtnRestore, MVG_ThemeEditor::OnButton)
    EVT_BUTTON(idBtnClose, MVG_ThemeEditor::OnButton)

    EVT_MENU_RANGE(idMenuClose,idMenuProfileTitle, MVG_ThemeEditor::OnMenu)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(MVG_ThemeEditor,wxFrame);

#define SETTINGS_STAY _T("stayontop")
#define SETTINGS_FLOAT _T("floatonparent")

MVG_ThemeEditor::MVG_ThemeEditor(wxWindow * parent, GaugeDescManager * gdmgr, int profileID)
{
    //ctor
    m_editorCaption = _T("Multi Graph theme editor for profile: ");
    m_profileID = profileID;
    m_multiGraph = NULL;
    m_gdMgr = gdmgr;
    m_mgMgr = m_gdMgr->GetMultiGraphDescMgr();

    long flags = wxDEFAULT_FRAME_STYLE;

    m_settings[SETTINGS_FLOAT] = true;
    m_settings[SETTINGS_STAY] = false;

    ConfigHashHelper::GetValues(this,m_settings, ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT ));

    if(m_settings[ SETTINGS_FLOAT ].GetBool() == true) {
        flags |= wxFRAME_FLOAT_ON_PARENT;
    }

    if(m_settings[ SETTINGS_STAY ].GetBool() == true) {
        flags |= wxSTAY_ON_TOP;
    }

    Create(parent, wxID_ANY, m_editorCaption + m_mgMgr->GetMultiGraphThemeProfileTitle(m_profileID), wxDefaultPosition, wxDefaultSize, flags);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_graphTheme = m_mgMgr->GetMultiGraphTheme(m_profileID);

    CreateBackup(m_graphTheme);

    InitEditor();

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    ConfigMgrFrameLayout cfgmgr(ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT ));
    cfgmgr.loadTo(this);

    wxSize size = GetSize();

    if(size.x < 200) size.x = 200;
    if(size.y < 400) size.y = 400;

    if(size != GetSize()) SetSize(size);

    Show(true);
}

MVG_ThemeEditor::~MVG_ThemeEditor()
{
    //dtor
    ConfigMgrFrameLayout cfgmgr(ConfigHelper2::getConfig( ConfigHelper2::CONFIG_LAYOUT ));
    cfgmgr.saveFrom(this);

    ConfigHashHelper::SaveValues(this,m_settings, ConfigHelper2::getConfig(ConfigHelper2::CONFIG_LAYOUT) );

    delete m_graphThemeBck;
}

void MVG_ThemeEditor::OnMenu(wxCommandEvent& evt)
{
    switch( evt.GetId() ) {
        case idMenuClose:
            Close();
            break;

        case idMenuFloat:
            m_settings[SETTINGS_FLOAT] = evt.IsChecked();
            break;

        case idMenuStay: {
            m_settings[SETTINGS_STAY] = evt.IsChecked();
            long flags = GetWindowStyleFlag();
            if(evt.IsChecked())
                flags |= wxSTAY_ON_TOP;
            else
                flags &= ~wxSTAY_ON_TOP;

            SetWindowStyle(flags);

            }break;

        case idMenuProfileTitle:{
            wxString ctitle = m_mgMgr->GetMultiGraphThemeProfileTitle(m_profileID);
            wxString title = wxGetTextFromUser(_T("The current profile title:"),_T("Profile settings"), ctitle );
            if(title != wxEmptyString && title != ctitle) {
                m_mgMgr->SetMultiGraphThemeProfileTitle(m_profileID, title);
                SetTitle( m_editorCaption + title );

                MVG_ProfilesEditor * pe = wxDynamicCast(GetParent(),MVG_ProfilesEditor);
                if(pe) pe->RefreshThemeList();

            }
            }break;
    }
}

void MVG_ThemeEditor::CreateBackup(MultiGraphTheme * sourceTheme)
{
    m_graphThemeBck = new MultiGraphTheme(*sourceTheme);
}

void MVG_ThemeEditor::InitEditor()
{
    wxMenuBar * mbar = new wxMenuBar();

    wxMenu * fmenu = new wxMenu();
    fmenu->Append(idBtnClose,_T("&Close editor"));


    wxMenu * smenu = new wxMenu();
    smenu->AppendCheckItem(idMenuFloat,_T("Float on parent"));
    smenu->AppendCheckItem(idMenuStay,_T("Stay on top"));
    smenu->Check( idMenuStay , m_settings[SETTINGS_STAY].GetBool() );
    smenu->Check( idMenuFloat , m_settings[SETTINGS_FLOAT].GetBool() );

    wxMenu * pmenu = new wxMenu();
    pmenu->Append(idMenuProfileTitle, _T("Set the current profile title") );


    mbar->Append(fmenu,_T("&File"));
    mbar->Append(smenu,_T("&Settings"));
    mbar->Append(pmenu,_T("&Profile"));

    SetMenuBar(mbar);

    //MultiGraphThemePropertyHash& propdesc = m_mgMgr->GetMultiGraphThemePropertyHash();
    MultiGraphThemePropertyArray& propdesc = m_mgMgr->GetMultiGraphThemePropertyArray();

    wxGridBagSizer * winsizer = new wxGridBagSizer();

    wxScrolledWindow  * swin = new wxScrolledWindow(this, -1,   wxDefaultPosition, wxDefaultSize, wxVSCROLL /*| wxBORDER_SUNKEN*/ );
    //swin->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));

    swin->SetScrollRate( 5, 10 );

    winsizer->AddGrowableCol(1);

    //for(MultiGraphThemePropertyHash::iterator it = propdesc.begin(); it != propdesc.end(); it++) {
    for(size_t a = 0; a < propdesc.GetCount(); a++ ) {

        int row = a;

        MultiGraphThemeProperty prop = propdesc[a];

        wxStaticText * title = new wxStaticText(swin,wxID_ANY,prop.Title);
        wxFont tfont = title->GetFont();
        tfont.SetWeight(wxFONTWEIGHT_BOLD);
        title->SetFont(tfont);

        winsizer->Add(title,wxGBPosition(row,0),wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 2 );

        wxString value = m_graphTheme->MakeString(prop.Name);

        wxGBSpan span(1,1);

        wxTextCtrlNullEx* ctrl =  new wxTextCtrlNullEx(swin,wxID_ANY, value );

        ThemeLine th(prop,ctrl);

        if(prop.Name.Find(_T("color")) != wxNOT_FOUND) {
            int dlgid = wxNewId();
            wxBitmapButton * dButt = new wxBitmapButton(swin, dlgid, DrawHelper::GetBitmap(_T("colorize16c.png")) );
            dButt->SetToolTip(_T("Open color dialog"));
            winsizer->Add(dButt, wxGBPosition(row,2),wxDefaultSpan, wxALL | wxALIGN_CENTER_VERTICAL, 2 );

            m_colorDlgLines[dlgid] = th;

            Connect( dlgid, wxEVT_COMMAND_BUTTON_CLICKED , wxCommandEventHandler(MVG_ThemeEditor::OnCDlgButton));

        } else {
            span.SetColspan(2);
            wxTextValidator validator(wxFILTER_NUMERIC);
            ctrl->SetValidator(validator);
        }

        m_themeLines.Add(th);

        winsizer->Add(ctrl,wxGBPosition(row,1),span, wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, 2 );
    }

	swin->SetSizer(winsizer);
	winsizer->Fit(swin);
	winsizer->SetSizeHints(swin);

    m_restoreButton = new wxButton(this, idBtnRestore, _T("Restore"));
    m_saveButton = new wxButton(this, idBtnSave, _T("Save"));
    m_applyButton = new wxButton(this, idBtnApply, _T("Apply"));
    wxButton * saveAsButton = new wxButton(this, idBtnSaveAs, _T("Save As"));

    wxBoxSizer * bsizer = new wxBoxSizer(wxHORIZONTAL);

    bsizer->Add(m_applyButton,0, wxALL,5);
    bsizer->Add(m_saveButton,0, wxALL,5);
    bsizer->Add(saveAsButton,0, wxALL,5);
    bsizer->Add(m_restoreButton,0, wxALL,5);
    bsizer->Add(new wxButton(this, idBtnClose, _T("Close Editor")),0, wxALL,5);

    m_applyButton->SetToolTip(_T(""));
    m_saveButton->SetToolTip(_T(""));
    saveAsButton->SetToolTip(_T(""));
    m_restoreButton->SetToolTip(_T(""));

    wxBoxSizer* mainsizer = new wxBoxSizer(wxVERTICAL);

    mainsizer->Add(swin, 1, wxALL | wxEXPAND, 0);
    mainsizer->Add(bsizer, 0, wxALL | wxALIGN_CENTER_HORIZONTAL, 5);

    SetSizer(mainsizer);
	mainsizer->Fit(this);

    wxSize es = GetSize();

	wxSize ds = DisplayHelper::GetDisplaySize();

	SetSize(-1,ds.y/2);

	SetMinSize( wxSize(es.x, 100) );

	m_restoreButton->Enable(false);

	m_applyButton->Enable(false);


    m_saveButton->Enable( m_graphTheme->IsChanged );

	Connect( EditorValueChangedEvent, wxCommandEventHandler(MVG_ThemeEditor::OnChange)  );
}

void MVG_ThemeEditor::OnCDlgButton(wxCommandEvent& evt)
{
    int id = evt.GetId();

    if( m_colorDlgLines.find(id) == m_colorDlgLines.end() ) return;

    ThemeLine th = m_colorDlgLines[id];

    if(th.Ctrl == NULL) return;


    wxColourData cd;
    wxColor c = m_graphTheme->GetColor( th.Prop.Name );
    cd.SetColour( c );
    wxColourDialog dlg(this,&cd);
    if(dlg.ShowModal() == wxID_OK) {
        wxColor nc = dlg.GetColourData().GetColour();
        DrawHelper::SetColorA(nc,c.Alpha());
        th.Ctrl->SetValue(DrawHelper::GetColorAsString(nc,wxC2S_CSS_SYNTAX));
    }


}

void MVG_ThemeEditor::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()){
        case idBtnApply:
            Apply();
            break;

        case idBtnSave:
            Save();
            break;

        case idBtnSaveAs:{
            wxString title = wxGetTextFromUser(_T("Save as theme to new profile"),_T("Profile title:"));
            if(title != wxEmptyString)
                m_mgMgr->SaveAsMultiGraphTheme( m_profileID, title );
            }break;

        case idBtnRestore:
            RestoreTheme();
            break;

        case idBtnClose:
            Close();
            break;
    }
}

void MVG_ThemeEditor::OnChange(wxCommandEvent& evt)
{
    ProcessChange(evt.GetInt());
}

void MVG_ThemeEditor::ProcessChange(int type)
{
    Logger::logf(LOG_MSG,_T("MVG_ThemeEditor::ProcessChange type=%i"),type);

    if( type & EVCE_STRUCT ) {
        m_restoreButton->Enable(true);
    }

    if( type & EVCE_TEXT ) {
        m_applyButton->Enable(true);
    }

    m_saveButton->Enable(true);

}

void MVG_ThemeEditor::RestoreTheme()
{
    m_wasRestore = true;

    m_mgMgr->SetMultiGraphTheme(m_graphThemeBck, m_profileID);

    if(m_multiGraph) m_multiGraph->RedrawWidget();

    delete m_graphTheme;
    m_graphTheme = m_graphThemeBck;

    CreateBackup(m_graphTheme);

    m_restoreButton->Enable(false);

    for(size_t a = 0; a < m_themeLines.GetCount(); a++ ) {
        ThemeLine tl = m_themeLines[a];

        if(tl.Ctrl == NULL) continue;

        tl.Ctrl->ChangeValueEx( m_graphTheme->MakeString(tl.Prop.Name) );
        tl.Ctrl->ResetChange();
    }


}

void MVG_ThemeEditor::Apply()
{

    for(size_t a = 0; a < m_themeLines.GetCount(); a++ ) {
        ThemeLine tl = m_themeLines[a];

        if(tl.Ctrl == NULL) continue;

        m_graphTheme->SetByString(tl.Prop.Name,tl.Ctrl->GetValue());

        tl.Ctrl->ResetChange();
    }

    m_graphTheme->IsChanged = true;

    m_applyButton->Enable(false);
    m_restoreButton->Enable(true);

    if(m_multiGraph) m_multiGraph->RedrawWidget();
}

void MVG_ThemeEditor::Save()
{
    if(m_applyButton->IsEnabled())
        Apply();


    m_mgMgr->SaveMultiGraphTheme( m_profileID );
    m_saveButton->Enable(false);
    m_restoreButton->Enable(false);
    delete m_graphThemeBck;
    m_graphThemeBck = new MultiGraphTheme(*m_graphTheme);

}

void MVG_ThemeEditor::SetMultiGraph(MultiValueGraph * mg)
{
    m_multiGraph = mg;
}
