#include "MVG_ToolbarPanel.h"

#include <wx/statline.h>
#include <wx/bmpbuttn.h>
#include <wx/dcbuffer.h>

#include "../../customwidgets/wxBitmapToggleButton.h"
#include "MultiValueGraph.h"
#include "../../DirHelper.h"
#include "../../DrawHelper.h"

#include "../../common/IconSetsHelper.h"

#include "../../bcWidgets/bcButton.h"
#include "../../wxImageHelper.h"
#include "../../bcWidgets/bcWrapper.h"

BEGIN_EVENT_TABLE(MVG_ToolbarPanel,wxPanel)

    EVT_TEXT(idZoomBox, MVG_ToolbarPanel::OnText)

    EVT_BUTTON(idButtonZoomIn, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonZoomOut, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonZoomOri, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonZoomFitSel, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonMarkerDlg, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonRTOnlineRefresh, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonOpenDescEditor, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonOpenThemeEditor, MVG_ToolbarPanel::OnButton)
    EVT_BUTTON(idButtonSearch, MVG_ToolbarPanel::OnButton)


    EVT_LEFT_UP(MVG_ToolbarPanel::OnMouse)
    EVT_LEFT_DOWN(MVG_ToolbarPanel::OnMouse)
    EVT_RIGHT_UP(MVG_ToolbarPanel::OnMouse)
    EVT_RIGHT_DOWN(MVG_ToolbarPanel::OnMouse)

    EVT_PAINT(MVG_ToolbarPanel::OnPaint)

END_EVENT_TABLE()

MVG_ToolbarPanel::MVG_ToolbarPanel(MultiValueGraph * graphParent)
{
    //ctor
    Create(graphParent);
    m_graph = graphParent;
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    CreateBcParent(this);

    wxBoxSizer * mainsizer = new wxBoxSizer(wxHORIZONTAL);

    m_zoomInBtn = new wxBitmapButton(this, idButtonZoomIn, DrawHelper::GetBitmap(_T("zoom16-in.png")) );
    m_zoomOutBtn = new wxBitmapButton(this, idButtonZoomOut, DrawHelper::GetBitmap(_T("zoom16-out.png")));
    m_zoomOriBtn = new wxBitmapButton(this, idButtonZoomOri, DrawHelper::GetBitmap(_T("zoom16-toori.png")));
    //m_zoomFitSelBtn = new wxBitmapButton(this, idButtonZoomFitSel, GetBitmap(_T("zoom16-tosel.png")));


    m_refreshButton = new bcButton(this, idButtonRefresh, IconSetsHelper::GetImage(IS_REFRESH,IS_16) , wxDefaultPosition, wxDefaultSize, bcBUTTON_DEFAULT_STYLE | bcBUTTON_AUTOSIZE | bcBUTTON_TOGGLE );

    wxImage img = IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_REFRESH,IS_16);
    m_refreshButton->SetImgLabel( img, bcBUTTON_IMAGE_PRESSED );
    wxImageHelper::ShiftLuminance(img,-0.3);
    m_refreshButton->SetImgLabel( img, bcBUTTON_IMAGE_DISABLED );


    m_refreshButton->SetToolTip(_T("Switch on/off gauge refresh"));

    m_zoomInBtn->SetToolTip(_T("Zoom in"));
    m_zoomOutBtn->SetToolTip(_T("Zoom out"));
    m_zoomOriBtn->SetToolTip(_T("Zoom to 100%"));
    //m_zoomFitSelBtn->SetToolTip(_T("Zoom to the selection"));

    m_zoomBox = new wxComboBox(this, idZoomBox, _T(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY );

    mainsizer->Add( new bcWrapper(m_refreshButton), 0, wxALL, 2 );
    mainsizer->Add( new wxStaticLine(this, wxID_ANY,wxDefaultPosition,wxSize(2,2),wxLI_VERTICAL),0,wxALL|wxEXPAND,2 );

    bcButton* bcbmpbtn = new bcButton(this, idButtonMarkerDlg, DrawHelper::GetIconImg(_T("drawingpin-16.png")), wxDefaultPosition, wxDefaultSize, bcBUTTON_DEFAULT_STYLE | bcBUTTON_AUTOSIZE );
    bcbmpbtn->SetToolTip(_T("Open markers settings dialog"));
    mainsizer->Add( new bcWrapper(bcbmpbtn) , 0, wxALL, 2 );

    bcbmpbtn = new bcButton(this, idButtonRTOnlineRefresh, _T("Update"),wxDefaultPosition, wxSize(-1,bcbmpbtn->GetSize().Ceil().y) , bcBUTTON_DEFAULT_STYLE );
    mainsizer->Add( new bcWrapper(bcbmpbtn), 0, wxALL, 2 );
    bcbmpbtn->SetToolTip(_T("Update graph loghistory"));

    mainsizer->Add( new wxStaticLine(this, wxID_ANY,wxDefaultPosition,wxSize(2,2),wxLI_VERTICAL),0,wxALL|wxEXPAND,2 );

    bcbmpbtn = new bcButton(this, idButtonOpenDescEditor, DrawHelper::GetIconImg(_T("graphedit-16.png")), wxDefaultPosition, wxDefaultSize, bcBUTTON_DEFAULT_STYLE | bcBUTTON_AUTOSIZE  );
    bcbmpbtn->SetToolTip(_T("Open descriptor editor"));
    mainsizer->Add( new bcWrapper(bcbmpbtn), 0, wxALL, 2 );

    bcbmpbtn = new bcButton(this, idButtonOpenThemeEditor, DrawHelper::GetIconImg(_T("themeedit-16.png")), wxDefaultPosition, wxDefaultSize, bcBUTTON_DEFAULT_STYLE | bcBUTTON_AUTOSIZE  );
    bcbmpbtn->SetToolTip(_T("Open theme editor"));

    mainsizer->Add( new bcWrapper(bcbmpbtn), 0, wxALL, 2 );

    mainsizer->Add( new wxStaticLine(this, wxID_ANY,wxDefaultPosition,wxSize(2,2),wxLI_VERTICAL),0,wxALL|wxEXPAND,2 );

    bcbmpbtn = new bcButton(this, idButtonSearch, IconSetsHelper::GetImage(IS_SEARCH,IS_16), wxDefaultPosition, wxDefaultSize, bcBUTTON_DEFAULT_STYLE | bcBUTTON_AUTOSIZE  );
    bcbmpbtn->SetImgLabel( IconSetsHelper::GetImage(IS_WHITE_RAISED,IS_SEARCH,IS_16), bcBUTTON_IMAGE_PRESSED );
    bcbmpbtn->SetToolTip(_T("Open search dialog"));
    mainsizer->Add( new bcWrapper(bcbmpbtn), 0, wxALL, 2 );

    mainsizer->AddStretchSpacer(1);
    mainsizer->Add( new wxStaticLine(this, wxID_ANY,wxDefaultPosition,wxSize(2,2),wxLI_VERTICAL),0,wxALL|wxEXPAND,2 );
    mainsizer->Add( m_zoomInBtn, 0, wxALL, 2 );
    mainsizer->Add( m_zoomOutBtn, 0, wxALL, 2 );
    mainsizer->Add( m_zoomOriBtn, 0, wxALL, 2 );
   // mainsizer->Add( m_zoomFitSelBtn, 0, wxALL, 2 );
    mainsizer->Add( m_zoomBox, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2 );


	SetSizer(mainsizer);
	//mainsizer->Fit(this);
	//mainsizer->SetSizeHints(this);

}

MVG_ToolbarPanel::~MVG_ToolbarPanel()
{
    //dtor
}

void MVG_ToolbarPanel::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    dc.SetBackground( wxBrush(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE)) );
    dc.Clear();

    PaintWidgets(dc);
}

void MVG_ToolbarPanel::OnButton(wxCommandEvent& evt)
{
    switch(evt.GetId()) {
        case idButtonMarkerDlg:
            m_graph->SendToolBarCommand(_T("openmarkersdlg"));
            break;

        case idButtonRTOnlineRefresh:
            m_graph->SendToolBarCommand(_T("refreshlockedlen"));
            m_graph->SetFocus();
            break;

        case idButtonZoomIn:
            m_graph->SendToolBarCommand(_T("deltazoom"), 1 );
            m_graph->SetFocus();
            break;

        case idButtonZoomOut:
            m_graph->SendToolBarCommand(_T("deltazoom"), -1 );
            m_graph->SetFocus();
            break;

        case idButtonZoomOri:
            m_graph->SendToolBarCommand(_T("setzoom"), _T("100") );
            m_graph->SetFocus();
            break;

        case idButtonZoomFitSel:
            m_graph->SendToolBarCommand(_T("fittosel"));
            m_graph->SetFocus();
            break;

        case idButtonOpenDescEditor:
            m_graph->SendToolBarCommand(_T("opendesceditor"));
            break;

        case idButtonOpenThemeEditor:
            m_graph->SendToolBarCommand(_T("openthemeeditor"));
            break;

        case idButtonSearch:
            m_graph->SendToolBarCommand(_T("startsearch"));
            break;
    }

}

void MVG_ToolbarPanel::OnMouse(wxMouseEvent& evt)
{
    m_graph->SetFocus();
}


void MVG_ToolbarPanel::OnText(wxCommandEvent& evt)
{
    m_graph->SendToolBarCommand(_T("setzoom"), m_zoomBox->GetStringSelection() );
    m_graph->SetFocus();
}

wxVariant MVG_ToolbarPanel::GetValue(const wxString& type)
{

    if( type == _T("refreshgauges") ) {

        return m_refreshButton->GetToggleValue();

    } else if( type == _T("") ) {


    }

    return 0L;
}

bool MVG_ToolbarPanel::SendCommand(const wxString& cmd, wxVariant var)
{

    if( cmd == _T("refreshgauges") ) {

        m_refreshButton->SetToggleValue( var.GetBool() );

    } else if( cmd == _T("zoomboxinit") ) {

        m_zoomBox->Clear();

        m_zoomBox->Append( var.GetArrayString() );

    } else if( cmd == _T("setzoom") ) {

        m_zoomBox->SetStringSelection( var.GetString() );

    } else if( cmd == _T("enable_refresh_button") ) {

        m_refreshButton->Enable( var.GetBool() );

    } else
        return false;


    return true;
}

void MVG_ToolbarPanel::UpdateSize(const wxSize& size)
{
    SetSize(size);
}
