#include "TableEditorPanel.h"

#include "../wxStaticTextEx.h"
#include "../logger.h"
#include "../vemsdialog/WindowTipper.h"
#include "GaugeGroupManager.h"
#include "GroupMember.h"
#include "../wxPointEx.h"
#include "../wxSizeEx.h"
#include "../wxVariantHelper.h"
#include "SelectableWidgetRendererAgg.h"



BEGIN_EVENT_TABLE(TitleBarControl, wxPanel)


    EVT_LEFT_DOWN(TitleBarControl::OnLeftDown)
    EVT_LEFT_UP(TitleBarControl::OnLeftUp)
    EVT_MOTION(TitleBarControl::OnMouseMove)

    //EVT_SET_FOCUS(TitleBarControl::OnFocus)
    //EVT_KILL_FOCUS(TitleBarControl::OutFocus)

END_EVENT_TABLE()

BEGIN_EVENT_TABLE(TableEditorPanel, wxPanel)

    EVT_SET_FOCUS(TableEditorPanel::OnFocus)
    EVT_KILL_FOCUS(TableEditorPanel::OutFocus)

    //EVT_CHILD_FOCUS(TableEditorPanel::OnChildFocus)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(TableEditorPanel,wxPanel)

TableEditorPanel::TableEditorPanel(wxWindow * parent, int id, const wxPoint &pos)
{
    //ctor
    Create(parent, id, pos, wxDefaultSize, wxBORDER_RAISED );
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();

    m_grpMember = NULL;

    SelectableWidgetRendererAgg* renderer = new SelectableWidgetRendererAgg();
    SetRenderer(renderer);

    m_mainSizer =  new wxBoxSizer(wxVERTICAL);

    m_titleBar = new TitleBarControl(this, wxID_ANY);

    m_mainSizer->Add( m_titleBar ,0, wxALL | wxEXPAND,1);

	SetSizer(m_mainSizer);
	m_mainSizer->Fit(this);
	m_mainSizer->SetSizeHints(this);

	m_titleBar->Connect( wxEVT_SET_FOCUS, wxFocusEventHandler(TableEditorPanel::OnFocus), NULL, this);
	m_titleBar->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler(TableEditorPanel::OutFocus), NULL, this);
}

TableEditorPanel::~TableEditorPanel()
{
    //dtor
    if(m_renderer) delete m_renderer;
}
/*
void TableEditorPanel::OnChildFocus(wxChildFocusEvent &evt)
{
    m_titleBar->SetActive(true);
    Logger::logf(LOG_MSG,_T("TableEditorPanel::OnChildFocus"));
}
*/

void TableEditorPanel::OnFocusMsg(int type)
{
    m_titleBar->SetActive( type == FPST_SETFOCUS ? true : false );
}

void TableEditorPanel::OutFocus(wxFocusEvent &evt)
{
    m_titleBar->SetActive(false);
}

void TableEditorPanel::OnFocus(wxFocusEvent &evt)
{
    m_titleBar->SetActive(true);
}

wxSizerItem * TableEditorPanel::SetTable( wxWindow* win )
//wxSizerItem * TableEditorPanel::SetTableEditor( TableEditor * te )
{
    m_tableEditor = win;
    wxSizerItem * si = m_mainSizer->Add( win, 1, wxEXPAND | wxALL, 1);
    m_mainSizer->Fit(this);

	m_tableEditor->Connect( wxEVT_SET_FOCUS, wxFocusEventHandler(TableEditorPanel::OnFocus), NULL, this);
	m_tableEditor->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler(TableEditorPanel::OutFocus), NULL, this);

    return si;
}

wxSizerItem * TableEditorPanel::AddElement( wxWindow * win )
{
    wxSizerItem * si = m_mainSizer->Add( win, 0, wxEXPAND | wxALL, 1);
    m_mainSizer->Fit(this);

	win->Connect( wxEVT_SET_FOCUS, wxFocusEventHandler(TableEditorPanel::OnFocus), NULL, this);
	win->Connect( wxEVT_KILL_FOCUS, wxFocusEventHandler(TableEditorPanel::OutFocus), NULL, this);

    return si;
}

void TableEditorPanel::ProcessWidgetCommand(wxVariantHash& cmdHash)
{

    //Logger::logf(LOG_MSG,_T("TableEditorPanel::ProcessWidgetCommand gm=%p"),m_grpMember);

    if(!m_grpMember) return;

    GaugeProperties& gauge_props = m_grpMember->GetGaugeProperties();

    if( cmdHash.find(_T("groupname")) == cmdHash.end() || cmdHash[_T("groupname")].GetString() != gauge_props.groupName) return;

    if(cmdHash[_T("target_id")].GetLong() == GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_select")) {
        m_isSelected = true;
        DrawSelectionBitmap(m_tableEditor);
        return;
    }

    if(cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_move") && m_isSelected) {
        wxPointEx delta = wxVariantHelper::GetPointEx(cmdHash[_T("deltapos")]);
        Move( GetPosition() + delta);
        return;
    }

    if(cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_size") && m_isSelected) {
        wxSizeEx delta = wxVariantHelper::GetSizeEx(cmdHash[_T("deltasize")]);
        SetSize( GetSize() + delta );
        return;
    }

    if(cmdHash[_T("cmd")].GetString() == _T("gauge_deselect") && m_isSelected) {

        m_isSelected = false;

        m_tableEditor->Refresh(false);

        return;
    }

    if( cmdHash[_T("sender_id")].GetLong() != GetId() && cmdHash[_T("cmd")].GetString() == _T("gauge_click") ) {

        m_isSelected = false;

        m_tableEditor->Refresh(false);

        return;
    }


}

/****************************************************************************************
                                    TitleBarControl
*/

TitleBarControl::TitleBarControl(wxWindow * parent, int id, const wxString& title)
{
    m_delta = wxDefaultPosition;

    Create(parent,id);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_INACTIVECAPTION));
    ClearBackground();

    m_text = new wxStaticText(this, wxID_ANY, title);
    m_text->SetForegroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_CAPTIONTEXT));
    wxFont font = m_text->GetFont();
    font.SetWeight(wxFONTWEIGHT_BOLD);
    m_text->SetFont( font );
    m_text->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_INACTIVECAPTION));
    SetInitialSize(wxSize(-1,30));
    m_winTip = 0;
    //m_winTip = new WindowTipper(this);

    wxBoxSizer * mainSizer =  new wxBoxSizer(wxVERTICAL);

    mainSizer->Add( m_text ,0,  wxALL | wxALIGN_CENTER_VERTICAL , 3 );

    m_text->Connect(wxEVT_RIGHT_DOWN, wxMouseEventHandler(TitleBarControl::OnTextMouse), NULL, this );
    m_text->Connect(wxEVT_MIDDLE_DOWN, wxMouseEventHandler(TitleBarControl::OnTextMouse), NULL, this );

    m_text->Connect(wxEVT_LEFT_DOWN, wxMouseEventHandler(TitleBarControl::OnLeftDown), NULL, this );
    m_text->Connect(wxEVT_MOTION, wxMouseEventHandler(TitleBarControl::OnMouseMove), NULL, this );
    m_text->Connect(wxEVT_LEFT_UP, wxMouseEventHandler(TitleBarControl::OnLeftUp), NULL, this );

	SetSizer(mainSizer);
	mainSizer->Fit(this);
	mainSizer->SetSizeHints(this);

}

void TitleBarControl::OutFocus(wxFocusEvent &evt)
{
    SetActive(false);
}

void TitleBarControl::OnFocus(wxFocusEvent &evt)
{
    SetActive(true);
}

void TitleBarControl::SetActive(bool isactive)
{
    wxColor bc = wxSystemSettings::GetColour(wxSYS_COLOUR_INACTIVECAPTION);
    wxColor fc = wxSystemSettings::GetColour(wxSYS_COLOUR_INACTIVECAPTIONTEXT);
    if(isactive) {
        bc = wxSystemSettings::GetColour(wxSYS_COLOUR_ACTIVECAPTION);
        fc = wxSystemSettings::GetColour(wxSYS_COLOUR_CAPTIONTEXT);
    }

    SetBackgroundColour(bc);
    ClearBackground();
    m_text->SetBackgroundColour(bc);
    m_text->SetForegroundColour(fc);
    m_text->ClearBackground();
    m_text->Refresh();
}

void TitleBarControl::SetText(const wxString &text)
{
    m_text->SetLabel(text);
}

wxString TitleBarControl::GetText()
{
    return m_text->GetLabel();
}

void TitleBarControl::OnTextMouse(wxMouseEvent& evt)
{
    SetActive(true);
}

void TitleBarControl::OnLeftDown(wxMouseEvent& evt)
{

    SetFocus();

    if( !m_ggmgr->IsEditMode() ) return;

    wxWindow* swin = NULL;

    wxObject* evobj = evt.GetEventObject();
    if(evobj)
        swin = dynamic_cast<wxWindow*>(evobj);


    if(!swin) swin = this;

    swin->CaptureMouse();

    wxPoint pos = swin->ClientToScreen(evt.GetPosition());

    wxPoint origin = GetParent()->GetPosition();
    int dx =  pos.x - origin.x;
    int dy = pos.y - origin.y;
    m_delta = wxPoint(dx, dy);


}

void TitleBarControl::OnMouseMove(wxMouseEvent& evt)
{
    if( !m_ggmgr->IsEditMode() ) return;

    if(m_delta == wxDefaultPosition) {
        wxWindow* gp = GetGrandParent();
        if(gp) gp->AddPendingEvent(evt);
        return;
    }

    wxWindow* swin = NULL;
    wxObject* evobj = evt.GetEventObject();
    if(evobj)
        swin = dynamic_cast<wxWindow*>(evobj);

    if(!swin) swin = this;

    if (evt.Dragging() && evt.LeftIsDown()) {

        wxPoint pt = evt.GetPosition();

        wxPoint pos = swin->ClientToScreen(pt);
        GetParent()->Move(wxPoint(pos.x - m_delta.x, pos.y - m_delta.y));

        if(m_winTip) m_winTip->ShowMsg(wxString::Format(_T("xpos=%i\nypos=%i"),GetParent()->GetPosition().x,GetParent()->GetPosition().y));

    }
}

void TitleBarControl::OnLeftUp(wxMouseEvent& evt)
{
    SetFocus();

    if( !m_ggmgr->IsEditMode() ) return;

    if(m_delta == wxDefaultPosition) {
        wxWindow* gp = GetGrandParent();
        if(gp) gp->AddPendingEvent(evt);
        return;
    }

    wxWindow* swin = NULL;
    wxObject* evobj = evt.GetEventObject();
    if(evobj)
        swin = dynamic_cast<wxWindow*>(evobj);

    if(!swin) swin = this;

    if (swin->HasCapture())
    {
        swin->ReleaseMouse();
    }

    m_delta = wxDefaultPosition;
}

