#include "VemsFlagWidget.h"

#include <wx/menu.h>
#include <wx/msgdlg.h>
#include <wx/dcbuffer.h>
#include <wx/filename.h>
#include <wx/html/htmlwin.h>

#include "../events/UpdateViewerEvent.h"
#include "../logger.h"
#include "../DirHelper.h"
#include "../wxVariantHelper.h"
#include "../wxSizeEx.h"
#include "GaugeGroupManager.h"
#include "../LayoutUpdaterBase.h"
#include "../common/MenuExtendsHandlerBase.h"
#include "../common/wxHtmlHelpTools.h"

#include "../appHelper.h"

int idQuit = wxNewId();

BEGIN_EVENT_TABLE(VemsFlagWidget, VemsWidgetBase)

    EVT_RIGHT_DOWN(VemsFlagWidget::OnMouseRightClick)

#if !defined (__WINCE_STANDARDSDK__)
    EVT_LEFT_DCLICK(VemsFlagWidget::OnDoubleClick)
#endif

    EVT_PAINT(VemsFlagWidget::OnPaint)

    EVT_TIMER(idFlashTimer, VemsFlagWidget::OnFlashTimer)

    EVT_MENU_HIGHLIGHT_ALL(VemsFlagWidget::OnMenuHL)

    EVT_MENU(idMenuShowHelp,VemsFlagWidget::OnShowHelpMenu)

END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(VemsFlagWidget, VemsWidgetBase);


VemsFlagWidget::VemsFlagWidget( wxWindow* parent, wxWindowID id, GaugeDescManager * gdmgr, GaugeProperties &prop)
    :m_stateFlashTimer(this, idFlashTimer)
{
    m_prevValue = -1;
    m_currStateDesc = NULL;
    m_parentLayoutUpdaterBase = NULL;
    m_menuExtender = NULL;

    double sider = 4.0;

    m_descMgr = gdmgr->GetVemsFlagDescMgr();


    m_prevSymbolName  = wxEmptyString;
    m_isSymbolChanged = true;
    m_isMouseDown   = false;

    //LOGP(_T("m_currSymbolName = ")+m_currSymbolName);

    m_isCloseMenu = prop.extraParams.find(_T("isCloseMenu")) == prop.extraParams.end() ? false : prop.extraParams[_T("isCloseMenu")].GetBool();
    m_symbolSaver = NULL;//manager;

//    if(prop.extraParams.find(_T("isCloseMenu")))


    m_props = prop;

    VemsFlagDesc& desc = m_descMgr->GetVemsFlagDesc( GetCurrSymbolName() );

    wxSize gSize;

    gSize.y = (int)(prop.width/sider);

    if(desc.Properties.find(_T("autosize")) != desc.Properties.end() && desc.Properties[_T("autosize")].GetLong()) {

        prop.width = GetMaxWidth( CalcFontSize(gSize.y) );

        Logger::logf(LOG_MSG,_T("prop.width=%i"),prop.width);
    }

    gSize.x = prop.width;


    prop.setSize(gSize);
	prop.extraParams[_T("windowStyle")] = (long)wxBORDER_SUNKEN;

    CreateWidget(parent, id, gdmgr, prop);
    m_hasOwnDescriptorMgr = true;

    m_sideRatioRestriction = sider;

    m_isSymbolMenu = false;
    m_isGaugeDescMenu = false;
    m_isThemeMenu = false;

    //createMenu();


}

VemsFlagWidget::~VemsFlagWidget()
{
    //Logger::logf( LOG_MSG, _T("VemsFlagWidget::Desctructor()") );
    //UnRegisterViewer();
}

int VemsFlagWidget::GetMaxWidth(int fontsize)
{
    wxMemoryDC dc;

    wxFont font = dc.GetFont();
    font.SetPointSize(fontsize);
    dc.SetFont(font);

    VemsFlagStates& states = m_descMgr->GetVemsFlagDescStates( GetCurrSymbolName() );

    int maxsize = -1;

    for(VemsFlagStates::iterator it = states.begin(); it != states.end(); it++ ) {
        wxSize s = dc.GetTextExtent(it->second.GetText());
        if( maxsize < s.x ) maxsize = s.x;
    }

    maxsize += 6;

    return maxsize;
}

void VemsFlagWidget::OnPaint(wxPaintEvent& evt)
{
    wxBufferedPaintDC dc(this);

    if(!m_currStateDesc) return;

    wxSize size = GetClientSize();

    int fs = CalcFontSize(size.y);

    bool isColor2 = false;

    if(m_currStateDesc->IsFlash() && m_currFlashState == 1) {
        isColor2 = true;
    }

    wxColor fgcolor = isColor2 ? m_currStateDesc->GetForegroundColor2() : m_currStateDesc->GetForegroundColor();
    wxColor bgcolor = isColor2 ? m_currStateDesc->GetBackgroundColor2() : m_currStateDesc->GetBackgroundColor();

    dc.SetBackground(wxBrush(bgcolor));

    dc.Clear();

    dc.SetTextForeground(fgcolor);

    wxString text = m_currStateDesc->GetText();

    wxFont font = dc.GetFont();
    font.SetPointSize(fs);
    dc.SetFont(font);

    wxSize bb = dc.GetTextExtent(text);

    dc.DrawText(text, (size.x-bb.x)/2 , (size.y-bb.y)/2 );

    DrawExtraLayer(dc);
}

int VemsFlagWidget::CalcFontSize(int height)
{
    //return (int)((width*0.16)-3.2);
    return (int)((height*0.85)-3.2);
}

void VemsFlagWidget::OnSize(wxSizeEvent& evt)
{

    Refresh(false);

    evt.Skip();
}

void VemsFlagWidget::InitGauge()
{
    SetValue(0);
    Connect( wxEVT_SIZE, wxSizeEventHandler(VemsFlagWidget::OnSize) );
}

void VemsFlagWidget::SetValue(double value)
{
    //Logger::logf(LOG_MSG,_T("VemsFlagWidget::SetValue name=")+GetSelectedSymbol());

    if(isDataAvailable()) {

    int tmp = (int)value;

        //Logger::logf(LOG_MSG,_T("isDataAvailable()=%i"),isDataAvailable());

    if(m_prevValue == tmp && tmp != -1 ) return;

    if(tmp == -1) tmp = 0;

    m_prevValue = tmp;


    if( tmp == -1 ) {
        if(m_isErrorSam0) {
            m_isErrorSam0 = false;
            wxMessageBox(_T("Not found flag expression for \"") + GetSelectedSymbol() + _T("\" !\nValue = -1"),_T("Error!"),wxICON_ERROR | wxOK );
        }

        return;
    }

    VemsFlagStates& states = m_descMgr->GetVemsFlagDescStates( GetCurrSymbolName() );

   // Logger::logf(LOG_MSG,_T("name=")+m_currSymbolName+_T(", tmp=%0.1f"),tmp);

    if(states.empty() || states.find(tmp) == states.end() ) {
//        if(m_isErrorSam1) {
//            m_isErrorSam1 = false;
//            //wxMessageBox(_T("Not found descriptor"),_T("Error!"),wxICON_ERROR | wxOK );
//        }
        LOGP(_T("Not found descriptor"));
        m_currStateDesc = &DefaultVemsFlagStateDesc;
    } else {
        m_currStateDesc = &(states[tmp]);
    }

    } else {
        m_currStateDesc = &DisabledVemsFlagStateDesc;
    }

    if(m_currStateDesc->IsFlash()) {
        m_stateFlashTimer.Start(m_currStateDesc->GetFlashInterval());
    }

    Refresh(false);
}

wxString VemsFlagWidget::GetSelectedSymbol()
{
    return GetCurrSymbolName();
}

void VemsFlagWidget::SetValueIdx(int idx)
{
    //Logger::logf(LOG_MSG,_T("VemsFlagWidget::SetValueIdx(%i)"),idx);
}

wxRegion VemsFlagWidget::GetShape()
{
    return wxRegion(0,0,GetSize().x,GetSize().y);
}


void VemsFlagWidget::createMenu(wxMenu* parentMenu)
{

    wxMenu* selectMenu = new wxMenu();

    VemsFlagDescArray descarr;

    m_descMgr->GetAllDesc(descarr);

    descarr.Sort( &VemsFlagDescManager::flag_compare_function_title );

    wxVariantHashArray& flagGroups = m_descMgr->GetGroups();

// add group titles to menu
    for(size_t a=0;a<flagGroups.GetCount();a++){
        wxVariantHash& group = flagGroups[a];
        selectMenu->AppendSubMenu(new wxMenu(), group[_T("title")].GetString());
    }

    int cid=0;
// add flag symbols to propered menu
    for(size_t a=0; a<descarr.GetCount();a++){

        VemsFlagDesc& desc = descarr[a];

        wxString grptitle = desc.Properties[_T("group_title")].GetString();

        int id = wxNewId();

        wxMenu* gmenu = selectMenu->FindItem(selectMenu->FindItem(grptitle))->GetSubMenu();
        if(!gmenu) continue;

        if(gmenu->GetMenuItemCount() % 20 == 0) gmenu->Break();

        wxString sname = desc.Properties[_T("name")].GetString();

        gmenu->AppendCheckItem( id, desc.Properties[_T("title")].GetString(), sname );
        Connect( id, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(VemsFlagWidget::OnPopupMenuSelectSymbol),NULL, this );

        if(GetCurrSymbolName() == sname) cid = id;

        m_menuId2FlagId[id] = desc.Properties[_T("id")].GetLong();


    }

    selectMenu->Check( cid, true );

    parentMenu->AppendSubMenu(selectMenu,_T("Select symbol"));


    if( GetItemID() != -1 )
    {
        parentMenu->SetTitle( m_props.groupName );
    }

    if( m_menuExtender ) m_menuExtender->ExtendMenu(parentMenu, this);

    VemsFlagDesc& desc = m_descMgr->GetVemsFlagDesc(GetCurrSymbolName());

    if(desc.Properties[_T("help_file")] != wxNullVariant) {
        if(wxFileName::FileExists( DirHelper::GetPath(dHTMLHELP) + desc.Properties[_T("help_file")].GetString())) {
            parentMenu->AppendSeparator();
            parentMenu->Append(idMenuShowHelp,_T("Show help"));
}
    }

}

void VemsFlagWidget::OnShowHelpMenu(wxCommandEvent& evt)
{
    VemsFlagDesc& desc = m_descMgr->GetVemsFlagDesc(GetCurrSymbolName());

    wxString filename = DirHelper::GetPath(dHTMLHELP) + desc.Properties[_T("help_file")].GetString();

    if(!wxFileName::FileExists( filename ) ) return;

    Logger::logf(LOG_MSG,_T("filename=")+filename);

    wxFrame * frame = wxHtmlHelpTools::ShowFileHelpFrame(filename);

    frame->CentreOnScreen();
    frame->Show();
}

void VemsFlagWidget::OnMenuHL(wxMenuEvent& evt)
{
    LOG
    evt.Skip();
}

void VemsFlagWidget::OnPopupMenuSelectSymbol(wxCommandEvent& event)
{
    wxSize oldSize = GetSize();

    VemsFlagDesc& last_desc = m_descMgr->GetVemsFlagDesc( GetCurrSymbolName() );

    int id = event.GetId();

    if(m_menuId2FlagId.find(id) == m_menuId2FlagId.end()) return;

    int newFlagId = m_menuId2FlagId[id];

    wxString currSymbolName = m_descMgr->GetVemsFlagDesc(newFlagId).Properties[_T("name")].GetString();

    SetCurrSymbolName( currSymbolName );

    if( currSymbolName != m_prevSymbolName ) { m_isSymbolChanged = true; }

    requireChanged();

    m_prevSymbolName = currSymbolName;

    m_prevValue = -1;
    SetValue( 0 );

    if(m_symbolSaver)
        m_symbolSaver->SaveSymbol( GetItemID(), currSymbolName );

    VemsFlagDesc& desc = m_descMgr->GetVemsFlagDesc( currSymbolName );

    if( (desc.Properties.find(_T("autosize")) != desc.Properties.end() && desc.Properties[_T("autosize")].GetLong()) ||
        (last_desc.Properties.find(_T("autosize")) != last_desc.Properties.end() && last_desc.Properties[_T("autosize")].GetLong()) ) {

        wxSize gSize;

        gSize.y = m_props.height;

        m_props.width = GetMaxWidth( CalcFontSize(gSize.y) );

        gSize.x = m_props.width;

        m_props.setSize(gSize);

        SetSize(gSize);


        if(m_parentLayoutUpdaterBase) {
            wxVariantHash params;
            wxSizeEx diff = oldSize - gSize;
            params[_T("size_diff")] << diff;
            params[_T("sender_id")] = (long)GetId();

            Logger::logf(LOG_MSG,_T("this=%p"),(wxControl*)this);

            m_parentLayoutUpdaterBase->UpdateLayout(params);

}
    }

}

void VemsFlagWidget::OnMouseRightClick( wxMouseEvent& event )
{
    if( m_groupManager->IsEditMode() ) { event.Skip(); return; }

    //SetSize( GetSize().x-10, GetSize().y );

    wxMenu menu;

    createMenu(&menu);

    PopupMenu( &menu );
}


void VemsFlagWidget::OnDoubleClick( wxMouseEvent& event )
{
    m_parent->ProcessEvent( event );
}

void VemsFlagWidget::OnFlashTimer(wxTimerEvent& evt)
{
    if(!m_currStateDesc->IsFlash()) {
        m_stateFlashTimer.Stop();
        return;
    }

    m_currFlashState = m_currFlashState ? 0 : 1;

    Refresh(false);
}
