#ifndef VEMSWIDGETBASE_H
#define VEMSWIDGETBASE_H

#include "GroupMember.h"
#include "../gaugedesc/GaugeDescManager.h"
#include "../gaugedesc/GaugeProperties.h"
#include "../rtdatamanager/RTDataViewerBase.h"
#include "../common/typedef.h"
#include "../common/types/Int2IntHash.h"
#include "../common/types/Int2StringHash.h"
#include "SelectableWidgetBase.h"

#include "vemsgaugedesc.h"

#include "../Symbol.h"
//#include <wx/keybinder.h>

#define sq_Int(w,h) (w < h ? w : h)
#define sq_wxPoint(p) wxSize((p.x < p.y ? p.x : p.y),(p.x < p.y ? p.x : p.y))

#define chkminmax(val,min,max) \
    if( (val) > (max) ) (val) = (max); \
    if( (val) < (min) ) (val) = (min);

#include "VemsControlBase.h"

#define DefaultSideRaitoRestriction -1.0

class GaugeGroupManager;
class UpdateViewerEvent;
class VemsWidgetBaseEditorBase;
class GaugeValueSelector;
class WindowTipper;
class DuplicateMenuItemDesc;

WX_DECLARE_HASH_MAP(int, wxString, wxIntegerHash, wxIntegerEqual, IntToSymbolNameHash );

WX_DECLARE_OBJARRAY(DuplicateMenuItemDesc, DuplicateMenuItemDescArray);

class VemsWidgetBase: public wxControl, public RTDataViewerBase, public GroupMember, public SelectableWidgetBase, public VemsControlBase
{
    DECLARE_ABSTRACT_CLASS(VemsWidgetBase);

    public:
        VemsWidgetBase();
        VemsWidgetBase(wxWindow* parent, wxWindowID id, GaugeDescManager * gaugemgr, GaugeProperties &prop );

        virtual ~VemsWidgetBase();

		void SetNewTheme(WidgetThemeTable *theme){};

        void CreateWidget(wxWindow* parent, wxWindowID id, GaugeDescManager * gaugemgr, GaugeProperties &prop );


        virtual void UpdateSize() = 0;
        virtual void setSize(int x , int y) = 0;

		virtual void SetValueIdx(int idx);

        virtual void require();
        virtual void unregisterSymbols();
		virtual void refresh(DoubleHash* table);

        void SetGaugeBySymbolName(const wxString symbolName);
        void SetGaugeDescByName(const wxString& descName);

        virtual void b_SetFocus() { GetTargetWin()->SetFocus(); };
        virtual void b_UnRegisterViewer() { UnRegisterViewer(); };
        virtual void b_SetSize(const wxSize& size) { GetTargetWin()->SetSize(size); };

		IntArray choices2gaugeDescA;

        wxString GetSelectedGaugeDesc();
        void SetDefaultGaugeDesc(size_t id);
        void SetGaugeDescA(const VEMSGaugeDescArray* gaugeDescA);

        VEMSGaugeDesc*  m_gd;

        size_t          m_defGaugeId;

        static int compare_function(Symbol **A,Symbol **B);

        int SrcGaugeDescBySymbolName( const wxString &name );

        bool isDefaultGaugeDesc();

        void setDescEditor( VemsWidgetBaseEditorBase * obj ) { m_gaugeDescEditor = obj; };

        void ShowDescEditor(VEMSGaugeDesc* gd = NULL);

        wxString GetSymbolTitle();

        Symbol * GetCurrSymbol();

        virtual void ResetMinMax();

        GaugeDescManager * GetGaugeDescMgr() { return m_gaugeDescMgr; };

        void SetFloating( bool isfloat ) { m_floating = isfloat; };
        bool IsFloating() { return m_floating; };

        void InitTheme(GaugeProperties &prop);

        static wxMenu* CreateDuplicateMenu(DuplicateMenuItemDescArray &items, wxEvtHandler * target, wxObjectEventFunction func);

        virtual void InitGauge() {};
        virtual void RedrawWidget();

        int GetGaugeType() { return m_props.gaugeType; };

        virtual void ProcessWidgetCommand(wxVariantHash& cmdHash);

        void MoveTarget(const wxPoint& delta);

    protected:

        //int m_gaugeType;

        WindowTipper * m_winTip;

        wxWindow * GetTargetWin();

        bool            m_floating;

        GaugeDescManager * m_gaugeDescMgr;


        uint8_t     m_graphQual;
        bool        m_isExtendedResize;
        bool        m_isMouseResize;
        bool        m_isGaugeDescMenu;
        bool        m_isSymbolMenu;
        bool        m_isThemeMenu;

        wxString& GetCurrSymbolName() { return m_props.symbolName; };
        void SetCurrSymbolName(const wxString& name) { m_props.symbolName = name; };


        //wxString    m_currSymbolName;
        SymbolGaugeDescArray m_symbolDescArr;

        wxArrayString   choices;


        double m_valueMax;
        double m_valueMin;

        bool m_isValueMaxChanged;
        bool m_isValueMinChanged;


        void OnGaugeDescMod(wxCommandEvent& event);
		void OnMouseWheel(wxMouseEvent& event);
		void OnLeftDown(wxMouseEvent& evt);
		void OnMouseMove(wxMouseEvent& evt);
		void OnLeftUp(wxMouseEvent& evt);
		void OnDoubleClick(wxMouseEvent& evt);
		void OnMouseRightClick(wxMouseEvent& event);
        void OnPopupMenuSelectSymbol(wxCommandEvent& event);
        void OnPopupMenuSelectGauge(wxCommandEvent& event);
        void OnQuit(wxCommandEvent& event);
        void OnSetLayout(wxCommandEvent& event);
        void OnAllMouseEvents(wxMouseEvent& evt);
        void OnChar(wxKeyEvent& event);
        void OnUpdateValue( UpdateViewerEvent& event );
        void OnThemeSelect(wxCommandEvent& event);
        void OnSizeBase(wxSizeEvent& evt);
        static int MenuListCompFunc( const void *arg1, const void *arg2);



        int LoadSettingsBase(wxVariantHash &datas);
        int SaveSettingsBase(wxVariantHash &datas);
        bool isDataAvailable() const { return m_isDataAvailable; }

        double m_sideRatioRestriction;

        void OnDuplicateMenu(wxCommandEvent& event);
        void DulpicateWidget(const DuplicateMenuItemDesc& desc);

        virtual void createWidgetMenu( wxMenu& popupMenu ) {}

        void DrawExtraLayer(wxDC& dc);

        wxBitmap*   m_extraLayerBmp;
        wxWindow*   m_parent;

        wxString    m_prevSymbolName;

        bool        m_isSymbolChanged;
        bool        m_hasOwnDescriptorMgr;

    private:

        bool        m_isDataAvailable;
        wxString    m_name;


        wxPoint     m_beginMousePoint;
		wxPoint     m_delta;



        const VEMSGaugeDescArray*   m_gaugeDescA;
        VemsWidgetBaseEditorBase*   m_gaugeDescEditor;
        GaugeValueSelector*         vselector;

        Int2IntHash     m_menuId2GaugeId;

        Int2StringHash  m_menuIDtoSymbolName;

        IntToSymbolNameHash m_menuId2SymbolName;

        DuplicateMenuItemDescArray m_dMenuItems;

        void createSymbolMenu( wxMenu& popupMenu );
        void createGaugeDescMenu( wxMenu& popupMenu );
        void createBasicMenu( wxMenu& popupMenu );
        void createEditMenu( wxMenu& popupMenu );


        DECLARE_EVENT_TABLE();
};

class DuplicateMenuItemDesc
{
    public:
        DuplicateMenuItemDesc() {};
        DuplicateMenuItemDesc(int id, int xrep, int yrep, const wxString& pngfile, const wxString& text);
        wxMenuItem* Init(wxMenu * parentmenu);

        wxPoint RePos(const wxPoint& oripos, const wxSize &size) const;

        int GetId() const { return Id; }

    private:

        int Id;
        int XRep;
        int YRep;

        wxString m_pngPath;
        wxString m_text;


};



#endif // VEMSWIDGETBASE_H
