#include "VemsWidgetBaseFactory.h"

#include "ArcBarGauge.h"
#include "VemsGaugeDigital.h"
#include "VemsTextWidget.h"
#include "VemsGraphWidgetAgg.h"
#include "VemsFlagWidget.h"
#include "VemsGaugeAgg.h"
#include "MultiGraph/MultiValueGraph.h"
#include "DifferentialGauge.h"
#include "GPSCompassGauge.h"
#include "GPSRouteGauge.h"
#include "VemsExpressionGridWidget.h"
#include "VemsGaugeDescDialog.h"
#include "GaugeType.h"
#include "GaugeCarrierBaseDesc.h"
#include "VemsWidgetBase.h"
#include "GaugeGroupManager.h"
#include "ColumnsGauge.h"

#include "../gaugedesc/GaugeDescManager.h"
#include "../stdatamanager/STDataMgrBase.h"
#include "../logger.h"
//#include "VemsControlBase.h"


IMPLEMENT_DYNAMIC_CLASS(VemsWidgetBaseFactory, WidgetFactoryBase);


VemsWidgetBaseFactory::VemsWidgetBaseFactory()
{
    //ctor
    m_gdMgr = NULL;
    m_groupMgr = NULL;
    m_stController = NULL;
    m_rtController = NULL;
    m_logHistory = NULL;
}

VemsWidgetBaseFactory::~VemsWidgetBaseFactory()
{
    //dtor
}


VemsControlBase* VemsWidgetBaseFactory::CreateWidget(GaugeProperties &prop, GaugeCarrierBaseDesc* desc)
{

    if(!m_groupMgr) {
        Logger::logf(LOG_MSG,_T("GaugeCarrierBase::AddGauge m_groupMgr is NULL!! getting from GaugeDescManager"));
        m_groupMgr = m_gdMgr->GetGaugeGrpMgr();
    }

    if( !m_logHistory ) m_logHistory = m_groupMgr->GetLogHistory();

    if(m_rtController == NULL) return NULL;

    VemsWidgetBase *gauge = NULL;

    int id = wxNewId();

    prop.winID = id;

    if( prop.extraParams.find(_T("showBorder")) != prop.extraParams.end() )
        if(!prop.extraParams[_T("showBorder")].GetBool() )
            prop.extraParams[_T("windowStyle")] = (long)wxNO_BORDER;

    switch( prop.gaugeType ) {
        case GaugeType::ROUND:
            gauge = new VemsGaugeAgg(desc->m_window, id, m_gdMgr, prop );
            desc->m_type = prop.gaugeType;
            break;

        case GaugeType::ARCBAR:
            gauge = new ArcBarGauge(desc->m_window, id, m_gdMgr, prop);
            break;

        case GaugeType::DIGITAL:
            gauge = new VemsGaugeDigital(desc->m_window, id, m_gdMgr, prop );
            break;

        case GaugeType::TEXT:
            prop.extraParams[_T("windowStyle")] = (long)wxBORDER_SUNKEN;
            gauge = new VemsTextWidget(desc->m_window, id, m_gdMgr, prop );
            break;

        case GaugeType::GRAPH:
            gauge = new VemsGraphWidgetAgg( desc->m_window, id, m_gdMgr, prop );
            break;

        case GaugeType::MULTIGRAPH:
            gauge = new MultiValueGraph( desc->m_window, id, m_gdMgr, prop, m_logHistory );
            break;

        case GaugeType::DIFFER:
            gauge = new DifferentialGauge( desc->m_window, id, m_gdMgr, prop );
            break;

        case GaugeType::EXPRESSION_GRID:
		{
            if(m_stController == NULL) return NULL;
            VemsExpressionGridWidget* widget = new VemsExpressionGridWidget( desc->m_window, id, m_gdMgr, prop );
            m_stController->RegisterViewer( widget );
            widget->update();
            gauge = widget;
		} break;

        case GaugeType::GPSCOMPASS:
            gauge = new GPSCompassGauge( desc->m_window, id, m_gdMgr, prop );
            break;

        case GaugeType::GPSROUTE:
            gauge = new GPSRouteGauge( desc->m_window, id, m_gdMgr, prop, m_logHistory );
            break;

        case GaugeType::FLAG:
            prop.extraParams[_T("windowStyle")] = (long)wxBORDER_SUNKEN;
            gauge = new VemsFlagWidget( desc->m_window, id, m_gdMgr, prop );
            ((VemsFlagWidget*)gauge)->SetSymbolSaver( m_groupMgr );
            break;

        case GaugeType::COLSGAUGE:
            gauge = new ColumnsGauge(desc->m_window, id, m_gdMgr, prop);
            break;

        default:
            Logger::logf(LOG_MSG,_T("VemsWidgetBaseFactory::CreateWidget wrong gaugetype!"));
            return NULL;
    }


    if(gauge != NULL){

        gauge->SetItemID( prop.itemID );

        gauge->SetFloating( !desc->m_floating );

//        Logger::logf(LOG_MSG,_T("gauge->IsFloating()=%i"),gauge->IsFloating());

        desc->m_IDtoControls[id] = gauge;
        //Logger::logf(LOG_MSG,_T("GaugeCarrierBase::AddGauge id=%i"),id);

        //Logger::logf(LOG_MSG,_T("desc->m_IDtoControls[%i]=%p"),id,.desc->m_IDtoControls[id]);
//        if(desc->m_CustomControlHash != NULL) (*desc->m_CustomControlHash)[id] = gauge;


        gauge->SetHotKeyStr( prop.groupHotkey );

        gauge->setDescEditor(new VemsWidgetBaseEditor(gauge));

        if( prop.extraParams.find(_T("showBorder")) != prop.extraParams.end() )
            if(!prop.extraParams[_T("showBorder")].GetBool())
                gauge->SetWindowStyle(wxNO_BORDER);

        gauge->SetGaugeBySymbolName( prop.symbolName );

        gauge->SetGaugeDescByName( prop.descName );

        m_rtController->RegisterViewer(gauge);
        gauge->SetDataMgr(m_rtController);

        gauge->UpdateSize();

        wxRegion regiontemp = gauge->GetShape();

        gauge->Show(desc->m_onlyOneGauge | desc->m_GaugeShowInit);

        if(desc->m_onlyOneGauge){

            int x,y,w,h;
            regiontemp.GetBox(x,y,w,h);
            desc->m_window->SetSize(w,h);

            wxRegion tmp3 = gauge->GetShape();
            tmp3.Offset(2,2);
            if(desc->m_frame) desc->m_frame->SetShape(tmp3);


        } else {
            regiontemp.Offset( prop.getPos() );
            desc->m_region.Union( regiontemp );
            if(desc->m_frame) desc->m_frame->SetShape(desc->m_region);
        }

        gauge->InitGauge();

    }

    return gauge;

}
