#ifndef BOOTMODEBASE_H
#define BOOTMODEBASE_H


#include "../pageexception.h"
#include "../Observable.h"
//#include "../ObserverBase.h"
#include "FirmwareData.h"
#include "ECUInfo.h"

class BootModeBase;
class CommunicationBase;

typedef int (BootModeBase::*pageHandleFV) (uint32_t);

class BootModeBase : public Observable
{
    public:
        BootModeBase(CommunicationBase* commBase);
        virtual ~BootModeBase();


        void setFirmwaredata(FirmwareData * fwData){
            m_firmwareData = fwData;

            /// V3 Marker word hack
            if(m_device == ECUInfo::DEVICE_V3 ){
                m_firmwareData->setV3MarkerWord();
            }
        }

        virtual int       verifyPage(uint32_t idx) = 0;
        virtual int       verifyFlash() = 0;
        virtual int       uploadPage(uint32_t idx) = 0;
        virtual int       uploadFlash() = 0;

        bool              checkFirmwareType();
        ECUInfo::Device   getDevice(){ return m_device;}
        //virtual uint32_t  getPageCheckSum(char *) =0;

    protected:
        CommunicationBase* m_commBase;
        FirmwareData *m_firmwareData;
        FirmwareData::Type m_type;
        ECUInfo::Device m_device;
        uint8_t m_buff[256];

        int selectPage(const char* pageCmd, uint32_t pageID );
        int verifyPageCommon(const char* cmd, uint32_t pageIdx );
        int uploadPageCommon(const char* cmd, uint32_t pageIdx);
        int verifyFlashCommon();
        int uploadFlashCommon(uint32_t maxPageIdx);

        bool    failSafePageHandle(uint32_t idx, pageHandleFV p);

        virtual bool    failSafeVerifyPage(uint32_t idx) = 0;
        virtual bool    failSafeUploadPage(uint32_t idx) = 0;
        virtual uint32_t getPageID(uint32_t pageIdx) = 0;

        virtual bool checksumVerify(uint32_t ecu_checksum, uint32_t file_checksum );

        void observerVerifyInit();
        void observerUploadInit();


    private:



};



#endif // BOOTMODEBASE_H
