#include "BootModeV3.h"

#include <wx/thread.h>

#include "../logger.h"
#include "../CommunicationBase.h"


NTCFinder BootModeV3::m_ntcfinder;// = NTCFinder();

/*
TODO: replace VemsProtocol -> CommunicationBase
send_AddressW -> sendNumber
sendStringBlock -> sendStringBlock
*/
/*
START:
    action: sendErase:
    state: WAIT_ERASE_FINISHED

WAIT_ERASE_FINISHED:


START_PAGE_WRITE
    action: send Address
    state: WAIT_PAGE_ADDRESS_OK

WAIT_PAGE_ADDRESS_OK:
    action: send pageDate
    state: WAIT_PAGE_DATA_CHECKSUM_OK
WAIT_PAGE_DATA_CHECKSUM_OK:
    exit PAGE_WRITE

TIMEOUT:
    WAIT_PAGE_ADDRESS_OK: (not received)
    WAIT_PAGE_DATA_CHECKSUM_OK:
    exit PAGE_WRITE


*/

BootModeV3::BootModeV3(CommunicationBase*  commBase)
:BootModeBase(commBase)
{
    //ctor
    m_type = FirmwareData::HEX;
    m_device = ECUInfo::DEVICE_V3;
}

BootModeV3::~BootModeV3()
{
    //dtor
}

uint32_t BootModeV3::getPageID(uint32_t pageIdx)
{
    uint32_t addressW = m_firmwareData->getPageAddressW(pageIdx);  ///!! WORD ADDRESS
    return addressW;
}

int BootModeV3::selectPageTMP(uint32_t idx)
{
    selectPage("A", idx);
    return 0;
}

int BootModeV3::verifyPage(uint32_t idx)
{
    selectPageTMP(idx);
    return verifyPageCommon("Dll", idx);
}

int BootModeV3::uploadPage(uint32_t idx)
{
    selectPageTMP(idx);
    return uploadPageCommon("Dkk", idx);
}

int BootModeV3::verifyFlash()
{
    if(! m_ntcfinder.isOk ){
        m_ntcfinder.getFileArrray(NTC_COOLANT);
        m_ntcfinder.getFileArrray(NTC_MAT);
        m_ntcfinder.toString();
    }

    int error_cnt = verifyFlashCommon();

    // NTC ?????

    if(error_cnt == 0){
        m_commBase->SendString((uint8_t*)"E");
    }

    //int result = m_commBase->ReadString(m_buff, 255);
    return error_cnt;

}

int BootModeV3::uploadFlash()
{
    Logger::logf(LOG_MSG, _T("BootModeV3::uploadFlash() START"));
    if( eraseFlash() == -1){
         observerNotify(_T("Error detected ECU left in BOOT Mode!"));
         return -1;
    }

    /// Leave bootmode and write page 503 (due to erase)
    /// auto enter bootMode (firmware erased) and without erase command page 503 wont be destrolyed by bootloader
    size_t pagesNum = m_firmwareData->getPages();
    if(pagesNum >= 503){
        m_commBase->SendString((uint8_t*)"E");
        /// needs some wait??
        /// just for sure wait 20 ms
    }

    wxThread::Sleep(20);
    m_commBase->clear();


    int error_cnt = uploadFlashCommon(503);

    if(error_cnt == 0){
        m_commBase->SendString((uint8_t*)"E");
    }else{
          observerNotify(_T("Error detected ECU left in BOOT Mode!"));
    }
    return error_cnt;
}

bool BootModeV3::failSafeVerifyPage(uint32_t idx)
{
    return failSafePageHandle(idx, &BootModeBase::verifyPage );
}

bool BootModeV3::failSafeUploadPage(uint32_t idx)
{
    return failSafePageHandle(idx, &BootModeBase::uploadPage );
}

////////////////////////

////////////////////////
int BootModeV3::eraseFlash()
{
    //Logger::logf(LOG_MSG, _T("FirmwareMgr::eraseFlash() START"));
    int MAX_TRY=30;
    observerInit(_T("Erasing all pages"), _T(""), MAX_TRY+1);
    observerNotify(_T("Erasing all pages...(be patient)") );
    //Logger::logf(LOG_MSG, _T("FirmwareMgr::eraseFlash() START 2"));
    m_commBase->clear();
    m_commBase->SendString((uint8_t*)"e");



    int tryCount =0;
    do{
        wxThread::Sleep(100); // 100 mS
        int result1 = m_commBase->ReadString(m_buff, 10);
        if (result1 != 1 || m_buff[0] != '\r')
        {
            Logger::logf(LOG_MSG, _T("BootModeV3::eraseFlash() waiting for erase completed size=%i"), result1);
            //throw PageException(PAGE_ERASE_FAILED, 0);
            tryCount++;
        }else{
            tryCount = MAX_TRY+1;
        }
        observerNotify(tryCount, tryCount);
    }
    while(tryCount < MAX_TRY);
    observerNotify(MAX_TRY, MAX_TRY);
    if(tryCount == MAX_TRY){
        observerNotify(_T("Erase FAILED!"));
        return -1;
    }
    observerNotify(_T("Erase OK!"));
    return 0;
}

