#include "ECUState.h"

#include "ECUInfo.h"
#include "../CommunicationBase.h"
#include "../UnicodeHelper.h"
#include "../AIMParser.h"
#include "../common/typedef.h"
#include "../logger.h"


/** @brief ECUState
  *
  * @todo: document this function
  */
ECUState::ECUState( int id, const wxString& name, CommunicationBase* commBase, const wxString& command, const wxString& response, int baudRate )
:ECUStateBase( id, name, commBase, baudRate)
{
    // it is better to store in both format
    //m_nameChar = UnicodeHelper::U2C( m_name );

    m_command = command;
    m_commandChar = UnicodeHelper::U2C( m_command );

    m_response = response;
    m_responseChar = UnicodeHelper::U2C( m_response );
}

/** @brief ~ECUState
  *
  * @todo: document this function
  */
 ECUState::~ECUState()
{

}

/*virtual*/
bool ECUState::check()
{
    /*
    currBaudRate = m_commBase->getBaudRate();
    if( !isValidBaudRate(currBaudRate) ){
        m_commBase->setBaudRate( m_baudRate );
    }
*/

    //Logger::logf(LOG_MSG,_T(" ECUState::check()1"));
    size_t expLen = strlen( m_responseChar );

    int tryCount = 3;
    int recLen = 0;
    do{
        m_commBase->clear();
        m_commBase->SendString( (const uint8_t*)m_commandChar );
#ifdef SFILE_LIB
		wxMilliSleep((expLen+strlen((const char*)m_commandChar)*20*(4-tryCount))*10*1000/(m_commBase->getBaudRate())); // len*10bit*msec/baudrate
#endif
        recLen = m_commBase->ReadString( m_buff, expLen+2 );
        //Logger::logf(LOG_MSG,_T(" ECUState::check() command: %s"),m_commandChar);
        //Logger::logf(LOG_MSG,_T(" ECUState::check() expected: %s"),m_responseChar);
        //Logger::logf(LOG_MSG,_T(" ECUState::check() resp: %s"), m_buff);
        //Logger::logf(LOG_MSG, _T("ECUState::check() ")+ m_name +_T(" (reclen, explen): (%i,%i)"), recLen, expLen);
        tryCount--;
    }while(tryCount && (recLen < (int)expLen) );
    if( recLen < (int)expLen ){
        return false;
    }
    //Logger::logf(LOG_MSG,_T(" ECUState::check()3"));
    if( strncmp( (const char*)m_buff, m_responseChar, expLen ) == 0 ){
		return true;
    }
    return false;
}

/*virtual*/
/*
void ECUState::goToState( int state )
{
    //Logger::logf( LOG_MSG, _T("ECUState::goToState - START") );
    if( m_changeStateH.find( state ) == m_changeStateH.end() )
    {
        return;
    }

    //m_commBase->setBaudRate( m_baudRate );
    m_commBase->SendString( m_changeStateH[ state ] );
    //Logger::logf( LOG_MSG, _T("ECUState::goToState - END") );
}
*/

ECUStateAIM::ECUStateAIM( CommunicationBase* commBase, int baudRate )
:ECUStateBase( ECUInfo::AIM, _T("Aim"), commBase, baudRate)
{
    //ctor
}

ECUStateAIM::~ECUStateAIM( )
{

}

/*virtual*/
bool ECUStateAIM::check()
{
/*
    m_commBase->setBaudRate( m_baudRate );

*/
//    Logger::logf( LOG_MSG, _T("ECUStateAIM::check - START") );

    m_commBase->clear();
    const size_t bufferSize = 10;
	uint8_t buffer[ bufferSize ];

    AIMParser parser;
    int tryCount = 0;
    do{
        m_commBase->ReadString( buffer, bufferSize );
        for( size_t i = 0; i < bufferSize; ++i )
        {
            if( parser.feed( buffer[ i ] )  == AIMParser::END )
            {
                return true;
            }
        }
        tryCount++;
    }while(tryCount < 5);

    return false;
}

/*virtual*/
/*
void ECUStateAIM::goToState( int state )
{
    //Logger::logf( LOG_MSG, _T("ECUStateAIM::goToState - START") );
    if( m_changeStateH.find( state ) == m_changeStateH.end() )
    {
        return;
    }

//    m_commBase->setBaudRate( m_baudRate );

    m_commBase->SendString( m_changeStateH[ state ] );
    //Logger::logf( LOG_MSG, _T("ECUStateAIM::goToState - END") );
}
*/
