#include "ECUStateMgrEventDtor.h"

#include <wx/event.h>

#include "../events/DeviceEvent.h"
#include "../events/ECUEvent.h"
#include "../logger.h"
#include "../SerialException.h"

ECUStateMgrEventDtor::ECUStateMgrEventDtor(ECUStateMgrBase* mgrBase, wxEvtHandler* evtHandler)
:ECUStateMgrBase(mgrBase->getCommunicationBase() )
{
    //ctor
    m_mgrBase = mgrBase;
    m_evtHandler = evtHandler;

//    m_state = -2;
//    m_device = -2;
}

ECUStateMgrEventDtor::~ECUStateMgrEventDtor()
{
    //dtor
}

void ECUStateMgrEventDtor::reset()
{
    m_mgrBase->reset();
}

int ECUStateMgrEventDtor::getDevice()
{
    return m_mgrBase->getDevice();
}

int ECUStateMgrEventDtor::getStoredState()
{
    return m_mgrBase->getStoredState();
}

int ECUStateMgrEventDtor::getState()
{
    int oldState = m_mgrBase->getStoredState();
    int oldDevice = m_mgrBase->getDevice();
    int state = -2;
    int device = ECUEvent::NOT_CONNECTED;
    try{
        state = m_mgrBase->getState();
        if(oldState != state){
            //m_rtDataMgr->sendInputEvent(INPUT_TYPE_RINGBUFFER_ECU, result);
            ECUEvent::State ecuEventID = ECUEvent::NOT_CONNECTED;
            switch(state){
                    case ECUInfo::BOOT:
                        ecuEventID = ECUEvent::BOOT_MODE;
                        break;
                    case ECUInfo::MEGATUNE:
                    case ECUInfo::MAN:
                        ecuEventID = ECUEvent::CONNECTED;
                       break;
                    case ECUInfo::AIM:
                       ecuEventID = ECUEvent::AIM_MODE;
                        break;
                    case ECUInfo::UNKNOWN:
                    default:
                        ecuEventID = ECUEvent::NOT_CONNECTED;
            }


            ECUEvent event;
            event.state = (ECUEvent::State)ecuEventID;
            m_evtHandler->AddPendingEvent(event);

            Logger::logf(LOG_MSG, _T("ECUStateMgrEventDtor::getState() send evt: ")+ event.getStateAsString() );
        }else{
            //Logger::logf(LOG_MSG, _T("ECUStateMgrEventDtor::getState() STATE NOT CHANGED: %i"), state );
        }


        device = m_mgrBase->getDevice();
        if(oldDevice != device){
            //send event;
            Logger::logf( LOG_MSG, _T("ECUStateMgrEventDtor::getState() - oldDevice: %d != device: %d"),
                            oldDevice, device );
            DeviceEvent event;
            event.device = (ECUInfo::Device) device;
            m_evtHandler->AddPendingEvent(event);
        }


    }catch(SerialException& se){

        if(state = -2){
            state = ECUInfo::UNKNOWN;
        }

        if(oldState != state){
            Logger::logf(LOG_MSG, _T("ECUStateMgrEventDtor::getState() state %i"),  state);
            state = ECUInfo::UNKNOWN;
            ECUEvent event;
            event.state = (ECUEvent::State)state;
            m_evtHandler->AddPendingEvent(event);

            device = ECUInfo::DEVICE_UNKNOWN;
            DeviceEvent deviceEvent;
            deviceEvent.device = (ECUInfo::Device) device;
            m_evtHandler->AddPendingEvent(deviceEvent);

        }
    }
    return state;
}

int ECUStateMgrEventDtor::setState( int state )
{
    return m_mgrBase->setState(state);
}

int ECUStateMgrEventDtor::setState( int fromState, int toState )
{
    return m_mgrBase->setState(fromState, toState);
}

const ECUStateBaseArray* ECUStateMgrEventDtor::getStates( int device )
{
    return m_mgrBase->getStates( device );
}

void ECUStateMgrEventDtor::setBaudRate( int baudRate )
{
    m_mgrBase->setBaudRate( baudRate );
}

int ECUStateMgrEventDtor::getBaudRate()
{
    return m_mgrBase->getBaudRate();
}
