#include "ECUStateMgrFirmwareInfoDtor.h"

#include "ECUInfo.h"
#include "../FirmwareInfo.h"
#include "../MTConfigHelper.h"
#include "../logger.h"


ECUStateMgrFirmwareInfoDtor::ECUStateMgrFirmwareInfoDtor(ECUStateMgrBase* mgrBase,
    FirmwareInfo* fwInfo)
:ECUStateMgrBase(mgrBase->getCommunicationBase() )
{
    //ctor
    m_mgrBase = mgrBase;
    m_fwInfo = fwInfo;

    command_ctime = (uint8_t*)"Manmdv";
    command_fwversion = (uint8_t*)"ManmdV"; //31 2E 31 2E 34 39 0A 00// 8
    command_serNum = (uint8_t*)"Manmci";    //76 33 2E 33 5F 6B 30 30 30 30 37 39// 12
    command_Signature = (uint8_t*)"Q";
    command_S = (uint8_t*)"S";
}

ECUStateMgrFirmwareInfoDtor::~ECUStateMgrFirmwareInfoDtor()
{
    //dtor
}

void ECUStateMgrFirmwareInfoDtor::reset()
{
    return m_mgrBase->reset();
}

int ECUStateMgrFirmwareInfoDtor::getDevice()
{
    return m_mgrBase->getDevice();
}

int ECUStateMgrFirmwareInfoDtor::getStoredState()
{
    return m_mgrBase->getStoredState();
}

int ECUStateMgrFirmwareInfoDtor::getState()
{
    int storedState = m_mgrBase->getStoredState();
    int state = m_mgrBase->getState();
    if(  storedState == state){ return state; }

    int device = m_mgrBase->getDevice();
    switch(device){
        case ECUInfo::DEVICE_V3:
            switch(state){
                case ECUInfo::AIM:
                    Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor::sync() - AIM storedState1:%i"), m_mgrBase->getStoredState() );
                    state = m_mgrBase->setState( ECUInfo::AIM, ECUInfo::MAN );
                    //m_state = m_stateMgr->getState();
                    Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor::sync() - AIM storedState2:%i"), m_mgrBase->getStoredState() );

                    //if ( m_state == ECUInfo::MEGATUNE ) { return true; }


                case ECUInfo::MAN:
                case ECUInfo::MEGATUNE:
                case ECUInfo::SIPR:
                    fetchV3FirmwareInfo(state);
					Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor::getState() START INI PARSE"));
                    MTConfigHelper::parseMTConfigData( *m_fwInfo );
					Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor::getState() STOP INI PARSE"));
                    break;
            }
            break;
        case ECUInfo::DEVICE_ROUND:
            MTConfigHelper::parseMTConfigDataByRound();
            //fetchRoundFirmwareInfo(state);
            break;
		case ECUInfo::DEVICE_UNKNOWN:
		default:
			Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor::getState() DEVICE UNKNOWN"));
			//MTConfigHelper::parseMTConfigDataByDefaultDevice();
    }
    return state;
}

void ECUStateMgrFirmwareInfoDtor::fetchV3FirmwareInfo(int state)
{
    switch(state){
        case ECUInfo::BOOT:
        case ECUInfo::AIM:
            break;
        case ECUInfo::MEGATUNE:
            state = m_mgrBase->setState( ECUInfo::MAN );
        case ECUInfo::MAN:
            fetchV3FirmwareInfo();
            state = m_mgrBase->setState( ECUInfo::MAN, ECUInfo::MEGATUNE );
            break;
        case ECUInfo::SIPR:
             m_fwInfo->setFirmwareVersion(_T("1.1.53"));
            //state = m_mgrBase->setState( ECUInfo::MAN );
            //fetchV3FirmwareInfo();
            //state = m_mgrBase->setState( ECUInfo::SIPR);

    }
}

void ECUStateMgrFirmwareInfoDtor::fetchV3FirmwareInfo()
{
    wxString response;
    CommunicationBase* comBase = m_mgrBase->getCommunicationBase();

    comBase->getAnswer(command_ctime, 32, 20, 5, &response);
    m_fwInfo->setCompilationTime(response);
    //Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor setCompilationTime() ") +response);
    comBase->getAnswer(command_fwversion, 10, 6, 5, &response);
    m_fwInfo->setFirmwareVersion(response);
    //Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor setFirmwareVersion() ") +response);
    comBase->getAnswer(command_serNum, 12, 12, 5, &response);
    m_fwInfo->setSerialNumber(response);
    //Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor setSerialNumber() ") +response);

    setState(ECUInfo::MEGATUNE);
    comBase->getAnswer(command_Signature, 26, 20, 5, &response);
    //Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor setSignature() ") +response);
    m_fwInfo->setSignature(response);

    comBase->getAnswer(command_S, 28, 20, 5, &response);
    //Logger::logf( LOG_MSG, _T("ECUStateMgrFirmwareInfoDtor command_S() ") +response);

    m_fwInfo->setFirmwareOptions(response);

    //m_fwInfo.setTableSize(response);
    //m_fwInfo.setKpa(response);

}

int ECUStateMgrFirmwareInfoDtor::setState( int state )
{
    return m_mgrBase->setState(state);
}

int ECUStateMgrFirmwareInfoDtor::setState( int fromState, int toState )
{
    return m_mgrBase->setState(fromState, toState);
}

const ECUStateBaseArray* ECUStateMgrFirmwareInfoDtor::getStates( int device )
{
    return m_mgrBase->getStates( device );
}

void ECUStateMgrFirmwareInfoDtor::setBaudRate( int baudRate )
{
    m_mgrBase->setBaudRate( baudRate );
}

int ECUStateMgrFirmwareInfoDtor::getBaudRate()
{
    return m_mgrBase->getBaudRate();
}
