#include "ECUStateSIPR.h"

#include "SIPRTypes.h"
#include "SIPRPacket.h"
#include "ECUInfo.h"
#include "../CommunicationBase.h"
#include "../logger.h"


ECUStateSIPR::ECUStateSIPR(CommunicationBase* commBase, int baudRate, int device)
    : ECUStateBase( ECUInfo::SIPR, _T("SIPR"), commBase, baudRate)
{
    //ctor
    m_device = device;
}

ECUStateSIPR::~ECUStateSIPR()
{
    //dtor
}

bool ECUStateSIPR::check()
{
    return checkDevice();
    /*
    switch(m_device){
        case ECUInfo::DEVICE_V3:
            return sendPing( m_commBase );
        case ECUInfo::DEVICE_ARM:
            return checkDevice();
    }
    return false;
    */
}

void ECUStateSIPR::goToState( int state )
{
/*    if( m_changeStateH.find( state ) == m_changeStateH.end() )
    {
        return;
    }
*/
    if(state != ECUInfo::MEGATUNE) return;

    enterMegatune( m_commBase ); /// go to Megatune
    //Logger::logf( LOG_MSG, _T("ECUState::goToState - END") );
}

bool ECUStateSIPR::sendPing(  CommunicationBase* cb )
{
    SIPRPacket* packet  = new SIPRPacket( 0, SIPR_ACK );

    cb->SendString( packet->packet_data(), packet->packet_size() );

    // TODO - clear this
    uint8_t res = readRecord( cb );

    if ( res == SIPRParser::SIPR_STATE_CRC_OK ) { return true; }

    return false;
}

bool ECUStateSIPR::checkDevice()
{

    SIPRPacket packet ( 0, SIPR_GET_MT_SIGNATURE);
    //Logger::logf( LOG_MSG, _T("ECUState::checkDevice send:")+ packet.toString() );

    m_commBase->SendString( packet.packet_data(), packet.packet_size() );
    // TODO - clear this
    uint8_t res = readRecord( m_commBase );

    if ( res == SIPRParser::SIPR_STATE_CRC_OK ) {
        if( m_parser.type() == SIPR_GET_MT_SIGNATURE){
            Logger::logf( LOG_MSG, _T("ECUState::checkDevice SIPR_GET_MT_SIGNATURE OK"));
            const uint8_t* data = m_parser.data();
            uint8_t dataSize = m_parser.size();
            uint8_t* tmp = new  uint8_t[dataSize+1];
            tmp[dataSize] = '\n';
            memcpy(tmp, data, dataSize);
            wxString dataStr = wxString::FromAscii((const char*) tmp);
            delete[] tmp;
            Logger::logf( LOG_MSG, _T("ECUState::checkDevice SIPR_GET_MT_SIGNATURE str: ")+ dataStr);
            switch(m_device){
                case ECUInfo::DEVICE_V3:
                    if(dataStr.StartsWith(_T("VEMS v") ) ){
                        return true;
                    }
                break;
                case ECUInfo::DEVICE_ARM:
                    if(dataStr.StartsWith(_T("VEMS ARM") ) ){
                        return true;
                    }
                break;
            }

            return false;
       }
    }else if(res == SIPRParser::SIPR_STATE_NO_DATA){
        //Logger::logf( LOG_MSG, _T("ECUState::checkDevice SIPR_STATE_NO_DATA FALSE"));
    }else{
        //Logger::logf( LOG_MSG, _T("ECUState::checkDevice CRC ERROR FALSE"));
    }
    //Logger::logf( LOG_MSG, _T("ECUState::checkDevice FALSE"));
    return false;
}

bool ECUStateSIPR::enterMegatune( CommunicationBase* cb )
{
    SIPRPacket* packet  = new SIPRPacket( 0, SIPR_ENTER_MEGATUNE );

    cb->SendString( packet->packet_data(), packet->packet_size() );

    // TODO - clear this
    uint8_t res = readRecord( cb );

    if ( res == SIPRParser::SIPR_STATE_CRC_OK ) { return true; }

    return false;
}


uint8_t ECUStateSIPR::readRecord( CommunicationBase* cb )
{
    m_parser.reset();

    uint8_t c = 0x00;
    int result;
    int recCount = 0;
    int tryCount = 0;
    do {
        result = cb->ReadString(&c, 1);
        //if(result){
        //    Logger::logf( LOG_MSG, _T("ECUStateSIPR::readRecord: %02X"), c );
        //}
        uint8_t state = m_parser.feed( c );
        recCount += result;
        if( state == SIPRParser::SIPR_STATE_CRC_OK ) return SIPRParser::SIPR_STATE_CRC_OK;
        if( state == SIPRParser::SIPR_STATE_CRC_ERROR ) return SIPRParser::SIPR_STATE_CRC_ERROR;

        if(result == 0){
            tryCount++;
            wxThread::Sleep(1);
        }
    } while(tryCount < 10);

    return SIPRParser::SIPR_STATE_NO_DATA;
}
