#ifndef FIRMWAREDATA_H
#define FIRMWAREDATA_H


#include "ECUInfo.h"
#include "../common/types/IntSet.h"
#include "../common/types/IntArray.h"
#include "../common/types/Int2uint8_tRefHash.h"
#include "../common/types/Int2IntHash.h"
#include <wx/file.h>

class FirmwareData
{
    public:
        enum Type{
            UNKNOWN,
            HEX,
            CHEX
        };
        FirmwareData(ECUInfo::Device device);
        virtual ~FirmwareData();

        void setType(FirmwareData::Type type){ m_type = type;}
        FirmwareData::Type getType(){ return m_type;}

        void setPageCheckSum(uint32_t idx, uint32_t chk);
        void setPageData(uint32_t idx, uint32_t offset, uint8_t * buff, uint32_t len);
        uint8_t * getPageData(uint32_t idx, bool& found);
        void writeToIhex(void);
//        uint16_t * getPageDataW(uint32_t idx, bool& found);

        uint32_t getPageSize(){
            return m_pageSize;
        }

        uint32_t getPageAddress(uint32_t idx){
            return idx * m_pageSize;
        }

        uint32_t getPageAddressW(uint32_t idx){
            return (getPageAddress(idx) >> 1);
        }
        size_t getPages(){
            return m_intArray.GetCount();
        }

        IntSet& getPageSet()
        {
            return m_PageSet;
        }

        void addPatch(FirmwareData* firmwarePatch);

        uint32_t getPageCheckSum(uint32_t idx);
        uint32_t getPageIndexbyOrderIdx(uint32_t orderIdx){
            return m_intArray.Item(orderIdx);
        }
        //void setChecksumBuff(uint32_t value, unsigned char * checksumStr);

        wxString getTypeAsString();
        void printFirmwareData();
        void setV3MarkerWord();
    protected:
    private:
        uint32_t m_pageSize;
        Type m_type;
        IntArray m_intArray;//order
        Int2uint8_tRefHash m_pageBufferHash;
        IntSet m_PageSet;

        Int2IntHash m_pageCheckSum;
        ECUInfo::Device m_device;

        uint16_t getLittleEndianWordValue(uint8_t *buf, uint32_t i);

};



#endif // FIRMWAREDATA_H
