#include "FirmwareMgr.h"

#include "../fileparser/IHexParser.h"
#include "FirmwareData.h"
//#include "ECUStateMgr.h"

#include "../Observable.h"
#include "../firmwareconfigitem.h" //FirmwareConfigItemArray
//#include "../SerialException.h"
#include "BootModeBase.h"
#include "../vemsprotocol.h"
#include "../logger.h"
#include "ECUInfo.h"

/*
TODO: replace VemsProtocol with ECUStateMgrBase
*/
FirmwareMgr::FirmwareMgr(VemsProtocol* vp, BootModeBase* bm)
{
    //ctor
    m_vp = vp;
    m_bootModeBase = bm;
    m_firmwareData = NULL;
}

FirmwareMgr::~FirmwareMgr()
{
    //dtor
    //delete m_ecuStateMgr;
}

int FirmwareMgr::setFirmwareData(FirmwareConfigItemArray* configItemA)
{
    LOG
    wxString hexFilePath = getHexFile(configItemA);
     IHexParser ihexp((ECUInfo::Device) m_vp->getDevice() );
     if( ihexp.parse(hexFilePath) != 0){
         LOG
        //observerNotify(_T("Error: hex File does not exist!"));
        return FIRMWARE_FILE_NOT_FOUND_ERROR;
        //return-1; //FIRMWARE_FILE_NOT_FOUND_ERROR;
     }

     m_firmwareData = ihexp.getFirmwareData();
     //m_firmwareData->printFirmwareData();
     return 0;
}

int FirmwareMgr::addPatchFiles(FirmwareConfigItemArray* configItemA)
{
    // add patch files
     wxArrayString * patchFileA = getPatchFiles(configItemA);
     for(uint16_t i = 0; i <patchFileA->Count(); i++ ){
           wxString pathFilePath = patchFileA->Item(i);
           IHexParser ihexPatch((ECUInfo::Device) m_vp->getDevice() );
            if( ihexPatch.parse(pathFilePath) != 0){
                observerNotify(_T("Warning: Patch File not found:")+ pathFilePath + _T("!") );
            }else{
                 Logger::logf(LOG_MSG, _T("FirmwareMgr::addPatchFiles(): ")+pathFilePath);
                FirmwareData *patchData = ihexPatch.getFirmwareData();
                m_firmwareData->addPatch(patchData);
                delete patchData;
            }
     }
     return 0;
}

int FirmwareMgr::verify(FirmwareConfigItemArray* configItemA)
{
     Logger::logf(LOG_MSG, _T("FirmwareMgr::verify(): START"));

    int result = setFirmwareData(configItemA);

    if ( result == FIRMWARE_FILE_NOT_FOUND_ERROR )
        return result;

    addPatchFiles(configItemA);
    m_bootModeBase->setFirmwaredata(m_firmwareData);

    int prev_state = m_vp->getState();

    Logger::logf(LOG_MSG, _T("FirmwareMgr::verify(): 1"));
    int res = m_vp->setState(ECUInfo::BOOT);
    Logger::logf(LOG_MSG, _T("FirmwareMgr::verify() state:%d"), res);
    /*
    if(res < 0){
        Logger::logf(LOG_MSG, _T("Firmware::firmwareMgr(): set STATE BOOT failed 1"));
         observerNotify(m_vp->getErrorMessage(res));
          m_vp->setState(ECUInfo::MEGATUNE);
         return FIRMWARE_COMMUNICATION_ERROR;
    }
    int state = m_vp->getState();
    if(state != ECUInfo::BOOT){
         Logger::logf(LOG_MSG, _T("Firmware::firmwareMgr(): set STATE BOOT failed 2"));
        observerNotify(m_vp->getErrorMessage(res));
         m_vp->setState(ECUInfo::MEGATUNE);
         return FIRMWARE_COMMUNICATION_ERROR;
    }

     Logger::logf(LOG_MSG, _T("FirmwareMgr::verify(): 2"));
    //m_vp->setState(ECUInfo::BOOT); // throw exception if unable to enter bootMode
    */
    result = m_bootModeBase->verifyFlash();
    Logger::logf(LOG_MSG, _T("FirmwareMgr::verify() result=%i"), result);

    if( prev_state == ECUInfo::BOOT){
        /// Stay in boot and notify user!
         observerNotify(_T("ECU left in boot mode! Please restart ECU manually!"));
    }else{
        m_vp->setState(ECUInfo::MEGATUNE);
    }
    return result;
}

int FirmwareMgr::upload(FirmwareConfigItemArray* configItemA)
{
    Logger::logf(LOG_MSG, _T("FirmwareMgr::upload() START"));
    setFirmwareData(configItemA);
    addPatchFiles(configItemA);
    m_bootModeBase->setFirmwaredata(m_firmwareData);
    int prev_state = m_vp->getState();

    Logger::logf(LOG_MSG, _T("FirmwareMgr::upload() START 1"));
    int res = m_vp->setState(ECUInfo::BOOT);
    Logger::logf(LOG_MSG, _T("FirmwareMgr::upload() state:%d"), res);

    int result = m_bootModeBase->uploadFlash();

    Logger::logf(LOG_MSG, _T("FirmwareMgr::upload() START 3"));
        //result = m_bootModeBase->verifyFlash();

        //wxString firmwareInfo = getProcessInfo();
        //wxMessageBox(firmwareInfo);
        //FirmwareInfo* getPageInfo();
    if( prev_state == ECUInfo::BOOT){
        /// Stay in boot and notify user!
         observerNotify(_T("ECU left in boot mode! Please restart ECU manually!"));
    }else{
        m_vp->setState(ECUInfo::MEGATUNE);
    }
    return result;
}


wxString FirmwareMgr::getHexFile(FirmwareConfigItemArray* configItemA)
{
    FirmwareConfigItem* configItem;
    for(uint16_t i = 0; i <configItemA->Count(); i++ ){
        configItem = configItemA->Item(i);

        if( configItem->type == _T("firmware") ){
            return configItem->getAbsolutePath();
        }
    }
    return wxEmptyString;
}

wxArrayString* FirmwareMgr::getPatchFiles(FirmwareConfigItemArray* configItemA)
{
    wxArrayString * A = new wxArrayString();

    FirmwareConfigItem* configItem = new FirmwareConfigItem();
    for(uint16_t i = 0; i <configItemA->Count(); i++ ){
        configItem = configItemA->Item(i);

        if( configItem->type == _T("firmware_patch") ){
            if(configItem->active){
                A->Add(configItem->getAbsolutePath());
            }
        }
    }
    return A;
}
