#include "FirmwareVerifyController.h"
#include "VemsDeviceControllerBase.h"
#include "../FirmwareTypes.h"
#include "../logger.h"
#include "../communication/clients/FirmwareVerifier.h"
#include "../communication/clients/FirmwareUploader.h"
#include "../communication/clients/DeviceModeSelector.h"
#include "../events/FirmwareVerifyEvent.h"
#include "../events/FirmwareUploadEvent.h"
#include "../events/CommunicationEvent.h"


IMPLEMENT_DYNAMIC_CLASS( FirmwareVerifyController, wxEvtHandler )


FirmwareVerifyController::FirmwareVerifyController()
{
    m_firmwareData      = NULL;
    m_eventHandler      = NULL;
    m_deviceController  = NULL;
}

FirmwareVerifyController::FirmwareVerifyController( VemsDeviceControllerBase* deviceController, wxEvtHandler* eventHandler )
{
    m_firmwareData      = NULL;
    m_eventHandler      = eventHandler;
    m_deviceController  = deviceController;
}

FirmwareVerifyController::~FirmwareVerifyController()
{
}

void FirmwareVerifyController::verify( FirmwareData* firmwareData )
{
    m_processType = VERIFY;

    m_firmwareData = firmwareData;

    if ( m_deviceController->isBootMode() ) {
        LOG
        m_deviceController->registerClient( new FirmwareVerifier( this, this, m_firmwareData ) );
    } else {
        LOG
        m_deviceController->registerClient( new DeviceModeSelector( this, this,
                                                                    m_deviceController->getCommBase(),
                                                                    m_deviceController->getSupportedBaudrates(),
                                                                    m_deviceController->getDeviceMode(),
                                                                    ECUInfo::BOOT ) );
    }

    m_deviceController->startCommControllerNow();
}

void FirmwareVerifyController::upload( FirmwareData* firmwareData )
{
    m_processType = UPLOAD;

    m_firmwareData = firmwareData;

    if ( m_deviceController->isBootMode() ) {
        m_deviceController->registerClient( new FirmwareUploader( this, this, m_firmwareData ) );
    } else {
        m_deviceController->registerClient( new DeviceModeSelector( this, this,
                                                                    m_deviceController->getCommBase(),
                                                                    m_deviceController->getSupportedBaudrates(),
                                                                    m_deviceController->getDeviceMode(),
                                                                    ECUInfo::BOOT ) );
    }

    m_deviceController->startCommControllerNow();
}


void FirmwareVerifyController::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
    switch ( deviceMode )
    {
        case ECUInfo::BOOT:
        {
            if ( m_processType == VERIFY ) {
                m_deviceController->registerClient( new FirmwareVerifier( this, this, m_firmwareData ) );
            } else {
                m_deviceController->registerClient( new FirmwareUploader( this, this, m_firmwareData ) );
            }
            m_deviceController->startCommControllerNow();
        } break;

        default:
            /// TODO - LOG
            break;
    }
}

void FirmwareVerifyController::OnFirmwareVerifyEvent( FirmwareVerifyEvent&  event )
{
    m_eventHandler->AddPendingEvent( event );
}

void FirmwareVerifyController::OnFirmwareUploaderEvent( FirmwareUploadEvent&  event )
{
    m_eventHandler->AddPendingEvent( event );
}

void FirmwareVerifyController::OnTimeout()
{
    LOG
    CommunicationEvent event( CommunicationEvent::TIMEOUT );
    event.SetEventObject( this );
    m_eventHandler->AddPendingEvent( event );
}
