#ifndef PROTOCOLBASE_H
#define PROTOCOLBASE_H

#include "../common/typedef.h"
#include "wx/wx.h"

#include "ECUInfo.h"
#include "../FirmwareVersion.h"


class VemsCommand;


class ProtocolBase
{
    public:
        ProtocolBase();
        virtual ~ProtocolBase();

        virtual int     rx( uint8_t* data, int size )       = 0;
        virtual int     tx( uint8_t* data, int& rxCount )   = 0;

        virtual void    request( VemsCommand* command ) {}             // must be pure virtual
        virtual int     type() const { return ECUInfo::UNKNOWN; }

        virtual bool    hasError() = 0;
        virtual bool    needClear() { return false; }
        virtual void    reset() {}
        virtual void    disable() {}
        virtual void    enable() {}
        virtual bool    isDisabled() { return false; }
        virtual bool    isIdle() { return false; }
        void            setTimeout( int t ) { m_iTimeout = t; }
        int             getTimeout() const { return m_iTimeout; }

        void            setDevice( int device ) { m_device = device; }
        void            setFirmwareVersion( FirmwareVersion version ) { m_firmwareVersion = version; }

    protected:
        int             m_iTimeout;
        int             m_device;
        FirmwareVersion m_firmwareVersion;
};


#endif // PROTOCOLBASE_H
