#ifndef SDCARDCONTROLLER_H
#define SDCARDCONTROLLER_H


#include "DeviceInfo.h"
#include "../communication/clients/DeviceModeSelectorListener.h"
#include "../communication/clients/BaudRateSelectorListener.h"
#include "../communication/clients/TimeoutListener.h"


class VemsDeviceControllerBase;
class SDCardEvent;
class MMCFilesDesc;


class SDCardController : public wxEvtHandler
    , public DeviceModeSelectorListener
    , public BaudRateSelectorListener
    , public TimeoutListener
{
    DECLARE_DYNAMIC_CLASS(SDCardController)

    public:
        SDCardController();
        SDCardController( VemsDeviceControllerBase* deviceController, wxEvtHandler* eventHandler );

        virtual ~SDCardController();

        void setDeviceController( VemsDeviceControllerBase* deviceController );
        void setEventHandler( wxEvtHandler* eventHandler  );

        void search();
        void download( const MMCFilesDesc& desc );
        void createSummary( const MMCFilesDesc& desc );
        void abort();
        void startLog();
        void stopLog();
        void erase();
        void getFwVersion();


        /// listeners
        void OnDeviceModeSelected( ECUInfo::EcuMode deviceMode );
        void OnBaudrateChanged( size_t baudrate );
        void OnTimeout();

    private:
        enum Task { IDLE, SEARCH, DOWNLOAD, CREATE_SUMMARY, START_LOG, STOP_LOG, ERASE, SD_FWVER };

        ECUInfo::EcuMode                m_deviceMode;
        wxEvtHandler*                   m_eventHandler;
        VemsDeviceControllerBase*       m_deviceController;

        Task                            m_task;

        void registerDeviceModeSelector();

        void OnSDCardEvent( SDCardEvent& event );

        bool isBaudrateSelected();
        size_t getCurrentBaudrate();
        size_t getConfigBaudrate();
};

#endif // SDCARDCONTROLLER_H
