#include "SIPRDescriptor.h"
#include "SIPRTypes.h"
#include "../logger.h"


//
SIPRPageDescriptor::SIPRPageDescriptor()
{
    m_strReadCommand    = wxEmptyString;
    m_strWriteCommand   = wxEmptyString;
    m_iPageNumber       = 0;
    m_iCommandType      = 0;
}

SIPRPageDescriptor::SIPRPageDescriptor( int page,
                                        const wxString& readCommand,
                                        const wxString& writeCommand,
                                        int type,
                                        const IntArray tables )
{
    m_strReadCommand    = readCommand;
    m_strWriteCommand   = writeCommand;
    m_tables            = tables;
    m_iPageNumber       = page;
    m_iCommandType      = type;
}

// parser methods
//void SIPRDescriptor::addPage( int pageNo, const wxString& type )
void SIPRDescriptor::addPage( int pageNo, const wxString& readCommand, const wxString& writeCommand, const wxString& type )
{
//    LOGP( wxString::Format( _T("pageNo: %d"), pageNo ) + _T(", readCommand: ") + readCommand +
//        _T(", writeCommand: ") + writeCommand + _T(", type: ") + type );
    m_pages[ pageNo ]  = new SIPRPageDescriptor();
    m_pages[ pageNo ]->setPageNo( pageNo );
    m_pages[ pageNo ]->setPageReadCommand( readCommand );
    m_pages[ pageNo ]->setPageWriteCommand( writeCommand );
    if ( m_pagesOrder.Index( pageNo ) == wxNOT_FOUND )
    {
        m_pagesOrder.Add( pageNo );
    }

    if ( type.IsSameAs( _T("SIPR_GET_TABLEDATA"), false ) )
    {
        m_pages[ pageNo ]->setType( SIPR_GET_TABLEDATA );
    }
    else if ( type.IsSameAs( _T("SIPR_GET_CONFIG_1"), false ) )
    {
        m_pages[ pageNo ]->setType( SIPR_GET_CONFIG_1 );

        if ( m_configs.find( SIPR_GET_CONFIG_1 ) == m_configs.end() )
        {
            m_configs[ SIPR_GET_CONFIG_1 ] = new SIPRTableDescriptor();
        }

        m_configs[ SIPR_GET_CONFIG_1 ]->addPage( pageNo );
    }
    else if ( type.IsSameAs( _T("SIPR_GET_CONFIG_2"), false ) )
    {
        m_pages[ pageNo ]->setType( SIPR_GET_CONFIG_2 );

        if ( m_configs.find( SIPR_GET_CONFIG_2 ) == m_configs.end() )
        {
            m_configs[ SIPR_GET_CONFIG_2 ] = new SIPRTableDescriptor();
        }

        m_configs[ SIPR_GET_CONFIG_2 ]->addPage( pageNo );
    }
}

SIPRDescriptor::SIPRDescriptor()
{
    m_RealTimeLookUpTable[ _T("SIPR_RT_MT_NORMAL") ]    = SiprSensorType::SIPR_RT_MT_NORMAL;
    m_RealTimeLookUpTable[ _T("SIPR_RT_MT_EXT") ]       = SiprSensorType::SIPR_RT_MT_EXT;
}

SIPRDescriptor::~SIPRDescriptor()
{
	clear();
}

void SIPRDescriptor::addSIPRTableToPage( int pageNo, const wxString& table )
{
    LOGP( wxString::Format( _T("pageNo: %d"), pageNo ) + _T(", table: ") + table );

    int tableType = getTableType( table );

    // unknown table
    if ( tableType == -1 ) { return; }

    m_pages[ pageNo ]->addTable( tableType );

    if ( m_tables.find( tableType ) == m_tables.end() )
    {
        m_tables[ tableType ] = new SIPRTableDescriptor();
    }

    m_tables[ tableType ]->addPage( pageNo );
}

void SIPRDescriptor::addSIPRTable( const wxString& table, int offset, int size, const wxString& id )
{
    int tableType = getTableType( table );

    // unknown table
    if ( tableType == -1 ) { return; }

    if ( m_tables.find( tableType ) == m_tables.end() )
    {
        m_tables[ tableType ] = new SIPRTableDescriptor();
    }
//    m_tables[ tableType ] = new SIPRTableDescriptor();
    m_tables[ tableType ]->setType( tableType );
    m_tables[ tableType ]->setOffset( offset );
    m_tables[ tableType ]->setSize( size );
    m_tables[ tableType ]->setConfigFileID( id );
}

void SIPRDescriptor::addSIPRConfig( const wxString& config, int offset, int size, const wxString& id )
{
    int configType = getConfigType( config );

    // unknown config
    if ( configType == -1 ) { return; }

    if ( m_configs.find( configType ) == m_configs.end() )
    {
        m_configs[ configType ] = new SIPRTableDescriptor();
    }

    //m_configs[ configType ] = new SIPRTableDescriptor();
    m_configs[ configType ]->setType( configType );
    m_configs[ configType ]->setOffset( offset );
    m_configs[ configType ]->setSize( size );
    m_configs[ configType ]->setConfigFileID( id );
}

void SIPRDescriptor::addRealTimePage( const wxString& mtReadCmd, const wxString& siprPageType )
{
    // find SiprSensorType id for siprPageType
    String2IntHash::const_iterator it = m_RealTimeLookUpTable.find( siprPageType );

    // found
    if ( it != m_RealTimeLookUpTable.end() )
    {
        m_RealTimePages[ it->second ] = new RealTimePageDesc( mtReadCmd, it->second );
    }
    else
    {
        Logger::logf( LOG_ERROR, _T("SIPRDescriptor::addRealTimePage() - SiprSensorType not found for ->") +
            siprPageType  + _T("<-"));
    }
}


wxString SIPRDescriptor::getMTReadRealTimeCommand( int type ) const
{
    // find parsed SiprSensorType ids
    RealTimePageDescHash::const_iterator it = m_RealTimePages.find( type );

    if ( it != m_RealTimePages.end() )
    {
        return it->second->MegatuneReadCommand;
    }

    return wxEmptyString;
}

wxString SIPRDescriptor::getMTReadRealTimeCommand( const wxString& type ) const
{
    // find SiprSensorType id for siprPageType
    String2IntHash::const_iterator it = m_RealTimeLookUpTable.find( type );

    // found
    if ( it != m_RealTimeLookUpTable.end() )
    {
        return getMTReadRealTimeCommand( it->second );
    }
    else
    {
        Logger::logf( LOG_ERROR, _T("SIPRDescriptor::getMTReadRealTimeCommand() - SiprSensorType not found for ->") +
            type  + _T("<-"));
    }

    return wxEmptyString;
}

// getter methods
bool SIPRDescriptor::isPageExist( int pageNo ) const
{
    IntToSIPRPageDescriptor::const_iterator iter = m_pages.find( pageNo );

    return ( iter != m_pages.end() );
}

int SIPRDescriptor::getPageType( int pageNo ) const
{
    IntToSIPRPageDescriptor::const_iterator IT = m_pages.find( pageNo );

    if ( IT == m_pages.end() ) { return -1; }
    else { return IT->second->getType(); }
}

IntArray SIPRDescriptor::getTables( int pageNo ) const
{
    IntToSIPRPageDescriptor::const_iterator IT = m_pages.find( pageNo );

    if ( IT == m_pages.end() ) {
        IntArray emptyArray;
        return emptyArray;
    } else {
        return IT->second->getTables();
    }
}

int SIPRDescriptor::getOffset( int pageType, int table ) const
{
    if ( pageType == SIPR_GET_TABLEDATA )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_tables.find( table );

        if ( IT == m_tables.end() ) { return -1; }
        else { return IT->second->getOffset(); }
    }
    else if (   ( pageType == SIPR_GET_CONFIG_1 ) ||
                ( pageType == SIPR_GET_CONFIG_2 ) )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_configs.find( pageType );

        if ( IT == m_configs.end() ) { return -1; }
        else { return IT->second->getOffset(); }
    }
    else { return -1; }
}

int SIPRDescriptor::getSize( int pageType, int table ) const
{
    if ( pageType == SIPR_GET_TABLEDATA )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_tables.find( table );

        if ( IT == m_tables.end() ) { return -1; }
        else { return IT->second->getSize(); }
    }
    else if (   ( pageType == SIPR_GET_CONFIG_1 ) ||
                ( pageType == SIPR_GET_CONFIG_2 ) )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_configs.find( table );

        if ( IT == m_configs.end() ) { return -1; }
        else { return IT->second->getSize(); }
    }
    else { return -1; }
}

wxString SIPRDescriptor::getConfigFileID(  int pageType, int table ) const
{
    if ( pageType == SIPR_GET_TABLEDATA )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_tables.find( table );

        if ( IT != m_tables.end() ) { return IT->second->getConfigFileID(); }
    }
    else if (   ( pageType == SIPR_GET_CONFIG_1 ) ||
                ( pageType == SIPR_GET_CONFIG_2 ) )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_configs.find( table );

        if ( IT != m_configs.end() ) { return IT->second->getConfigFileID(); }
    }

    return wxEmptyString;
}

IntArray SIPRDescriptor::getPages( int pageType, int table ) const
{
    IntArray emptyArray;

    if ( pageType == SIPR_GET_TABLEDATA )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_tables.find( table );

        if ( IT == m_tables.end() ) { return emptyArray; }
        else { return IT->second->getPages(); }
    }
    else if (   ( pageType == SIPR_GET_CONFIG_1 ) ||
                ( pageType == SIPR_GET_CONFIG_2 ) )
    {
        IntToSIPRTableDescriptor::const_iterator IT = m_configs.find( table );

        if ( IT == m_configs.end() ) { return emptyArray; }
        else { return IT->second->getPages(); }
    }
    else { return emptyArray; }
}

int SIPRDescriptor::getTableType( const wxString& table )
{
    if ( table.IsSameAs( _T("SIPR_TABLE_RPM"), false ) )
    {
        return SIPR_TABLE_RPM;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_KPA"), false ) )
    {
        return SIPR_TABLE_KPA;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_IGN"), false ) )
    {
        return SIPR_TABLE_IGN;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_LAMBDA"), false ) )
    {
        return SIPR_TABLE_LAMBDA;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_EGOC"), false ) )
    {
        return SIPR_TABLE_EGOC;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_VE"), false ) )
    {
        return SIPR_TABLE_VE;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_H"), false ) )
    {
        return SIPR_TABLE_H;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_BOOST_DC"), false ) )
    {
        return SIPR_TABLE_BOOST_DC;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_BOOST_TPS"), false ) )
    {
        return SIPR_TABLE_BOOST_TPS;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_TEMPERATURE"), false ) )
    {
        return SIPR_TABLE_TEMPERATURE;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_MAPCORR"), false ) )
    {
        return SIPR_TABLE_MAPCORR;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_ALT_KNOCK"), false ) )
    {
        return SIPR_TABLE_ALT_KNOCK;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_A340E"), false ) )
    {
        return SIPR_TABLE_A340E;
    }
     else if ( table.IsSameAs( _T("SIPR_TABLE_MAT_TPS_RETARD"), false ) )
    {
        return SIPR_TABLE_MAT_TPS_RETARD;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_VVTI"), false ) )
    {
        return SIPR_TABLE_VVTI;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_CONFIG1"), false ) )
    {
        return SIPR_TABLE_CONFIG1;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_CONFIG1_EXT"), false ) )
    {
        return SIPR_TABLE_CONFIG1_EXT;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_CONFIG2"), false ) )
    {
        return SIPR_TABLE_CONFIG2;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_LAUNCH_RPM"), false ) )
    {
        return SIPR_TABLE_LAUNCH_RPM;
    }
    else if ( table.IsSameAs( _T("SIPR_TABLE_CONFIG_SHARED"), false ) )
    {
        return SIPR_TABLE_CONFIG_SHARED;
    }
    else { return -1; } // unknown table
}

int SIPRDescriptor::getConfigType( const wxString& config )
{
    if ( config.IsSameAs( _T("SIPR_GET_CONFIG_1"), false ) )
    {
        return SIPR_GET_CONFIG_1;
    }
    else if ( config.IsSameAs( _T("SIPR_GET_CONFIG_2"), false ) )
    {
        return SIPR_GET_CONFIG_2;
    }
    else { return -1; } // unknown config
}

wxString SIPRDescriptor::getPageReadCommand( int page ) const
{
    IntToSIPRPageDescriptor::const_iterator IT = m_pages.find( page );

    if ( IT == m_pages.end() )
    {
        return wxEmptyString;
    }
    else
    {
        return IT->second->getPageReadCommand();
    }
}

wxString SIPRDescriptor::getPageWriteCommand( int page ) const
{
    IntToSIPRPageDescriptor::const_iterator IT = m_pages.find( page );

    if ( IT == m_pages.end() ) { return wxEmptyString; }
    else { return IT->second->getPageWriteCommand(); }
}


int SIPRDescriptor::getPageNumber( int idx ) const
{
    if ( idx < 0 || (size_t)idx >= m_pagesOrder.GetCount() ) { return -1; }

    return m_pagesOrder.Item( idx );
}

size_t SIPRDescriptor::getSiprTableCount() const
{
    return m_tables.size();
}

void SIPRDescriptor::clear()
{
    WX_CLEAR_HASH_MAP( IntToSIPRPageDescriptor , m_pages );
    m_pagesOrder.Clear();
    WX_CLEAR_HASH_MAP( IntToSIPRTableDescriptor, m_tables );
    WX_CLEAR_HASH_MAP( IntToSIPRTableDescriptor, m_configs );
    m_strBurnCommand.Clear();

	WX_CLEAR_HASH_MAP( RealTimePageDescHash, m_RealTimePages );
	//WX_CLEAR_HASH_MAP( String2IntHash, m_RealTimeLookUpTable );
}


int SIPRDescriptor::getPageSize(  int pageNo ) const
{
    int size = 0;

    int pageType = getPageType( pageNo );

    if ( pageType == -1 ) {
        LOGP( wxString::Format( _T("INVALID PAGETYPE - pageNo: %d"), pageNo ) );
    } else if ( ( pageType == SIPR_GET_CONFIG_1 ) || ( pageType == SIPR_GET_CONFIG_2 ) ) { ///CONFIG PAGE
        size = getConfigPageSize( pageNo );
    } else {
        size = getTablePageSize( pageNo );
    }

    return size;
}

int SIPRDescriptor::getConfigPageSize(  int pageNo ) const
{
    int size = 0;

    int pageType = getPageType( pageNo );

    IntToSIPRTableDescriptor::const_iterator iter = m_configs.find( pageType );

    if ( iter != m_configs.end() ) {
        size += iter->second->getSize();
    }

    return size;
}

int SIPRDescriptor::getTablePageSize(  int pageNo ) const
{
    int size = 0;

    IntArray tables = getTables( pageNo );

    for ( size_t i = 0; i < tables.GetCount(); ++i ) {

        IntToSIPRTableDescriptor::const_iterator IT = m_tables.find( tables[i] );

        if ( IT != m_tables.end() ) {
            size += IT->second->getSize();
        }
    }

    return size;
}


