#include "SIPRPacket.h"

#include <wx/listimpl.cpp>
//#ifdef __WINCE_STANDARDSDK__
//	#include "../wince/src/timer.h"
//#elif (_MSC_VER >= 1400) //  1400 = VS2005 1500 = VS2008
//	#include "../v3gui/src/timer.h"
//#else
//	#include <wx/ctb/timer.h>
//#endif

#include "../logger.h"
#include "../Math.h"

#include "SIPRTypes.h"

WX_DEFINE_LIST( SIPRPacketList );

const uint8_t   SIPRPacket::SIPR_SIGN           = 0x0A;
const int       SIPRPacket::HEADER_SIZE         = 4;
const int       SIPRPacket::SIPR_FRAME_SIZE     = 6;
const int       SIPRPacket::SIPR_MAX_DATA_SIZE  = 255;

SIPRPacket::SIPRPacket()
{
    //Logger::logf( LOG_MSG, _T("SIPRPacket::SIPRPacket()") );
    m_pData         = new uint8_t[ SIPR_FRAME_SIZE ];
//    m_pTimer        = NULL;                             // timer for timeout
//    m_timeout_flag  = 0;                                 // timer set this flag
    m_timeout       = -1;
}

SIPRPacket::SIPRPacket( uint8_t sequence,
                        uint8_t type,
                        uint8_t size,
                        const uint8_t* data,
                        uint16_t crc )
{
    //Logger::logf( LOG_MSG, _T("SIPRPacket::SIPRPacket( params )") );
    init( sequence, type, size, data, crc );
}

SIPRPacket::SIPRPacket(uint8_t sequence, uint8_t type, uint8_t size, const IntArray& data)
{
     m_timeout       = -1;
//    m_timeout_flag  = 0;
//    m_pTimer        = NULL;

    m_pData     = new uint8_t[ SIPR_FRAME_SIZE + size + 3 ];
    m_pData[0]  = SIPR_SIGN;
    m_pData[1]  = sequence;
    m_pData[2]  = type;
    m_pData[3]  = size;

    m_pData[4] = 0;
    m_pData[5] = 0;
    m_pData[6] = SiprSensorType::SIPR_RT_MT_EXT;

    for(size_t i = 0; i < size; ++i){
        m_pData[HEADER_SIZE+3+i] = data.Item(i);
    }
    setCRC( calculateCRC( &m_pData[1], size + 3 ) );

}

SIPRPacket::SIPRPacket( const SIPRPacket& packet )
{
    //Logger::logf( LOG_MSG, _T("SIPRPacket::SIPRPacket( const SIPRPacket& packet )") );
    m_timeout       = -1;
//    m_timeout_flag  = 0;
//    m_pTimer        = NULL;

    m_pData     = new uint8_t[ packet.packet_size() ];
    memcpy( m_pData, packet.packet_data(), packet.packet_size() );
}

void SIPRPacket::init(  uint8_t sequence,
                        uint8_t type,
                        uint8_t size,
                        const uint8_t* data,
                        uint16_t crc )
{
    m_timeout       = -1;
//    m_timeout_flag  = 0;
//    m_pTimer        = NULL;

    m_pData     = new uint8_t[ SIPR_FRAME_SIZE + size ];
    m_pData[0]  = SIPR_SIGN;
    m_pData[1]  = sequence;
    m_pData[2]  = type;
    m_pData[3]  = size;
    if ( size && data ) { memcpy( &m_pData[HEADER_SIZE], data, size ); }
    if ( crc )
    {
        setCRC( crc );
    }
    else
    {
        setCRC( calculateCRC( &m_pData[1], size + 3 ) );
    }
}

SIPRPacket::~SIPRPacket()
{
//    //Logger::logf( LOG_MSG, _T("SIPRPacket::~SIPRPacket() - Destructor") );
//    if ( m_pTimer )
//    {
//        m_pTimer->stop();
//        delete m_pTimer;
//    }

    delete[] m_pData;
}

void SIPRPacket::operator=(const SIPRPacket& packet)
{
    //Logger::logf( LOG_MSG, _T("SIPRPacket::operator=() : size = %u, type : %u"), packet.packet_size(), packet.type() );
    m_timeout       = -1;
//    m_timeout_flag  = 0;
//    m_pTimer        = NULL;

    delete[] m_pData;
    m_pData     = new uint8_t[ packet.packet_size() ];
    memcpy( m_pData, packet.packet_data(), packet.packet_size() );

    //Logger::logf( LOG_MSG, _T("SIPRPacket::operator=() : %u"), packet.packet_data()[2] );
    //Logger::logf( LOG_MSG, _T("SIPRPacket::operator=() : %u"), m_pData[2] );
}

void SIPRPacket::setCRC( uint16_t crc )
{
    /// Little Endian CRC16
    m_pData[ 4 + size() ]     = crc & 255;
    m_pData[ 4 + size() + 1 ] = crc >> 8;
}

uint16_t SIPRPacket::getCRC() const
{
    uint16_t crc = 0;

    crc =   m_pData[ 4 + size() ];
    crc +=  m_pData[ 4 + size() + 1 ] << 8;

    return crc;
}

uint16_t SIPRPacket::calculateCRC( uint8_t* data, int size )
{
    uint16_t crc = 0xFFFF;

    for( int i = 0; i < size; ++i)
    {
        crc = Math::crc16_sipr( crc, data[i] );
    }

    return crc;
}

void SIPRPacket::set_timeout( int timeout )
{
    m_timeout = timeout;
}

void SIPRPacket::start_timer()
{
//    if ( !m_pTimer && m_timeout > 0 )
//    {
//        m_pTimer        = new timer( m_timeout, &m_timeout_flag, NULL );
//        m_pTimer->start();
//    }
    m_startTime = wxDateTime::Now();
}

bool SIPRPacket::is_timed_out() const
{
    wxDateTime now = wxDateTime::UNow();
    wxTimeSpan timeSpan = now.Subtract( m_startTime );
    long elapsedTime = timeSpan.GetMilliseconds().ToLong();
    return ( m_timeout < elapsedTime );
//    return ( m_timeout_flag > 0 );
}

int SIPRPacket::getSubType() const
{
    switch( type() ){
        case SIPR_GET_TABLEDATA:
            return data()[0];
        case SIPR_READ_SENSORS:
            return data()[2];
        default:
            return -1;
    }
}

wxString SIPRPacket::toString()
{
    wxString s;
    for ( size_t i = 0; i < (size_t)packet_size(); ++i )
    {
        s += wxString::Format( _T("%02X"), packet_data()[i] );
        //s += wxString::Format( _T("%d"), i );
    }
    return s;
}
