#include "SerialPortHandlerCTB.h"


#if defined (__WINCE_STANDARDSDK__)
	#include "wince/src/getopt.h"
	#include "wince/src/iobase.h"
	#include "wince/src/serport.h"
	#include "wince/src/timer.h"
#elif (_MSC_VER >= 1400) //  1400 = VS2005 1500 = VS2008 1600 = VS2010
	#include "../v3gui/src/getopt.h"
	#include "../v3gui/src/iobase.h"
	#include "../v3gui/src/serport.h"
	#include "../v3gui/src/timer.h"
#else
    #include <wx/ctb/getopt.h>
    #include <wx/ctb/iobase.h>
    #include <wx/ctb/serport.h>
    #include <wx/ctb/timer.h>
#endif

#include "../serialconfig.h"
#include "../SerialException.h"
#include "../Math.h"

#include "../UnicodeHelper.h"
#include "../logger.h"

SerialPortHandlerCTB::SerialPortHandlerCTB(SerialConfig* serialConfig)
{

#ifdef CTB15
    dev = new ctb::SerialPort();
#else
    dev = new wxSerialPort();
#endif

    m_serialConfig = serialConfig;
    setBaudRate(m_serialConfig->getBaudRate() );
}

SerialPortHandlerCTB::~SerialPortHandlerCTB()
{
    //dtor
    Logger::log(LOG_MSG, _T("SerialPortHadlerCTB DTOR") );
    dev->Close();
    delete dev;
}


bool SerialPortHandlerCTB::IsOpen(){
    return dev->IsOpen();
}

int SerialPortHandlerCTB::open()
{
//fprintf(stdout, "SerialPortHandlerCTB this: %p\n",this);

    //fprintf(stdout,  "SerialPortHandlerCTB:open() START: dev: %p\n", dev);
    //Logger::logf(LOG_WARNING, _T("SerialPortHandlerCTB::open() init!!") );
    if(dev == NULL){
        #ifdef CTB15
            dev = new ctb::SerialPort();
        #else
            dev = new wxSerialPort();
        #endif
         Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::open() new instance!!") );
    }
       // Logger::log(LOG_MSG, _T("SerialPortHandlerCTB:open()dev NOT NULL!!") );
    if( dev->IsOpen() ){ return 0; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::open() init!!") );
    //initSerialPort();
	// try to open the given port appr 500 times (you guess right, windows sucks!)
    int trycnt=4;
    while( trycnt > 0 && dev->Open( m_serialConfig->getPortNameC() ) < 0 ) {

        //Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::open() Cannot open: ") + m_serialConfig->getPortName() );
        //Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::open() Cannot openC: %s"), m_serialConfig->getPortNameC() );
        //wxLogError(_T("Cannot open: %s\n"),devname );
        //printf(" SerialPortHandlerCTB::open() %s", m_serialConfig->getPortNameC() );
        trycnt--;
        // sleep(1); or wxsleep() ?
    }
    if( trycnt == 0 ){
//        Logger::logf(LOG_MSG,  _T("Giving up opening ")+ m_serialConfig->getPortName() );
        // Wrong: Logger::log(_T("Giving up opening 25 %s\n"),UnicodeHelper::U2C( portName.c_str() ) );
        //delete dev;
        //dev = NULL;
        //if(dev == NULL){
        //    fprintf(stdout, "dev is NULL\n");
        //}
        //serialConfig.setStatus("Unable to open!");
        throw SerialException(SP_CANNOT_OPEN);
        //return SP_CANNOT_OPEN;

    }


    #ifdef CTB15
        dev->SetBaudrate(m_baudrate);
    #else
        dev->SetBaudrate((wxBaud)m_baudrate);
    #endif
    // Success! port opened
    // set the baudrate
    //((wxSerialPort*)dev)->SetBaudRate(baudrate);
    //return 0;
    // Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::SerialPortHandlerCTB() open END") );
   //  isInit = 1;
   //serialConfig.setStatus("Opened!");
   return 0;
}

void SerialPortHandlerCTB::close()
{
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::close() ") );
    if( dev == NULL) { return; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::close() dev not null") );
    if( !dev->IsOpen() ){ return; }
    //Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::close() dev->Close()") );
    dev->Close();
}

int SerialPortHandlerCTB::SendString(const uint8_t * str)
{
     //fprintf(stdout, "LINE: %s\n", str );
     return SendString(str,  strlen((char*)str));
}

int SerialPortHandlerCTB::SendString(const uint8_t* str, size_t len)
{
    open();
    int result = dev->Write( (char *) str, len);
    if((int)len != result){
           Logger::log(LOG_ERROR, _T("SerialPortHandlerCTB::SendString() WRITE FAILED!") );
            throw SerialException(SP_WRITE_ERROR);
            //return SP_WRITE_ERROR;
     }
     return result;
}

int SerialPortHandlerCTB::SendString(const wxString& str)
{
    const char * c = UnicodeHelper::U2C(str);
    int res = SendString( (uint8_t*)c );
    delete (char *)c;
    return res;
     /*
     int timeout = 10;
     int to = 0;
     size_t len = str->Length();

     timer t(timeout, &to, NULL);
     t.start();

     return dev->Writev((char *) str->c_str(), len, &to);
    */
}
/*
int SerialPortHandlerCTB::SendString(wxChar * chr)
{
   wxString * str = new wxString(chr);
   return SendString(str);
}
*/


int SerialPortHandlerCTB::Read(uint8_t* str, size_t len)
{
        open();
        int rd = dev->Read((char*)str, len);
        return rd;
}

/**
return <0: FAILED: error code
       >0: OK:  number of read byte
*/
int SerialPortHandlerCTB::ReadString(uint8_t* str, size_t len)
{
  //      Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::ReadString()"));
        open();
  //      Logger::logf(LOG_MSG, _T("SerialPortHandlerCTB::ReadString() sp open"));
       // if( open() == SP_CANNOT_OPEN){ return SP_CANNOT_OPEN;}
        //if(dev == NULL){return SP_UNKNOWN;}
        //int timeout = 3 * len; //?????
        int timeout = getTimeOut(len);
        int to = 0;
        //size_t len = strlen(str);
         wxString msg;

    #ifdef CTB15
        ctb::Timer t(timeout, &to, NULL);
    #else
        timer t(timeout, &to, NULL);
    #endif
        t.start();

        int rd = dev->Readv ((char*)str, len, &to, true);
        return rd;
}

int SerialPortHandlerCTB::ReadString(uint8_t* data, size_t len, int maxTryCount)
{
    int tryCount =0;
    int result = 0;
    do{
        size_t expLen = len - result;
        int tmp = ReadString(&data[result], expLen);
        if(tmp == 0 ){
            tryCount++;
            wxThread::Sleep(1);
        }
        result += tmp;

    }while( (result < (int)len) && (tryCount < maxTryCount));
    return result;
}

int SerialPortHandlerCTB::getBaudRate()
{
    return m_baudrate;
}

wxString SerialPortHandlerCTB::getPortName() const
{
    return m_serialConfig->getPortName();
}


void SerialPortHandlerCTB::setBaudRate(int bRate)
{
    Logger::logf( LOG_MSG, _T("SerialPortHandlerCTB::setBaudRate: %i"), bRate );
    //open();
    //if( open() == SP_CANNOT_OPEN) return ;
    wxString msg;

    #ifdef CTB15
        m_baudrate = bRate;
    #else
        switch(bRate)
        {
            case (4800):
                    m_baudrate = wxBAUD_4800;
                    break;
            case (9600):
                    m_baudrate = wxBAUD_9600;
                    break;
            case (19200):
                    m_baudrate = wxBAUD_19200;
                    break;
             case (38400):
                    m_baudrate = wxBAUD_38400;
                    break;
             case (57600):
                    m_baudrate = wxBAUD_57600;
                    break;
             case (115200):
                    m_baudrate = wxBAUD_115200;
                    break;
            default:
                    m_baudrate = wxBAUD_19200;

        }
    #endif

    m_magicTimeoutConstA = 12000.0/m_baudrate;
    m_magicTimeoutConstB = 12.0 * 10000.0/m_baudrate;
    //open();

    #ifdef CTB15
        if( IsOpen() ) dev->SetBaudrate(m_baudrate);
    #else
        if( IsOpen() ) dev->SetBaudrate((wxBaud)m_baudrate);
    #endif
}


int SerialPortHandlerCTB::clearSerialBuffer(int len)
{
        uint8_t * tmp = new uint8_t[len+32];
        int res = ReadString(tmp, len);
        delete[] tmp;
        return res;
}

int SerialPortHandlerCTB::clearSerialBufferCircle(int len, int count)
{
    int res;
    int c = 0;
    while(count > 0){
        res = clearSerialBuffer(len);
        if(res < len)return (c * len + res);
        count--;
        c++;
    }
   return -1;
   // throw SerialException(SP_CANNOT_CLEAR_BUFF);
   // return SP_CANNOT_CLEAR_BUFF;
}

/*virtual*/
int SerialPortHandlerCTB::clear()
{
    return clearSerialBufferCircle(8, 64);
}

int SerialPortHandlerCTB::sendStringBlock(uint8_t* value, size_t len, size_t blockLen)
{
    //char * p = (char *) value;
    int result = 0;
    size_t i = 0;
    size_t currBlock;
    for(i=0; i< len; i += blockLen){
        currBlock = len - i;
        if( currBlock > blockLen ){
            currBlock = blockLen;
        }
        result += SendString( (&(value[i])), currBlock);
    }
    return result;
}

int SerialPortHandlerCTB::getTimeOut(size_t len)
{
    return (int)(len * m_magicTimeoutConstA + m_magicTimeoutConstB);

    //return ((len*10)>>3) + 12; //9600 10000/baudRate

    //return ((len*10000)/m_baudrate) + 12;
}

int SerialPortHandlerCTB::sendNumber(uint32_t number, int length)
{
    uint8_t str[4];
    Math::getBytes(number, str, length);
    return SendString(str, length);
}

int SerialPortHandlerCTB::sendNumber(uint16_t number)
{
    uint8_t str[2];

    str[0] = (number >> 8) &0xFF;
    str[1] = number & 0xFF;

    return SendString( str, 2);
}
