#ifndef SIPR_H
#define SIPR_H

#include "SIPRPacket.h"
#include "SIPRParser.h"
#include "SIPRTypes.h"
#include "ProtocolBase.h"

class CommunicationBase;
class SIPRDescriptor;
class wxEvtHandler;

class Sipr : public ProtocolBase
{
    public:
    public:
        Sipr( SIPRDescriptor* siprDesc, int sensorType, wxEvtHandler* eventHandler );
        virtual ~Sipr();

        //ProtocolBase virtual functions
        int rx( uint8_t* c, int size );
        int tx( uint8_t* c, int& size );
        void request( VemsCommand* command );
        void enable();
        void disable();
        bool isDisabled();
        //bool hasError() { return false; }
        bool hasError() { return m_bHasError; }
        void reset();

        void setEventHandler( wxEvtHandler* eventHandler );
        int type() const { return ECUInfo::SIPR; }

        // SIPR commands
        void ping();
        void get_version();
        void set_baud( SiprBaud b );

        void get_table_size( );
        void get_table_data( int page );
        void set_table_data(  uint8_t page, uint8_t offset, uint8_t value );
        void get_hwsig();
        void begin_flash();
        void save_config();

        void                readRealtime( bool read, unsigned int delay = 0 ); // emulate always read_sensor command
        //void                setSensorType( int type ) { m_iSensorType = type; }
        bool isIdle() { return m_state == IDLE; }

    private:
        enum { IDLE, WAIT };

    private:
        SIPRDescriptor*     m_pSIPRDescriptor;
        wxEvtHandler*       m_eventHandler;
        SIPRParser          m_parser;
        uint8_t             m_seqence;
        SIPRPacketList      m_txQueue;          // packets waiting for sending to device
        SIPRPacketList      m_waitingQueue;     // transmited packets waiting for response
        bool                m_isReadSensors;    // for emulate always read_sensor command
        unsigned int        m_delayReadSensors; // delay for emulate read_sensor command
        timer*              m_timerReadSensors;
        int                 m_timerTimedOutFlag;
        bool                m_bIsDisabled;
        bool                m_bNeedDisable;
        int                 m_state;
        bool                m_bHasError;
        int                 m_iSensorType;

    private:
        // SIPR commands
        void                read_sensor( uint8_t count, uint8_t delay, uint8_t page );

        int                 push( SIPRPacket* packet );             // push packet to tx queue
        void                notify();
        bool                checkTimeout();
        void                sendEvent( const SIPRPacket& packet, int eventType );
        void                readPage( int mtPage );
        void                readAllPage();
        void                writePage( int mtPage, int mtOffset, uint8_t value );
        void                enableReadRealtime( bool enable );
};


#endif // SIPR_H
