#include "StateGraph.h"
#include "../common/types/Int2IntHash.h"


void StateGraph::addEdge( int from, int to )
{
    if ( m_nodesWithEdges.find( from ) == m_nodesWithEdges.end() ||
        m_nodesWithEdges.find( to ) == m_nodesWithEdges.end() ) return;

    m_nodesWithEdges[ from ]->insert( to );
}


void StateGraph::addNode( int node )
{
    if ( !m_nodesWithEdges[ node ] )
    {
        m_nodesWithEdges[ node ] = new IntSet();
    }
}


/*

V[1..N]:=0, Sorba(A)
V[A]:= -1 {mindegy, csak ne 0 legyen}
Ciklus amíg a sor nem üres
   Sorból(X)
   Ciklus I:=1..N
      Ha van él X és I között és V[I]=0 akkor Sorba(I), V[I]:=X
   Ciklus vége
Ciklus vége
*/


IntArray StateGraph::getRoute( int from, int to )
{
    Int2IntHash nodeToParent; //    [node] = 0;

    IntArray searchQueue;

    searchQueue.Add( from );
    nodeToParent[ from ] = -1;

    while( !searchQueue.IsEmpty() )
    {
        int state = searchQueue.Item( 0 );
        searchQueue.RemoveAt( 0 );

        IntSet* targetSet = m_nodesWithEdges[ state ];

        for( IntSet::const_iterator it = targetSet->begin(); it != targetSet->end(); ++it )
        {
            int targetState = *it;

            if ( targetState  == to )
            {
                nodeToParent[ targetState ] = state;
                break;
            }

            if ( nodeToParent[ targetState ] == 0 )
            {
                nodeToParent[ targetState ] = state;
                searchQueue.Add( targetState );
            }
        }
    }

    IntArray path;

    //no route to target
    if ( nodeToParent[ to ] == 0 ) return path;

    int  id = to;

    while ( id != from )
    {
        path.Insert( id, 0u );
        id = nodeToParent[ id ];
    }

    path.Insert( id, 0u );

    return path;
}

