#include "TerminalPanel.h"

#include <wx/timer.h>
#include "../logger.h"


TerminalPanel* TerminalLog::s_pTerminal = NULL;


void TerminalLog::init( TerminalPanel* terminal )
{
    s_pTerminal = terminal;
}


void TerminalLog::rxLog( uint8_t* data, int size )
{
    if ( s_pTerminal ) { s_pTerminal->rxLog( data, size ); }
}


void TerminalLog::txLog( uint8_t* data, int size )
{
    if ( s_pTerminal ) { s_pTerminal->txLog( data, size ); }
}


void TerminalLog::portLog( const wxString& portName )
{
    if ( s_pTerminal ) { s_pTerminal->portLog( portName ); }
}


void TerminalLog::baudLog( int baudRate )
{
    if ( s_pTerminal ) { s_pTerminal->baudLog( baudRate ); }
}


void TerminalLog::protocolLog( const wxString& protocolName )
{
    if ( s_pTerminal ) { s_pTerminal->protocolLog( protocolName ); }
}


///**********************************************************************TerminalPanel
int TERMINAL_TIMER_ID   = wxNewId();
int CLEAR_ALL_BTN_ID    = wxNewId();

BEGIN_EVENT_TABLE( TerminalPanel, wxPanel )
    EVT_TIMER( TERMINAL_TIMER_ID, TerminalPanel::OnTimer )
    EVT_BUTTON( CLEAR_ALL_BTN_ID, TerminalPanel::OnClearAllButton )
END_EVENT_TABLE()


TerminalPanel::TerminalPanel( wxWindow* parent, wxWindowID id )
    : m_timer( this, TERMINAL_TIMER_ID )
    , m_rxBuffer( 256 )
    , m_txBuffer( 256 )
{
    Create( parent, id );

    /// main
    wxBoxSizer* mainsizer = new wxBoxSizer( wxHORIZONTAL );

    /// controller box
    wxStaticBoxSizer* controllerSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Controller") );

    /// top toolbar
    wxBoxSizer* toolSizerTop = new wxBoxSizer( wxHORIZONTAL );

    /// port
    m_pTextCtrlPort = new wxTextCtrl( this, wxID_ANY );
    wxStaticBoxSizer* portSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Port") );
    portSizer->Add( m_pTextCtrlPort, 1, wxEXPAND );
    toolSizerTop->Add( portSizer, 1, wxEXPAND | wxALL, 5 );

    /// baud
    m_pTextCtrlBaud = new wxTextCtrl( this, wxID_ANY );
    wxStaticBoxSizer* baudSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Baudrate") );
    baudSizer->Add( m_pTextCtrlBaud, 1, wxEXPAND );
    toolSizerTop->Add( baudSizer, 1, wxEXPAND | wxALL, 5 );

    /// protocol
    m_pTextCtrlProtocol = new wxTextCtrl( this, wxID_ANY );
    wxStaticBoxSizer* protocolSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Protocol") );
    protocolSizer->Add( m_pTextCtrlProtocol, 1, wxEXPAND );
    toolSizerTop->Add( protocolSizer, 1, wxEXPAND | wxALL, 5 );

    controllerSizer->Add( toolSizerTop, 0 );

    /// rx
    m_pTextCtrlRx = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE );
    wxStaticBoxSizer* rxSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Receive") );
    rxSizer->Add( m_pTextCtrlRx, 1, wxEXPAND );
    controllerSizer->Add( rxSizer, 1, wxEXPAND | wxALL, 5 );

    /// tx
    m_pTextCtrlTx = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE );
    wxStaticBoxSizer* txSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Transmit") );
    txSizer->Add( m_pTextCtrlTx, 1, wxEXPAND );
    controllerSizer->Add( txSizer, 1, wxEXPAND | wxALL, 5 );

    /// bottom toolbar
    wxBoxSizer* toolSizer = new wxBoxSizer( wxHORIZONTAL );

    /// rx count
    m_pTextCtrlRxCount = new wxTextCtrl( this, wxID_ANY );
    wxStaticBoxSizer* rxCountSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Rx") );
    rxCountSizer->Add( m_pTextCtrlRxCount, 1, wxEXPAND );
    toolSizer->Add( rxCountSizer, 1, wxEXPAND | wxALL, 5 );

    /// tx count
    m_pTextCtrlTxCount = new wxTextCtrl( this, wxID_ANY );
    wxStaticBoxSizer* txCountSizer = new wxStaticBoxSizer( wxVERTICAL, this, _T("Tx") );
    txCountSizer->Add( m_pTextCtrlTxCount, 1, wxEXPAND );
    toolSizer->Add( txCountSizer, 1, wxEXPAND | wxALL, 5 );

    toolSizer->AddStretchSpacer();

    /// clear all button
    wxButton* clearAllBtn = new wxButton( this, CLEAR_ALL_BTN_ID, _T("Clear all") );
    toolSizer->Add( clearAllBtn, 0,  wxALL | wxALIGN_CENTER_VERTICAL, 5 );

    controllerSizer->Add( toolSizer, 0, wxEXPAND, 0 );

    /// protocol box

    mainsizer->Add( controllerSizer, 1, wxEXPAND | wxRIGHT, 5 );


    SetSizer( mainsizer );
	mainsizer->Fit( this );
	mainsizer->SetSizeHints( this );


    m_iRxCount      = 0;
    m_iTxCount      = 0;

    m_timer.Start( 100 );

}


void TerminalPanel::rxLog( uint8_t* data, int size )
{
    wxCriticalSectionLocker locker( m_criticalSection );

    m_rxBuffer.AppendData( data, size );
}


void TerminalPanel::txLog( uint8_t* data, int size )
{
    wxCriticalSectionLocker locker( m_criticalSection );

    m_txBuffer.AppendData( data, size );
}


void TerminalPanel::portLog( const wxString& portName )
{
    //m_pTextCtrlPort->SetValue( portName );
}


void TerminalPanel::baudLog( int baudRate )
{
    //m_pTextCtrlBaud->SetValue( wxString::Format( _T("%d"), baudRate ) );
}


void TerminalPanel::protocolLog( const wxString& protocolName )
{
    //m_pTextCtrlProtocol->SetValue( protocolName );
}


void TerminalPanel::OnTimer( wxTimerEvent& event )
{
    wxCriticalSectionLocker locker( m_criticalSection );

    /// show rx
    size_t size = m_rxBuffer.GetDataLen();
    uint8_t* data = (uint8_t*)m_rxBuffer.GetData();
    m_iRxCount += size;

    m_pTextCtrlRx->AppendText( createString( data, size ) );
    m_pTextCtrlRxCount->SetValue( wxString::Format( _T("%d"), m_iRxCount ) );

    m_rxBuffer.SetDataLen( 0 );

    /// show tx
    size = m_txBuffer.GetDataLen();
    data = (uint8_t*)m_txBuffer.GetData();
    m_iTxCount += size;

    m_pTextCtrlTx->AppendText( createString( data, size ) );
    m_pTextCtrlTxCount->SetValue( wxString::Format( _T("%d"), m_iTxCount ) );

    m_txBuffer.SetDataLen( 0 );
}


wxString TerminalPanel::createString( const uint8_t* data, size_t size )
{
    wxString s = wxEmptyString;

    for ( size_t i = 0; i < size; ++i )
    {
        if ( data[i] < 32 || data[i] == 127 ) { s += wxString::Format( _T("<%02X>"), data[i] ); }
        else { s += wxString::Format( _T("%c"), data[i] ); }
    }

    return s;
}

void TerminalPanel::OnClearAllButton( wxCommandEvent& event )
{
    m_pTextCtrlRx->Clear();
    m_pTextCtrlRxCount->Clear();
    m_iRxCount = 0;

    m_pTextCtrlTx->Clear();
    m_pTextCtrlTxCount->Clear();
    m_iTxCount = 0;
}
