#include "VemsDeviceControllerFSM.h"
#include "VemsDeviceController.h"
#include "VemsDeviceControllerStates.h"
#include "../events/SerialPortScannerEvent.h"
#include "../events/InputEvent.h"
#include "../events/CommunicationEvent.h"
#include "../logger.h"

#include <typeinfo>

VemsDeviceControllerFSM::VemsDeviceControllerFSM( VemsDeviceController* controller )
{
    m_controller    = controller;
    m_state         = new Detecting( this );
}

VemsDeviceControllerFSM::~VemsDeviceControllerFSM()
{
    delete m_state;
}

bool VemsDeviceControllerFSM::isConnected() const
{
    return ( dynamic_cast<Connected*>( m_state ) != NULL );
}

bool VemsDeviceControllerFSM::isDetecting() const
{
     return ( dynamic_cast<Detecting*>( m_state ) != NULL );
}

bool VemsDeviceControllerFSM::isReconnecting() const
{
     return ( dynamic_cast<Reconnecting*>( m_state ) != NULL );
}

void VemsDeviceControllerFSM::resume( bool getPages )
{
    LOGP( wxString::FromAscii( typeid(*m_state).name() ) );
//    m_state = new Resuming( this, getPages );
    m_state = new Resuming( this, true );
}

///RECONNECTING
void VemsDeviceControllerFSM::OnSIPRInput( InputEvent& event )
{
    if ( m_controller->processPage( event ) ) {
        m_state->OnAllPageReceived();
//        LOG
//        m_state = CONNECTED;
//        m_controller->disconnectFSMSiprInput();
        //m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_RECONNECTED );

    }
}

//void VemsDeviceControllerFSM::OnCommunicationEvent( CommunicationEvent& event )
//{
//    LOG
//    m_controller->AddPendingEvent( event );
//}

DeviceInfo VemsDeviceControllerFSM::getDeviceInfo() const
{
    return m_controller->getDeviceInfo();
}

void VemsDeviceControllerFSM::OnDeviceFound( PortScannerEvent& event )
{
    m_state->OnDeviceFound( event );
}

void VemsDeviceControllerFSM::OnDeviceNotFound()
{
    LOGP( wxString::FromAscii( typeid(*m_state).name() ) );
    m_state->OnDeviceNotFound();
}

void VemsDeviceControllerFSM::OnFirmwareInfoFetched(  const FirmwareInfo& fwInfo )
{
    LOG
    m_state->OnFirmwareInfoFetched( fwInfo );
}

void VemsDeviceControllerFSM::OnFirmwareDataSynced()
{
    m_state->OnFirmwareDataSynced();
    //        m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_FIRMWAREDATA_SYNCED );
    //        m_controller->connect();

}

void VemsDeviceControllerFSM::OnTimeout()
{
    m_state->OnTimeout();
}

void VemsDeviceControllerFSM::OnDeviceModeSelected( ECUInfo::EcuMode deviceMode )
{
   m_state->OnDeviceModeSelected( deviceMode );
}

void VemsDeviceControllerFSM::OnBaudrateChanged( size_t baudrate )
{
    m_state->OnBaudrateChanged( baudrate );
}

void VemsDeviceControllerFSM::OnDeviceProtected()
{
    m_state->OnDeviceProtected();
}

void VemsDeviceControllerFSM::OnDeviceNotProtected()
{
    m_state->OnDeviceNotProtected();
}

void VemsDeviceControllerFSM::setState( VemsDeviceControllerState* newState )
{
    delete m_state;
    m_state = newState;
}

void VemsDeviceControllerFSM::setDevice( ECUInfo::Device device )
{
    m_controller->m_deviceInfo.device = device;
}

void VemsDeviceControllerFSM::setDeviceMode( ECUInfo::EcuMode deviceMode )
{
    m_controller->m_deviceInfo.deviceMode = deviceMode;
}

void VemsDeviceControllerFSM::setBaudrate( size_t baudrate )
{
    m_controller->m_deviceInfo.setBaudrate( baudrate );
}

void VemsDeviceControllerFSM::setFirmwareInfo( const FirmwareInfo& fwInfo )
{
    m_controller->m_deviceInfo.firmwareInfo = fwInfo;
}

void VemsDeviceControllerFSM::setFirmwareVersion( const FirmwareVersion& fwVersion )
{
    m_controller->m_megatuneConfig.fwVersion = fwVersion;
}

bool VemsDeviceControllerFSM::isDeviceResumed() const
{
    return m_controller->isDeviceResumed();
}

bool VemsDeviceControllerFSM::isBaudrateResumed() const
{
    return m_controller->isBaudrateResumed();
}

bool VemsDeviceControllerFSM::isDeviceReconnected() const
{
    return m_controller->isDeviceReconnected();
}

void VemsDeviceControllerFSM::deviceResumed()
{
    m_controller->deviceResumed();
}

void VemsDeviceControllerFSM::resumeBaudrate()
{
    m_controller->resumeBaudrate();
}

bool VemsDeviceControllerFSM::hasPreferredBaudrate()
{
    return m_controller->hasPreferredBaudrate();
}

void VemsDeviceControllerFSM::changeToPreferredBaudrate()
{
    m_controller->changeToPreferredBaudrate();
}

void VemsDeviceControllerFSM::startDetectingTimer()
{
    m_controller->startDetectingTimer();
}

void VemsDeviceControllerFSM::selectMegatuneMode()
{
    m_controller->selectMegatuneMode();
}

void VemsDeviceControllerFSM::checkPassword()
{
    m_controller->checkPassword();
}

void VemsDeviceControllerFSM::fetchFirmwareInfo()
{
    m_controller->fetchFirmwareInfo();
}

void VemsDeviceControllerFSM::bootModeFound()
{
    m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_FOUND );
}

void VemsDeviceControllerFSM::sync()
{
    m_controller->sync();
}

void VemsDeviceControllerFSM::connect()
{
    ///TODO
    m_controller->connect();
}

void VemsDeviceControllerFSM::getAllPages()
{
    m_controller->getAllPages();
}

void VemsDeviceControllerFSM::reset()
{
    setState( new Detecting( this ) );
}

/**
    reconnect - device connected / disconnected / reconnect

    resume - device connected / read SDCard, ... / resume

*/

void VemsDeviceControllerFSM::baudrateChanged()
{
    m_controller->baudrateChanged();
}

void VemsDeviceControllerFSM::timeout()
{
    m_controller->timeout();
}

void VemsDeviceControllerFSM::sendDeviceNotFoundEvent()
{
    m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_NOT_FOUND );
}

void VemsDeviceControllerFSM::sendDeviceReconnectedEvent()
{
    m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_RECONNECTED );
}

void VemsDeviceControllerFSM::sendDeviceResumedEvent()
{
    m_controller->sendDeviceControllerEvent( VDC_EVENT_RESUMED );
}

void VemsDeviceControllerFSM::sendDeviceProtectedEvent()
{
    m_controller->sendDeviceControllerEvent( VDC_EVENT_DEVICE_PROTECTED );
}

void VemsDeviceControllerFSM::deviceFound()
{
    m_controller->deviceFound();
}

void VemsDeviceControllerFSM::deviceNotFound()
{
    m_controller->deviceNotFound();
}

bool VemsDeviceControllerFSM::isUseSerialComm() const
{
    return m_controller->isUseSerialComm();
}

bool VemsDeviceControllerFSM::isLeaveBootModeEnabled()
{
    return m_controller->isLeaveBootModeEnabled();
}
