#include "SerialRecognizer.h"
#include "../../common/TxData.h"
#include "../../logger.h"

#include <wx/regex.h>

SerialRecognizer::SerialRecognizer( size_t length )
{
    m_length = length;
    m_received = 0;
    m_buffer = new uint8_t[m_length];
}

SerialRecognizer::~SerialRecognizer()
{
    delete[] m_buffer;
}

bool SerialRecognizer::feed( char c )
{
    wxRegEx regExp( _T("(\\w{1}\\d{1}.\\d{1}_\\w{1}\\d{6})"), wxRE_ADVANCED );

    if ( m_received < m_length )
    {
        m_buffer[m_received] = (uint8_t)c;
        ++m_received;

        wxString response = wxString::From8BitData( (char*)m_buffer, m_received );

        //LOGP( response );

        if ( regExp.Matches( response ) ) {

            m_serial = regExp.GetMatch( response, 1 );
            //LOGP( m_serial );
        }
    }

    return ( m_received == m_length );
}

void SerialRecognizer::reset()
{
    m_received = 0;
    delete[] m_buffer;
    m_buffer = new uint8_t[m_length];
}

TxData* SerialRecognizer::getReceivedData()
{
    return new TxData( m_serial.mb_str() );
}
