#include "GPSIcon.h"

#include <wx/event.h>
#include <wx/menu.h>

#include "../logger.h"
//#include "../rtdatamanager/RTDataMgrBase.h"
#include "../rtdatamanager/RTDataMgrFacade.h"
#include "../DirHelper.h"

#include "../events/GPSEvent.h"
#include "../events/GPSCommEvent.h"


BEGIN_EVENT_TABLE(GPSIcon, VemsToolIconBase)
     //EVT_PAINT(GPSIcon::OnPaint)
    //EVT_LEFT_DOWN(GPSIcon::OnLeftDown)
    //EVT_LEFT_UP(GPSIcon::OnLeftUp)

	EVT_ACTIVATE(GPSIcon::OnActivate)

END_EVENT_TABLE()


GPSIcon::GPSIcon(wxWindow *parent, wxWindowID id, const wxString& icondir, const wxPoint pos, wxEvtHandler* eventHandler, RTDataMgrBase   *rtDataMgr)
:VemsToolIconBase(parent,id,icondir,pos,_T("tbGPS-offline.png"))
{
    //ctor
    m_menu = new wxMenu();

    m_sGpsConn = new wxBitmapLayer(0, icondir +  _T("tbGPS-true.png"));
    m_sGpsNotConn = new wxBitmapLayer(0, icondir +  _T("tbGPS-false.png"));

    m_sGpsNoData = new wxBitmapLayer(0, icondir +  _T("tbGPS-warning.png"));
    m_sGpsOff = new wxBitmapLayer(0, icondir +  _T("tbGPS-offline.png"));


    m_eventHandler = eventHandler;

    SetDataMgr(rtDataMgr);

    m_gpsState = GPSInfo::NOT_CONNECTED;
    m_gpsCommState = GPSInfo::ONLINE;


}

GPSIcon::~GPSIcon()
{
    //dtor
    delete m_menu;
    delete m_sGpsNoData;
    delete m_sGpsNotConn;

    delete m_sGpsConn;
    delete m_sGpsOff;
}

void GPSIcon::IconInit()
{

    IconInitBase(m_sGpsNotConn);

}


void GPSIcon::SetDataMgr(RTDataMgrBase *rtDataMgr)
{
    if(rtDataMgr == NULL) return;

    m_rtDataMgr = rtDataMgr;

    SetMenuState();

}

void GPSIcon::SetMenuState()
{

    if(m_rtDataMgr == NULL) return;

    /*int actualtype =*/ ((RTDataMgrFacade*)m_rtDataMgr)->getInputType();

    wxBitmap tmp = wxBitmap();

    Refresh(false);

}

void GPSIcon::OnLeftDown(wxMouseEvent& evt)
{

//    m_bmpLayers->AddBitmap(m_inputDownBmp);
    Refresh(false);

}

void GPSIcon::OnLeftUp(wxMouseEvent& evt)
{
//    m_bmpLayers->AddBitmap(m_inputUpBmp);
    SetMenuState();
    PopupMenu(m_menu);
}

void GPSIcon::RefreshIcon()
{
    if(m_rtDataMgr == NULL) return;

    int actualtype = ((RTDataMgrFacade*)m_rtDataMgr)->getInputType();

        switch(actualtype){
        case INPUT_TYPE_NONE:

          break;
        case INPUT_TYPE_ECU:

          break;
        case INPUT_TYPE_FILE:

          break;
    }

}

void GPSIcon::OnSelectGaugeInput(wxCommandEvent& event)
{
    if(m_rtDataMgr == NULL) return;

//    int id = event.GetId();

    SetMenuState();
}

void GPSIcon::notify(int i, int idx)
{

}



void GPSIcon::OnActivate(wxActivateEvent& evt)
{
    Logger::logf(LOG_MSG,_T("GPSIcon::OnActivate()"));

}

void GPSIcon::connectEventHandler()
{
    m_eventHandler->Connect( wxEVT_GPS, GPSEventHandler( GPSIcon::OnGpsEvent ),NULL, this);
    m_eventHandler->Connect( wxEVT_GPS_COMM, GPSCommEventHandler( GPSIcon::OnGpsCommState ),NULL, this);
}


void GPSIcon::OnGpsEvent(GPSEvent& event)
{
    m_gpsState = event.state;
    show();
    event.Skip();
}

void GPSIcon::OnGpsCommState(GPSCommEvent& event)
{
    m_gpsCommState = event.state;
    show();
    event.Skip();
}

void GPSIcon::show()
{


/*
    if(m_GpsCommState == GPSInfo::OFFLINE){
        m_bmpLayers->AddBitmap(m_sGpsOff);

    }else{
        if(m_GpsState == GPSInfo::CONNECTED){

            m_bmpLayers->AddBitmap(m_sGpsConn);
        }
		else if(m_GpsState == GPSInfo::NO_GPS_DATA){
            m_bmpLayers->AddBitmap(m_sGpsNoData);
		}
        else{
            m_bmpLayers->AddBitmap(m_sGpsOff);

        }

    }
  */
    Logger::logf(LOG_MSG,_T("GPSIcon::show() m_gpsState=%i,m_gpsCommState=%i"),m_gpsState,m_gpsCommState);


    if( m_gpsCommState == GPSInfo::OFFLINE )
    {
        m_bmpLayers->AddBitmap(m_sGpsOff);
    }
    else
    {
        switch( m_gpsState )
        {
            case GPSInfo::CONNECTED:
                m_bmpLayers->AddBitmap(m_sGpsConn);
                break;

            case GPSInfo::NO_GPS_DATA:
                m_bmpLayers->AddBitmap(m_sGpsNoData);
                break;

            case GPSInfo::NOT_CONNECTED:
                m_bmpLayers->AddBitmap(m_sGpsNotConn);
                break;
        }
    }

    Refresh(false);

}

