#include "SelectInputIcon.h"

#include <wx/menu.h>

#include "../logger.h"
#include "../DirHelper.h"

//#include "../rtdatamanager/RTDataMgrBase.h"
#include "../rtdatamanager/RTDataMgrFacade.h"

#include "../FileDialog.h"
#include "../VemsTuneController.h"

#include "../events/ECUEvent.h"
#include "../events/EcuCommEvent.h"

#include "../events/GPSEvent.h"
#include "../events/GPSCommEvent.h"
#include "../events/ECULogEvent.h"

#define INPUTSTRINGDEPTH    10
#define ECUSTATEDEPTH       20

int idInputECUFast = wxNewId();
int idInputECU = wxNewId();
int idInputFile = wxNewId();
int idInputNone = wxNewId();


BEGIN_EVENT_TABLE(SelectInputIcon, VemsToolIconBase)
     //EVT_PAINT(SelectInputIcon::OnPaint)
    EVT_LEFT_DOWN(SelectInputIcon::OnLeftDown)
    EVT_LEFT_UP(SelectInputIcon::OnLeftUp)

    EVT_MENU(idInputNone, SelectInputIcon::OnSelectGaugeInput)
    EVT_MENU(idInputECU, SelectInputIcon::OnSelectGaugeInput)
    EVT_MENU(idInputECUFast, SelectInputIcon::OnSelectGaugeInput)
	EVT_MENU(idInputFile, SelectInputIcon::OnSelectGaugeInput)

	EVT_ACTIVATE(SelectInputIcon::OnActivate)

END_EVENT_TABLE()


SelectInputIcon::SelectInputIcon(wxWindow *parent, wxWindowID id, const wxString& icondir, const wxPoint pos,
                                 RTDataMgrBase   *rtDataMgr
                                )
         :VemsToolIconBase(parent,id,icondir,pos,_T("input.png"))
{
    //ctor
    Logger::logf(LOG_MSG,_T("SelectInputIcon::SelectInputIcon CTOR start"));

    m_vemsTuneController = NULL;
    m_EcuState = ECUEvent::NOT_CONNECTED;
    m_EcuCommState = ECUCommEvent::OFFLINE;

    //m_gpsState = GPSInfo::State::;
    //m_gpsCommState;

    m_menu = new wxMenu();

    m_menu->AppendRadioItem(idInputNone,_("None"));
    //m_menu->AppendRadioItem(idInputECU,_("ECU test"));
    m_menu->AppendRadioItem(idInputECUFast,_("ECU"));
    m_menu->AppendRadioItem(idInputFile,_("File"));


    m_inputUpBmp   = new wxBitmapLayer(0, icondir +  _T("input.png"));
    m_inputDownBmp = new wxBitmapLayer(0, icondir +  _T("input-down.png"));

    m_iFileBmp = new wxBitmapLayer(INPUTSTRINGDEPTH, icondir +  _T("inputFile.png"));
    m_iECUBmp  = new wxBitmapLayer(INPUTSTRINGDEPTH, icondir +  _T("inputECU.png"));
    m_iNoneBmp = new wxBitmapLayer(INPUTSTRINGDEPTH, icondir +  _T("inputNone.png"));

    m_sConnBmp = new wxBitmapLayer(ECUSTATEDEPTH, icondir +  _T("ECUconn.png"));
    m_sNotCBmp = new wxBitmapLayer(ECUSTATEDEPTH, icondir +  _T("ECUnotconn.png"));
    m_sLoggBmp = new wxBitmapLayer(ECUSTATEDEPTH + 1, icondir +  _T("ECUonlog.png"));

    Logger::logf(LOG_MSG,_T("SelectInputIcon::SelectInputIcon CTOR bef set"));

    SetDataMgr(rtDataMgr);

    Logger::logf(LOG_MSG,_T("SelectInputIcon::SelectInputIcon CTOR END"));
}

SelectInputIcon::~SelectInputIcon()
{
    //dtor
    /*delete m_menu;
    delete m_iFileBmp;
    delete m_iECUBmp;
    delete m_iNoneBmp;
    delete m_sConnBmp;
    delete m_sNotCBmp;
    delete m_sLoggBmp;

    delete m_inputUpBmp;
    delete m_inputDownBmp;*/
}

void SelectInputIcon::IconInit()
{

    IconInitBase(m_inputUpBmp);

}


void SelectInputIcon::SetDataMgr(RTDataMgrBase *rtDataMgr)
{
    if(rtDataMgr == NULL) return;

    m_rtDataMgr = rtDataMgr;

    SetMenuState();

}

void SelectInputIcon::SetMenuState()
{

    if(m_rtDataMgr == NULL) return;

    int actualtype = ((RTDataMgrFacade*)m_rtDataMgr)->getInputType();

    wxBitmap tmp = wxBitmap();

    switch(actualtype){
        case INPUT_TYPE_NONE:
            m_menu->Check(idInputNone,1);
            m_bmpLayers->AddBitmap(m_iNoneBmp);
          break;
        case INPUT_TYPE_ECU:
            m_menu->Check(idInputECU,1);
            m_bmpLayers->AddBitmap(m_iECUBmp);
          break;
        case INPUT_TYPE_FILE:
            m_menu->Check(idInputFile,1);
            m_bmpLayers->AddBitmap(m_iFileBmp);
          break;
        case INPUT_TYPE_RINGBUFFER:
            m_menu->Check(idInputECUFast,1);
            m_bmpLayers->AddBitmap(m_iECUBmp);
          break;

    }

    Refresh(false);

}

void SelectInputIcon::OnLeftDown(wxMouseEvent& evt)
{

    m_bmpLayers->AddBitmap(m_inputDownBmp);
    Refresh(false);

}

void SelectInputIcon::OnLeftUp(wxMouseEvent& evt)
{
    m_bmpLayers->AddBitmap(m_inputUpBmp);
    SetMenuState();
    PopupMenu(m_menu);
}

void SelectInputIcon::RefreshIcon()
{
    if(m_rtDataMgr == NULL) return;

    int actualtype = ((RTDataMgrFacade*)m_rtDataMgr)->getInputType();

        switch(actualtype){
        case INPUT_TYPE_NONE:

          break;
        case INPUT_TYPE_ECU:

          break;
        case INPUT_TYPE_FILE:

          break;
    }

}

void SelectInputIcon::OnSelectGaugeInput(wxCommandEvent& event)
{
    if(m_rtDataMgr == NULL) return;

    int id = event.GetId();

/*
    if(id == idInputNone){

        m_vemsTuneController->selectInputNone();

    }else*/ if(id == idInputECU){
/*
//        Logger::logf(LOG_MSG,_T("SelectInputIcon::OnSelectGaugeInput: idInputECU start"));

        ((RTDataMgrFacade*)m_rtDataMgr)->SetInput(new RTDataMgrSerial(VemsProtocolHelper::getVemsProtocol(),MTConfigHelper::getMTConfigData()));

//        ((RTDataMgrFacade*)m_rtDataMgr)->SetInput(m_rtDataMgrSerial);

//        Logger::logf(LOG_MSG,_T("SelectInputIcon::OnSelectGaugeInput: idInputECU end"));
*/

        //VemsProtocol *m_vptest = m_vemsTuneController->getVemsProtocol();
/*
        FirmwareInfo *fwi = m_vemsTuneController->getVemsProtocol();
        wxString temp = _T("VemsProtocol::getFirmwareInfo test result:\n");
        Logger::logf(LOG_MSG,_T("Firmwareinfo test START"));

        for(short a=0;a<20;a++) {
            Logger::logf(LOG_MSG,_T("Firmwareinfo test, count=%i"),a);
            fwi = NULL;
            try{
                fwi = m_vptest->getFirmwareInfo();
            }catch(SerialException e){
                Logger::logf(LOG_MSG,_T("Firmwareinfo test Exception occured!!!!"));
                continue;
            }
            if(fwi == NULL) continue;
            temp += fwi->getCompilationTime() + _T("\n");
        }



        Logger::logf(LOG_MSG,_T("Firmwareinfo test END"));
        Logger::logf(LOG_MSG,temp);
*/
    }else if(id == idInputFile){

//        ((RTDataMgrFacade*)m_rtDataMgr)->SetInput(m_rtDataLogSeq);


        FileDialog fdlg(this);
        fdlg.SetDir( DirHelper::GetPath(dLOG) );

        if( fdlg.ShowModal() == wxID_OK) {
            m_vemsTuneController->selectInputFile(fdlg.GetPath());
        }


    }else if(id == idInputECUFast){

        //Logger::logf(LOG_MSG,_T("SelectInputIcon m_vemsTuneController = %p"),m_vemsTuneController);

        m_vemsTuneController->setECUOnline(true);

        m_vemsTuneController->selectInputECU();
    }

    SetMenuState();
}

void SelectInputIcon::OnActivate(wxActivateEvent& evt)
{
    Logger::logf(LOG_MSG,_T("SelectInputIcon::OnActivate()"));

}

void SelectInputIcon::SetVemsTuneController(VemsTuneController *ctrl)
{
    m_vemsTuneController = ctrl;

    m_vemsTuneController->Connect( wxEVT_ECU, ECUEventHandler( SelectInputIcon::OnECUEvent ),NULL, this);
    m_vemsTuneController->Connect( wxEVT_ECU_COMM, ECUCommEventHandler( SelectInputIcon::OnECUCommState ),NULL, this);

    m_vemsTuneController->Connect( wxEVT_GPS, GPSEventHandler( SelectInputIcon::OnGPSEvent ),NULL, this);
    m_vemsTuneController->Connect( wxEVT_GPS_COMM, GPSCommEventHandler( SelectInputIcon::OnGPSCommEvent ), NULL, this );

    m_vemsTuneController->Connect( wxEVT_ECU_LOG, ECULogEventHandler(SelectInputIcon::OnLogEvent), NULL, this );

    /// ECULogEvent

}


void SelectInputIcon::OnECUEvent(ECUEvent& event)
{
    m_EcuState = event.state;
    showEcuState();
    event.Skip();
}

void SelectInputIcon::OnECUCommState(ECUCommEvent& event)
{
    m_EcuCommState = event.state;
    showEcuState();
    event.Skip();
}

void SelectInputIcon::showEcuState()
{

    if(m_EcuCommState == ECUCommEvent::OFFLINE){
        m_bmpLayers->AddBitmap(m_sNotCBmp);

    }else{
        if(m_EcuState == ECUEvent::CONNECTED){

            m_bmpLayers->AddBitmap(m_sConnBmp);
        }
        else{
            m_bmpLayers->AddBitmap(m_sNotCBmp);

        }

    }

    Refresh(false);

}

void SelectInputIcon::OnGPSEvent( GPSEvent& event )
{
    //Logger::logf( LOG_MSG, _T("TbTbGPS::OnGPSEvent() - state : %i"), event.state );
    //m_gpsState = event.state;
    event.Skip();
}

void SelectInputIcon::OnGPSCommEvent( GPSCommEvent& event )
{
    //m_gpsCommState = event.state;
    event.Skip();
}

void SelectInputIcon::OnLogEvent(ECULogEvent &event)
{



}

