#ifndef DOWNLOADMANAGERBASE_H
#define DOWNLOADMANAGERBASE_H

#include <wx/wx.h>
#include "../common/ProcessBase.h"
#include "../events/DownloadingStatusEvent.h"
#include "../Observable.h"
#include "../ObserverBase.h"
#include "../common/types/wxVariantHashArray.h"
#include "ContentLoaderBase.h"


class DownloaderProcess: public ProcessBase, public Observable
{
    public:
        DownloaderProcess(int id, ContentLoaderBase* loader, wxVariantHash params);

        virtual bool process();
        virtual void reset() {};

        wxVariantHash& GetParams() { return m_params; };

        int GetTryCount() { return m_tryCount; };

        bool IsTimeOuted(int timeoutMS);

    protected:

        ContentLoaderBase*  m_loader;
        wxVariantHash       m_params;

    private:

        int m_tryCount;
        int m_id;
        wxDateTime m_startTime;

        virtual void finished();
};

WX_DECLARE_HASH_MAP( int, DownloaderProcess*, wxIntegerHash, wxIntegerEqual, DownloaderProcessHash);

class ContentLoaderManager: public wxEvtHandler, public ObserverBase
{
    public:
        ContentLoaderManager(int id, ContentLoaderBase * loader, wxEvtHandler* resultHandler, int threadCount = 1, int timebasems = 50);
        virtual ~ContentLoaderManager();

        void SetLoaderProvider(ContentLoaderBase * loader) { m_loaderProvider = loader; }

        void AddQuery(wxVariantHash parameters);
        void AddURL(const wxString& url);
        void AddFile(const wxString& path);

        virtual void notify( wxVariantHash& hash );

        void SetMaxThreads(int count) { m_maxThreadCount = count; };
        void SetTryCount(int count) { m_tryCount = count; };
        void SetTimeout(int seconds) { m_timeout = seconds*1000; };
        void SetTimeoutMs(int milliseconds) { m_timeout = milliseconds; };

    protected:

        void OnTimer(wxTimerEvent& evt);
        //void OnContentProgressEnd(wxCommandEvent& evt);
        void OnContentProgressEnd(DownloadingStatusEvent& evt);

        void RemoveProcess(int pid, bool isKill = false);

        void ThrowRemoveThread(int pid);

    private:

        int m_id;

        enum {
            idTimer
        };

        ContentLoaderBase *     m_loaderProvider;

        wxVariantHashArray      m_queue;

        wxTimer                 m_mainTimer;

        wxEvtHandler*           m_resultHandler;

        DownloaderProcessHash   m_processes;

        size_t  m_maxThreadCount;
        int     m_tryCount;
        int     m_timeout;
        int     m_timeBase;
};


#endif // DOWNLOADMANAGERBASE_H

