#include "WebInterfacePanel.h"


#include <wx/tokenzr.h>
#include <wx/artprov.h>
#include <wx/imaglist.h>

#include <wx/filename.h>
#include <wx/dir.h>
#include "../DirHelper.h"
#include "../logger.h"
#include "WebHelper.h"
#include "../common/FileDropTarget.h"
#include "../common/ThreadProcess.h"

#include "../configmanager/ConfigHelper2.h"

#include "../UserLog.h"

#include "../wxListCtrlHelper.h"

BEGIN_EVENT_TABLE(WebInterfacePanel, wxFrame)

    EVT_COMBOBOX(idDevList, WebInterfacePanel::OnDevList)

    //EVT_SIZE(WebInterfacePanel::OnSize)

    EVT_LIST_COL_CLICK(idRemoteList, WebInterfacePanel::OnListHeaderClick)
    EVT_LIST_COL_CLICK(idLocalList, WebInterfacePanel::OnListHeaderClick)

    EVT_LIST_ITEM_RIGHT_CLICK(idLocalList, WebInterfacePanel::OnLocalItemClick)

    EVT_BUTTON(idDownloadButton, WebInterfacePanel::OnButton)
    EVT_BUTTON(idUploadButton, WebInterfacePanel::OnButton)
    EVT_BUTTON(idInstallButton, WebInterfacePanel::OnButton)
    EVT_BUTTON(idCloseButton, WebInterfacePanel::OnButton)


    EVT_MENU(idInstallMenu, WebInterfacePanel::OnMenu)

    EVT_TIMER(itTimeoutTimer, WebInterfacePanel::OnTimeOutTimer)

    EVT_CLOSE(WebInterfacePanel::OnClose)

    EVT_COMMAND_ENTER(idListDownloadEnd,WebInterfacePanel::OnDownloadList)
    EVT_COMMAND_ENTER(idListDownloadStart,WebInterfacePanel::OnDownloadList)

    EVT_CHECKBOX(idCBAutoInstall, WebInterfacePanel::OnAutoInstall)

END_EVENT_TABLE()


WebInterfacePanel::WebInterfacePanel(wxWindow *parent, int id, const wxString& title, const wxPoint& pos, const wxSize& size, long style)
    :m_timeoutTimer(this,itTimeoutTimer)
{
    //ctor
    style |= wxSTAY_ON_TOP;

    Create(parent, id, title, pos, size, style);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
    ClearBackground();



#if USE_TEMPLATE_DND
    new FileDropTarget<WebInterfacePanel>( this );
#endif

    m_contentLoader = new ThreadProcess<WebInterfacePanel>( this );

    SetIcon(wxIcon(_T("vemsLogoIcon2")));

    Connect(idTransferEnd,wxEVT_COMMAND_ENTER, wxCommandEventHandler(WebInterfacePanel::OnTransferDone));
    Connect(idTransferProgress,wxEVT_COMMAND_ENTER, wxCommandEventHandler(WebInterfacePanel::OnTransferProgress));

}

WebInterfacePanel::~WebInterfacePanel()
{
    //dtor
}

void WebInterfacePanel::OnDownloadList(wxCommandEvent& evt)
{

    switch(evt.GetId()) {
        case idListDownloadStart:

            UserLog::uLog(ULOG_INFO, _T("Download firmware list from: ")+evt.GetString());

            break;

        case idListDownloadEnd:{

            wxString list = evt.GetString();

            m_timeoutTimer.Stop();

            ShowLoaderLabel(false);

            if(list == wxEmptyString) {

                wxMessageBox(_T("The server (") +m_desc.GetServer()+ _T(") is not available.\nMaybe the server is offline, or this computer is not connected to the internet."),_T("Error"),wxOK |wxICON_ERROR );

                return;
            }

            m_remoteList->DeleteAllItems();

            FillRemoteListByStr(list);

            m_remoteList->SetColumnWidth(1,-1);
            m_remoteList->SetColumnWidth(2,-1);

            wxListCtrlHelper::SortColoumData(m_remoteList,0,false);

            m_romoteCols[0] = true;

            }break;
    }

}

void WebInterfacePanel::OnClose(wxCloseEvent& evt)
{
    evt.Skip();
}

void WebInterfacePanel::OnTimeOutTimer(wxTimerEvent& evt)
{
    m_contentLoader->kill();
    ShowLoaderLabel(false);

    wxMessageBox(_T("The server (") + m_desc.GetServer() + _T(") is not available."),_T("Error"),wxOK |wxICON_ERROR );
}

void WebInterfacePanel::OnAutoInstall(wxCommandEvent& evt)
{
    ConfigBase* cfg = ConfigHelper2::getConfig();
    cfg->setPath(_T("/webservice"));
    cfg->write(_T("auto_install"), evt.IsChecked() );
    cfg->save();
    cfg->flush();
}

void WebInterfacePanel::ShowLoaderLabel(bool isShow)
{
    wxSizerItem * sitem = m_mainsizer->GetItem( m_loadingLabel, true );

    if(!sitem) return;

    sitem->Show(isShow);
    m_mainsizer->Layout();
}

/// don't call GUI elements
bool WebInterfacePanel::process()
{
    wxString devstring = m_remPathList->GetValue();

    wxString link;

    if(m_desc.Handler != wxEmptyString)
        link = m_desc.GetServer() + _T("?command=list&type=") + m_desc.RemotePathTypeList[devstring];
    else
        link = m_desc.BaseServer;

    wxCommandEvent evts(wxEVT_COMMAND_ENTER,idListDownloadStart);
    evts.SetString(link);
    ProcessEvent(evts);

    Logger::logf(LOG_MSG,_T("WebInterfacePanel download list from: ")+link);

    wxString list = WebHelper::DownloadPage( link );


    wxCommandEvent evt(wxEVT_COMMAND_ENTER,idListDownloadEnd);
    evt.SetString(list);
    AddPendingEvent(evt);

    return false;
}

void WebInterfacePanel::FillRemoteListByStr(const wxString& content)
{

    if(m_desc.Handler == wxEmptyString) {

        ParseApacheFileList(content);

    } else {

        wxArrayString files = wxStringTokenize(content,_T("\n"));
        for(uint16_t a=0; a< files.GetCount();a++){
            AddLine(files[a], m_remoteList);
        }

    }

}

void WebInterfacePanel::ParseApacheFileList(const wxString& lines)
{

    wxRegEx exp1;
    exp1.Compile(_T("<tr>.+>(.+)<\\/a>.+<td align=\"right\">(.+)<\\/td><td align=\"right\">(.+)<\\/td><\\/tr>"));

    wxArrayString arr = wxStringTokenize(lines,_T("\n"));

    for(size_t a=0; a<arr.GetCount();a++){
        if(!exp1.Matches(arr[a])) continue;
        wxString fname = exp1.GetMatch(arr[a],1);
        wxString fdate = exp1.GetMatch(arr[a],2);
        wxString fsize = exp1.GetMatch(arr[a],3);
        wxDateTime date;
        date.ParseDateTime(fdate);

        wxString size = GetSizeFromHumanReadableSize(fsize);

        wxString sline = fname + _T("|") + size + _T("|") + date.FormatISODate() + _T(" ") + date.FormatISOTime();

        AddLine(sline, m_remoteList);
    }


}

wxString WebInterfacePanel::GetSizeFromHumanReadableSize(const wxString& str)
{
    wxString last = str.GetChar( str.Len()-1 );

    wxString tmpstr = str;

    if( last.CmpNoCase(_T("k")) == 0 ) {

        long tmp;
        tmpstr.RemoveLast();
        tmpstr.ToLong(&tmp);
        tmp *= 1024;
        return wxString::Format(_T("%i"),tmp);

    } else if( last.CmpNoCase(_T("m")) == 0 ) {

        long tmp;
        tmpstr.RemoveLast();
        tmpstr.ToLong(&tmp);
        tmp *= 1024*1024;
        return wxString::Format(_T("%i"),tmp);

    } else {

        return str;
    }
}

#if USE_TEMPLATE_DND
bool WebInterfacePanel::onDropFiles( const wxArrayString& filenames )
#else
bool WebInterfacePanel::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString&  filenames)
#endif
{
    for(size_t a=0; a < filenames.GetCount(); a++){

        wxFileName fn(filenames[a]);

        if( !fn.FileExists() ) continue;

        wxString filen = fn.GetFullName();

        wxString targetpath = DirHelper::GetPath(dWEBCACHE) + m_desc.LocalPath + DirHelper::Separ();

        if( wxFileName::FileExists(targetpath + filen) ) continue;

        wxRegEx rx(m_desc.FileFilter);
        if( !rx.Matches( filen ) ) continue;

        if( !wxCopyFile( filenames[a], targetpath + filen ) ) continue;

        if( !Install( filen ) ) continue;

        wxString line = filen + _T("|");
        line += fn.GetSize().ToString() + _T("|");
        line += fn.GetModificationTime().Format(_T("%Y.%m.%d. %H:%M"));

        int res = AddLine( line, m_localList );

        if( res < 0 ) continue;

    }

    return true;

}

bool WebInterfacePanel::Install(const wxString& filename)
{
    if( m_desc.LocalInstaller == NULL) return false;

    bool res = m_desc.LocalInstaller->DoInstall( filename, &m_desc)!=0?true:false;

    long item = SearchCell(m_localList, 0, filename);

    if(item != wxNOT_FOUND) {
        wxString status = m_desc.LocalChecker->GetStatus( filename, &m_desc);
        SetListCtrlCellText( m_localList, item, 3, status );
    }

    return res;
}

bool WebInterfacePanel::Install(long item)
{
    if( m_desc.LocalInstaller == NULL) return false;

    wxString filename = GetListCtrlCellText( m_localList, item, 0);

    bool res = m_desc.LocalInstaller->DoInstall( filename, &m_desc)!=0?true:false;

    SetListCtrlCellText( m_localList, item, 3, m_desc.LocalChecker->GetStatus( filename, &m_desc));

    return res;
}

bool WebInterfacePanel::Install()
{
    if( m_desc.LocalInstaller == NULL) return false;

    long item = -1;

    while(1){
        item = m_localList->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

        if(item == -1) break;

        Install(item);
}

    return true;

}

long WebInterfacePanel::SearchCell(wxListCtrl* listctrl, int column, const wxString& text)
{

    long item = -1;

    while(1){
        item = listctrl->GetNextItem(item, wxLIST_NEXT_ALL,wxLIST_STATE_DONTCARE);

        if(item == -1) break;

        wxListItem info;
        info.SetMask(wxLIST_MASK_TEXT );
        info.SetId(item);
        info.SetColumn(column);
        listctrl->GetItem(info);

        if( info.GetText() == text ) return item;
    }

    return wxNOT_FOUND;
}

void WebInterfacePanel::OnMenu(wxCommandEvent &evt)
{
    int id = evt.GetId();

    switch(id){
        case idInstallMenu:
            Install();
            break;


        default:
            break;

    }
}

void WebInterfacePanel::OnButton(wxCommandEvent &evt)
{
    int id = evt.GetId();

    switch(id){
        case idDownloadButton:
            DownloadSelectedFiles();
            //DeselectAllItems(m_remoteList);
            break;

        case idUploadButton:
            UploadSelectedFiles();
            break;

        case idInstallButton:
            Install();
            break;

        case idCloseButton:
            Close();
            break;

        default:
            break;

    }
}

void WebInterfacePanel::ReSize()
{
    Layout();

    int a = m_remoteList->GetColumnWidth(1);
    int b = m_remoteList->GetColumnWidth(2);
    int c = m_localList->GetColumnWidth(3);

    m_remoteList->SetColumnWidth(0, m_remoteList->GetClientSize().x - a - b);
    m_localList->SetColumnWidth(0, m_localList->GetClientSize().x - a - b - c);
}

void WebInterfacePanel::OnSize(wxSizeEvent &evt)
{
    ReSize();
}

void WebInterfacePanel::Init(WIPDesc& desc)
{

    if( !desc.IsOk() ) {
        Show();
        wxMessageBox(_T("Wrong panel descriptor!"),_T("Error"));
        return;
    }

    //Logger::logf(LOG_MSG,_T("WebInterfacePanel::Init START"));

    m_desc = desc;

    SetTitle(desc.Title);

    wxStaticText *title = new wxStaticText(this, wxID_ANY, desc.Title,wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);

    wxFont font = title->GetFont();
    font.SetPointSize(13);
    font.SetWeight(wxFONTWEIGHT_BOLD);
    title->SetFont(font);

    wxStaticText *dtitle = new wxStaticText(this, wxID_ANY, _T("Device: "),wxDefaultPosition,wxDefaultSize);
    m_remPathList = new wxComboBox(this, idDevList, _T(""), wxDefaultPosition, wxDefaultSize, 0, NULL, wxCB_READONLY );
    //m_remPathList->SetWindowStyleFlag( wxCB_READONLY );
    for(StringHash::iterator it = desc.RemotePathTypeList.begin(); it != desc.RemotePathTypeList.end(); it++ ) {
       m_remPathList->Append( it->first );
    }
    m_remPathList->SetSelection(0);

    m_autoInstall = new wxCheckBox(this, idCBAutoInstall, _T("Auto install after download"));

    ConfigBase* cfg = ConfigHelper2::getConfig();
    cfg->setPath(_T("/webservice"));
    bool autoinstall = cfg->read(_T("auto_install"),true);

    m_autoInstall->SetValue(autoinstall);

    //wxButton *rb = new wxButton(this, idRefreshButton, _T("Refresh"));

    wxStaticText *remtitle = new wxStaticText(this, wxID_ANY, _T("Web"),wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);
    wxStaticText *loctitle = new wxStaticText(this, wxID_ANY, _T("Local"),wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);

    wxStaticText *remdesc = new wxStaticText(this, wxID_ANY, desc.BaseServer,wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);
    wxStaticText *locdesc = new wxStaticText(this, wxID_ANY, DirHelper::GetPath(dWEBCACHE) + desc.LocalPath,wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);

    m_loadingLabel = new wxStaticText(this, wxID_ANY, _T("Loading from the server..."),wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);

    remtitle->SetFont(font);
    loctitle->SetFont(font);

    m_remoteList = new wxListCtrl(this, idRemoteList, wxDefaultPosition, wxSize(400,300), wxLC_REPORT);
    m_localList  = new wxListCtrl(this, idLocalList , wxDefaultPosition, wxSize(400,300), wxLC_REPORT);

#if !USE_TEMPLATE_DND
    m_localList->SetDropTarget(this);
#endif

    InitLists();

    m_progressBar = new ProgressBar(this,wxID_ANY,wxDefaultPosition,wxSize(-1,20));
    m_progressBar->SetWindowStyle( wxBORDER_SUNKEN );
    m_progressBar->SetProgressStyle(PB_PERCENTTEXT_INNER);
    m_progressBar->Update(0);



    if( m_desc.TransferDirection & WIPDesc::WIP_DOWNLOAD )
        m_downButton = new wxButton(this, idDownloadButton, _T("Download"));

    if( m_desc.TransferDirection & WIPDesc::WIP_UPLOAD )
        m_upButton = new wxButton(this, idUploadButton, _T("Upload"));


    m_installButton = new wxButton(this, idInstallButton, m_desc.PopupMenuStr );

    //m_downButton->Enable(false);


    m_popupRightMenu = new wxMenu();
    m_popupRightMenu->Append(idInstallMenu, _T("---") );


    ///  sizers

    //wxGridBagSizer * mainsizer = new wxGridBagSizer();      mainsizer->Add(title, wxGBPosition(0,0), wxGBSpan(1,2), wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, 5);
    m_mainsizer = new wxBoxSizer(wxVERTICAL);

    m_mainsizer->Add(title, 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, 5 );

        wxBoxSizer * devsizer = new wxBoxSizer(wxHORIZONTAL);

        devsizer->Add(dtitle, 0, wxALIGN_CENTER_VERTICAL );
        devsizer->Add(m_remPathList, 1, wxLEFT, 5);



    m_mainsizer->Add(devsizer, 0, wxALL, 5);

    m_mainsizer->Add(new wxStaticLine(this), 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, 5 );

        wxFlexGridSizer * listsizer = new wxFlexGridSizer(2);
        listsizer->AddGrowableCol(0);
        listsizer->AddGrowableCol(1);
        listsizer->AddGrowableRow(2);

        listsizer->Add(remtitle,0,wxALIGN_CENTER_HORIZONTAL|wxALL,1);
        listsizer->Add(loctitle,0,wxALIGN_CENTER_HORIZONTAL|wxALL,1);
        listsizer->Add(remdesc,0,wxALIGN_CENTER_HORIZONTAL|wxALL,1);
        listsizer->Add(locdesc,0,wxALIGN_CENTER_HORIZONTAL|wxALL,1);
            wxBoxSizer * rs = new wxBoxSizer(wxVERTICAL);
            rs->Add(m_loadingLabel,0,wxALIGN_CENTER_HORIZONTAL|wxALL,5);
            rs->Add(m_remoteList,1,wxEXPAND|wxALL,1);
        listsizer->Add(rs,1,wxEXPAND|wxALL,1);
        listsizer->Add(m_localList ,1,wxEXPAND|wxALL,1);

        listsizer->Add(m_autoInstall, 0, wxALIGN_RIGHT|wxALL | wxALIGN_CENTER_VERTICAL,1);

        listsizer->Add(m_installButton, 0, wxALIGN_RIGHT|wxALL,1);

    m_mainsizer->Add(listsizer,1,wxEXPAND | wxALL, 3);

    m_mainsizer->Add(new wxStaticLine(this), 0, wxALL | wxEXPAND | wxALIGN_CENTER_HORIZONTAL, 5 );

    m_mainsizer->Add(m_progressBar,0,wxALL | wxEXPAND,5);

    wxBoxSizer * buttonsizer = new wxBoxSizer(wxHORIZONTAL);

    if( m_desc.TransferDirection & WIPDesc::WIP_DOWNLOAD )
        buttonsizer->Add(m_downButton,0,wxALL | wxALIGN_CENTER_HORIZONTAL,5);

    if( m_desc.TransferDirection & WIPDesc::WIP_UPLOAD )
        buttonsizer->Add(m_upButton,0,wxALL | wxALIGN_CENTER_HORIZONTAL,5);

    buttonsizer->Add(new wxButton(this, idCloseButton, _T("Close dialog")),0,wxALL | wxALIGN_CENTER_HORIZONTAL,5);

    m_mainsizer->Add(buttonsizer, 0, wxALIGN_CENTER_VERTICAL | wxALIGN_CENTER_HORIZONTAL );

	SetSizer(m_mainsizer);
	m_mainsizer->Fit(this);
	m_mainsizer->SetSizeHints(this);

    Show();

    FillRemoteList( m_remPathList->GetStringSelection() );

    FillLocalList();

    m_dataTransMgr = new DataTransferMgr();
    m_dataTransMgr->registerObserver( this );

    ReSize();

    Connect(wxEVT_SIZE, wxSizeEventHandler(WebInterfacePanel::OnSize));

    //Logger::logf(LOG_MSG,_T("WebInterfacePanel::Init END"));
}

wxString WebInterfacePanel::GetListCtrlCellText(wxListCtrl * lc, long item, int col)
{
    wxListItem info;
    info.SetColumn(col);
    info.SetId(item);
    info.SetMask(wxLIST_MASK_TEXT);

    lc->GetItem(info);

    return info.GetText();
}

void WebInterfacePanel::SetListCtrlCellText(wxListCtrl * lc, long item, int col, const wxString &str)
{
    lc->SetItem( item, col, str);
}

void WebInterfacePanel::SetListCtrlCellText(wxListCtrl * lc, int col, const wxString &str)
{
    long item = lc->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

    lc->SetItem( item, col, str);
}

wxString WebInterfacePanel::GetListCtrlCellText(wxListCtrl * lc, int col)
{
    long item = lc->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

    wxListItem info;
    info.SetColumn(col);
    info.SetId(item);
    info.SetMask(wxLIST_MASK_TEXT);

    lc->GetItem(info);

    return info.GetText();
}

void WebInterfacePanel::OnLocalItemClick(wxListEvent &evt)
{
    if( !m_desc.LocalInstaller ) return;

    if( !m_popupRightMenu ) return;

    wxString status = GetListCtrlCellText(m_localList,3);

    int sc = m_desc.LocalChecker->StatusChecker( status );

    m_popupRightMenu->SetLabel(idInstallMenu, m_desc.PopupMenuStrings[sc]);

    PopupMenu( m_popupRightMenu );

}

bool WebInterfacePanel::UploadSelectedFiles()
{
    m_dataTransMgr->SetUploadProvider( m_desc.UploaderProvider );

    wxListCtrl * lc = m_localList;

    wxString page = WebHelper::DownloadPage( m_desc.GetServer() + _T("?command=path&type=") + m_desc.RemotePathTypeList[m_remPathList->GetStringSelection()] );

    long item = -1;
    while(1)
    {
        item = lc->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        if ( item == -1 )
            break;

        wxString name = wxListCtrlHelper::GetCellLabel(lc, item, 0);

        m_dataTransMgr->AddItem( page, m_desc.LocalPath + _T("/") + name);
    }

    m_remPathList->Enable( false );
    m_remoteList->Enable( false );

    m_transferedFiles.Clear();

    if( m_desc.TransferDirection & WIPDesc::WIP_DOWNLOAD )
        m_downButton->Enable( false );

    if( m_desc.TransferDirection & WIPDesc::WIP_UPLOAD )
        m_upButton->Enable( false );


    return m_dataTransMgr->Start( WIPDesc::WIP_UPLOAD );

}

bool WebInterfacePanel::DownloadSelectedFiles()
{

    wxListCtrl * lc = m_remoteList;

    wxString filepathurl;
    if(m_desc.Handler == wxEmptyString)
        filepathurl = m_desc.BaseServer;
    else
        filepathurl = WebHelper::DownloadPage( m_desc.GetServer() + _T("?command=url&type=") + m_desc.RemotePathTypeList[m_remPathList->GetStringSelection()] );

    long item = -1;
    while(1)
    {
        item = lc->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
        if ( item == -1 )
            break;

        wxString name = wxListCtrlHelper::GetCellLabel(lc, item, 0);

        m_dataTransMgr->AddItem( filepathurl + _T("/") + name, m_desc.LocalPath + _T("/") + name);
    }

    m_remPathList->Enable( false );
    m_remoteList->Enable( false );

    m_transferedFiles.Clear();

    if( m_desc.TransferDirection & WIPDesc::WIP_DOWNLOAD )
        m_downButton->Enable( false );

    if( m_desc.TransferDirection & WIPDesc::WIP_UPLOAD )
        m_upButton->Enable( false );

    return m_dataTransMgr->Start( WIPDesc::WIP_DOWNLOAD );
}

void WebInterfacePanel::OnListHeaderClick(wxListEvent &evt)
{
    //Logger::logf(LOG_MSG,_T("OnListHeaderClick"));

    int col = evt.GetColumn();

    wxListCtrl * lc;

    bool isAsc = true;

    switch(evt.GetId()){

        case idRemoteList:
            lc = m_remoteList;
            isAsc = m_romoteCols[col];
            m_romoteCols[col] = !isAsc;
            break;

        case idLocalList:
            lc = m_localList;
            isAsc = m_localCols[col];
            m_localCols[col] = !isAsc;
            break;

        default :
            return;
    }

    wxListCtrlHelper::SortColoumData(lc, col, isAsc);

}

void WebInterfacePanel::MoveUpLine(wxListCtrl * lc, int idx, int dist)
{
    if( (idx - dist) < 0) return;

    uint8_t a=0;

    for(a=0; a < 4; a++){

        wxListItem info1;
        info1.SetColumn(a);
        info1.SetId(idx-dist);
        info1.SetMask(wxLIST_MASK_TEXT);
        lc->GetItem(info1);

        wxListItem info2;
        info2.SetColumn(a);
        info2.SetId(idx);
        info2.SetMask(wxLIST_MASK_TEXT);
        lc->GetItem(info2);

        info1.SetId(idx);
        info2.SetId(idx-dist);

        lc->SetItem(info1);
        lc->SetItem(info2);
    }

}

void WebInterfacePanel::OnDevList(wxCommandEvent &evt)
{

    FillRemoteList(m_remPathList->GetValue());

}

void WebInterfacePanel::DeselectAllItems(wxListCtrl * lc)
{
    long item = -1;
    while( 1 )
    {
        item = lc->GetNextItem(item );
        if ( item == -1 )
            break;

        Logger::logf(LOG_MSG,_T("item = %i"),item);

        lc->SetItemState( item , wxLIST_STATE_SELECTED , wxLIST_MASK_STATE  );
    }

}

void WebInterfacePanel::FillRemoteList(const wxString& devstring)
{
//    Logger::logf(LOG_MSG,_T("WebInterfacePanel::FillRemoteList START"));

    m_timeoutTimer.Start( m_desc.Timeout * 1000, true );

    ShowLoaderLabel(true);

    m_contentLoader->start();
}

int WebInterfacePanel::AddLine(const wxString& str, wxListCtrl * listctrl)
{
//    Logger::logf(LOG_MSG,_T("WebInterfacePanel::AddLine START"));

    wxArrayString cols = wxStringTokenize(str, _T("|"));

    if(cols.GetCount() < 3) return -1;


    wxRegEx rx(m_desc.FileFilter);
    if( !rx.Matches( cols[0] ) ) return -1;

    wxListItem item0, item1, item2, item3;

    int i = listctrl->GetItemCount();

    item0.SetId( i );
    item1.SetId( i );
    item2.SetId( i );
    item3.SetId( i );

    item0.SetColumn(0);
    item1.SetColumn(1);
    item2.SetColumn(2);
    item3.SetColumn(3);

    item0.SetText(cols[0]);
    item1.SetText(cols[2]);

    long fsize = 0;
    cols[1].ToLong(&fsize);
    item2.SetText(wxFileName::GetHumanReadableSize( (const wxULongLong)fsize ) );
    item2.SetData( fsize );

    long tmp = listctrl->InsertItem(item0);
    listctrl->SetItemData(tmp, i);
    listctrl->SetItem(item1);
    listctrl->SetItem(item2);

    if( m_desc.LocalChecker && listctrl == m_localList ){

        wxString status = m_desc.LocalChecker->GetStatus( cols[0], &m_desc );
        //Logger::logf(LOG_MSG,_T("status = ")+status);
        item3.SetText( status );
        listctrl->SetItem(item3);

    }


    return 0;

}

void WebInterfacePanel::InitLists()
{
//    wxString locpath = DirHelper::GetPath(dWEBCACHE) + m_desc;

    int w = m_remoteList->GetSize().x-4;

    wxListItem info1;
    wxListItem info2;
    wxListItem info3;
    wxListItem info4;

/*
    wxImageList *imglist = new wxImageList(12,12,true);
    imglist->Add( wxArtProvider::GetBitmap(wxART_GO_UP ));
    imglist->Add( wxArtProvider::GetBitmap(wxART_GO_DOWN  ));

    m_remoteList->SetImageList(imglist, wxIMAGE_LIST_SMALL);

    imglist = new wxImageList(12,12,true);
    imglist->Add( wxArtProvider::GetBitmap(wxART_GO_UP ));
    imglist->Add( wxArtProvider::GetBitmap(wxART_GO_DOWN  ));

    m_localList->SetImageList(imglist, wxIMAGE_LIST_SMALL);
*/

    info1.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT );
    info2.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT );
    info3.SetMask( wxLIST_MASK_DATA | wxLIST_MASK_TEXT );

    info1.SetWidth(w/2);
    info1.SetAlign(wxLIST_FORMAT_LEFT);
    info1.SetText(_T("Name"));
    info1.SetData( 1 );


    info2.SetWidth(w/4);
    info2.SetAlign(wxLIST_FORMAT_LEFT);
    info2.SetText(_T("Date"));
    info2.SetData( 1 );


    info3.SetWidth(w/4);
    info3.SetAlign(wxLIST_FORMAT_RIGHT);
    info3.SetText(_T("Size"));
    info3.SetData( 1 );

    info4.SetWidth(w/5);
    info4.SetAlign(wxLIST_FORMAT_RIGHT);
    info4.SetText(_T("Status"));
    info4.SetData( 1 );

    m_remoteList->InsertColumn(0,info1);
    m_remoteList->InsertColumn(1,info2);
    m_remoteList->InsertColumn(2,info3);

    m_localList->InsertColumn(0,info1);
    m_localList->InsertColumn(1,info2);
    m_localList->InsertColumn(2,info3);
    m_localList->InsertColumn(3,info4);

    m_localCols[0] = false;
    m_localCols[1] = false;
    m_localCols[2] = false;

    m_romoteCols[0] = false;
    m_romoteCols[1] = false;
    m_romoteCols[2] = false;

}

void WebInterfacePanel::AddLocalFileToList(const wxString& name)
{

    //Logger::logf(LOG_MSG,_T("+++++++++++name=")+name);

    wxFileName fname(name);

    wxDateTime acc, mod, created;
    fname.GetTimes(&acc, &mod, &created);
    //wxString date = mod.FormatISODate() + _T(" ") + mod.FormatISOTime();
    wxString date = mod.Format( _T("%Y.%m.%d. %H:%M") );

    wxString line = fname.GetFullName() + _T("|") + wxString::Format(_T("%i"),fname.GetSize().ToULong() ) + _T("|") + date ;

    AddLine( line, m_localList );



}

void WebInterfacePanel::FillLocalList()
{
    m_localList->DeleteAllItems();

    wxString path = DirHelper::GetPath(dWEBCACHE) + m_desc.LocalPath + DirHelper::Separ();

    //Logger::logf(LOG_MSG,_T("path=")+path);

    wxDir dir(path);
    wxString tmp;
    bool res;

    res = dir.GetFirst( &tmp , wxEmptyString, wxDIR_FILES );

    if(res)
        AddLocalFileToList( path + tmp );

     while(res){
        if(!dir.GetNext(&tmp)) break;
            AddLocalFileToList( path + tmp );
    }

    m_localList->SetColumnWidth(1,-1);
    m_localList->SetColumnWidth(2,-1);


    wxListCtrlHelper::SortColoumData(m_localList, 0, false);
}

void WebInterfacePanel::OnTransferDone(wxCommandEvent& evt)
{
        m_remPathList->Enable( true );
        m_remoteList->Enable( true );

        if( m_desc.TransferDirection & WIPDesc::WIP_DOWNLOAD ) {
            m_downButton->Enable( true );
            FillLocalList();
        }

        if( m_desc.TransferDirection & WIPDesc::WIP_UPLOAD ){
            m_upButton->Enable( true );
            FillRemoteList(m_remPathList->GetValue());
        }

        //DeselectAllItems(m_remoteList);
        m_progressBar->Update(100.0, false);

    if(m_autoInstall->GetValue()) {
        for(size_t a=0;a<m_transferedFiles.GetCount();a++){
            wxFileName fn(m_transferedFiles[a]);
            Install(fn.GetFullName());
    }
}

}

void WebInterfacePanel::notify(const wxString& str)
{
    m_transferedFiles.Add(str);
}

void WebInterfacePanel::notify( int i )
{
    if( i == WEB_ENDSESSION ) {  /// all transfer is done
        wxCommandEvent evt(wxEVT_COMMAND_ENTER, idTransferEnd);
        AddPendingEvent(evt);
    }
}

void WebInterfacePanel::OnTransferProgress(wxCommandEvent& evt)
{
    double percent = (double)evt.GetInt() / 100.0;
    m_progressBar->Update( percent );
}

void WebInterfacePanel::notify(int i, int idx)
{
    wxCommandEvent evt(wxEVT_COMMAND_ENTER, idTransferProgress);
    evt.SetInt( (int)((double)i / (double)idx * 10000.0) );
    ProcessEvent(evt);
}
