#include <wx/filename.h>

#include "../DirHelper.h"

#include "wx/wxprec.h"
#include <wx/toplevel.h>
#include <wx/wx.h>

#include "../vemsgauge/vemsgauge.h"
#include "../vemsgauge/VemsGaugeDigital.h"
#include "../vemsgauge/ArcBarGauge.h"

#include "../vemsgauge/gaugegroupmanager.h"
#include "../vemsgauge/Multigaugegroupmanager.h"
#include "../vemsgauge/GaugeType.h"

#include "../logger.h"
#include "../MTConfigHelper.h"
#include "../VemsProtocolHelper.h"
#include "../confighelper.h"
#include "../DisplayHelper.h"

#include "../rtdatamanager/RTDataMgrFacade.h"
#include "../rtdatamanager/RTDataMgrLog.h"
#include "../rtdatamanager/RTDataMgrLogSeq.h"

#include "../rtdatamanager/RTDataLogWriter.h"


#include "../fileparser/vemslogparser.h"
#include "../fileparser/V3iniParser.h"
#include "../VemsFlagConfigHelper.h"


#include "../mtconfigdata.h"
#include "../gaugevalueselector.h"

#include "../WindowReg.h"

#include "../VemsTuneController.h"

#include <src/getopt.h>
#include <src/iobase.h>
#include <src/serport.h>
#include <src/timer.h>
#include <wx/clipbrd.h>

#include "../expression/RPNTester.h"

#include "../VemsInitialScreen.h"
#include "../firmwareconfig/FirmwareConfigData.h"
#include "../ThemesHelper.h"
#include "../project/ProjectHelper.h"
#include "../project/ProjectManager.h"
#include "../project/ProjectSettingsFrame.h"
#include "../eval.h"
#include "../fileparser/vemslogparser.h"
#include "../fileparser/FirmwareConfigParser.h"
#
#include "minimal.h"

//#include <shellsdk.h>		// for WinAPI support
#include <aygshell.h>		// for WinAPI support

#ifdef __BORLANDC__
    #pragma hdrstop
#endif


#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif

int TIMER_ID = wxNewId();

int idSelectInputECU = wxNewId();
int idSelectInputFile = wxNewId();
int idSelectInputNone = wxNewId();


int ID_BUTTON_CN = wxNewId();
int ID_BUTTON_OK = wxNewId();

int ID_RADIOB_HighLog = wxNewId();
int ID_RADIOB_LowLog = wxNewId();
int ID_BUTTON_LogStart = wxNewId();
int ID_BUTTON_LogStop = wxNewId();


int idLoggerSetup = wxNewId();
int idMinimize = wxNewId();

wxWindowID DialogFrameId = wxNewId();

int RT_TIMER = 150;

#if !defined(__WXMSW__) && !defined(__WXPM__)
    #include "../sample.xpm"
#endif


IMPLEMENT_APP(MyApp)

bool MyApp::OnInit()
{

	m_CurrentAngle = 0;

	m_TaskBarIconApp = NULL;

	wxFileName rootd = wxFileName(argv[0]);

	DirHelper::SetProgramRootDir(rootd.GetPath());


	Logger::initLog();
	Logger::log(LOG_MSG, _T("MyApp start from: ") + wxString(argv[0]) );

    if ( !wxApp::OnInit() )
        return false;

    wxInitAllImageHandlers();

    m_CurrentAngle = DisplayHelper::GetOrientation();

	Logger::logf(LOG_MSG,_T("MyApp::OnInit m_CurrentAngle=%i"),m_CurrentAngle);

//	RPNTester tmp;

//	tmp.testInfixCache();

    DisplayHelper::SetOrientation(DMDO_DEFAULT);

    m_initscreen = new VemsInitialScreen(NULL,wxID_ANY,_T("Vems Init"), wxPoint(0,0),DisplayHelper::GetDisplaySize(),wxNO_BORDER);
    m_initscreen->Show(1);
    m_initscreen->Raise();


    m_initscreen->SetThreadStatusListener(this);

    //Logger::log(LOG_MSG,wxString::Format(_T("m_initscreen shown = %i"), m_initscreen->IsShownOnScreen()));

	//m_initscreen->Connect(wxEVT_ACTIVATE, wxActivateEventHandler(MyApp::OnInitframeActivate), NULL, this);

	HWND initHwnd = (HWND)(m_initscreen->GetHandle());
	//SetTopWindow(m_initscreen);

    SHFullScreen( initHwnd , SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);


    m_initscreen->SetEvtHandlerObj(this);

    Logger::log(LOG_MSG,_T("Init: Parse Config Files"));


	ConfigHelper::initConfig();


    m_initscreen->ConfigInit();



    Connect(wxEVT_VEMS_CONTROL, VemsControlEventHandler(MyApp::OnThreadEnd));

  //  VemsInit();

    return true;
}


void MyApp::OnThreadEnd(VemsControlEvent &event)
{
    Logger::log(LOG_MSG,_T("MyApp::OnThreadEnd"));

    //wxExit();
    VemsInit();
}

void MyApp::TSNotify(ThreadState state)
{

    Logger::log(LOG_MSG,_T("Parsing ended"));

    switch(state) {
        case TS_EXIT:
            VemsInit();
          break;

        default:
          break;
    }

}

void MyApp::OnInitframeActivate(wxActivateEvent &evt)
{
	Logger::log(LOG_MSG,wxString::Format(_T("OnInitframeActivate is = %i, shown = %i"),evt.GetActive(), m_initscreen->IsShownOnScreen()));
}
void MyApp::checkCommPort()
{
    SerialConfig * sc = ConfigHelper::getECUSerialConfig();
    wxString port = sc->getPortName();
    if(port.length() < 1){
          wxLogError(_T("Serial Port has not been set %s \n"), port.c_str() );
        /*SerialConfigDialog* frame = new SerialConfigDialog( NULL );
        frame->CenterOnScreen();
        if( frame->ShowModal() == wxOK){
            //SerialConfig* scECU = ConfigHelper::getECUSerialConfig();
            //
        }else{
            //EXIT
        }*/
     }else{

        //Helper::setStateBoot();
        //VemsProtocol *vp = Helper::getVemsProtocol();
        //vp->setState(ECUInfo::MAN);


    }
    Logger::log(LOG_MSG, _T("MainFrame::checkCommPort() port=")+port);
}
bool MyApp::VemsInit()
{
/*
        MTConfigHelper::parseMTConfigData();

        VemsFlagConfigHelper::parseVemsFlagConfig();
*/

	/// Init
		Logger::log(LOG_MSG,_T("Init: RTDataMgrFacade"));

		//m_rtDataMgr = new RTDataMgrFacade();


        m_initscreen->SetInitText(_T("Loading: Guiconfig file..."));

        

		wxString autoLog = ConfigHelper::getConfig()->Read(_T("autolog"));

        m_initscreen->SetInitText(_T("Loading: FlagDescriptor file..."));
        VemsFlagConfigHelper::parseVemsFlagConfig();

	/// Init MainFrame:  GaugeFrameBase
        m_initscreen->SetInitText(_T("Loading: MainFrame..."));
		Logger::log(LOG_MSG,_T("Init: GaugeFrameBase"));


        Logger::log(LOG_MSG,_T("Init: VemsTuneController"));
		
		checkCommPort();
		m_vemsTuneController = new VemsTuneController();
		//MTConfigHelper::createMTConfigData();

		m_vemsTuneController->initEcuStateMgr();
		Logger::log(LOG_MSG,_T("MyApp::VemsInit() initVemsProtocol"));
       //m_vemsTuneController->init();

		//Logger::logf(LOG_MSG,_T("MyApp::VemsInit() m_vemsTuneController = %p"),m_vemsTuneController);
    GaugeType::InitDefaults();
    GaugeType::Init();

//        m_frame = new GaugeFrameBase(NULL,wxID_ANY,m_vemsTuneController, _T("VemsCockPit"),wxPoint(0,0),DisplayHelper::GetDisplaySize(),wxNO_BORDER);
        m_frame = new GaugeFrameBase(NULL,wxID_ANY,_T("VemsCockPit"),wxPoint(0,0),DisplayHelper::GetDisplaySize(),wxNO_BORDER);





      /// destroy the init screen
       // m_initscreen->Destroy();



		m_frame->InitInputSources();

		SetTopWindow(m_frame);
		hWnd = (HWND)(m_frame->GetHandle());


	/// Init Timer class
        m_initscreen->SetInitText(_T("Loading: Main Timer, and threads"));
		//Logger::log(LOG_MSG,_T("Init: EcuTimer class & StartTimer"));

		/*
		m_ECUTimer = new EcuTimer(m_rtDataMgr);
		m_ECUTimer->StartTimer();
		m_ECUTimer->setLocked(false);
		m_frame->SetEcuTimer(m_ECUTimer);
		*/






	/// Init: install font files
        m_initscreen->SetInitText(_T("Loading: Font Files..."));
		Logger::log(LOG_MSG,_T("Init: Install Font Files"));
		if(!wxFileExists(_T("Windows\\Fonts\\7 Segmentdot.ttf")))
			wxCopyFile(DirHelper::GetPath(dGRAPHICS) + _T("7 Segmentdot.ttf"), _T("Windows\\Fonts\\7 Segmentdot.ttf"));

		//maybe the Fonts does not exist!!

/*	/// Parse config files
		Logger::log(LOG_MSG,_T("Init: Parse vemsv3.ini, VemsFlagDesc.xml"));
        MTConfigHelper::parseMTConfigData();
		VemsFlagConfigHelper::parseVemsFlagConfig();

*/
	/*
    if ( m_vemsTuneController->getDevice() == ECUInfo::DEVICE_V3 )
    {
        if ( m_vemsTuneController->getStoredState() == ECUInfo::AIM ){
            MTConfigHelper::parseMTConfigData();
        }else{
            MTConfigHelper::parseMTConfigData( m_vemsTuneController->getFirmwareInfo() );
        }
    }else if(m_vemsTuneController->getDevice() == ECUInfo::DEVICE_ROUND){
        MTConfigHelper::parseMTConfigData();

    }*/
	/*else if ( Project* p = ProjectHelper::GetPrjMgr()->GetCurrPrj() )
    {
        wxArrayString configFiles = p->GetConfigFiles();
        FirmwareConfigData fwConfigData;
        FirmwareConfigParser parser( &fwConfigData );
        parser.parseFiles( configFiles );

        bool isIniValid = MTConfigHelper::parseMTConfigData( p->FwVersion,
                                           fwConfigData.is12x12(),
                                           fwConfigData.isHighBoost() );

        m_vemsTuneController->m_configIniFile   = p->GetConfigIniFile();
        m_vemsTuneController->m_configFiles     = configFiles;
        m_vemsTuneController->setIniParsed(isIniValid);

    }*/

	//MTConfigHelper::parseMTConfigData( m_vemsTuneController->getFirmwareInfo() );
    //MTConfigHelper::parseMTConfigData();
	m_vemsTuneController->init();

    m_rtDataMgr = m_vemsTuneController->getRTDataMgr();
    m_stController = m_vemsTuneController->getSTDataMgr();

	wxConfig * v3conf = ConfigHelper::getConfig();
    wxString oldPath = v3conf->GetPath();
    v3conf->SetPath(_T("/guiLayout"));
    wxString pers = v3conf->Read(_T("perspective"),wxEmptyString);
    v3conf->SetPath(oldPath);

	/// Init MultiGaugeGroupMgr
        m_initscreen->SetInitText(_T("Loading: GroupManager..."));
		Logger::log(LOG_MSG,_T("Init: MultiGaugeGroupMgr"));
		wxMenu *tmpmenu = new wxMenu();
		MultiGaugeGroupManager *mggmgr = new MultiGaugeGroupManager(m_frame, tmpmenu, m_vemsTuneController,false, true);
		mggmgr->SetGaugeFrameBase(m_frame);
		mggmgr->load( DirHelper::GetPath(dCONFIG) + _T("pdascreen.xml"));


	/// Init WindowReg for minimize the app
        m_initscreen->SetInitText(_T("Loading: Minimize Feature"));
		Logger::log(LOG_MSG,_T("Init: WindowReg"));
		wreg = new WindowReg(m_frame);
		m_frame->SetWindowReg(wreg);


    /// Init TaskBarIcon
        m_initscreen->SetInitText(_T("Loading: Task Bar Icon"));
		Logger::log(LOG_MSG,_T("Init: TaskBarIcon"));
		wxBitmap tempbmp1;
		tempbmp1.LoadFile( DirHelper::GetPath(dICON) + _T("vemsicon.gif"),wxBITMAP_TYPE_GIF);

		wxIcon icon1;
		icon1.CopyFromBitmap(tempbmp1);

		m_TaskBarIconApp = new VemsTaskBarIcon(m_frame);
		m_TaskBarIconApp->SetIcon(icon1,_T("tooltip"));
		Logger::logf(LOG_MSG,_T("VemsTaskBarIcon->IsRestored()=%i"),m_TaskBarIconApp->IsRestored());

		m_frame->setTaskBarIcon(m_TaskBarIconApp);


	/// Init FullScreen Mode  (Winapi func.)
        m_initscreen->SetInitText(_T("Loading: Full Screen Mode"));
		Logger::log(LOG_MSG,_T("Init: FullScreen Mode"));
		//hWnd = (HWND)(m_frame->GetHandle());
		SHFullScreen( hWnd , SHFS_HIDETASKBAR | SHFS_HIDESIPBUTTON | SHFS_HIDESTARTICON);

	/// App init

/*
		if(autoLog == _T("0")) {

		// Show default screen
			Logger::log(LOG_MSG,_T("Init: Show default screen"));
			mggmgr->showDefaultGroup(ori);
		}

		if(autoLog == _T("1")) {

			mggmgr->AddOrCreateLoggerScreen(true);
			wreg->UpdateList();
			wreg->Minimize();
		}
*/

	///start screen
	m_initscreen->SetInitText(_T("Loading Default Screen..."));
		Logger::log(LOG_MSG,_T("Init: Show default screen"));
		mggmgr->showDefaultGroup();


    m_initscreen->SetInitText(_T("Start!!!!"));

    Logger::log(LOG_MSG,_T("Init: Hide initial screen"));
    m_frame->Show(true);
    m_frame->Raise();


    m_initscreen->Hide();


    m_vemsTuneController->setECULog(true);

    Logger::log(LOG_MSG,_T("Initializing Ended"));

	
	m_vemsTuneController->startThreads();

	return true;
}

int MyApp::OnExit()
{
	Logger::logf(LOG_MSG,_T("MyApp::OnExit"));

    DisplayHelper::SetOrientation(m_CurrentAngle);

	Logger::logf(LOG_MSG,_T("MyApp::OnExit m_CurrentAngle=%i"),m_CurrentAngle);


	m_TaskBarIconApp->RemoveIcon();
	delete m_TaskBarIconApp;
	if(wxFileExists(_T("Windows\\Fonts\\7 Segmentdot.ttf")))
		wxRemoveFile(_T("Windows\\Fonts\\7 Segmentdot.ttf"));

	return 0;
}
