#include "IACLockPanel.h"

#include "../logger.h"

#include "../vemsprotocol/VemsCommand.h"
#include "../vemsprotocol/CommandRequestBase.h"
#include "IACControl.h"

BEGIN_EVENT_TABLE( IACLockPanel, wxPanel )
    EVT_TEXT_ENTER(wxID_ANY, IACLockPanel::OnEnter)
    EVT_KILL_FOCUS( IACLockPanel::OnKillFocus )
    EVT_KEY_DOWN(IACLockPanel::OnKeyDown)
    EVT_BUTTON( wxID_APPLY, IACLockPanel::OnEnter )
    EVT_BUTTON( wxID_CLOSE, IACLockPanel::OnCloseButton )
END_EVENT_TABLE()

IACLockPanel::IACLockPanel(wxWindow * parent, CommandRequestBase* requestMgr)
{
    //ctor

    m_requestMgr = requestMgr;
    buildGUI(parent);
    m_autoCtrl = NULL;
}

IACLockPanel::~IACLockPanel()
{
    //dtor
    Logger::logf( LOG_MSG, _T("IACLockPanel dtor!") );
    commandRequest(0);
    if(m_autoCtrl){
        delete m_autoCtrl;
}
}


void IACLockPanel::buildGUI(wxWindow* parent)
{
    Create(parent, wxID_ANY);

    SetBackgroundColour( wxSystemSettings::GetColour( wxSYS_COLOUR_3DFACE ) );
    ClearBackground();

    wxBoxSizer* m_mainsizer = new wxBoxSizer( wxVERTICAL );
    //m_mainsizer  = new wxBoxSizer( wxVERTICAL );

    wxString msg = _T("Idle Air Controller Lock Dialog\n\n To activate set 1-255\n To disable set 0 or close");
    wxStaticText* text2 = new wxStaticText( this, -1, msg);

    //wxString  warningMsg = _T("\nmax value limited by MaxSteps To Extend");
    wxString  warningMsg = _T("The following settings could disable IAC lock functionality:\n\n");

    warningMsg += _T("Menu: Starting/Idle\n\n");

    warningMsg += _T("1. PWM idle hardware settings:\n    Max/Min PWM duty allowed\n\n");
    warningMsg += _T("2. Stepper idle hardware settings:\n    Max/Min Steps allowed\n");
    wxStaticText* warningST = new wxStaticText( this, -1, warningMsg);
    warningST->SetForegroundColour(wxColour(255,0,0) );
    //text2->SetFont( font );

    m_mainsizer->Add( text2, 0, wxALL | wxEXPAND, 5 );


    wxFlexGridSizer* grid = new wxFlexGridSizer( 3, 1, 5 );
    m_mainsizer->Add( grid, 0, wxEXPAND | wxALL, 0 );

    wxStaticText* textName= new wxStaticText( this, -1, _T("IAC position:"));
    grid->Add( textName, 0, wxALL | wxEXPAND, 5 );

    m_textctrl = new wxTextCtrl(this, -1, _T(""), wxDefaultPosition,
    wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add( m_textctrl, 0, wxALL | wxEXPAND, 5 );

    wxStaticText* textUnit = new wxStaticText( this, -1, _T(""));
    grid->Add( textUnit, 0, wxALL | wxEXPAND, 5 );
///////////

    textName = new wxStaticText( this, -1, _T("Target rpm:"));
    grid->Add( textName, 0, wxALL | wxEXPAND, 5 );

    m_targetRPMID = wxNewId();
    m_targetRPMctrl = new wxTextCtrl(this, m_targetRPMID, _T(""), wxDefaultPosition,
    wxDefaultSize, wxTE_PROCESS_ENTER | wxTE_BESTWRAP );
    grid->Add( m_targetRPMctrl, 0, wxALL | wxEXPAND, 5 );

    textUnit = new wxStaticText( this, -1, _T("1/min"));
    grid->Add( textUnit, 0, wxALL | wxEXPAND, 5 );
///////////
    m_mainsizer->Add( warningST, 0, wxALL | wxEXPAND, 5 );


    wxButton* sendButton = new wxButton( this, wxID_APPLY , _T("Send") );
    wxButton* closeButton = new wxButton( this, wxID_CLOSE , _T("Close") );

    wxBoxSizer* buttonSizer = new wxBoxSizer( wxHORIZONTAL );

    buttonSizer->Add( sendButton, 0, wxALL, 5 );
    buttonSizer->AddStretchSpacer();
    buttonSizer->Add( closeButton, 0, wxALL, 5 );

    m_mainsizer->Add( buttonSizer, 1, wxALL | wxEXPAND | wxALIGN_BOTTOM , 5 );


    SetSizer( m_mainsizer );
    m_mainsizer->Fit( this );
    m_mainsizer->SetSizeHints( this );
}


void IACLockPanel::OnEnter( wxCommandEvent& event )
{
    Logger::logf( LOG_MSG, _T("IACLockPanel::OnEnter()") );
    if(event.GetId() == m_targetRPMID){

        Logger::logf( LOG_MSG, _T("IACLockPanel::OnEnter() target rpm") );

        wxString valueStr = m_targetRPMctrl->GetValue();
        double dValue;
        bool result = valueStr.ToDouble(&dValue);
        if(!result) return;
        m_autoCtrl->setTargetRPM((int)dValue);
        return;
    }

    wxString valueStr = m_textctrl->GetValue();
    double dValue;
    bool result = valueStr.ToDouble(&dValue);
    if(!result) return;

    //dValue *= 4;    /// deg * 4;
    int binaryValue = (int)dValue;
    Logger::logf( LOG_MSG, _T("IACLockPanel::OnEnter() valu=%d"), binaryValue);
    commandRequest(binaryValue);
}

/// O means desable ing lock
void IACLockPanel::commandRequest(int ignition)
{
    if( (ignition < 0) || (ignition > 255) ) return;
    wxString command = wxString::Format(_T("Manmdi%02Xmbye"), ignition);

    Logger::logf( LOG_MSG, _T("IACLockPanel::OnEnter() iac. lock command: ")+ command);

    VemsCommand* vemsCommand = new StringCommand(command);
    m_requestMgr->request(vemsCommand);

}

void IACLockPanel::OnKillFocus( wxFocusEvent& event )
{
     Logger::logf( LOG_MSG, _T("IACLockPanel::OnKillFocus()") );
}

void IACLockPanel::OnKeyDown( wxKeyEvent& event )
{
    int key = event.GetKeyCode();

    if ( key == WXK_TAB )
    {
        if ( event.ShiftDown() )
        {

            return;
        }

        return;
    }

    event.Skip();
}


void IACLockPanel::OnCloseButton( wxCommandEvent& event )
{
    wxWindow* parent = GetParent();

    if ( parent ) {
        parent->Destroy();
    }
}

void IACLockPanel::setAutoControl(IACControl* autoCtrl)
{
    m_autoCtrl = autoCtrl;
}
