#include "wxHSLColor.h"

#define ifdoubleeq(a,b) if(fabs((a)-(b)) < 0.00001)

wxHSLColor::wxHSLColor()
{
    m_hue = 0;
    m_sat = 0;
    m_lum = 0;
}

wxHSLColor::wxHSLColor(const wxColor& color)
{
    FromRGB(color);
}

wxHSLColor::wxHSLColor(double h, double s, double l)
{
    m_hue = h;
    m_sat = s;
    m_lum = l;
}

wxHSLColor::~wxHSLColor()
{
    //dtor
}

void wxHSLColor::OffsetLum(double o)
{
    m_lum += o;
    if(m_lum < 0) m_lum = 0;
    if(m_lum > 1) m_lum = 1;
}

void wxHSLColor::ToRGB(uint8_t* red, uint8_t* green, uint8_t* blue, uint8_t* alpha)
{
    ///TODO: implement
}

void wxHSLColor::ToRGB(wxColor &color)
{
    if(fabs(m_sat) < 0.00001)  {

        uint8_t v = (uint8_t)(m_lum * 255);

        color.Set( v,v,v,m_alpha );

    } else {

        uint8_t r,g,b;
        double ra,ga,ba;

        double temp2 = m_lum < 0.5 ? ( m_lum*(m_sat+1) ) : ( m_lum+m_sat - m_lum*m_sat );
        double temp1 = 2.0 * m_lum - temp2;

        double h = m_hue / 6.0;

        double Rtemp3 = h + 1.0 / 3.0;
        double Gtemp3 = h;
        double Btemp3 = h - 1.0 / 3.0;

        if(Rtemp3 < 0) Rtemp3 += 1.0;
        if(Gtemp3 < 0) Gtemp3 += 1.0;
        if(Btemp3 < 0) Btemp3 += 1.0;

        if(Rtemp3 > 1) Rtemp3 -= 1.0;
        if(Gtemp3 > 1) Gtemp3 -= 1.0;
        if(Btemp3 > 1) Btemp3 -= 1.0;

        if(6.0*Rtemp3 < 1) ra = temp1+(temp2-temp1) * 6.0 * Rtemp3;
        else if(2.0*Rtemp3 < 1) ra = temp2;
        else if(3.0*Rtemp3 < 2) ra = temp1 + (temp2-temp1)*((2.0/3.0)-Rtemp3)*6.0;
        else ra = temp1;

        if(6.0*Gtemp3 < 1) ga = temp1+(temp2-temp1) * 6.0 * Gtemp3;
        else if(2.0*Gtemp3 < 1) ga = temp2;
        else if(3.0*Gtemp3 < 2) ga = temp1 + (temp2-temp1)*((2.0/3.0)-Gtemp3)*6.0;
        else ga = temp1;

        if(6.0*Btemp3 < 1) ba = temp1+(temp2-temp1) * 6.0 * Btemp3;
        else if(2.0*Btemp3 < 1) ba = temp2;
        else if(3.0*Btemp3 < 2) ba = temp1 + (temp2-temp1)*((2.0/3.0)-Btemp3)*6.0;
        else ba = temp1;

        r = (int)ceil(ra*255);
        b = (int)ceil(ba*255);
        g = (int)ceil(ga*255);

        color.Set(r,g,b,m_alpha);
    }
}

wxColor wxHSLColor::ToRGB()
{
    wxColor color;

    ToRGB(color);

    return color;
}

void wxHSLColor::FromRGB(uint8_t red, uint8_t green, uint8_t blue, uint8_t alpha)
{
    m_alpha = alpha;
    double r = red;
    double g = green;
    double b = blue;

    double ra = r / 255.0;
    double ga = g / 255.0;
    double ba = b / 255.0;

    double maxv = wxMax(wxMax(ra,ga),ba);
    double minv = wxMin(wxMin(ra,ga),ba);

    m_lum = (maxv+minv)/2;

    if(fabs(maxv-minv) < 0.00001) {
        m_sat = 0;
        m_hue = 0;
    } else {
        m_sat = m_lum < 0.5 ? ( (maxv-minv)/(maxv+minv) ) : ( (maxv-minv)/(2.0-maxv-minv) );

        ifdoubleeq(ra,maxv) m_hue = (ga-ba) / (maxv-minv);
        ifdoubleeq(ga,maxv) m_hue = 2.0 + (ba-ra) / (maxv-minv);
        ifdoubleeq(ba,maxv) m_hue = 4.0 + (ra-ga) / (maxv-minv);
    }
}

void wxHSLColor::FromRGB(const wxColor& color)
{
    FromRGB(color.Red(),color.Green(),color.Blue(),color.Alpha());
}

wxString wxHSLColor::ToString(int dec)
{
    wxString sf = wxString::Format(_T("h=%%.%if, s=%%.%if, l=%%.%if"),dec,dec,dec);

    return wxString::Format(sf,m_hue,m_sat,m_lum);
}
